@@ -1,5 +1,5 @@
 /*
- * $Id: common.h 15162 2008-04-01 22:28:45Z cbiere $
+ * $Id: common.h 19200 2011-06-01 16:44:40Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -38,33 +38,6 @@
 
 #include "config.h"
 
-/*
- * Constants
- */
-
-#define GTA_PRODUCT_NAME "gtk-gnutella"	/**< Normally "gtk-gnutella" */
-#define GTA_VERSION 0				/**< major version */
-#define GTA_SUBVERSION 96			/**< minor version */
-#define GTA_PATCHLEVEL 5			/**< patch level or teeny version */
-#define GTA_REVISION "stable"		/**< unstable, beta, stable */
-#define GTA_REVCHAR ""				/**< (u)nstable, (b)eta, none -> stable */
-#define GTA_RELEASE "2008-04-01"	/**< ISO 8601 format YYYY-MM-DD */
-#define GTA_WEBSITE "http://gtk-gnutella.sourceforge.net/"
-
-#if defined(USE_GTK1)
-#define GTA_INTERFACE "GTK1"
-#elif defined(USE_GTK2)
-#define GTA_INTERFACE "GTK2"
-#elif defined(USE_TOPLESS)
-#define GTA_INTERFACE "Topless"
-#else
-#error "Expected Gtk+ 1.2, Gtk+ 2.x or headless as user-interface."
-#endif
-
-#ifndef HAS_LIBXML2
-#error "You need libxml2 (http://www.xmlsoft.org/) to compile gtk-gnutella"
-#endif
-
 /*
  * Main includes
  */
@@ -96,38 +69,6 @@
 #undef KERNEL
 #endif
 
-#ifdef MINGW32
-#include <ws2tcpip.h>
-#include <winsock.h>
-#define ECONNRESET WSAECONNRESET
-#define ECONNREFUSED WSAECONNREFUSED
-#define ECONNABORTED WSAECONNABORTED
-#define ENETUNREACH WSAENETUNREACH
-#define EHOSTUNREACH WSAEHOSTUNREACH
-#define ETIMEDOUT WSAETIMEDOUT
-#define EINPROGRESS WSAEINPROGRESS
-#define ENOTCONN WSAENOTCONN
-#define ENOBUFS WSAENOBUFS
-#define EADDRNOTAVAIL WSAEADDRNOTAVAIL
-#define ENETRESET WSAENETRESET
-#define ENETDOWN WSAENETDOWN
-#define EHOSTDOWN WSAEHOSTDOWN
-#define ENOPROTOOPT WSAENOPROTOOPT
-#define EPROTONOSUPPORT WSAEPROTONOSUPPORT
-
-#define SHUT_RD   SD_RECEIVE
-#define SHUT_WR   SD_SEND
-#define SHUT_RDWR SD_BOTH
-
-#define S_IXGRP  _S_IEXEC
-#define S_IWGRP  _S_IWRITE
-#define S_IRGRP  _S_IREAD
-
-#define S_IRWXG _S_IREAD
-#define S_IRWXO _S_IREAD
-
-#endif /* !MINGW32 */
-
 #ifdef I_SYS_SOCKET
 #include <sys/socket.h>
 #endif
@@ -144,35 +85,22 @@
 #include <sys/stat.h>
 #endif
 
-#ifdef MINGW32
-struct iovec 
-{
-	char  *iov_base;
-	int  iov_len; 
-};
-
-struct passwd
-{
-	char *pw_name;                /* Username.  */
-	char *pw_passwd;              /* Password.  */
-#if 0
-	__uid_t pw_uid;               /* User ID.  */
-	__gid_t pw_gid;               /* Group ID.  */
-	char *pw_gecos;               /* Real name.  */
-	char *pw_dir;                 /* Home directory.  */
-	char *pw_shell;               /* Shell program.  */
-#endif /* 0 */
-};
-
-#else /* !MINGW32 */
-
-#include <sys/uio.h>		/* For writev(), readv(), struct iovec */
-#include <sys/wait.h>
-
 #ifdef I_NETINET_IN
 #include <netinet/in.h>
 #endif
 
+#ifdef MINGW32
+
+/* Provided for convenience to reduce ifdef hell */
+#define is_running_on_mingw() 1
+
+#else /* !MINGW32 */
+
+/* Provided for convenience to reduce ifdef hell */
+#define is_running_on_mingw() 0
+
+#include <sys/uio.h>		/* For writev(), readv(), struct iovec */
+#include <sys/wait.h>
 #include <netinet/tcp.h>
 
 #endif /* MINGW32 */
@@ -227,22 +155,13 @@ struct passwd
 
 #ifndef MAP_FAILED
 #define MAP_FAILED ((void *) -1)
-#endif	/* !MMAP_FAILED */
+#endif	/* !MAP_FAILED */
 
 #ifdef I_SYS_SENDFILE
 #include <sys/sendfile.h>
 #else	/* !I_SYS_SENDFILE */
 #ifdef HAS_SENDFILE
 #define USE_BSD_SENDFILE	/**< No <sys/sendfile.h>, assume BSD version */
-#else
-/*
- * Proper mmap() support for memory-mapped files requires ISO C functions like
- * sigsetjmp().
- */
-#if defined(HAS_MMAP) && defined(HAS_SIGSETJMP)
-#define USE_MMAP 1
-#endif	/* ISO C */
-
 #endif	/* HAS_SENDFILE */
 #endif	/* I_SYS_SENDFILE_H */
 
@@ -251,28 +170,6 @@ struct passwd
 #include <netinet/ip.h>
 #endif
 
-#if defined(HAS_POLL)
-#include <poll.h>
-#endif	/* HAS_POLL */
-
-#if !defined(AF_LOCAL) && defined(AF_UNIX)
-#define AF_LOCAL AF_UNIX
-#endif	/* !AF_LOCAL && AF_UNIX */
-
-#if !defined(PF_LOCAL) && defined(PF_UNIX)
-#define PF_LOCAL PF_UNIX
-#endif	/* !PF_LOCAL && PF_UNIX */
-
-/* For pedantic lint checks, define USE_LINT. We override some definitions
- * and hide ``inline'' to prevent certain useless warnings. */
-#ifdef USE_LINT
-#undef G_GNUC_INTERNAL
-#define G_GNUC_INTERNAL
-#undef G_INLINE_FUNC
-#define G_INLINE_FUNC
-#define inline
-#endif
-
 /*
  * Determine how large an I/O vector the kernel can accept.
  */
@@ -289,8 +186,24 @@ struct passwd
 #define MAX_IOV_COUNT	16				/**< Unknown, use required minimum */
 #endif
 
+/*
+ * For pedantic lint checks, define USE_LINT. We override some definitions
+ * and hide ``inline'' to prevent certain useless warnings.
+ */
+
+#ifdef USE_LINT
+#define inline
+#endif
+
 #include <glib.h>
 
+#ifdef USE_LINT
+#undef G_GNUC_INTERNAL
+#define G_GNUC_INTERNAL
+#undef G_INLINE_FUNC
+#define G_INLINE_FUNC
+#endif
+
 #if defined(USE_GLIB1) && !defined(GLIB_MAJOR_VERSION)
 #error "Install GLib 1.2 to compile gtk-gnutella against GLib 1.2."
 #endif
@@ -299,16 +212,72 @@ struct passwd
 #error "Install GLib 2.x to compile gtk-gnutella against GLib 2.x."
 #endif
 
-#ifdef USE_LINT
-#undef G_GNUC_INTERNAL
-#define G_GNUC_INTERNAL
-#undef G_INLINE_FUNC
-#define G_INLINE_FUNC
-#define inline
-#endif
-
 typedef guint64 filesize_t; /**< Use filesize_t to hold filesizes */
 
+#include "lib/mingw32.h"
+
+#ifndef MINGW32
+
+typedef struct iovec iovec_t;
+typedef off_t fileoffset_t;
+typedef struct stat filestat_t;
+
+/**
+ * These functions are required because under MINGW struct iovec has
+ * different member names and order.
+ */
+static inline void *
+iovec_base(const struct iovec *iov)
+{
+	return iov->iov_base;
+}
+
+static inline void
+iovec_set_base(struct iovec *iov, const void *base)
+{
+	iov->iov_base = (void *) base;
+}
+
+static inline size_t
+iovec_len(const struct iovec *iov)
+{
+	return iov->iov_len;
+}
+
+static inline void
+iovec_set_len(struct iovec *iov, size_t len)
+{
+	iov->iov_len = len;
+}
+
+/* FIXME: Get rid of these: */
+typedef int socket_fd_t;
+#define INVALID_SOCKET (-1)
+
+#define socket_fd(fd)		(fd)
+
+/*
+ * The following are for file-like operations on sockets, which we need to trap
+ * so that we grab the possible error condition through WSAGetLastError().
+ *
+ * To distinguish them from regular read() or write() on plain file descriptors,
+ * we prefix them with "s_".
+ */
+#define s_write write
+#define s_writev writev
+#define s_read read
+#define s_readv readv
+#define s_close close
+#endif /* !MINGW32 */
+
+#if !defined(AF_LOCAL) && defined(AF_UNIX)
+#define AF_LOCAL AF_UNIX
+#endif	/* !AF_LOCAL && AF_UNIX */
+
+#if !defined(PF_LOCAL) && defined(PF_UNIX)
+#define PF_LOCAL PF_UNIX
+#endif	/* !PF_LOCAL && PF_UNIX */
+
 #ifdef I_STDARG
 #include <stdarg.h>
 #endif
@@ -320,10 +289,20 @@ typedef guint64 filesize_t; /**< Use filesize_t to hold filesizes */
 #ifdef USE_GLIB1
 typedef void (*GCallback) (void);
 #define G_STRLOC __FILE__ ":" STRINGIFY(__LINE__)
+
+#if defined (__STDC_VERSION__) && (__STDC_VERSION__ >= 19901L)
+#define G_STRFUNC (__func__)
+#elif defined(__GNUC__)
+#define G_STRFUNC (__PRETTY_FUNCTION__)
+#else
+#define G_STRFUNC (G_STRLOC)
 #endif
+
+#endif	/* USE_GLIB1 */
+
 #ifdef USE_GLIB2
 #include <glib-object.h>
-#endif
+#endif	/* USE_GLIB2 */
 
 /*
  * Array size determination
@@ -338,8 +317,21 @@ typedef void (*GCallback) (void);
 
 /*
  * Can only use the `args' obtained via va_start(args) ONCE.  If we need
- * to call another vararg routine, we need to copy the original args.
+ * to call a second vararg routine, we need to copy the original args.
  * The __va_copy macro is a GNU extension.
+ *
+ * Using the `args' means calling va_arg(args, TYPE) on the variable argument
+ * list to process the arguments.  Passing the argument list as a va_list
+ * pointer does not use the `args' and does not require any VA_COPY to be done.
+ *
+ * However, one must assume that passing the `args' as a va_list pointer will
+ * cause the routine to use the `args', and therefore if a second call with
+ * the same `args' is required, the first one should be made through a copy
+ * of the `args' through VA_COPY(), and the second with the original `args'.
+ *
+ * After a VA_COPY(dest, src), one must call va_end(dest) to properly cleanup
+ * the copied list, just like one would do after a va_start(dest).  The va_end()
+ * should occur in the same routine where va_start() or VA_COPY() is done.
  */
 #ifdef va_copy
 #define VA_COPY(dest, src) va_copy(dest, src)
@@ -372,6 +364,13 @@ typedef void (*GCallback) (void);
 #define S_ISLNK(mode) (((mode) & S_IFMT) == S_IFLNK)
 #endif	/* S_ISLNK */
 
+/*
+ * Allow code to blindly say lstat() even if not defined.
+ */
+#ifndef HAS_LSTAT
+#define lstat(_p,_b)	stat((_p),(_b))
+#endif
+
 /*
  * These macros determine the maximum/minimum value of the given integer type
  * "t". This works for signed as well as unsigned types. This code does
@@ -382,7 +381,7 @@ typedef void (*GCallback) (void);
 
 #define MAX_INT_VAL_STEP(t) \
 	((t) 1 << (CHAR_BIT * sizeof(t) - 1 - ((t) -1 < 1)))
-	 
+
 #define MAX_INT_VAL(t) \
 	((MAX_INT_VAL_STEP(t) - 1) + MAX_INT_VAL_STEP(t))
 
@@ -395,9 +394,13 @@ typedef void (*GCallback) (void);
 #endif /* TIME_T_MAX */
 
 #ifndef OFF_T_MAX
-#define OFF_T_MAX MAX_INT_VAL(off_t)
+#define OFF_T_MAX MAX_INT_VAL(fileoffset_t)
 #endif /* OFF_T_MAX */
 
+#ifndef SIZE_MAX
+#define SIZE_MAX MAX_INT_VAL(size_t)
+#endif
+
 /*
  * Other common macros.
  */
@@ -426,34 +429,22 @@ typedef void (*GCallback) (void);
 	PACKAGE_SOURCE_DIR G_DIR_SEPARATOR_S "extra_files"
 #endif
 
-/**
- * Calls g_free() and sets the pointer to NULL afterwards. You should use
- * this instead of a bare g_free() to prevent double-free bugs and dangling
- * pointers.
- */
-#define G_FREE_NULL(p)	\
-G_STMT_START {			\
-	if (p) {			\
-		g_free(p);		\
-		p = NULL;		\
-	}					\
-} G_STMT_END
-
 #if defined(__GNUC__) && defined(__GNUC_MINOR__)
 
-/** HAVE_GCC allows conditionalization depending on the version of gcc
- *  being used to compile the source.
+/**
+ * HAS_GCC allows conditionalization depending on the version of gcc
+ * being used to compile the source.
  *
- *  Check each version at "http://gcc.gnu.org/onlinedocs/" for
- *  support.  Specific functionality may also be broken in some
- *  compiler revisions, so it is useful to conditionalize on the
- *  version.
+ * Check each version at "http://gcc.gnu.org/onlinedocs/" for
+ * support.  Specific functionality may also be broken in some
+ * compiler revisions, so it is useful to conditionalize on the
+ * version.
  */
-#define HAVE_GCC(major, minor) \
+#define HAS_GCC(major, minor) \
 	((__GNUC__ > (major)) || \
 	 (__GNUC__ == (major) && __GNUC_MINOR__ >= (minor)))
 #else
-#define HAVE_GCC(major, minor) 0
+#define HAS_GCC(major, minor) 0
 #endif
 
 /*
@@ -464,7 +455,7 @@ G_STMT_START {			\
  *
  * void my_memcpy(void *dst, const void *src, size_t n) NON_NULL_PARAM((1, 2));
  */
-#if HAVE_GCC(3, 3)
+#if defined(HASATTRIBUTE) && HAS_GCC(3, 3)
 #define NON_NULL_PARAM(x) __attribute__((__nonnull__ x))
 #else /* GCC < 3.3 */
 #define NON_NULL_PARAM(x)
@@ -474,29 +465,34 @@ G_STMT_START {			\
  * This is the same G_GNUC_FORMAT() but for function pointers. Older versions
  * of GCC do not allow function attributes for function pointers.
  */
-#if HAVE_GCC(3, 0)
+#if defined(HASATTRIBUTE) && HAS_GCC(3, 0)
 #define PRINTF_FUNC_PTR(x, y) __attribute__((format(__printf__, (x), (y))))
 #else /* GCC < 3.0 */
 #define PRINTF_FUNC_PTR(x, y)
 #endif
 
-/* Functions using this attribute cause a warning if the returned
- * value is not used. */
-#if HAVE_GCC(3, 4)
+/*
+ * Functions using this attribute cause a warning if the returned
+ * value is not used.
+ */
+#if defined(HASATTRIBUTE) && HAS_GCC(3, 4)
 #define WARN_UNUSED_RESULT __attribute__((__warn_unused_result__))
 #else /* GCC < 3.4 */
 #define WARN_UNUSED_RESULT
 #endif
 
-/* Instructs the compiler to emit code for this function even if it is
- * or seems to be unused. */
-#if HAVE_GCC(3, 1)
+/*
+ * Instructs the compiler to emit code for this function even if it is
+ * or seems to be unused.
+ */
+#if defined(HASATTRIBUTE) && HAS_GCC(3, 1)
 #define KEEP_FUNCTION __attribute__((__used__))
 #else /* GCC < 3.1 || !GCC */
 #define KEEP_FUNCTION
 #endif
 
-/* The antidote for WARN_UNUSED_RESULT. This attribute is sometimes
+/*
+ * The antidote for WARN_UNUSED_RESULT. This attribute is sometimes
  * misused for functions that return a result which SHOULD NOT be
  * ignored in contrast to MUST NOT. Unfortunately, a simple "(void)"
  * does not suppress this warning.
@@ -504,21 +500,25 @@ G_STMT_START {			\
 #define IGNORE_RESULT(x) \
 	G_STMT_START { switch (0 != (x)) { default: ; } }  G_STMT_END
 
-/* Functions using this attribute cause a warning if the variable
- * argument list does not contain a NULL pointer. */
+/*
+ * Functions using this attribute cause a warning if the variable
+ * argument list does not contain a NULL pointer.
+ */
 #ifndef G_GNUC_NULL_TERMINATED
-#if HAVE_GCC(4, 0)
+#if defined(HASATTRIBUTE) && HAS_GCC(4, 0)
 #define G_GNUC_NULL_TERMINATED __attribute__((__sentinel__))
 #else	/* GCC < 4 */
 #define G_GNUC_NULL_TERMINATED
 #endif	/* GCC >= 4 */
 #endif	/* G_GNUC_NULL_TERMINATED */
 
-/* Define G_LIKELY() and G_UNLIKELY() so that they are available when
+/*
+ * Define G_LIKELY() and G_UNLIKELY() so that they are available when
  * using GLib 1.2 as well. These allow optimization by static branch
- * prediction with GCC. */
+ * prediction with GCC.
+ */
 #ifndef G_LIKELY
-#if HAVE_GCC(3, 4)	/* Just a guess, a Configure check would be better */
+#if HAS_GCC(3, 4)	/* Just a guess, a Configure check would be better */
 #define G_LIKELY(x)		(__builtin_expect(x, 1))
 #define G_UNLIKELY(x)	(__builtin_expect(x, 0))
 #else /* !GCC >= 3.4 */
@@ -527,21 +527,39 @@ G_STMT_START {			\
 #endif /* GCC >= 3.4 */
 #endif /* !G_LIKELY */
 
+/**
+ * A pure function has no effects except its return value and the return value
+ * depends only on the parameters and/or global variables.
+ */
+#ifndef G_GNUC_PURE
+#if defined(HASATTRIBUTE) && HAS_GCC(2, 96)
+#define G_GNUC_PURE __attribute__((__pure__))
+#else
+#define G_GNUC_PURE
+#endif	/* GCC >= 3.0 */
+#endif	/* G_GNUC_MALLOC */
+
 #ifndef G_GNUC_MALLOC
-#if HAVE_GCC(3, 0)
+#if defined(HASATTRIBUTE) && HAS_GCC(3, 0)
 #define G_GNUC_MALLOC __attribute__((__malloc__))
 #else
 #define G_GNUC_MALLOC
 #endif	/* GCC >= 3.0 */
 #endif	/* G_GNUC_MALLOC */
 
-#if HAVE_GCC(3, 1)
+#if defined(HASATTRIBUTE) && HAS_GCC(3, 1)
 #define ALWAYS_INLINE __attribute__((__always_inline__))
 #else
 #define ALWAYS_INLINE
 #endif	/* GCC >= 3.1 */
 
-#ifdef HAS_REGPARM
+#if defined(HASATTRIBUTE) && HAS_GCC(3, 1)
+#define NO_INLINE __attribute__((__noinline__))
+#else
+#define NO_INLINE
+#endif	/* GCC >= 3.1 */
+
+#if defined(HASATTRIBUTE) && defined(HAS_REGPARM)
 #define REGPARM(n)	__attribute__((__regparm__((n))))
 #else
 #define REGPARM(n)
@@ -574,35 +592,36 @@ G_STMT_START {			\
 #define STATIC_ASSERT(x) \
 	do { switch (0) { case ((x) ? 1 : 0): case 0: break; } } while(0)
 
-#if defined(GTA_PATCHLEVEL) && (GTA_PATCHLEVEL != 0)
-#define GTA_VERSION_NUMBER \
-	STRINGIFY(GTA_VERSION) "." \
-	STRINGIFY(GTA_SUBVERSION) "." \
-	STRINGIFY(GTA_PATCHLEVEL) GTA_REVCHAR
-#else
-#define GTA_VERSION_NUMBER \
-	STRINGIFY(GTA_VERSION) "." STRINGIFY(GTA_SUBVERSION) GTA_REVCHAR
-#endif
-
-#define GTA_PORT			6346	/**< Default "standard" port */
 #define MAX_HOSTLEN			256		/**< Max length for FQDN host */
-#define GTA_BUILD					"$Revision: 15162 $"
 
 /* The next two defines came from huge.h --- Emile */
 #define SHA1_BASE16_SIZE 	40		/**< 160 bits in base16 representation */
 #define SHA1_BASE32_SIZE 	32		/**< 160 bits in base32 representation */
 #define SHA1_RAW_SIZE		20		/**< 160 bits in binary representation */
 
-#define TTH_BASE32_SIZE 	39		/**< 160 bits in base32 representation */
-#define TTH_RAW_SIZE		24		/**< 160 bits in binary representation */
+#define TTH_BASE16_SIZE 	48		/**< 192 bits in base16 representation */
+#define TTH_BASE32_SIZE 	39		/**< 192 bits in base32 representation */
+#define TTH_RAW_SIZE		24		/**< 192 bits in binary representation */
 
 #define BITPRINT_BASE32_SIZE 	72	/**< 352 bits in base32 representation
 									 **  includes dot between sha1 and tiger */
+#define BITPRINT_BASE16_SIZE	89	/**< 352 bits in base16 + separation dot */
 #define BITPRINT_RAW_SIZE		44	/**< 352 bits in binary representation */
 
 /** Maximum bytes in filename i.e., including NUL */
 #define	FILENAME_MAXBYTES 256
 
+/* Maximum path length, in bytes */
+#if defined(PATH_MAX)
+#define MAX_PATH_LEN	PATH_MAX	/* POSIX, first choice */
+#elif defined(MAXPATHLEN)
+#define MAX_PATH_LEN	MAXPATHLEN
+#elif defined(PATH_LEN)
+#define MAX_PATH_LEN	PATH_LEN
+#else
+#define MAX_PATH_LEN	2048
+#endif
+
 /*
  * Forbidden glib calls.
  */
@@ -628,10 +647,8 @@ typedef gboolean (*reclaim_fd_t)(void);
 #  ifdef gettext_noop
 #    define N_(String) gettext_noop(String)
 #  else
-#    define N_(String) (String)
+#    define N_(String) "" String ""
 #  endif
-
-
 #else
 #  define textdomain(String) (String)
 #  define gettext(String) (String)
@@ -640,7 +657,7 @@ typedef gboolean (*reclaim_fd_t)(void);
 #  define bindtextdomain(Domain,Directory) (Domain)
 #  define ngettext(Single, Plural, Number) ((Number) == 1 ? (Single) : (Plural))
 #  define _(String) (String)
-#  define N_(String) (String)
+#  define N_(String) "" String ""
 #  define Q_(String) g_strip_context ((String), (String))
 #endif /* ENABLE_NLS */
 
@@ -668,18 +685,29 @@ ngettext_(const gchar *msg1, const gchar *msg2, gulong n)
  */
 #define RCSID(x) \
 static KEEP_FUNCTION inline const char *	\
-get_rcsid(void)	\
+get_rcsid_string(void)	\
 {	\
 	static const char rcsid[] = "@(#) " x;	\
-	const char *s = rcsid;	\
-	while (*s != '\0') {	\
-		if (*s++ == '$')	\
-			break;	\
-	}	\
-	return s;	\
+	return rcsid;	\
 }
 
+/**
+ * Composes a 32-bit native endian integer from four characters (bytes) given
+ * in big endian byte order.
+ */
+#define FOURCC_NATIVE(a,b,c,d) ( \
+	((guint32) (unsigned char) ((a) & 0xffU) << 24) | \
+	((guint32) (unsigned char) ((b) & 0xffU) << 16) | \
+	((guint32) (unsigned char) ((c) & 0xffU) << 8)  | \
+	((guint32) (unsigned char) ((d) & 0xffU)))
+
+/**
+ * Zero memory used by structure pointed at.
+ */
+#define ZERO(x)		memset((x), 0, sizeof *(x))
+
 #include "casts.h"
+#include "lib/fast_assert.h"
 
 #endif /* _common_h_ */
 
