@@ -1,5 +1,5 @@
 /*
- * $Id: alive.c,v 1.12 2005/06/25 01:37:38 daichik Exp $
+ * $Id: alive.c 19142 2011-05-23 20:44:04Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,16 +35,18 @@
 
 #include "common.h"
 
-RCSID("$Id: alive.c,v 1.12 2005/06/25 01:37:38 daichik Exp $");
+RCSID("$Id: alive.c 19142 2011-05-23 20:44:04Z rmanfredi $")
 
 #include "alive.h"
 #include "nodes.h"
 #include "guid.h"
 #include "gmsg.h"
-#include "pmsg.h"
 #include "mq.h"
 #include "pcache.h"
 #include "lib/atoms.h"
+#include "lib/glib-missing.h"
+#include "lib/pmsg.h"
+#include "lib/tm.h"
 #include "lib/walloc.h"
 #include "if/gnet_property_priv.h"
 #include "lib/override.h"		/* Must be the last header included */
@@ -57,8 +59,8 @@ RCSID("$Id: alive.c,v 1.12 2005/06/25 01:37:38 daichik Exp $");
 struct alive {
 	struct gnutella_node *node;
 	GSList *pings;				/**< Pings we sent (struct alive_ping) */
-	gint count;					/**< Amount of pings in list */
-	gint maxcount;				/**< Maximum amount of pings we remember */
+	int count;					/**< Amount of pings in list */
+	int maxcount;				/**< Maximum amount of pings we remember */
 	guint32 min_rt;				/**< Minimum roundtrip time (ms) */
 	guint32 max_rt;				/**< Maximim roundtrip time (ms) */
 	guint32 avg_rt;				/**< Average (EMA) roundtrip time (ms) */
@@ -66,8 +68,8 @@ struct alive {
 };
 
 struct alive_ping {
-	gchar *muid;				/**< The GUID of the message */
-	GTimeVal sent;				/**< Time at which we sent the message */
+	const struct guid *muid;	/**< The GUID of the message */
+	tm_t sent;					/**< Time at which we sent the message */
 };
 
 static void alive_trim_upto(struct alive *a, GSList *item);
@@ -76,14 +78,14 @@ static void alive_trim_upto(struct alive *a, GSList *item);
  * Create an alive_ping, with proper message ID.
  */
 static struct alive_ping *
-ap_make(gchar *muid)
+ap_make(struct guid *muid)
 {
 	struct alive_ping *ap;
 
-	ap = walloc(sizeof(*ap));
+	WALLOC(ap);
 
 	ap->muid = atom_guid_get(muid);
-	g_get_current_time(&ap->sent);
+	tm_now_exact(&ap->sent);
 
 	return ap;
 }
@@ -95,7 +97,7 @@ static void
 ap_free(struct alive_ping *ap)
 {
 	atom_guid_free(ap->muid);
-	wfree(ap, sizeof(*ap));
+	WFREE(ap);
 }
 
 /**
@@ -103,11 +105,11 @@ ap_free(struct alive_ping *ap)
  * Returned as an opaque pointer.
  */
 gpointer
-alive_make(struct gnutella_node *n, gint max)
+alive_make(struct gnutella_node *n, int max)
 {
 	struct alive *a;
 
-	a = g_malloc0(sizeof(*a));
+	WALLOC0(a);
 	a->node = n;
 	a->maxcount = max;
 	a->min_rt = INFINITY;
@@ -121,14 +123,14 @@ alive_make(struct gnutella_node *n, gint max)
 void
 alive_free(gpointer obj)
 {
-	struct alive *a = (struct alive *) obj;
-	GSList *l;
+	struct alive *a = obj;
+	GSList *sl;
 
-	for (l = a->pings; l; l = l->next)
-		ap_free((struct alive_ping *) l->data);
+	for (sl = a->pings; sl; sl = g_slist_next(sl))
+		ap_free(sl->data);
 
-	g_slist_free(a->pings);
-	G_FREE_NULL(a);
+	gm_slist_free_null(&a->pings);
+	WFREE(a);
 }
 
 /**
@@ -142,7 +144,7 @@ alive_remove_link(struct alive *a, GSList *item)
 
 	a->pings = g_slist_remove_link(a->pings, item);
 	a->count--;
-	ap_free((struct alive_ping *) item->data);
+	ap_free(item->data);
 	g_slist_free_1(item);
 }
 
@@ -150,15 +152,15 @@ alive_remove_link(struct alive *a, GSList *item)
  * Drop alive ping record when message is dropped.
  */
 static void
-alive_ping_drop(struct alive *a, const gchar *muid)
+alive_ping_drop(struct alive *a, const struct guid *muid)
 {
-	GSList *l;
+	GSList *sl;
 
-	for (l = a->pings; l; l = l->next) {
-		struct alive_ping *ap = (struct alive_ping *) l->data;
+	for (sl = a->pings; sl; sl = g_slist_next(sl)) {
+		struct alive_ping *ap = sl->data;
 
 		if (guid_eq(ap->muid, muid)) {		/* Found it! */
-			alive_remove_link(a, l);
+			alive_remove_link(a, sl);
 			return;
 		}
 	}
@@ -169,13 +171,13 @@ alive_ping_drop(struct alive *a, const gchar *muid)
  * We send only the LATEST registered one.
  */
 static gboolean
-alive_ping_can_send(pmsg_t *mb, mqueue_t *q)
+alive_ping_can_send(pmsg_t *mb, const mqueue_t *q)
 {
-	gnutella_node_t *n = mq_node(q);
-	gchar *muid = pmsg_start(mb);
-	struct alive *a = (struct alive *) n->alive_pings;
+	const gnutella_node_t *n = mq_node(q);
+	struct alive *a = n->alive_pings;
+	const struct guid *muid = cast_to_guid_ptr_const(pmsg_start(mb));
 
-	g_assert(((struct gnutella_header *) muid)->function == GTA_MSG_INIT);
+	g_assert(gnutella_header_get_function(muid) == GTA_MSG_INIT);
 	g_assert(a->count == 0 || a->pings != NULL);
 
 	/*
@@ -184,8 +186,8 @@ alive_ping_can_send(pmsg_t *mb, mqueue_t *q)
 	 */
 
 	if (a->count) {
-		GSList *l = g_slist_last(a->pings);
-		struct alive_ping *ap = (struct alive_ping *) l->data;
+		const GSList *l = g_slist_last(a->pings);
+		const struct alive_ping *ap = l->data;
 
 		if (guid_eq(ap->muid, muid))		/* It's the latest one */
 			return TRUE;					/* We can send it */
@@ -197,10 +199,9 @@ alive_ping_can_send(pmsg_t *mb, mqueue_t *q)
 
 	alive_ping_drop(a, muid);
 
-	if (dbg > 1)
-		printf("ALIVE node %s (%s) dropped old alive ping %s, %d remain\n",
-			node_ip(a->node), node_vendor(a->node),
-			guid_hex_str(muid), a->count);
+	if (GNET_PROPERTY(alive_debug))
+		g_debug("ALIVE %s dropped old alive ping %s, %d remain",
+			node_infostr(a->node), guid_hex_str(muid), a->count);
 
 	return FALSE;		/* Don't send message */
 }
@@ -211,18 +212,19 @@ alive_ping_can_send(pmsg_t *mb, mqueue_t *q)
 static void
 alive_pmsg_free(pmsg_t *mb, gpointer arg)
 {
-	struct gnutella_node *n = (struct gnutella_node *) arg;
+	struct gnutella_node *n = arg;
 
 	g_assert(pmsg_is_extended(mb));
 
 	if (pmsg_was_sent(mb)) {
 		n->n_ping_sent++;
-		if (dbg > 1)
-			printf("ALIVE sent ping %s to node %s (%s)\n",
-				guid_hex_str(pmsg_start(mb)), node_ip(n), node_vendor(n));
+		if (GNET_PROPERTY(alive_debug))
+			g_debug("ALIVE sent ping %s to %s",
+				guid_hex_str(cast_to_guid_ptr_const(pmsg_start(mb))),
+				node_infostr(n));
 	} else {
-		gchar *muid = pmsg_start(mb);
-		struct alive *a = (struct alive *) n->alive_pings;
+		struct guid *muid = cast_to_guid_ptr(pmsg_start(mb));
+		struct alive *a = n->alive_pings;
 
 		g_assert(a->node == n);
 
@@ -238,32 +240,31 @@ alive_pmsg_free(pmsg_t *mb, gpointer arg)
 gboolean
 alive_send_ping(gpointer obj)
 {
-	struct alive *a = (struct alive *) obj;
-	gchar muid[16];
+	struct alive *a = obj;
+	struct guid muid;
 	struct alive_ping *ap;
-	struct gnutella_msg_init *m;
+	gnutella_msg_init_t *m;
 	guint32 size;
 	pmsg_t *mb;
 
-	STATIC_ASSERT(23 == sizeof *m);
 	g_assert(a->count == 0 || a->pings != NULL);
 
 	if (a->count >= a->maxcount)
 		return FALSE;
 
-	guid_ping_muid(muid);
+	guid_ping_muid(&muid);
 
-	ap = ap_make(muid);
+	ap = ap_make(&muid);
 	a->count++;
 	a->pings = g_slist_append(a->pings, ap);
-	a->node->last_alive_ping = time(NULL);
+	a->node->last_alive_ping = tm_time();
 
 	/*
 	 * Build ping message and attach a pre-sender callback, as well as
 	 * a free routine to see whether the message is sent.
 	 */
 
-	m = build_ping_msg(muid, 1, FALSE, &size);
+	m = build_ping_msg(&muid, 1, FALSE, &size);
 
 	g_assert(size == sizeof(*m));	/* No trailing GGEP extension */
 
@@ -279,14 +280,14 @@ alive_send_ping(gpointer obj)
  * Acknowledge alive_ping `ap', and update roundtrip statistics in `a'.
  */
 static void
-ap_ack(struct alive_ping *ap, struct alive *a)
+ap_ack(const struct alive_ping *ap, struct alive *a)
 {
 	GTimeVal now;
-	gint delay;					/**< Between sending and reception, in ms */
+	int delay;					/**< Between sending and reception, in ms */
 
-	g_get_current_time(&now);
+	tm_now_exact(&now);
 
-	delay = (gint) ((now.tv_sec - ap->sent.tv_sec) * 1000 +
+	delay = (int) ((now.tv_sec - ap->sent.tv_sec) * 1000 +
 		(now.tv_usec - ap->sent.tv_usec) / 1000);
 
 	a->last_rt = delay;
@@ -295,12 +296,12 @@ ap_ack(struct alive_ping *ap, struct alive *a)
 	 * Update min-max roundtrips.
 	 */
 
-	if (delay < (gint) a->min_rt) {
+	if (delay < (int) a->min_rt) {
 		if (a->min_rt == INFINITY)
 			a->avg_rt = delay;			/* First time */
 		a->min_rt = delay;
 	}
-	if (delay > (gint) a->max_rt)
+	if (delay > (int) a->max_rt)
 		a->max_rt = delay;
 
 	/*
@@ -310,10 +311,10 @@ ap_ack(struct alive_ping *ap, struct alive *a)
 
 	a->avg_rt += (delay >> 2) - (a->avg_rt >> 2);
 
-	if (dbg > 4)
-		printf("ALIVE node %s (%s) "
-		"delay=%dms min=%dms, max=%dms, agv=%dms [%d queued]\n",
-			node_ip(a->node), node_vendor(a->node),
+	if (GNET_PROPERTY(alive_debug) > 1)
+		g_debug("ALIVE %s "
+		"delay=%dms min=%dms, max=%dms, agv=%dms [%d queued]",
+			node_infostr(a->node),
 			a->last_rt, a->min_rt, a->max_rt, a->avg_rt, a->count);
 }
 
@@ -323,20 +324,17 @@ ap_ack(struct alive_ping *ap, struct alive *a)
 static void
 alive_trim_upto(struct alive *a, GSList *item)
 {
-	GSList *l;
+	GSList *sl;
+	gboolean found = FALSE;
 
 	g_assert(a->count && a->pings != NULL);
 
-	for (l = a->pings; l; l = a->pings) {
-		alive_remove_link(a, l);
-
-		if (l == item) {		/* Found it, removed it */
-			g_assert(a->count >= 0);
-			return;
-		}
+	for (sl = a->pings; sl && !found; sl = a->pings) {
+		found = sl == item;
+		alive_remove_link(a, sl);
 	}
-
-	g_assert_not_reached();				/* Must have found the item */
+	g_assert(found); /* Must have found the item */
+	g_assert(a->count >= 0);
 }
 
 /**
@@ -345,22 +343,22 @@ alive_trim_upto(struct alive *a, GSList *item)
  * @return TRUE if it was indeed an ACK for a ping we sent.
  */
 gboolean
-alive_ack_ping(gpointer obj, gchar *muid)
+alive_ack_ping(gpointer obj, const struct guid *muid)
 {
-	struct alive *a = (struct alive *) obj;
-	GSList *l;
+	struct alive *a = obj;
+	GSList *sl;
 
 	g_assert(a->count == 0 || a->pings != NULL);
 
-	for (l = a->pings; l; l = l->next) {
-		struct alive_ping *ap = (struct alive_ping *) l->data;
+	for (sl = a->pings; sl; sl = g_slist_next(sl)) {
+		const struct alive_ping *ap = sl->data;
 
 		if (guid_eq(ap->muid, muid)) {		/* Found it! */
-			if (dbg > 1)
-				printf("ALIVE got alive pong %s from %s (%s)\n",
-					guid_hex_str(muid), node_ip(a->node), node_vendor(a->node));
+			if (GNET_PROPERTY(alive_debug))
+				g_debug("ALIVE got alive pong %s from %s",
+					guid_hex_str(muid), node_infostr(a->node));
 			ap_ack(ap, a);
-			alive_trim_upto(a, l);
+			alive_trim_upto(a, sl);
 			return TRUE;
 		}
 	}
@@ -373,11 +371,11 @@ alive_ack_ping(gpointer obj, gchar *muid)
  * replying to an alive ping.
  */
 void
-alive_ack_first(gpointer obj, gchar *muid)
+alive_ack_first(gpointer obj, const struct guid *muid)
 {
-	struct alive *a = (struct alive *) obj;
-	GSList *l;
+	struct alive *a = obj;
 	struct alive_ping *ap;
+	GSList *sl;
 
 	g_assert(a->count == 0 || a->pings != NULL);
 
@@ -392,19 +390,19 @@ alive_ack_first(gpointer obj, gchar *muid)
 	 * Assume they're replying to the first ping we sent, ignore otherwise.
 	 */
 
-	l = a->pings;		/* First ping we sent, chronologically */
+	sl = a->pings;		/* First ping we sent, chronologically */
 
-	if (l == NULL)
+	if (NULL == sl)
 		return;
 
-	ap = (struct alive_ping *) l->data;
+	ap = sl->data;
 
-	if (dbg > 1)
-		printf("ALIVE TTL=0 ping from %s (%s), assuming it acks %s\n",
-			node_ip(a->node), node_vendor(a->node), guid_hex_str(ap->muid));
+	if (GNET_PROPERTY(alive_debug))
+		g_debug("ALIVE TTL=0 ping from %s, assuming it acks %s",
+			node_infostr(a->node), guid_hex_str(ap->muid));
 
 	ap_ack(ap, a);
-	alive_remove_link(a, l);
+	alive_remove_link(a, sl);
 }
 
 /**
@@ -412,14 +410,14 @@ alive_ack_first(gpointer obj, gchar *muid)
  * Values are expressed in milliseconds.
  */
 void
-alive_get_roundtrip_ms(gpointer obj, guint32 *avg, guint32 *last)
+alive_get_roundtrip_ms(gconstpointer obj, guint32 *avg, guint32 *last)
 {
-	const struct alive *a = (const struct alive *) obj;
+	const struct alive *a = obj;
 
-	g_assert(obj != NULL);
+	g_assert(NULL != obj);
 
 	if (avg)	*avg = a->avg_rt;
 	if (last)	*last = a->last_rt;
 }
 
-/* vi: set ts=4: */
+/* vi: set ts=4 sw=4 cindent: */
