@@ -1,5 +1,5 @@
 /*
- * $Id: bogons.c 11437 2006-08-05 00:24:20Z cbiere $
+ * $Id: bogons.c 19266 2011-06-15 16:02:31Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -35,15 +35,20 @@
 
 #include "common.h"
 
-RCSID("$Id: bogons.c 11437 2006-08-05 00:24:20Z cbiere $")
+RCSID("$Id: bogons.c 19266 2011-06-15 16:02:31Z rmanfredi $")
 
 #include "bogons.h"
 #include "settings.h"
 
+#include "lib/ascii.h"
 #include "lib/file.h"
-#include "lib/misc.h"
 #include "lib/glib-missing.h"
+#include "lib/halloc.h"
+#include "lib/host_addr.h"
 #include "lib/iprange.h"
+#include "lib/parse.h"
+#include "lib/path.h"
+#include "lib/tm.h"
 #include "lib/walloc.h"
 #include "lib/watcher.h"
 
@@ -52,30 +57,36 @@ RCSID("$Id: bogons.c 11437 2006-08-05 00:24:20Z cbiere $")
 
 #include "lib/override.h"		/* Must be the last header included */
 
-#define THERE	GUINT_TO_POINTER(0x2)
+static const char bogon[] = "bogon";
 
-static const gchar bogons_file[] = "bogons.txt";
-static const gchar bogons_what[] = "Bogus IP addresses";
+static const char bogons_file[] = "bogons.txt";
+static const char bogons_what[] = "Bogus IP addresses";
 
-static gpointer bogons_db;		/**< The database of bogus CIDR ranges */
+static struct iprange_db *bogons_db; /**< The database of bogus CIDR ranges */
+static time_t bogons_mtime;			 /**< Modification time of loaded file */
 
 /**
  * Load bogons data from the supplied FILE.
  *
  * @returns the amount of entries loaded.
  */
-static gint
+static int
 bogons_load(FILE *f)
 {
-	gchar line[1024];
-	gchar *p;
+	char line[1024];
+	char *p;
 	guint32 ip, netmask;
 	int linenum = 0;
-	gint count = 0;
-	gint bits;
+	int bits;
 	iprange_err_t error;
+	filestat_t buf;
 
-	bogons_db = iprange_make(NULL, NULL);
+	bogons_db = iprange_new();
+	if (-1 == fstat(fileno(f), &buf)) {
+		g_warning("cannot stat %s: %s", bogons_file, g_strerror(errno));
+	} else {
+		bogons_mtime = buf.st_mtime;
+	}
 
 	while (fgets(line, sizeof(line), f)) {
 		linenum++;
@@ -104,19 +115,12 @@ bogons_load(FILE *f)
 		}
 
 		bits = netmask_to_cidr(netmask);
-		error = iprange_add_cidr(bogons_db, ip, bits, THERE);
+		error = iprange_add_cidr(bogons_db, ip, bits,
+					deconstify_gpointer(bogon));
 
 		switch (error) {
 		case IPR_ERR_OK:
 			break;
-		case IPR_ERR_RANGE_OVERLAP:
-			error = iprange_add_cidr_force(bogons_db, ip, bits, THERE, NULL);
-			if (error == IPR_ERR_OK) {
-				g_warning("%s, line %d: "
-					"entry \"%s\" (%s/%d) superseded earlier smaller range",
-					bogons_file, linenum, line, ip_to_string(ip), bits);
-				break;
-			}
 			/* FALL THROUGH */
 		default:
 			g_warning("%s, line %d: rejected entry \"%s\" (%s/%d): %s",
@@ -124,21 +128,17 @@ bogons_load(FILE *f)
 				iprange_strerror(error));
 			continue;
 		}
-
-		count++;
 	}
 
-	if (dbg) {
-		iprange_stats_t stats;
+	iprange_sync(bogons_db);
 
-		iprange_get_stats(bogons_db, &stats);
-
-		g_message("loaded %d bogus IP ranges", count);
-		g_message("bogons stats: count=%d level2=%d heads=%d enlisted=%d",
-			stats.count, stats.level2, stats.heads, stats.enlisted);
+	if (GNET_PROPERTY(reload_debug)) {
+		g_debug("loaded %u bogus IP ranges (%u hosts)",
+			iprange_get_item_count(bogons_db),
+			iprange_get_host_count(bogons_db));
 	}
 
-	return count;
+	return iprange_get_item_count(bogons_db);
 }
 
 /**
@@ -146,11 +146,11 @@ bogons_load(FILE *f)
  * addresses changed.
  */
 static void
-bogons_changed(const gchar *filename, gpointer unused_udata)
+bogons_changed(const char *filename, gpointer unused_udata)
 {
 	FILE *f;
-	gchar buf[80];
-	gint count;
+	char buf[80];
+	int count;
 
 	(void) unused_udata;
 
@@ -182,8 +182,8 @@ static void
 bogons_retrieve(void)
 {
 	FILE *f;
-	gint idx;
-	gchar *filename;
+	int idx;
+	char *filename;
 #ifndef OFFICIAL_BUILD
 	static file_path_t fp[3];
 #else
@@ -204,7 +204,7 @@ bogons_retrieve(void)
 
 	filename = make_pathname(fp[idx].dir, fp[idx].name);
 	watcher_register(filename, bogons_changed, NULL);
-	G_FREE_NULL(filename);
+	HFREE_NULL(filename);
 
 	bogons_load(f);
 	fclose(f);
@@ -225,10 +225,7 @@ bogons_init(void)
 void
 bogons_close(void)
 {
-	if (bogons_db) {
-		iprange_free_each(bogons_db, NULL);
-		bogons_db = NULL;
-	}
+	iprange_free(&bogons_db);
 }
 
 /**
@@ -239,10 +236,20 @@ bogons_close(void)
 gboolean
 bogons_check(const host_addr_t ha)
 {
-	if (NET_TYPE_IPV4 == host_addr_net(ha)) {
+	/*
+	 * If the bogons file is too ancient, there is a risk it may flag an
+	 * IP as bogus whereas it is no longer reserved.  IPv4 address shortage
+	 * makes that likely.
+	 *		--RAM, 2010-11-07
+	 */
+
+	if (delta_time(tm_time(), bogons_mtime) > 15552000)	/* ~6 months */
+		return !host_addr_is_routable(ha);
+
+	if (host_addr_is_ipv4(ha)) {
 		guint32 ip = host_addr_ipv4(ha);
-		return bogons_db != NULL && THERE == iprange_get(bogons_db, ip);
-	} else if (NET_TYPE_IPV6 == host_addr_net(ha)) {
+		return bogons_db && iprange_get(bogons_db, ip);
+	} else if (host_addr_is_ipv6(ha)) {
 		host_addr_t to;
 
 		if (host_addr_convert(ha, &to, NET_TYPE_IPV4))
