@@ -1,5 +1,5 @@
 /*
- * $Id: bsched.h 12599 2006-12-10 15:21:49Z cbiere $
+ * $Id: bsched.h 19067 2011-05-11 10:15:00Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -38,21 +38,18 @@
 
 #include "common.h"
 
-#include "sockets.h"		/* For enum socket_type */
-
+#include "lib/gnet_host.h"
 #include "lib/inputevt.h"
 #include "lib/tm.h"
-#include "if/core/hosts.h"	/* For gnet_host_t */
 #include "if/core/nodes.h"	/* For node_peer_t */
 #include "if/core/bsched.h"
+#include "if/core/sockets.h"
 
 typedef struct sendfile_ctx {
 	void *map;
-	off_t map_start, map_end;
+	fileoffset_t map_start, map_end;
 } sendfile_ctx_t;
 
-struct iovec;
-
 /*
  * Public interface.
  */
@@ -68,20 +65,19 @@ void bsched_enable_all(void);
 bio_source_t *bsched_source_add(bsched_bws_t bs, wrap_io_t *wio, guint32 flags,
 	inputevt_handler_t callback, gpointer arg);
 void bsched_source_remove(bio_source_t *bio);
-void bsched_set_bandwidth(bsched_bws_t bs, gint bandwidth);
-bio_source_t *bsched_source_add(bsched_bws_t bs, wrap_io_t *wio, guint32 flags,
-	inputevt_handler_t callback, gpointer arg);
+void bsched_set_bandwidth(bsched_bws_t bs, int bandwidth);
 void bio_add_callback(bio_source_t *bio,
 	inputevt_handler_t callback, gpointer arg);
 void bio_remove_callback(bio_source_t *bio);
+unsigned bio_get_bufsize(const bio_source_t *bio, enum socket_buftype type);
 ssize_t bio_write(bio_source_t *bio, gconstpointer data, size_t len);
-ssize_t bio_writev(bio_source_t *bio, struct iovec *iov, gint iovcnt);
+ssize_t bio_writev(bio_source_t *bio, iovec_t *iov, int iovcnt);
 ssize_t bio_sendto(bio_source_t *bio, const gnet_host_t *to,
 	gconstpointer data, size_t len);
-ssize_t bio_sendfile(sendfile_ctx_t *ctx, bio_source_t *bio, gint in_fd,
-	off_t *offset, size_t len);
+ssize_t bio_sendfile(sendfile_ctx_t *ctx, bio_source_t *bio, int in_fd,
+	fileoffset_t *offset, size_t len);
 ssize_t bio_read(bio_source_t *bio, gpointer data, size_t len);
-ssize_t bio_readv(bio_source_t *bio, struct iovec *iov, gint iovcnt);
+ssize_t bio_readv(bio_source_t *bio, iovec_t *iov, int iovcnt);
 ssize_t bws_write(bsched_bws_t bs, wrap_io_t *wio,
 			gconstpointer data, size_t len);
 ssize_t bws_read(bsched_bws_t bs, wrap_io_t *wio, gpointer data, size_t len);
@@ -95,8 +91,10 @@ void bws_sock_connect_failed(enum socket_type type);
 void bws_sock_closed(enum socket_type type, gboolean remote);
 gboolean bws_can_connect(enum socket_type type);
 
-void bws_udp_count_written(gint len);
-void bws_udp_count_read(gint len);
+void bws_udp_count_read(int len, gboolean dht);
+gboolean bws_allow_stealing(bsched_bws_t bws, gboolean allow);
+gboolean bws_ignore_stolen(bsched_bws_t bws, gboolean ignore);
+gboolean bws_uniform_allocation(bsched_bws_t bws, gboolean uniform);
 
 gboolean bsched_enough_up_bandwidth(void);
 gboolean bsched_saturated(bsched_bws_t bws);
@@ -105,10 +103,15 @@ gulong bsched_avg_bps(bsched_bws_t bws);
 gulong bsched_pct(bsched_bws_t bws);
 gulong bsched_avg_pct(bsched_bws_t bws);
 gulong bsched_bw_per_second(bsched_bws_t bws);
+int bsched_urgent(bsched_bws_t bws);
+void bsched_set_urgent(bsched_bws_t bws, int amount);
 
 void bsched_config_steal_http_gnet(void);
 void bsched_config_steal_gnet(void);
 
+bsched_bws_t bsched_out_select_by_addr(const host_addr_t);
+bsched_bws_t bsched_in_select_by_addr(const host_addr_t);
+
 #endif	/* _core_bsched_h_ */
 
 /* vi: set ts=4 sw=4 cindent: */
