@@ -1,5 +1,5 @@
 /*
- * $Id: dh.c,v 1.15 2005/08/31 21:37:28 rmanfredi Exp $
+ * $Id: dh.c 19142 2011-05-23 20:44:04Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -35,15 +35,17 @@
 
 #include "common.h"
 
-RCSID("$Id: dh.c,v 1.15 2005/08/31 21:37:28 rmanfredi Exp $");
+RCSID("$Id: dh.c 19142 2011-05-23 20:44:04Z rmanfredi $")
 
 #include "dh.h"
 #include "nodes.h"
 #include "gmsg.h"
-#include "mq.h"
+#include "mq_tcp.h"
+#include "mq_udp.h"
 #include "gnet_stats.h"
 
 #include "lib/atoms.h"
+#include "lib/glib-missing.h"
 #include "lib/misc.h"
 #include "lib/tm.h"
 #include "lib/walloc.h"
@@ -56,7 +58,7 @@ RCSID("$Id: dh.c,v 1.15 2005/08/31 21:37:28 rmanfredi Exp $");
 #define DH_MIN_HITS		250		/**< Minimum amount of hits we try to relay */
 #define DH_POPULAR_HITS	500		/**< Query deemed popular after that many hits */
 #define DH_MAX_HITS		1000	/**< Maximum hits after which we heavily drop */
-#define DH_THRESH_HITS	10		/**< Consider we have no hits if less than that */
+#define DH_THRESH_HITS	10		/**< We have no hits if less than that */
 
 /**
  * Information about query hits received.
@@ -72,7 +74,7 @@ typedef struct dqhit {
 /*
  * Meta-information about the query hit message.
  */
-struct pmsg_info {
+struct dh_pmsg_info {
 	guint32 hits;			/**< Amount of query hits held in message */
 };
 
@@ -125,12 +127,14 @@ dh_table_clear(GHashTable *ht)
  * Free specified hash table.
  */
 static void
-dh_table_free(GHashTable *ht)
+dh_table_free(GHashTable **ptr)
 {
-	g_assert(ht != NULL);
-
-	g_hash_table_foreach_remove(ht, free_muid_true, NULL);
-	g_hash_table_destroy(ht);
+	if (*ptr) {
+		GHashTable *ht = *ptr;
+		g_hash_table_foreach_remove(ht, free_muid_true, NULL);
+		g_hash_table_destroy(ht);
+		*ptr = NULL;
+	}
 }
 
 /**
@@ -139,12 +143,15 @@ dh_table_free(GHashTable *ht)
  * @returns located record, or NULL if not found.
  */
 static dqhit_t *
-dh_locate(const gchar *muid)
+dh_locate(const struct guid *muid)
 {
 	gboolean found = FALSE;
 	gpointer key;
 	gpointer value;
 
+	if (NULL == by_muid_old)
+		return NULL;		/* DH layer shutdown occurred already */
+
 	/*
 	 * Look in the old table first.  If we find something there, move it
 	 * to the new table to keep te record "alive" since we still get hits
@@ -157,10 +164,10 @@ dh_locate(const gchar *muid)
 		g_hash_table_remove(by_muid_old, key);
 		g_assert(!g_hash_table_lookup(by_muid, key));
 		g_hash_table_insert(by_muid, key, value);
-		return (dqhit_t *) value;
+		return value;
 	}
 
-	return (dqhit_t *) g_hash_table_lookup(by_muid, muid);
+	return g_hash_table_lookup(by_muid, muid);
 }
 
 /**
@@ -168,15 +175,15 @@ dh_locate(const gchar *muid)
  * New record is registered in the current table.
  */
 static dqhit_t *
-dh_create(const gchar *muid)
+dh_create(const struct guid *muid)
 {
 	dqhit_t *dh;
-	gchar *key;
+	const struct guid *key;
 
-	dh = walloc0(sizeof(*dh));
+	WALLOC0(dh);
 	key = atom_guid_get(muid);
 
-	g_hash_table_insert(by_muid, key, dh);
+	gm_hash_table_insert_const(by_muid, key, dh);
 
 	return dh;
 }
@@ -185,7 +192,7 @@ dh_create(const gchar *muid)
  * Called every time we successfully parsed a query hit from the network.
  */
 void
-dh_got_results(const gchar *muid, gint count)
+dh_got_results(const struct guid *muid, int count)
 {
 	dqhit_t *dh;
 
@@ -222,8 +229,8 @@ dh_timer(time_t now)
 
 	last_rotation = now;
 
-	if (dh_debug > 19)
-		printf("DH rotated tables, current has %d, old has %d\n",
+	if (GNET_PROPERTY(dh_debug) > 19)
+		g_debug("DH rotated tables, current has %d, old has %d",
 			g_hash_table_size(by_muid), g_hash_table_size(by_muid_old));
 }
 
@@ -233,13 +240,13 @@ dh_timer(time_t now)
 static void
 dh_pmsg_free(pmsg_t *mb, gpointer arg)
 {
-	struct pmsg_info *pmi = (struct pmsg_info *) arg;
-	gchar *muid;
+	struct dh_pmsg_info *pmi = arg;
+	const struct guid *muid;
 	dqhit_t *dh;
 
 	g_assert(pmsg_is_extended(mb));
 
-	muid = ((struct gnutella_header *) pmsg_start(mb))->muid;
+	muid = gnutella_header_get_muid(pmsg_start(mb));
 	dh = dh_locate(muid);
 
 	if (dh == NULL)
@@ -268,7 +275,7 @@ dh_pmsg_free(pmsg_t *mb, gpointer arg)
 
 	/* FALL THROUGH */
 cleanup:
-	wfree(pmi, sizeof(*pmi));
+	WFREE(pmi);
 }
 
 /**
@@ -279,10 +286,9 @@ cleanup:
 static enum dh_drop
 dh_can_forward(dqhit_t *dh, mqueue_t *mq, gboolean test)
 {
-	g_assert(mq != NULL);
+	const char *teststr = test ? "[test] " : "";
 
-	if (dh_debug > 19) printf("DH ");
-	if (dh_debug > 19 && test) printf("[test] ");
+	g_assert(mq != NULL);
 
 	/*
 	 * The heart of the "dynamic hit routing" algorithm is here.
@@ -299,7 +305,8 @@ dh_can_forward(dqhit_t *dh, mqueue_t *mq, gboolean test)
 		mq_size(mq) > mq_hiwat(mq) &&		/* Implies we're flow-controlled */
 		(dh->hits_sent >= DH_THRESH_HITS || dh->hits_queued >= DH_MIN_HITS)
 	) {
-		if (dh_debug > 19) printf("queue size > hiwat, dropping\n");
+		if (GNET_PROPERTY(dh_debug) > 19)
+			g_debug("DH %squeue size > hiwat, dropping", teststr);
 		return DH_DROP_FC;
 	}
 
@@ -314,7 +321,8 @@ dh_can_forward(dqhit_t *dh, mqueue_t *mq, gboolean test)
 		mq_is_swift_controlled(mq) &&
 		(dh->hits_sent >= DH_MIN_HITS || dh->hits_queued >= DH_MIN_HITS)
 	) {
-		if (dh_debug > 19) printf("queue in SWIFT mode, dropping\n");
+		if (GNET_PROPERTY(dh_debug) > 19)
+			g_debug("DH %squeue in SWIFT mode, dropping", teststr);
 		return DH_DROP_FC;
 	}
 
@@ -332,7 +340,8 @@ dh_can_forward(dqhit_t *dh, mqueue_t *mq, gboolean test)
 			 (dh->hits_sent + dh->hits_queued) >= DH_MIN_HITS + DH_THRESH_HITS)
 		)
 	) {
-		if (dh_debug > 19) printf("queue in FLOWC mode, dropping\n");
+		if (GNET_PROPERTY(dh_debug) > 19)
+			g_debug("DH %squeue in FLOWC mode, dropping", teststr);
 		return DH_DROP_FC;
 	}
 
@@ -347,7 +356,8 @@ dh_can_forward(dqhit_t *dh, mqueue_t *mq, gboolean test)
 		dh->hits_sent >= DH_POPULAR_HITS &&
 		dh->hits_queued >= (DH_MIN_HITS / 2)
 	) {
-		if (dh_debug > 19) printf("queue size > lowat, dropping\n");
+		if (GNET_PROPERTY(dh_debug) > 19)
+			g_debug("DH %squeue size > lowat, dropping", teststr);
 		return DH_DROP_FC;
 	}
 
@@ -360,7 +370,8 @@ dh_can_forward(dqhit_t *dh, mqueue_t *mq, gboolean test)
 		dh->hits_sent >= DH_POPULAR_HITS &&
 		dh->hits_queued >= DH_MIN_HITS
 	) {
-		if (dh_debug > 19) printf("enough hits queued, throttling\n");
+		if (GNET_PROPERTY(dh_debug) > 19)
+			g_debug("DH %senough hits queued, throttling", teststr);
 		return DH_DROP_THROTTLE;
 	}
 
@@ -374,7 +385,8 @@ dh_can_forward(dqhit_t *dh, mqueue_t *mq, gboolean test)
 		dh->hits_sent < DH_MAX_HITS &&
 		dh->hits_queued > (DH_MIN_HITS / 2) &&
 		(dh->hits_queued + dh->hits_sent) >= DH_MAX_HITS) {
-		if (dh_debug > 19) printf("enough queued, nearing max, throttling\n");
+		if (GNET_PROPERTY(dh_debug) > 19)
+			g_debug("DH %senough queued, nearing max, throttling", teststr);
 		return DH_DROP_THROTTLE;
 	}
 
@@ -384,11 +396,13 @@ dh_can_forward(dqhit_t *dh, mqueue_t *mq, gboolean test)
 	 */
 
 	if (dh->hits_sent >= DH_MAX_HITS && dh->hits_queued) {
-		if (dh_debug > 19) printf("max sendable hits reached, throttling\n");
+		if (GNET_PROPERTY(dh_debug) > 19)
+			g_debug("DH %smax sendable hits reached, throttling", teststr);
 		return DH_DROP_THROTTLE;
 	}
 
-	if (dh_debug > 19) printf("forwarding\n");
+	if (GNET_PROPERTY(dh_debug) > 19)
+		g_debug("DH %sforwarding", teststr);
 
 	return DH_FORWARD;
 }
@@ -397,28 +411,29 @@ dh_can_forward(dqhit_t *dh, mqueue_t *mq, gboolean test)
  * Route query hits from one node to the other.
  */
 void
-dh_route(gnutella_node_t *src, gnutella_node_t *dest, gint count)
+dh_route(gnutella_node_t *src, gnutella_node_t *dest, int count)
 {
 	pmsg_t *mb;
-	struct pmsg_info *pmi;
-	gchar *muid;
+	struct dh_pmsg_info *pmi;
+	const struct guid *muid;
 	dqhit_t *dh;
 	mqueue_t *mq;
 
-	g_assert(src->header.function == GTA_MSG_SEARCH_RESULTS);
+	g_assert(
+		gnutella_header_get_function(&src->header) == GTA_MSG_SEARCH_RESULTS);
 	g_assert(count >= 0);
 
 	if (!NODE_IS_WRITABLE(dest))
 		goto drop_shutdown;
 
-	muid = src->header.muid;
+	muid = gnutella_header_get_muid(&src->header);
 	dh = dh_locate(muid);
 
 	g_assert(dh != NULL);		/* Must have called dh_got_results() first! */
 
-	if (dh_debug > 19) {
-		printf("DH %s got %d hit%s: "
-			"msg=%u, hits_recv=%u, hits_sent=%u, hits_queued=%u\n",
+	if (GNET_PROPERTY(dh_debug) > 19) {
+		g_debug("DH %s got %d hit%s: "
+			"msg=%u, hits_recv=%u, hits_sent=%u, hits_queued=%u",
 			guid_hex_str(muid), count, count == 1 ? "" : "s",
 			dh->msg_recv, dh->hits_recv, dh->hits_sent,
 			dh->hits_queued);
@@ -444,13 +459,13 @@ dh_route(gnutella_node_t *src, gnutella_node_t *dest, gint count)
 	 * Allow message through.
 	 */
 
-	pmi = walloc(sizeof(*pmi));
+	WALLOC(pmi);
 	pmi->hits = count;
 
 	dh->hits_queued += count;
 	dh->msg_queued++;
 
-	g_assert(dh->hits_queued >= (guint) count);
+	g_assert(dh->hits_queued >= UNSIGNED(count));
 
 	/*
 	 * Magic: we create an extended version of a pmsg_t that contains a
@@ -460,16 +475,40 @@ dh_route(gnutella_node_t *src, gnutella_node_t *dest, gint count)
 	 * This enables us to track how much results we already queued/sent.
 	 */
 
-	mb = gmsg_split_to_pmsg_extend(
-		(guchar *) &src->header, src->data,
-		src->size + sizeof(struct gnutella_header),
-		dh_pmsg_free, pmi);
+	if (NODE_IS_UDP(dest)) {
+		gnet_host_t to;
+		pmsg_t *mbe;
 
-	mq_putq(mq, mb);
+		gnet_host_set(&to, dest->addr, dest->port);
 
-	if (dh_debug > 19)
-		printf("DH enqueued %d hit%s for %s\n",
-			count, count == 1 ? "" : "s", guid_hex_str(muid));
+		/*
+		 * With GUESS we may route back a query hit to an UDP node.
+		 */
+
+		if (GNET_PROPERTY(guess_server_debug) > 19) {
+			g_debug("GUESS sending %d hit%s (%s) for %s to %s",
+				count, 1 == count ? "" : "s",
+				NODE_CAN_INFLATE(dest) ? "possibly deflated" : "uncompressed",
+				guid_hex_str(muid), node_infostr(dest));
+		}
+
+		mb = gmsg_split_to_deflated_pmsg(&src->header, src->data,
+				src->size + GTA_HEADER_SIZE);
+		mbe = pmsg_clone_extend(mb, dh_pmsg_free, pmi);
+		pmsg_free(mb);
+
+		mq_udp_putq(mq, mbe, &to);
+	} else {
+		mb = gmsg_split_to_pmsg_extend(&src->header, src->data,
+				src->size + GTA_HEADER_SIZE, dh_pmsg_free, pmi);
+		mq_tcp_putq(mq, mb, src);
+
+		if (GNET_PROPERTY(dh_debug) > 19) {
+			g_debug("DH enqueued %d hit%s for %s to %s",
+				count, count == 1 ? "" : "s", guid_hex_str(muid),
+				node_infostr(dest));
+		}
+	}
 
 	return;
 
@@ -479,7 +518,7 @@ drop_shutdown:
 
 drop_flow_control:
 	gnet_stats_count_dropped(src, MSG_DROP_FLOW_CONTROL);
-	gnet_stats_count_flowc(&src->header);
+	gnet_stats_count_flowc(&src->header, TRUE);
 	return;
 
 drop_throttle:
@@ -491,7 +530,7 @@ drop_throttle:
  * If we had to route hits to the specified node destination, would we?
  */
 gboolean
-dh_would_route(const gchar *muid, gnutella_node_t *dest)
+dh_would_route(const struct guid *muid, gnutella_node_t *dest)
 {
 	dqhit_t *dh;
 
@@ -523,8 +562,8 @@ dh_init(void)
 void
 dh_close(void)
 {
-	dh_table_free(by_muid);
-	dh_table_free(by_muid_old);
+	dh_table_free(&by_muid);
+	dh_table_free(&by_muid_old);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
