@@ -1,5 +1,5 @@
 /*
- * $Id: features.c 12201 2006-11-03 05:14:47Z cbiere $
+ * $Id: features.c 19142 2011-05-23 20:44:04Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -35,23 +35,27 @@
 
 #include "common.h"
 
-RCSID("$Id: features.c 12201 2006-11-03 05:14:47Z cbiere $")
+RCSID("$Id: features.c 19142 2011-05-23 20:44:04Z rmanfredi $")
 
 #include "features.h"
 
-#include "lib/walloc.h"
-#include "lib/header.h"
-#include "lib/misc.h"
-#include "lib/glib-missing.h"
-
 #include "if/gnet_property_priv.h"
+#include "if/core/main.h"
+
+#include "lib/ascii.h"
+#include "lib/header.h"
+#include "lib/glib-missing.h"
+#include "lib/parse.h"
+#include "lib/walloc.h"
 
 #include "lib/override.h"		/* Must be the last header included */
 
 struct header_x_feature {
-	gchar *name;
+	char *name;
 	int major;
 	int minor;
+	const gboolean *guard;
+	gboolean (*guardfn)(void);
 };
 
 struct features {
@@ -90,10 +94,9 @@ header_features_cleanup(xfeature_t xf)
 		struct header_x_feature *header = cur->data;
 
 		G_FREE_NULL(header->name);
-		wfree(header, sizeof *header);
+		WFREE(header);
 	}
-	g_list_free(features->list);
-	features->list = NULL;
+	gm_list_free_null(&features->list);
 }
 
 void
@@ -107,14 +110,12 @@ features_close(void)
 }
 
 /**
- * Add support for feature_name with the specified version to the X-Features
- * header.
+ * Add conditional support for feature ``name'': if at run-time the value
+ * pointed-at by ``guard'' is FALSE, the feature is not emitted.
  */
 void
-header_features_add(xfeature_t xf,
-	const gchar *feature_name,
-	int feature_version_major,
-	int feature_version_minor)
+header_features_add_guarded(xfeature_t xf,
+	const char *name, int major, int minor, const gboolean *guard)
 {
 	struct header_x_feature *item;
 	struct features *features;
@@ -122,13 +123,49 @@ header_features_add(xfeature_t xf,
 	features = features_get(xf);
 	g_return_if_fail(features);
 
-	item = walloc(sizeof *item);
-	item->name = g_strdup(feature_name);
-	item->major = feature_version_major;
-	item->minor = feature_version_minor;
+	WALLOC(item);
+	item->name = g_strdup(name);
+	item->major = major;
+	item->minor = minor;
+	item->guard = guard;
+	item->guardfn = NULL;
 
 	features->list = g_list_append(features->list, item);
 }
+ 
+/**
+ * Add conditional support for feature ``name'': if at run-time the value
+ * returned by the ``guardfn'' function is FALSE, the feature is not emitted.
+ */
+void
+header_features_add_guarded_function(xfeature_t xf,
+	const char *name, int major, int minor, gboolean (*guardfn)(void))
+{
+	struct header_x_feature *item;
+	struct features *features;
+
+	features = features_get(xf);
+	g_return_if_fail(features);
+
+	WALLOC(item);
+	item->name = g_strdup(name);
+	item->major = major;
+	item->minor = minor;
+	item->guard = NULL;
+	item->guardfn = guardfn;
+
+	features->list = g_list_append(features->list, item);
+}
+ 
+/**
+ * Add support for feature ``name'' with the specified version to the
+ * X-Features header.
+ */
+void
+header_features_add(xfeature_t xf, const char *name, int major, int minor)
+{
+	header_features_add_guarded(xf, name, major, minor, NULL);
+}
 
 /**
  * Adds the X-Features header to a HTTP request.
@@ -141,12 +178,12 @@ header_features_add(xfeature_t xf,
  * *rw is changed too *rw + bytes written
  */
 void
-header_features_generate(xfeature_t xf, gchar *dst, size_t len, size_t *rw)
+header_features_generate(xfeature_t xf, char *dst, size_t len, size_t *rw)
 {
 	static const char hdr[] = "X-Features";
 	struct features *features;
 	GList *cur;
-	gpointer fmt;
+	header_fmt_t *fmt;
 
 	g_assert(len <= INT_MAX);
 	g_assert(*rw <= INT_MAX);
@@ -161,11 +198,17 @@ header_features_generate(xfeature_t xf, gchar *dst, size_t len, size_t *rw)
 	if (g_list_first(features->list) == NULL)
 		return;
 
-	fmt = header_fmt_make(hdr, ", ", len - *rw);
+	fmt = header_fmt_make(hdr, ", ", 0, len - *rw);
 
 	for (cur = g_list_first(features->list); cur; cur = g_list_next(cur)) {
 		struct header_x_feature *item = cur->data;
-		gchar buf[50];
+		char buf[50];
+
+		if (item->guard && !*item->guard)
+			continue;
+
+		if (item->guardfn && !(*item->guardfn)())
+			continue;
 
 		gm_snprintf(buf, sizeof buf, "%s/%d.%d",
 			item->name, item->major, item->minor);
@@ -175,11 +218,12 @@ header_features_generate(xfeature_t xf, gchar *dst, size_t len, size_t *rw)
 
 	header_fmt_end(fmt);
 
-	if ((size_t) header_fmt_length(fmt) < len - *rw) {
-		*rw += gm_snprintf(&dst[*rw], len - *rw, "%s", header_fmt_string(fmt));
+	if (header_fmt_length(fmt) < len - *rw) {
+		*rw += clamp_strncpy(&dst[*rw], len - *rw,
+				header_fmt_string(fmt), header_fmt_length(fmt));
 	}
 
-	header_fmt_free(fmt);
+	header_fmt_free(&fmt);
 }
 
 /**
@@ -188,21 +232,21 @@ header_features_generate(xfeature_t xf, gchar *dst, size_t len, size_t *rw)
  * is returned.
  */
 gboolean
-header_get_feature(const gchar *feature_name, const header_t *header,
-	guint *feature_version_major, guint *feature_version_minor)
+header_get_feature(const char *name, const header_t *header,
+	guint *major, guint *minor)
 {
-	gchar *buf = NULL;
-	gchar *start;
+	static const char x_features[] = "X-Features";
+	char *buf, *start;
 
-	if (feature_version_major)
-		*feature_version_major = 0;
-	if (feature_version_minor)
-		*feature_version_minor = 0;
+	if (major)
+		*major = 0;
+	if (minor)
+		*minor = 0;
 
-	buf = header_get(header, "X-Features");
+	buf = header_get(header, x_features);
 
 	/*
-	 * We could also try to scan for the header: feature_name, so this would
+	 * We could also try to scan for the header: name, so this would
      * make this function even more generic. But I would suggest another
      * function for this though.
      */
@@ -212,49 +256,45 @@ header_get_feature(const gchar *feature_name, const header_t *header,
 		 * Actually the 'specs' say we should assume it is supported if the
 		 * X-Features header is not there. But I wouldn't count on it, and
 		 * it was only for "legacy" attributes in the HTTP file exchange.
-		 * Better safe than sorry.
+		 *
+		 * Also, for optimization purposes, the X-Features line will be sent
+		 * once per persistent HTTP connection, as the client is expected to
+		 * cache the supported features.
 		 */
 
 		return FALSE;
 	}
 
 	/*
-	 * We must locate the feature_name exactly, and not a subpart of another
+	 * We must locate the name exactly, and not a subpart of another
 	 * feature.  If we look for "bar", then we must not match on "foobar".
 	 */
 
 	start = buf;
 	for (;;) {
-		gint pc;			/* Previous char */
+		int pc;			/* Previous char */
 
-		buf = ascii_strcasestr(buf, feature_name);
+		buf = ascii_strcasestr(buf, name);
 
 		if (buf == NULL)
 			return FALSE;
 		if (buf == start)
 			break;
 
-		pc = *(buf - 1);
-		if (is_ascii_space(pc) || pc == ',' || pc == ';')
-			break;			/* Found it! */
-
 		/*
 		 * Since we're looking for whole words separated by a space or the
 		 * regular header punctuation, the next match can't occur before
 		 * the end of the current string we matched...
 		 */
 
-		buf += strlen(feature_name);
-	}
+		pc = *(buf - 1);
+		buf += strlen(name);
 
-	buf += strlen(feature_name);		/* Should now be on the "/" sep */
+		if (*buf != '/')
+			continue;		/* Matched "barcode" when looking for "bar" */
 
-	if (*buf != '/') {
-		g_warning("[header] Malformed X-Features header, ignoring");
-		if (dbg > 2)
-			header_dump(header, stderr);
-
-		return FALSE;
+		if (is_ascii_space(pc) || pc == ',' || pc == ';')
+			break;			/* Found it! */
 	}
 
 	buf++;
@@ -262,9 +302,7 @@ header_get_feature(const gchar *feature_name, const header_t *header,
 	if (*buf == '\0')
 		return FALSE;
 
-	
-	return 0 == parse_major_minor(buf, NULL,
-					feature_version_major, feature_version_minor);
+	return 0 == parse_major_minor(buf, NULL, major, minor);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
