@@ -1,5 +1,5 @@
 /*
- * $Id: file_object.c 11879 2006-09-05 19:22:45Z cbiere $
+ * $Id: file_object.c 19142 2011-05-23 20:44:04Z rmanfredi $
  *
  * Copyright (c) 2006, Christian Biere
  *
@@ -50,7 +50,7 @@
  * The current offset is shared that means, you should always use pread()
  * instead of read(), pwrite() instead of write() etc. The replacement
  * functions do not restore the original file offset and they are NOT
- * thread-safe. As Gtk-Gnutella is mono-threaded this should never be a
+ * thread-safe. As gtk-gnutella is mono-threaded this should never be a
  * problem.
  *
  * The file objects created with O_RDWR are returned for all
@@ -60,14 +60,14 @@
  * Normally, file objects should be acquired as follows:
  *
  *  // Try to reuse an existing file object
- *  file = file_object(path, mode);
+ *  file = file_object_open(pathname, mode);
  *  if (!file) {
  *      int fd;
  * 
  *      // If none exists, create a new file object
- *      fd = open_the_file(path, mode);
+ *      fd = open_the_file(pathname, mode);
  *      if (fd >= 0) {
- *      	file = file_object_new(fd, path, mode);
+ *      	file = file_object_new(fd, pathname, mode);
  *      }
  *  }
  *  if (!file) {
@@ -86,14 +86,17 @@
 
 #include "common.h"
 
-RCSID("$Id: file_object.c 11879 2006-09-05 19:22:45Z cbiere $")
+RCSID("$Id: file_object.c 19142 2011-05-23 20:44:04Z rmanfredi $")
 
 #include "file_object.h"
 
 #include "lib/atoms.h"
+#include "lib/compat_pio.h"
+#include "lib/fd.h"
 #include "lib/file.h"
+#include "lib/glib-missing.h"
 #include "lib/iovec.h"
-#include "lib/misc.h"
+#include "lib/path.h"
 #include "lib/walloc.h"
 
 #include "lib/override.h"       /* Must be the last header included */
@@ -102,14 +105,15 @@ static GHashTable *ht_file_objects_rdonly;	/* read-only file objects */
 static GHashTable *ht_file_objects_wronly;	/* write-only file objects */
 static GHashTable *ht_file_objects_rdwr;	/* read+write-able file objects */
 
-enum file_object_magic { FILE_OBJECT_MAGIC = 0xeb084325 };	/**< Magic number */
+enum file_object_magic { FILE_OBJECT_MAGIC = 0x6b084325 };	/**< Magic number */
 
 struct file_object {
-	char *pathname;	/* atom */
+	enum file_object_magic magic;
+	const char *pathname;	/* atom */
 	int ref_count;
 	int fd;
 	int accmode;	/* O_RDONLY, O_WRONLY, O_RDWR */
-	enum file_object_magic magic;
+	int removed;
 };
 
 /**
@@ -212,7 +216,6 @@ file_object_check(const struct file_object * const fo)
 	g_assert(FILE_OBJECT_MAGIC == fo->magic);
 	g_assert(fo->ref_count > 0);
 	g_assert(fo->ref_count < INT_MAX);
-	g_assert(fo->fd >= 0);
 	g_assert(fo->pathname);
 }
 
@@ -251,14 +254,27 @@ file_object_find(const char * const pathname, int accmode)
 	g_return_val_if_fail(is_absolute_path(pathname), NULL);
 
 	fo = g_hash_table_lookup(file_object_mode_get_table(O_RDWR), pathname);
-	if (!fo && O_RDWR != accmode) {
-		fo = g_hash_table_lookup(file_object_mode_get_table(accmode), pathname);
+
+	/*
+	 * We need to find a more specific file object if looking for O_WRONLY
+	 * or O_RDONLY ones.
+	 */
+
+	if (O_RDWR != accmode) {
+		struct file_object *xfo;
+		xfo = g_hash_table_lookup(file_object_mode_get_table(accmode), pathname);
+		if (xfo != NULL) {
+			g_assert(xfo->accmode == accmode);
+			fo = xfo;
+		}
 	}
 
 	if (fo) {
 		file_object_check(fo);
+		g_assert(is_valid_fd(fo->fd));
 		g_assert(0 == strcmp(pathname, fo->pathname));
 		g_assert(accmode_is_valid(fo->fd, accmode));
+		g_assert(!fo->removed);
 	}
 
 	return fo;
@@ -279,7 +295,7 @@ file_object_alloc(const int fd, const char * const pathname, int accmode)
 	ht = file_object_mode_get_table(accmode);
 	g_return_val_if_fail(ht, NULL);
 
-	fo = walloc(sizeof *fo);
+	WALLOC(fo);
 	*fo = zero_fo;
 	fo->magic = FILE_OBJECT_MAGIC;
 	fo->ref_count = 1;
@@ -288,30 +304,49 @@ file_object_alloc(const int fd, const char * const pathname, int accmode)
 	fo->pathname = atom_str_get(pathname);
 
 	file_object_check(fo);
-	g_hash_table_insert(file_object_mode_get_table(fo->accmode),
+	g_assert(is_valid_fd(fo->fd));
+	gm_hash_table_insert_const(file_object_mode_get_table(fo->accmode),
 		fo->pathname, fo);
 
 	return fo;
 }
 
+static void
+file_object_remove(struct file_object * const fo)
+{
+	const struct file_object *xfo;
+	
+	file_object_check(fo);
+	g_return_if_fail(!fo->removed);
+
+	xfo = file_object_find(fo->pathname, fo->accmode);
+	g_assert(xfo == fo);
+
+	g_hash_table_remove(file_object_mode_get_table(fo->accmode), fo->pathname);
+	fo->removed = TRUE;
+}
+
 static void
 file_object_free(struct file_object * const fo)
 {
 	g_return_if_fail(fo);
-	g_return_if_fail(1 == fo->ref_count);
-	g_return_if_fail(fo->fd >= 0);
-	g_return_if_fail(file_object_find(fo->pathname, fo->accmode));
-
 	file_object_check(fo);
-	g_hash_table_remove(file_object_mode_get_table(fo->accmode), fo->pathname);
-	g_assert(!g_hash_table_lookup(file_object_mode_get_table(fo->accmode),
-				fo->pathname));
+	g_return_if_fail(1 == fo->ref_count);
 
-	close(fo->fd);
-	fo->fd = -1;
+	if (fo->removed) {
+		const struct file_object *xfo;
+
+		xfo = g_hash_table_lookup(file_object_mode_get_table(fo->accmode),
+				fo->pathname);
+		g_assert(xfo != fo);
+	} else {
+		file_object_remove(fo);
+	}
+
+	fd_close(&fo->fd);
 	atom_str_free_null(&fo->pathname);
 	fo->magic = 0;
-	wfree(fo, sizeof *fo);
+	WFREE(fo);
 }
 
 /**
@@ -333,6 +368,7 @@ file_object_open(const char * const pathname, int accmode)
 	fo = file_object_find(pathname, accmode);
 	if (fo) {
 		fo->ref_count++;
+		g_assert(is_valid_fd(fo->fd));
 	}
 	return fo;
 }
@@ -385,6 +421,256 @@ file_object_release(struct file_object **fo_ptr)
 	}
 }
 
+/**
+ * Special operations that we can perform on file objects.
+ *
+ * These require special treatment because Windows frowns upon renaming
+ * and unlinking of opened files and will not, unlike UNIX, keep an already
+ * opened file accessible after it has been removed from the filesystem.
+ *
+ * Special operations provide uniform semantics on every platform:
+ *
+ * - FO_OP_UNLINK unlinks the file, denying further access to the file
+ *   even if it was already opened.
+ *
+ * - FO_OP_RENAME renames the file, making it transparent for users whith
+ *   current accesses on the file being renamed.
+ *
+ * - FO_OP_MOVED notifies that the file was moved around (accross file
+ *   systems possibly) and that current users of the old path should be
+ *   transparently remapped to the new file with the same access levels,
+ *   the old location being unlinked afterwards.
+ */
+enum file_object_op {
+	FO_OP_UNLINK = 0,		/**< Unlink file, further access denied */
+	FO_OP_RENAME,			/**< Rename file (on same filesystem) */
+	FO_OP_MOVED				/**< Moving notification */
+};
+
+/**
+ * Convert special operation to string.
+ */
+static const char *
+file_object_op_to_string(enum file_object_op op)
+{
+	switch (op) {
+	case FO_OP_UNLINK:	return "unlink()";
+	case FO_OP_RENAME:	return "rename()";
+	case FO_OP_MOVED:	return "unlink()";	/* "unlink()" is NOT a typo */
+	}
+
+	g_assert_not_reached();
+	return NULL;
+}
+
+/**
+ * Execute special operation.
+ *
+ * @param old_name	An absolute pathname, the old file name.
+ * @param new_name	An absolute pathname, the new file name.
+ *
+ * @return TRUE if operation was successful, FALSE otherwise, with errno set.
+ */
+static gboolean
+file_object_special_op(enum file_object_op op,
+	const char * const old_name, const char * const new_name)
+{
+	const int accmodes[] = { O_RDONLY, O_WRONLY, O_RDWR };
+	unsigned i;
+	GSList *objects = NULL;
+	gboolean ok = TRUE;
+	int saved_errno = 0;
+
+	errno = EINVAL;		/* In case one of the soft assertions fails */
+
+	g_return_val_if_fail(old_name, FALSE);
+	g_return_val_if_fail(is_absolute_path(old_name), FALSE);
+	if (op != FO_OP_UNLINK) {
+		g_return_val_if_fail(new_name, FALSE);
+		g_return_val_if_fail(is_absolute_path(new_name), FALSE);
+	}
+
+	/*
+	 * Identify all the file objects currently active for the old name.
+	 */
+
+	for (i = 0; i < G_N_ELEMENTS(accmodes); i++) {
+		struct file_object *fo;
+
+		fo = file_object_find(old_name, accmodes[i]);
+		if (fo != NULL) {
+			if (NULL == g_slist_find(objects, fo)) {
+				objects = g_slist_prepend(objects, fo);
+			}
+		}
+	}
+
+	/*
+	 * On Windows, close all the files prior renaming / unlinking.
+	 *
+	 * On UNIX, only close all the files on unlink and moving.  There is
+	 * no need to do anything for a rename() operation.
+	 */
+
+	if (op != FO_OP_RENAME || is_running_on_mingw()) {
+		GSList *sl;
+
+		GM_SLIST_FOREACH(objects, sl) {
+			struct file_object *fo = sl->data;
+
+			fd_forget_and_close(&fo->fd);
+		}
+	}
+
+	/*
+	 * Perform the operation.
+	 */
+
+	switch (op) {
+	case FO_OP_UNLINK:
+	case FO_OP_MOVED:
+		ok = unlink(old_name) != -1;
+		break;
+	case FO_OP_RENAME:
+		ok = rename(old_name, new_name) != -1;
+		break;
+	}
+
+	if (!ok) {
+		saved_errno = errno;
+		goto reopen;
+	}
+
+	if (op != FO_OP_UNLINK) {
+		GSList *sl;
+
+		/*
+		 * Re-index the file objects with their new name.
+		 */
+
+		GM_SLIST_FOREACH(objects, sl) {
+			struct file_object *fo = sl->data;
+
+			g_hash_table_remove(file_object_mode_get_table(fo->accmode),
+				fo->pathname);
+			atom_str_change(&fo->pathname, new_name);
+			gm_hash_table_insert_const(file_object_mode_get_table(fo->accmode),
+				fo->pathname, fo);
+		}
+	} else {
+		GSList *sl;
+
+		/*
+		 * Revoke the file objects on successful unlinking.
+		 *
+		 * This will prevent further file_object_open() pointing to the (now
+		 * removed) path from returning an existing file object.
+		 * It will also invalidate all current file objects: further read/write
+		 * attempts on these will return EBADF.
+		 */
+
+		GM_SLIST_FOREACH(objects, sl) {
+			struct file_object *fo = sl->data;
+
+			file_object_remove(fo);
+		}
+
+		goto done;		/* No re-opening required with unlink */
+	}
+
+	/* FALL THROUGH */
+
+reopen:
+
+	/*
+	 * On Windows, reopen all the files.
+	 *
+	 * On UNIX, reopen the files after a move operation.  There is nothing
+	 * to be done on a rename() since we did not close the files before the
+	 * operation and the kernel will do the right thing.
+	 */
+
+	if (FO_OP_MOVED == op || is_running_on_mingw()) {
+		GSList *sl;
+
+		GM_SLIST_FOREACH(objects, sl) {
+			struct file_object *fo = sl->data;
+
+			fo->fd = file_absolute_open(fo->pathname, fo->accmode, 0);
+
+			if (!is_valid_fd(fo->fd)) {
+				g_warning("cannot reopen \"%s\" mode 0%o "
+					"after %s %s of \"%s\": %s",
+					fo->pathname, fo->accmode,
+					ok ? "successful" : "failed", file_object_op_to_string(op),
+					old_name, g_strerror(errno));
+			}
+		}
+	}
+
+	/* FALL THROUGH */
+
+done:
+
+	gm_slist_free_null(&objects);
+
+	if (!ok)
+		errno = saved_errno;
+
+	return ok;
+}
+
+/**
+ * Renames a file and transparently re-opens all the file objets pointing to
+ * the old name, re-inserting the file objects with the new names, assuming
+ * renaming was successful.
+ *
+ * @param old_name	An absolute pathname, the old file name.
+ * @param new_name	An absolute pathname, the new file name.
+ *
+ * @return TRUE if renaming was successful, FALSE otherwise, with errno set.
+ */
+gboolean
+file_object_rename(const char * const old_name, const char * const new_name)
+{
+	return file_object_special_op(FO_OP_RENAME, old_name, new_name);
+}
+
+/**
+ * Notification that a file was successfully copied.  Access by file objects
+ * to the old path are transferred to the new one and the old file is unlinked.
+ *
+ * @param old_name	An absolute pathname, the old file name.
+ * @param new_name	An absolute pathname, the new file name.
+ *
+ * @return TRUE if renaming was successful, FALSE otherwise, with errno set.
+ */
+gboolean
+file_object_moved(const char * const old_name, const char * const new_name)
+{
+	return file_object_special_op(FO_OP_MOVED, old_name, new_name);
+}
+
+/**
+ * Deletes a file.
+ *
+ * @param path		An absolute pathname, the file to unkink()
+ *
+ * @return TRUE if unlinking was successful, FALSE otherwise, with errno set.
+ */
+gboolean
+file_object_unlink(const char * const path)
+{
+	return file_object_special_op(FO_OP_UNLINK, path, NULL);
+}
+
+static ssize_t
+file_object_ebadf(void)
+{
+	errno = EBADF;
+	return (ssize_t) -1;
+}
+
 /**
  * Write the given data to a file object at the given offset.
  *
@@ -398,30 +684,15 @@ file_object_release(struct file_object **fo_ptr)
  */
 ssize_t
 file_object_pwrite(const struct file_object * const fo,
-	const void * const data, const size_t size, const filesize_t pos)
-#ifdef HAS_PWRITE
+	const void * const data, const size_t size, const filesize_t offset)
 {
-	off_t offset;
-
 	file_object_check(fo);
-	offset = filesize_to_off_t(pos);
-	if ((off_t) -1 == offset) {
-		errno = EINVAL;
-		return -1;
-	} else {
-		return pwrite(fo->fd, data, size, offset);
-	}
-}
-#else	/* !HAS_PWRITE */
-{
-	struct iovec iov;
 
-	file_object_check(fo);
-	iov = iov_get(deconstify_gpointer(data), size);
-	return file_object_pwritev(fo, &iov, 1, pos);
-}
-#endif	/* HAS_PWRITE */
+	if (!is_valid_fd(fo->fd))
+		return file_object_ebadf();
 
+	return compat_pwrite(fo->fd, data, size, offset);
+}
 
 /**
  * Write the given data to a file object at the given offset.
@@ -436,51 +707,17 @@ file_object_pwrite(const struct file_object * const fo,
  */
 ssize_t
 file_object_pwritev(const struct file_object * const fo,
-	const struct iovec * const iov, const int iov_cnt, const filesize_t pos)
-#ifdef HAS_PWRITEV
+	const iovec_t * iov, const int iov_cnt, const filesize_t offset)
 {
-	off_t offset;
-
 	file_object_check(fo);
 	g_assert(iov);
 	g_assert(iov_cnt > 0);
-
-	offset = filesize_to_off_t(pos);
-	if ((off_t) -1 == offset) {
-		errno = EINVAL;
-		return -1;
-	} else {
-		return pwritev(fo->fd, iov, MIN(iov_cnt, MAX_IOV_COUNT), offset);
-	}
-}
-#else	/* !HAS_PWRITEV */
-{
-	ssize_t ret;
-
-	file_object_check(fo);
-	g_assert(iov);
-	g_assert(iov_cnt > 0);
-
-#ifdef HAS_PWRITE
-	if (1 == iov_cnt) {
-		return file_object_pwrite(fo, iov->iov_base, iov->iov_len, pos);
-	}
-#endif /* HAS_PWRITE */
 	
-	if (0 != seek_to_filepos(fo->fd, pos)) {
-		int saved_errno = errno;
+	if (!is_valid_fd(fo->fd))
+		return file_object_ebadf();
 
-		g_warning("failed to seek at offset %s (%s) for \"%s\" ",
-			uint64_to_string(pos), g_strerror(errno), fo->pathname);
-
-		errno = saved_errno;
-		ret = -1;
-	} else {
-		ret = writev(fo->fd, iov, MIN(iov_cnt, MAX_IOV_COUNT));
-	}
-	return ret;
+	return compat_pwritev(fo->fd, iov, iov_cnt, offset);
 }
-#endif	/* HAS_PWRITEV */
 
 /**
  * Read data from the file object from the given offset.
@@ -495,30 +732,15 @@ file_object_pwritev(const struct file_object * const fo,
  */
 ssize_t
 file_object_pread(const struct file_object * const fo,
-	void * const data, const size_t size, const filesize_t pos)
-#ifdef HAS_PREAD
+	void * const data, const size_t size, const filesize_t offset)
 {
-	off_t offset;
-
 	file_object_check(fo);
-	offset = filesize_to_off_t(pos);
-	if ((off_t) -1 == offset) {
-		errno = EINVAL;
-		return -1;
-	} else {
-		return pread(fo->fd, data, size, offset);
-	}
-}
-#else	/* !HAS_PREAD */
-{
-	struct iovec iov;
 
-	file_object_check(fo);
-	iov = iov_get(data, size);
-	return file_object_preadv(fo, &iov, 1, pos);
-}
-#endif	/* HAS_PREAD */
+	if (!is_valid_fd(fo->fd))
+		return file_object_ebadf();
 
+	return compat_pread(fo->fd, data, size, offset);
+}
 
 /**
  * Read data from a file object from the given offset.
@@ -533,52 +755,31 @@ file_object_pread(const struct file_object * const fo,
  */
 ssize_t
 file_object_preadv(const struct file_object * const fo,
-	struct iovec * const iov, const int iov_cnt, const filesize_t pos)
-#ifdef HAS_PREADV
+	iovec_t * const iov, const int iov_cnt, const filesize_t offset)
 {
-	off_t offset;
-
 	file_object_check(fo);
 	g_assert(iov);
 	g_assert(iov_cnt > 0);
 
-	offset = filesize_to_off_t(pos);
-	if ((off_t) -1 == offset) {
-		errno = EINVAL;
-		return -1;
-	} else {
-		return preadv(fo->fd, iov, MIN(iov_cnt, MAX_IOV_COUNT), offset);
-	}
+	if (!is_valid_fd(fo->fd))
+		return file_object_ebadf();
+
+	return compat_preadv(fo->fd, iov, MIN(iov_cnt, MAX_IOV_COUNT), offset);
 }
-#else	/* !HAS_PREADV */
+
+/**
+ * Get opened file status.
+ *
+ * @return 0 if OK, -1 on failure with errno set.
+ */
+int
+file_object_fstat(const struct file_object * const fo, filestat_t *buf)
 {
-	ssize_t ret;
-
 	file_object_check(fo);
-	g_assert(iov);
-	g_assert(iov_cnt > 0);
+	g_assert(is_valid_fd(fo->fd));
 
-#ifdef HAS_PREAD
-	if (1 == iov_cnt) {
-		return file_object_pread(fo, iov->iov_base, iov->iov_len, pos);
-	}
-#endif /* HAS_PREAD */
-
-	if (0 != seek_to_filepos(fo->fd, pos)) {
-		int saved_errno = errno;
-
-		g_warning("failed to seek at offset %s (%s) for \"%s\" ",
-			uint64_to_string(pos), g_strerror(errno), fo->pathname);
-
-		errno = saved_errno;
-		ret = -1;
-	} else {
-		ret = readv(fo->fd, iov, MIN(iov_cnt, MAX_IOV_COUNT));
-	}
-	return ret;
+	return fstat(fo->fd, buf);
 }
-#endif	/* HAS_PREADV */
-
 
 /**
  * Get the file descriptor associated with a file object. This should
@@ -637,7 +838,7 @@ file_object_show_item(gpointer key, gpointer value, gpointer unused_udata)
 	file_object_check(fo);
 	g_assert(fo->pathname == key);
 
-	g_message("leaked file object: ref.count=%d fd=%d pathname=\"%s\"",
+	g_warning("leaked file object: ref.count=%d fd=%d pathname=\"%s\"",
 		fo->ref_count, fo->fd, fo->pathname);
 }
 
@@ -678,4 +879,4 @@ file_object_close(void)
 #undef D
 }
 
- /* vi: set ts=4 sw=4 cindent: */
+/* vi: set ts=4 sw=4 cindent: */
