@@ -1,5 +1,5 @@
 /*
- * $Id: geo_ip.c,v 1.25 2005/12/17 10:32:06 cbiere Exp $
+ * $Id: geo_ip.c 18596 2011-01-30 21:12:45Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -35,16 +35,21 @@
 
 #include "common.h"
 
-RCSID("$Id: geo_ip.c,v 1.25 2005/12/17 10:32:06 cbiere Exp $");
+RCSID("$Id: geo_ip.c 18596 2011-01-30 21:12:45Z rmanfredi $")
 
 #include "geo_ip.h"
 #include "settings.h"
 
 #include "lib/file.h"
-#include "lib/misc.h"
+#include "lib/ascii.h"
 #include "lib/glib-missing.h"
+#include "lib/halloc.h"
+#include "lib/host_addr.h"
 #include "lib/iprange.h"
 #include "lib/iso3166.h"
+#include "lib/parse.h"
+#include "lib/path.h"
+#include "lib/tm.h"
 #include "lib/walloc.h"
 #include "lib/watcher.h"
 
@@ -53,21 +58,21 @@ RCSID("$Id: geo_ip.c,v 1.25 2005/12/17 10:32:06 cbiere Exp $");
 
 #include "lib/override.h"		/* Must be the last header included */
 
-static const gchar gip_file[] = "geo-ip.txt";
-static const gchar gip_what[] = "Geographic IP mappings";
+static const char gip_file[] = "geo-ip.txt";
+static const char gip_what[] = "Geographic IP mappings";
 
-static gpointer geo_db;			/**< The database of bogus CIDR ranges */
+static struct iprange_db *geo_db;	/**< The database of bogus CIDR ranges */
+static time_t geo_mtime;			/**< Modification time of loaded file */
 
 /**
  * Context used during ip_range_split() calls.
  */
 struct range_context {
+	char *line;				/**< The line from the input file */
+	int linenum;				/**< Line number in input file, for errors */
 	guint32 ip1;				/**< Original lower IP in global range */
 	guint32 ip2;				/**< Original upper IP in global range */
-	gint country;				/**< Country code (numerical encoded) */
-	gint count;					/**< Amount of ranges we added, for stats */
-	gchar *line;				/**< The line from the input file */
-	gint linenum;				/**< Line number in input file, for errors */
+	guint16 country;			/**< Country code (numerical encoded) */
 };
 
 /**
@@ -78,28 +83,22 @@ struct range_context {
 static void
 gip_add_cidr(guint32 ip, guint bits, gpointer udata)
 {
-	struct range_context *ctx = (struct range_context *) udata;
+	struct range_context *ctx = udata;
 	iprange_err_t error;
 	gpointer ccode;
+	guint cc;
 
-	if (dbg > 4)
+	if (GNET_PROPERTY(reload_debug) > 4)
 		printf("GEO adding %s/%d for \"%s\"\n",
 			ip_to_string(ip), bits, ctx->line);
 
-	ccode = GUINT_TO_POINTER(ctx->country);
+	cc = ctx->country;
+	ccode = GUINT_TO_POINTER(cc);
 	error = iprange_add_cidr(geo_db, ip, bits, ccode);
 
 	switch (error) {
 	case IPR_ERR_OK:
 		break;
-	case IPR_ERR_RANGE_OVERLAP:
-		error = iprange_add_cidr_force(geo_db, ip, bits, ccode, NULL);
-		if (error == IPR_ERR_OK) {
-			g_warning("%s, line %d: "
-				"entry \"%s\" (%s/%d) superseded earlier smaller range",
-				gip_file, ctx->linenum, ctx->line, ip_to_string(ip), bits);
-			break;
-		}
 		/* FALL THROUGH */
 	default:
 		g_warning("%s, line %d: rejected entry \"%s\" (%s/%d): %s",
@@ -107,8 +106,6 @@ gip_add_cidr(guint32 ip, guint bits, gpointer udata)
 			iprange_strerror(error));
 		return;
 	}
-
-	ctx->count++;
 }
 
 /**
@@ -116,18 +113,24 @@ gip_add_cidr(guint32 ip, guint bits, gpointer udata)
  *
  * @return The amount of entries loaded.
  */
-static gint
+static guint
 gip_load(FILE *f)
 {
-	gchar line[1024];
-	gchar *p;
-	gint linenum = 0;
-	const gchar *end;
-	gint c, code;
+	char line[1024];
+	char *p;
+	int linenum = 0;
+	const char *end;
+	guint16 code;
+	int c;
 	struct range_context ctx;
+	filestat_t buf;
 
-	geo_db = iprange_make(NULL, NULL);
-	ctx.count = 0;
+	geo_db = iprange_new();
+	if (-1 == fstat(fileno(f), &buf)) {
+		g_warning("cannot stat %s: %s", gip_file, g_strerror(errno));
+	} else {
+		geo_mtime = buf.st_mtime;
+	}
 
 	while (fgets(line, sizeof(line), f)) {
 		linenum++;
@@ -151,7 +154,7 @@ gip_load(FILE *f)
 		 *
 		 *    15.0.0.0 - 15.130.191.255 fr
 		 *
-		 * So we don't have to parse the two IP addresses, and compute
+		 * So we just have to parse the two IP addresses, then compute
 		 * all the ranges they cover in order to insert them into
 		 * the IP database.
 		 */
@@ -214,7 +217,7 @@ gip_load(FILE *f)
 		}
 
 		code = iso3166_encode_cc(end);
-		if (-1 == code) {
+		if (ISO3166_INVALID == code) {
 			g_warning("%s, line %d: bad country code in \"%s\"",
 				gip_file, linenum, line);
 			continue;
@@ -235,17 +238,14 @@ gip_load(FILE *f)
 		ip_range_split(ctx.ip1, ctx.ip2, gip_add_cidr, &ctx);
 	}
 
-	if (dbg) {
-		iprange_stats_t stats;
+	iprange_sync(geo_db);
 
-		iprange_get_stats(geo_db, &stats);
-
-		g_message("loaded %d geographical IP ranges", ctx.count);
-		g_message("geo IP stats: count=%d level2=%d heads=%d enlisted=%d",
-			stats.count, stats.level2, stats.heads, stats.enlisted);
+	if (GNET_PROPERTY(reload_debug)) {
+		g_debug("loaded %u geographical IP ranges (%u hosts)",
+			iprange_get_item_count(geo_db),
+			iprange_get_host_count(geo_db));
 	}
-
-	return ctx.count;
+	return iprange_get_item_count(geo_db);
 }
 
 /**
@@ -253,11 +253,11 @@ gip_load(FILE *f)
  * geographic IP mappings changed.
  */
 static void
-gip_changed(const gchar *filename, gpointer unused)
+gip_changed(const char *filename, gpointer unused)
 {
 	FILE *f;
-	gchar buf[80];
-	gint count;
+	char buf[80];
+	guint count;
 
 	(void) unused;
 
@@ -269,7 +269,7 @@ gip_changed(const gchar *filename, gpointer unused)
 	count = gip_load(f);
 	fclose(f);
 
-	gm_snprintf(buf, sizeof(buf), "Reloaded %d geographic IP ranges.", count);
+	gm_snprintf(buf, sizeof(buf), "Reloaded %u geographic IP ranges.", count);
 	gcu_statusbar_message(buf);
 }
 
@@ -290,8 +290,8 @@ static void
 gip_retrieve(void)
 {
 	FILE *f;
-	gint idx;
-	gchar *filename;
+	int idx;
+	char *filename;
 #ifndef OFFICIAL_BUILD
 	static file_path_t fp[3];
 #else
@@ -312,14 +312,14 @@ gip_retrieve(void)
 
 	filename = make_pathname(fp[idx].dir, fp[idx].name);
 	watcher_register(filename, gip_changed, NULL);
-	G_FREE_NULL(filename);
+	HFREE_NULL(filename);
 
 	gip_load(f);
 	fclose(f);
 }
 
 /**
- * Called on startup. Loads the bogons.txt into memory.
+ * Called on startup. Loads the geo-ip.txt file into memory.
  */
 void
 gip_init(void)
@@ -333,36 +333,73 @@ gip_init(void)
 void
 gip_close(void)
 {
-	if (geo_db) {
-		iprange_free_each(geo_db, NULL);
-		geo_db = NULL;
-	}
+	iprange_free(&geo_db);
 }
 
 /**
  * Retrieves the country an address is assigned to.
  *
  * @param ha the host address to look up.
- * @return the country mapped to this IP address as an numerical encoded
- * country code, * or -1 when unknown.
+ * @return the country mapped to this IP address as a numerically-encoded
+ *         country code, or ISO3166_INVALID when unknown.
  */
-gint
+guint16
 gip_country(const host_addr_t ha)
 {
 	host_addr_t to;
 
 	if (
 		host_addr_convert(ha, &to, NET_TYPE_IPV4) ||
-		host_addr_6to4_to_ipv4(ha, &to)
+		host_addr_tunnel_client(ha, &to)
 	) {
 		gpointer code;
 		guint32 ip;
 
 		ip = host_addr_ipv4(to);
 		if (geo_db && NULL != (code = iprange_get(geo_db, ip)))
-			return (GPOINTER_TO_INT(code) >> 1) - 1;
+			return (GPOINTER_TO_UINT(code) >> 1) - 1;
 	}
-	return -1;
+	return ISO3166_INVALID;
+}
+
+/**
+ * Same as gip_country() only returns ISO3166_INVALID if the geo_ip file
+ * is too ancient: the risk of having a wrong mapping is too high.
+ */
+guint16
+gip_country_safe(const host_addr_t ha)
+{
+	if (delta_time(tm_time(), geo_mtime) > 15552000)	/* ~6 months */
+		return ISO3166_INVALID;
+
+	return gip_country(ha);
+}
+
+/**
+ * Convenience routine to return the full contry name of an address.
+ */
+const char *
+gip_country_name(const host_addr_t ha)
+{
+	return iso3166_country_name(gip_country(ha));
+}
+
+/**
+ * Convenience routine to return the contry code of an address.
+ */
+const char *
+gip_country_cc(const host_addr_t ha)
+{
+	return iso3166_country_cc(gip_country(ha));
+}
+
+/**
+ * Returns the modification time of the loaded geo_ip file.
+ */
+time_t
+gip_mtime(void)
+{
+	return geo_mtime;
 }
 
 /* vi: set ts=4 sw=4 cindent: */
