@@ -1,5 +1,5 @@
 /*
- * $Id: geo_ip.h,v 1.6 2005/09/10 08:17:28 daichik Exp $
+ * $Id: geo_ip.h 16995 2009-09-26 22:50:43Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -36,12 +36,18 @@
 #ifndef _core_geo_ip_h_
 #define _core_geo_ip_h_
 
-#include <glib.h>
+#include "common.h"
 #include "lib/host_addr.h"
 
-gint gip_country(const host_addr_t addr);
 void gip_init(void);
 void gip_close(void);
+time_t gip_mtime(void);
+
+guint16 gip_country(const host_addr_t addr);
+guint16 gip_country_safe(const host_addr_t ha);
+
+const char *gip_country_cc(const host_addr_t ha);
+const char *gip_country_name(const host_addr_t ha);
 
 #endif /* _core_geo_ip_h_ */
 
