@@ -1,5 +1,5 @@
 /*
- * $Id: ggep.c,v 1.22 2005/09/10 08:17:28 daichik Exp $
+ * $Id: ggep.c 19252 2011-06-13 22:44:40Z rmanfredi $
  *
  * Copyright (c) 2002-2004, Raphael Manfredi
  *
@@ -35,11 +35,14 @@
 
 #include "common.h"
 
-RCSID("$Id: ggep.c,v 1.22 2005/09/10 08:17:28 daichik Exp $");
+RCSID("$Id: ggep.c 19252 2011-06-13 22:44:40Z rmanfredi $")
+
+#include <zlib.h>	/* Z_DEFAULT_COMPRESSION */
 
 #include "ggep.h"
 #include "extensions.h"
 #include "lib/cobs.h"
+#include "lib/halloc.h"
 #include "lib/misc.h"
 #include "lib/walloc.h"
 #include "lib/zlib_util.h"
@@ -153,7 +156,7 @@ ggep_stream_cleanup(ggep_stream_t *gs)
  * @return FALSE if there's not enough room in the output.
  */
 static inline gboolean
-ggep_stream_appendc(ggep_stream_t *gs, gchar c)
+ggep_stream_appendc(ggep_stream_t *gs, char c)
 {
 	g_assert(ggep_stream_is_valid(gs));
 
@@ -195,9 +198,9 @@ ggep_stream_append(ggep_stream_t *gs, gconstpointer data, size_t len)
  * On error, the stream is left in a clean state.
  */
 gboolean
-ggep_stream_begin(ggep_stream_t *gs, const gchar *id, guint32 wflags)
+ggep_stream_begin(ggep_stream_t *gs, const char *id, guint32 wflags)
 {
-	gint idlen;
+	int idlen;
 	guint8 flags = 0;
 
 	g_assert(ggep_stream_is_valid(gs));
@@ -318,10 +321,10 @@ cleanup:
  * @return TRUE if OK.  On error, the stream is brought back to a clean state.
  */
 gboolean
-ggep_stream_writev(ggep_stream_t *gs, const struct iovec *iov, gint iovcnt)
+ggep_stream_writev(ggep_stream_t *gs, const iovec_t *iov, int iovcnt)
 {
-	const struct iovec *xiov;
-	gint i;
+	const iovec_t *xiov;
+	int i;
 
 	g_assert(ggep_stream_is_valid(gs));
 	g_assert(gs->begun);
@@ -335,7 +338,8 @@ ggep_stream_writev(ggep_stream_t *gs, const struct iovec *iov, gint iovcnt)
 		g_assert(gs->zd != NULL);
 
 		for (i = iovcnt, xiov = iov; i--; xiov++) {
-			if (!zlib_deflate_data(gs->zd, xiov->iov_base, xiov->iov_len))
+			if (!zlib_deflate_data(gs->zd, 
+				iovec_base(xiov), iovec_len(xiov)))
 				goto cleanup;
 		}
 
@@ -348,7 +352,8 @@ ggep_stream_writev(ggep_stream_t *gs, const struct iovec *iov, gint iovcnt)
 
 	if (gs->flags & GGEP_F_COBS) {
 		for (i = iovcnt, xiov = iov; i--; xiov++) {
-			if (!cobs_stream_write(&gs->cs, xiov->iov_base, xiov->iov_len))
+			if (!cobs_stream_write(&gs->cs, 
+				  iovec_base(xiov), iovec_len(xiov)))
 				goto cleanup;
 		}
 
@@ -360,7 +365,8 @@ ggep_stream_writev(ggep_stream_t *gs, const struct iovec *iov, gint iovcnt)
 	 */
 
 	for (i = iovcnt, xiov = iov; i--; xiov++) {
-		if (!ggep_stream_append(gs, xiov->iov_base, xiov->iov_len))
+		if (!ggep_stream_append(gs, 
+			  iovec_base(xiov), iovec_len(xiov)))
 			goto cleanup;
 	}
 
@@ -379,12 +385,13 @@ cleanup:
 gboolean
 ggep_stream_write(ggep_stream_t *gs, gconstpointer data, size_t len)
 {
-	struct iovec iov;
-	const struct iovec *p_iov = &iov;
+	iovec_t iov;
+	const iovec_t *p_iov = &iov;
 
 	g_assert(len <= INT_MAX);
-	iov.iov_base = deconstify_gpointer(data);
-	iov.iov_len = len;
+
+	iovec_set_base(&iov, deconstify_gpointer(data));
+	iovec_set_len(&iov, len);
 
 	return ggep_stream_writev(gs, p_iov, 1);
 }
@@ -430,10 +437,10 @@ ggep_stream_end(ggep_stream_t *gs)
 			gpointer data;
 			gboolean ok;
 
-			if (ggep_debug > 2)
+			if (GNET_PROPERTY(ggep_debug) > 2)
 				g_warning("GGEP \"%.*s\" not compressing %d bytes into %d",
-					(gint) (*gs->fp & GGEP_F_IDLEN), gs->fp + 1,
-					(gint) ilen, (gint) plen);
+					(int) (*gs->fp & GGEP_F_IDLEN), gs->fp + 1,
+					(int) ilen, (int) plen);
 
 			data = zlib_uncompress(zlib_deflater_out(gs->zd), plen, ilen);
 			if (data == NULL)
@@ -459,14 +466,14 @@ ggep_stream_end(ggep_stream_t *gs)
 			/* No overwriting */
 			g_assert((size_t) (gs->end - gs->o) <= gs->size);
 			ok = ggep_stream_write(gs, data, ilen);
-			G_FREE_NULL(data);
+			HFREE_NULL(data);
 
 			if (!ok)
 				goto cleanup;
-		} else if (ggep_debug > 2)
-			g_warning("GGEP \"%.*s\" compressed %d bytes into %d",
-				(gint) (*gs->fp & GGEP_F_IDLEN),
-				gs->fp + 1, (gint) ilen, (gint) plen);
+		} else if (GNET_PROPERTY(ggep_debug) > 3)
+			g_debug("GGEP \"%.*s\" compressed %d bytes into %d",
+				(int) (*gs->fp & GGEP_F_IDLEN),
+				gs->fp + 1, (int) ilen, (int) plen);
 	}
 
 	/*
@@ -476,7 +483,7 @@ ggep_stream_end(ggep_stream_t *gs)
 	 */
 
 	if ((gs->flags & GGEP_F_DEFLATE) && (gs->flags & GGEP_F_COBS)) {
-		gchar *deflated = zlib_deflater_out(gs->zd);
+		char *deflated = zlib_deflater_out(gs->zd);
 
 		if (!cobs_stream_write(&gs->cs, deflated, plen))
 			goto cleanup;
@@ -512,10 +519,10 @@ ggep_stream_end(ggep_stream_t *gs)
 			if (NULL == cobs_decode(gs->lp + 1, plen, &ilen, TRUE))
 				goto cleanup;
 
-			if (ggep_debug > 2)
+			if (GNET_PROPERTY(ggep_debug) > 2)
 				g_warning("GGEP \"%.*s\" no need to COBS %d bytes into %d",
-					(gint) (*gs->fp & GGEP_F_IDLEN), gs->fp + 1,
-					(gint) ilen, (gint) plen);
+					(int) (*gs->fp & GGEP_F_IDLEN), gs->fp + 1,
+					(int) ilen, (int) plen);
 
 			/*
 			 * Remove any COBS indication.
@@ -533,9 +540,9 @@ ggep_stream_end(ggep_stream_t *gs)
 			gs->o = &gs->lp[1 + plen];	/* +1 to skip NUL "length" byte */
 			/* No overwriting */
 			g_assert((size_t) (gs->end - gs->o) <= gs->size);
-		} else if (ggep_debug > 2)
-			g_message("GGEP \"%.*s\" COBS-ed into %d bytes",
-				(gint) (*gs->fp & GGEP_F_IDLEN), gs->fp + 1, (gint) plen);
+		} else if (GNET_PROPERTY(ggep_debug) > 3)
+			g_debug("GGEP \"%.*s\" COBS-ed into %d bytes",
+				(int) (*gs->fp & GGEP_F_IDLEN), gs->fp + 1, (int) plen);
 	}
 
 	/*
@@ -553,10 +560,10 @@ ggep_stream_end(ggep_stream_t *gs)
 	 * stored at the beginning.  We only reserved 1 byte for the length.
 	 */
 
-	if (ggep_debug > 3)
-		g_message("GGEP \"%.*s\" payload holds %d byte%s",
-			(gint) (*gs->fp & GGEP_F_IDLEN), gs->fp + 1,
-			(gint) plen, plen == 1 ? "" : "s");
+	if (GNET_PROPERTY(ggep_debug) > 7)
+		g_debug("GGEP \"%.*s\" payload holds %d byte%s",
+			(int) (*gs->fp & GGEP_F_IDLEN), gs->fp + 1,
+			(int) plen, plen == 1 ? "" : "s");
 
 	if (plen <= 63) {
 		slen = 1;
@@ -571,8 +578,8 @@ ggep_stream_end(ggep_stream_t *gs)
 		hlen[1] = GGEP_L_CONT | ((plen >> GGEP_L_VSHIFT) & GGEP_L_VALUE);
 		hlen[2] = GGEP_L_LAST | (plen & GGEP_L_VALUE);
 	} else {
-		g_warning("too large GGEP payload length (%d bytes) for \"%.*s\"",
-			(gint) plen, (gint) (*gs->fp & GGEP_F_IDLEN), gs->fp + 1);
+		g_carp("too large GGEP payload length (%d bytes) for \"%.*s\"",
+			(int) plen, (int) (*gs->fp & GGEP_F_IDLEN), gs->fp + 1);
 		goto cleanup;
 	}
 
@@ -585,7 +592,7 @@ ggep_stream_end(ggep_stream_t *gs)
 	g_assert((size_t) (gs->end - gs->o) <= gs->size);	/* No overwriting */
 
 	if (slen > 1) {
-		gchar *start = gs->lp + 1;
+		char *start = gs->lp + 1;
 
 		if ((size_t) (gs->end - gs->o) < (slen - 1))
 			goto cleanup;
@@ -642,6 +649,7 @@ ggep_stream_close(ggep_stream_t *gs)
 	gs->outbuf = NULL;				/* Mark stream as closed */
 
 	g_assert(len <= gs->size);
+
 	return len;
 }
 
@@ -652,7 +660,7 @@ ggep_stream_close(ggep_stream_t *gs)
  */
 gboolean
 ggep_stream_packv(ggep_stream_t *gs,
-	const gchar *id, const struct iovec *iov, gint iovcnt, guint32 wflags)
+	const char *id, const iovec_t *iov, int iovcnt, guint32 wflags)
 {
 	g_assert(iovcnt >= 0);
 
@@ -687,17 +695,17 @@ ggep_stream_packv(ggep_stream_t *gs,
  */
 gboolean
 ggep_stream_pack(ggep_stream_t *gs,
-	const gchar *id, gconstpointer payload, size_t plen, guint32 wflags)
+	const char *id, gconstpointer payload, size_t plen, guint32 wflags)
 {
-	struct iovec iov;
-	const struct iovec *p_iov = &iov;
+	iovec_t iov;
+	const iovec_t *p_iov = &iov;
 
 	g_assert(id);
 	g_assert(0 == plen || NULL != payload);
 	g_assert(plen <= INT_MAX);
 
-	iov.iov_base = deconstify_gpointer(payload);
-	iov.iov_len = plen;
+	iovec_set_base(&iov, deconstify_gpointer(payload));
+	iovec_set_len(&iov, plen);
 
 	return ggep_stream_packv(gs, id, p_iov, 1, wflags);
 }
