@@ -1,5 +1,5 @@
 /*
- * $Id: ggep.h 9219 2005-08-16 04:15:22Z cbiere $
+ * $Id: ggep.h 17775 2010-11-19 20:28:00Z nonamer $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -36,13 +36,13 @@
 #ifndef _core_ggep_h_
 #define _core_ggep_h_
 
+#include "common.h"
+
 #include "extensions.h"
 #include "lib/cobs.h"
 #include "lib/zlib_util.h"
 
-#include <glib.h>
-
-#define GGEP_MAGIC		'\xc3'		/**< GGEP extension prefix */
+#define GGEP_MAGIC		((unsigned char) 0xC3U)	/**< GGEP extension prefix */
 
 /*
  * GGEP Extension Header Flags.
@@ -91,12 +91,12 @@ enum ggep_magic { GGEP_MAGIC_ID = 0x62961da4U };
  */
 typedef struct ggep_stream {
 	enum ggep_magic magic;/**< Magic number */
-	gchar *outbuf;			/**< Base address of output buffer */
-	gchar *end;				/**< First address beyond output buffer */
-	gchar *o;				/**< Where next output should go */
-	gchar *fp;				/**< Where flags for current extension are */
-	gchar *lp;				/**< Where length should be written when known */
-	gchar *last_fp;			/**< Flags of last successfully written ext. */
+	char *outbuf;			/**< Base address of output buffer */
+	char *end;				/**< First address beyond output buffer */
+	char *o;				/**< Where next output should go */
+	char *fp;				/**< Where flags for current extension are */
+	char *lp;				/**< Where length should be written when known */
+	char *last_fp;			/**< Flags of last successfully written ext. */
 	size_t size;			/**< Size of the outbuf buffer */
 	guint8 flags;			/**< Extension flags (COBS / DEFLATE) */
 	gboolean magic_emitted;	/**< Whether leading magic was emitted */
@@ -109,21 +109,19 @@ typedef struct ggep_stream {
  * Public interface.
  */
 
-struct iovec;
-
-gint ggep_decode_into(extvec_t *exv, gchar *buf, size_t len);
+int ggep_decode_into(extvec_t *exv, char *buf, size_t len);
 
 void ggep_stream_init(ggep_stream_t *gs, gpointer data, size_t len);
 gboolean ggep_stream_begin(ggep_stream_t *gs, const char *id, guint32 wflags);
 gboolean ggep_stream_writev(ggep_stream_t *gs,
-	const struct iovec *iov, gint iovcnt);
+	const iovec_t *iov, int iovcnt);
 gboolean ggep_stream_write(ggep_stream_t *gs, gconstpointer data, size_t len);
 gboolean ggep_stream_end(ggep_stream_t *gs);
 size_t ggep_stream_close(ggep_stream_t *gs);
 gboolean ggep_stream_packv(ggep_stream_t *gs,
-	const gchar *id, const struct iovec *iov, gint iovcnt, guint32 wflags);
+	const char *id, const iovec_t *iov, int iovcnt, guint32 wflags);
 gboolean ggep_stream_pack(ggep_stream_t *gs,
-	const gchar *id, gconstpointer payload, size_t plen, guint32 wflags);
+	const char *id, gconstpointer payload, size_t plen, guint32 wflags);
 
 gboolean ggep_stream_is_valid(ggep_stream_t *gs);
 
