@@ -1,5 +1,5 @@
 /*
- * $Id: gmsg.h 12147 2006-10-27 19:55:09Z cbiere $
+ * $Id: gmsg.h 19301 2011-06-22 22:44:49Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -36,12 +36,14 @@
 #ifndef _core_gmsg_h_
 #define _core_gmsg_h_
 
+#include "common.h"
+
 #include "gnutella.h"
-#include "pmsg.h"
 
 #include "if/core/search.h"
 
 #include "lib/endian.h"
+#include "lib/pmsg.h"
 
 struct gnutella_node;
 struct route_dest;
@@ -88,7 +90,7 @@ typedef enum {
 	GMSG_VALID = 0,				/* Payload <= 64KiB, no flags */
 	GMSG_VALID_MARKED,			/* Payload <= 64KiB, marked but no flags */
 	GMSG_INVALID,				/* Payload > 64KiB, no mark for flags */
-	GMSG_VALID_NO_PROCESS,		/* Marked for flags we do not know */
+	GMSG_VALID_NO_PROCESS		/* Marked for flags we do not know */
 } gmsg_valid_t;
 
 /*
@@ -96,11 +98,13 @@ typedef enum {
  */
 
 void gmsg_init(void);
-const gchar *gmsg_name(guint function);
+const char *gmsg_name(guint function);
 gmsg_valid_t gmsg_size_valid(gconstpointer msg, guint16 *size);
 
 pmsg_t *gmsg_to_pmsg(gconstpointer msg, guint32 size);
 pmsg_t *gmsg_to_deflated_pmsg(gconstpointer msg, guint32 size);
+pmsg_t *gmsg_split_to_deflated_pmsg(const void *head,
+			const void *data, guint32 size);
 pmsg_t *gmsg_to_ctrl_pmsg(gconstpointer msg, guint32 size);
 pmsg_t * gmsg_to_ctrl_pmsg_extend(gconstpointer msg, guint32 size,
 			pmsg_free_t free_cb, gpointer arg);
@@ -110,7 +114,9 @@ pmsg_t * gmsg_split_to_pmsg_extend(gconstpointer head, gconstpointer data,
 			guint32 size, pmsg_free_t free_cb, gpointer arg);
 
 void gmsg_mb_sendto_all(const GSList *sl, pmsg_t *mb);
-void gmsg_mb_sendto_one(struct gnutella_node *n, pmsg_t *mb);
+void gmsg_mb_sendto_one(const struct gnutella_node *n, pmsg_t *mb);
+void gmsg_mb_routeto_one(const struct gnutella_node *from,
+	const struct gnutella_node *to, pmsg_t *mb);
 
 void gmsg_sendto_one(struct gnutella_node *n, gconstpointer msg, guint32 size);
 void gmsg_ctrl_sendto_one(struct gnutella_node *n,
@@ -118,36 +124,35 @@ void gmsg_ctrl_sendto_one(struct gnutella_node *n,
 void gmsg_split_sendto_one(struct gnutella_node *n,
 		gconstpointer head, gconstpointer data, guint32 size);
 void gmsg_sendto_all(const GSList *l, gconstpointer msg, guint32 size);
-void gmsg_split_sendto_all(const GSList *l,
+void gmsg_split_routeto_all(const GSList *l,
+		const struct gnutella_node *from,
 		gconstpointer head, gconstpointer data, guint32 size);
-void gmsg_split_sendto_all_but_one(const GSList *sl,
-		const struct gnutella_node *n, gconstpointer head, gconstpointer data,
-		guint32 size);
 void gmsg_sendto_route(struct gnutella_node *n, struct route_dest *rt);
 
-gboolean gmsg_can_drop(gconstpointer pdu, gint size);
+gboolean gmsg_can_drop(gconstpointer pdu, int size);
 gboolean gmsg_is_oob_query(gconstpointer msg);
 gboolean gmsg_split_is_oob_query(gconstpointer head, gconstpointer data);
-gint gmsg_cmp(gconstpointer pdu1, gconstpointer pdu2);
-const gchar *gmsg_infostr(gconstpointer msg);
-gchar *gmsg_infostr_full(gconstpointer msg);
-gchar *gmsg_infostr_full_split(gconstpointer head, gconstpointer data);
+int gmsg_cmp(gconstpointer pdu1, gconstpointer pdu2, gboolean pdu2_complete);
+const char *gmsg_infostr(gconstpointer msg);
+const char *gmsg_node_infostr(const struct gnutella_node *n);
+char *gmsg_infostr_full(gconstpointer msg, size_t msg_len);
+char *gmsg_infostr_full_split(gconstpointer head,
+	gconstpointer data, size_t data_len);
+size_t gmsg_infostr_full_split_to_buf(gconstpointer head, gconstpointer data,
+	size_t data_len, char *buf, size_t buf_size);
 
 void gmsg_install_presend(pmsg_t *mb);
 
-void gmsg_log_dropped(gconstpointer msg,
-	const gchar *reason, ...) G_GNUC_PRINTF(2, 3);
 void gmsg_log_bad(const struct gnutella_node *n,
-	const gchar *reason, ...) G_GNUC_PRINTF(2, 3);
-
-void gmsg_sendto_route_ggep(
-	struct gnutella_node *n, struct route_dest *rt, guint32 regular_size);
-void gmsg_sendto_one_ggep(struct gnutella_node *n,
-	gconstpointer msg, guint32 size, guint32 regular_size);
-void gmsg_ctrl_sendto_one_ggep(struct gnutella_node *n,
-	gconstpointer msg, guint32 size, guint32 regular_size);
-void gmsg_sendto_all_ggep(const GSList *sl,
-	gconstpointer msg, guint32 size, guint32 regular_size);
+	const char *reason, ...) G_GNUC_PRINTF(2, 3);
+void gmsg_log_dropped_pmsg(pmsg_t *msg,
+	const char *reason, ...) G_GNUC_PRINTF(2, 3);
+void gmsg_log_split_dropped(
+	gconstpointer head, gconstpointer data, size_t data_len,
+	const char *reason, ...) G_GNUC_PRINTF(4, 5);
+void gmsg_log_split_duplicate(
+	gconstpointer head, gconstpointer data, size_t data_len,
+	const char *reason, ...) G_GNUC_PRINTF(4, 5);
 
 void gmsg_search_sendto_one(struct gnutella_node *n, gnet_search_t sh,
 	gconstpointer msg, guint32 size);
