@@ -1,5 +1,5 @@
 /*
- * $Id: hosts.c,v 1.21 2005/09/11 13:39:57 rmanfredi Exp $
+ * $Id: hosts.c 19266 2011-06-15 16:02:31Z rmanfredi $
  *
  * Copyright (c) 2001-2004, Raphael Manfredi
  *
@@ -35,72 +35,88 @@
 
 #include "common.h"
 
-RCSID("$Id: hosts.c,v 1.21 2005/09/11 13:39:57 rmanfredi Exp $");
+RCSID("$Id: hosts.c 19266 2011-06-15 16:02:31Z rmanfredi $")
 
-#include "sockets.h"
 #include "hosts.h"
-#include "nodes.h"
-#include "share.h"			/* For files_scanned and kbytes_scanned. */
-#include "routing.h"
-#include "gmsg.h"
-#include "pcache.h"
-#include "whitelist.h"
-#include "gwcache.h"
-#include "settings.h"
 #include "bogons.h"
+#include "gmsg.h"
+#include "hostiles.h"
+#include "nodes.h"
+#include "pcache.h"
+#include "routing.h"
+#include "settings.h"
+#include "share.h"			/* For files_scanned and kbytes_scanned. */
+#include "sockets.h"
+#include "udp.h"
 #include "uhc.h"
+#include "whitelist.h"
 
 #include "if/gnet_property_priv.h"
+#include "if/dht/dht.h"		/* For dht_fill_random() */
 
+#include "lib/aging.h"
 #include "lib/endian.h"
 #include "lib/glib-missing.h"
-#include "lib/misc.h"
+#include "lib/parse.h"
+#include "lib/stringify.h"
+#include "lib/tm.h"
+#include "lib/walloc.h"
+
 #include "lib/override.h"	/* Must be the last header included */
 
-gboolean host_low_on_pongs = FALSE;			/**< True when less than 12% full */
+#define HOST_PINGING_PERIOD		30		/**< Try pinging every 30 seconds */
+#define HOST_DHT_MAX			50		/**< Get that many random hosts */
+#define HOST_CONNECT_FREQ		120		/**< At most 1 connection per 2 mins */
+
+gboolean host_low_on_pongs = FALSE;		/**< True when less than 12% full */
 
 static gboolean in_shutdown = FALSE;
+static aging_table_t *node_connects;
 
-/***
- *** Host hashing.
- ***/
-
-/**
- * Hash function for use in g_hash_table_new.
+/*
+ * Avoid nodes being stuck helplessly due to completely stale caches.
+ * @return TRUE if an UHC may be contact, FALSE if it's not permissable.
  */
-guint
-host_hash(gconstpointer key)
+static gboolean
+host_cache_allow_bypass(void)
 {
-	const gnet_host_t *host = key;
+	static time_t last_try;
 
-	return host_addr_hash(host->addr) ^ ((host->port << 16) | host->port);
+	if (node_count() > 0)
+		return FALSE;
+
+	/* Wait at least 2 minutes after starting up */
+	if (delta_time(tm_time(), GNET_PROPERTY(start_stamp)) < 2 * 60)
+		return FALSE;
+
+	/*
+	 * Allow again after 12 hours, useful after unexpected network outage
+	 * or downtime.
+	 */
+
+	if (last_try && delta_time(tm_time(), last_try) < 12 * 3600)
+		return FALSE;
+
+	last_try = tm_time();
+	return TRUE;
 }
 
 /**
- * Compare function which returns TRUE if the hosts are equal.
- *
- * @note For use in g_hash_table_new.
+ * Attempt Gnutella host connection.
+ * @return TRUE if OK, FALSE if attempt was throttled
  */
-gint
-host_eq(gconstpointer v1, gconstpointer v2)
+static gboolean
+host_gnutella_connect(host_addr_t addr, guint16 port)
 {
-	const gnet_host_t *h1 = v1, *h2 = v2;
+	if (aging_lookup(node_connects, &addr))
+		return FALSE;
 
-	return host_addr_equal(h1->addr, h2->addr) && h1->port == h2->port;
+	node_add_socket(NULL, addr, port, 0);
+	aging_insert(node_connects, wcopy(&addr, sizeof addr), GUINT_TO_POINTER(1));
+
+	return TRUE;
 }
 
-/**
- * Compare function which returns 0 if the hosts are equal, otherwise 1.
- *
- * @note For use in g_list_find_custom.
- */
-gint
-host_cmp(gconstpointer v1, gconstpointer v2)
-{
-	return host_eq(v1, v2) ? 0 : 1;
-}
-
-
 /***
  *** Host periodic timer.
  ***/
@@ -111,33 +127,40 @@ host_cmp(gconstpointer v1, gconstpointer v2)
 void
 host_timer(void)
 {
-	static gint called = 0;
     guint count;
-	gint missing;
+	int missing;
 	host_addr_t addr;
 	guint16 port;
 	host_type_t htype;
 	guint max_nodes;
+	gboolean empty_cache = FALSE;
 
-	if (in_shutdown || !online_mode)
+	if (in_shutdown || !GNET_PROPERTY(online_mode))
 		return;
 
-	max_nodes = (current_peermode == NODE_P_LEAF) ?
-		max_ultrapeers : max_connections;
-	count = node_count();
+	max_nodes = settings_is_leaf() ?
+		GNET_PROPERTY(max_ultrapeers) : GNET_PROPERTY(max_connections);
+	count = node_count();			/* Established + connecting */
 	missing = node_keep_missing();
 
+	if (GNET_PROPERTY(host_debug) > 1)
+		g_debug("host_timer - count %u, missing %u", count, missing);
+
 	/*
 	 * If we are not connected to the Internet, apparently, make sure to
 	 * connect to at most one host, to avoid using all our hostcache.
 	 * Also, we don't connect each time we are called.
 	 */
 
-	if (!is_inet_connected && missing) {
-		if (0 == (called++ & 0xf))		/* Once every 16 attempts */
-			missing = 1;
-		else
-			missing = 0;			/* Don't connect this run */
+	if (!GNET_PROPERTY(is_inet_connected)) {
+		static time_t last_try;
+
+		if (last_try && delta_time(tm_time(), last_try) < 20)
+			return;
+		last_try = tm_time();
+
+		if (GNET_PROPERTY(host_debug))
+			g_debug("host_timer - not connected, trying to connect");
 	}
 
 	/*
@@ -146,32 +169,28 @@ host_timer(void)
 	 * than quick_connect_pool_size   This is the "greedy mode".
 	 */
 
-	if (count >= quick_connect_pool_size)
-		if (dbg > 10) {
-			g_message("host_timer - count %d >= pool size %d",
-				count, quick_connect_pool_size);
+	if (count >= GNET_PROPERTY(quick_connect_pool_size)) {
+		if (GNET_PROPERTY(host_debug) > 1)
+			g_debug("host_timer - count %u >= pool size %u",
+				count, GNET_PROPERTY(quick_connect_pool_size));
 		return;
 	}
 
 	if (count < max_nodes)
 		missing -= whitelist_connect();
 
-	if (dbg > 10 && missing > 0)
-		g_message("host_timer - missing %d host%s",
-			missing, missing == 1 ? "" : "s");
-
 	/*
 	 * If we are under the number of connections wanted, we add hosts
 	 * to the connection list
 	 */
 
-	htype = (current_peermode == NODE_P_NORMAL) ?
-        HOST_ANY : HOST_ULTRA;
+	htype = HOST_ULTRA;
 
 	if (
-        current_peermode == NODE_P_ULTRA &&
-        node_normal_count < normal_connections &&
-        node_ultra_count >= (up_connections - normal_connections)
+        settings_is_ultra() &&
+        GNET_PROPERTY(node_normal_count) < GNET_PROPERTY(normal_connections) &&
+        GNET_PROPERTY(node_ultra_count) >=
+			(GNET_PROPERTY(up_connections) - GNET_PROPERTY(normal_connections))
 	) {
 		htype = HOST_ANY;
     }
@@ -179,14 +198,37 @@ host_timer(void)
 	if (hcache_size(htype) == 0)
 		htype = HOST_ANY;
 
-    if (!stop_host_get) {
-        if (missing > 0) {
-            guint fan;
-            guint max_pool = MAX(quick_connect_pool_size, max_nodes);
-            guint to_add;
+	if (hcache_size(htype) == 0)
+		empty_cache = TRUE;
 
-            fan = (missing * quick_connect_pool_size) / max_nodes;
-            to_add = is_inet_connected ? fan : (guint) missing;
+	if (GNET_PROPERTY(host_debug) && missing > 0)
+		g_debug("host_timer - missing %d host%s%s",
+			missing, missing == 1 ? "" : "s",
+			empty_cache ? " [empty caches]" : "");
+
+    if (!GNET_PROPERTY(stop_host_get)) {
+        if (missing > 0) {
+			static time_t last_try;
+            unsigned fan, max_pool, to_add;
+
+            max_pool = MAX(GNET_PROPERTY(quick_connect_pool_size), max_nodes);
+            fan = (missing * GNET_PROPERTY(quick_connect_pool_size))/ max_pool;
+			fan = MAX(1, fan);
+            to_add = GNET_PROPERTY(is_inet_connected) ? fan : (guint) missing;
+
+			/*
+			 * Every so many calls, attempt to ping all our neighbours to
+			 * get fresh pongs, in case our host cache is not containing
+			 * sufficiently fresh hosts and we keep getting connection failures.
+			 */
+
+			if (
+				0 == last_try ||
+				delta_time(tm_time(), last_try) >= HOST_PINGING_PERIOD
+			) {
+				ping_all_neighbours();
+				last_try = tm_time();
+			}
 
             /*
              * Make sure that we never use more connections then the
@@ -195,35 +237,67 @@ host_timer(void)
             if (to_add + count > max_pool)
                 to_add = max_pool - count;
 
-            if (dbg > 10) {
-                g_message("host_timer - connecting - add: %d fan:%d  miss:%d "
-                     "max_hosts:%d   count:%d   extra:%d",
+            if (GNET_PROPERTY(host_debug) > 2) {
+                g_debug("host_timer - connecting - "
+					"add: %d fan:%d miss:%d max_hosts:%d count:%d extra:%d",
 					 to_add, fan, missing, max_nodes, count,
-					 quick_connect_pool_size);
+					 GNET_PROPERTY(quick_connect_pool_size));
             }
 
             missing = to_add;
 
-			while (hcache_size(htype) && missing-- > 0) {
-				hcache_get_caught(htype, &addr, &port);
-				node_add(addr, port, 0);
+			if (missing > 0 && (0 == connected_nodes() || host_low_on_pongs)) {
+				gnet_host_t host[HOST_DHT_MAX];
+				int hcount;
+				int i;
+
+				hcount = dht_fill_random(host,
+					MIN(UNSIGNED(missing), G_N_ELEMENTS(host)));
+
+				missing -= hcount;
+
+				for (i = 0; i < hcount; i++) {
+					addr = gnet_host_get_addr(&host[i]);
+					port = gnet_host_get_port(&host[i]);
+					if (!hcache_node_is_bad(addr)) {
+						if (GNET_PROPERTY(host_debug) > 3) {
+							g_debug("host_timer - UHC pinging and connecting "
+								"to DHT node at %s",
+								host_addr_port_to_string(addr, port));
+						}
+						/* Try to use the host as an UHC before connecting */
+						udp_send_ping(NULL, addr, port, TRUE);
+						if (!host_gnutella_connect(addr, port)) {
+							missing++;	/* Did not use entry */
+						}
+					} else {
+						missing++;	/* Did not use entry */
+					}
+				}
 			}
 
-			if (missing > 0 && hcache_read_finished()) {
-				static gint rotate = 0;
+			while (hcache_size(htype) && missing-- > 0) {
+				if (hcache_get_caught(htype, &addr, &port)) {
+					if (!(hostiles_check(addr) || hcache_node_is_bad(addr))) {
+						if (!host_gnutella_connect(addr, port)) {
+							missing++;	/* Did not use entry */
+						}
+					} else {
+						missing++;	/* Did not use entry */
+					}
+				}
+			}
 
-				if (!uhc_is_waiting() && !gwc_is_waiting()) {
-					if (!udp_active() || (0x3 == (rotate++ & 0x3)))
-						gwc_get_hosts(); 	/* Get from web host cache */
-					else
-						uhc_get_hosts();	/* Get from UDP pong caches */
-				} else if (bootstrap_debug > 2)
-					g_message("BOOT host_timer - waiting for reply from %s",
-						uhc_is_waiting() ? "UDP host cache" : "web cache");
+			if (missing > 0 && (empty_cache || host_cache_allow_bypass())) {
+				if (!uhc_is_waiting()) {
+					if (GNET_PROPERTY(host_debug))
+						g_debug("host_timer - querying UDP host cache");
+					uhc_get_hosts();	/* Get new hosts from UHCs */
+				}
 			}
 		}
 
-	} else if (use_netmasks) {
+	} else if (GNET_PROPERTY(use_netmasks)) {
 		/* Try to find better hosts */
 		if (hcache_find_nearby(htype, &addr, &port)) {
 			if (node_remove_worst(TRUE))
@@ -238,39 +312,20 @@ host_timer(void)
  *** Hosts
  ***/
 
-void host_init(void)
+void
+host_init(void)
 {
 	pcache_init();
+	node_connects = aging_make(HOST_CONNECT_FREQ,
+		host_addr_hash_func, host_addr_eq_func, wfree_host_addr);
 }
 
 /**
- * @return the address:port of a host
- */
-const gchar *
-host_to_string(const gnet_host_t *h)
-{
-	static gchar buf[64];
-	gchar host[48];
-
-	g_assert(h != NULL);
-
-	host_addr_to_string_buf(h->addr, host, sizeof host);
-	gm_snprintf(buf, sizeof buf, "%s:%u", host, h->port);
-	return buf;
-}
-
-/**
- * Check whether host is connectible.
- *
- * i.e. that it has a valid port and that its IP address is not private
- * not bogus.
+ * Check whether host's address is usable: routable and not bogus.
  */
 gboolean
-host_is_valid(const host_addr_t addr, guint16 port)
+host_address_is_usable(const host_addr_t addr)
 {
-	if (!port_is_valid(port))
-		return FALSE;
-
 	if (!host_addr_is_routable(addr))
 		return FALSE;
 
@@ -280,6 +335,21 @@ host_is_valid(const host_addr_t addr, guint16 port)
 	return TRUE;
 }
 
+/**
+ * Check whether host is connectible.
+ *
+ * i.e. that it has a valid port and that its IP address is not private
+ * nor bogus.
+ */
+gboolean
+host_is_valid(const host_addr_t addr, guint16 port)
+{
+	if (!port_is_valid(port))
+		return FALSE;
+
+	return host_address_is_usable(addr);
+}
+
 /**
  * Add a new host to our pong reserve.
  *
@@ -288,33 +358,27 @@ host_is_valid(const host_addr_t addr, guint16 port)
 void
 host_add(const host_addr_t addr, guint16 port, gboolean do_connect)
 {
-	if (!hcache_add_caught(HOST_ANY, addr, port, "pong"))
+	if (!do_connect || !hcache_add_caught(HOST_ANY, addr, port, "pong"))
 		return;
 
 	/*
 	 * If we are under the number of connections wanted, we add this host
 	 * to the connection list.
-	 *
-	 * Note: we're not using `node_count()' for the comparison with
-	 * `up_connections' but connected_nodes().	The node_add() routine also
-	 * compare `node_count' with `max_connections' to ensure we don't
-	 * launch too many connections, but comparing here as well may help
-	 * avoid useless call to connected_nodes() and/or node_add().
-	 *				--RAM, 20/09/2001
 	 */
 
 
-	if (do_connect) {
-		if (node_keep_missing() > 0)
-				node_add(addr, port, 0);
-		else {
-			/* If we are above the max connections, delete a non-nearby
-			 * connection before adding this better one
-			 */
-			if (use_netmasks && host_is_nearby(addr) && node_remove_worst(TRUE))
-				node_add(addr, port, 0);
-		}
-
+	if (node_keep_missing() > 0)
+		node_add(addr, port, 0);
+	else if (
+		GNET_PROPERTY(use_netmasks) &&
+		host_is_nearby(addr) &&
+		node_remove_worst(TRUE)
+	) {
+		/*
+		 * If we are above the max connections, delete a non-nearby
+		 * connection before adding this better one
+		 */
+		node_add(addr, port, 0);
 	}
 }
 
@@ -344,11 +408,10 @@ guint32 number_local_networks;
 /**
  * frees the local networks array
  */
-static
-void free_networks(void)
+static void
+free_networks(void)
 {
-	if (local_networks)
-		G_FREE_NULL(local_networks);
+	G_FREE_NULL(local_networks);
 }
 
 /**
@@ -356,10 +419,10 @@ void free_networks(void)
  * the local_networks array. IP's are in network order.
  */
 void
-parse_netmasks(const gchar *str)
+parse_netmasks(const char *str)
 {
-	gchar **masks = g_strsplit(str, ";", 0);
-	gchar *p;
+	char **masks = g_strsplit(str, ";", 0);
+	char *p;
 	guint32 mask_div;
 	int i;
 
@@ -391,7 +454,7 @@ parse_netmasks(const gchar *str)
 					g_warning("parse_netmasks(): Invalid netmask: \"%s\"", p);
 			}
 			else {
-				gint error;
+				int error;
 
 				mask_div = parse_uint32(p, NULL, 10, &error);
 				mask_div = MIN(32, mask_div);
@@ -422,7 +485,7 @@ host_is_nearby(const host_addr_t addr)
 {
 	guint i;
 
-	if (NET_TYPE_IPV4 == host_addr_net(addr)) {
+	if (host_addr_is_ipv4(addr)) {
 		for (i = 0; i < number_local_networks; i++) {
 			guint32 m_mask = local_networks[i].mask;
 			guint32 m_ip = local_networks[i].net;
@@ -430,7 +493,7 @@ host_is_nearby(const host_addr_t addr)
 			if ((host_addr_ipv4(addr) & m_mask) == (m_ip & m_mask))
 				return TRUE;
 		}
-	} else if (NET_TYPE_IPV6 == host_addr_net(addr)) {
+	} else if (host_addr_is_ipv6(addr)) {
 		/* XXX: Implement this! */
 	}
 	return FALSE;
@@ -453,6 +516,7 @@ host_close(void)
 {
 	pcache_close();
 	free_networks();
+	aging_destroy(&node_connects);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
