@@ -1,5 +1,5 @@
 /*
- * $Id: hsep.c 14955 2007-09-28 19:25:45Z bpringlemeir $
+ * $Id: hsep.c 19262 2011-06-15 10:34:36Z rmanfredi $
  *
  * Copyright (c) 2004, Thomas Schuerger & Jeroen Asselman
  *
@@ -95,21 +95,24 @@
 
 #include "common.h"
 
-RCSID("$Id: hsep.c 14955 2007-09-28 19:25:45Z bpringlemeir $")
+RCSID("$Id: hsep.c 19262 2011-06-15 10:34:36Z rmanfredi $")
 
-#include "gmsg.h"
-#include "routing.h"
-#include "nodes.h"
 #include "hsep.h"
-#include "uploads.h"
-#include "share.h"
 #include "features.h"
+#include "gmsg.h"
+#include "nodes.h"
+#include "routing.h"
+#include "settings.h"
+#include "share.h"
+#include "uploads.h"
 
 #include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
 
 #include "lib/endian.h"
 #include "lib/glib-missing.h"
+#include "lib/random.h"
+#include "lib/stringify.h"
 #include "lib/tm.h"
 #include "lib/walloc.h"
 #include "lib/override.h"
@@ -205,7 +208,7 @@ hsep_sanity_check(void)
 	hsep_triple sum[G_N_ELEMENTS(hsep_global_table)];
 	unsigned int i, j;
 
-	memset(sum, 0, sizeof sum);
+	ZERO(&sum);
 
 	g_assert(1 == hsep_own[HSEP_IDX_NODES]);
 
@@ -385,7 +388,7 @@ hsep_reset(void)
 	const GSList *sl;
 	guint i;
 
-	memset(hsep_global_table, 0, sizeof hsep_global_table);
+	ZERO(&hsep_global_table);
 
 	for (sl = node_all_nodes(); sl; sl = g_slist_next(sl)) {
 		struct gnutella_node *n = sl->data;
@@ -397,8 +400,8 @@ hsep_reset(void)
 
 		g_assert(n->hsep);
 
-		memset(n->hsep->table, 0, sizeof n->hsep->table);
-		memset(n->hsep->sent_table, 0, sizeof n->hsep->sent_table);
+		ZERO(&n->hsep->table);
+		ZERO(&n->hsep->sent_table);
 
 		/* this is what we know before receiving the first message */
 
@@ -421,7 +424,7 @@ hsep_reset(void)
  */
 
 void
-hsep_connection_init(struct gnutella_node *n)
+hsep_connection_init(struct gnutella_node *n, guint8 major, guint8 minor)
 {
 	static const hsep_ctx_t zero_hsep;
 	time_t now = tm_time();
@@ -433,9 +436,11 @@ hsep_connection_init(struct gnutella_node *n)
 		printf("HSEP: Initializing node %s\n",
 			host_addr_port_to_string(n->addr, n->port));
 
-	n->hsep = walloc(sizeof *n->hsep);
+	WALLOC(n->hsep);
 	*n->hsep = zero_hsep; /* Initializes everything to 0 */
 	n->hsep->last_sent = now;
+	n->hsep->major = major;
+	n->hsep->minor = minor;
 
 	/* this is what we know before receiving the first message */
 
@@ -512,9 +517,8 @@ hsep_timer(time_t now)
  * to be closed. The connection's HSEP data is restored to
  * zero and the CAN_HSEP attribute is cleared.
  */
-
 void
-hsep_connection_close(struct gnutella_node *n)
+hsep_connection_close(struct gnutella_node *n, gboolean in_shutdown)
 {
 	unsigned int i, j;
 
@@ -525,6 +529,9 @@ hsep_connection_close(struct gnutella_node *n)
 		printf("HSEP: Deinitializing node %s\n",
 			host_addr_port_to_string(n->addr, n->port));
 
+	if (in_shutdown)
+		goto cleanup;
+
 	for (i = 1; i < G_N_ELEMENTS(hsep_global_table); i++) {
 
 		for (j = 0; j < G_N_ELEMENTS(hsep_global_table[0]); j++) {
@@ -533,19 +540,20 @@ hsep_connection_close(struct gnutella_node *n)
 		}
 	}
 
+	if (GNET_PROPERTY(hsep_debug) > 1)
+		hsep_dump_table();
+
+	hsep_fire_global_table_changed(tm_time());
+
 	/*
 	 * Clear CAN_HSEP attribute so that the HSEP code
 	 * will not use the node any longer.
 	 */
 
+cleanup:
 	n->attrs &= ~NODE_A_CAN_HSEP;
-	wfree(n->hsep, sizeof *n->hsep);
+	WFREE(n->hsep);
 	n->hsep = NULL;
-
-	if (GNET_PROPERTY(hsep_debug) > 1)
-		hsep_dump_table();
-
-	hsep_fire_global_table_changed(tm_time());
 }
 
 static inline void
@@ -658,7 +666,7 @@ hsep_process_msg(struct gnutella_node *n, time_t now)
 	for (k = 0, i = 1; k < max; k++, i++) {
 
 		if (GNET_PROPERTY(hsep_debug) > 1) {
-			gchar buf[G_N_ELEMENTS(messaget[0])][32];
+			char buf[G_N_ELEMENTS(messaget[0])][32];
 
 			for (j = 0; j < G_N_ELEMENTS(buf); j++)
 				uint64_to_string_buf(messaget[k][j], buf[j], sizeof buf[0]);
@@ -719,7 +727,7 @@ void
 hsep_send_msg(struct gnutella_node *n, time_t now)
 {
 	hsep_triple tmp[G_N_ELEMENTS(n->hsep->sent_table)], other;
-	unsigned int i, j, msglen, triples, opttriples;
+	unsigned int i, j, msglen, msgsize, triples, opttriples;
 	gnutella_msg_hsep_t *msg;
 	hsep_ctx_t *hsep;
 
@@ -727,6 +735,7 @@ hsep_send_msg(struct gnutella_node *n, time_t now)
 	g_assert(n->hsep);
 
 	hsep = n->hsep;
+	ZERO(&other);
 
 	/*
 	 * If we are a leaf, we just need to send one triple,
@@ -736,16 +745,14 @@ hsep_send_msg(struct gnutella_node *n, time_t now)
 	 * it contains only our own triple, which is correct.
 	 */
 
-	triples = NODE_P_LEAF == GNET_PROPERTY(current_peermode)
-				? 1
-				: G_N_ELEMENTS(tmp);
+	triples = settings_is_leaf() ? 1 : G_N_ELEMENTS(tmp);
 
 	/*
 	 * Allocate and initialize message to send.
 	 */
 
-	msglen = GTA_HEADER_SIZE + triples * (sizeof *msg - GTA_HEADER_SIZE);
-	msg = g_malloc(msglen);
+	msgsize = GTA_HEADER_SIZE + triples * (sizeof *msg - GTA_HEADER_SIZE);
+	msg = walloc(msgsize);
 
 	{
 		gnutella_header_t *header;
@@ -782,7 +789,7 @@ hsep_send_msg(struct gnutella_node *n, time_t now)
 	if (
 		0 == memcmp(tmp, hsep->sent_table, sizeof tmp)
 	) {
-		G_FREE_NULL(msg);
+		WFREE_NULL(msg, msgsize);
 		goto charge_timer;
 	}
 
@@ -810,7 +817,7 @@ hsep_send_msg(struct gnutella_node *n, time_t now)
 
 	for (i = 0; i < opttriples; i++) {
 		if (GNET_PROPERTY(hsep_debug) > 1) {
-			gchar buf[G_N_ELEMENTS(hsep_own)][32];
+			char buf[G_N_ELEMENTS(hsep_own)][32];
 
 			for (j = 0; j < G_N_ELEMENTS(buf); j++) {
 				guint64 v;
@@ -837,7 +844,7 @@ hsep_send_msg(struct gnutella_node *n, time_t now)
 	/* send message to peer node */
 	gmsg_sendto_one(n, msg, msglen);
 
-	G_FREE_NULL(msg);
+	WFREE_NULL(msg, msgsize);
 
 	/*
 	 * Update counters.
@@ -869,7 +876,7 @@ hsep_notify_shared(guint64 own_files, guint64 own_kibibytes)
 	) {
 
 		if (GNET_PROPERTY(hsep_debug)) {
-			printf("HSEP: Shared files changed to %s (%s KiB)\n",
+			g_debug("HSEP: Shared files changed to %s (%s KiB)",
 			    uint64_to_string(own_files), uint64_to_string2(own_kibibytes));
 		}
 
@@ -1008,10 +1015,10 @@ hsep_get_non_hsep_triple(hsep_triple *tripledest)
  * @attention
  * NB: The static buffers for each column are disjunct.
  */
-const gchar *
-hsep_get_static_str(gint row, gint column)
+const char *
+hsep_get_static_str(int row, int column)
 {
-	const gchar *ret = NULL;
+	const char *ret = NULL;
 	hsep_triple hsep_table[G_N_ELEMENTS(hsep_global_table)];
 	hsep_triple other[1];
 	guint64 v;
@@ -1022,7 +1029,7 @@ hsep_get_static_str(gint row, gint column)
     switch (column) {
     case HSEP_IDX_NODES:
 		{
-			static gchar buf[UINT64_DEC_BUFLEN];
+			static char buf[UINT64_DEC_BUFLEN];
 
 			v = hsep_table[row][HSEP_IDX_NODES] + other[0][HSEP_IDX_NODES];
 			uint64_to_string_buf(v, buf, sizeof buf);
@@ -1032,7 +1039,7 @@ hsep_get_static_str(gint row, gint column)
 
     case HSEP_IDX_FILES:
 		{
-			static gchar buf[UINT64_DEC_BUFLEN];
+			static char buf[UINT64_DEC_BUFLEN];
 
 			v = hsep_table[row][HSEP_IDX_FILES] + other[0][HSEP_IDX_FILES];
 			uint64_to_string_buf(v, buf, sizeof buf);
@@ -1042,7 +1049,7 @@ hsep_get_static_str(gint row, gint column)
 
 	case HSEP_IDX_KIB:
 		{
-			static gchar buf[UINT64_DEC_BUFLEN];
+			static char buf[UINT64_DEC_BUFLEN];
 
 			/* Make a copy because concurrent usage of short_kb_size()
 	 	 	 * could be hard to discover. */
@@ -1062,7 +1069,7 @@ hsep_get_static_str(gint row, gint column)
 /**
  * @returns the size of the global hsep table
  */
-gint
+int
 hsep_get_table_size(void)
 {
 	hsep_triple hsep_table[G_N_ELEMENTS(hsep_global_table)];
