@@ -1,5 +1,5 @@
 /*
- * $Id: ignore.h,v 1.5 2005/08/01 09:39:32 rmanfredi Exp $
+ * $Id: ignore.h 17029 2009-10-04 15:41:06Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -36,27 +36,32 @@
 #ifndef _core_ignore_h_
 #define _core_ignore_h_
 
-#include <glib.h>
+#include "common.h"
 
 void ignore_init(void);
 void ignore_close(void);
 
 enum ignore_val {
 	IGNORE_FALSE = 0,		/**< Do not ignore */
-	IGNORE_SHA1 = 1,		/**< Ignore because of SHA1 */
-	IGNORE_NAMESIZE = 2,	/**< Ignore because of Name & Size */
-	IGNORE_LIBRARY = 3,		/**< Ignore because SHA1 present in library */
-	IGNORE_HOSTILE = 4,		/**< Ignore because IP address is hostile */
-	IGNORE_OURSELVES = 5	/**< Ignore because IP:port points to ourselves */
+	IGNORE_SHA1,			/**< Ignore because of SHA1 */
+	IGNORE_NAMESIZE,		/**< Ignore because of Name & Size */
+	IGNORE_LIBRARY,			/**< Ignore because SHA1 present in library */
+	IGNORE_HOSTILE,			/**< Ignore because IP address is hostile */
+	IGNORE_OURSELVES,		/**< Ignore because IP:port points to ourselves */
+	IGNORE_LIMIT,			/**< Ignore because of country limits */
+	IGNORE_SPAM				/**< Ignore because SHA1 is known spam */
 };
 
+struct sha1;
+
 void ignore_timer(time_t now);
 enum ignore_val ignore_is_requested(
-	const gchar *file, filesize_t size, gchar *sha1);
+	const char *file, filesize_t size, const struct sha1 *sha1);
+const char *ignore_reason_to_string(enum ignore_val);
 
-void ignore_add_filesize(const gchar *file, filesize_t size);
-void ignore_add_sha1(const gchar *file, const gchar *sha1);
-gchar *ignore_sha1_filename(const gchar *sha1);
+void ignore_add_filesize(const char *file, filesize_t size);
+void ignore_add_sha1(const char *file, const struct sha1 *sha1);
+const char *ignore_sha1_filename(const struct sha1 *sha1);
 
 #endif	/* _core_ignore_h_ */
 
