@@ -1,5 +1,5 @@
 /*
- * $Id: inet.h,v 1.7 2005/08/17 13:44:05 cbiere Exp $
+ * $Id: inet.h 19293 2011-06-19 17:30:31Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -27,7 +27,7 @@
  * @ingroup core
  * @file
  *
- * Pong caching (LimeWire's ping/pong reducing scheme).
+ * Internet status.
  *
  * @author Raphael Manfredi
  * @date 2001-2003
@@ -47,7 +47,7 @@ void inet_init(void);
 void inet_close(void);
 
 void inet_firewalled(void);
-void inet_udp_firewalled(void);
+void inet_udp_firewalled(gboolean new_env);
 
 gboolean inet_can_answer_ping(void);
 
@@ -55,6 +55,8 @@ void inet_got_incoming(const host_addr_t addr);
 void inet_read_activity(void);
 void inet_udp_got_incoming(const host_addr_t addr);
 void inet_udp_record_sent(const host_addr_t addr);
+void inet_udp_got_unsolicited_incoming(void);
+void inet_udp_check_unsolicited(void);
 
 void inet_connection_attempted(const host_addr_t addr);
 void inet_connection_succeeded(const host_addr_t addr);
