@@ -1,5 +1,5 @@
 /*
- * $Id: ioheader.c,v 1.8 2005/06/29 14:24:23 daichik Exp $
+ * $Id: ioheader.c 19142 2011-05-23 20:44:04Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: ioheader.c,v 1.8 2005/06/29 14:24:23 daichik Exp $");
+RCSID("$Id: ioheader.c 19142 2011-05-23 20:44:04Z rmanfredi $")
 
 #include "ioheader.h"
 #include "sockets.h"
@@ -44,7 +44,9 @@ RCSID("$Id: ioheader.c,v 1.8 2005/06/29 14:24:23 daichik Exp $");
 #include "lib/getline.h"
 #include "lib/header.h"
 #include "lib/inputevt.h"
+#include "lib/log.h"		/* For log_printable() */
 #include "lib/misc.h"
+#include "lib/str.h"
 #include "lib/walloc.h"
 
 #include "if/gnet_property_priv.h"
@@ -53,63 +55,74 @@ RCSID("$Id: ioheader.c,v 1.8 2005/06/29 14:24:23 daichik Exp $");
 
 #define DFLT_SIZE	2048	/**< Expected headers should not be larger */
 
+enum io_opaque_magic { IO_OPAQUE_MAGIC = 0x578da161U };
+
 /**
  * This structure is used to encapsulate the various arguments required
  * by the header parsing I/O callbacks.
  */
 struct io_header {
-	gpointer resource;					/**< Resource for which we're parsing */
-	gpointer *io_opaque;				/**< Where we're referenced in resource */
-	struct gnutella_socket *socket;		/**< Socket on which we're reading */
-	bsched_t *bs;						/**< Bandwidth scheduler to use */
+	enum io_opaque_magic magic;
+	gpointer resource;				/**< Resource for which we're parsing */
+	gpointer *io_opaque;			/**< Where we're referenced in resource */
+	struct gnutella_socket *socket;	/**< Socket on which we're reading */
+	bsched_bws_t bws;				/**< Bandwidth scheduler to use */
 	header_t *header;
 	getline_t *getline;
-	const struct io_error *error;		/**< Error callbacks */
-	io_done_cb_t process_header;		/**< Called when all headers are read */
-	io_start_cb_t header_read_start;	/**< Called when reading first byte */
-	GString *text;						/**< Full header text */
-	gint flags;
+	const struct io_error *error;	/**< Error callbacks */
+	io_done_cb_t process_header;	/**< Called when all headers are read */
+	io_start_cb_t header_read_start;/**< Called when reading first byte */
+	str_t *text;					/**< Full header text */
+	guint read_bytes;				/**< Amount of bytes read */
+	int flags;
 };
 
+/**
+ * Internal consistency checks.
+ */
+static void
+io_check(const gpointer opaque)
+{
+	struct io_header *ih = opaque;
+
+	g_assert(ih);
+	g_assert(IO_OPAQUE_MAGIC == ih->magic);
+	g_assert(ih->io_opaque);
+	g_assert((char *) ih->io_opaque > (char *) ih->resource);
+	g_assert(((char *) ih->io_opaque - (char *) ih->resource) < 1024);
+	g_assert(*ih->io_opaque == opaque);
+}
+
 /**
  * Free the opaque I/O data.
  */
 void
 io_free(gpointer opaque)
 {
-	struct io_header *ih = (struct io_header *) opaque;
-
-	g_assert(ih);
-	g_assert(ih->io_opaque);
-	g_assert((gchar *) ih->io_opaque > (gchar *) ih->resource);
-	g_assert(((gchar *) ih->io_opaque - (gchar *) ih->resource) < 1024);
-	g_assert(*ih->io_opaque == opaque);
+	struct io_header *ih = opaque;
 
+	io_check(opaque);
 	*ih->io_opaque = NULL;
 
 	if (ih->header)
 		header_free(ih->header);
 	if (ih->getline)
 		getline_free(ih->getline);
-	if (ih->text)
-		g_string_free(ih->text, TRUE);
+	str_destroy_null(&ih->text);
 
-	wfree(ih, sizeof(*ih));
+	ih->magic = 0;
+	WFREE(ih);
 }
 
 /**
  * Fetch header structure from opaque I/O data.
  */
 struct header *
-io_header(gpointer opaque)
+io_header(const gpointer opaque)
 {
-	struct io_header *ih = (struct io_header *) opaque;
+	struct io_header *ih = opaque;
 
-	g_assert(ih);
-	g_assert(ih->io_opaque);
-	g_assert((gchar *) ih->io_opaque > (gchar *) ih->resource);
-	g_assert(((gchar *) ih->io_opaque - (gchar *) ih->resource) < 1024);
-	g_assert(*ih->io_opaque == opaque);
+	io_check(opaque);
 
 	return ih->header;
 }
@@ -118,15 +131,11 @@ io_header(gpointer opaque)
  * Fetch getline structure from opaque I/O data.
  */
 struct getline *
-io_getline(gpointer opaque)
+io_getline(const gpointer opaque)
 {
-	struct io_header *ih = (struct io_header *) opaque;
+	struct io_header *ih = opaque;
 
-	g_assert(ih);
-	g_assert(ih->io_opaque);
-	g_assert((gchar *) ih->io_opaque > (gchar *) ih->resource);
-	g_assert(((gchar *) ih->io_opaque - (gchar *) ih->resource) < 1024);
-	g_assert(*ih->io_opaque == opaque);
+	io_check(opaque);
 
 	return ih->getline;
 }
@@ -137,20 +146,29 @@ io_getline(gpointer opaque)
  * It is up to the caller to strdup the data if needed.
  * The returned data will be freed when io_free() is called.
  */
-gchar *
-io_gettext(gpointer opaque)
+char *
+io_gettext(const gpointer opaque)
 {
-	struct io_header *ih = (struct io_header *) opaque;
+	struct io_header *ih = opaque;
 
-	g_assert(ih);
-	g_assert(ih->io_opaque);
-	g_assert((gchar *) ih->io_opaque > (gchar *) ih->resource);
-	g_assert(((gchar *) ih->io_opaque - (gchar *) ih->resource) < 1024);
-	g_assert(*ih->io_opaque == opaque);
+	io_check(opaque);
 	g_assert(ih->flags & IO_SAVE_HEADER);	/* They must have requested this */
 	g_assert(ih->text);
 
-	return ih->text->str;
+	return str_2c(ih->text);
+}
+
+/**
+ * How many bytes we received so far during header parsing.
+ */
+guint
+io_get_read_bytes(const gpointer opaque)
+{
+	struct io_header *ih = opaque;
+
+	io_check(opaque);
+
+	return ih->read_bytes;
 }
 
 /**
@@ -161,10 +179,9 @@ static void
 io_header_parse(struct io_header *ih)
 {
 	struct gnutella_socket *s = ih->socket;
-	getline_t *getline = ih->getline;
 	header_t *header = ih->header;
-	gint parsed;
-	gint error;
+	size_t parsed;
+	int error;
 
 	/*
 	 * Read header a line at a time.  We have exacly s->pos chars to handle.
@@ -172,25 +189,25 @@ io_header_parse(struct io_header *ih)
 	 */
 
 nextline:
-	switch (getline_read(getline, s->buffer, s->pos, &parsed)) {
+	switch (getline_read(ih->getline, s->buf, s->pos, &parsed)) {
 	case READ_OVERFLOW:
 		g_warning("io_header_parse: line too long, disconnecting from %s",
-			ip_to_gchar(s->ip));
-		dump_hex(stderr, "Leading Data", s->buffer, MIN(s->pos, 256));
-		fprintf(stderr, "------ Header Dump:\n");
-		header_dump(header, stderr);
-		fprintf(stderr, "------\n");
-		(*ih->error->line_too_long)(ih->resource);
+			host_addr_to_string(s->addr));
+		if (log_printable(LOG_STDERR)) {
+			dump_hex(stderr, "Leading Data", s->buf, MIN(s->pos, 256));
+			fprintf(stderr, "------ Header Dump:\n");
+			header_dump(stderr, header, "-----");
+		}
+		(*ih->error->line_too_long)(ih->resource, header);
 		return;
 		/* NOTREACHED */
 	case READ_DONE:
-		if (s->pos != (size_t) parsed)
-			memmove(s->buffer, s->buffer + parsed, s->pos - parsed);
+		if (s->pos != parsed)
+			memmove(s->buf, &s->buf[parsed], s->pos - parsed);
 		s->pos -= parsed;
 		break;
 	case READ_MORE:		/* ok, but needs more data */
-	default:
-		g_assert((size_t) parsed == s->pos);
+		g_assert(parsed == s->pos);
 		s->pos = 0;
 		return;
 	}
@@ -209,9 +226,9 @@ nextline:
 		 */
 
 		g_assert(ih->text != NULL);
-		/* No g_string_append_len() in glib 1.2, use g_string_append() --RAM */
-		g_string_append(ih->text, getline_str(getline));
-		g_string_append_c(ih->text, '\n');
+		str_cat_len(ih->text,
+			getline_str(ih->getline), getline_length(ih->getline));
+		str_putc(ih->text, '\n');
 	}
 
 	if (ih->flags & IO_SAVE_FIRST) {
@@ -221,11 +238,10 @@ nextline:
 		 * the following header lines.
 		 */
 
-		g_assert(s->getline == 0);
-		s->getline = getline_make(getline_length(getline) + 1);
-
-		getline_copy(getline, s->getline);
-		getline_reset(getline);
+		g_assert(NULL == s->getline);
+		s->getline = getline_make(getline_length(ih->getline) + 1);
+		getline_copy(ih->getline, s->getline);
+		getline_reset(ih->getline);
 		ih->flags &= ~IO_SAVE_FIRST;
 		goto nextline;
 	}
@@ -238,19 +254,18 @@ nextline:
 
 		g_assert(s->gdk_tag);
 
-		g_source_remove(s->gdk_tag);
-		s->gdk_tag = 0;
+		socket_evt_clear(s);
 
 		ih->process_header(ih->resource, ih->header);
 		return;
 	}
 
 	error = header_append(header,
-		getline_str(getline), getline_length(getline));
-
+		getline_str(ih->getline), getline_length(ih->getline));
+	
 	switch (error) {
 	case HEAD_OK:
-		getline_reset(getline);
+		getline_reset(ih->getline);
 		goto nextline;			/* Go process other lines we may have read */
 		/* NOTREACHED */
 	case HEAD_EOH:				/* We reached the end of the header */
@@ -262,26 +277,28 @@ nextline:
 		/* FALL THROUGH */
 	case HEAD_EOH_REACHED:
 		g_warning("io_header_parse: %s, disconnecting from %s",
-			header_strerror(error),	ip_to_gchar(s->ip));
-		fprintf(stderr, "------ Header Dump:\n");
-		header_dump(header, stderr);
-		fprintf(stderr, "------\n");
-		dump_hex(stderr, "Header Line", getline_str(getline),
-			MIN(getline_length(getline), 128));
+			header_strerror(error),	host_addr_to_string(s->addr));
+		if (log_printable(LOG_STDERR)) {
+			fprintf(stderr, "------ Header Dump:\n");
+			header_dump(stderr, header, "-----");
+			dump_hex(stderr, "Header Line", getline_str(ih->getline),
+				MIN(getline_length(ih->getline), 128));
+		}
 		(*ih->error->header_error)(ih->resource, error);
 		return;
 		/* NOTREACHED */
 	default:					/* Error, but try to continue */
-		if (dbg) {
+		if (GNET_PROPERTY(dbg)) {
 			g_warning("io_header_parse: %s, from %s",
-				header_strerror(error), ip_to_gchar(s->ip));
-			dump_hex(stderr, "Header Line",
-				getline_str(getline), getline_length(getline));
-			fprintf(stderr, "------ Header Dump (so far):\n");
-			header_dump(header, stderr);
-			fprintf(stderr, "------\n");
+				header_strerror(error), host_addr_to_string(s->addr));
+			if (log_printable(LOG_STDERR)) {
+				dump_hex(stderr, "Header Line",
+					getline_str(ih->getline), getline_length(ih->getline));
+				fprintf(stderr, "------ Header Dump (so far):\n");
+				header_dump(stderr, header, "-----");
+			}
 		}
-		getline_reset(getline);
+		getline_reset(ih->getline);
 		goto nextline;			/* Go process other lines we may have read */
 	}
 
@@ -290,12 +307,12 @@ nextline:
 	 */
 
 	if ((ih->flags & IO_HEAD_ONLY) && s->pos) {
-        if (dbg) {
-            g_message("remote %s sent extra bytes after headers",
-                ip_to_gchar(s->ip));
-            dump_hex(stderr, "Extra Data", s->buffer, MIN(s->pos, 512));
+        if (GNET_PROPERTY(dbg)) {
+            g_debug("remote %s sent extra bytes after headers",
+                host_addr_to_string(s->addr));
+            dump_hex(stderr, "Extra Data", s->buf, MIN(s->pos, 512));
         }
-		(*ih->error->header_extra_data)(ih->resource);
+		(*ih->error->header_extra_data)(ih->resource, ih->header);
 		return;
 	}
 
@@ -328,8 +345,7 @@ nextline:
 
 		g_assert(s->gdk_tag);
 
-		g_source_remove(s->gdk_tag);
-		s->gdk_tag = 0;
+		socket_evt_clear(s);
 	}
 
 	ih->process_header(ih->resource, ih->header);
@@ -342,9 +358,9 @@ nextline:
  * Read data is then handed out to io_header_parse() for analysis.
  */
 static void
-io_read_data(gpointer data, gint unused_source, inputevt_cond_t cond)
+io_read_data(gpointer data, int unused_source, inputevt_cond_t cond)
 {
-	struct io_header *ih = (struct io_header *) data;
+	struct io_header *ih = data;
 	struct gnutella_socket *s = ih->socket;
 	size_t count;
 	ssize_t r;
@@ -355,7 +371,7 @@ io_read_data(gpointer data, gint unused_source, inputevt_cond_t cond)
 
 	if (cond & INPUT_EVENT_EXCEPTION) {
 		socket_eof(s);
-		(*ih->error->input_exception)(ih->resource);
+		(*ih->error->input_exception)(ih->resource, ih->header);
 		return;
 	}
 
@@ -374,12 +390,12 @@ io_read_data(gpointer data, gint unused_source, inputevt_cond_t cond)
 	 * never happen.
 	 */
 
-	g_assert(sizeof(s->buffer) >= s->pos);
-	count = sizeof(s->buffer) - s->pos;
+	g_assert(s->buf_size >= s->pos);
+	count = s->buf_size - s->pos;
 	if (count < 1) {
 		g_warning("ih_header_read: incoming buffer full, "
-			"disconnecting from %s", ip_to_gchar(s->ip));
-		dump_hex(stderr, "Leading Data", s->buffer, MIN(s->pos, 256));
+			"disconnecting from %s", host_addr_to_string(s->addr));
+		dump_hex(stderr, "Leading Data", s->buf, MIN(s->pos, 256));
 		(*ih->error->input_buffer_full)(ih->resource);
 		return;
 	}
@@ -390,13 +406,13 @@ io_read_data(gpointer data, gint unused_source, inputevt_cond_t cond)
 	 * errors to our client.
 	 */
 
-	r = bws_read(ih->bs, &s->wio, s->buffer + s->pos, count);
+	r = bws_read(ih->bws, &s->wio, &s->buf[s->pos], count);
 	if (r == 0) {
 		socket_eof(s);
-		(*ih->error->header_read_eof)(ih->resource);
+		(*ih->error->header_read_eof)(ih->resource, ih->header);
 		return;
 	} else if ((ssize_t) -1 == r) {
-	  	if (errno != EAGAIN) {
+	  	if (!is_temporary_error(errno)) {
 			socket_eof(s);
 			(*ih->error->header_read_error)(ih->resource, errno);
 		}
@@ -410,10 +426,42 @@ io_read_data(gpointer data, gint unused_source, inputevt_cond_t cond)
 	 */
 
 	s->pos += r;
+	ih->read_bytes += r;
 
 	io_header_parse(ih);
 }
 
+/**
+ * This routine is only used when doing HTTP pipelining, to be able to
+ * process already received network traffic from a connection, which is
+ * expected to be found in the socket's buffer.
+ *
+ * Read data is then handed out to io_header_parse() for analysis.
+ */
+void
+io_add_header(gpointer opaque)
+{
+	struct io_header *ih = opaque;
+	struct gnutella_socket *s;
+
+	io_check(opaque);
+
+	s = ih->socket;
+	g_assert(s != NULL);
+
+	/*
+	 * First time we read data, notify them once if needed.
+	 */
+
+	if (ih->header_read_start) {
+		(*ih->header_read_start)(ih->resource);
+		ih->header_read_start = NULL;
+	}
+
+	ih->read_bytes += s->pos;
+	io_header_parse(ih);
+}
+
 /**
  * Setup input callback and context for reading/parsing the header.
  * The I/O parsing context is directly written into the structure.
@@ -427,21 +475,20 @@ void
 io_get_header(
 	gpointer resource,			/**< Resource for which we're reading headers */
 	gpointer *io_opaque,		/**< Field address in resource's structure */
-	bsched_t *bs,				/**< B/w scheduler from which we read */
+	bsched_bws_t bws,			/**< B/w scheduler from which we read */
 	struct gnutella_socket *s,	/**< Socket from which we're reading */
-	gint flags,					/**< I/O parsing flags */
+	int flags,					/**< I/O parsing flags */
 	io_done_cb_t done,			/**< Mandatory: final callback when all done */
 	io_start_cb_t start,		/**< Optional: called when reading 1st byte */
 	const struct io_error *error) /**< Mandatory: error callbacks for resource */
 {
 	struct io_header *ih;
-	gint mask;
 
 	g_assert(resource);
 	g_assert(io_opaque);
-	g_assert((gchar *) io_opaque > (gchar *) resource);
-	g_assert(((gchar *) io_opaque - (gchar *) resource) < 1024);
-	g_assert(bs);
+	g_assert((char *) io_opaque > (char *) resource);
+	g_assert(((char *) io_opaque - (char *) resource) < 1024);
+	g_assert(bws < NUM_BSCHED_BWS);
 	g_assert(s);
 	g_assert(done);
 	g_assert(error);
@@ -452,18 +499,20 @@ io_get_header(
 	 * Create and initialize the callback argument used during header reading.
 	 */
 
-	ih = walloc(sizeof(*ih));
+	WALLOC(ih);
+	ih->magic = IO_OPAQUE_MAGIC;
 	ih->resource = resource;
 	ih->io_opaque = io_opaque;
 	ih->getline = getline_make(HEAD_MAX_SIZE);
 	ih->socket = s;
 	ih->flags = flags;
-	ih->bs = bs;
+	ih->bws = bws;
 	ih->error = error;
 	ih->process_header = done;
 	ih->header_read_start = start;
 	ih->header = (flags & IO_SINGLE_LINE) ? NULL : header_make();
-	ih->text = (flags & IO_SAVE_HEADER) ? g_string_sized_new(DFLT_SIZE) : NULL;
+	ih->text = (flags & IO_SAVE_HEADER) ? str_new(DFLT_SIZE) : NULL;
+	ih->read_bytes = 0;
 
 	/*
 	 * Associate the structure with the resource.
@@ -479,9 +528,7 @@ io_get_header(
 
 	g_assert(s->gdk_tag == 0);
 
-	mask = INPUT_EVENT_READ | INPUT_EVENT_EXCEPTION;
-	s->gdk_tag = inputevt_add(s->wio.fd(&s->wio), mask, io_read_data,
-					(gpointer) ih);
+	socket_evt_set(s, INPUT_EVENT_RX, io_read_data, ih);
 
 	/*
 	 * There may be pending input in the socket buffer, so go handle
@@ -502,13 +549,13 @@ io_get_header(
 void
 io_continue_header(
 	gpointer opaque,			/**< Existing header parsing context */
-	gint flags,					/**< New I/O parsing flags */
+	int flags,					/**< New I/O parsing flags */
 	io_done_cb_t done,			/**< Mandatory: final callback when all done */
 	io_start_cb_t start)		/**< Optional: called when reading 1st byte */
 {
-	struct io_header *ih = (struct io_header *) opaque;
+	struct io_header *ih = opaque;
 
-	g_assert(ih);
+	io_check(opaque);
 	g_assert(ih->flags & IO_3_WAY);
 	g_assert(!(flags & IO_3_WAY));
 
@@ -526,18 +573,18 @@ io_continue_header(
 		g_assert(ih->flags & IO_SAVE_HEADER);
 
 		if (flags & IO_SAVE_HEADER)
-			g_string_truncate(ih->text, 0);
+			str_setlen(ih->text, 0);
 		else {
-			g_string_free(ih->text, TRUE);
-			ih->text = NULL;
+			str_destroy_null(&ih->text);
 		}
 	} else if (flags & IO_SAVE_HEADER)
-		ih->text = g_string_sized_new(DFLT_SIZE);
+		ih->text = str_new(DFLT_SIZE);
 
 	header_reset(ih->header);
 	ih->flags = flags;
 	ih->process_header = done;
 	ih->header_read_start = start;
+	ih->read_bytes = 0;
 }
 
 /* vi: set ts=4 sw=4 cindent: */
