@@ -1,5 +1,5 @@
 /*
- * $Id: matching.c 12058 2006-09-27 19:33:55Z cbiere $
+ * $Id: matching.c 19142 2011-05-23 20:44:04Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -27,19 +27,22 @@
 
 #include "common.h"
 
-RCSID("$Id: matching.c 12058 2006-09-27 19:33:55Z cbiere $")
+RCSID("$Id: matching.c 19142 2011-05-23 20:44:04Z rmanfredi $")
 
 #include "matching.h"
 #include "qrp.h"				/* For qhvec_add() */
 #include "share.h"
+#include "search.h"				/* For lazy_safe_search() */
 
 #include "lib/atoms.h"
+#include "lib/ascii.h"
+#include "lib/halloc.h"
 #include "lib/pattern.h"
-#include "lib/misc.h"
+#include "lib/random.h"
+#include "lib/stringify.h"	/* For hex_escape() */
 #include "lib/utf8.h"
 #include "lib/wordvec.h"
 #include "lib/walloc.h"
-#include "lib/zalloc.h"
 
 #include "if/gnet_property_priv.h"
 
@@ -81,48 +84,57 @@ RCSID("$Id: matching.c 12058 2006-09-27 19:33:55Z cbiere $")
 
 #define ST_MIN_BIN_SIZE		4
 
-struct shared_file;
-
 struct st_entry {
-	gchar *string;				/* atom */
-	struct shared_file *sf;
+	const char *string;				/* atom */
+	shared_file_t *sf;
 	guint32 mask;
 };
 
 struct st_bin {
-	gint nslots, nvals;
+	int nslots, nvals;
 	struct st_entry **vals;
 };
 
-struct _search_table {
-	gint nentries, nchars, nbins;
+enum search_table_magic { SEARCH_TABLE_MAGIC = 0x0cf66242 };
+
+struct search_table {
+	enum search_table_magic magic;
+	int nentries, nchars, nbins;
 	struct st_bin **bins;
 	struct st_bin all_entries;
-	guchar index_map[(guchar) -1];
+	guchar index_map[MAX_INT_VAL(guchar)];
+	guchar fold_map[MAX_INT_VAL(guchar)];
 };
 
+static inline void
+search_table_check(const struct search_table * const st)
+{
+	g_assert(st != NULL);
+	g_assert(SEARCH_TABLE_MAGIC == st->magic);
+}
+
 static void
 destroy_entry(struct st_entry *entry)
 {
 	g_assert(entry != NULL);
 
 	atom_str_free_null(&entry->string);
-	shared_file_unref(entry->sf);
-	wfree(entry, sizeof *entry);
+	shared_file_unref(&entry->sf);
+	WFREE(entry);
 }
 
 /**
  * Initialize a bin.
  */
 static void
-bin_initialize(struct st_bin *bin, gint size)
+bin_initialize(struct st_bin *bin, int size)
 {
-	gint i;
+	int i;
 
 	bin->nvals = 0;
 	bin->nslots = size;
 
-	bin->vals = g_malloc(bin->nslots * sizeof bin->vals[0]);
+	bin->vals = halloc(bin->nslots * sizeof bin->vals[0]);
 	for (i = 0; i < bin->nslots; i++)
 		bin->vals[i] = NULL;
 }
@@ -133,8 +145,9 @@ bin_initialize(struct st_bin *bin, gint size)
 static struct st_bin *
 bin_allocate(void)
 {
-	struct st_bin *bin = walloc(sizeof *bin);
+	struct st_bin *bin;
 
+	WALLOC(bin);
 	bin_initialize(bin, ST_MIN_BIN_SIZE);
 	return bin;
 }
@@ -147,7 +160,7 @@ bin_allocate(void)
 static void
 bin_destroy(struct st_bin *bin)
 {
-	G_FREE_NULL(bin->vals);
+	HFREE_NULL(bin->vals);
 	bin->nslots = 0;
 	bin->nvals = 0;
 }
@@ -160,7 +173,7 @@ bin_insert_item(struct st_bin *bin, struct st_entry *entry)
 {
 	if (bin->nvals == bin->nslots) {
 		bin->nslots *= 2;
-		bin->vals = g_realloc(bin->vals, bin->nslots * sizeof bin->vals[0]);
+		bin->vals = hrealloc(bin->vals, bin->nslots * sizeof bin->vals[0]);
 	}
 	bin->vals[bin->nvals++] = entry;
 }
@@ -173,17 +186,21 @@ bin_compact(struct st_bin *bin)
 {
 	g_assert(bin->vals != NULL);	/* Or it would not have been allocated */
 
-	bin->vals = g_realloc(bin->vals, bin->nvals * sizeof bin->vals[0]);
+	bin->vals = hrealloc(bin->vals, bin->nvals * sizeof bin->vals[0]);
 	bin->nslots = bin->nvals;
 }
 
-static guchar map[(guchar) -1];
+static guchar map[MAX_INT_VAL(guchar)];
 
 static void
 setup_map(void)
 {
+	static gboolean done;
 	guint i;
 
+	if (done)
+		return;
+
 	for (i = 0; i < G_N_ELEMENTS(map); i++)	{
 		guchar c;
 
@@ -202,18 +219,21 @@ setup_map(void)
 		}
 		map[i] = c;
 	}
+
+	done = TRUE;
 }
 
 /**
  * Initialize permanent data in search table.
  */
-void
+static void
 st_initialize(search_table_t *table)
 {
-	static guchar fold_map[(guchar) -1];
 	guchar cur_char = '\0';
 	guint i;
 
+	search_table_check(table);
+
 	table->nentries = table->nchars = 0;
 	setup_map();
 
@@ -224,10 +244,10 @@ st_initialize(search_table_t *table)
 	for (i = 0; i < G_N_ELEMENTS(table->index_map); i++) {
 		guchar map_char = map[i];
 
-		if (fold_map[map_char]) {
-			table->index_map[i] = fold_map[map_char];
+		if (table->fold_map[map_char]) {
+			table->index_map[i] = table->fold_map[map_char];
 		} else {
-			fold_map[map_char] = cur_char;
+			table->fold_map[map_char] = cur_char;
 			table->index_map[i] = cur_char;
 			cur_char++;
 		}
@@ -238,42 +258,29 @@ st_initialize(search_table_t *table)
 	table->bins = NULL;
 	table->all_entries.vals = 0;
 
-	if (dbg > 3)
-		printf("search table will use max of %d bins (%d indexing chars)\n",
-			table->nbins, table->nchars);
-}
+	if (GNET_PROPERTY(matching_debug)) {
+		static gboolean done;
 
-/**
- * Allocates a new search_table_t. Use st_free() to free it.
- */
-search_table_t *
-st_alloc(void)
-{
-	search_table_t *table = walloc(sizeof *table);
-	return table;
-}
-
-void
-st_free(search_table_t **ptr)
-{
-	g_assert(ptr);
-	if (*ptr) {
-		search_table_t *table = *ptr;
-		wfree(table, sizeof *table);
-		*ptr = NULL;
+		if (!done) {
+			done = TRUE;
+			g_debug("MATCH search tables will use %d bins max "
+				"(%d indexing chars)", table->nbins, table->nchars);
+		}
 	}
 }
 
-
 /**
  * Recreate variable parts of the search table.
  */
-void
-st_create(search_table_t *table)
+static void
+st_recreate(search_table_t *table)
 {
-	gint i;
+	int i;
 
-	table->bins = g_malloc(table->nbins * sizeof table->bins[0]);
+	search_table_check(table);
+	g_assert(NULL == table->bins);
+
+	table->bins = halloc(table->nbins * sizeof table->bins[0]);
 	for (i = 0; i < table->nbins; i++)
 		table->bins[i] = NULL;
 
@@ -283,10 +290,12 @@ st_create(search_table_t *table)
 /**
  * Destroy a search table.
  */
-void
+static void
 st_destroy(search_table_t *table)
 {
-	gint i;
+	int i;
+
+	search_table_check(table);
 
 	if (table->bins) {
 		for (i = 0; i < table->nbins; i++) {
@@ -294,10 +303,10 @@ st_destroy(search_table_t *table)
 
 			if (bin) {
 				bin_destroy(bin);
-				wfree(bin, sizeof *bin);
+				WFREE(bin);
 			}
 		}
-		G_FREE_NULL(table->bins);
+		HFREE_NULL(table->bins);
 	}
 
 	if (table->all_entries.vals) {
@@ -309,12 +318,55 @@ st_destroy(search_table_t *table)
 	}
 }
 
+/**
+ * Allocates a new search_table_t.
+ * Use st_free() to free it.
+ */
+search_table_t *
+st_create(void)
+{
+	search_table_t *table;
+
+	WALLOC0(table);
+	table->magic = SEARCH_TABLE_MAGIC;
+	st_initialize(table);
+	st_recreate(table);
+	return table;
+}
+
+/**
+ * Free search table, nullifying its pointer.
+ */
+void
+st_free(search_table_t **ptr)
+{
+	g_assert(ptr);
+	if (*ptr) {
+		search_table_t *table = *ptr;
+		st_destroy(table);
+		table->magic = 0;
+		WFREE(table);
+		*ptr = NULL;
+	}
+}
+
+/**
+ * @return amount of entries in the table.
+ */
+int
+st_count(const search_table_t *table)
+{
+	search_table_check(table);
+
+	return table->all_entries.nvals;
+}
+
 /**
  * Compute character mask "hash", using one bit per letter of the alphabet,
  * plus one for any digit.
  */
 static guint32
-mask_hash(const gchar *s) {
+mask_hash(const char *s) {
 	guchar c;
 	guint32 mask = 0;
 
@@ -324,7 +376,7 @@ mask_hash(const gchar *s) {
 		else if (is_ascii_digit(c))
 			mask |= MASK_DIGIT;
 		else {
-			gint idx = ascii_tolower(c) - 97;
+			int idx = ascii_tolower(c) - 97;
 			if (idx >= 0 && idx < 26)
 				mask |= MASK_LETTER(idx);
 		}
@@ -336,8 +388,8 @@ mask_hash(const gchar *s) {
 /**
  * Get key of two-char pair.
  */
-static inline gint
-st_key(search_table_t *table, const gchar k[2])
+static inline int
+st_key(search_table_t *table, const char k[2])
 {
 	return table->index_map[(guchar) k[0]] * table->nchars +
 		table->index_map[(guchar) k[1]];
@@ -350,7 +402,7 @@ st_key(search_table_t *table, const gchar k[2])
  * @return TRUE if the item was inserted; FALSE otherwise.
  */
 gboolean
-st_insert_item(search_table_t *table, const gchar *s, gpointer data)
+st_insert_item(search_table_t *table, const char *s, const shared_file_t *sf)
 {
 	size_t i, len;
 	struct st_entry *entry;
@@ -362,14 +414,14 @@ st_insert_item(search_table_t *table, const gchar *s, gpointer data)
 
 	seen_keys = g_hash_table_new(NULL, NULL);
 
-	entry = walloc(sizeof *entry);
+	WALLOC(entry);
 	entry->string = atom_str_get(s);
-	entry->sf = shared_file_ref(data);
+	entry->sf = shared_file_ref(sf);
 	entry->mask = mask_hash(entry->string);
 
 	len = strlen(entry->string);
 	for (i = 0; i < len - 1; i++) {
-		gint key = st_key(table, &entry->string[i]);
+		int key = st_key(table, &entry->string[i]);
 
 		/* don't insert item into same bin twice */
 		if (g_hash_table_lookup(seen_keys, GINT_TO_POINTER(key)))
@@ -397,7 +449,7 @@ st_insert_item(search_table_t *table, const gchar *s, gpointer data)
 void
 st_compact(search_table_t *table)
 {
-	gint i;
+	int i;
 
 	if (!table->all_entries.nvals)
 		return;			/* Nothing in table */
@@ -413,7 +465,7 @@ st_compact(search_table_t *table)
  * Patterns are lazily compiled as needed, using pattern_compile_fast().
  */
 static gboolean
-entry_match(const gchar *text, size_t tlen,
+entry_match(const char *text, size_t tlen,
 	cpattern_t **pw, word_vec_t *wovec, size_t wn)
 {
 	size_t i;
@@ -425,7 +477,7 @@ entry_match(const gchar *text, size_t tlen,
 			pw[i] = pattern_compile_fast(wovec[i].word, wovec[i].len);
 
 		for (j = 0; j < amount; j++) {
-			const gchar *pos;
+			const char *pos;
 
 			pos = pattern_qsearch(pw[i], text, tlen, offset, qs_begin);
 			if (pos)
@@ -441,58 +493,114 @@ entry_match(const gchar *text, size_t tlen,
 }
 
 /**
- * Do an actual search.
+ * Fill non-NULL query hash vector for query routing.
+ *
+ * This needs to be called when st_search() is not called when processing
+ * a query, otherwise the qhery hash vector won't be properly initialized
+ * and the query would be improperly dropped by qrt_build_query_target(),
+ * hence never routed.
  */
 void
+st_fill_qhv(const char *search_term, query_hashvec_t *qhv)
+{
+	char *search;
+	word_vec_t *wovec;
+	guint wocnt;
+	guint i;
+
+	if (NULL == qhv)
+		return;
+
+	search = UNICODE_CANONIZE(search_term);
+	wocnt = word_vec_make(search, &wovec);
+
+	for (i = 0; i < wocnt; i++) {
+		if (wovec[i].len >= QRP_MIN_WORD_LENGTH)
+			qhvec_add(qhv, wovec[i].word, QUERY_H_WORD);
+	}
+
+	if (search != search_term)
+		HFREE_NULL(search);
+
+	if (wocnt > 0)
+		word_vec_free(wovec, wocnt);
+}
+
+/**
+ * Do an actual search.
+ *
+ * @param table			table containing organized entries to search from
+ * @param search_term	the query string
+ * @param callback		routine to invoke for each match
+ * @param ctx			user-supplied data to pass on to callback
+ * @param max_res		maximum amount of results to return
+ * @param qhv			query hash vector built from query string, for routing
+ *
+ * @return number of hits we produced
+ */
+int
 st_search(
 	search_table_t *table,
-	gchar *search,
+	const char *search_term,
 	st_search_callback callback,
 	gpointer ctx,
-	gint max_res,
+	int max_res,
 	query_hashvec_t *qhv)
 {
-	gint key, nres = 0;
+	char *search;
+	int key, nres = 0;
 	guint i, len;
 	struct st_bin *best_bin = NULL;
-	gint best_bin_size = INT_MAX;
+	int best_bin_size = INT_MAX;
 	word_vec_t *wovec;
 	guint wocnt;
 	cpattern_t **pattern;
 	struct st_entry **vals;
-	gint vcnt;
-	gint scanned = 0;		/* measure search mask efficiency */
+	guint vcnt;
+	int scanned = 0;		/* measure search mask efficiency */
 	guint32 search_mask;
 	size_t minlen;
+	guint random_offset;  /* Randomizer for search returns */
 
+	search = UNICODE_CANONIZE(search_term);
+
+	if (GNET_PROPERTY(query_debug) > 4 && 0 != strcmp(search, search_term)) {
+		char *safe_search = hex_escape(search, FALSE);
+		char *safe_search_term = hex_escape(search_term, FALSE);
+		g_debug("original search term: \"%s\"", safe_search_term);
+		g_debug("canonical search term: \"%s\"", safe_search);
+		if (safe_search != search)
+			HFREE_NULL(safe_search);
+		if (safe_search_term != search_term)
+			HFREE_NULL(safe_search_term);
+	}
 	len = strlen(search);
 
 	/*
 	 * Find smallest bin
 	 */
 
-	if (len < 2)
-		return;
+	if (len >= 2) {
+		for (i = 0; i < len - 1; i++) {
+			struct st_bin *bin;
+			if (is_ascii_space(search[i]) || is_ascii_space(search[i+1]))
+				continue;
+			key = st_key(table, search + i);
+			if ((bin = table->bins[key]) == NULL) {
+				best_bin = NULL;
+				break;
+			}
+			if (bin->nvals < best_bin_size) {
+				best_bin = bin;
+				best_bin_size = bin->nvals;
+			}
+		}
 
-	for (i = 0; i < len - 1; i++) {
-		struct st_bin *bin;
-		if (is_ascii_space(search[i]) || is_ascii_space(search[i+1]))
-			continue;
-		key = st_key(table, search + i);
-		if ((bin = table->bins[key]) == NULL) {
-			best_bin = NULL;
-			break;
-		}
-		if (bin->nvals < best_bin_size) {
-			best_bin = bin;
-			best_bin_size = bin->nvals;
-		}
+		if (GNET_PROPERTY(matching_debug) > 4)
+			g_debug("MATCH st_search(): str=\"%s\", len=%d, best_bin_size=%d",
+				lazy_safe_search(search_term), len, best_bin_size);
 	}
 
-	if (dbg > 6)
-		printf("st_search(): str=\"%s\", len=%d, best_bin_size=%d\n",
-			search, len, best_bin_size);
-
 	/*
 	 * If the best_bin is NULL, we did not find a matching bin, and we're
 	 * sure we won't be able to find the search string.
@@ -509,7 +617,7 @@ st_search(
 		 */
 
 		if (qhv == NULL)
-			return;
+			goto finish;
 	}
 
 	/*
@@ -532,7 +640,7 @@ st_search(
 	if (wocnt == 0 || best_bin == NULL) {
 		if (wocnt > 0)
 			word_vec_free(wovec, wocnt);
-		return;
+		goto finish;
 	}
 
 	g_assert(best_bin_size > 0);	/* Allocated bin, it must hold something */
@@ -576,34 +684,51 @@ st_search(
 
 	vcnt = best_bin->nvals;
 	vals = best_bin->vals;
+	random_offset = random_u32() % vcnt;
 
-	while (nres < max_res && vcnt-- > 0) {
-		struct st_entry *e = *vals++;
-		struct shared_file *sf = e->sf;
+	nres = 0;
+	for (i = 0; i < vcnt; i++) {
+		const struct st_entry *e;
+		shared_file_t *sf;
 		size_t canonic_len;
 
+		/*
+		 * As we only return a limited count of results, pick a random
+		 * offset, so that repeated searches will match different items
+		 * instead of always the first - with some probability.
+		 */
+		e = vals[(i + random_offset) % vcnt];
+		
 		if ((e->mask & search_mask) != search_mask)
 			continue;		/* Can't match */
 
+		sf = e->sf;
+
 		canonic_len = shared_file_name_canonic_len(sf);
 		if (canonic_len < minlen)
 			continue;		/* Can't match */
 
 		scanned++;
 
-		if (
-			entry_match(e->string, canonic_len, pattern, wovec, wocnt)
-		) {
-			if (dbg > 5)
-				printf("MATCH: %s\n", shared_file_name_nfc(sf));
-			callback(ctx, sf);
-			nres++;
+		if (entry_match(e->string, canonic_len, pattern, wovec, wocnt)) {
+			if (GNET_PROPERTY(matching_debug) > 4) {
+				g_debug("MATCH \"%s\" matches %s",
+					search, shared_file_name_nfc(sf));
+			}
+
+			if ((*callback)(ctx, sf)) {
+				nres++;
+				if (nres >= max_res)
+					break;
+			}
 		}
 	}
 
-	if (dbg > 6)
-		printf("st_search(): scanned %d entry from the %d in bin, %d matches\n",
-			scanned, best_bin_size, nres);
+	if (GNET_PROPERTY(matching_debug) > 3)
+		g_debug("MATCH st_search(): scanned %d entr%s from the %d in bin, "
+			"got %d match%s",
+			scanned, 1 == scanned ? "y" : "ies",
+			best_bin_size, nres, 1 == nres ? "" : "es");
 
 	for (i = 0; i < wocnt; i++)
 		if (pattern[i])					/* Lazily compiled by entry_match() */
@@ -611,6 +736,13 @@ st_search(
 
 	wfree(pattern, wocnt * sizeof *pattern);
 	word_vec_free(wovec, wocnt);
+
+finish:
+	if (search != search_term) {
+		HFREE_NULL(search);
+	}
+
+	return nres;
 }
 
 /* vi: set ts=4 sw=4 cindent: */
