@@ -1,5 +1,5 @@
 /*
- * $Id: matching.h 11948 2006-09-12 21:41:53Z cbiere $
+ * $Id: matching.h 19112 2011-05-18 16:31:04Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -64,33 +64,40 @@
 #ifndef _core_matching_h_
 #define _core_matching_h_
 
-typedef struct _search_table search_table_t;
+#include "common.h"
+
+typedef struct search_table search_table_t;
 
 struct query_hashvec;
-
-search_table_t *st_alloc(void);
-void st_initialize(search_table_t *);
-void st_create(search_table_t *table);
-void st_destroy(search_table_t *);
-void st_free(search_table_t **);
-gboolean st_insert_item(search_table_t *, const gchar *key, gpointer data);
-void st_compact(search_table_t *);
-
 struct shared_file;
 
-/**
- * Needs brief description here.
- */
-typedef void (*st_search_callback)(gpointer ctx, gpointer data);
+search_table_t *st_create(void);
+void st_free(search_table_t **);
+gboolean st_insert_item(search_table_t *, const char *key,
+	const struct shared_file *sf);
+void st_compact(search_table_t *);
+int st_count(const search_table_t *st);
 
-void st_search(
+/**
+ * Callback for st_search().
+ *
+ * @param ctx		the search context (opaque for st_search)
+ * @param data		the matched data (shared file, opaque for st_search)
+ *
+ * @return TRUE if the match must be accounted as a valid result.
+ */
+typedef gboolean (*st_search_callback)(gpointer ctx, gpointer data);
+
+int st_search(
 	search_table_t *table,
-	gchar *search,
+	const char *search,
 	st_search_callback callback,
 	gpointer ctx,
-	gint max_res,
+	int max_res,
 	struct query_hashvec *qhv);
 
+void st_fill_qhv(const char *search_term, struct query_hashvec *qhv);
+
 #endif	/* _core_matching_h_ */
 
 /* vi: set ts=4 sw=4 cindent: */
