@@ -1,10 +1,8 @@
 /*
- * $Id: move.c,v 1.4 2004/12/09 04:28:05 cbiere Exp $
+ * $Id: move.c 19142 2011-05-23 20:44:04Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
- * Asychronous file moving operations.
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -25,67 +23,82 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup core
+ * @file
+ *
+ * Asychronous file moving operations.
+ *
+ * @author Raphael Manfredi
+ * @date 2002-2003
+ */
+
 #include "common.h"
 
-RCSID("$Id: move.c,v 1.4 2004/12/09 04:28:05 cbiere Exp $");
+RCSID("$Id: move.c 19142 2011-05-23 20:44:04Z rmanfredi $")
 
+#include "move.h"
 #include "downloads.h"
 #include "fileinfo.h"
-#include "move.h"
+#include "file_object.h"
 
 #include "lib/atoms.h"
 #include "lib/bg.h"
+#include "lib/compat_misc.h"
+#include "lib/fd.h"
 #include "lib/file.h"
+#include "lib/halloc.h"
+#include "lib/stringify.h"
+#include "lib/tm.h"
+#include "lib/unsigned.h"
 #include "lib/walloc.h"
 
 #include "if/gnet_property.h"
 #include "if/gnet_property_priv.h"
 
-#include "lib/override.h"		/* Must be the last header included */
+#include "lib/override.h"	/* Must be the last header included */
 
-#define COPY_BLOCK_SHIFT	12			/* Power of two of copy unit credit */
-#define COPY_BUF_SIZE		65536		/* Size of the reading buffer */
+#define COPY_BLOCK_FRAGMENT	4096		/**< Power of two of copy unit credit */
+#define COPY_BUF_SIZE		65536		/**< Size of the reading buffer */
 
-static gpointer move_daemon = NULL;
+static struct bgtask *move_daemon;
 
-#define MOVED_MAGIC	0x00c0b100
+enum moved_magic_t { MOVED_MAGIC = 0x0ac0b103 };
 
-/*
+/**
  * Moving daemon context.
  */
 struct moved {
-	gint magic;				/* Magic number */
-	struct download *d;		/* Download for which we're moving file */
-	gint rd;				/* Opened file descriptor for read, -1 if none */
-	gint wd;				/* Opened file descriptor for write, -1 if none */
-	time_t start;			/* Start time, to determine copying rate */
-	filesize_t size;		/* Size of file */
-	filesize_t copied;		/* Amount of data copied so far */
-	gchar *buffer;			/* Large buffer, where data is read */
-	gchar *target;			/* Target file name, in case an error occurs */
-	gint error;				/* Error code */
+	enum moved_magic_t magic;	/**< Magic number */
+	struct download *d;		/**< Download for which we're moving file */
+	char *buffer;			/**< Large buffer, where data is read */
+	char *target;			/**< Target file name, in case an error occurs */
+	time_t start;			/**< Start time, to determine copying rate */
+	filesize_t size;		/**< Size of file */
+	filesize_t copied;		/**< Amount of data copied so far */
+	struct file_object *rd;	/**< The file object to read the file. */
+	int wd;					/**< File descriptor for write, -1 if none */
+	int error;				/**< Error code */
 };
 
-/*
+/**
  * Work queue entry.
  */
 struct work {
-	struct download *d;		/* Download to move */
-	gchar *dest;			/* Target directory (atom) */
-	gchar *ext;				/* Trailing extension (atom) */
+	struct download *d;		/**< Download to move */
+	const char *dest;		/**< Target directory (atom) */
+	const char *ext;		/**< Trailing extension (atom) */
 };
 
-/*
- * we_alloc
- *
+/**
  * Allocate work queue entry.
  */
 static struct work *
-we_alloc(struct download *d, const gchar *dest, const gchar *ext)
+we_alloc(struct download *d, const char *dest, const char *ext)
 {
 	struct work *we;
 
-	we = walloc(sizeof(*we));
+	WALLOC(we);
 	we->d = d;
 	we->dest = atom_str_get(dest);
 	we->ext = atom_str_get(ext);
@@ -93,28 +106,26 @@ we_alloc(struct download *d, const gchar *dest, const gchar *ext)
 	return we;
 }
 
-/*
- * we_free
- *
+/**
  * Freeing of work queue entry.
  */
 static void
 we_free(gpointer data)
 {
-	struct work *we = (struct work *) data;
+	struct work *we = data;
 
-	atom_str_free(we->dest);
-	atom_str_free(we->ext);
-	wfree(we, sizeof(*we));
+	atom_str_free_null(&we->dest);
+	atom_str_free_null(&we->ext);
+	WFREE(we);
 }
 
 /**
  * Signal handler for termination.
  */
 static void
-d_sighandler(gpointer unused_h, gpointer u, bgsig_t sig)
+d_sighandler(struct bgtask *unused_h, gpointer u, bgsig_t sig)
 {
-	struct moved *md = (struct moved *) u;
+	struct moved *md = u;
 
 	(void) unused_h;
 	g_assert(md->magic == MOVED_MAGIC);
@@ -135,33 +146,28 @@ d_sighandler(gpointer unused_h, gpointer u, bgsig_t sig)
 	}
 }
 
-/*
- * d_free
- *
+/**
  * Freeing of computation context.
  */
 static void
 d_free(gpointer ctx)
 {
-	struct moved *md = (struct moved *) ctx;
+	struct moved *md = ctx;
 
 	g_assert(md->magic == MOVED_MAGIC);
 
-	if (md->rd != -1)
-		close(md->rd);
-
-	if (md->wd != -1)
-		close(md->wd);
-
-	G_FREE_NULL(md->buffer);
-	wfree(md, sizeof(*md));
+	file_object_release(&md->rd);
+	fd_forget_and_close(&md->wd);
+	HFREE_NULL(md->buffer);
+	md->magic = 0;
+	WFREE(md);
 }
 
 /**
  * Daemon's notification of start/stop.
  */
 static void
-d_notify(gpointer unused_h, gboolean on)
+d_notify(struct bgtask *unused_h, gboolean on)
 {
 	(void) unused_h;
 	gnet_prop_set_boolean_val(PROP_FILE_MOVING, on);
@@ -171,42 +177,49 @@ d_notify(gpointer unused_h, gboolean on)
  * Daemon's notification: starting to work on item.
  */
 static void
-d_start(gpointer h, gpointer ctx, gpointer item)
+d_start(struct bgtask *h, gpointer ctx, gpointer item)
 {
-	struct moved *md = (struct moved *) ctx;
-	struct work *we = (struct work *) item;
+	struct moved *md = ctx;
+	struct work *we = item;
 	struct download *d = we->d;
-	gchar *source = NULL;
-	struct stat buf;
-	const gchar *name;
+	filestat_t buf;
+	const char *name;
 
 	g_assert(md->magic == MOVED_MAGIC);
-	g_assert(md->rd == -1);
+	g_assert(md->rd == NULL);
 	g_assert(md->wd == -1);
 	g_assert(md->target == NULL);
 
 	download_move_start(d);
 	bg_task_signal(h, BG_SIG_TERM, d_sighandler);
 
-	source = make_pathname(download_path(d), download_outname(d));
-	g_return_if_fail(NULL != source);
-
 	md->d = we->d;
-	md->rd = file_open(source, O_RDONLY);
 
-	if (md->rd == -1) {
-		md->error = errno;
+	md->rd = file_object_open(download_pathname(d), O_RDONLY);
+	if (NULL == md->rd) {
+		int fd = file_absolute_open(download_pathname(d), O_RDONLY, 0);
+		if (fd < 0) {
+			md->error = errno;
+			goto abort_read;
+		}
+		md->rd = file_object_new(fd, download_pathname(d), O_RDONLY);
+	}
+
+	if (NULL == md->rd) {
+		md->error = EINVAL;
 		goto abort_read;
 	}
 
-	if (-1 == fstat(md->rd, &buf)) {
+	if (file_object_fstat(md->rd, &buf)) {
 		md->error = errno;
-		g_warning("can't fstat \"%s\": %s", source, g_strerror(errno));
+		g_warning("can't fstat \"%s\": %s",
+			download_pathname(d), g_strerror(errno));
 		goto abort_read;
 	}
 
 	if (!S_ISREG(buf.st_mode)) {
-		g_warning("file \"%s\" is not a regular file", source);
+		g_warning("cannot move file \"%s\": not a regular file",
+			download_pathname(d));
 		goto abort_read;
 	}
 
@@ -216,36 +229,31 @@ d_start(gpointer h, gpointer ctx, gpointer item)
 
 	name = file_info_readable_filename(d->file_info);
 
-	md->target = unique_filename(we->dest, name, we->ext);
+	md->target = file_info_unique_filename(we->dest, name, we->ext);
 	if (NULL == md->target)
 		goto abort_read;
 
 	md->wd = file_create(md->target, O_WRONLY | O_TRUNC, buf.st_mode);
-
-	if (md->wd == -1)
+	if (md->wd < 0)
 		goto abort_read;
 
-	md->start = time(NULL);
+	md->start = tm_time();
 	md->size = download_filesize(d);
 	md->copied = 0;
 	md->error = 0;
 
-	if (dbg > 1)
-		g_message("Moving \"%s\" to \"%s\"",
-				download_outname(d), md->target);
+	compat_fadvise_sequential(file_object_get_fd(md->rd), 0, 0);
 
-	G_FREE_NULL(source);
+	if (GNET_PROPERTY(move_debug) > 1)
+		g_debug("MOVE starting moving \"%s\" to \"%s\"",
+				file_object_get_pathname(md->rd), md->target);
 
 	return;
 
 abort_read:
 	md->error = errno;
-	if (md->rd != -1)
-		close(md->rd);
-	md->rd = -1;
-	g_warning("can't copy \"%s\" to \"%s\"", source, we->dest);
-	if (NULL != source)
-		G_FREE_NULL(source);
+	file_object_release(&md->rd);
+	g_warning("can't copy \"%s\" to \"%s\"", download_pathname(d), we->dest);
 	return;
 }
 
@@ -253,27 +261,28 @@ abort_read:
  * Daemon's notification: finished working on item.
  */
 static void
-d_end(gpointer h, gpointer ctx, gpointer item)
+d_end(struct bgtask *h, gpointer ctx, gpointer item)
 {
-	struct moved *md = (struct moved *) ctx;
+	struct moved *md = ctx;
 	struct download *d = md->d;
-	gint elapsed = 0;
+	int elapsed = 0;
 
 	g_assert(md->magic == MOVED_MAGIC);
 	g_assert(md->d == ((struct work *) item)->d);
 
 	bg_task_signal(h, BG_SIG_TERM, NULL);
 
-	if (md->rd == -1) {			/* Did not start properly */
+	if (NULL == md->rd) {			/* Did not start properly */
 		g_assert(md->error);
 		goto finish;
 	}
 
-	close(md->rd);
-	md->rd = -1;
-
-	close(md->wd);
-	md->wd = -1;
+	file_object_release(&md->rd);
+	if (fd_forget_and_close(&md->wd)) {
+		md->error = errno;
+		g_warning("error whilst closing copy target \"%s\": %s",
+			md->target, g_strerror(errno));
+	}
 
 	/*
 	 * If copying went well, get rid of the source file.
@@ -283,107 +292,140 @@ d_end(gpointer h, gpointer ctx, gpointer item)
 	 */
 
 	if (md->error == 0) {
-		gchar *source;
+		filestat_t buf;
 
 		g_assert(md->copied == md->size);
 
-		source = make_pathname(download_path(md->d), download_outname(md->d));
-		if (NULL == source || -1 == unlink(source))
+		/*
+		 * As a precaution, stat() the file.  When moving the file accross
+		 * NFS where the target filesystem is full, write() or close() may
+		 * not return ENOSPC.  Double-check here otherwise they'll lose
+		 * a perfectly good file.
+		 *		--RAM, 2007-07-30.
+		 */
+
+		if (-1 == stat(md->target, &buf)) {
+			md->error = errno;
+			g_warning("cannot stat copy target \"%s\": %s",
+				md->target, g_strerror(errno));
+			goto error;
+		}
+
+		if (
+			!S_ISREG(buf.st_mode) ||
+			(filesize_t) 0 + buf.st_size != (fileoffset_t) 0 + md->copied
+		) {
+			md->error = ENOSPC;
+			g_warning("target size mismatch for \"%s\": got only %s",
+				md->target, fileoffset_t_to_string(buf.st_size));
+			goto error;
+		}
+
+		if (!file_object_moved(download_pathname(md->d), md->target)) {
 			g_warning("cannot unlink \"%s\": %s",
-				download_outname(md->d), g_strerror(errno));
-		if (NULL != source)
-			G_FREE_NULL(source);
+				download_basename(md->d), g_strerror(errno));
+		}
 	} else {
 		if (md->target != NULL && -1 == unlink(md->target))
 			g_warning("cannot unlink \"%s\": %s",
 				md->target, g_strerror(errno));
 	}
 
-	elapsed = delta_time(time(NULL), md->start);
+	/* FALL THROUGH */
+
+error:
+	elapsed = delta_time(tm_time(), md->start);
 	elapsed = MAX(1, elapsed);		/* time warp? clock not monotic? */
 
-	if (dbg > 1)
-		printf("Moved file \"%s\" at %lu bytes/sec [error=%d]\n",
-			download_outname(md->d), (gulong) md->size / elapsed, md->error);
+	if (GNET_PROPERTY(move_debug) > 1)
+		g_debug("MOVE moved file \"%s\" at %lu bytes/sec [error=%d]\n",
+			download_basename(md->d), (gulong) md->size / elapsed, md->error);
+ 
+	/* FALL THROUGH */
 
 finish:
-	G_FREE_NULL(md->target);
-
-	if (md->error == 0)
-		download_move_done(d, elapsed);
-	else
+	if (md->error == 0) {
+		file_info_mark_stripped(d->file_info);
+		download_move_done(d, md->target, elapsed);
+	} else
 		download_move_error(d);
+
+	HFREE_NULL(md->target);
 }
 
 /**
  * Copy file around, incrementally.
  */
 static bgret_t
-d_step_copy(gpointer h, gpointer u, gint ticks)
+d_step_copy(struct bgtask *h, gpointer u, int ticks)
 {
-	struct moved *md = (struct moved *) u;
+	struct moved *md = u;
 	ssize_t r;
 	size_t amount;
-	guint64 remain;
-	gint used;
+	filesize_t remain;
+	int used = 0;
+	int t;
 
 	g_assert(md->magic == MOVED_MAGIC);
 
-	if (md->rd == -1)			/* Could not open the file */
+	if (NULL == md->rd)			/* Could not open the file */
 		return BGR_DONE;		/* Computation done */
 
 	if (md->size == 0)			/* Empty file */
 		return BGR_DONE;
 
-	remain = md->size - md->copied;
+again:		/* Avoids indenting all this code */
 
-	g_assert(remain > 0);
+	g_assert(md->size > md->copied);
+	remain = md->size - md->copied;
+	remain = MIN(remain, COPY_BUF_SIZE);
 
 	/*
-	 * Each tick we have can buy us 2^COPY_BLOCK_SHIFT bytes.
+	 * Each tick we have can buy us COPY_BLOCK_FRAGMENT bytes.
 	 *
 	 * We read into a COPY_BUF_SIZE bytes buffer, and at most md->size
 	 * bytes total, to stop before the fileinfo trailer.
 	 */
 
-	amount = ticks << COPY_BLOCK_SHIFT;
-	remain = MIN(remain, COPY_BUF_SIZE);
+	amount = MAX(0, ticks);
+	amount = size_saturate_mult(amount, COPY_BLOCK_FRAGMENT);
 	amount = MIN(amount, remain);
 
 	g_assert(amount > 0);
 
-	r = read(md->rd, md->buffer, amount);
+	r = file_object_pread(md->rd, md->buffer, amount, md->copied);
 	if ((ssize_t) -1 == r) {
 		md->error = errno;
 		g_warning("error while reading \"%s\" for moving: %s",
-			download_outname(md->d), g_strerror(errno));
+			file_object_get_pathname(md->rd), g_strerror(errno));
 		return BGR_DONE;
 	} else if (r == 0) {
 		g_warning("EOF while reading \"%s\" for moving!",
-			download_outname(md->d));
+			file_object_get_pathname(md->rd));
 		md->error = -1;
 		return BGR_DONE;
 	}
 
+	g_assert((size_t) r == amount);
+
 	/*
 	 * Any partially read block counts as one block, hence the second term.
 	 */
 
-	used = (r >> COPY_BLOCK_SHIFT) +
-		((r & ((1 << COPY_BLOCK_SHIFT) - 1)) ? 1 : 0);
+	t = (r / COPY_BLOCK_FRAGMENT) + (r % COPY_BLOCK_FRAGMENT ? 1 : 0);
+	used += t;
 
-	if (used != ticks)
-		bg_task_ticks_used(h, used);
+	bg_task_ticks_used(h, used);
 
 	r = write(md->wd, md->buffer, amount);
-	if (r < 0) {
+	if ((ssize_t) -1 == r) {
 		md->error = errno;
 		g_warning("error while writing for moving \"%s\": %s",
-			download_outname(md->d), g_strerror(errno));
+			download_basename(md->d), g_strerror(errno));
 		return BGR_DONE;
 	} else if ((size_t) r < amount) {
 		md->error = -1;
-		g_warning("short write whilst moving \"%s\"", download_outname(md->d));
+		g_warning("short write whilst moving \"%s\"", download_basename(md->d));
 		return BGR_DONE;
 	}
 
@@ -392,14 +434,25 @@ d_step_copy(gpointer h, gpointer u, gint ticks)
 	md->copied += r;
 	download_move_progress(md->d, md->copied);
 
-	return md->copied == md->size ? BGR_DONE : BGR_MORE;
+	if (md->copied == md->size)
+		return BGR_DONE;
+
+	/*
+	 * If we still have unused ticks, repeat.
+	 */
+
+	ticks -= t;
+	if (ticks > 0)
+		goto again;
+
+	return BGR_MORE;
 }
 
 /**
  * Enqueue completed download file for verification.
  */
 void
-move_queue(struct download *d, const gchar *dest, const gchar *ext)
+move_queue(struct download *d, const char *dest, const char *ext)
 {
 	struct work *we;
 
@@ -416,11 +469,11 @@ move_init(void)
 	struct moved *md;
 	bgstep_cb_t step = d_step_copy;
 
-	md = walloc(sizeof(*md));
+	WALLOC(md);
 	md->magic = MOVED_MAGIC;
-	md->rd = -1;
+	md->rd = NULL;
 	md->wd = -1;
-	md->buffer = g_malloc(COPY_BUF_SIZE);
+	md->buffer = halloc(COPY_BUF_SIZE);
 	md->target = NULL;
 
 	move_daemon = bg_daemon_create("file moving",
@@ -433,7 +486,8 @@ move_init(void)
 /**
  * Called at shutdown time.
  */
-void move_close(void)
+void
+move_close(void)
 {
 	bg_task_cancel(move_daemon);
 }
