@@ -1,5 +1,5 @@
 /*
- * $Id: mq.c,v 1.8 2005/05/04 17:37:23 graaff Exp $
+ * $Id: mq.c 19060 2011-05-09 15:17:15Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -28,31 +28,298 @@
  * @file
  *
  * Message queues, common code between TCP and UDP sending stacks.
+ *
+ * @author Raphael Manfredi
+ * @date 2002-2003
  */
 
 #include "common.h"
 
-RCSID("$Id: mq.c,v 1.8 2005/05/04 17:37:23 graaff Exp $");
+RCSID("$Id: mq.c 19060 2011-05-09 15:17:15Z rmanfredi $")
+
+#define MQ_INTERNAL
+#include "mq.h"
 
 #include "nodes.h"
-#include "mq.h"
-#include "pmsg.h"
 #include "gmsg.h"
 #include "gnet_stats.h"
 
-#include "lib/walloc.h"
 #include "lib/cq.h"
+#include "lib/glib-missing.h"	/* For gm_snprintf() */
+#include "lib/halloc.h"
+#include "lib/pmsg.h"
+#include "lib/unsigned.h"		/* For size_saturate_add() */
+#include "lib/walloc.h"
 
 #include "if/gnet_property_priv.h"
 
 #include "lib/override.h"		/* Must be the last header included */
 
+#define MQ_DEBUG_LVL(q)	(*q->debug)
+
 static void qlink_free(mqueue_t *q);
 static void mq_update_flowc(mqueue_t *q);
 static gboolean make_room_header(
-	mqueue_t *q, gchar *header, guint prio, gint needed, gint *offset);
+	mqueue_t *q, char *header, guint prio, int needed, int *offset);
 static void mq_swift_timer(cqueue_t *cq, gpointer obj);
 
+/**
+ * @return queue's fullness status.
+ */
+mq_status_t
+mq_status(const mqueue_t *q)
+{
+	g_assert(q != NULL);
+	g_assert(MQ_MAGIC == q->magic);
+
+	if (0 == q->count)
+		return MQ_S_EMPTY;
+
+	if (q->flags & MQ_SWIFT)
+		return MQ_S_SWIFT;
+
+	if (q->flags & MQ_FLOWC)
+		return MQ_S_FLOWC;
+
+	return (q->count > q->lowat) ? MQ_S_WARNZONE : MQ_S_DELAY;
+}
+
+guint32 mq_debug(const mqueue_t *q)
+{
+	return *q->debug;
+}
+
+gboolean
+mq_is_flow_controlled(const mqueue_t *q)
+{
+	return 0 != (q->flags & MQ_FLOWC);
+}
+
+gboolean
+mq_is_swift_controlled(const mqueue_t *q)
+{
+	return 0 != (q->flags & MQ_SWIFT);
+}
+
+int
+mq_maxsize(const mqueue_t *q)
+{
+	return q->maxsize;
+}
+
+int
+mq_size(const mqueue_t *q)
+{
+	return q->size;
+}
+
+int
+mq_lowat(const mqueue_t *q)
+{
+	return q->lowat;
+}
+
+int
+mq_hiwat(const mqueue_t *q)
+{
+	return q->hiwat;
+}
+
+int
+mq_count(const mqueue_t *q)
+{
+	return q->count;
+}
+
+int
+mq_pending(const mqueue_t *q)
+{
+	return q->size + tx_pending(q->tx_drv);
+}
+
+struct bio_source *
+mq_bio(const mqueue_t *q)
+{
+	return tx_bio_source(q->tx_drv);
+}
+
+struct gnutella_node *
+mq_node(const mqueue_t *q)
+{
+	return q->node;
+}
+
+/**
+ * Would `additional' bytes of traffic cause the queue to enter flow-control?
+ */
+gboolean
+mq_would_flow_control(const mqueue_t *q, size_t additional)
+{
+	return size_saturate_add(q->size, additional) >= UNSIGNED(q->hiwat);
+}
+
+/**
+ * Are we already at or above the low watermark?
+ */
+gboolean
+mq_above_low_watermark(const mqueue_t *q)
+{
+	return q->size >= q->lowat;
+}
+
+/**
+ * Compute readable queue information.
+ */
+const char *
+mq_info(const mqueue_t *q)
+{
+	static char buf[160];
+
+	if (q->magic != MQ_MAGIC) {
+		gm_snprintf(buf, sizeof(buf),
+			"queue 0x%lx INVALID (bad magic)", (gulong) q);
+	} else {
+		gboolean udp = NODE_IS_UDP(q->node);
+
+		gm_snprintf(buf, sizeof(buf),
+			"queue 0x%lx [%s %s node %s%s%s%s%s] (%d item%s, %d byte%s)",
+			(gulong) q, udp ? "UDP" : "TCP",
+			NODE_IS_ULTRA(q->node) ? "ultra" :
+			udp ? "remote" : "leaf", node_addr(q->node),
+			(q->flags & MQ_FLOWC) ? " FLOWC" : "",
+			(q->flags & MQ_DISCARD) ? " DISCARD" : "",
+			(q->flags & MQ_SWIFT) ? " SWIFT" : "",
+			(q->flags & MQ_WARNZONE) ? " WARNZONE" : "",
+			q->count, q->count == 1 ? "" : "s",
+			q->size, q->size == 1 ? "" : "s"
+		);
+	}
+
+	return buf;
+}
+
+#ifdef MQ_DEBUG
+/*
+ * This hashtable tracks the queue owning a given glist linkable.
+ */
+static GHashTable *qown = NULL;
+
+/**
+ * Add linkable into queue
+ */
+static void
+mq_add_linkable(mqueue_t *q, GList *l)
+{
+	mqueue_t *owner;
+
+	g_assert(q->magic == MQ_MAGIC);
+	g_assert(l != NULL);
+	g_assert(l->data != NULL);
+	
+	if (qown == NULL)
+		qown = NOT_LEAKING(g_hash_table_new(pointer_hash_func, NULL));
+
+	owner = g_hash_table_lookup(qown, l);
+	if (owner) {
+		g_carp("BUG: added linkable 0x%lx already owned by %s%s",
+			(gulong) l, owner == q ? "ourselves" : "other", mq_info(owner));
+		if (owner != q)
+			g_warning("BUG: will make linkable 0x%lx belong to %s",
+				(gulong) l, mq_info(q));
+		g_assert_not_reached();
+	}
+
+	g_hash_table_insert(qown, l, q);
+}
+
+/**
+ * Remove linkable from queue
+ */
+static void
+mq_remove_linkable(mqueue_t *q, GList *l)
+{
+	mqueue_t *owner;
+
+	g_assert(q->magic == MQ_MAGIC);
+	g_assert(l != NULL);
+	g_assert(qown != NULL);		/* Must have added something before */
+
+	owner = g_hash_table_lookup(qown, l);
+
+	if (owner == NULL)
+		g_error("BUG: removed linkable 0x%lx from %s belongs to no queue!",
+			(gulong) l, mq_info(q));
+	else if (owner != q)
+		g_error("BUG: removed linkable 0x%lx from %s is from another queue!",
+			(gulong) l, mq_info(q));
+	else
+		g_hash_table_remove(qown, l);
+}
+
+/**
+ * Check queue's sanity.
+ */
+void
+mq_check_track(mqueue_t *q, int offset, const char *where, int line)
+{
+	int qcount;
+	int qlink_alive = 0;
+	int n;
+
+	g_assert(q);
+
+	if (qown == NULL)
+		qown = NOT_LEAKING(g_hash_table_new(pointer_hash_func, NULL));
+
+	if (q->magic != MQ_MAGIC)
+		g_error("BUG: %s at %s:%d", mq_info(q), where, line);
+
+	qcount = g_list_length(q->qhead);
+	if (qcount != q->count)
+		g_error("BUG: "
+			"%s has wrong q->count of %d (counted %d in list) at %s:%d",
+			mq_info(q), q->count, qcount, where, line);
+
+	if (q->qlink == NULL)
+		return;
+
+	for (n = 0; n < q->qlink_count; n++) {
+		GList *item = q->qlink[n];
+		mqueue_t *owner;
+
+		if (item == NULL)
+			continue;
+
+		qlink_alive++;
+		if (item->data == NULL)
+			g_error("BUG: linkable #%d/%d from %s is NULL at %s:%d",
+				n, q->qlink_count, mq_info(q), where, line);
+
+		g_assert(qown);		/* If we have a qlink, we have added items */
+
+		owner = g_hash_table_lookup(qown, item);
+		if (owner != q)
+			g_error("BUG: linkable #%d/%d from %s "
+				"%s at %s:%d",
+				n, q->qlink_count, mq_info(q),
+				owner == NULL ?
+					"does not belong to any queue" :
+					"belongs to foreign queue",
+				where, line);
+	}
+
+	if (qlink_alive != qcount + offset)
+		g_error("BUG: qlink discrepancy for %s "
+		"(counted %d alive linkable, expected %d, queue has %d items) at %s:%d",
+		mq_info(q), qlink_alive, qcount + offset, qcount, where, line);
+}
+#else	/* !MQ_DEBUG */
+
+#define	mq_add_linkable(x,y)
+#define	mq_remove_linkable(x,y)
+
+#endif	/* MQ_DEBUG */
+
 /*
  * Polymorphic operations.
  */
@@ -69,13 +336,15 @@ void
 mq_free(mqueue_t *q)
 {
 	GList *l;
-	gint n;
+	int n;
 
 	tx_free(q->tx_drv);		/* Get rid of lower layers */
 
 	for (n = 0, l = q->qhead; l; l = g_list_next(l)) {
 		n++;
-		pmsg_free((pmsg_t *) l->data);
+		pmsg_free(l->data);
+		l->data = NULL;
+		mq_remove_linkable(q, l);
 	}
 
 	g_assert(n == q->count);
@@ -83,10 +352,11 @@ mq_free(mqueue_t *q)
 	if (q->qlink)
 		qlink_free(q);
 
-	if (q->swift_ev)
-		cq_cancel(callout_queue, q->swift_ev);
+	cq_cancel(&q->swift_ev);
+	gm_list_free_null(&q->qhead);
+	pmsg_slist_free(&q->qwait);
 
-	g_list_free(q->qhead);
+	q->magic = 0;
 	wfree(q, sizeof(*q));
 }
 
@@ -98,17 +368,22 @@ mq_free(mqueue_t *q)
  * queue is updated, but not the flow-control information.
  */
 static GList *
-mq_rmlink_prev(mqueue_t *q, GList *l, gint size)
+mq_rmlink_prev(mqueue_t *q, GList *l, int size)
 {
 	GList *prev = g_list_previous(l);
 
+	mq_remove_linkable(q, l);
 	q->qhead = g_list_remove_link(q->qhead, l);
 	if (q->qtail == l)
 		q->qtail = prev;
+
+	g_assert(q->size >= size);
 	q->size -= size;
+	g_assert(q->count > 0);
 	q->count--;
 
-	pmsg_free((pmsg_t *) l->data);
+	pmsg_free(l->data);
+	l->data = NULL;
 	g_list_free_1(l);
 
 	return prev;
@@ -120,14 +395,14 @@ mq_rmlink_prev(mqueue_t *q, GList *l, gint size)
 static void
 mq_swift_checkpoint(mqueue_t *q, gboolean initial)
 {
-	gint elapsed = q->swift_elapsed;	/* Elapsed since we were scheduled */
-	gint target_to_lowmark;
-	gint flushed_till_next_timer;
-	gint added_till_next_timer;
-	gfloat period_ratio;
-	gint added;
-	gint needed;
-	gint extra;
+	int elapsed = q->swift_elapsed;	/* Elapsed since we were scheduled */
+	int target_to_lowmark;
+	int flushed_till_next_timer;
+	int added_till_next_timer;
+	float period_ratio;
+	int added;
+	int needed;
+	int extra;
 
 	g_assert(q->flags & MQ_FLOWC);
 	g_assert(q->size > q->lowat);	/* Or we would have left FC */
@@ -139,6 +414,7 @@ mq_swift_checkpoint(mqueue_t *q, gboolean initial)
 	 */
 
 	q->swift_elapsed = node_flowc_swift_period(q->node) * 1000;
+	q->swift_elapsed = MAX(q->swift_elapsed, 1);
 
 	/*
 	 * Compute target to reach the low watermark, and then the amount we
@@ -147,12 +423,12 @@ mq_swift_checkpoint(mqueue_t *q, gboolean initial)
 	 * the queue.
 	 */
 
-	period_ratio = (gfloat) q->swift_elapsed / (gfloat) elapsed;
+	period_ratio = (float) q->swift_elapsed / (float) elapsed;
 	target_to_lowmark = q->size - q->lowat;
 	added = q->size - q->last_size + q->flowc_written;
 
-	flushed_till_next_timer = (gint) (q->flowc_written * period_ratio);
-	added_till_next_timer = added <= 0 ? 0 : (gint) (added * period_ratio);
+	flushed_till_next_timer = (int) (q->flowc_written * period_ratio);
+	added_till_next_timer = added <= 0 ? 0 : (int) (added * period_ratio);
 
 	/*
 	 * Now compute the amount of bytes we need to forcefully drop to be
@@ -190,12 +466,12 @@ mq_swift_checkpoint(mqueue_t *q, gboolean initial)
 		 * Leave our queries in for now (they have hops=0).
 		 */
 
-		q->header.function = GTA_MSG_SEARCH;
-		q->header.hops = 1;
-		q->header.ttl = max_ttl;
+		gnutella_header_set_function(&q->header, GTA_MSG_SEARCH);
+		gnutella_header_set_hops(&q->header, 1);
+		gnutella_header_set_ttl(&q->header, GNET_PROPERTY(max_ttl));
 
 		if (needed > 0)
-			make_room_header(q, (gchar*) &q->header, PMSG_P_DATA, needed, NULL);
+			make_room_header(q, (char*) &q->header, PMSG_P_DATA, needed, NULL);
 
 		/*
 		 * Whether or not we were able to make enough room at this point
@@ -206,7 +482,7 @@ mq_swift_checkpoint(mqueue_t *q, gboolean initial)
 		 */
 
 	} else {
-		gint ttl;
+		int ttl;
 
 		/*
 		 * We're going to drop query hits...
@@ -220,21 +496,26 @@ mq_swift_checkpoint(mqueue_t *q, gboolean initial)
 		 * than our comparison point.
 		 */
 
-		q->header.function = GTA_MSG_SEARCH_RESULTS;
+		gnutella_header_set_function(&q->header, GTA_MSG_SEARCH_RESULTS);
 
 		/*
 		 * Loop until we reach hops=hard_ttl_limit or we have finished
 		 * removing enough data from the queue.
 		 */
 
-		for (ttl = hard_ttl_limit; needed > 0 && ttl >= 0; ttl--) {
-			gint old_size = q->size;
+		for (ttl = GNET_PROPERTY(hard_ttl_limit); ttl >= 0; ttl--) {
+			int old_size;
 
-			q->header.hops = hard_ttl_limit - ttl;
-			q->header.ttl = ttl;
+			if (needed <= 0)
+				break;
+
+			old_size = q->size;
+			gnutella_header_set_hops(&q->header,
+				GNET_PROPERTY(hard_ttl_limit) - ttl);
+			gnutella_header_set_ttl(&q->header, ttl);
 
 			if (
-				make_room_header(q, (gchar*) &q->header, PMSG_P_DATA,
+				make_room_header(q, (char*) &q->header, PMSG_P_DATA,
 					needed, NULL)
 			)
 				break;
@@ -255,8 +536,7 @@ mq_swift_checkpoint(mqueue_t *q, gboolean initial)
 	if (q->flags & MQ_SWIFT) {
 		q->flowc_written = 0;
 		q->last_size = q->size;
-		q->swift_ev = cq_insert(callout_queue,
-			q->swift_elapsed, mq_swift_timer, q);
+		q->swift_ev = cq_main_insert(q->swift_elapsed, mq_swift_timer, q);
 	}
 }
 
@@ -266,7 +546,7 @@ mq_swift_checkpoint(mqueue_t *q, gboolean initial)
 static void
 mq_swift_timer(cqueue_t *unused_cq, gpointer obj)
 {
-	mqueue_t *q = (mqueue_t *) obj;
+	mqueue_t *q = obj;
 
 	(void) unused_cq;
 	g_assert((q->flags & (MQ_FLOWC|MQ_SWIFT)) == (MQ_FLOWC|MQ_SWIFT));
@@ -280,7 +560,7 @@ mq_swift_timer(cqueue_t *unused_cq, gpointer obj)
 static void
 mq_enter_swift(cqueue_t *unused_cq, gpointer obj)
 {
-	mqueue_t *q = (mqueue_t *) obj;
+	mqueue_t *q = obj;
 
 	(void) unused_cq;
 	g_assert((q->flags & (MQ_FLOWC|MQ_SWIFT)) == MQ_FLOWC);
@@ -312,14 +592,14 @@ mq_enter_flowc(mqueue_t *q)
 	q->flowc_written = 0;
 	q->last_size = q->size;
 	q->swift_elapsed = node_flowc_swift_grace(q->node) * 1000;
-	q->swift_ev =
-		cq_insert(callout_queue, q->swift_elapsed, mq_enter_swift, q);
+	q->swift_elapsed = MAX(q->swift_elapsed, 1);
+	q->swift_ev = cq_main_insert(q->swift_elapsed, mq_enter_swift, q);
 
 	node_tx_enter_flowc(q->node);	/* Signal flow control */
 
-	if (dbg > 4)
-		printf("entering FLOWC for node %s (%d bytes queued)\n",
-			node_ip(q->node), q->size);
+	if (MQ_DEBUG_LVL(q) > 4)
+		g_debug("MQ entering FLOWC for node %s (%d bytes queued)",
+			node_addr(q->node), q->size);
 }
 
 /**
@@ -330,20 +610,16 @@ mq_leave_flowc(mqueue_t *q)
 {
 	g_assert(q->flags & MQ_FLOWC);
 
-	if (dbg > 4)
-		printf("leaving %s for node %s (%d bytes queued)\n",
+	if (MQ_DEBUG_LVL(q) > 4)
+		g_debug("MQ leaving %s for node %s (%d bytes queued)",
 			(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
-			node_ip(q->node), q->size);
+			node_addr(q->node), q->size);
 
 	q->flags &= ~(MQ_FLOWC|MQ_SWIFT);	/* Under low watermark, clear */
 	if (q->qlink)
 		qlink_free(q);
 
-	if (q->swift_ev) {
-		cq_cancel(callout_queue, q->swift_ev);
-		q->swift_ev = NULL;
-	}
-
+	cq_cancel(&q->swift_ev);
 	node_tx_leave_flowc(q->node);	/* Signal end flow control */
 }
 
@@ -381,15 +657,23 @@ mq_update_flowc(mqueue_t *q)
 void
 mq_clear(mqueue_t *q)
 {
-	GList *l;
-
 	g_assert(q);
 
 	if (q->count == 0)
 		return;					/* Queue is empty */
 
-	while ((l = q->qhead)) {
-		pmsg_t *mb = (pmsg_t *) l->data;
+	/*
+	 * If there are extended message blocks in the queue, freeing them
+	 * could cause the callback to attempt to queue something again.  Hence
+	 * we must mark we're clearing the queue to avoid deadly recursions that
+	 * would corrupt the qlink array.
+	 */
+
+	q->flags |= MQ_CLEAR;
+
+	while (q->qhead) {
+		GList *l = q->qhead;
+		pmsg_t *mb = l->data;
 
 		/*
 		 * Break if we started to write this message, i.e. if we read
@@ -407,6 +691,8 @@ mq_clear(mqueue_t *q)
 	if (q->qlink)
 		qlink_free(q);
 
+	q->flags &= ~MQ_CLEAR;
+
 	mq_update_flowc(q);
 
 	/*
@@ -424,28 +710,50 @@ mq_clear(mqueue_t *q)
  * Forbid further writes to the queue.
  */
 void
-mq_shutdown(mqueue_t *q)
+mq_discard(mqueue_t *q)
 {
 	g_assert(q);
 
 	q->flags |= MQ_DISCARD;
 }
 
+/**
+ * Disable all further writes from the queue.
+ */
+void
+mq_shutdown(mqueue_t *q)
+{
+	g_assert(q);
+
+	tx_shutdown(q->tx_drv);		/* No further output will be made */
+}
+
+/**
+ * Request explicit flush of all buffered data in the TX stack.
+ */
+void
+mq_flush(mqueue_t *q)
+{
+	g_assert(q);
+
+	tx_flush(q->tx_drv);
+}
+
 /**
  * Compare two pointers to links based on their relative priorities, then
  * based on their held Gnutella messages.
  * -- qsort() callback
  */
-static gint
-qlink_cmp(const void *lp1, const void *lp2)
+static int
+qlink_cmp(const void *a, const void *b)
 {
-	const pmsg_t *m1 = (const pmsg_t *) (*(const GList * const *) lp1)->data;
-	const pmsg_t *m2 = (const pmsg_t *) (*(const GList * const *) lp2)->data;
+	const GList * const *l1 = a, * const *l2 = b;
+	const pmsg_t *m1 = (*l1)->data, *m2 = (*l2)->data;
 
 	if (pmsg_prio(m1) == pmsg_prio(m2))
-		return gmsg_cmp(pmsg_start(m1), pmsg_start(m2));
-
-	return pmsg_prio(m1) < pmsg_prio(m2) ? -1 : +1;
+		return gmsg_cmp(pmsg_start(m1), pmsg_start(m2), TRUE);
+	else
+		return pmsg_prio(m1) < pmsg_prio(m2) ? -1 : +1;
 }
 
 /**
@@ -454,13 +762,12 @@ qlink_cmp(const void *lp1, const void *lp2)
 static void
 qlink_create(mqueue_t *q)
 {
-	GList **qlink;
 	GList *l;
-	gint n;
+	int n;
 
 	g_assert(q->qlink == NULL);
 
-	qlink = q->qlink = (GList **) g_malloc(q->count * sizeof(GList *));
+	q->qlink = halloc(q->count * sizeof q->qlink[0]);
 
 	/*
 	 * Prepare sorting of queued messages.
@@ -469,12 +776,14 @@ qlink_create(mqueue_t *q)
 	 * gmsg_cmp() routine to compare the Gnutella messages.
 	 */
 
-	for (l = q->qhead, n = 0; l && n < q->count; l = g_list_next(l), n++)
-		qlink[n] = l;
+	for (l = q->qhead, n = 0; l && n < q->count; l = g_list_next(l), n++) {
+		g_assert(l->data != NULL);
+		q->qlink[n] = l;
+	}
 
 	if (l || n != q->count)
-		g_warning("BUG: queue count of %d for 0x%lx is wrong (found %d)",
-			q->count, (gulong) q, n);
+		g_error("BUG: queue count of %d for 0x%lx is wrong (has %d)",
+			q->count, (gulong) q, g_list_length(q->qhead));
 
 	/*
 	 * We use `n' and not `q->count' in case the warning above is emitted,
@@ -482,7 +791,9 @@ qlink_create(mqueue_t *q)
 	 */
 
 	q->qlink_count = n;
-	qsort(qlink, n, sizeof(GList *), qlink_cmp);
+	qsort(q->qlink, n, sizeof q->qlink[0], qlink_cmp);
+
+	mq_check(q, 0);
 }
 
 /**
@@ -493,7 +804,7 @@ qlink_free(mqueue_t *q)
 {
 	g_assert(q->qlink);
 
-	G_FREE_NULL(q->qlink);
+	HFREE_NULL(q->qlink);
 	q->qlink_count = 0;
 }
 
@@ -502,17 +813,20 @@ qlink_free(mqueue_t *q)
  * before the position indicated by `hint'.
  */
 static void
-qlink_insert_before(mqueue_t *q, gint hint, GList *l)
+qlink_insert_before(mqueue_t *q, int hint, GList *l)
 {
 	g_assert(hint >= 0 && hint < q->qlink_count);
 	g_assert(qlink_cmp(&q->qlink[hint], &l) >= 0);	/* `hint' >= `l' */
+	g_assert(l->data != NULL);
+
+	mq_check(q, -1);
 
 	/*
 	 * Lookup before the message for a NULL entry that we could fill.
 	 */
 
-	if (hint > 0 && q->qlink[hint-1] == NULL) {
-		q->qlink[hint-1] = l;
+	if (hint > 0 && q->qlink[hint - 1] == NULL) {
+		q->qlink[hint - 1] = l;
 		return;
 	}
 
@@ -520,11 +834,17 @@ qlink_insert_before(mqueue_t *q, gint hint, GList *l)
 	 * Extend the array then, and insert right at `hint' in the new array.
 	 */
 
-	q->qlink = g_realloc(q->qlink, ++q->qlink_count * sizeof(GList *));
+	q->qlink_count++;
+	q->qlink = hrealloc(q->qlink, q->qlink_count * sizeof q->qlink[0]);
 
-	memmove(&q->qlink[hint+1], &q->qlink[hint],
-		(q->qlink_count - hint - 1) * sizeof(GList *));	/* Shift right */
+	{
+		int i;
 
+		/* Shift right */
+		for (i = q->qlink_count - 1; i > hint; i--) {
+			q->qlink[i] = q->qlink[i - 1];
+		} 
+	}
 	q->qlink[hint] = l;
 }
 
@@ -534,17 +854,22 @@ qlink_insert_before(mqueue_t *q, gint hint, GList *l)
 static void
 qlink_insert(mqueue_t *q, GList *l)
 {
-	gint low = 0;
-	gint high = q->qlink_count - 1;
+	int low = 0;
+	int high = q->qlink_count - 1;
 	GList **qlink = q->qlink;
 
+	g_assert(l->data != NULL);
+
+	mq_check(q, -1);
+
 	/*
 	 * If `qlink' is empty, create a slot for the new entry.
 	 */
 
 	if (high < 0) {
 		g_assert(q->count == 1);		/* `l' is already part of the queue */
-		q->qlink = g_realloc(q->qlink, ++q->qlink_count * sizeof(GList *));
+		q->qlink_count++;
+		q->qlink = hrealloc(q->qlink, q->qlink_count * sizeof q->qlink[0]);
 		q->qlink[0] = l;
 		return;
 	}
@@ -563,7 +888,8 @@ qlink_insert(mqueue_t *q, GList *l)
 	 */
 
 	if (qlink[high] != NULL && qlink_cmp(&l, &qlink[high]) >= 0) {
-		q->qlink = g_realloc(q->qlink, ++q->qlink_count * sizeof(GList *));
+		q->qlink_count++;
+		q->qlink = hrealloc(q->qlink, q->qlink_count * sizeof q->qlink[0]);
 		q->qlink[q->qlink_count - 1] = l;
 		return;
 	}
@@ -575,17 +901,17 @@ qlink_insert(mqueue_t *q, GList *l)
 	 */
 
 	while (low <= high) {
-		gint mid = low + (high - low) / 2;
-		gint c;
+		int mid = low + (high - low) / 2;
+		int c;
 
 		/*
 		 * If we end up in a NULL spot, inspect the [low, high] range.
 		 */
 
 		if (qlink[mid] == NULL) {
-			gint n;
-			gint lowest_non_null = -1;
-			gint highest_non_null = -1;
+			int n;
+			int lowest_non_null = -1;
+			int highest_non_null = -1;
 
 			/*
 			 * Go back towards `low' to find a non-NULL entry.
@@ -681,32 +1007,40 @@ qlink_insert(mqueue_t *q, GList *l)
 }
 
 /**
- * Remove the entry in the `qlink' linkable array.
+ * Remove the entry in the `qlink' linkable array, allowing compaction
+ * when there are too many holes.
+ *
+ * @param q			the message queue
+ * @param l			the linkable to remove from the qlink indexer
  */
 static void
 qlink_remove(mqueue_t *q, GList *l)
 {
 	GList **qlink = q->qlink;
-	gint n = q->qlink_count;
+	int n = q->qlink_count;
 
 	g_assert(qlink);
 	g_assert(n > 0);
+	g_assert(l->data != NULL);
+
+	mq_check(q, 0);
 
 	/*
 	 * If more entries in `qlink' than 3 times the amount of queued messages,
 	 * we have too many NULL in the array.
 	 */
 
-	if  (n > q->count * 3) {
+	if (n > q->count * 3) {
 		GList **dest = qlink;
-		gint copied = 0;
+		int copied = 0;
 		gboolean found = FALSE;
 
 		while (n-- > 0) {
-			GList *entry = *qlink++;;
-			if (l == entry || entry == NULL) {
-				if (entry == l)
-					found = TRUE;
+			GList *entry = *qlink++;
+			if (entry == NULL)
+				continue;
+			else if (l == entry) {
+				found = TRUE;
 				continue;
 			}
 			*dest++ = entry;
@@ -715,57 +1049,60 @@ qlink_remove(mqueue_t *q, GList *l)
 
 		q->qlink_count = copied;
 
-		g_assert(found);
-
+		if (found)
+			return;
 	} else {
 		while (n-- > 0) {
-			if (l == *qlink++) {
-				*(--qlink) = NULL;
+			if (l == *qlink) {
+				*qlink = NULL;
 				return;
 			}
+			qlink++;
 		}
 
-		g_assert(0);		/* Must have been found */
+		/* Should have been found -- FALL THROUGH */
 	}
+
+	g_error("BUG: linkable 0x%lx for %s not found "
+		"(qlink has %d slots, queue has %d counted items, really %d) at %s:%d",
+		(gulong) l, mq_info(q),
+		q->qlink_count, q->count, g_list_length(q->qhead),
+		_WHERE_, __LINE__);
 }
 
 /**
- * Remove from the queue enough messages that are less prioritary than
- * the current one, so as to make sure we can enqueue it.
+ * Attempt to make room in the queue to be able to enqueue the new message
+ * whose header is specified.
  *
  * If `offset' is not null, it may be set with the offset within qlink where
- * the message immediately more prioritary than `mb' can be found.  It is
- * up to the caller to initialize it with -1 and check whether it has been
- * set.
+ * the message immediately more prioritary than the new one can be found.
+ * It is up to the caller to initialize it with -1 and check whether it has
+ * been set.
  *
- * Returns TRUE if we were able to make enough room.
+ * @param q			the queue
+ * @param header	pointer to the header of the new message
+ * @param msglen	if non-zero, header points to a full PDU of msglen bytes
+ * @param prio		the priority of the new message we want to enqueue
+ * @param needed	the amount of room we want to make in the queue
+ * @param offset	see above description
+ *
+ * @returns TRUE if we were able to make enough room.
  */
 static gboolean
-make_room(mqueue_t *q, pmsg_t *mb, gint needed, gint *offset)
+make_room_internal(mqueue_t *q,
+	char *header, size_t msglen, guint prio, int needed, int *offset)
 {
-	gchar *header = pmsg_start(mb);
-	guint prio = pmsg_prio(mb);
-
-	return make_room_header(q, header, prio, needed, offset);
-}
-
-/**
- * Same as make_room(), but we are not given a "pmsg_t" as a comparison
- * point but a Gnutella header and a message priority explicitly.
- */
-static gboolean
-make_room_header(
-	mqueue_t *q, gchar *header, guint prio, gint needed, gint *offset)
-{
-	gint n;
-	gint dropped = 0;				/* Amount of messages dropped */
+	int n;
+	int dropped = 0;				/* Amount of messages dropped */
+	gboolean qlink_corrupted = FALSE;	/* BUG catcher */
 
 	g_assert(needed > 0);
+	mq_check(q, 0);
 
-	if (dbg > 5)
-		printf("%s try to make room for %d bytes in queue 0x%lx (node %s)\n",
+	if (MQ_DEBUG_LVL(q) > 5)
+		g_debug("MQ %s try to make room for %d bytes in queue 0x%lx (node %s)",
 			(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
-			needed, (gulong) q, node_ip(q->node));
+			needed, (gulong) q, node_addr(q->node));
 
 	if (q->qhead == NULL)			/* Queue is empty */
 		return FALSE;
@@ -795,11 +1132,12 @@ make_room_header(
 	 * network, so we can NULLify the corresponding slot in the qlink array.
 	 */
 
+restart:
 	for (n = 0; needed >= 0 && n < q->qlink_count; n++) {
 		GList *item = q->qlink[n];
 		pmsg_t *cmb;
-		gchar *cmb_start;
-		gint cmb_size;
+		char *cmb_start;
+		int cmb_size;
 
 		/*
 		 * If slot was NULLified, skip it.
@@ -808,7 +1146,33 @@ make_room_header(
 		if (item == NULL)
 			continue;
 
-		cmb = (pmsg_t *) item->data;
+		/*
+		 * BUG catcher -- I've seen situations where item->data is NULL
+		 * at this point, which means something is deeply corrupted...
+		 *		--RAM, 2006-07-14
+		 */
+
+		if (item->data == NULL) {
+			g_error("BUG: NULL data for qlink item #%d/%d in %s at %s:%d",
+				n, q->qlink_count, mq_info(q), _WHERE_, __LINE__);
+
+			if (qlink_corrupted) {
+				g_error(
+					"BUG: trying to ignore still invalid qlink entry at %s:%d",
+						_WHERE_, __LINE__);
+				continue;
+			}
+
+			qlink_corrupted = TRUE;		/* Try to mend it */
+			qlink_free(q);
+			qlink_create(q);
+
+			g_error("BUG: recreated qlink and restarting at %s:%d",
+				_WHERE_, __LINE__);
+			goto restart;
+		}
+
+		cmb = item->data;
 		cmb_start = pmsg_start(cmb);
 
 		/*
@@ -828,7 +1192,7 @@ make_room_header(
 		 * (it's necessarily >= 0 if we're in the loop)
 		 */
 
-		if (gmsg_cmp(cmb_start, header) >= 0) {
+		if (gmsg_cmp(cmb_start, header, msglen != 0) >= 0) {
 			if (offset != NULL)
 				*offset = n;
 			break;
@@ -850,28 +1214,35 @@ make_room_header(
 		 * Drop message.
 		 */
 
-		if (dbg > 4)
-			gmsg_log_dropped(pmsg_start(cmb),
-				"to %s node %s, in favor of %s",
+		if (MQ_DEBUG_LVL(q) > 4) {
+			gmsg_log_dropped_pmsg(cmb, "to %s %s node %s, in favor of %s",
 				(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
-				node_ip(q->node), gmsg_infostr(header));
+				NODE_IS_UDP(q->node) ? "UDP" : "TCP",
+				node_addr(q->node), msglen ?
+					gmsg_infostr_full(header, msglen) : gmsg_infostr(header));
+		}
 
-		gnet_stats_count_flowc(pmsg_start(cmb));
+		gnet_stats_count_flowc(pmsg_start(cmb), FALSE);
 		cmb_size = pmsg_size(cmb);
 
+		g_assert(q->qlink[n] == item);
+
 		needed -= cmb_size;
-		(void) mq_rmlink_prev(q, q->qlink[n], cmb_size);
 		q->qlink[n] = NULL;
+		(void) mq_rmlink_prev(q, item, cmb_size);
+
 		dropped++;
+
+		mq_check(q, 0);
 	}
 
 	if (dropped)
 		node_add_txdrop(q->node, dropped);	/* Dropped during TX */
 
-	if (dbg > 5)
-		printf("%s end purge: %d bytes (count=%d) for node %s, need=%d\n",
+	if (MQ_DEBUG_LVL(q) > 5)
+		g_debug("MQ %s end purge: %d bytes (count=%d) for node %s, need=%d",
 			(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
-			q->size, q->count, node_ip(q->node), needed);
+			q->size, q->count, node_addr(q->node), needed);
 
 	/*
 	 * In case we emptied the whole queue, disable servicing.
@@ -900,18 +1271,52 @@ make_room_header(
 	return needed <= 0;		/* Can be 0 if we broke out loop above */
 }
 
+/**
+ * Remove from the queue enough messages that are less prioritary than
+ * the current one, so as to make sure we can enqueue it.
+ *
+ * If `offset' is not null, it may be set with the offset within qlink where
+ * the message immediately more prioritary than `mb' can be found.  It is
+ * up to the caller to initialize it with -1 and check whether it has been
+ * set.
+ *
+ * @returns TRUE if we were able to make enough room.
+ */
+static gboolean
+make_room(mqueue_t *q, pmsg_t *mb, int needed, int *offset)
+{
+	char *header = pmsg_start(mb);
+	guint prio = pmsg_prio(mb);
+	size_t msglen = pmsg_written_size(mb);
+
+	return make_room_internal(q, header, msglen, prio, needed, offset);
+}
+
+/**
+ * Same as make_room(), but we are not given a "pmsg_t" as a comparison
+ * point but a Gnutella header and a message priority explicitly.
+ */
+static gboolean
+make_room_header(
+	mqueue_t *q, char *header, guint prio, int needed, int *offset)
+{
+	return make_room_internal(q, header, 0, prio, needed, offset);
+}
+
 /**
  * Put message in this queue.
  */
 static void
-mq_puthere(mqueue_t *q, pmsg_t *mb, gint msize)
+mq_puthere(mqueue_t *q, pmsg_t *mb, int msize)
 {
-	gint needed;
-	gint qlink_offset = -1;
+	int needed;
+	int qlink_offset = -1;
 	GList *new = NULL;
 	gboolean make_room_called = FALSE;
 	gboolean has_normal_prio = (pmsg_prio(mb) == PMSG_P_DATA);
 
+	mq_check(q, 0);
+
 	/*
 	 * If we're flow-controlled and the message can be dropped, acccept it
 	 * if we can manage to make room for at least the size of the message,
@@ -926,12 +1331,13 @@ mq_puthere(mqueue_t *q, pmsg_t *mb, gint msize)
 		!make_room(q, mb, msize, &qlink_offset)
 	) {
 		g_assert(pmsg_is_unread(mb));			/* Not partially written */
-		if (dbg > 4)
-			gmsg_log_dropped(pmsg_start(mb),
-				"to FLOWC node %s, %d bytes queued",
-				node_ip(q->node), q->size);
+		if (MQ_DEBUG_LVL(q) > 4)
+			gmsg_log_dropped_pmsg(mb, "to %s %s node %s, %d bytes queued",
+				(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
+				NODE_IS_UDP(q->node) ? "UDP" : "TCP",
+				node_addr(q->node), q->size);
 
-		gnet_stats_count_flowc(pmsg_start(mb));
+		gnet_stats_count_flowc(pmsg_start(mb), FALSE);
 		pmsg_free(mb);
 		node_inc_txdrop(q->node);		/* Dropped during TX */
 		return;
@@ -961,26 +1367,37 @@ mq_puthere(mqueue_t *q, pmsg_t *mb, gint msize)
 
 		g_assert(pmsg_is_unread(mb));			/* Not partially written */
 
-		gnet_stats_count_flowc(pmsg_start(mb));
+		gnet_stats_count_flowc(pmsg_start(mb), FALSE);
 
 		if (has_normal_prio) {
-			if (dbg > 4)
-				gmsg_log_dropped(pmsg_start(mb),
-					"to FLOWC node %s, %d bytes queued [FULL]",
-					node_ip(q->node), q->size);
+			if (MQ_DEBUG_LVL(q) > 4)
+				gmsg_log_dropped_pmsg(mb,
+					"to %s %s node %s, %d bytes queued [FULL]",
+					(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
+					NODE_IS_UDP(q->node) ? "UDP" : "TCP",
+					node_addr(q->node), q->size);
 
 			node_inc_txdrop(q->node);		/* Dropped during TX */
 		} else {
-			if (dbg > 4)
-				gmsg_log_dropped(pmsg_start(mb),
-					"to FLOWC node %s, %d bytes queued [KILLING]",
-					node_ip(q->node), q->size);
+			if (MQ_DEBUG_LVL(q) > 4)
+				gmsg_log_dropped_pmsg(mb,
+					"to %s %s node %s, %d bytes queued [KILLING]",
+					(q->flags & MQ_SWIFT) ? "SWIFT" : "FLOWC",
+					NODE_IS_UDP(q->node) ? "UDP" : "TCP",
+					node_addr(q->node), q->size);
 
-			/* XXX: Is the check for UDP the correct fix or just a
-			 *		workaround? node_bye_v() asserts that the node isn't
-			 *		the UDP node.
-			 *		-- cbiere, 2004-12-20
+			/*
+			 * Is the check for UDP the correct fix or just a
+			 * workaround? node_bye_v() asserts that the node isn't
+			 * the UDP node.	-- cbiere, 2004-12-20
+			 *
+			 * Yes, it's the correct fix: node_bye() should only be called
+			 * when a TCP connection is made.  We could teach about UDP
+			 * in node_bye(), but I don't like routines with an unclear
+			 * contract (which attempt to "do the right thing" depending on
+			 * the parameter values). --RAM, 2006-12-29
 			 */
+
 			if (!NODE_IS_UDP(q->node)) {
 				node_bye(q->node, 502, "Send queue reached %d bytes",
 					q->maxsize);
@@ -1020,7 +1437,7 @@ mq_puthere(mqueue_t *q, pmsg_t *mb, gint msize)
 		 */
 
 		for (l = q->qtail; l; l = l->prev) {
-			pmsg_t *m = (pmsg_t *) l->data;
+			pmsg_t *m = l->data;
 
 			if (
 				pmsg_is_unread(m) &&			/* Not partially written */
@@ -1064,6 +1481,8 @@ mq_puthere(mqueue_t *q, pmsg_t *mb, gint msize)
 		}
 	}
 
+	mq_add_linkable(q, new);
+
 	q->size += msize;
 	q->count++;
 
@@ -1108,10 +1527,10 @@ mq_putq(mqueue_t *q, pmsg_t *mb)
 }
 
 static const struct mq_cops mq_cops = {
-	mq_puthere,				/* puthere */
-	qlink_remove,			/* qlink_remove */
-	mq_rmlink_prev,			/* rmlink_prev */
-	mq_update_flowc,		/* update_flowc */
+	mq_puthere,				/**< puthere */
+	qlink_remove,			/**< qlink_remove */
+	mq_rmlink_prev,			/**< rmlink_prev */
+	mq_update_flowc,		/**< update_flowc */
 };
 
 /**
