@@ -1,5 +1,5 @@
 /*
- * $Id: mq.h 13015 2007-03-08 19:07:02Z cbiere $
+ * $Id: mq.h 19060 2011-05-09 15:17:15Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -39,10 +39,12 @@
 #include "common.h"
 
 #include "gnutella.h"
-#include "pmsg.h"
 #include "tx.h"
 
+#include "if/core/mq.h"
+
 #include "lib/cq.h"
+#include "lib/pmsg.h"
 #include "lib/slist.h"
 
 typedef struct mqueue mqueue_t;
@@ -56,12 +58,19 @@ struct mq_ops {
 };
 
 struct mq_cops {
-	void (*puthere)(mqueue_t *q, pmsg_t *mb, gint msize);
+	void (*puthere)(mqueue_t *q, pmsg_t *mb, int msize);
 	void (*qlink_remove)(mqueue_t *q, GList *l);
-	GList *(*rmlink_prev)(mqueue_t *q, GList *l, gint size);
+	GList *(*rmlink_prev)(mqueue_t *q, GList *l, int size);
 	void (*update_flowc)(mqueue_t *q);
 };
 
+#ifdef MQ_INTERNAL
+
+/*
+ * The declarations in this section are only visible to the various files
+ * implementing the message queue.
+ */
+
 enum mq_magic {
 	MQ_MAGIC = 0x33990ee
 };
@@ -94,18 +103,19 @@ struct mqueue {
 	GList *qhead, *qtail, **qlink;
 	slist_t *qwait;			/**< Waiting queue during putq recursions */
 	cevent_t *swift_ev;		/**< Callout queue event in "swift" mode */
-	gint swift_elapsed;		/**< Scheduled elapsed time, in ms */
-	gint qlink_count;		/**< Amount of entries in `qlink' */
-	gint maxsize;			/**< Maximum size of this queue (total queued) */
-	gint count;				/**< Amount of messages queued */
-	gint hiwat;				/**< High watermark */
-	gint lowat;				/**< Low watermark */
-	gint size;				/**< Current amount of bytes queued */
-	gint flags;				/**< Status flags */
-	gint last_written;		/**< Amount last written by service routine */
-	gint flowc_written;		/**< Amount written during flow control */
-	gint last_size;			/**< Queue size at last "swift" event callback */
-    gint putq_entered;		/**< For recursion checks in mq_putq() */
+	const guint32 *debug;	/**< Debug config variable for this queue */
+	int swift_elapsed;		/**< Scheduled elapsed time, in ms */
+	int qlink_count;		/**< Amount of entries in `qlink' */
+	int maxsize;			/**< Maximum size of this queue (total queued) */
+	int count;				/**< Amount of messages queued */
+	int hiwat;				/**< High watermark */
+	int lowat;				/**< Low watermark */
+	int size;				/**< Current amount of bytes queued */
+	int flags;				/**< Status flags */
+	int last_written;		/**< Amount last written by service routine */
+	int flowc_written;		/**< Amount written during flow control */
+	int last_size;			/**< Queue size at last "swift" event callback */
+    int putq_entered;		/**< For recursion checks in mq_putq() */
 };
 
 /*
@@ -113,22 +123,45 @@ struct mqueue {
  */
 
 enum {
-	MQ_FLOWC	= (1 << 0),	/**< In flow control */
-	MQ_DISCARD	= (1 << 1),	/**< No writing, discard message */
+	MQ_CLEAR	= (1 << 4),	/**< Running mq_clear() */
+	MQ_WARNZONE	= (1 << 3),	/**< Between hiwat and lowat */
 	MQ_SWIFT	= (1 << 2),	/**< Swift mode, dropping more traffic */
-	MQ_WARNZONE	= (1 << 3)	/**< Between hiwat and lowat */
+	MQ_DISCARD	= (1 << 1),	/**< No writing, discard message */
+	MQ_FLOWC	= (1 << 0)	/**< In flow control */
 };
 
-gboolean mq_is_flow_controlled(const struct mqueue *q);
-gboolean mq_is_swift_controlled(const struct mqueue *q);
-gint mq_maxsize(const struct mqueue *q);
-gint mq_size(const struct mqueue *q);
-gint mq_lowat(const struct mqueue *q);
-gint mq_hiwat(const struct mqueue *q);
-gint mq_count(const struct mqueue *q);
-gint mq_pending(const struct mqueue *q);
-struct bio_source *mq_bio(const struct mqueue *q);
-struct gnutella_node *mq_node(const struct mqueue *q);
+/*
+ * Message queue assertions.
+ */
+
+#if 0
+#define MQ_DEBUG			/**< Activates mq_check() assertions */
+#endif
+
+#ifdef MQ_DEBUG
+void mq_check_track(mqueue_t *q, int offset, const char *where, int line);
+
+#define mq_check(x,y)	mq_check_track((x), (y), _WHERE_, __LINE__)
+#else
+#define mq_check(x,y)
+#endif
+
+#endif /* MQ_INTERNAL */
+
+mq_status_t mq_status(const mqueue_t *q);
+gboolean mq_is_flow_controlled(const mqueue_t *q);
+gboolean mq_is_swift_controlled(const mqueue_t *q);
+gboolean mq_would_flow_control(const mqueue_t *q, size_t additional);
+gboolean mq_above_low_watermark(const mqueue_t *q);
+guint32 mq_debug(const mqueue_t *q);
+int mq_maxsize(const mqueue_t *q);
+int mq_size(const mqueue_t *q);
+int mq_lowat(const mqueue_t *q);
+int mq_hiwat(const mqueue_t *q);
+int mq_count(const mqueue_t *q);
+int mq_pending(const mqueue_t *q);
+struct bio_source *mq_bio(const mqueue_t *q);
+struct gnutella_node *mq_node(const mqueue_t *q);
 
 /*
  * Public interface
@@ -137,25 +170,13 @@ struct gnutella_node *mq_node(const struct mqueue *q);
 void mq_putq(mqueue_t *q, pmsg_t *mb);
 void mq_free(mqueue_t *q);
 void mq_clear(mqueue_t *q);
+void mq_flush(mqueue_t *q);
 void mq_discard(mqueue_t *q);
 void mq_shutdown(mqueue_t *q);
 void mq_fill_ops(struct mq_ops *ops);
 
 const struct mq_cops *mq_get_cops(void);
-const gchar *mq_info(const mqueue_t *q);
-
-/*
- * Message queue assertions.
- */
-
-#define MQ_DEBUG
-#ifdef MQ_DEBUG
-void mq_check_track(mqueue_t *q, gint offset, const gchar *where, gint line);
-
-#define mq_check(x,y)	mq_check_track((x), (y), _WHERE_, __LINE__)
-#else
-#define mq_check(x,y)
-#endif
+const char *mq_info(const mqueue_t *q);
 
 #endif	/* _core_mq_h_ */
 
