@@ -1,5 +1,5 @@
 /*
- * $Id: mq_tcp.c,v 1.3 2005/05/04 17:37:23 graaff Exp $
+ * $Id: mq_tcp.c 19142 2011-05-23 20:44:04Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -28,77 +28,86 @@
  * @file
  *
  * Message queues, writing to a TCP stack.
+ *
+ * @author Raphael Manfredi
+ * @date 2002-2003
  */
 
 #include "common.h"
 
-RCSID("$Id: mq_tcp.c,v 1.3 2005/05/04 17:37:23 graaff Exp $");
+RCSID("$Id: mq_tcp.c 19142 2011-05-23 20:44:04Z rmanfredi $")
+
+#define MQ_INTERNAL
+#include "mq.h"
 
 #include "nodes.h"
-#include "mq.h"
 #include "mq_tcp.h"
-#include "pmsg.h"
 #include "gmsg.h"
 #include "tx.h"
 #include "gnet_stats.h"
+#include "dump.h"
 
+#include "lib/pmsg.h"
 #include "lib/walloc.h"
 
 #include "if/gnet_property_priv.h"
+#include "if/core/main.h"
 
 #include "lib/override.h"		/* Must be the last header included */
 
-#define MQ_MAXIOV		256		/* Our limit on the I/O vectors we build */
-#define MQ_MINIOV		2		/* Minimum amount of I/O vectors in service */
-#define MQ_MINSEND		256		/* Minimum size we try to send */
+#define MQ_MAXIOV		256		/**< Our limit on the I/O vectors we build */
+#define MQ_MINIOV		2		/**< Minimum amount of I/O vectors in service */
+#define MQ_MINSEND		256		/**< Minimum size we try to send */
 
 static void mq_tcp_service(gpointer data);
-static struct mq_ops mq_tcp_ops;
+static const struct mq_ops mq_tcp_ops;
 
 /**
  * Create new message queue capable of holding `maxsize' bytes, and
  * owned by the supplied node.
  */
-mqueue_t *mq_tcp_make(
-	gint maxsize, struct gnutella_node *n, struct txdriver *nd)
+mqueue_t *
+mq_tcp_make(int maxsize, struct gnutella_node *n, struct txdriver *nd)
 {
 	mqueue_t *q;
 
-	q = walloc0(sizeof(*q));
-
+	WALLOC0(q);
+	q->magic = MQ_MAGIC;
 	q->node = n;
 	q->tx_drv = nd;
 	q->maxsize = maxsize;
 	q->lowat = maxsize >> 2;		/* 25% of max size */
 	q->hiwat = maxsize >> 1;		/* 50% of max size */
+	q->qwait = slist_new();
 	q->ops = &mq_tcp_ops;
 	q->cops = mq_get_cops();
+	q->debug = GNET_PROPERTY_PTR(mq_tcp_debug);
 
 	tx_srv_register(nd, mq_tcp_service, q);
 
 	return q;
 }
 
-/*
- * mq_tcp_service
- *
+/**
  * Service routine for TCP message queue.
  */
-static void mq_tcp_service(gpointer data)
+static void
+mq_tcp_service(gpointer data)
 {
 	mqueue_t *q = (mqueue_t *) data;
-	static struct iovec iov[MQ_MAXIOV];
-	gint iovsize;
-	gint iovcnt;
-	gint sent;
-	gint r;
+	static iovec_t iov[MQ_MAXIOV];
+	int iovsize;
+	int iovcnt;
+	int sent;
+	ssize_t r;
 	GList *l;
-	gint dropped;
-	gint maxsize;
+	int dropped;
+	int maxsize;
 	gboolean saturated;
 	gboolean has_prioritary = FALSE;
 
 again:
+	mq_check(q, 0);
 	g_assert(q->count);		/* Queue is serviced, we must have something */
 
 	iovcnt = 0;
@@ -118,9 +127,9 @@ again:
 	maxsize = MAX(MQ_MINSEND, maxsize);
 
 	for (l = q->qtail; l && iovsize > 0; /* empty */) {
-		struct iovec *ie;
+		iovec_t *ie;
 		pmsg_t *mb = (pmsg_t *) l->data;
-		gchar *mbs = pmsg_start(mb);
+		char *mbs = pmsg_start(mb);
 
 		/*
 		 * Don't build too much.
@@ -138,13 +147,13 @@ again:
 			l = g_list_previous(l);
 			iovsize--;
 			ie = &iov[iovcnt++];
-			ie->iov_base = mb->m_rptr;
-			ie->iov_len = pmsg_size(mb);
-			maxsize -= ie->iov_len;
+			iovec_set_base(ie, deconstify_gpointer(mb->m_rptr));
+			iovec_set_len(ie, pmsg_size(mb));
+			maxsize -= iovec_len(ie);
 			if (pmsg_prio(mb))
 				has_prioritary = TRUE;
 		} else {
-			gnet_stats_count_flowc(mbs);	/* Done before message freed */
+			gnet_stats_count_flowc(mbs, FALSE);	/* Done before message freed */
 			if (q->qlink)
 				q->cops->qlink_remove(q, l);
 
@@ -155,6 +164,7 @@ again:
 		}
 	}
 
+	mq_check(q, 0);
 	g_assert(iovcnt > 0 || dropped > 0);
 
 	if (dropped > 0)
@@ -172,15 +182,20 @@ again:
 
 	r = tx_writev(q->tx_drv, iov, iovcnt);
 
-	if (r <= 0) {
+	g_assert((ssize_t) -1 == r || !tx_has_error(q->tx_drv));
+
+	if ((ssize_t) -1 == r || r == 0) {
 		q->last_written = 0;
 		if (r == 0)
 			goto update_servicing;
 		return;
 	}
 
-	if (has_prioritary)
+	if (has_prioritary) {
 		tx_flush(q->tx_drv);
+		if (tx_has_error(q->tx_drv))
+			return;
+	}
 
 	node_add_tx_given(q->node, r);
 	q->last_written = r;
@@ -198,16 +213,15 @@ again:
 	saturated = FALSE;
 
 	for (l = q->qtail; l && r > 0 && iovsize > 0; iovsize--) {
-		struct iovec *ie = &iov[iovcnt++];
+		iovec_t *ie = &iov[iovcnt++];
 		pmsg_t *mb = (pmsg_t *) l->data;
 
-		if ((guint) r >= ie->iov_len) {			/* Completely written */
-			gchar *mb_start = pmsg_start(mb);
+		if ((guint) r >= iovec_len(ie)) {		/* Completely written */
+			char *mb_start = pmsg_start(mb);
 			guint8 function = gmsg_function(mb_start);
 			sent++;
 			pmsg_mark_sent(mb);
-            gnet_stats_count_sent(q->node,
-				function, gmsg_hops(mb_start), pmsg_size(mb));
+            gnet_stats_count_sent(q->node, function, mb_start, pmsg_size(mb));
 			switch (function) {
 			case GTA_MSG_SEARCH:
 				node_inc_tx_query(q->node);
@@ -218,10 +232,10 @@ again:
 			default:
 				break;
 			}
-			r -= ie->iov_len;
+			r -= iovec_len(ie);
 			if (q->qlink)
 				q->cops->qlink_remove(q, l);
-			l = q->cops->rmlink_prev(q, l, ie->iov_len);
+			l = q->cops->rmlink_prev(q, l, iovec_len(ie));
 		} else {
 			g_assert(r > 0 && r < pmsg_size(mb));
 			g_assert(r < q->size);
@@ -233,6 +247,7 @@ again:
 		}
 	}
 
+	mq_check(q, 0);
 	g_assert(r == 0 || iovsize > 0);
 	g_assert(q->size >= 0 && q->count >= 0);
 
@@ -279,37 +294,79 @@ update_servicing:
 
 /**
  * Enqueue message, which becomes owned by the queue.
+ *
+ * @param q			the queue to which we enqueue message
+ * @param mb		the message block to enqueue
+ * @param from		for TX traffic dump: the origin of message, NULL if local
  */
-static void mq_tcp_putq(mqueue_t *q, pmsg_t *mb)
+void
+mq_tcp_putq(mqueue_t *q, pmsg_t *mb, const struct gnutella_node *from)
 {
-	gint size = pmsg_size(mb);
-	gchar *mbs = pmsg_start(mb);
-	guint8 function = gmsg_function(mbs);
-	guint8 hops = gmsg_hops(mbs);
-	gboolean prioritary = pmsg_prio(mb) != PMSG_P_DATA;
+	int size;				/* Message size */
+	char *mbs;				/* Start of message */
+	guint8 function;		/* Gnutella message function */
+	gboolean prioritary;	/* Is message prioritary? */
+	gboolean error = FALSE;
 
+	dump_tx_tcp_packet(from, q->node, mb);
+
+again:
 	g_assert(q);
 	g_assert(!pmsg_was_sent(mb));
 	g_assert(pmsg_is_unread(mb));
+	g_assert(q->ops == &mq_tcp_ops);	/* Is a TCP queue */
 
+	/*
+	 * Trap messages enqueued whilst in the middle of an mq_clear() operation
+	 * by marking them as sent and dropping them.  Idem if queue was
+	 * put in "discard" mode.
+	 */
+
+	if (q->flags & (MQ_CLEAR | MQ_DISCARD)) {
+		pmsg_mark_sent(mb);	/* Let them think it was sent */
+		pmsg_free(mb);		/* Drop message */
+		return;
+	}
+
+	mq_check(q, 0);
+
+	size = pmsg_size(mb);
 	if (size == 0) {
-		g_warning("mq_putq: called with empty message");
+		g_warning("mq_tcp_putq: called with empty message");
 		goto cleanup;
 	}
 
-	if (q->flags & MQ_DISCARD) {
-		g_warning("mq_putq: called whilst queue shutdown");
-		goto cleanup;
+	/*
+	 * Protect against recursion: we must not invoke puthere() whilst in
+	 * the middle of another putq() or we would corrupt the qlink array:
+	 * Messages received during recursion are inserted into the qwait list
+	 * and will be stuffed back into the queue when the initial putq() ends.
+	 *		--RAM, 2006-12-29
+	 */
+
+	if (q->putq_entered > 0) {
+		if (debugging(20))
+			g_warning(
+				"mq_tcp_putq: %s recursion detected (%d already pending)",
+				mq_info(q), slist_length(q->qwait));
+		slist_append(q->qwait, mb);
+		return;
 	}
 
-	gnet_stats_count_queued(q->node, function, hops, size);
+	q->putq_entered++;
+	
+	mbs = pmsg_start(mb);
+	function = gmsg_function(mbs);
+	prioritary = pmsg_prio(mb) != PMSG_P_DATA;
+
+	gnet_stats_count_queued(q->node, function, mbs, size);
 
 	/*
 	 * If queue is empty, attempt a write immediatly.
 	 */
 
 	if (q->qhead == NULL) {
-		gint written;
+		ssize_t written;
 
 		if (pmsg_check(mb, q)) {
 			if (prioritary)
@@ -317,12 +374,28 @@ static void mq_tcp_putq(mqueue_t *q, pmsg_t *mb)
 
 			written = tx_write(q->tx_drv, mbs, size);
 
+			/*
+			 * If that assertion fails, then it means there is an error
+			 * in the TX stack which set tx_has_error() even though it
+			 * reports data written and not -1.
+			 *		--RAM, 2006-12-29
+			 */
+			g_assert((ssize_t) -1 == written || !tx_has_error(q->tx_drv));
+
+			if ((ssize_t) -1 == written) {
+				error = TRUE;
+				goto cleanup;
+			}
+
 			if (prioritary && written == size) {
 				tx_flush(q->tx_drv);
-				node_unflushq(q->node);
+				if (tx_has_error(q->tx_drv)) {
+					error = TRUE;
+					goto cleanup;
+				}
 			}
 		} else {
-			gnet_stats_count_flowc(mbs);
+			gnet_stats_count_flowc(mbs, FALSE);
 			node_inc_txdrop(q->node);		/* Dropped during TX */
 			written = -1;
 		}
@@ -335,7 +408,7 @@ static void mq_tcp_putq(mqueue_t *q, pmsg_t *mb)
 		if (written == size) {
 			pmsg_mark_sent(mb);
 			node_inc_sent(q->node);
-            gnet_stats_count_sent(q->node, function, hops, size);
+            gnet_stats_count_sent(q->node, function, mbs, size);
 			switch (function) {
 			case GTA_MSG_SEARCH:
 				node_inc_tx_query(q->node);
@@ -360,15 +433,60 @@ static void mq_tcp_putq(mqueue_t *q, pmsg_t *mb)
 	 */
 
 	q->cops->puthere(q, mb, size);
-	return;
+	mb = NULL;
 
 cleanup:
-	pmsg_free(mb);
+	if (mb) {
+		pmsg_free(mb);
+		mb = NULL;
+	}
+	g_assert(q->putq_entered >= 0);
+
+	/*
+	 * When reaching that point with a zero putq_entered counter, it means
+	 * we triggered an early error condition.  Bail out.
+	 */
+
+	if (q->putq_entered == 0)
+		error = TRUE;
+	else
+		q->putq_entered--;
+
+	mq_check(q, 0);
+
+	/*
+	 * If we're exiting here with no other putq() registered, then we must
+	 * pop an item off the head of the list and iterate again.  We stop as
+	 * soon as a write error is reported by the TX stack.
+	 */
+
+	if (0 == q->putq_entered && !error) {
+		mb = slist_shift(q->qwait);
+		if (mb) {
+			if (debugging(20))
+				g_warning(
+					"mq_tcp_putq: %s flushing waiting (%d still pending)",
+					mq_info(q), slist_length(q->qwait));
+			goto again;
+		}
+	}
+
 	return;
 }
 
-static struct mq_ops mq_tcp_ops = {
-	mq_tcp_putq,			/* putq */
+/**
+ * Disable plain mq_putq() operation on a TCP queue.
+ */
+static void
+mq_no_putq(mqueue_t *unused_q, pmsg_t *unused_mb)
+{
+	(void) unused_q;
+	(void) unused_mb;
+	g_error("plain mq_putq() forbidden on TCP queue -- use mq_tcp_putq()");
+}
+
+static const struct mq_ops mq_tcp_ops = {
+	mq_no_putq,			/* putq */
 };
 
-/* vi: set ts=4: */
+/* vi: set ts=4 sw=4 cindent: */
