@@ -1,5 +1,5 @@
 /*
- * $Id: mq_tcp.h,v 1.2 2005/05/29 08:48:35 graaff Exp $
+ * $Id: mq_tcp.h 16160 2009-03-15 15:22:28Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -36,13 +36,16 @@
 #ifndef _core_mq_tcp_h_
 #define _core_mq_tcp_h_
 
+#include "common.h"
+
 #include "mq.h"
-#include "pmsg.h"
+#include "lib/pmsg.h"
 
 struct txdriver;
 
-mqueue_t *mq_tcp_make(
-	gint maxsize, struct gnutella_node *n, struct txdriver *nd);
+void mq_tcp_putq(mqueue_t *q, pmsg_t *mb, const struct gnutella_node *from);
+mqueue_t *mq_tcp_make(int maxsize,
+	struct gnutella_node *n, struct txdriver *nd);
 
 #endif	/* _core_mq_tcp_h_ */
 
