@@ -1,5 +1,5 @@
 /*
- * $Id: nodes.h 12185 2006-11-02 14:28:28Z cbiere $
+ * $Id: nodes.h 19313 2011-06-27 21:45:52Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -41,11 +41,18 @@
 #include "if/core/guid.h"
 #include "if/core/hcache.h"
 #include "if/core/nodes.h"
+#include "if/dht/routing.h"
 
+#include "lib/cq.h"
 #include "lib/header.h"
+#include "lib/sequence.h"
+
+struct guid;
+struct nid;
 
 typedef enum {
-	NODE_MAGIC = 0x67f8e02f
+	NODE_MAGIC = 0x67f8e02f,		/* Gnutella node (TCP or "fake" UDP) */
+	NODE_UDP_MAGIC = 0x498e00de		/* UDP routing node */
 } node_magic_t;
 
 /**
@@ -80,10 +87,17 @@ struct node_rxfc_mon {
  * range, based on the hop count.
  */
 
-#define MAX_CACHE_HOPS	6		/* We won't handle anything larger */
+#define MAX_CACHE_HOPS	4		/* We won't handle anything larger */
 
 #define CACHE_HOP_IDX(h)	(((h) > MAX_CACHE_HOPS) ? MAX_CACHE_HOPS : (h))
 
+/**
+ * @def NODE_LEAF_MIN_FLOW
+ * The value of the hops-flow under which the leaf is not fully searcheable,
+ * and which will NOT have its QRP included in the inter-UP QRP table.
+ */
+#define NODE_LEAF_MIN_FLOW		3	  /**< Or leaf is not fully searcheable */
+
 /*
  * Throttle periods for ping reception, depending on the peer mode.
  * This is applicable for regular pings, not alive pings.
@@ -91,14 +105,16 @@ struct node_rxfc_mon {
 #define PING_REG_THROTTLE		3		/**< seconds, regular peer */
 #define PING_LEAF_THROTTLE		60		/**< seconds, peer is leaf node */
 
+#define NODE_ID_SELF (node_id_get_self())
+
 typedef struct gnutella_node {
 	node_magic_t magic;			/**< Magic value for consistency checks */
-    gnet_node_t node_handle;    /**< Handle of this node */
 	node_peer_t peermode;		/**< Operating mode (leaf, ultra, normal) */
 	node_peer_t start_peermode;	/**< Operating mode when handshaking begun */
 
-	gchar error_str[256];		/**< To sprintf() error strings with vars */
+	char error_str[256];		/**< To sprintf() error strings with vars */
 	struct gnutella_socket *socket;		/**< Socket of the node */
+	guint16 country;			/**< Country of origin -- encoded ISO3166 */
 	guint8 proto_major;			/**< Handshaking protocol major number */
 	guint8 proto_minor;			/**< Handshaking protocol minor number */
 
@@ -106,19 +122,19 @@ typedef struct gnutella_node {
 	guint8 qrp_minor;			/**< Query routing protocol minor number */
 	guint8 uqrp_major;			/**< UP Query routing protocol major number */
 	guint8 uqrp_minor;			/**< UP Query routing protocol minor number */
-	gchar *vendor;				/**< Vendor information (always UTF-8) */
-	gint country;				/**< Country of origin -- encoded ISO3166 */
-	union vendor_code vcode;	/**< Vendor code (vcode.u32 == 0 if unknown) */
+	const char *vendor;			/**< Vendor information (always UTF-8) */
+	vendor_code_t vcode;		/**< Vendor code (vcode.u32 == 0 if unknown) */
 	gpointer io_opaque;			/**< Opaque I/O callback information */
 
 	gnutella_header_t header;		/**< Header of the current message */
 	extvec_t extvec[MAX_EXTVEC];	/**< GGEP extensions in "fat" messages */
-	gint extcount;					/**< Amount of extensions held */
+	int extcount;					/**< Amount of extensions held */
 
 	guint16 size; /**< How many bytes we need to read for the current message */
 	guint16 header_flags;		/**< Header flags (new message architecture) */
+	guint32 msg_flags;			/**< Message flags we set during analysis */
 
-	gchar *data;				/**< data of the current message */
+	char *data;					/**< data of the current message */
 	guint32 pos;				/**< write position in data */
 
 	gnet_node_state_t status;	/**< See possible values below */
@@ -160,12 +176,11 @@ typedef struct gnutella_node {
 	time_t leaf_flowc_start;	/**< Time when leaf flow-controlled queries */
 	time_delta_t shutdown_delay; /**< How long we can stay in shutdown mode */
 
-	const gchar *remove_msg;	/**< Reason of removing */
+	const char *remove_msg;	/**< Reason of removing */
 
 	host_addr_t addr;			/**< ip of the node */
 	guint16 port;				/**< port of the node */
 
-	gchar *guid;				/**< GUID of node (atom) for push-proxying */
 	host_addr_t proxy_addr;		/**< ip of the node for push proxyfication */
 	guint16 proxy_port;			/**< port of the node for push proxyfication */
 
@@ -173,11 +188,11 @@ typedef struct gnutella_node {
 	squeue_t *searchq;			/**< TX Search queue */
 	rxdrv_t *rx;				/**< RX stack top */
 
-	gpointer routing_data;		/**< Opaque info, for gnet message routing */
-	gpointer sent_query_table;	/**< Opaque info, query table sent to node */
-	gpointer recv_query_table;	/**< Opaque info, query table recved from node */
-	gpointer qrt_update;		/**< Opaque info, query routing update handle */
-	gpointer qrt_receive;		/**< Opaque info, query routing reception */
+	struct route_data *routing_data;		/**< for gnet message routing */
+	struct routing_table *sent_query_table;	/**< query table sent to node */
+	struct routing_table *recv_query_table;	/**< query table recved from node */
+	struct qrt_update *qrt_update;			/**< query routing update handle */
+	struct qrt_receive *qrt_receive;		/**< query routing reception */
 	qrt_info_t *qrt_info;		/**< Info about received query table */
 
 	gpointer alive_pings;		/**< Opaque info, for alive ping checks */
@@ -185,6 +200,7 @@ typedef struct gnutella_node {
 	time_delta_t alive_period;	/**< Period for sending alive pings (secs) */
 
 	wrap_buf_t hello;			/**< Spill buffer for GNUTELLA HELLO */
+	cevent_t *dht_nope_ev;		/**< Periodic event for NOPE DHT publishing */
 
 	/*
 	 * Round-trip time (RTT) measurements operated via Time Sync
@@ -194,18 +210,18 @@ typedef struct gnutella_node {
 
 	guint32 tcp_rtt;			/**< RTT when exchange takes place over TCP */
 	guint32 udp_rtt;			/**< RTT when exchange takes place over UDP  */
-	gpointer tsync_ev;			/**< Time sync event */
+	cevent_t *tsync_ev;			/**< Time sync event */
 
 	/*
 	 * Data structures used by the ping/pong reduction scheme.
 	 *		--RAM, 02/02/2002
 	 */
 
-	guint32 id;					/**< Unique internal ID */
+	struct nid *id;				/**< Unique internal ID */
 	guint ping_throttle;		/**< Period for accepting new pings (secs) */
 	time_t ping_accept;			/**< Time after which we accept new pings */
 	time_t next_ping;			/**< When to send a ping, for "OLD" clients */
-	gchar ping_guid[GUID_RAW_SIZE];	/**< The GUID of the last accepted ping */
+	struct guid ping_guid;		/**< The GUID of the last accepted ping */
 	guchar pong_needed[MAX_CACHE_HOPS+1];	/**< Pongs needed, by hop value */
 	guchar pong_missing;	/**< Sum(pong_needed[i]), i = 0..MAX_CACHE_HOPS */
 
@@ -215,7 +231,9 @@ typedef struct gnutella_node {
 	guint32 gnet_kbytes_count;	/**< Used to answer "Crawling" pings */
 	host_addr_t gnet_pong_addr;	/**< When != 0, last IP we got in pong */
 	host_addr_t gnet_qhit_addr;	/**< When != 0, last IP we got in query hit */
-	gchar *gnet_guid;			/**< GUID of node (atom) seen on the network */
+	const struct guid *guid;	/**< GUID of node (atom) seen on the network */
+
+	guint32 svn_release_revision;	/**< Latest SVN release known by the node */
 
 	guint32 n_ping_throttle;  /**< Number of pings we throttled */
 	guint32 n_ping_accepted;  /**< Number of pings we accepted */
@@ -228,13 +246,13 @@ typedef struct gnutella_node {
 	 * Traffic statistics -- RAM, 13/05/2002.
 	 */
 
-	gint32 tx_given;			/**< Bytes fed to the TX stack (from top) */
-	gint32 tx_deflated;			/**< Bytes deflated by the TX stack */
-	gint32 tx_written;			/**< Bytes written by the TX stack */
+	guint64 tx_given;			/**< Bytes fed to the TX stack (from top) */
+	guint64 tx_deflated;		/**< Bytes deflated by the TX stack */
+	guint64 tx_written;			/**< Bytes written by the TX stack */
 
-	gint32 rx_given;			/**< Bytes fed to the RX stack (from bottom) */
-	gint32 rx_inflated;			/**< Bytes inflated by the RX stack */
-	gint32 rx_read;				/**< Bytes read from the RX stack */
+	guint64 rx_given;			/**< Bytes fed to the RX stack (from bottom) */
+	guint64 rx_inflated;		/**< Bytes inflated by the RX stack */
+	guint64 rx_read;			/**< Bytes read from the RX stack */
 
 	/*
 	 * Various Gnutella statistics -- RAM, 10/12/2003.
@@ -258,63 +276,83 @@ typedef struct gnutella_node {
 
 } gnutella_node_t;
 
-/*
+/**
  * Node flags.
  */
+enum {
+	NODE_F_NOT_GENUINE	= 1 << 27,	/**< Vendor cannot be genuine */
+	NODE_F_VMSG_SUPPORT	= 1 << 26,	/**< Indicated which VMSGs are supported */
+	NODE_F_CAN_TLS		= 1 << 25,	/**< Indicated support for TLS */
+	NODE_F_TLS			= 1 << 24,	/**< TLS-tunneled */
+	NODE_F_NO_OOB_PROXY	= 1 << 23,	/**< Do not OOB proxy the leaf */
+	NODE_F_FORCE		= 1 << 22,	/**< Connection is forced */
+	NODE_F_GTKG			= 1 << 21,	/**< Node is another gtk-gnutella */
+	NODE_F_TSYNC_TCP	= 1 << 20,	/**< No replies via UDP, use TCP */
+	NODE_F_TSYNC_WAIT	= 1 << 19,	/**< Time sync pending via TCP */
+	NODE_F_QRP_SENT		= 1 << 18,	/**< Undergone 1 complete QRP sending */
+	NODE_F_PROXIED		= 1 << 17,	/**< We are push-proxy for that node */
+	NODE_F_PROXY		= 1 << 16,	/**< Sent a push-proxy request to it */
+	NODE_F_FAKE_NAME	= 1 << 15,	/**< Was unable to validate GTKG name */
+	NODE_F_CRAWLER		= 1 << 14,	/**< Is a Gnutella Crawler */
+	NODE_F_LEAF			= 1 << 13,	/**< Is one of our leaves */
+	NODE_F_ULTRA		= 1 << 12,	/**< Is one of our ultra nodes */
+	NODE_F_CLOSING		= 1 << 11,	/**< Initiated bye or shutdown */
+	NODE_F_EOF_WAIT		= 1 << 10,	/**< At final shutdown, waiting EOF */
+	NODE_F_NOREAD		= 1 << 9,	/**< Prevent further reading from node */
+	NODE_F_BYE_SENT		= 1 << 8,	/**< Bye message was queued */
+	NODE_F_READABLE		= 1 << 7,	/**< Node is readable, process queries */
+	NODE_F_WRITABLE		= 1 << 6,	/**< Node is writable */
+	NODE_F_ALIEN_IP		= 1 << 5,	/**< Pong-IP did not match TCP/IP addr */
+	NODE_F_VALID		= 1 << 4,	/**< Handshaked with a Gnutella node */
+	NODE_F_ESTABLISHED	= 1 << 3,	/**< Gnutella connection established */
+	NODE_F_INCOMING		= 1 << 2,	/**< Incoming (permanent) connection */
+	NODE_F_STALE_QRP	= 1 << 1,	/**< Is sending a stale QRP patch */
+	NODE_F_HDSK_PING	= 1 << 0	/**< Expecting handshake ping */
+};
 
-#define NODE_F_HDSK_PING	0x00000001 /**< Expecting handshake ping */
-#define NODE_F_STALE_QRP	0x00000002 /**< Is sending a stale QRP patch */
-#define NODE_F_INCOMING		0x00000004 /**< Incoming (permanent) connection */
-#define NODE_F_ESTABLISHED	0x00000008 /**< Gnutella connection established */
-#define NODE_F_VALID		0x00000010 /**< Handshaked with a Gnutella node */
-#define NODE_F_ALIEN_IP		0x00000020 /**< Pong-IP did not match TCP/IP addr */
-#define NODE_F_WRITABLE		0x00000040 /**< Node is writable */
-#define NODE_F_READABLE		0x00000080 /**< Node is readable, process queries */
-#define NODE_F_BYE_SENT		0x00000100 /**< Bye message was queued */
-#define NODE_F_NODELAY		0x00000200 /**< TCP_NODELAY was activated */
-#define NODE_F_NOREAD		0x00000400 /**< Prevent further reading from node */
-#define NODE_F_EOF_WAIT		0x00000800 /**< At final shutdown, waiting EOF */
-#define NODE_F_CLOSING		0x00001000 /**< Initiated bye or shutdown */
-#define NODE_F_ULTRA		0x00002000 /**< Is one of our ultra nodes */
-#define NODE_F_LEAF			0x00004000 /**< Is one of our leaves */
-#define NODE_F_CRAWLER		0x00008000 /**< Is a Gnutella Crawler */
-#define NODE_F_FAKE_NAME	0x00010000 /**< Was unable to validate GTKG name */
-#define NODE_F_PROXY		0x00020000 /**< Sent a push-proxy request to it */
-#define NODE_F_PROXIED		0x00040000 /**< We are push-proxy for that node */
-#define NODE_F_QRP_SENT		0x00080000 /**< Undergone 1 complete QRP sending */
-#define NODE_F_TLS			0x00100000 /**< TLS-tunneled */
-#define NODE_F_TSYNC_WAIT	0x00200000 /**< Time sync pending via TCP */
-#define NODE_F_TSYNC_TCP	0x00400000 /**< No replies via UDP, use TCP */
-#define NODE_F_GTKG			0x00800000 /**< Node is another gtk-gnutella */
-#define NODE_F_FORCE		0x01000000 /**< Connection is forced */
-
-/*
+/**
  * Node attributes.
  */
+enum {
+	NODE_A_CAN_WHAT		= 1 << 26,	/**< Node supports "What's New?" queries? */
+	NODE_A_GUESS		= 1 << 25,	/**< Node advertized GUESS support */
+	NODE_A_CAN_DHT		= 1 << 24,	/**< Indicated support for DHT */
+	NODE_A_FIREWALLED	= 1 << 23,	/**< Node determined to be firewalled */
+	NODE_A_CAN_SVN_NOTIFY =	1 << 22,	/**< Supports SVN release notifies */
+	NODE_A_CAN_HEAD		= 1 << 21,	/**< Supports HEAD ping (vendor message) */
+	NODE_A_CAN_INFLATE	= 1 << 20,	/**< Node capable of inflating */
+	NODE_A_CAN_ULTRA	= 1 << 19,	/**< Node is ultra capable */
+	NODE_A_CAN_VENDOR	= 1 << 18,	/**< Node supports vendor messages */
+	NODE_A_CAN_QRP		= 1 << 17,	/**< Node supports query routing */
+	NODE_A_CAN_HSEP		= 1 << 16,	/**< Node supports HSEP */
+	NODE_A_UDP			= 1 << 15,	/**< Node uses UDP for traffic */
+	NODE_A_DQ_PROBE		= 1 << 14,	/**< Node can be safely probed by DQ */
+	NODE_A_CAN_SFLAG	= 1 << 13,	/**< Node supports flags in headers */
+	NODE_A_DYN_QUERY	= 1 << 12,	/**< Node can perform dynamic queries */
+	NODE_A_CRAWLABLE	= 1 << 11,	/**< Node can be UDP-crawled */
+	NODE_A_TIME_SYNC	= 1 << 10,	/**< Supports time sync */
+	NODE_A_GUIDANCE		= 1 << 9,	/**< Can leaf-guide dyn queries */
+	NODE_A_UP_QRP		= 1 << 8,	/**< Supports intra-UP QRP */
+	NODE_A_NO_ULTRA		= 1 << 7,	/**< Node is NOT ultra capable */
+	NODE_A_ULTRA		= 1 << 6,	/**< Node wants to be an Ultrapeer */
+	NODE_A_TX_DEFLATE	= 1 << 5,	/**< Sending compressed data */
+	NODE_A_RX_INFLATE	= 1 << 4,	/**< Reading compressed data */
+	NODE_A_QHD_NO_VTAG	= 1 << 3,	/**< Servent has no vendor tag in QHD */
+	NODE_A_PONG_ALIEN	= 1 << 2,	/**< Alien Pong-Caching scheme */
+	NODE_A_PONG_CACHING	= 1 << 1,	/**< Supports Pong-Caching */
+	NODE_A_BYE_PACKET	= 1 << 0	/**< Supports Bye-Packet */
+};
 
-#define NODE_A_BYE_PACKET	0x00000001 /**< Supports Bye-Packet */
-#define NODE_A_PONG_CACHING	0x00000002 /**< Supports Pong-Caching */
-#define NODE_A_PONG_ALIEN	0x00000004 /**< Alien Pong-Caching scheme */
-#define NODE_A_QHD_NO_VTAG	0x00000008 /**< Servent has no vendor tag in QHD */
-#define NODE_A_RX_INFLATE	0x00000010 /**< Reading compressed data */
-#define NODE_A_TX_DEFLATE	0x00000020 /**< Sending compressed data */
-#define NODE_A_ULTRA		0x00000040 /**< Node wants to be an Ultrapeer */
-#define NODE_A_NO_ULTRA		0x00000080 /**< Node is NOT ultra capable */
-#define NODE_A_UP_QRP		0x00000100 /**< Supports intra-UP QRP */
-#define NODE_A_GUIDANCE		0x00000200 /**< Can leaf-guide dyn queries */
-#define NODE_A_TIME_SYNC	0x00000400 /**< Supports time sync */
-#define NODE_A_CRAWLABLE	0x00000800 /**< Node can be UDP-crawled */
-#define NODE_A_DYN_QUERY	0x00001000 /**< Node can perform dynamic queries */
-#define NODE_A_CAN_SFLAG	0x00002000 /**< Node supports flags in headers */
- 
-#define NODE_A_NO_KEPT_ZERO	0x01000000 /**< For GTKG < 2006-08-15: no kept=0! */
-#define NODE_A_NO_DUPS		0x02000000 /**< For broken old GTKG: no dups! */
-#define NODE_A_CAN_HSEP		0x04000000 /**< Node supports HSEP */
-#define NODE_A_CAN_QRP		0x08000000 /**< Node supports query routing */
-#define NODE_A_CAN_VENDOR	0x10000000 /**< Node supports vendor messages */
-#define NODE_A_CAN_GGEP		0x20000000 /**< Node supports big pongs, etc.. */
-#define NODE_A_CAN_ULTRA	0x40000000 /**< Node is ultra capable */
-#define NODE_A_CAN_INFLATE	0x80000000 /**< Node capable of inflating */
+/**
+ * Message flags, set during parsing / processing.
+ */
+enum {
+	NODE_M_WHATS_NEW	= 1 << 4,	/**< Facing a "What's New?" query */
+	NODE_M_STRIP_GGEP_u	= 1 << 3,	/**< Must strip GGEP "u" */
+	NODE_M_STRIP_GUESS	= 1 << 2,	/**< Must strip all GUESS extensions */
+	NODE_M_EXT_CLEANUP	= 1 << 1,	/**< Must cleanup extensions */
+	NODE_M_COMPACTED	= 1 << 0	/**< Compaction occurred */
+};
  
 /*
  * UDP crawling "feature" flags.
@@ -370,6 +408,9 @@ typedef struct gnutella_node {
 
 #define NODE_SQUEUE(n) ((n)->searchq)
 
+#define NODE_MQUEUE_STATUS(n) \
+	((n)->outq ? mq_status((n)->outq) : MQ_S_EMPTY)
+
 #define NODE_MQUEUE_COUNT(n) \
 	((n)->outq ? mq_count((n)->outq) : 0)
 
@@ -402,10 +443,11 @@ typedef struct gnutella_node {
 #define NODE_ID(n)				((n)->id)
 
 #define NODE_CAN_SFLAG(n)		((n)->attrs & NODE_A_CAN_SFLAG)
-#define NODE_CAN_GGEP(n)		((n)->attrs & NODE_A_CAN_GGEP)
 #define NODE_UP_QRP(n)			((n)->attrs & NODE_A_UP_QRP)
 #define NODE_LEAF_GUIDE(n)		((n)->attrs & NODE_A_GUIDANCE)
 #define NODE_CAN_INFLATE(n)		((n)->attrs & NODE_A_CAN_INFLATE)
+#define NODE_USES_UDP(n)		((n)->attrs & NODE_A_UDP)
+#define NODE_CAN_WHAT(n)		((n)->attrs & NODE_A_CAN_WHAT)
 
 /*
  * Peer inspection macros
@@ -415,14 +457,17 @@ typedef struct gnutella_node {
 #define NODE_IS_NORMAL(n)		((n)->peermode == NODE_P_NORMAL)
 #define NODE_IS_ULTRA(n)		((n)->peermode == NODE_P_ULTRA)
 #define NODE_IS_UDP(n)			((n)->peermode == NODE_P_UDP)
+#define NODE_IS_DHT(n)			((n)->peermode == NODE_P_DHT)
 
 /*
  * Macros.
  */
 
 #define node_vendor(n)		((n)->vendor != NULL ? (n)->vendor : "????")
-#define node_type(n)		\
-	(NODE_IS_LEAF(n) ? "leaf" : NODE_IS_ULTRA(n) ? "ultra" : "legacy")
+#define node_type(n)			\
+	(NODE_IS_UDP(n) ? "UDP" :	\
+	 NODE_IS_LEAF(n) ? "leaf" :	\
+	 NODE_IS_ULTRA(n) ? "ultra" : "legacy")
 
 #define node_inc_sent(n)            node_add_sent(n, 1)
 #define node_inc_txdrop(n)          node_add_txdrop(n, 1)
@@ -481,12 +526,7 @@ typedef struct gnutella_node {
 #define GNUTELLA_HELLO "GNUTELLA CONNECT/"
 #define GNUTELLA_HELLO_LENGTH	(sizeof(GNUTELLA_HELLO) - 1)
 
-#define NODE_ID_LOCAL	0x0U		/**< ID for "local node" (ourselves) */
-
-extern gchar *start_rfc822_date;
-
-extern GHookList node_added_hook_list;
-extern struct gnutella_node *node_added;
+extern const char *start_rfc822_date;
 
 /*
  * Global Functions
@@ -498,7 +538,7 @@ void node_slow_timer(time_t now);
 void node_timer(time_t now);
 guint connected_nodes(void);
 guint node_count(void);
-guint node_keep_missing(void);
+int node_keep_missing(void);
 guint node_missing(void);
 guint node_leaves_missing(void);
 guint node_outdegree(void);
@@ -508,16 +548,17 @@ gboolean node_host_is_connected(const host_addr_t addr, guint16 port);
 void node_add_socket(struct gnutella_socket *s, const host_addr_t addr,
 		guint16 port, guint32 flags);
 void node_remove(struct gnutella_node *,
-	const gchar * reason, ...) G_GNUC_PRINTF(2, 3);
-void node_bye(gnutella_node_t *, gint code,
-	const gchar * reason, ...) G_GNUC_PRINTF(3, 4);
+	const char * reason, ...) G_GNUC_PRINTF(2, 3);
+guint node_remove_by_addr(const host_addr_t addr, guint16 port);
+void node_bye(gnutella_node_t *, int code,
+	const char * reason, ...) G_GNUC_PRINTF(3, 4);
 void node_real_remove(gnutella_node_t *);
 void node_eof(struct gnutella_node *n,
-	const gchar * reason, ...) G_GNUC_PRINTF(2, 3);
+	const char * reason, ...) G_GNUC_PRINTF(2, 3);
 void node_shutdown(struct gnutella_node *n,
-	const gchar * reason, ...) G_GNUC_PRINTF(2, 3);
-void node_bye_if_writable(struct gnutella_node *n, gint code,
-	const gchar * reason, ...) G_GNUC_PRINTF(3, 4);
+	const char * reason, ...) G_GNUC_PRINTF(2, 3);
+void node_bye_if_writable(struct gnutella_node *n, int code,
+	const char * reason, ...) G_GNUC_PRINTF(3, 4);
 void node_init_outgoing(struct gnutella_node *);
 void node_sent_ttl0(struct gnutella_node *n);
 void node_disableq(struct gnutella_node *n);
@@ -535,68 +576,110 @@ gboolean node_bye_pending(void);
 void node_close(void);
 gboolean node_remove_worst(gboolean non_local);
 
-void node_qrt_changed(gpointer query_table);
+void node_qrt_changed(struct routing_table *query_table);
 void node_qrt_discard(struct gnutella_node *n);
-void node_qrt_install(struct gnutella_node *n, gpointer query_table);
-void node_qrt_patched(struct gnutella_node *n, gpointer query_table);
+void node_qrt_install(struct gnutella_node *n, struct routing_table *);
+void node_qrt_patched(struct gnutella_node *n, struct routing_table *);
 
 void send_node_error(struct gnutella_socket *s, int code,
-	const gchar *msg, ...) G_GNUC_PRINTF(3, 4);
+	const char *msg, ...) G_GNUC_PRINTF(3, 4);
 
-void node_add_sent(gnutella_node_t *n, gint x);
-void node_add_txdrop(gnutella_node_t *n, gint x);
-void node_add_rxdrop(gnutella_node_t *n, gint x);
+void node_add_sent(gnutella_node_t *n, int x);
+void node_add_txdrop(gnutella_node_t *n, int x);
+void node_add_rxdrop(gnutella_node_t *n, int x);
 
-void node_set_vendor(gnutella_node_t *n, const gchar *vendor);
+void node_set_vendor(gnutella_node_t *n, const char *vendor);
 
 void node_set_hops_flow(gnutella_node_t *n, guint8 hops);
 void node_set_online_mode(gboolean on);
 void node_current_peermode_changed(node_peer_t mode);
-const gchar *node_addr(const gnutella_node_t *n);
-const gchar *node_addr2(const gnutella_node_t *n);
-const gchar *node_gnet_addr(const gnutella_node_t *n);
+const char *node_addr(const gnutella_node_t *n);
+const char *node_addr2(const gnutella_node_t *n);
+const char *node_gnet_addr(const gnutella_node_t *n);
+const char *node_infostr(const gnutella_node_t *n);
+const char *node_id_infostr(const struct nid *node_id);
 
 void node_connect_back(const gnutella_node_t *n, guint16 port);
 void node_connected_back(struct gnutella_socket *s);
 
 void node_mark_bad_vendor(struct gnutella_node *n);
 
-void node_proxying_remove(gnutella_node_t *n, gboolean discard);
-gboolean node_proxying_add(gnutella_node_t *n, const gchar *guid);
+void node_proxying_remove(gnutella_node_t *n);
+gboolean node_proxying_add(gnutella_node_t *n, const struct guid *guid);
 void node_proxy_add(gnutella_node_t *n, const host_addr_t addr, guint16 port);
 void node_proxy_cancel_all(void);
-void node_http_proxies_add(
-	gchar *buf, gint *retval, gpointer arg, guint32 flags);
-GSList *node_push_proxies(void);
+size_t node_http_fw_node_info_add(char *buf, size_t size, gboolean);
+size_t node_http_proxies_add(char *buf, size_t size,
+			gpointer arg, guint32 flags);
+sequence_t *node_push_proxies(void);
+const gnet_host_t *node_oldest_push_proxy(void);
 const GSList *node_all_nodes(void);
-const GSList *node_all_but_broken_gtkg(void);
-gnutella_node_t *node_active_by_id(guint32 id);
-void node_set_leaf_guidance(guint32 id, gboolean supported);
+
+gnutella_node_t *node_by_id(const struct nid *node_id);
+gnutella_node_t *node_active_by_id(const struct nid *node_id);
+void node_set_leaf_guidance(const struct nid *node_id, gboolean supported);
 
 void node_became_firewalled(void);
 void node_became_udp_firewalled(void);
-void node_set_socket_rx_size(gint rx_size);
+void node_set_socket_rx_size(int rx_size);
 
 mqueue_t *node_udp_get_outq(enum net_type net);
-void node_udp_enable(void);
+gboolean node_dht_is_flow_controlled(void);
+gboolean node_dht_would_flow_control(size_t additional);
+gboolean node_dht_above_low_watermark(void);
 void node_udp_disable(void);
-void node_udp_process(struct gnutella_socket *s);
+void node_udp_process(gnutella_node_t *n, struct gnutella_socket *s);
 gnutella_node_t *node_udp_get_addr_port(const host_addr_t addr, guint16 port);
+gnutella_node_t *node_dht_get_addr_port(const host_addr_t addr, guint16 port);
+gnutella_node_t * node_udp_route_get_addr_port(
+	const host_addr_t addr, guint16 port, gboolean can_deflate);
 
 void node_can_tsync(gnutella_node_t *n);
-void node_crawl(gnutella_node_t *n, gint ucnt, gint lcnt, guint8 features);
+void node_crawl(gnutella_node_t *n, int ucnt, int lcnt, guint8 features);
 
 void node_update_udp_socket(void);
 void node_check_remote_ip_header(const host_addr_t peer, header_t *head);
 
 guint feed_host_cache_from_headers(header_t *headers,
-	host_type_t sender, gboolean gnet, const host_addr_t peer);
+	host_type_t sender, gboolean gnet, const host_addr_t peer,
+	const char *vendor);
 
 gnutella_node_t *node_browse_prepare(
-	gnet_host_t *host, const gchar *vendor, gnutella_header_t *header,
-	gchar *data, guint32 size);
+	gnet_host_t *host, const char *vendor, gnutella_header_t *header,
+	char *data, guint32 size);
 void node_browse_cleanup(gnutella_node_t *n);
 void node_kill_hostiles(void);
+void node_supports_tls(struct gnutella_node *);
+void node_supports_whats_new(struct gnutella_node *);
+void node_supports_dht(struct gnutella_node *, dht_mode_t);
+void node_is_firewalled(gnutella_node_t *n);
+
+const struct nid *node_id_get_self(void);
+gboolean node_id_self(const struct nid *node_id);
+
+static inline void
+node_check(const struct gnutella_node * const n)
+{
+	g_assert(n != NULL);
+	g_assert(NODE_MAGIC == n->magic);
+}
+
+static inline const struct guid *
+node_guid(const struct gnutella_node * const n)
+{
+	node_check(n);
+	return n->guid;
+}
+
+static inline struct nid *
+node_get_id(const struct gnutella_node * const n)
+{
+	node_check(n);
+	return n->id;
+}
+
+gboolean node_set_guid(struct gnutella_node *n, const struct guid *guid);
+struct gnutella_node *node_by_guid(const struct guid *guid);
 
 #endif /* _core_nodes_h_ */
 
