@@ -1,5 +1,5 @@
 /*
- * $Id: oob.c 13777 2007-05-29 00:23:11Z cbiere $
+ * $Id: oob.c 19142 2011-05-23 20:44:04Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -35,14 +35,13 @@
 
 #include "common.h"
 
-RCSID("$Id: oob.c 13777 2007-05-29 00:23:11Z cbiere $")
+RCSID("$Id: oob.c 19142 2011-05-23 20:44:04Z rmanfredi $")
 
 #include "oob.h"
 #include "hosts.h"
 #include "nodes.h"
 #include "share.h"
 #include "guid.h"
-#include "pmsg.h"
 #include "mq.h"
 #include "mq_udp.h"
 #include "vmsg.h"
@@ -55,6 +54,9 @@ RCSID("$Id: oob.c 13777 2007-05-29 00:23:11Z cbiere $")
 #include "lib/atoms.h"
 #include "lib/cq.h"
 #include "lib/fifo.h"
+#include "lib/glib-missing.h"
+#include "lib/pmsg.h"
+#include "lib/random.h"
 #include "lib/walloc.h"
 #include "lib/override.h"		/* Must be the last header included */
 
@@ -78,14 +80,14 @@ typedef enum {
  */
 struct oob_results {
 	oob_results_magic_t	magic;
-	gint refcount;
+	int refcount;
 	cevent_t *ev_expire;	/**< Global expiration event */
 	cevent_t *ev_timeout;	/**< Reply waiting timeout */
-	const gchar *muid;		/**< (atom) MUID of the query that generated hits */
+	const struct guid *muid;/**< (atom) MUID of the query that generated hits */
 	GSList *files;			/**< List of shared_file_t */
 	gnet_host_t dest;		/**< The host to which we must deliver */
-	gint count;				/**< Amount of hits to deliver */
-	gint notify_requeued;	/**< Amount of LIME/12v2 requeued after dropping */
+	int count;				/**< Amount of hits to deliver */
+	int notify_requeued;	/**< Amount of LIME/12v2 requeued after dropping */
 	gboolean secure;		/**< TRUE -> secure OOB, FALSE -> normal OOB */
 	gboolean ggep_h;		/**< TRUE -> use GGEP H, FALSE -> plain text */
 };
@@ -94,7 +96,7 @@ struct oob_results {
  * Indexes all OOB queries by MUID.
  * This hash table records MUID => "struct oob_results"
  */
-static GHashTable *results_by_muid = NULL;
+static GHashTable *results_by_muid;
 
 /**
  * Each servent, as identified by its IP:port, is given a FIFO for queuing
@@ -134,7 +136,7 @@ static void results_destroy(cqueue_t *cq, gpointer obj);
 static void servent_free(struct gservent *s);
 static void oob_send_reply_ind(struct oob_results *r);
 
-static gint num_oob_records;	/**< Leak and duplicate free detector */
+static int num_oob_records;	/**< Leak and duplicate free detector */
 static gboolean oob_shutdown_running;
 
 static void
@@ -151,25 +153,25 @@ oob_results_check(const struct oob_results *r)
  * results delivery via the sent LIME/12v2 and the expected LIME/11v2 reply.
  */
 static struct oob_results *
-results_make(const gchar *muid, GSList *files, gint count, gnet_host_t *to,
-	gboolean secure, gboolean ggep_h)
+results_make(const struct guid *muid, GSList *files, int count,
+	gnet_host_t *to, gboolean secure, gboolean ggep_h)
 {
 	static const struct oob_results zero_results;
 	struct oob_results *r;
 
 	g_return_val_if_fail(!g_hash_table_lookup(results_by_muid, muid), NULL);
 
-	r = walloc(sizeof *r);
+	WALLOC(r);
 	*r = zero_results;
 	r->magic = OOB_RESULTS_MAGIC;
 	r->muid = atom_guid_get(muid);
 	r->files = files;
 	r->count = count;
-	r->dest = *to;			/* Struct copy */
+	gnet_host_copy(&r->dest, to);
 	r->secure = secure;
 	r->ggep_h = ggep_h;
 
-	r->ev_expire = cq_insert(callout_queue, OOB_EXPIRE_MS, results_destroy, r);
+	r->ev_expire = cq_main_insert(OOB_EXPIRE_MS, results_destroy, r);
 	r->refcount++;
 
 	gm_hash_table_insert_const(results_by_muid, r->muid, r);
@@ -177,7 +179,7 @@ results_make(const gchar *muid, GSList *files, gint count, gnet_host_t *to,
 	g_assert(num_oob_records >= 0);
 	num_oob_records++;
 	if (GNET_PROPERTY(query_debug) > 1)
-		g_message("results_make: num_oob_records=%d", num_oob_records);
+		g_debug("results_make: num_oob_records=%d", num_oob_records);
 
 	return r;
 }
@@ -193,12 +195,12 @@ results_free_remove(struct oob_results *r)
 	oob_results_check(r);
 	
 	if (r->ev_expire) {
-		cq_cancel(callout_queue, &r->ev_expire);
+		cq_cancel(&r->ev_expire);
 		g_assert(r->refcount > 0);
 		r->refcount--;
 	}
 	if (r->ev_timeout) {
-		cq_cancel(callout_queue, &r->ev_timeout);
+		cq_cancel(&r->ev_timeout);
 		g_assert(r->refcount > 0);
 		r->refcount--;
 	}
@@ -215,16 +217,15 @@ results_free_remove(struct oob_results *r)
 			shared_file_t *sf = sl->data;
 			shared_file_unref(&sf);
 		}
-		g_slist_free(r->files);
-		r->files = NULL;
+		gm_slist_free_null(&r->files);
 
 		g_assert(num_oob_records > 0);
 		num_oob_records--;
 		if (GNET_PROPERTY(query_debug) > 2)
-			g_message("results_free: num_oob_records=%d", num_oob_records);
+			g_debug("results_free: num_oob_records=%d", num_oob_records);
 
 		r->magic = 0;
-		wfree(r, sizeof *r);
+		WFREE(r);
 	}
 }
 
@@ -240,7 +241,7 @@ results_destroy(cqueue_t *unused_cq, gpointer obj)
 	oob_results_check(r);
 
 	if (GNET_PROPERTY(query_debug))
-		g_message("OOB query %s from %s expired with unclaimed %d hit%s",
+		g_debug("OOB query %s from %s expired with unclaimed %d hit%s",
 			guid_hex_str(r->muid), gnet_host_to_string(&r->dest),
 			r->count, r->count == 1 ? "" : "s");
 
@@ -264,7 +265,7 @@ results_timeout(cqueue_t *unused_cq, gpointer obj)
 	oob_results_check(r);
 
 	if (GNET_PROPERTY(query_debug))
-		g_message("OOB query %s, no ACK from %s to claim %d hit%s",
+		g_debug("OOB query %s, no ACK from %s to claim %d hit%s",
 			guid_hex_str(r->muid), gnet_host_to_string(&r->dest),
 			r->count, r->count == 1 ? "" : "s");
 
@@ -292,7 +293,7 @@ servent_free_remove(struct gservent *s)
  * Per a suggestion of Daniel Stutzbach, we wait BASE + RAND*random secs,
  * where "random" is a real random number between 0 and 1.
  */
-static gint
+static int
 deliver_delay(void)
 {
 	return OOB_DELIVER_BASE_MS + random_value(OOB_DELIVER_RAND_MS);
@@ -320,9 +321,10 @@ servent_service(cqueue_t *cq, gpointer obj)
 		goto udp_disabled;
 
 	if (GNET_PROPERTY(udp_debug) > 19)
-		g_message("UDP queuing OOB %s to %s for %s",
-			gmsg_infostr_full(pmsg_start(mb)), gnet_host_to_string(s->host),
-			guid_hex_str(pmsg_start(mb)));
+		g_debug("UDP queuing OOB %s to %s for %s",
+			gmsg_infostr_full(pmsg_start(mb), pmsg_written_size(mb)),
+			gnet_host_to_string(s->host),
+			guid_hex_str(cast_to_guid_ptr_const(pmsg_start(mb))));
 
 	/*
 	 * Count enqueued deflated payloads, only when server was marked as
@@ -361,9 +363,8 @@ servent_make(gnet_host_t *host, gboolean can_deflate)
 {
 	struct gservent *s;
 
-	s = walloc(sizeof *s);
-	s->host = walloc(sizeof *s->host);
-	*s->host = *host;		/* Struct copy */
+	WALLOC(s);
+	s->host = gnet_host_dup(host);
 	s->fifo = fifo_make();
 	s->ev_service = NULL;
 	s->can_deflate = can_deflate;
@@ -390,10 +391,10 @@ free_pmsg(gpointer item, gpointer unused_udata)
 static void
 servent_free(struct gservent *s)
 {
-	cq_cancel(callout_queue, &s->ev_service);
-	wfree(s->host, sizeof *s->host);
+	cq_cancel(&s->ev_service);
+	gnet_host_free(s->host);
 	fifo_free_all(s->fifo, free_pmsg, NULL);
-	wfree(s, sizeof *s);
+	WFREE(s);
 }
 
 /**
@@ -421,12 +422,12 @@ oob_record_hit(gpointer data, size_t len, gpointer udata)
  * @param token		the token for secure OOB
  */
 void
-oob_deliver_hits(struct gnutella_node *n, const gchar *muid, guint8 wanted,
-	const struct array *token)
+oob_deliver_hits(struct gnutella_node *n, const struct guid *muid,
+	guint8 wanted, const struct array *token)
 {
 	struct oob_results *r;
 	struct gservent *s;
-	gint deliver_count;
+	int deliver_count;
 	gboolean servent_created = FALSE;
 
 	g_assert(NODE_IS_UDP(n));
@@ -515,7 +516,7 @@ oob_deliver_hits(struct gnutella_node *n, const gchar *muid, guint8 wanted,
 	deliver_count = (wanted == 255) ? r->count : MIN(wanted, r->count);
 
 	if (GNET_PROPERTY(query_debug) || GNET_PROPERTY(udp_debug))
-		g_message("OOB query %s: host %s wants %d hit%s, delivering %d",
+		g_debug("OOB query %s: host %s wants %d hit%s, delivering %d",
 			guid_hex_str(r->muid), node_addr(n), wanted, wanted == 1 ? "" : "s",
 			deliver_count);
 
@@ -574,7 +575,7 @@ oob_pmsg_free(pmsg_t *mb, gpointer arg)
 		} else {
 
 			if (GNET_PROPERTY(query_debug) || GNET_PROPERTY(udp_debug))
-				g_message("OOB query %s, notified %s about %d hit%s",
+				g_debug("OOB query %s, notified %s about %d hit%s",
 					guid_hex_str(r->muid), gnet_host_to_string(&r->dest),
 					r->count, r->count == 1 ? "" : "s");
 
@@ -582,8 +583,7 @@ oob_pmsg_free(pmsg_t *mb, gpointer arg)
 			 * If we don't get any ACK back, we'll discard the results.
 			 */
 
-			r->ev_timeout = cq_insert(callout_queue, OOB_TIMEOUT_MS,
-					results_timeout, r);
+			r->ev_timeout = cq_main_insert(OOB_TIMEOUT_MS, results_timeout, r);
 			r->refcount++;
 		}
 	} else {
@@ -592,7 +592,7 @@ oob_pmsg_free(pmsg_t *mb, gpointer arg)
 		 */
 
 		if (GNET_PROPERTY(query_debug))
-			g_message("OOB query %s, previous LIME12/v2 #%d was dropped",
+			g_debug("OOB query %s, previous LIME12/v2 #%d was dropped",
 					guid_hex_str(r->muid), r->notify_requeued);
 
 		if (++r->notify_requeued < OOB_MAX_RETRY)
@@ -624,7 +624,7 @@ oob_send_reply_ind(struct oob_results *r)
 	pmsg_free(mb);
 
 	if (GNET_PROPERTY(query_debug) || GNET_PROPERTY(udp_debug))
-		g_message("OOB query %s, notifying %s about %d hit%s, try #%d",
+		g_debug("OOB query %s, notifying %s about %d hit%s, try #%d",
 			guid_hex_str(r->muid), gnet_host_to_string(&r->dest),
 			r->count, r->count == 1 ? "" : "s", r->notify_requeued);
 
@@ -643,7 +643,7 @@ oob_send_reply_ind(struct oob_results *r)
  */
 void
 oob_got_results(struct gnutella_node *n, GSList *files,
-	gint count, gboolean secure, gboolean ggep_h)
+	int count, gboolean secure, gboolean ggep_h)
 {
 	struct oob_results *r;
 	gnet_host_t to;
@@ -669,7 +669,7 @@ void
 oob_init(void)
 {
 	results_by_muid = g_hash_table_new(guid_hash, guid_eq);
-	servent_by_host = g_hash_table_new(host_hash, host_eq);
+	servent_by_host = g_hash_table_new(gnet_host_hash, gnet_host_eq);
 }
 
 /**
@@ -718,12 +718,10 @@ oob_shutdown(void)
 	oob_shutdown_running = TRUE;
 
 	g_hash_table_foreach(results_by_muid, free_oob_kv, NULL);
-	g_hash_table_destroy(results_by_muid);
-	results_by_muid = NULL;
+	gm_hash_table_destroy_null(&results_by_muid);
 
 	g_hash_table_foreach(servent_by_host, free_servent_kv, NULL);
-	g_hash_table_destroy(servent_by_host);
-	servent_by_host = NULL;
+	gm_hash_table_destroy_null(&servent_by_host);
 
 	g_assert(num_oob_records >= 0);
 	if (num_oob_records > 0)
