@@ -1,5 +1,5 @@
 /*
- * $Id: oob_proxy.h,v 1.3 2005/06/25 01:37:40 daichik Exp $
+ * $Id: oob_proxy.h 17703 2010-11-07 19:57:48Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -36,22 +36,25 @@
 #ifndef _core_oob_proxy_h_
 #define _core_oob_proxy_h_
 
-#include <glib.h>
+#include "common.h"
 
 /*
  * Public interface.
  */
 
+struct array;
 struct gnutella_node;
+struct guid;
 
 void oob_proxy_init(void);
 void oob_proxy_close(void);
 
-void oob_proxy_create(struct gnutella_node *n);
+gboolean oob_proxy_create(struct gnutella_node *n);
 gboolean oob_proxy_pending_results(
-	struct gnutella_node *n, gchar *muid, gint hits, gboolean udp_firewalled);
+	struct gnutella_node *n, const struct guid *muid,
+	int hits, gboolean udp_firewalled, const struct array *token);
 gboolean oob_proxy_got_results(struct gnutella_node *n, guint results);
-gboolean oob_proxy_muid_proxied(gchar *muid);
+const struct guid *oob_proxy_muid_proxied(const struct guid *muid);
 
 #endif /* _core_oob_proxy_h_ */
 
