@@ -1,5 +1,5 @@
 /*
- * $Id: pproxy.h 10923 2006-05-20 05:34:03Z cbiere $
+ * $Id: pproxy.h 17521 2010-10-17 18:08:04Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -36,9 +36,14 @@
 #ifndef _core_pproxy_h_
 #define _core_pproxy_h_
 
-#include <glib.h>
+#include "common.h"
 
 #include "if/core/pproxy.h"
+#include "lib/array.h"
+#include "lib/gnet_host.h"
+#include "lib/sequence.h"
+
+struct guid;
 
 /***
  *** Server side
@@ -49,14 +54,14 @@
  */
 struct pproxy {
 	struct gnutella_socket *socket;
-	gint error_sent;		/**< HTTP error code sent back */
+	int error_sent;		/**< HTTP error code sent back */
 	time_t last_update;
 
 	host_addr_t addr_v4;	/**< IPv4 of the requesting servent */
 	host_addr_t addr_v6;	/**< IPv6 of the requesting servent */
 	guint16 port;			/**< Port where GIV should be sent back */
-	gchar *user_agent;		/**< User-Agent string */
-	gchar *guid;			/**< GUID (atom) to which push should be sent */
+	const char *user_agent;/**< User-Agent string */
+	const struct guid *guid;/**< GUID (atom) to which push should be sent */
 	guint32 file_idx;		/**< File index to request (0 if none supplied) */
 	guint32 flags;
 	gpointer io_opaque;		/**< Opaque I/O callback information */
@@ -66,7 +71,7 @@ struct pproxy {
 
 void pproxy_add(struct gnutella_socket *s);
 void pproxy_remove(struct pproxy *pp,
-	const gchar *reason, ...) G_GNUC_PRINTF(2, 3);
+	const char *reason, ...) G_GNUC_PRINTF(2, 3);
 void pproxy_timer(time_t now);
 void pproxy_close(void);
 
@@ -75,14 +80,37 @@ void pproxy_close(void);
  ***/
 
 struct cproxy *cproxy_create(struct download *d,
-	const host_addr_t addr, guint16 port, gchar *guid, guint32 file_idx);
+	const host_addr_t addr, guint16 port, const struct guid *guid,
+	guint32 file_idx);
 void cproxy_free(struct cproxy *cp);
 void cproxy_reparent(struct download *d, struct download *cd);
 
-const gchar *build_push(size_t *size_ptr, guint8 ttl, guint8 hops,
-	const gchar *guid, host_addr_t addr_v4, host_addr_t addr_v6,
+struct array build_push(guint8 ttl, guint8 hops,
+	const struct guid *guid, host_addr_t addr_v4, host_addr_t addr_v6,
 	guint16 port, guint32 file_idx, gboolean supports_tls);
 
+/***
+ *** Push proxy set
+ ***/
+
+typedef struct pproxy_set pproxy_set_t;
+
+pproxy_set_t *pproxy_set_allocate(size_t max_proxies);
+void pproxy_set_free_null(pproxy_set_t **ps_ptr);
+gboolean pproxy_set_add(pproxy_set_t *ps, const host_addr_t addr, guint16 port);
+void pproxy_set_add_vec(pproxy_set_t *ps, const gnet_host_vec_t *vec);
+void pproxy_set_add_array(pproxy_set_t *ps,
+	gnet_host_t *proxies, int proxy_count);
+gboolean pproxy_set_remove(pproxy_set_t *ps,
+	const host_addr_t addr, guint16 port);
+size_t pproxy_set_count(const pproxy_set_t *ps);
+gboolean pproxy_set_older_than(const pproxy_set_t *ps, time_t t);
+void pproxy_set_foreach(const pproxy_set_t *ps, GFunc func, void *user_data);
+gnet_host_t *pproxy_set_head(const pproxy_set_t *ps);
+sequence_t *pproxy_set_sequence(const pproxy_set_t *ps);
+gnet_host_vec_t *pproxy_set_host_vec(const pproxy_set_t *ps);
+const gnet_host_t *pproxy_set_oldest(const pproxy_set_t *ps);
+
 #endif	/* _core_pproxy_h_ */
 
 /* vi: set ts=4 sw=4 cindent: */
