@@ -1,5 +1,5 @@
 /*
- * $Id: qhit.c,v 1.38 2005/11/21 21:12:27 rmanfredi Exp $
+ * $Id: qhit.c 19310 2011-06-25 22:59:41Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -35,33 +35,44 @@
 
 #include "common.h"
 
-RCSID("$Id: qhit.c,v 1.38 2005/11/21 21:12:27 rmanfredi Exp $");
+RCSID("$Id: qhit.c 19310 2011-06-25 22:59:41Z rmanfredi $")
 
-#include "qhit.h"
-#include "gnutella.h"
+#include "gtk-gnutella.h"
+
+#include "bsched.h"
+#include "dmesh.h"		/* For dmesh_fill_alternate() */
 #include "ggep.h"
 #include "ggep_type.h"
 #include "gmsg.h"
-#include "share.h"
+#include "gnutella.h"
 #include "nodes.h"
-#include "bsched.h"
-#include "dmesh.h"		/* For dmesh_fill_alternate() */
-#include "uploads.h"	/* For count_uploads */
+#include "qhit.h"
 #include "settings.h"	/* For listen_ip() */
+#include "share.h"
+#include "ipp_cache.h"
+#include "uploads.h"	/* For count_uploads */
+#include "sockets.h"	/* For socket_listen_port() */
 
 #include "if/gnet_property_priv.h"
+#include "if/core/main.h"			/* For main_get_build() */
 
+#include "lib/array.h"
 #include "lib/getdate.h"
 #include "lib/endian.h"
 #include "lib/misc.h"
+#include "lib/sequence.h"
 #include "lib/tm.h"
 #include "lib/override.h"			/* Must be the last header included */
 
-#define QHIT_SIZE_THRESHOLD	2016	/**< Flush query hits larger than this */
-#define QHIT_MAX_RESULTS	255		/**< Maximum amount of hits in a query hit */
-#define QHIT_MAX_ALT		15		/**< Send out 15 alt-locs per entry, max */
-#define QHIT_MAX_PROXIES	5		/**< Send out 5 push-proxies at most */
+/*
+ * NOTE: LimeWire doesn't like more than 10 results per QHIT packet or
+ *       more than 10 alt-locs per QHIT packet.
+ */
+#define QHIT_MAX_RESULTS	10		/**< Maximum amount of hits in a query hit */
+#define QHIT_MAX_ALT		10		/**< Send out 10 alt-locs per entry, max */
+#define QHIT_MAX_PROXIES	8		/**< Send out 8 push-proxies at most */
 #define QHIT_MAX_GGEP		512		/**< Allocated room for trailing GGEP */
+#define QHIT_SIZE_THRESHOLD	2016	/**< Flush query hits larger than this */
 
 /*
  * Minimal trailer length is our code NAME, the open flags, and the GUID.
@@ -78,15 +89,17 @@ RCSID("$Id: qhit.c,v 1.38 2005/11/21 21:12:27 rmanfredi Exp $");
  */
 
 struct found_struct {
-	gchar data[64 * 1024];		/**< data */
+	char data[64 * 1024];		/**< data */
 	size_t pos;					/**< current write position */
 	size_t files;				/**< amount of file entries */
 	size_t max_size;			/**< max query hit size */
-	gboolean use_ggep_h;		/**< whether to use GGEP "H" to send SHA1 */
-	const gchar *muid;			/**< the MUID to put in all query hits */
+	const struct guid *muid;	/**< the MUID to put in all query hits */
+	const struct array *token;	/**< Optional secure OOB token */
+	GHashTable *ht;				/**< Records file indices and SHA1 atoms */
 	qhit_process_t process;		/**< processor once query hit is built */
 	gpointer udata;				/**< processor argument */
 	gboolean open;				/**< Set if found_open() was used */
+	gboolean use_ggep_h;        /**< whether to use GGEP "H" to send SHA-1 */
 };
 
 static struct found_struct *
@@ -114,13 +127,13 @@ found_add_files(size_t n)
 	found_get()->files += n;
 }
 
-static gboolean 
+static gboolean
 found_ggep_h(void)
 {
 	return found_get()->use_ggep_h;
 }
 
-static gchar *
+static char *
 found_open(void)
 {
 	struct found_struct *f = found_get();
@@ -173,9 +186,8 @@ found_write(gconstpointer data, size_t length)
 	g_assert(length <= INT_MAX);
 	g_assert(!f->open);
 
-	if (length > sizeof f->data - f->pos) {
+	if (length > sizeof f->data - f->pos)
 		return FALSE;
-	}
 
 	g_assert(f->pos < sizeof f->data);
 	memcpy(&f->data[f->pos], data, length);
@@ -187,45 +199,56 @@ found_write(gconstpointer data, size_t length)
 static void
 found_set_header(void)
 {
-	static const size_t head_size = sizeof(struct gnutella_header);
 	struct found_struct *f = found_get();
-	struct gnutella_header *packet_head;
-	struct gnutella_search_results_out *search_head;
+	gnutella_msg_search_results_t *msg;
 	guint32 connect_speed;		/* Connection speed, in kbits/s */
 	size_t len;
 
 	g_assert(!f->open);
-	g_assert(f->pos >= head_size);
-	len = f->pos - head_size;
+	g_assert(f->pos >= GTA_HEADER_SIZE);
+	len = f->pos - GTA_HEADER_SIZE;
 	g_assert(len < sizeof f->data);
 
-	packet_head = (struct gnutella_header *) f->data;
-	packet_head->ttl = 1;		/* Overriden later if sending inbound */
-	packet_head->hops = 0;
-	memcpy(f->data, f->muid, 16);
+	msg = (gnutella_msg_search_results_t *) f->data;
 
-	packet_head->function = GTA_MSG_SEARCH_RESULTS;
-	WRITE_GUINT32_LE(len, packet_head->size);
+	{
+		gnutella_header_t *header = gnutella_msg_search_results_header(msg);
 
-	search_head = (gpointer) &f->data[head_size];
-	search_head->num_recs = f->files; /* One byte */
+		gnutella_header_set_muid(header, f->muid);
+		gnutella_header_set_function(header, GTA_MSG_SEARCH_RESULTS);
+		/* The TTL is overriden later if sending inbound */
+		gnutella_header_set_ttl(header, 1);
+		gnutella_header_set_hops(header, 0);
+		gnutella_header_set_size(header, len);
+	}
+
+	gnutella_msg_search_results_set_num_recs(msg, f->files); /* One byte */
 
 	/*
 	 * Compute connection speed dynamically if requested.
 	 */
 
-	connect_speed = connection_speed;
-	if (compute_connection_speed) {
-		connect_speed = max_uploads == 0 ?
-			0 : (MAX(bsched_avg_bps(bws.out), bsched_bwps(bws.out)) * 8 / 1024);
-		if (max_uploads > 0 && connect_speed == 0)
-			connect_speed = 32;		/* No b/w limit set and no traffic yet */
+	connect_speed = GNET_PROPERTY(connection_speed);
+	if (GNET_PROPERTY(compute_connection_speed)) {
+		if (GNET_PROPERTY(max_uploads) > 0) {
+			connect_speed = bsched_avg_bps(BSCHED_BWS_OUT);
+			connect_speed = MAX(connect_speed,
+								bsched_bw_per_second(BSCHED_BWS_OUT));
+			connect_speed /= 1024 / 8;
+			if (connect_speed == 0) {
+				/* No b/w limit set and no traffic yet */
+				connect_speed = 32;
+			}
+		} else {
+			connect_speed = 0;
+		}
 	}
-	connect_speed /= MAX(1, max_uploads);	/* Upload speed expected per slot */
+	/* Upload speed expected per slot */
+	connect_speed /= MAX(1, GNET_PROPERTY(max_uploads));
 
-	WRITE_GUINT16_LE(listen_port, search_head->host_port);
-	WRITE_GUINT32_BE(host_addr_ipv4(listen_addr()), search_head->host_ip);
-	WRITE_GUINT32_LE(connect_speed, search_head->host_speed);
+	gnutella_msg_search_results_set_host_port(msg, socket_listen_port());
+	gnutella_msg_search_results_set_host_ip(msg, host_addr_ipv4(listen_addr()));
+	gnutella_msg_search_results_set_host_speed(msg, connect_speed);
 }
 
 static void
@@ -233,8 +256,7 @@ found_clear(void)
 {
 	struct found_struct *f = found_get();
 
-	f->pos = sizeof(struct gnutella_header) +
-		sizeof(struct gnutella_search_results_out);
+	f->pos = GTA_HEADER_SIZE + sizeof(gnutella_search_results_t);
 	g_assert(f->pos > 0 && f->pos < sizeof f->data);
 	f->files = 0;
 	f->open = FALSE;
@@ -249,15 +271,24 @@ found_process(void)
 	f->process(f->data, f->pos, f->udata);
 }
 
+static const struct array *
+found_token(void)
+{
+	struct found_struct *f = found_get();
+	return f->token;
+}
+
 static void
-found_init(size_t max_size, const gchar *xuid, gboolean ggep_h,
-	qhit_process_t proc, gpointer udata)
+found_init(size_t max_size, const struct guid *xuid, gboolean ggep_h,
+	qhit_process_t proc, gpointer udata, const struct array *token)
 {
 	struct found_struct *f = found_get();
 
 	g_assert(max_size <= INT_MAX);
 	g_assert(xuid != NULL);
 	g_assert(proc != NULL);
+	g_assert(token != NULL);
+	g_assert(NULL == f->ht);
 
 	f->max_size = max_size;
 	f->muid = xuid;
@@ -265,6 +296,32 @@ found_init(size_t max_size, const gchar *xuid, gboolean ggep_h,
 	f->process = proc;
 	f->udata = udata;
 	f->open = FALSE;
+	f->token = token;
+	f->ht = g_hash_table_new(pointer_hash_func, NULL);
+}
+
+static void
+found_done(void)
+{
+	struct found_struct *f = found_get();
+
+	gm_hash_table_destroy_null(&f->ht);
+}
+
+static gboolean
+found_contains(const void *key)
+{
+	struct found_struct *f = found_get();
+
+	return gm_hash_table_contains(f->ht, key);
+}
+
+static void
+found_insert(const void *key)
+{
+	struct found_struct *f = found_get();
+
+	gm_hash_table_insert_const(f->ht, key, NULL);
 }
 
 static time_t release_date;
@@ -275,14 +332,15 @@ static time_t release_date;
 static void
 qhit_send_node(gpointer data, size_t len, gpointer udata)
 {
-	gnutella_node_t *n = (gnutella_node_t *) udata;
-	struct gnutella_header *packet_head = (struct gnutella_header *) data;
+	gnutella_node_t *n = udata;
+	gnutella_header_t *packet_head = data;
+	guint ttl;
 
-	if (dbg > 3) {
-		g_message("flushing query hit (%d entr%s, %d bytes sofar) to %s\n",
-			(gint) found_file_count(),
+	if (GNET_PROPERTY(dbg) > 3) {
+		g_debug("flushing query hit (%u entr%s, %u bytes sofar) to %s",
+			(guint) found_file_count(),
 			found_file_count() == 1 ? "y" : "ies",
-			(gint) found_size(),
+			(guint) found_size(),
 			node_addr(n));
 	}
 
@@ -297,30 +355,39 @@ qhit_send_node(gpointer data, size_t len, gpointer udata)
 	 *			 --RAM, 02/02/2001
 	 */
 
-	if (n->header.hops == 0) {
-		g_warning
-			("search_request(): hops=0, bug in route_message()?\n");
-		n->header.hops++;	/* Can't send message with TTL=0 */
+	if (gnutella_header_get_hops(&n->header) == 0) {
+		g_warning("qhit_send_node(): hops=0, bug in route_message()?");
+		/* Can't send message with TTL=0 */
+		gnutella_header_set_hops(&n->header, 1);
 	}
 
-	packet_head->ttl = MIN((guint) n->header.hops + 5, hard_ttl_limit);
+	ttl = gnutella_header_get_hops(&n->header) + 5U;
+	ttl = MIN(ttl, GNET_PROPERTY(hard_ttl_limit));
+	gnutella_header_set_ttl(packet_head, ttl);
 
 	gmsg_sendto_one(n, data, len);
 }
 
+static void
+qhit_log_ggep_write_failure(const char *id)
+{
+	if (GNET_PROPERTY(qhit_debug))
+		g_warning("QHIT could not write GGEP \"%s\" extension", id);
+}
+
 /**
  * Flush pending search request to the network.
  */
 static void
 flush_match(void)
 {
-	gchar trailer[7];
-	gint ggep_len = 0;			/* Size of the GGEP trailer */
+	char trailer[7];
+	int ggep_len = 0;			/* Size of the GGEP trailer */
 	ggep_stream_t gs;
-	gchar *trailer_start;
+	char *trailer_start;
 
 	/*
-	 * Build Gtk-Gnutella trailer.
+	 * Build gtk-gnutella trailer.
 	 * It is compatible with BearShare's one in the "open data" section.
 	 */
 
@@ -329,11 +396,11 @@ flush_match(void)
 	trailer[5] = 0x04 | 0x08 | 0x20;	/* Valid flags we set */
 	trailer[6] = 0x01;				/* Our flags (valid firewall bit) */
 
-	if (ul_running >= max_uploads)
+	if (GNET_PROPERTY(ul_running) >= GNET_PROPERTY(max_uploads))
 		trailer[6] |= 0x04;			/* Busy flag */
-	if (total_uploads > 0)
+	if (GNET_PROPERTY(total_uploads) > 0)
 		trailer[6] |= 0x08;			/* One file uploaded, at least */
-	if (is_firewalled)
+	if (GNET_PROPERTY(is_firewalled))
 		trailer[5] |= 0x01;			/* Firewall bit set in enabling byte */
 
 	/*
@@ -358,18 +425,19 @@ flush_match(void)
 	ggep_stream_init(&gs, found_open(), QHIT_MAX_GGEP);
 
 	/*
-	 * Build the "GTKGV1" GGEP extension.
+	 * Build the "GTKGV" GGEP extension.
 	 */
 
 	{
 		guint8 major = GTA_VERSION;
 		guint8 minor = GTA_SUBVERSION;
-		gchar *revp = GTA_REVCHAR;
+		const char *revp = GTA_REVCHAR;
 		guint8 revchar = (guint8) revp[0];
 		guint8 patch;
 		guint32 release;
 		guint32 date = release_date;
-		guint32 start;
+		guint32 build;
+		guint8 version = 0;		/* This is GTKGV version 0 */
 		gboolean ok;
 
 #ifdef GTA_PATCHLEVEL
@@ -378,21 +446,33 @@ flush_match(void)
 		patch = 0;
 #endif
 
-		WRITE_GUINT32_BE(date, &release);
-		WRITE_GUINT32_BE(start_stamp, &start);
+		poke_be32(&release, date);
+		poke_be32(&build, main_get_build());
 
 		ok =
-			ggep_stream_begin(&gs, GGEP_NAME(GTKGV1), 0) &&
+			ggep_stream_begin(&gs, GGEP_NAME(GTKGV), 0) &&
+			ggep_stream_write(&gs, &version, 1) &&
 			ggep_stream_write(&gs, &major, 1) &&
 			ggep_stream_write(&gs, &minor, 1) &&
 			ggep_stream_write(&gs, &patch, 1) &&
 			ggep_stream_write(&gs, &revchar, 1) &&
 			ggep_stream_write(&gs, &release, 4) &&
-			ggep_stream_write(&gs, &start, 4) &&
+			ggep_stream_write(&gs, &build, 4) &&
 			ggep_stream_end(&gs);
 
 		if (!ok)
-			g_warning("could not write GGEP \"GTKGV1\" extension in query hit");
+			qhit_log_ggep_write_failure("GTKGV");
+	}
+
+	{
+		const struct array *token = found_token();
+		
+		if (
+			token->data &&
+			!ggep_stream_pack(&gs, GGEP_NAME(SO), token->data, token->size, 0)
+		) {
+			qhit_log_ggep_write_failure("SO");
+		}
 	}
 
 	/*
@@ -400,37 +480,62 @@ flush_match(void)
 	 * our current push proxies.  Prepare payload in `proxies'.
 	 */
 
-	if (is_firewalled) {
-		GSList *nodes = node_push_proxies();
+	if (GNET_PROPERTY(is_firewalled)) {
+		sequence_t *seq = node_push_proxies();
 
-		if (nodes != NULL) {
-			GSList *l;
-			gint count;
-			gchar proxy[6];
+		if (!sequence_is_empty(seq)) {
+			guchar tls_bytes[(QHIT_MAX_PROXIES + 7) / 8];
+			guint tls_index, tls_length;
+			sequence_iter_t *iter;
 			gboolean ok;
 
 			ok = ggep_stream_begin(&gs, GGEP_NAME(PUSH), 0);
+			ZERO(&tls_bytes);
+			tls_index = 0;
+			tls_length = 0;
 
-			for (
-				l = nodes, count = 0;
-				ok && l && count < QHIT_MAX_PROXIES;
-				l = g_slist_next(l), count++
-			) {
-				struct gnutella_node *n = l->data;
+			/*
+			 * We iterate backwards to get the most stable of our push proxies,
+			 * namely those to which we've been connected for the longest time.
+			 */
 
-				if (NET_TYPE_IPV4 == host_addr_net(n->proxy_addr)) {
-					WRITE_GUINT32_BE(host_addr_ipv4(n->proxy_addr), &proxy[0]);
-					WRITE_GUINT16_LE(n->proxy_port, &proxy[4]);
-					ok = ggep_stream_write(&gs, proxy, sizeof proxy);
+			iter = sequence_backward_iterator(seq, TRUE);
+
+			while (ok && sequence_iter_has_previous(iter)) {
+				const gnet_host_t *host = sequence_iter_previous(iter);
+				host_addr_t addr = gnet_host_get_addr(host);
+				guint16 port = gnet_host_get_port(host);
+				char proxy[6];
+
+				if (!host_addr_is_ipv4(addr))
+					continue;
+
+				poke_be32(&proxy[0], host_addr_ipv4(addr));
+				poke_le16(&proxy[4], port);
+				ok = ggep_stream_write(&gs, proxy, sizeof proxy);
+				if (tls_cache_lookup(addr, port)) {
+					tls_bytes[tls_index >> 3] |= 0x80U >> (tls_index & 7);
+					tls_length = (tls_index >> 3) + 1;
 				}
+				tls_index++;
+				if (tls_index >= QHIT_MAX_PROXIES)
+					break;
 			}
 
 			ok = ok && ggep_stream_end(&gs);
+			sequence_iterator_release(&iter);
 
 			if (!ok)
-				g_warning("could not write GGEP \"PUSH\" extension "
-					"in query hit");
+				qhit_log_ggep_write_failure("PUSH");
+
+			if (ok && tls_length > 0) {
+				ok = ggep_stream_pack(&gs, GGEP_NAME(PUSH_TLS),
+						tls_bytes, tls_length, 0);
+				if (!ok)
+					qhit_log_ggep_write_failure("PUSH_TLS");
+			}
 		}
+		sequence_release(&seq);
 	}
 
 	/*
@@ -438,45 +543,56 @@ flush_match(void)
 	 * server's hostname.
 	 */
 
-	if (!is_firewalled && give_server_hostname && '\0' != server_hostname[0]) {
+	if (
+		!GNET_PROPERTY(is_firewalled) &&
+		GNET_PROPERTY(give_server_hostname) &&
+		!is_null_or_empty(GNET_PROPERTY(server_hostname))
+	) {
 		gboolean ok;
 
 		ok = ggep_stream_pack(&gs, GGEP_NAME(HNAME),
-				server_hostname, strlen(server_hostname), 0);
+				GNET_PROPERTY(server_hostname),
+				strlen(GNET_PROPERTY(server_hostname)),
+				0);
 
 		if (!ok)
-			g_warning("could not write GGEP \"HNAME\" extension "
-				"in query hit");
+			qhit_log_ggep_write_failure("HNAME");
 	}
 
-#ifdef USE_IPV6
-	if (
-		NET_TYPE_IPV6 == host_addr_net(listen_addr()) &&
-		!host_addr_can_convert(listen_addr(), NET_TYPE_IPV4)
-	) {
-		const host_addr_t addr = listen_addr();
-		const guint8 *ipv6 = host_addr_ipv6(&addr);
+	{
+		const host_addr_t addr = listen_addr6();
 
-		if (!ggep_stream_pack(&gs, GGEP_GTKG_NAME(IPV6), ipv6, 16, 0))
-			g_warning("could not write GGEP \"GTKG.IPV6\" extension "
-				"into query hit");
+		if (host_addr_is_ipv6(addr)) {
+			const guint8 *ipv6 = host_addr_ipv6(&addr);
+
+			if (!ggep_stream_pack(&gs, GGEP_GTKG_NAME(IPV6), ipv6, 16, 0))
+				qhit_log_ggep_write_failure("GTKG.IPV6");
+		}
 	}
-#endif /* USE_IPV6 */
 
-#ifdef HAS_GNUTLS
-	if (!ggep_stream_pack(&gs, GGEP_GTKG_NAME(TLS), NULL, 0, 0))
-		g_warning("could not write GGEP \"GTKG.TLS\" extension into query hit");
-#endif /* HAS_GNUTLS */
+	if (tls_enabled()) {
+		if (!ggep_stream_pack(&gs, GGEP_NAME(TLS), NULL, 0, 0))
+			qhit_log_ggep_write_failure("TLS");
 
+		/*
+		 * FIXME TODO: Drop sending GTKG.TLS after 0.96.7 is out, only
+		 * sending the now standardized "TLS" extension which GTKG has been
+		 * understanding since 0.96.6.
+		 *		--RAM, 2009-10-24
+		 */
+
+		if (!ggep_stream_pack(&gs, GGEP_GTKG_NAME(TLS), NULL, 0, 0))
+			qhit_log_ggep_write_failure("GTKG.TLS");
+	}
 
 	/*
 	 * Advertise the Browse Host extension in the results if the feature is
 	 * enabled.
      */
 
-	if (browse_host_enabled) {
+	if (GNET_PROPERTY(browse_host_enabled)) {
 		if (!ggep_stream_pack(&gs, GGEP_NAME(BH), NULL, 0, 0))
-			g_warning("could not write GGEP \"BH\" extension into query hit");
+			qhit_log_ggep_write_failure("BH");
 	}
 
 	ggep_len = ggep_stream_close(&gs);
@@ -489,7 +605,7 @@ flush_match(void)
 	 * Store the GUID in the last 16 bytes of the query hit.
 	 */
 
-	if (!found_write(servent_guid, 16))
+	if (!found_write(GNET_PROPERTY(servent_guid), GUID_RAW_SIZE))
 		goto failure;
 
 	found_set_header();
@@ -498,7 +614,9 @@ flush_match(void)
 
 failure:
 
-	g_warning("Created query hit was too big, discarding");
+	if (GNET_PROPERTY(qhit_debug))
+		g_warning("QHIT message was too big, discarding");
+
 	found_clear();
 }
 
@@ -509,23 +627,37 @@ failure:
  * lack of space.
  */
 static gboolean
-add_file(const struct shared_file *sf)
+add_file(const shared_file_t *sf)
 {
-	size_t needed = 8 + 2 + sf->name_nfc_len;		/* size of hit entry */
 	gboolean sha1_available;
 	gnet_host_t hvec[QHIT_MAX_ALT];
-	gint hcnt = 0;
+	int hcnt = 0;
 	guint32 fs32, fs32_le, idx_le;
-	gint ggep_len;
+	int ggep_len;
 	gboolean ok;
 	ggep_stream_t gs;
-	size_t left;
+	size_t left, needed;
 	gpointer start;
+	gboolean is_partial;
+	guint32 file_index;
 
-	g_assert(sf->fi == NULL);	/* Cannot match partially downloaded files */
+	is_partial = shared_file_is_partial(sf);
+	needed = 8 + 2 + shared_file_name_nfc_len(sf);	/* size of hit entry */
+	sha1_available = sha1_hash_available(sf);
 
-	sha1_available = SHARE_F_HAS_DIGEST ==
-		(sf->flags & (SHARE_F_HAS_DIGEST | SHARE_F_RECOMPUTING));
+	/*
+	 * Make sure we never insert duplicate indices in a query hit.
+	 *
+	 * This code assumes there will never be any collision between shared
+	 * file indices and pointers to SHA1, which will always hold fortunately
+	 * in real life.
+	 */
+
+	file_index = shared_file_index(sf);
+
+	g_assert(!found_contains(uint_to_pointer(file_index)));
+
+	found_insert(uint_to_pointer(file_index));
 
 	/*
 	 * In case we emit the SHA1 as a GGEP "H", we'll grow the buffer
@@ -538,8 +670,25 @@ add_file(const struct shared_file *sf)
 	 */
 
 	if (sha1_available) {
+		const sha1_t *sha1 = shared_file_sha1(sf);
+
+		/*
+		 * They can share twice or more identical files.  Make sure we only
+		 * include each SHA1 once in the query hits we return for a query:
+		 * having multiple entries would waste bandwidth anyway.
+		 */
+
+		if (found_contains(sha1)) {
+			if (GNET_PROPERTY(qhit_debug))
+				g_warning("QHIT not including SHA1 %s twice",
+					sha1_base32(sha1));
+			return TRUE;		/* Entry consumed, but not included */
+		}
+
+		found_insert(sha1);		/* SHA1 are atoms, address is unique */
+
 		needed += 9 + SHA1_BASE32_SIZE;
-		hcnt = dmesh_fill_alternate(sf->sha1_digest, hvec, QHIT_MAX_ALT);
+		hcnt = dmesh_fill_alternate(sha1, hvec, G_N_ELEMENTS(hvec));
 		needed += hcnt * 6 + 6;
 	}
 
@@ -549,7 +698,7 @@ add_file(const struct shared_file *sf)
 
 	if (
 		found_size() + needed + QHIT_MIN_TRAILER_LEN
-			> search_answers_forward_size
+			> GNET_PROPERTY(search_answers_forward_size)
 	)
 		return FALSE;
 
@@ -566,15 +715,15 @@ add_file(const struct shared_file *sf)
 	 * use the "LF" GGEP extension to hold the real size.
 	 */
 
-	fs32 = sf->file_size >= (1U << 31) ? ~0U : sf->file_size;
+	fs32 = shared_file_size(sf) >= (1U << 31) ? ~0U : shared_file_size(sf);
 
-	WRITE_GUINT32_LE(sf->file_index, &idx_le);
+	poke_le32(&idx_le, file_index);
 	if (!found_write(&idx_le, sizeof idx_le))
 		return FALSE;
-	WRITE_GUINT32_LE(fs32, &fs32_le);
+	poke_le32(&fs32_le, fs32);
 	if (!found_write(&fs32_le, sizeof fs32_le))
 		return FALSE;
-	if (!found_write(sf->name_nfc, sf->name_nfc_len))
+	if (!found_write(shared_file_name_nfc(sf), shared_file_name_nfc_len(sf)))
 		return FALSE;
 
 	/* Position equals the next byte to be written to */
@@ -591,13 +740,12 @@ add_file(const struct shared_file *sf)
 	 */
 
 	if (sha1_available && !found_ggep_h()) {
-		static const gchar urnsha1[] = "urn:sha1:";
-		const gchar *b32 = sha1_base32(sf->sha1_digest);
-		/* Good old way: ASCII URN */
+		const struct sha1 * const sha1 = shared_file_sha1(sf);
 
-		if (!found_write(urnsha1, CONST_STRLEN(urnsha1)))
+		/* Good old way: ASCII URN */
+		if (!found_write(sha1_to_urn_string(sha1), SHA1_URN_LENGTH))
 			return FALSE;
-		if (!found_write(b32, SHA1_BASE32_SIZE))
+		if (!found_write("\x1c", 1))
 			return FALSE;
 	}
 
@@ -610,21 +758,61 @@ add_file(const struct shared_file *sf)
 	ggep_stream_init(&gs, start, left);
 
 	/*
-	 * Emit the SHA1 as GGEP "H" if they said they understand it.
+	 * If we matched a partial file, let them know (unless the file is
+	 * being seeded, in which case it is really complete).
+	 *
+	 * For now we don't emit the available ranges (need to build the tree
+	 * of 1 KiB blocks and send numbers of the highest node in the tree
+	 * encompassing an available chunk) in PR0, PR1, PR2, PR3 or PR4 keys.
+	 *
+	 * We just emit the "PRU" key, signaling that it's a partial result
+	 * and that its data is still unverified (since we don't verify available
+	 * chunks using the TTH for now).
+	 *		--RAM, 2011-05-15
+	 */
+
+	if (is_partial && !shared_file_is_finished(sf)) {
+		ok = ggep_stream_pack(&gs, GGEP_NAME(PRU), NULL, 0, GGEP_W_COBS);
+		if (!ok)
+			qhit_log_ggep_write_failure("PRU");
+	}
+
+	/*
+	 * Emit the SHA1 as GGEP "H" if they said they understand it. The modern
+	 * way is GGEP "H" for binary URN but only gtk-gnutella implements it.
 	 */
 
 	if (sha1_available && found_ggep_h()) {
-		/* Modern way: GGEP "H" for binary URN */
-		guint8 type = GGEP_H_SHA1;
+		const struct sha1 * const sha1 = shared_file_sha1(sf);
+		const struct tth * const tth = shared_file_tth(sf);
+		const guint8 type = tth ? GGEP_H_BITPRINT : GGEP_H_SHA1;
 
 		ok =
 			ggep_stream_begin(&gs, GGEP_NAME(H), GGEP_W_COBS) &&
 			ggep_stream_write(&gs, &type, 1) &&
-			ggep_stream_write(&gs, sf->sha1_digest, SHA1_RAW_SIZE) &&
+			ggep_stream_write(&gs, sha1->data, SHA1_RAW_SIZE) &&
+			(tth ? ggep_stream_write(&gs, tth->data, TTH_RAW_SIZE) : TRUE) &&
 			ggep_stream_end(&gs);
 
 		if (!ok)
-			g_warning("could not write GGEP \"H\" extension in query hit");
+			qhit_log_ggep_write_failure("H");
+	}
+
+	/*
+	 * First LimeWire emitted TTHs as plain text urn:ttroot:<base32 TTH>.
+	 * Now they are still unaware of GGEP "H" but emit GGEP "TT" with the
+	 * hash in binary form.
+	 */
+
+	if (sha1_available && !found_ggep_h()) {
+		const struct tth * const tth = shared_file_tth(sf);
+
+		if (tth) {
+			ok = ggep_stream_pack(&gs,
+						GGEP_NAME(TT), tth->data, TTH_RAW_SIZE, GGEP_W_COBS);
+			if (!ok)
+				qhit_log_ggep_write_failure("TT");
+		}
 	}
 
 	/*
@@ -633,16 +821,16 @@ add_file(const struct shared_file *sf)
 	 */
 
 	if (fs32 == ~0U) {
-		gchar buf[sizeof(guint64)];
-		gint len;
+		char buf[sizeof(guint64)];
+		int len;
 
-		len = ggept_lf_encode(sf->file_size, buf);
+		len = ggept_filesize_encode(shared_file_size(sf), buf);
 
-		g_assert(len > 0 && len <= (gint) sizeof buf);
+		g_assert(len > 0 && UNSIGNED(len) <= sizeof buf);
 
 		ok = ggep_stream_pack(&gs, GGEP_NAME(LF), buf, len, GGEP_W_COBS);
 		if (!ok)
-			g_warning("could not write GGEP \"LF\" extension in query hit");
+			qhit_log_ggep_write_failure("LF");
 	}
 
 	/*
@@ -651,26 +839,84 @@ add_file(const struct shared_file *sf)
 	 */
 
 	if (hcnt > 0) {
-		gchar alt[6];
-		gint i;
+		guchar tls_bytes[(G_N_ELEMENTS(hvec) + 7) / 8];
+		guint tls_index, tls_length;
+		int i;
 
 		g_assert(hcnt <= QHIT_MAX_ALT);
+		ZERO(&tls_bytes);
+		tls_index = 0;
+		tls_length = 0;
 
 		ok = ggep_stream_begin(&gs, GGEP_NAME(ALT), GGEP_W_COBS);
 
 		for (i = 0; ok && i < hcnt; i++) {
+			host_addr_t addr;
+			guint16 port;
+			char alt[6];
+
 			g_assert(start == gs.outbuf);
-			if (NET_TYPE_IPV4 == host_addr_net(hvec[i].addr)) {
-				WRITE_GUINT32_BE(host_addr_ipv4(hvec[i].addr), &alt[0]);
-				WRITE_GUINT16_LE(hvec[i].port, &alt[4]);
-				ok = ggep_stream_write(&gs, alt, sizeof alt);
+			if (!gnet_host_is_ipv4(&hvec[i]))
+				continue;
+
+			addr = gnet_host_get_addr(&hvec[i]);
+			port = gnet_host_get_port(&hvec[i]);
+			poke_be32(&alt[0], host_addr_ipv4(addr));
+			poke_le16(&alt[4], port);
+			ok = ggep_stream_write(&gs, &alt, sizeof alt);
+
+			if (tls_cache_lookup(addr, port)) {
+				tls_bytes[tls_index >> 3] |= 0x80U >> (tls_index & 7);
+				tls_length = (tls_index >> 3) + 1;
 			}
+			tls_index++;
 		}
 
 		ok = ok && ggep_stream_end(&gs);
 
 		if (!ok)
-			g_warning("could not write GGEP \"ALT\" extension in query hit");
+			qhit_log_ggep_write_failure("ALT");
+
+		if (ok && tls_length > 0) {
+			ok = ggep_stream_pack(&gs, GGEP_NAME(ALT_TLS),
+					tls_bytes, tls_length, GGEP_W_COBS);
+			if (!ok)
+				qhit_log_ggep_write_failure("ALT_TLS");
+		}
+	}
+
+	{
+		const char *rp = shared_file_relative_path(sf);
+		
+		if (rp) {
+			ok = ggep_stream_pack(&gs, GGEP_NAME(PATH), rp, strlen(rp), 0);
+			if (!ok)
+				qhit_log_ggep_write_failure("PATH");
+		}
+	}
+
+	{
+		time_t mtime;	
+
+		/* FIXME: Create time != modification time */
+		mtime = shared_file_modification_time(sf);
+		if ((time_t) -1 != mtime) {
+			char buf[sizeof(guint64)];
+			int len;
+
+			/*
+			 * Suppress negative values (if time_t is signed) as this would
+			 * be interpreted as a date far in this future.
+			 */
+			mtime = MAX(0, mtime);
+
+			len = ggept_ct_encode(mtime, buf);
+			g_assert(UNSIGNED(len) <= sizeof buf);
+
+			ok = ggep_stream_pack(&gs, GGEP_NAME(CT), buf, len, GGEP_W_COBS);
+			if (!ok)
+				qhit_log_ggep_write_failure("CT");
+		}
 	}
 
 	/*
@@ -709,8 +955,6 @@ add_file(const struct shared_file *sf)
  *
  * @param max_size the maximum size in bytes of individual query hits
  *
- * @param use_ggep_h whether GGEP "H" can be used to send the SHA1 of files
- *
  * @param muid is the MUID that should be put in all the generated hits.
  * This must point to a memory location that is guaranteed to stay accurate
  * during all the processing.
@@ -723,12 +967,12 @@ add_file(const struct shared_file *sf)
  * have to be sent out-of-bound
  */
 static void
-found_reset(size_t max_size, const gchar *muid,
-	gboolean use_ggep_h, qhit_process_t process, gpointer udata)
+found_reset(size_t max_size, const struct guid *muid, gboolean ggep_h,
+	qhit_process_t process, gpointer udata, const struct array *token)
 {
 	g_assert(process != NULL);
 	g_assert(max_size <= INT_MAX);
-	found_init(max_size, muid, use_ggep_h, process, udata);
+	found_init(max_size, muid, ggep_h, process, udata, token);
 	found_clear();
 }
 
@@ -740,15 +984,13 @@ found_reset(size_t max_size, const gchar *muid,
  * @param files			the list of shared_file_t entries that make up results
  * @param count			the amount of results
  * @param muid			the query's MUID
- * @param use_ggep_h	whether GGEP "H" can be used to send the SHA1 of files
  */
 void
-qhit_send_results(
-	struct gnutella_node *n, GSList *files, gint count,
-	const gchar *muid, gboolean use_ggep_h)
+qhit_send_results(struct gnutella_node *n, GSList *files, int count,
+	const struct guid *muid, gboolean ggep_h)
 {
 	GSList *sl;
-	gint sent = 0;
+	int sent = 0;
 
 	/*
 	 * We can't use n->header.muid as the query's MUID but must rely on the
@@ -758,13 +1000,14 @@ qhit_send_results(
 	 * to forward to other nodes).
 	 */
 
-	found_reset(QHIT_SIZE_THRESHOLD, muid, use_ggep_h, qhit_send_node, n);
+	found_reset(QHIT_SIZE_THRESHOLD, muid, ggep_h, qhit_send_node, n,
+		&zero_array);
 
 	for (sl = files; sl; sl = g_slist_next(sl)) {
-		shared_file_t *sf = (shared_file_t *) sl->data;
+		shared_file_t *sf = sl->data;
 		if (add_file(sf))
 			sent++;
-		shared_file_unref(sf);
+		shared_file_unref(&sf);
 	}
 
 	if (0 != found_file_count())	/* Still some unflushed results */
@@ -772,8 +1015,10 @@ qhit_send_results(
 
 	g_slist_free(files);
 
-	if (dbg > 3)
-		g_message("sent %d/%d hits to %s\n", sent, count, node_addr(n));
+	if (GNET_PROPERTY(dbg) > 3)
+		g_debug("sent %d/%d hits to %s", sent, count, node_addr(n));
+
+	found_done();
 }
 
 /**
@@ -797,20 +1042,19 @@ qhit_send_results(
  * @param cb			the processor callback to invoke on each built hit
  * @param udata			argument to pass to callback
  * @param muid			the MUID to use on each generated hit
- * @param use_ggep_h	whether GGEP "H" can be used to send the SHA1 of files
  */
 void
-qhit_build_results(
-	GSList *files, gint count,
-	size_t max_msgsize,
-	qhit_process_t cb, gpointer udata,
-	const gchar *muid, gboolean use_ggep_h)
+qhit_build_results(const GSList *files, int count, size_t max_msgsize,
+	qhit_process_t cb, gpointer udata, const struct guid *muid, gboolean ggep_h,
+	const struct array *token)
 {
 	const GSList *sl;
-	gint sent;
+	int sent;
 
 	g_assert(cb != NULL);
-	found_reset(max_msgsize, muid, use_ggep_h, cb, udata);
+	g_assert(token);
+
+	found_reset(max_msgsize, muid, ggep_h, cb, udata, token);
 
 	for (sl = files, sent = 0; sl && sent < count; sl = g_slist_next(sl)) {
 		const shared_file_t *sf = sl->data;
@@ -822,6 +1066,8 @@ qhit_build_results(
 	if (0 != found_file_count())	/* Still some unflushed results */
 		flush_match();				/* Send last packet */
 
+	found_done();
+
 	/*
 	 * Nothing to free, since everything is the property of the calling module.
 	 */
