@@ -1,5 +1,5 @@
 /*
- * $Id: routing.h 12496 2006-11-26 09:22:49Z cbiere $
+ * $Id: routing.h 19256 2011-06-14 19:11:47Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -30,6 +30,8 @@
 
 #include "gnutella.h"
 
+struct guid;
+
 /**
  * Route destination types.
  */
@@ -38,8 +40,7 @@ typedef enum {
 	ROUTE_NONE = 0,			/**< No route, message stops here */
 	ROUTE_ONE,				/**< Route to single node */
 	ROUTE_ALL_BUT_ONE,		/**< Route to all nodes but one */
-	ROUTE_MULTI,			/**< Route to list of nodes */
-	ROUTE_NO_DUPS_BUT_ONE,	/**< Temporary: watch out broken GTKGs */
+	ROUTE_MULTI				/**< Route to list of nodes */
 } route_type_t;
 
 /**
@@ -52,24 +53,35 @@ struct route_dest {
 		struct gnutella_node *u_node;
 		GSList *u_nodes;	/**< For ROUTE_MULTI */
 	} ur;
+	unsigned duplicate:1;	/**< Set if message was a duplicate */
 };
 
+/**
+ * Starving GUID callback.
+ */
+typedef void (*route_starving_cb_t)(const struct guid *);
+
 /*
  * Global Functions
  */
 
+void gnet_reset_guid(void);
+
 void routing_init(void);
 void routing_close(void);
 void message_set_muid(gnutella_header_t *header, guint8 function);
 gboolean route_message(struct gnutella_node **, struct route_dest *);
-void routing_node_remove(struct gnutella_node *);
-void message_add(const gchar *muid, guint8, struct gnutella_node *);
-GSList *route_towards_guid(const gchar *guid);
-gboolean route_exists_for_reply(const gchar *muid, guint8 function);
+void routing_node_remove(void *node);
+void message_add(const struct guid *muid, guint8, struct gnutella_node *);
+GSList *route_towards_guid(const struct guid *guid);
+gboolean route_exists_for_reply(const struct guid *muid, guint8 function);
 
-gboolean route_proxy_add(const gchar *guid, struct gnutella_node *n);
-void route_proxy_remove(const gchar *guid);
-struct gnutella_node *route_proxy_find(const gchar *guid);
+gboolean route_proxy_add(const struct guid *guid, struct gnutella_node *n);
+void route_proxy_remove(const struct guid *guid);
+struct gnutella_node *route_proxy_find(const struct guid *guid);
+
+void route_starving_add(const struct guid *guid, route_starving_cb_t cb);
+void route_starving_remove(const struct guid *guid);
 
 #endif /* _core_routing_h_ */
 /* vi: set ts=4 sw=4 cindent: */
