@@ -1,5 +1,5 @@
 /*
- * $Id: rx.c,v 1.13 2005/11/30 16:43:41 rmanfredi Exp $
+ * $Id: rx.c 19142 2011-05-23 20:44:04Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -40,10 +40,12 @@
 
 #include "common.h"
 
-RCSID("$Id: rx.c,v 1.13 2005/11/30 16:43:41 rmanfredi Exp $");
+RCSID("$Id: rx.c 19142 2011-05-23 20:44:04Z rmanfredi $")
 
 #include "rx.h"
 #include "nodes.h"
+
+#include "lib/glib-missing.h"
 #include "lib/walloc.h"
 #include "lib/override.h"		/* Must be the last header included */
 
@@ -67,16 +69,33 @@ static GSList *rx_freed = NULL;
 
 /**
  * Tell upper layer that it got new data from us.
+ * @return FALSE if there was on error or the receiver wants no more data.
  */
-static void
+static gboolean
 rx_data_ind(rxdrv_t *rx, pmsg_t *mb)
 {
-	g_assert(rx);
+	rx_check(rx);
 
 	if (rx->upper == NULL)
 		g_error("Forgot to call rx_set_data_ind() on the RX stack.");
 
-	rx_recv(rx->upper, mb);
+	return rx_recv(rx->upper, mb);
+}
+
+/**
+ * This data indication callback is installed when the RX stack is freed
+ * so that further indication to the user-level code is blocked.
+ * @return FALSE
+ */
+static gboolean
+rx_data_ind_freed(rxdrv_t *rx, pmsg_t *mb)
+{
+	rx_check(rx);
+	g_assert(rx->upper == NULL);
+	g_assert(rx->flags & RX_F_FREED);
+
+	pmsg_free(mb);
+	return FALSE;		/* Stop sending more data */
 }
 
 /*
@@ -104,11 +123,11 @@ rx_make(
 	g_assert(owner);
 	g_assert(ops);
 
-	rx = walloc0(sizeof *rx);
-
+	WALLOC0(rx);
+	rx->magic = RXDRV_MAGIC;
 	rx->owner = owner;
 	rx->ops = ops;
-	rx->host = *host;		/* Struct copy */
+	gnet_host_copy(&rx->host, host);
 	rx->upper = NULL;
 	rx->lower = NULL;
 
@@ -135,22 +154,51 @@ rx_make(
 void
 rx_set_data_ind(rxdrv_t *rx, rx_data_t data_ind)
 {
-	g_assert(rx);
+	rx_check(rx);
 	g_assert(rx->upper == NULL);			/* Called on topmost driver */
-	g_assert(data_ind != rx_data_ind);		/* Must not use internal routine */
-	g_assert(rx->data_ind == rx_data_ind);	/* No data_ind set already */
+	g_assert(!(rx->flags & RX_F_FREED));
 
 	rx->data_ind = data_ind;
 }
 
+/**
+ * Fetch current `data_ind' callback.
+ */
+rx_data_t
+rx_get_data_ind(rxdrv_t *rx)
+{
+	rx_check(rx);
+	g_assert(rx->upper == NULL);			/* Called on topmost driver */
+
+	return rx->data_ind;
+}
+
+/**
+ * Replace the `data_ind' callback, returning the old one.
+ */
+rx_data_t
+rx_replace_data_ind(rxdrv_t *rx, rx_data_t data_ind)
+{
+	rx_data_t old_data_ind;
+
+	rx_check(rx);
+	g_assert(rx->upper == NULL);			/* Called on topmost driver */
+	g_assert(!(rx->flags & RX_F_FREED));
+
+	old_data_ind = rx->data_ind;
+	rx->data_ind = data_ind;
+
+	return old_data_ind;
+}
+
 /**
  * Called when an upper driver (urx) is attached on top of us.
  */
 static void
 rx_attached(rxdrv_t *rx, rxdrv_t *urx)
 {
-	g_assert(rx);
-	g_assert(urx);
+	rx_check(rx);
+	rx_check(urx);
 	g_assert(rx->upper == NULL);			/* Can only attach ONE layer */
 
 	rx->upper = urx;
@@ -166,14 +214,14 @@ rx_make_above(rxdrv_t *lrx, const struct rxdrv_ops *ops, gconstpointer args)
 {
 	rxdrv_t *rx;
 
-	g_assert(lrx);
+	rx_check(lrx);
 	g_assert(lrx->upper == NULL);		/* Nothing above yet */
 	g_assert(ops);
 
-	rx = walloc0(sizeof(*rx));
-
+	WALLOC0(rx);
+	rx->magic = RXDRV_MAGIC;
 	rx->owner = lrx->owner;
-	rx->host = lrx->host;				/* Struct copy */
+	gnet_host_copy(&rx->host, &lrx->host);
 	rx->ops = ops;
 	rx->upper = NULL;
 	rx->lower = lrx;
@@ -200,13 +248,27 @@ rx_make_above(rxdrv_t *lrx, const struct rxdrv_ops *ops, gconstpointer args)
 static void
 rx_deep_free(rxdrv_t *rx)
 {
-	g_assert(rx);
+	rx_check(rx);
 
 	if (rx->lower)
 		rx_deep_free(rx->lower);
 
 	RX_DESTROY(rx);
-	wfree(rx, sizeof(*rx));
+	rx->magic = 0;
+	WFREE(rx);
+}
+
+/**
+ * Change RX stack owner.
+ */
+void
+rx_change_owner(rxdrv_t *rx, gpointer owner)
+{
+	rx_check(rx);
+
+	rx->owner = owner;
+	if (rx->lower)
+		rx_change_owner(rx->lower, owner);
 }
 
 /**
@@ -216,10 +278,13 @@ rx_deep_free(rxdrv_t *rx)
 void
 rx_free(rxdrv_t *rx)
 {
-	g_assert(rx);
+	rx_check(rx);
 	g_assert(rx->upper == NULL);
+	g_assert(!(rx->flags & RX_F_FREED));
 
+	rx_set_data_ind(rx, rx_data_ind_freed);
 	rx_disable(rx);
+	rx->flags |= RX_F_FREED;
 	rx_freed = g_slist_prepend(rx_freed, rx);
 }
 
@@ -233,23 +298,23 @@ rx_collect(void)
 
 	for (sl = rx_freed; sl; sl = g_slist_next(sl)) {
 		rxdrv_t *rx = sl->data;
+		g_assert(rx->flags & RX_F_FREED);
 		rx_deep_free(rx);
 	}
 
-	g_slist_free(rx_freed);
-	rx_freed = NULL;
+	gm_slist_free_null(&rx_freed);
 }
 
 /**
  * Inject data into driver, from lower layer.
  */
-void
+gboolean
 rx_recv(rxdrv_t *rx, pmsg_t *mb)
 {
-	g_assert(rx);
+	rx_check(rx);
 	g_assert(mb);
 
-	RX_RECV(rx, mb);
+	return RX_RECV(rx, mb);
 }
 
 /**
@@ -271,7 +336,7 @@ rx_deep_enable(rxdrv_t *rx)
 void
 rx_enable(rxdrv_t *rx)
 {
-	g_assert(rx);
+	rx_check(rx);
 	g_assert(rx->upper == NULL);
 
 	rx_deep_enable(rx);
@@ -296,7 +361,7 @@ rx_deep_disable(rxdrv_t *rx)
 void
 rx_disable(rxdrv_t *rx)
 {
-	g_assert(rx);
+	rx_check(rx);
 	g_assert(rx->upper == NULL);
 
 	rx_deep_disable(rx);
@@ -320,7 +385,7 @@ rx_deep_bottom(rxdrv_t *rx)
 rxdrv_t *
 rx_bottom(rxdrv_t *rx)
 {
-	g_assert(rx);
+	rx_check(rx);
 	g_assert(rx->upper == NULL);
 
 	return rx_deep_bottom(rx);
@@ -334,7 +399,7 @@ rx_bio_source(rxdrv_t *rx)
 {
 	rxdrv_t *bottom;
 
-	g_assert(rx);
+	rx_check(rx);
 	g_assert(rx->upper == NULL);
 
 	bottom = rx_bottom(rx);
