@@ -1,5 +1,5 @@
 /*
- * $Id: settings.h,v 1.8 2005/08/17 13:44:05 cbiere Exp $
+ * $Id: settings.h 19262 2011-06-15 10:34:36Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -27,7 +27,7 @@
  * @ingroup core
  * @file
  *
- * Gtk-Gnutella configuration.
+ * gtk-gnutella configuration.
  *
  * @author Raphael Manfredi
  * @author Richard Eckart
@@ -48,23 +48,29 @@
 
 extern struct in_addr *local_netmasks;
 
-
 /*
  * Global Functions
  */
 
-host_addr_t listen_addr(void);
+gboolean is_my_address(const host_addr_t addr);
+gboolean is_my_address_and_port(const host_addr_t addr, guint16 port);
+
+void settings_early_init(void);
+gboolean settings_is_unique_instance(void);
 void settings_init(void);
 void settings_save_if_dirty(void);
 void settings_shutdown(void);
 void settings_addr_changed(const host_addr_t new_addr, const host_addr_t peer);
 guint32 settings_max_msg_size(void);
+void settings_add_randomness();
 void settings_close(void);
-void settings_ask_for_property(const gchar *name,
-	const gchar *value) G_GNUC_NORETURN;
 
 guint32 get_average_servent_uptime(time_t now);
-guint32 get_average_ip_lifetime(time_t now);
+guint32 get_average_ip_lifetime(time_t now, enum net_type net);
+
+gboolean settings_is_leaf(void);
+gboolean settings_is_ultra(void);
 
 #endif /* _core_settings_h_ */
+
 /* vi: set ts=4 sw=4 cindent: */
