@@ -1,5 +1,5 @@
 /*
- * $Id: sockets.h 15053 2007-12-08 01:15:45Z cbiere $
+ * $Id: sockets.h 19067 2011-05-11 10:15:00Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  * Copyright (c) 2000 Daniel Walker (dwalker@cats.ucsc.edu)
@@ -66,6 +66,7 @@ struct socket_tls_ctx {
 };
 
 struct sockaddr;
+struct udpctx;
 
 /*
  * Connection directions.
@@ -78,30 +79,14 @@ enum socket_direction {
 	SOCK_CONN_PROXY_OUTGOING
 };
 
-/**
- * Connection types.
- */
-
-enum socket_type {
-	SOCK_TYPE_UNKNOWN = 0,
-	SOCK_TYPE_CONTROL,
-	SOCK_TYPE_DOWNLOAD,
-	SOCK_TYPE_UPLOAD,
-	SOCK_TYPE_HTTP,
-    SOCK_TYPE_SHELL,
-    SOCK_TYPE_CONNBACK,
-    SOCK_TYPE_PPROXY,
-    SOCK_TYPE_DESTROYING,
-	SOCK_TYPE_UDP
-};
-
 typedef enum {
 	SOCKET_MAGIC = 0x1fb7ddeb
 } socket_magic_t;
 
+
 struct gnutella_socket {
 	socket_magic_t magic;	/**< magic for consistency checks */
-	gint file_desc;			/**< file descriptor */
+	socket_fd_t file_desc;	/**< file descriptor */
 	guint32 flags;			/**< operating flags */
 	guint gdk_tag;			/**< gdk tag */
 
@@ -109,8 +94,8 @@ struct gnutella_socket {
 	enum socket_type type;
 	enum net_type net;
 
-	gint adns;				/**< status of ADNS resolution */
-	const gchar *adns_msg;	/**< ADNS error message */
+	int adns;				/**< status of ADNS resolution */
+	const char *adns_msg;	/**< ADNS error message */
 
 	host_addr_t addr;		/**< IP   of our partner */
 	guint16 port;			/**< Port of our partner */
@@ -129,23 +114,56 @@ struct gnutella_socket {
 		struct upload *upload;
 		struct pproxy *pproxy;
 		struct cproxy *cproxy;
+		struct udpctx *udp;
 		gpointer handle;
 	} resource;
 
 	struct getline *getline;	/**< Line reader object */
 
 	size_t pos;			/**< write position in the buffer */
-	size_t buf_size;	/**< write position in the buffer */
-	gchar *buf;			/**< buffer to put in the data read */
+	size_t buf_size;	/**< allocated buffer size */
+	char *buf;			/**< buffer to put in the data read */
+
+	unsigned so_rcvbuf;	/**< Configured RX buffer size, 0 if unknown */
+	unsigned so_sndbuf;	/**< Configured TX buffer size, 0 if unknown */
+};
+
+/**
+ * The UDP data indication callback.
+ *
+ * @param s				the receiving socket
+ * @param truncated		whether received data was truncated
+ *
+ * Data is held in s->buf and is s->pos byte-long.
+ */
+typedef void (*socket_udp_data_ind_t)(
+	struct gnutella_socket *s, gboolean truncated);
+
+/**
+ * UDP socket context.
+ */
+struct udpctx {
+	void *socket_addr;					/**< To get reception address */
+	socket_udp_data_ind_t data_ind;		/**< Callback on datagram reception */
 };
 
 static inline void
 socket_check(const struct gnutella_socket * const s)
 {
-	g_assert(s);
+	g_assert(s != NULL);
 	g_assert(SOCKET_MAGIC == s->magic);
 }
 
+static inline void
+socket_buffer_check(const struct gnutella_socket * const s)
+{
+	g_assert(s != NULL);
+	g_assert(SOCKET_MAGIC == s->magic);
+	g_assert((0 == s->buf_size) ^ (NULL != s->buf));
+	g_assert(0 == s->pos || s->buf != NULL);
+}
+
+
 /*
  * Global Data
  */
@@ -206,22 +224,25 @@ socket_listen_port(void)
 void socket_init(void);
 void socket_register_fd_reclaimer(reclaim_fd_t callback);
 void socket_eof(struct gnutella_socket *s);
+void socket_connection_reset(struct gnutella_socket *s);
 void socket_free_null(struct gnutella_socket **s_ptr);
 struct gnutella_socket *socket_connect(const host_addr_t, guint16,
 		enum socket_type, guint32 flags);
 struct gnutella_socket *socket_connect_by_name(
-	const gchar *host, guint16, enum socket_type, guint32 flags);
+	const char *host, guint16, enum socket_type, guint32 flags);
 struct gnutella_socket *socket_tcp_listen(const host_addr_t, guint16);
-struct gnutella_socket *socket_udp_listen(const host_addr_t, guint16);
-struct gnutella_socket *socket_local_listen(const gchar *pathname);
+struct gnutella_socket *socket_udp_listen(const host_addr_t, guint16,
+	socket_udp_data_ind_t data_ind);
+struct gnutella_socket *socket_local_listen(const char *pathname);
+void socket_set_single(struct gnutella_socket *s, gboolean on);
 
 void socket_evt_set(struct gnutella_socket *s,
 	inputevt_cond_t cond, inputevt_handler_t handler, gpointer data);
 void socket_evt_clear(struct gnutella_socket *s);
 
 void socket_cork(struct gnutella_socket *s, gboolean on);
-void socket_send_buf(struct gnutella_socket *s, gint size, gboolean shrink);
-void socket_recv_buf(struct gnutella_socket *s, gint size, gboolean shrink);
+void socket_send_buf(struct gnutella_socket *s, int size, gboolean shrink);
+void socket_recv_buf(struct gnutella_socket *s, int size, gboolean shrink);
 void socket_nodelay(struct gnutella_socket *s, gboolean on);
 void socket_tx_shutdown(struct gnutella_socket *s);
 void socket_tos_default(const struct gnutella_socket *s);
@@ -233,16 +254,17 @@ gboolean socket_bad_hostname(struct gnutella_socket *s);
 void socket_disable_token(struct gnutella_socket *s);
 gboolean socket_omit_token(struct gnutella_socket *s);
 void socket_set_bind_address(const host_addr_t addr);
-gint socket_evt_fd(struct gnutella_socket *s);
+int socket_evt_fd(struct gnutella_socket *s);
 gboolean socket_is_local(const struct gnutella_socket *s);
+gboolean socket_local_addr(const struct gnutella_socket *s, host_addr_t *ap);
 
 void socket_timer(time_t now);
 void socket_shutdown(void);
 
-ssize_t safe_readv(wrap_io_t *wio, struct iovec *iov, gint iovcnt);
-ssize_t safe_readv_fd(gint fd, struct iovec *iov, gint iovcnt);
-ssize_t safe_writev(wrap_io_t *wio, const struct iovec *iov, gint iovcnt);
-ssize_t safe_writev_fd(gint fd, const struct iovec *iov, gint iovcnt);
+ssize_t safe_readv(wrap_io_t *wio, iovec_t *iov, int iovcnt);
+ssize_t safe_readv_fd(int fd, iovec_t *iov, int iovcnt);
+ssize_t safe_writev(wrap_io_t *wio, const iovec_t *iov, int iovcnt);
+ssize_t safe_writev_fd(int fd, const iovec_t *iov, int iovcnt);
 
 #endif /* _core_sockets_h_ */
 
