@@ -1,5 +1,5 @@
 /*
- * $Id: spam.c 14434 2007-08-13 09:58:30Z rmanfredi $
+ * $Id: spam.c 19142 2011-05-23 20:44:04Z rmanfredi $
  *
  * Copyright (c) 2006, Christian Biere
  * Copyright (c) 2004, Raphael Manfredi
@@ -41,7 +41,7 @@
 
 #include "common.h"
 
-RCSID("$Id: spam.c 14434 2007-08-13 09:58:30Z rmanfredi $")
+RCSID("$Id: spam.c 19142 2011-05-23 20:44:04Z rmanfredi $")
 
 #include "spam.h"
 #include "settings.h"
@@ -52,8 +52,10 @@ RCSID("$Id: spam.c 14434 2007-08-13 09:58:30Z rmanfredi $")
 #include "lib/bit_array.h"
 #include "lib/file.h"
 #include "lib/getdate.h"
-#include "lib/misc.h"
+#include "lib/halloc.h"
 #include "lib/glib-missing.h"
+#include "lib/parse.h"
+#include "lib/path.h"
 #include "lib/walloc.h"
 #include "lib/watcher.h"
 #include "lib/utf8.h"
@@ -64,8 +66,8 @@ RCSID("$Id: spam.c 14434 2007-08-13 09:58:30Z rmanfredi $")
 
 #include "lib/override.h"		/* Must be the last header included */
 
-static const gchar spam_text_file[] = "spam.txt";
-static const gchar spam_what[] = "Spam database";
+static const char spam_text_file[] = "spam.txt";
+static const char spam_what[] = "Spam database";
 
 /****** BEGIN IDEAS ONLY ******/
 
@@ -142,7 +144,7 @@ typedef enum {
 
 static const struct spam_tag {
 	spam_tag_t	tag;
-	const gchar *str;
+	const char *str;
 } spam_tag_map[] = {
 	/* Must be sorted alphabetically for dichotomic search */
 
@@ -159,7 +161,7 @@ static const struct spam_tag {
 
 
 static spam_tag_t
-spam_string_to_tag(const gchar *s)
+spam_string_to_tag(const char *s)
 {
 	STATIC_ASSERT(G_N_ELEMENTS(spam_tag_map) == NUM_SPAM_TAGS - 1U);
 
@@ -170,7 +172,7 @@ spam_string_to_tag(const gchar *s)
 } G_STMT_END
 
 	/* Perform a binary search to find ``s'' */
-	BINARY_SEARCH(const gchar *, s, G_N_ELEMENTS(spam_tag_map), strcmp,
+	BINARY_SEARCH(const char *, s, G_N_ELEMENTS(spam_tag_map), strcmp,
 		GET_ITEM, FOUND);
 
 #undef FOUND
@@ -185,7 +187,7 @@ struct namesize_item {
 };
 
 static gboolean 
-spam_add_name_and_size(const gchar *name,
+spam_add_name_and_size(const char *name,
 	filesize_t min_size, filesize_t max_size)
 {
 	struct namesize_item *item;
@@ -194,15 +196,15 @@ spam_add_name_and_size(const gchar *name,
 	g_return_val_if_fail(name, TRUE);
 	g_return_val_if_fail(min_size <= max_size, TRUE);
 
-	item = walloc(sizeof *item);
+	WALLOC(item);
 	error = regcomp(&item->pattern, name, REG_EXTENDED | REG_NOSUB);
 	if (error) {
-		gchar buf[1024];
+		char buf[1024];
 
 		regerror(error, &item->pattern, buf, sizeof buf);
 		g_warning("spam_add_name_and_size(): regcomp() failed: %s", buf);
 		regfree(&item->pattern);
-		wfree(item, sizeof *item);
+		WFREE(item);
 		return TRUE;
 	} else {
 		item->min_size = min_size;
@@ -214,7 +216,7 @@ spam_add_name_and_size(const gchar *name,
 
 struct spam_item {
 	struct sha1 sha1;
-	gchar		*name;
+	char		*name;
 	filesize_t  min_size;
 	filesize_t  max_size;
 	gboolean	done;
@@ -238,7 +240,7 @@ spam_load(FILE *f)
 {
 	static const struct spam_item zero_item;
 	struct spam_item item;
-	gchar line[1024];
+	char line[1024];
 	guint line_no = 0;
 	bit_array_t tag_used[BIT_ARRAY_SIZE(NUM_SPAM_TAGS)];
 	gulong item_count = 0;
@@ -247,11 +249,11 @@ spam_load(FILE *f)
 
 	/* Reset state */
 	item = zero_item;
-	bit_array_clear_range(tag_used, 0, NUM_SPAM_TAGS - 1U);
+	bit_array_init(tag_used, NUM_SPAM_TAGS);
 
 	while (fgets(line, sizeof line, f)) {
-		const gchar *tag_name, *value;
-		gchar *sp, *nl;
+		const char *tag_name, *value;
+		char *sp, *nl;
 		spam_tag_t tag;
 
 		line_no++;
@@ -331,16 +333,16 @@ spam_load(FILE *f)
 					item.damaged = TRUE;
 					g_warning("spam_load(): Filename pattern is not UTF-8.");
 				} else {
-					item.name = g_strdup(value);
+					item.name = h_strdup(value);
 				}
 			}
 			break;
 
 		case SPAM_TAG_SIZE:
 			{
-				const gchar *endptr;
+				const char *endptr;
 				guint64 u;
-				gint error;
+				int error;
 					
 				u = parse_uint64(value, &endptr, 10, &error);
 				if (error) {
@@ -422,7 +424,7 @@ spam_load(FILE *f)
 
 		if (item.done) {
 			/* Reset state */
-			G_FREE_NULL(item.name);
+			HFREE_NULL(item.name);
 			item = zero_item;
 			bit_array_clear_range(tag_used, 0, NUM_SPAM_TAGS - 1U);
 		}
@@ -438,7 +440,7 @@ spam_load(FILE *f)
  * changed.
  */
 static void
-spam_changed(const gchar *filename, gpointer unused_udata)
+spam_changed(const char *filename, gpointer unused_udata)
 {
 	FILE *f;
 
@@ -446,7 +448,7 @@ spam_changed(const gchar *filename, gpointer unused_udata)
 
 	f = file_fopen(filename, "r");
 	if (f) {
-		gchar buf[80];
+		char buf[80];
 		gulong count;
 
 		spam_close();
@@ -459,9 +461,9 @@ spam_changed(const gchar *filename, gpointer unused_udata)
 }
 
 static void
-spam_retrieve_from_file(FILE *f, const gchar *path, const gchar *filename)
+spam_retrieve_from_file(FILE *f, const char *path, const char *filename)
 {
-	gchar *pathname;
+	char *pathname;
 
 	g_assert(f);
 	g_assert(path);
@@ -469,7 +471,7 @@ spam_retrieve_from_file(FILE *f, const gchar *path, const gchar *filename)
 
 	pathname = make_pathname(path, filename);
 	watcher_register(pathname, spam_changed, NULL);
-	G_FREE_NULL(pathname);
+	HFREE_NULL(pathname);
 	spam_load(f);
 }
 
@@ -485,7 +487,7 @@ spam_retrieve(void)
 	static file_path_t fp[3];
 	guint num_fp = G_N_ELEMENTS(fp) - 1;
 	FILE *f;
-	gint idx;
+	int idx;
 	
 	file_path_set(&fp[0], settings_config_dir(), spam_text_file);
 	file_path_set(&fp[1], PRIVLIB_EXP, spam_text_file);
@@ -525,10 +527,9 @@ spam_close(void)
 
 		g_assert(item);
 		regfree(&item->pattern);
-		wfree(item, sizeof *item);
+		WFREE(item);
 	}
-	g_slist_free(spam_lut.sl_names);
-	spam_lut.sl_names = NULL;
+	gm_slist_free_null(&spam_lut.sl_names);
 	spam_sha1_close();
 }
 
@@ -539,7 +540,7 @@ spam_close(void)
  * @returns TRUE if found, and FALSE if not.
  */
 gboolean
-spam_check_filename_and_size(const char *filename, filesize_t size)
+spam_check_filename_size(const char *filename, filesize_t size)
 {
 	const GSList *sl;
 
