@@ -1,5 +1,5 @@
 /*
- * $Id: sq.c,v 1.12 2005/08/31 21:37:28 rmanfredi Exp $
+ * $Id: sq.c 19262 2011-06-15 10:34:36Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Alex Bennee <alex@bennee.com> & Raphael Manfredi
  *
@@ -46,27 +46,25 @@
 
 #include "common.h"
 
-RCSID("$Id: sq.c,v 1.12 2005/08/31 21:37:28 rmanfredi Exp $");
+RCSID("$Id: sq.c 19262 2011-06-15 10:34:36Z rmanfredi $")
 
 #include "sq.h"					/* search_queue structures */
-#include "pmsg.h"
+#include "dq.h"
+#include "mq_tcp.h"
 #include "nodes.h"
 #include "search.h"
-#include "dq.h"
+#include "settings.h"
 
 #include "if/gnet_property_priv.h"
 
 #include "lib/atoms.h"
+#include "lib/glib-missing.h"
+#include "lib/nid.h"
+#include "lib/pmsg.h"
 #include "lib/tm.h"
 #include "lib/walloc.h"
 #include "lib/override.h"		/* Must be the last header included */
 
-/*
- * Compute start of search string (which is NUL terminated) in query.
- * The "+2" skips the "speed" field in the query.
- */
-#define QUERY_TEXT(m)	((m) + sizeof(struct gnutella_header) + 2)
-
 /**
  * A search queue entry.
  *
@@ -87,9 +85,8 @@ typedef struct smsg {
  * react when the message queue informs us it has processed it.
  */
 struct smsg_info {
-	gpointer search;			/**< The search object which sends the query */
-	guint32 id;					/**< The unique search ID */
-	guint32 node_id;			/**< The unique node ID to which we're sending */
+	struct nid *node_id; 	/**< The unique node ID to which we're sending */
+	gnet_search_t shandle;	/**< Handle to search that originated query */
 };
 
 static squeue_t *global_sq = NULL;
@@ -102,7 +99,7 @@ static void cap_queue(squeue_t *sq);
 static void
 sq_pmsg_free(pmsg_t *mb, gpointer arg)
 {
-	struct smsg_info *smi = (struct smsg_info *) arg;
+	struct smsg_info *smi = arg;
 
 	g_assert(pmsg_is_extended(mb));
 
@@ -111,10 +108,11 @@ sq_pmsg_free(pmsg_t *mb, gpointer arg)
 	 * query for it to the specified node ID.
 	 */
 
-	if (current_peermode == NODE_P_LEAF)
-		search_notify_sent(smi->search, smi->id, smi->node_id);
+	if (settings_is_leaf())
+		search_notify_sent(smi->shandle, smi->node_id);
 
-	wfree(smi, sizeof(*smi));
+	nid_unref(smi->node_id);
+	WFREE(smi);
 }
 
 /***
@@ -127,8 +125,9 @@ sq_pmsg_free(pmsg_t *mb, gpointer arg)
 static smsg_t *
 smsg_alloc(gnet_search_t sh, pmsg_t *mb, query_hashvec_t *qhv)
 {
-	smsg_t *sb = walloc(sizeof(*sb));
+	smsg_t *sb;
 
+	WALLOC(sb);
 	sb->shandle = sh;
 	sb->mb = mb;
 	sb->qhv = qhv;
@@ -144,7 +143,7 @@ smsg_free(smsg_t *sb)
 {
 	g_assert(sb);
 
-	wfree(sb, sizeof(*sb));
+	WFREE(sb);
 }
 
 /**
@@ -171,9 +170,9 @@ smsg_mutate(smsg_t *sb, struct gnutella_node *n)
 	struct smsg_info *smi;
 	pmsg_t *omb;
 
-	smi = (struct smsg_info *) walloc(sizeof(*smi));
-	smi->id = search_get_id(sb->shandle, &smi->search);
-	smi->node_id = n->id;
+	WALLOC(smi);
+	smi->shandle = sb->shandle;
+	smi->node_id = nid_ref(NODE_ID(n));
 
 	omb = sb->mb;
 	sb->mb = pmsg_clone_extend(omb, sq_pmsg_free, smi);
@@ -240,8 +239,6 @@ sq_make(struct gnutella_node *node)
 {
     squeue_t *sq;
 
-    sq = walloc(sizeof(*sq));
-
 	/*
 	 * By initializing `last_sent' to the current time and not to `0', we
 	 * ensure that we won't send the query to the node during the first
@@ -252,6 +249,7 @@ sq_make(struct gnutella_node *node)
 	 *		--RAM, 01/05/2002
 	 */
 
+    WALLOC(sq);
 	sq->count		= 0;
 	sq->last_sent 	= tm_time();
 	sq->searches 	= NULL;
@@ -273,20 +271,18 @@ sq_clear(squeue_t *sq)
 
 	g_assert(sq);
 
-	if (dbg > 3)
-		printf("clearing sq node %s (sent=%d, dropped=%d)\n",
+	if (GNET_PROPERTY(sq_debug) > 3)
+		g_debug("clearing sq node %s (sent=%d, dropped=%d)",
 			sq->node ? node_addr(sq->node) : "GLOBAL",
 			sq->n_sent, sq->n_dropped);
 
 	for (l = sq->searches; l; l = g_list_next(l)) {
-		smsg_t *sb = (smsg_t *) l->data;
+		smsg_t *sb = l->data;
 
 		smsg_discard(sb);
 	}
 
-	g_list_free(sq->searches);
-
-	sq->searches = NULL;
+	gm_list_free_null(&sq->searches);
 	sq->count = 0;
 }
 
@@ -299,8 +295,8 @@ sq_free(squeue_t *sq)
 	g_assert(sq);
 
 	sq_clear(sq);
-	g_hash_table_destroy(sq->handles);
-	wfree(sq, sizeof(*sq));
+	gm_hash_table_destroy_null(&sq->handles);
+	WFREE(sq);
 }
 
 /**
@@ -314,8 +310,12 @@ sq_puthere(squeue_t *sq, gnet_search_t sh, pmsg_t *mb, query_hashvec_t *qhv)
 	g_assert(sq);
 	g_assert(mb);
 
-	if (sqh_exists(sq, sh))
+	if (sqh_exists(sq, sh)) {
+		pmsg_free(mb);
+		if (qhv)
+			qhvec_free(qhv);
 		return;						/* Search already in queue */
+	}
 
 	sb = smsg_alloc(sh, mb, qhv);
 
@@ -323,7 +323,7 @@ sq_puthere(squeue_t *sq, gnet_search_t sh, pmsg_t *mb, query_hashvec_t *qhv)
 	sq->searches = g_list_prepend(sq->searches, sb);
 	sq->count++;
 
-	if (sq->count > search_queue_size)
+	if (sq->count > GNET_PROPERTY(search_queue_size))
 		cap_queue(sq);
 }
 
@@ -366,6 +366,7 @@ sq_global_putq(gnet_search_t sh, pmsg_t *mb, query_hashvec_t *qhv)
 void
 sq_process(squeue_t *sq, time_t now)
 {
+    time_delta_t spacing = GNET_PROPERTY(search_queue_spacing);
 	GList *item;
 	smsg_t *sb;
 	struct gnutella_node *n;
@@ -389,7 +390,7 @@ retry:
 	if (sq->count == 0)
 		return;
 
-    if (delta_time(now, sq->last_sent) < search_queue_spacing)
+    if (delta_time(now, sq->last_sent) < spacing)
 		return;
 
 	n = sq->node;					/* Will be NULL for the global SQ */
@@ -411,11 +412,11 @@ retry:
 		 * Processing the global SQ.
 		 */
 
-		if (current_peermode != NODE_P_ULTRA)
+		if (settings_is_leaf())
 			return;
 
-		if (node_keep_missing() * 3 > 2 * up_connections)
-			return;							/* Not enough nodes for querying */
+		if (3*UNSIGNED(node_keep_missing()) > 2*GNET_PROPERTY(up_connections))
+			return;		/* Not enough nodes for querying */
 	}
 
 	/*
@@ -426,7 +427,7 @@ retry:
 	g_assert(sq->searches);
 
 	item = g_list_first(sq->searches);
-	sb = (smsg_t *) item->data;
+	sb = item->data;
 
 	g_assert(sq->count > 0);
 	sq->count--;
@@ -435,9 +436,10 @@ retry:
 	if (n == NULL) {
 		g_assert(sb->qhv != NULL);		/* Enqueued via sq_global_putq() */
 
-		if (dbg > 2)
-			printf("sq GLOBAL, queuing \"%s\" (%u left, %d sent)\n",
-				QUERY_TEXT(pmsg_start(sb->mb)), sq->count, sq->n_sent);
+		if (GNET_PROPERTY(sq_debug) > 2)
+			g_debug("sq GLOBAL, queuing \"%s\" (%u left, %d sent)",
+				gnutella_msg_search_get_text(pmsg_start(sb->mb)),
+				sq->count, sq->n_sent);
 
 		dq_launch_local(sb->shandle, sb->mb, sb->qhv);
 
@@ -449,9 +451,9 @@ retry:
 
 		g_assert(sb->qhv == NULL);		/* Enqueued via sq_putq() */
 
-		if (dbg > 2)
-			printf("sq for node %s, queuing \"%s\" (%u left, %d sent)\n",
-				node_addr(n), QUERY_TEXT(pmsg_start(sb->mb)),
+		if (GNET_PROPERTY(sq_debug) > 2)
+			g_debug("sq for node %s, queuing \"%s\" (%u left, %d sent)",
+				node_addr(n), gnutella_msg_search_get_text(pmsg_start(sb->mb)),
 				sq->count, sq->n_sent);
 
 		/*
@@ -461,15 +463,15 @@ retry:
 		 * queries that go out.
 		 */
 
-		if (current_peermode == NODE_P_LEAF)
+		if (settings_is_leaf())
 			smsg_mutate(sb, n);
 
-		mq_putq(n->outq, sb->mb);
+		mq_tcp_putq(n->outq, sb->mb, NULL);
 
 	} else {
-		if (dbg > 4)
-			printf("sq for node %s, ignored \"%s\" (%u left, %d sent)\n",
-				node_addr(n), QUERY_TEXT(pmsg_start(sb->mb)),
+		if (GNET_PROPERTY(sq_debug) > 4)
+			g_debug("sq for node %s, ignored \"%s\" (%u left, %d sent)",
+				node_addr(n), gnutella_msg_search_get_text(pmsg_start(sb->mb)),
 				sq->count, sq->n_sent);
 		pmsg_free(sb->mb);
 		if (sb->qhv)
@@ -503,9 +505,9 @@ retry:
 static void
 cap_queue(squeue_t *sq)
 {
-    while (sq->count > search_queue_size) {
+    while (sq->count > GNET_PROPERTY(search_queue_size)) {
     	GList *item = g_list_last(sq->searches);
-		smsg_t *sb = (smsg_t *) item->data;
+		smsg_t *sb = item->data;
 
 		sq->searches = g_list_remove_link(sq->searches, item);
 
@@ -513,9 +515,10 @@ cap_queue(squeue_t *sq)
 		sq->count--;
 		sq->n_dropped++;
 
-		if (dbg > 4)
-			printf("sq for node %s, dropped \"%s\" (%u left, %d dropped)\n",
-				node_addr(sq->node), QUERY_TEXT(pmsg_start(sb->mb)),
+		if (GNET_PROPERTY(sq_debug) > 4)
+			g_debug("sq for node %s, dropped \"%s\" (%u left, %d dropped)",
+				node_addr(sq->node),
+				gnutella_msg_search_get_text(pmsg_start(sb->mb)),
 				sq->count, sq->n_dropped);
 
 		sqh_remove(sq, sb->shandle);
@@ -535,7 +538,7 @@ sq_search_closed(squeue_t *sq, gnet_search_t sh)
 	GList *next;
 
 	for (l = sq->searches; l; l = next) {
-		smsg_t *sb = (smsg_t *) l->data;
+		smsg_t *sb = l->data;
 
 		next = g_list_next(l);
 
@@ -546,10 +549,10 @@ sq_search_closed(squeue_t *sq, gnet_search_t sh)
 		sq->count--;
 		sq->searches = g_list_remove_link(sq->searches, l);
 
-		if (dbg > 4)
-			printf("sq for node %s, dropped \"%s\" on search close (%u left)\n",
+		if (GNET_PROPERTY(sq_debug) > 4)
+			g_debug("sq for node %s, dropped \"%s\" on search close (%u left)",
 				sq->node ? node_addr(sq->node) : "GLOBAL",
-				QUERY_TEXT(pmsg_start(sb->mb)), sq->count);
+				gnutella_msg_search_get_text(pmsg_start(sb->mb)), sq->count);
 
 		sqh_remove(sq, sb->shandle);
 		smsg_discard(sb);
