@@ -1,8 +1,8 @@
 /*
- * $Id: thex_download.c 14877 2007-09-10 11:50:11Z cbiere $
+ * $Id: thex_download.c 19190 2011-05-30 22:09:25Z rmanfredi $
  *
- * Copyright (c) 2005, Raphael Manfredi
  * Copyright (c) 2005, Martijn van Oosterhout <kleptog@svana.org>
+ * Copyright (c) 2005, 2011 Raphael Manfredi
  *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
@@ -30,37 +30,45 @@
  *
  * Handles downloads of THEX data.
  *
- * @author Raphael Manfredi
  * @author Martijn van Oosterhout
  * @date 2005
+ *
+ * Conversion to built-in XML support.
+ *
+ * @author Raphael Manfredi
+ * @date 2005, 2011
  */
 
 #include "common.h"
 
-RCSID("$Id: thex_download.c 14877 2007-09-10 11:50:11Z cbiere $")
-
-#include <libxml/parser.h>                                                      
-#include <libxml/tree.h>                                                        
+RCSID("$Id: thex_download.c 19190 2011-05-30 22:09:25Z rmanfredi $")
 
 #include "bsched.h"
 #include "dime.h"
 #include "downloads.h"
-#include "pmsg.h"
 #include "rx_inflate.h"
 #include "thex.h"
 #include "thex_download.h"
 
+#include "xml/vxml.h"
+#include "xml/xnode.h"
+
 #include "if/gnet_property_priv.h"
 
 #include "lib/atoms.h"
+#include "lib/ascii.h"
 #include "lib/endian.h"
+#include "lib/parse.h"
+#include "lib/pmsg.h"
+#include "lib/stringify.h"
 #include "lib/tigertree.h"
+#include "lib/halloc.h"
 #include "lib/walloc.h"
 
 #include "lib/override.h"	/* Must be the last header included */
 
 #define THEX_DOWNLOAD_DEFAULT_SIZE	4096	/* Default data buffer size */
-#define THEX_DOWNLOAD_MAX_SIZE		(64 * 1024)
+#define THEX_DOWNLOAD_MAX_SIZE		(260 * 1024)	/* 256 KiB + overhead */
 
 struct thex_download {
 	gpointer owner;					/**< Download owning us */
@@ -69,6 +77,7 @@ struct thex_download {
 	char *data;						/**< Where payload data is stored */
 	size_t data_size;				/**< Size of data buffer */
 	size_t pos;						/**< Reading position */
+	size_t max_size;				/**< Max size we'll read */
 	const struct sha1 *sha1;		/**< SHA1 atom; refers to described file */
 	const struct tth *tth;			/**< TTH atom; refers to described file */
 	struct tth *leaves;				/**< g_memdup()ed leave TTHs */
@@ -78,32 +87,6 @@ struct thex_download {
 	gboolean finished;
 };
 
-/** Get rid of the obnoxious (xmlChar *) */
-static inline char *
-xml_get_string(xmlNode *node, const char *id)
-{
-	return (char *) xmlGetProp(node, (const xmlChar *) id);
-}
-
-/**
- * Uses this to free strings returned by xml_get_string().
- */
-static inline void
-xml_string_free(char **p)
-{
-	g_assert(p);
-	if (*p) {
-		xmlFree(*p);
-		*p = NULL;
-	}
-}
-
-static inline const xmlChar *
-string_to_xmlChar(const char *p)
-{
-	return (const xmlChar *) p;
-}
-
 /**
  * Initialize the THEX download context.
  */
@@ -118,10 +101,10 @@ thex_download_create(gpointer owner, gnet_host_t *host,
 	g_return_val_if_fail(sha1, NULL);
 	g_return_val_if_fail(tth, NULL);
 
-	ctx = walloc(sizeof *ctx);
+	WALLOC(ctx);
 	*ctx = zero_ctx;
 	ctx->owner = owner;
-	ctx->host = *host;			/* Struct copy */
+	gnet_host_copy(&ctx->host, host);
 	ctx->sha1 = atom_sha1_get(sha1);
 	ctx->tth = atom_tth_get(tth);
 	ctx->filesize = filesize;
@@ -144,12 +127,12 @@ thex_download_data_read(struct thex_download *ctx, pmsg_t *mb)
 	g_assert(ctx->pos <= ctx->data_size);
 
 	while ((size = pmsg_size(mb)) > 0) {
-		if (ctx->pos + size > THEX_DOWNLOAD_MAX_SIZE)
+		if (ctx->pos + size > ctx->max_size)
 			return TRUE;
 
 		if (size > ctx->data_size - ctx->pos) {
 			ctx->data_size += MAX(size, ctx->data_size);
-			ctx->data = g_realloc(ctx->data, ctx->data_size);
+			ctx->data = hrealloc(ctx->data, ctx->data_size);
 		}
 		ctx->pos += pmsg_read(mb, &ctx->data[ctx->pos], size);
 	}
@@ -173,7 +156,7 @@ thex_download_data_ind(rxdrv_t *rx, pmsg_t *mb)
 	download_check(d);
 
 	/*
-	 * When we receive browse-host data with an advertised size, the remote
+	 * When we receive THEX data with an advertised size, the remote
 	 * end will simply stop emitting data when we're done and could maintain
 	 * the HTTP connection alive.  Therefore, since we don't intend to
 	 * issue any more request on that connection, we must check for completion.
@@ -194,84 +177,85 @@ thex_download_data_ind(rxdrv_t *rx, pmsg_t *mb)
 }
 
 /* XML helper functions */
-static xmlNode * 
-find_element_by_name(xmlNode *start, const char *name)
+static xnode_t * 
+find_element_by_name(xnode_t *p, const char *name)
 {
-   xmlNode *cur_node;
+   xnode_t *xn;
 
-    for (cur_node = start; cur_node; cur_node = cur_node->next) {
-        if (XML_ELEMENT_NODE == cur_node->type) {
-            if (0 == xmlStrcmp(cur_node->name, string_to_xmlChar(name))) {
-				return cur_node;
-			}
-        }
+    for (xn = xnode_first_child(p); xn != NULL; xn = xnode_next_sibling(xn)) {
+        if (xnode_is_element(xn) && 0 == strcmp(xnode_element_name(xn), name))
+			return xn;
     }
     return NULL;
 }
 
 static gboolean
-verify_element(xmlNode *node, const char *prop, const char *expect)
+verify_element(xnode_t *node, const char *prop, const char *expect)
 {
-	gboolean success = FALSE;
-	char *value;
+	const char *value;
 	
-	value = STRTRACK(xml_get_string(node, prop));
+	value = xnode_prop_get(node, prop);
   	if (NULL == value) {
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("Couldn't find property \"%s\" of node \"%s\"",
-				prop, node->name);
+			g_debug("TTH couldn't find property \"%s\" of node \"%s\"",
+				prop, xnode_element_name(node));
 		}
-		goto finish;
+		return FALSE;
 	}
 	if (0 != strcmp(value, expect)) {
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("Property %s/%s doesn't match expected value \"%s\", "
+			g_debug("TTH property %s/%s doesn't match expected value \"%s\", "
 				"got \"%s\"",
-				node->name, prop, expect, value);
+				xnode_element_name(node), prop, expect, value);
 		}
-		goto finish;
+		return FALSE;
 	}
-	success = TRUE;
 
-finish:
-	xml_string_free(&value);
-	return success;
+	return TRUE;
 }
 
 static char *
 thex_download_handle_xml(struct thex_download *ctx,
 	const char *data, size_t size)
 {
-	xmlNode *root, *hashtree, *node;
-	xmlDocPtr doc = NULL;
+	xnode_t *hashtree = NULL, *node;
 	char *hashtree_id = NULL;
 	gboolean success = FALSE;
+	vxml_parser_t *vp;
+	vxml_error_t e;
 
 	if (size <= 0) {
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("XML record has no data");
+			g_debug("TTH XML record has no data");
 		}
 		goto finish;
 	}
-	
-	doc = xmlReadMemory(data, size, "noname.xml", NULL, 0);
-	if (NULL == doc) {
+
+	/*
+	 * Parse the XML record.
+	 */
+
+	vp = vxml_parser_make("THEX record", VXML_O_STRIP_BLANKS);
+	vxml_parser_add_data(vp, data, size);
+	e = vxml_parse_tree(vp, &hashtree);
+	vxml_parser_free(vp);
+
+	if (VXML_E_OK != e) {
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("Cannot parse XML record");
+			g_warning("TTH cannot parse XML record: %s", vxml_strerror(e));
+			dump_hex(stderr, "XML record", data, size);
 		}
 		goto finish;
 	}
-	root = xmlDocGetRootElement(doc);
-	
-  	hashtree = find_element_by_name(root, "hashtree");
-	if (NULL == hashtree) {
+
+	if (0 != strcmp("hashtree", xnode_element_name(hashtree))) {
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("Couldn't find hashtree element");
+			g_debug("TTH couldn't find root hashtree element");
 		}
 		goto finish;
 	}
 	
-	node = find_element_by_name(hashtree->children, "file");
+	node = find_element_by_name(hashtree, "file");
 	if (node) {
 		if (!verify_element(node, "size", filesize_to_string(ctx->filesize)))
 			goto finish;
@@ -279,12 +263,12 @@ thex_download_handle_xml(struct thex_download *ctx,
 			goto finish;
 	} else {
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("Couldn't find hashtree/file element");
+			g_debug("TTH couldn't find hashtree/file element");
 		}
 		goto finish;
 	}
 
-	node = find_element_by_name(hashtree->children, "digest");
+	node = find_element_by_name(hashtree, "digest");
 	if (node) {
 		if (!verify_element(node, "algorithm", THEX_HASH_ALGO))
 			goto finish;
@@ -292,35 +276,34 @@ thex_download_handle_xml(struct thex_download *ctx,
 			goto finish;
 	} else {
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("Couldn't find hashtree/digest element");
+			g_debug("TTH couldn't find hashtree/digest element");
 		}
     	goto finish;
 	}
   
-	node = find_element_by_name(hashtree->children, "serializedtree");
+	node = find_element_by_name(hashtree, "serializedtree");
 	if (node) {
-		char *value;
+		const char *value;
 		int error;
 		
 		if (!verify_element(node, "type", THEX_TREE_TYPE))
     		goto finish;
 
-		value = STRTRACK(xml_get_string(node, "uri"));
+		value = xnode_prop_get(node, "uri");
 		if (NULL == value) {
 			if (GNET_PROPERTY(tigertree_debug)) {
-				g_message("Couldn't find property \"uri\" of node \"%s\"",
-					node->name);
+				g_debug("TTH couldn't find property \"uri\" of node \"%s\"",
+					xnode_element_name(node));
 			}
 			goto finish;
 		}
-		hashtree_id = g_strdup(value);
-		xml_string_free(&value);
+		hashtree_id = h_strdup(value);
 
-		value = STRTRACK(xml_get_string(node, "depth"));
+		value = xnode_prop_get(node, "depth");
 		if (NULL == value) {
 			if (GNET_PROPERTY(tigertree_debug)) {
-				g_message("Couldn't find property \"depth\" of node \"%s\"",
-					node->name);
+				g_debug("TTH couldn't find property \"depth\" of node \"%s\"",
+					xnode_element_name(node));
 			}
 			goto finish;
 		}
@@ -329,28 +312,24 @@ thex_download_handle_xml(struct thex_download *ctx,
 		error |= ctx->depth > tt_full_depth(ctx->filesize);
 		if (error) {
 			ctx->depth = 0;
-			g_message("Bad value for \"depth\" of node \"%s\": \"%s\"",
-				node->name, value);
+			g_warning("TTH bad value for \"depth\" of node \"%s\": \"%s\"",
+				xnode_element_name(node), value);
 		}
-		xml_string_free(&value);
 		if (error)
 			goto finish;
 	} else {
 		if (GNET_PROPERTY(tigertree_debug))
-			g_message("Couldn't find hashtree/serializedtree element");
+			g_debug("TTH couldn't find hashtree/serializedtree element");
 		goto finish;
 	}
 
 	success = TRUE;
 
 finish:
-	if (doc) {
-		xmlFreeDoc(doc);
-		doc = NULL;
-	}
-	if (!success) {
-		G_FREE_NULL(hashtree_id);
-	}
+	if (!success)
+		HFREE_NULL(hashtree_id);
+	xnode_tree_free_null(&hashtree);
+
 	return hashtree_id;
 }
 
@@ -360,32 +339,33 @@ thex_download_handle_hashtree(struct thex_download *ctx,
 {
 	gboolean success = FALSE;
 	size_t n_nodes, n_leaves, n, start;
-	unsigned good_depth, min_depth;
+	unsigned good_depth;
 	const struct tth *leaves;
 	struct tth tth;
 
 	if (size <= 0) {
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("Hashtree record has no data");
+			g_debug("TTH hashtree record has no data");
 		}
 		goto finish;
 	}
 	if (size < TTH_RAW_SIZE) {
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("Hashtree record is too small");
+			g_debug("TTH hashtree record is too small");
 		}
 		goto finish;
 	}
 	if (size % TTH_RAW_SIZE) {
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("Hashtree has bad size");
+			g_debug("TTH hashtree has bad size");
 		}
 		goto finish;
 	}
 	memcpy(tth.data, data, TTH_RAW_SIZE);
 	if (!tth_eq(&tth, ctx->tth)) {
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("Hashtree has different root hash %s", tth_base32(&tth));
+			g_debug("TTH hashtree has different root hash %s",
+				tth_base32(&tth));
 		}
 		goto finish;
 	}
@@ -393,9 +373,8 @@ thex_download_handle_hashtree(struct thex_download *ctx,
 	n_nodes = size / TTH_RAW_SIZE;
 	n_leaves = tt_node_count_at_depth(ctx->filesize, ctx->depth);
 
-	/* Shareaza use a fixed depth of 9, allow one level less like others */
+	/* Shareaza uses a fixed depth of 9, allow one level less like others */
 	good_depth = tt_good_depth(ctx->filesize);
-	min_depth = good_depth - (good_depth > 0);
 
 	ctx->depth = MIN(ctx->depth, good_depth);
 	if (n_nodes < n_leaves * 2 - 1) {
@@ -407,17 +386,16 @@ thex_download_handle_hashtree(struct thex_download *ctx,
 			ctx->depth--;
 		}
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("Calculated depth of hashtree: %u", ctx->depth);
+			g_debug("TTH calculated depth of hashtree: %u", ctx->depth);
 		}
 		n_leaves = tt_node_count_at_depth(ctx->filesize, ctx->depth);
 	}
 
-	if (ctx->depth < min_depth) {
+	if (ctx->depth < good_depth) {
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("Tree depth (%u) is below the acceptable depth (%u)",
-				ctx->depth, min_depth);
+			g_debug("TTH tree depth (%u) is below the good depth (%u)",
+				ctx->depth, good_depth);
 		}
-		goto finish;
 	}
 
 	start = 0;
@@ -429,7 +407,7 @@ thex_download_handle_hashtree(struct thex_download *ctx,
 
 	if (n_nodes < start + n_leaves) {
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("Hashtree has too few nodes "
+			g_debug("TTH hashtree has too few nodes "
 				"(filesize=%s depth=%u nodes=%lu expected=%lu)",
 				filesize_to_string(ctx->filesize),
 				ctx->depth,
@@ -445,7 +423,8 @@ thex_download_handle_hashtree(struct thex_download *ctx,
 	tth = tt_root_hash(leaves, n_leaves);
 	if (!tth_eq(&tth, ctx->tth)) {
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("Hashtree does not match root hash %s", tth_base32(&tth));
+			g_debug("TTH hashtree does not match root hash %s",
+				tth_base32(&tth));
 		}
 		goto finish;
 	}
@@ -508,7 +487,7 @@ dime_find_record(const GSList *records, const char *type, const char *id)
 	}
 
 	if (GNET_PROPERTY(tigertree_debug)) {
-		g_message("Could not find record (type=\"%s\", id=%s%s%s)",
+		g_debug("TTH could not find record (type=\"%s\", id=%s%s%s)",
 			type,
 			id ? "\"" : "",
 			id ? id : "<none>",
@@ -533,6 +512,11 @@ thex_download_finished(struct thex_download *ctx)
 	g_assert(ctx->pos <= ctx->data_size);
 	ctx->data_size = ctx->pos;	/* Amount which is actually valid */
 
+	if (NULL == ctx->data) {
+		records = NULL;
+		goto finish;			/* Did not receive anything */
+	}
+
 	records = dime_parse_records(ctx->data, ctx->data_size);
 	if (records) {
 		const struct dime_record *record;
@@ -553,7 +537,7 @@ thex_download_finished(struct thex_download *ctx)
 		hashtree_id = thex_download_handle_xml(ctx, data, size);
 		if (NULL == hashtree_id) {
 			if (GNET_PROPERTY(tigertree_debug)) {
-				g_message("Could not determine hashtree ID");
+				g_debug("TTH could not determine hashtree ID");
 				dump_hex(stderr, "THEX data", ctx->data, ctx->data_size);
 			}
 			/* Bug workaround:
@@ -570,7 +554,7 @@ thex_download_finished(struct thex_download *ctx)
 			 */
 			record = dime_find_record(records, THEX_TREE_TYPE, NULL);
 		}
-		G_FREE_NULL(hashtree_id);
+		HFREE_NULL(hashtree_id);
 
 		if (NULL == record) {
 			if (GNET_PROPERTY(tigertree_debug)) {
@@ -587,7 +571,7 @@ thex_download_finished(struct thex_download *ctx)
 
 	} else {
 		if (GNET_PROPERTY(tigertree_debug)) {
-			g_message("Could not parse DIME records");
+			g_debug("TTH could not parse DIME records");
 			dump_hex(stderr, "THEX data", ctx->data, ctx->data_size);
 		}
 		goto finish;
@@ -596,7 +580,7 @@ thex_download_finished(struct thex_download *ctx)
 
 finish:
 	dime_list_free(&records);
-	G_FREE_NULL(ctx->data);
+	HFREE_NULL(ctx->data);
 	return success;
 }
 
@@ -656,17 +640,18 @@ static const struct rx_inflate_cb thex_rx_inflate_cb = {
 };
 
 /**
- * Prepare reception of query hit data by building an appropriate RX stack.
+ * Prepare reception of THEX data by building an appropriate RX stack.
  *
  * @return TRUE if we may continue with the download.
  */
 gboolean
 thex_download_receive(struct thex_download *ctx,
+	filesize_t content_length,
 	gnet_host_t *host, struct wrap_io *wio, guint32 flags)
 {
 	g_assert(ctx != NULL);
 
-	ctx->host = *host;			/* Struct copy */
+	gnet_host_copy(&ctx->host, host);
 
 	/*
 	 * Freeing of the RX stack must be asynchronous: each time we establish
@@ -680,11 +665,25 @@ thex_download_receive(struct thex_download *ctx,
 		ctx->rx = NULL;
 	}
 
+	/*
+	 * If there is a Content-Length indication in the HTTP reply, it is
+	 * supplied here and will be used as a limit of the data we'll read.
+	 *
+	 * If there was none (for instance if the output is chunked), then 0
+	 * is given and we'll use a hardwired maximum.
+	 */
+
+	if (content_length > MAX_INT_VAL(size_t))
+		return FALSE;
+
+	ctx->max_size = content_length ?
+		(size_t) content_length : THEX_DOWNLOAD_MAX_SIZE;
+
 	{
 		struct rx_link_args args;
 
 		args.cb = &thex_rx_link_cb;
-		args.bws = BSCHED_BWS_IN;
+		args.bws = bsched_in_select_by_addr(gnet_host_get_addr(&ctx->host));
 		args.wio = wio;
 
 		ctx->rx = rx_make(ctx, &ctx->host, rx_link_get_ops(), &args);
@@ -763,7 +762,7 @@ thex_download_close(struct thex_download *ctx)
 }
 
 /**
- * Terminate host browsing.
+ * Terminate THEX download.
  */
 void
 thex_download_free(struct thex_download **ptr)
@@ -775,11 +774,11 @@ thex_download_free(struct thex_download **ptr)
 			rx_free(ctx->rx);
 			ctx->rx = NULL;
 		}
-		G_FREE_NULL(ctx->data);
+		HFREE_NULL(ctx->data);
 		G_FREE_NULL(ctx->leaves);
 		atom_sha1_free_null(&ctx->sha1);
 		atom_tth_free_null(&ctx->tth);
-		wfree(ctx, sizeof *ctx);
+		WFREE(ctx);
 		*ptr = NULL;
 	}
 }
