@@ -1,5 +1,5 @@
 /*
- * $Id: tsync.h 8376 2005-06-25 01:37:45Z daichik $
+ * $Id: tsync.h 18899 2011-03-30 15:17:39Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -36,7 +36,7 @@
 #ifndef _core_tsync_h_
 #define _core_tsync_h_
 
-#include <glib.h>
+#include "common.h"
 
 #include "lib/tm.h"
 
@@ -45,11 +45,12 @@
  */
 
 struct gnutella_node;
+struct nid;
 
 void tsync_init(void);
 void tsync_close(void);
 
-void tsync_send(struct gnutella_node *n, guint32 node_id);
+void tsync_send(struct gnutella_node *n, const struct nid *node_id);
 void tsync_send_timestamp(tm_t *orig, tm_t *final);
 void tsync_got_request(struct gnutella_node *n, tm_t *got);
 void tsync_got_reply(struct gnutella_node *n,
