@@ -1,5 +1,5 @@
 /*
- * $Id: tx.c 12141 2006-10-27 01:17:39Z cbiere $
+ * $Id: tx.c 19142 2011-05-23 20:44:04Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -40,10 +40,12 @@
 
 #include "common.h"
 
-RCSID("$Id: tx.c 12141 2006-10-27 01:17:39Z cbiere $")
+RCSID("$Id: tx.c 19142 2011-05-23 20:44:04Z rmanfredi $")
 
 #include "tx.h"
 #include "nodes.h"
+
+#include "lib/glib-missing.h"
 #include "lib/walloc.h"
 #include "lib/override.h"	/* Must be the last header included */
 
@@ -51,18 +53,18 @@ RCSID("$Id: tx.c 12141 2006-10-27 01:17:39Z cbiere $")
  * Dynamic dispatch of polymorphic routines.
  */
 
-#define TX_INIT(o,a)		((o)->ops->init((o), (a)))
-#define TX_DESTROY(o)		((o)->ops->destroy((o)))
-#define TX_WRITE(o,d,l)		((o)->ops->write((o), (d), (l)))
-#define TX_WRITEV(o,i,c)	((o)->ops->writev((o), (i), (c)))
-#define TX_SENDTO(o,t,d,l)	((o)->ops->sendto((o), (t), (d), (l)))
-#define TX_ENABLE(o)		((o)->ops->enable((o)))
-#define TX_DISABLE(o)		((o)->ops->disable((o)))
-#define TX_PENDING(o)		((o)->ops->pending((o)))
-#define TX_BIO_SOURCE(o)	((o)->ops->bio_source((o)))
-#define TX_FLUSH(o)			((o)->ops->flush((o)))
-#define TX_SHUTDOWN(o)		((o)->ops->shutdown((o)))
-#define TX_CLOSE(o,c,a)		((o)->ops->close((o), (c), (a)))
+#define TX_INIT(o,a)		((*(o)->ops->init)((o), (a)))
+#define TX_DESTROY(o)		((*(o)->ops->destroy)((o)))
+#define TX_WRITE(o,d,l)		((*(o)->ops->write)((o), (d), (l)))
+#define TX_WRITEV(o,i,c)	((*(o)->ops->writev)((o), (i), (c)))
+#define TX_SENDTO(o,t,d,l)	((*(o)->ops->sendto)((o), (t), (d), (l)))
+#define TX_ENABLE(o)		((*(o)->ops->enable)((o)))
+#define TX_DISABLE(o)		((*(o)->ops->disable)((o)))
+#define TX_PENDING(o)		((*(o)->ops->pending)((o)))
+#define TX_BIO_SOURCE(o)	((*(o)->ops->bio_source)((o)))
+#define TX_FLUSH(o)			((*(o)->ops->flush)((o)))
+#define TX_SHUTDOWN(o)		((*(o)->ops->shutdown)(o))
+#define TX_CLOSE(o,c,a)		((*(o)->ops->close)((o), (c), (a)))
 
 /**
  * To guarantee that destruction of the stack always happens asynchronously
@@ -78,7 +80,7 @@ static GSList *tx_freed = NULL;
  * @return NULL if there is an initialization problem.
  */
 txdrv_t *
-tx_make(gpointer owner, gnet_host_t *host,
+tx_make(gpointer owner, const gnet_host_t *host,
 	const struct txdrv_ops *ops, gpointer args)
 {
 	txdrv_t *tx;
@@ -86,8 +88,7 @@ tx_make(gpointer owner, gnet_host_t *host,
 	g_assert(owner);
 	g_assert(ops);
 
-	tx = walloc0(sizeof(*tx));
-
+	WALLOC0(tx);
 	tx->owner = owner;
 	tx->host = *host;					/* stuct copy */
 
@@ -128,10 +129,9 @@ tx_make_above(txdrv_t *ltx, const struct txdrv_ops *ops, gpointer args)
 	g_assert(ltx->upper == NULL);		/* Nothing above yet */
 	g_assert(ops);
 
-	tx = walloc0(sizeof(*tx));
-
+	WALLOC0(tx);
 	tx->owner = ltx->owner;
-	tx->host = ltx->host;				/* Struct copy */
+	gnet_host_copy(&tx->host, &ltx->host);
 	tx->ops = ops;
 	tx->upper = NULL;
 	tx->lower = ltx;
@@ -184,7 +184,7 @@ tx_deep_free(txdrv_t *tx)
 		tx_deep_free(tx->lower);
 
 	TX_DESTROY(tx);
-	wfree(tx, sizeof(*tx));
+	WFREE(tx);
 }
 
 /**
@@ -222,8 +222,7 @@ tx_collect(void)
 		tx_deep_free(tx);
 	}
 
-	g_slist_free(tx_freed);
-	tx_freed = NULL;
+	gm_slist_free_null(&tx_freed);
 }
 
 /**
@@ -250,7 +249,7 @@ tx_write(txdrv_t *tx, gconstpointer data, size_t len)
  * @return amount of bytes written, or -1 on error with errno set.
  */
 ssize_t
-tx_writev(txdrv_t *tx, struct iovec *iov, gint iovcnt)
+tx_writev(txdrv_t *tx, iovec_t *iov, int iovcnt)
 {
 	g_assert(tx);
 
@@ -317,7 +316,7 @@ tx_srv_disable(txdrv_t *tx)
 {
 	g_assert(tx);
 	g_assert(tx->srv_routine != NULL);
-	g_assert(tx->flags & TX_SERVICE);
+	g_return_if_fail(tx->flags & TX_SERVICE);
 
 	/*
 	 * In "eager mode", servicing is forced and cannot be disabled that way.
@@ -479,7 +478,7 @@ tx_close_next(txdrv_t *tx, gpointer arg)
 		cb = carg->cb;
 		arg2 = carg->arg;
 
-		wfree(carg, sizeof(*carg));
+		WFREE(carg);
 
 		(*cb)(top, arg2);
 		return;
@@ -510,7 +509,7 @@ tx_close(txdrv_t *tx, tx_closed_t cb, gpointer arg)
 	g_assert(tx);
 	g_assert(tx->upper == NULL);
 
-	carg = walloc(sizeof(*carg));
+	WALLOC(carg);
 	carg->top = tx;
 	carg->cb = cb;
 	carg->arg = arg;
@@ -561,7 +560,7 @@ tx_no_write(txdrv_t *unused_tx, gconstpointer unused_data, size_t unused_len)
  * The writev() operation is forbidden.
  */
 ssize_t
-tx_no_writev(txdrv_t *unused_tx, struct iovec *unused_iov, gint unused_iovcnt)
+tx_no_writev(txdrv_t *unused_tx, iovec_t *unused_iov, int unused_iovcnt)
 {
 	(void) unused_tx;
 	(void) unused_iov;
