@@ -1,5 +1,5 @@
 /*
- * $Id: tx_chunk.c,v 1.12 2005/10/31 06:31:48 cbiere Exp $
+ * $Id: tx_chunk.c 19142 2011-05-23 20:44:04Z rmanfredi $
  *
  * Copyright (c) 2005, Raphael Manfredi
  *
@@ -55,7 +55,7 @@
 
 #include "common.h"
 
-RCSID("$Id: tx_chunk.c,v 1.12 2005/10/31 06:31:48 cbiere Exp $");
+RCSID("$Id: tx_chunk.c 19142 2011-05-23 20:44:04Z rmanfredi $")
 
 #include "tx.h"
 #include "tx_chunk.h"
@@ -73,13 +73,13 @@ RCSID("$Id: tx_chunk.c,v 1.12 2005/10/31 06:31:48 cbiere Exp $");
  * upper layers until we have some room to send them.
  */
 
-#define CHUNK_DIGITS	4		/* At most that many digits in hexa */
+#define CHUNK_DIGITS 16 /* At most that many digits in hexa; 64-bit */
 
 /*
  * Private attributes for the link.
  */
 struct attr {
-	gchar head[CHUNK_DIGITS+5];	/**< Chunk header: hexa size + 2CRLF + NUL */
+	char head[CHUNK_DIGITS + 5];/**< Chunk header: hexa size + 2CRLF + NUL */
 	ssize_t head_len;			/**< Length of chunk header */
 	ssize_t head_remain;		/**< Amount of unwritten header data */
 	ssize_t data_remain;		/**< Data required to complete chunk */
@@ -141,14 +141,14 @@ static ssize_t
 chunk_begin(txdrv_t *tx, size_t len, gboolean final)
 {
 	struct attr *attr = tx->opaque;
-	gint hlen = 0;
+	size_t hlen = 0;
 
 	g_assert(0 == attr->data_remain);
 	g_assert(0 == attr->head_remain);
 	g_assert(final || ((size_t) -1 != len && len > 0));
 
-	if (dbg > 9)
-		printf("chunk_begin: %s chunk %lu byte%s\n",
+	if (GNET_PROPERTY(tx_debug) > 9)
+		g_debug("chunk_begin: %s chunk %lu byte%s",
 			final ? "final" :
 			attr->first ? "first" :
 			"next", (gulong) len, len == 1 ? "" : "s");
@@ -272,7 +272,7 @@ tx_chunk_init(txdrv_t *tx, gpointer unused_args)
 	(void) unused_args;
 	g_assert(tx);
 
-	attr = walloc(sizeof *attr);
+	WALLOC(attr);
 
 	attr->head_remain = 0;		/* No committed length yet */
 	attr->data_remain = 0;		/* No data yet */
@@ -297,7 +297,7 @@ tx_chunk_destroy(txdrv_t *tx)
 {
 	struct attr *attr = tx->opaque;
 
-	wfree(attr, sizeof *attr);
+	WFREE(attr);
 }
 
 /**
@@ -306,11 +306,11 @@ tx_chunk_destroy(txdrv_t *tx)
  * @return amount of data bytes written, or -1 on error.
  */
 static ssize_t
-tx_chunk_write(txdrv_t *tx, gpointer data, size_t len)
+tx_chunk_write(txdrv_t *tx, gconstpointer data, size_t len)
 {
 	struct attr *attr = tx->opaque;
 	size_t remain = len;
-	gchar *ptr = data;
+	const char *ptr = data;
 	size_t written = 0;
 
 	g_assert((size_t) -1 != len && len > 0);
@@ -358,7 +358,7 @@ tx_chunk_write(txdrv_t *tx, gpointer data, size_t len)
  * @return amount of bytes written, or -1 on error.
  */
 static ssize_t
-tx_chunk_writev(txdrv_t *tx, struct iovec *iov, gint iovcnt)
+tx_chunk_writev(txdrv_t *tx, iovec_t *iov, int iovcnt)
 {
 	ssize_t written = 0;
 
@@ -368,12 +368,13 @@ tx_chunk_writev(txdrv_t *tx, struct iovec *iov, gint iovcnt)
 	 */
 
 	while (iovcnt--) {
-		ssize_t r = tx_chunk_write(tx, iov->iov_base, iov->iov_len);
+		ssize_t r = tx_chunk_write(tx, 
+			  iovec_base(iov), iovec_len(iov));
 		if (-1 == r)
 			return -1;
 		if (r > 0)
 			written += r;
-		if ((size_t) r != iov->iov_len)		/* Not able to write everything */
+		if ((size_t) r != iovec_len(iov))	/* Not able to write everything */
 			break;							/* Lower-level flow-controls us */
 		iov++;
 	}
