@@ -1,5 +1,5 @@
 /*
- * $Id: tx_deflate.c 11957 2006-09-13 23:27:04Z cbiere $
+ * $Id: tx_deflate.c 19262 2011-06-15 10:34:36Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -37,12 +37,15 @@
 
 #include "common.h"
 
-RCSID("$Id: tx_deflate.c 11957 2006-09-13 23:27:04Z cbiere $")
+RCSID("$Id: tx_deflate.c 19262 2011-06-15 10:34:36Z rmanfredi $")
+
+#include <zlib.h>
 
-#include "sockets.h"
-#include "hosts.h"
 #include "tx.h"
 #include "tx_deflate.h"
+#include "hosts.h"
+#include "settings.h"
+#include "sockets.h"
 
 #include "if/gnet_property_priv.h"
 
@@ -67,10 +70,10 @@ RCSID("$Id: tx_deflate.c 11957 2006-09-13 23:27:04Z cbiere $")
 #define BUFFER_NAGLE	200		/**< 200 ms */
 
 struct buffer {
-	gchar *arena;				/**< Buffer arena */
-	gchar *end;					/**< First byte outside buffer */
-	gchar *wptr;				/**< Write pointer (first byte to write) */
-	gchar *rptr;				/**< Read pointer (first byte to read) */
+	char *arena;				/**< Buffer arena */
+	char *end;					/**< First byte outside buffer */
+	char *wptr;				/**< Write pointer (first byte to write) */
+	char *rptr;				/**< Read pointer (first byte to read) */
 };
 
 /*
@@ -80,15 +83,15 @@ struct attr {
 	struct buffer buf[BUFFER_COUNT];
 	size_t buffer_size;			/**< Buffer size used */
 	size_t buffer_flush;		/**< Flush after that many bytes */
-	gint fill_idx;				/**< Filled buffer index */
-	gint send_idx;				/**< Buffer to be sent */
+	int fill_idx;				/**< Filled buffer index */
+	int send_idx;				/**< Buffer to be sent */
 	z_streamp outz;				/**< Compressing stream */
 	txdrv_t *nd;				/**< Network driver, underneath us */
 	size_t unflushed;			/**< Amount of bytes written since last flush */
-	gint flags;					/**< Operating flags */
+	int flags;					/**< Operating flags */
 	cqueue_t *cq;				/**< The callout queue to use for Nagle */
-	gpointer tm_ev;				/**< The timer event */
-	struct tx_deflate_cb *cb;	/**< Layer-specific callbacks */
+	cevent_t *tm_ev;			/**< The timer event */
+	const struct tx_deflate_cb *cb;	/**< Layer-specific callbacks */
 	tx_closed_t closed;			/**< Callback to invoke when layer closed */
 	gpointer closed_arg;		/**< Argument for closing routine */
 	gboolean nagle;				/**< Whether to use Nagle or not */
@@ -140,12 +143,12 @@ deflate_send(txdrv_t *tx)
 
 	r = tx_write(tx->lower, b->rptr, len);
 
-	if (dbg > 9)
-		printf("deflate_send: (%s) wrote %d bytes (buffer #%d) [%c%c]\n",
-			host_to_string(&tx->host), (gint) r, attr->send_idx,
+	if (GNET_PROPERTY(tx_debug) > 9) {
+		g_debug("deflate_send: (%s) wrote %lu bytes (buffer #%d) [%c%c]",
+			gnet_host_to_string(&tx->host), (unsigned long) r, attr->send_idx,
 			(attr->flags & DF_FLOWC) ? 'C' : '-',
 			(attr->flags & DF_FLUSH) ? 'f' : '-');
-
+	}
 	if ((ssize_t) -1 == r) {
 		tx->flags |= TX_ERROR;
 		return;
@@ -156,10 +159,10 @@ deflate_send(txdrv_t *tx)
 	 */
 
 	if ((size_t) r == len) {
-		if (dbg > 9)
-			printf("deflate_send: (%s) buffer #%d is empty\n",
-				host_to_string(&tx->host), attr->send_idx);
-
+		if (GNET_PROPERTY(tx_debug) > 9) {
+			g_debug("deflate_send: (%s) buffer #%d is empty",
+				gnet_host_to_string(&tx->host), attr->send_idx);
+		}
 		attr->send_idx = -1;			/* Signals: is now free */
 		b->wptr = b->rptr = b->arena;	/* Buffer is now empty */
 		return;
@@ -206,9 +209,7 @@ deflate_nagle_stop(txdrv_t *tx)
 	g_assert(attr->flags & DF_NAGLE);
 	g_assert(NULL != attr->tm_ev);
 
-	cq_cancel(attr->cq, attr->tm_ev);
-
-	attr->tm_ev = NULL;
+	cq_cancel(&attr->tm_ev);
 	attr->flags &= ~DF_NAGLE;
 }
 
@@ -239,12 +240,12 @@ deflate_rotate_and_send(txdrv_t *tx)
 	if (attr->fill_idx >= BUFFER_COUNT)
 		attr->fill_idx = 0;
 
-	if (dbg > 9)
-		printf("deflate_rotate_and_send: (%s) fill buffer now #%d [%c%c]\n",
-			host_to_string(&tx->host), attr->fill_idx,
+	if (GNET_PROPERTY(tx_debug) > 9) {
+		g_debug("deflate_rotate_and_send: (%s) fill buffer now #%d [%c%c]",
+			gnet_host_to_string(&tx->host), attr->fill_idx,
 			(attr->flags & DF_FLOWC) ? 'C' : '-',
 			(attr->flags & DF_FLUSH) ? 'f' : '-');
-
+	}
 	deflate_send(tx);
 }
 
@@ -259,18 +260,28 @@ deflate_flush(txdrv_t *tx)
 	struct attr *attr = tx->opaque;
 	z_streamp outz = attr->outz;
 	struct buffer *b;
-	gint ret;
-	gint old_avail;
+	int ret;
+	int old_avail;
+
+	/*
+	 * Normally we come here with something already written, so outz->next_in
+	 * should be initialized.  If it is not the case, then it means we
+	 * never wrote anything, so there is nothing for us to do: we're simply
+	 * called through tx_deflate_close().
+	 */
+
+	if (NULL == outz->next_in)
+		goto done;
 
 retry:
 	b = &attr->buf[attr->fill_idx];	/* Buffer we fill */
 
-	if (dbg > 9)
-		printf("deflate_flush: (%s) flushing (buffer #%d) [%c%c]\n",
-			host_to_string(&tx->host), attr->fill_idx,
+	if (GNET_PROPERTY(tx_debug) > 9) {
+		g_debug("deflate_flush: (%s) flushing (buffer #%d) [%c%c]",
+			gnet_host_to_string(&tx->host), attr->fill_idx,
 			(attr->flags & DF_FLOWC) ? 'C' : '-',
 			(attr->flags & DF_FLUSH) ? 'f' : '-');
-
+	}
 	/*
 	 * Prepare call to deflate().
 	 *
@@ -284,7 +295,6 @@ retry:
 	outz->avail_in = 0;
 
 	g_assert(outz->avail_out > 0);
-	g_assert(outz->next_in);		/* We previously wrote something */
 
 	ret = deflate(outz, (tx->flags & TX_CLOSING) ? Z_FINISH : Z_SYNC_FLUSH);
 
@@ -299,7 +309,7 @@ retry:
 		tx->flags |= TX_ERROR;
 
 		/* XXX: The callback must not destroy the tx! */
-		attr->cb->shutdown(tx->owner, "Compression flush failed: %s",
+		(*attr->cb->shutdown)(tx->owner, "Compression flush failed: %s",
 				zlib_strerror(ret));
 		return FALSE;
 	}
@@ -325,10 +335,10 @@ retry:
 		if (attr->send_idx >= 0) {				/* Send buffer not sent yet */
 			attr->flags |= DF_FLOWC|DF_FLUSH;	/* Enter flow control */
 
-			if (dbg > 4)
-				printf("Compressing TX stack for peer %s enters FLOWC/FLUSH\n",
-					host_to_string(&tx->host));
-
+			if (GNET_PROPERTY(tx_debug) > 4) {
+				g_debug("compressing TX stack for peer %s enters FLOWC/FLUSH",
+					gnet_host_to_string(&tx->host));
+			}
 			return TRUE;
 		}
 
@@ -340,6 +350,8 @@ retry:
 		goto retry;
 	}
 
+done:
+
 	attr->unflushed = 0;
 	attr->flags &= ~DF_FLUSH;
 
@@ -385,13 +397,13 @@ deflate_nagle_timeout(cqueue_t *unused_cq, gpointer arg)
 	(void) unused_cq;
 	if (-1 != attr->send_idx) {		/* Send buffer still incompletely sent */
 
-		if (dbg > 9)
-			printf("deflate_nagle_timeout: (%s) buffer #%d unsent,"
-				" exiting [%c%c]\n",
-				host_to_string(&tx->host), attr->send_idx,
+		if (GNET_PROPERTY(tx_debug) > 9) {
+			g_debug("deflate_nagle_timeout: (%s) buffer #%d unsent,"
+				" exiting [%c%c]",
+				gnet_host_to_string(&tx->host), attr->send_idx,
 				(attr->flags & DF_FLOWC) ? 'C' : '-',
 				(attr->flags & DF_FLUSH) ? 'f' : '-');
-
+		}
 
 		attr->tm_ev =
 			cq_insert(attr->cq, BUFFER_NAGLE, deflate_nagle_timeout, tx);
@@ -401,12 +413,11 @@ deflate_nagle_timeout(cqueue_t *unused_cq, gpointer arg)
 	attr->flags &= ~DF_NAGLE;
 	attr->tm_ev = NULL;
 
-	if (dbg > 9) {
-		printf("deflate_nagle_timeout: (%s) flushing (buffer #%d) [%c%c]\n",
-			host_to_string(&tx->host), attr->fill_idx,
+	if (GNET_PROPERTY(tx_debug) > 9) {
+		g_debug("deflate_nagle_timeout: (%s) flushing (buffer #%d) [%c%c]",
+			gnet_host_to_string(&tx->host), attr->fill_idx,
 			(attr->flags & DF_FLOWC) ? 'C' : '-',
 			(attr->flags & DF_FLUSH) ? 'f' : '-');
-		fflush(stdout);
 	}
 
 	deflate_flush_send(tx);
@@ -418,30 +429,29 @@ deflate_nagle_timeout(cqueue_t *unused_cq, gpointer arg)
  *
  * @return the amount of input bytes that were consumed ("added"), -1 on error.
  */
-static gint
-deflate_add(txdrv_t *tx, gconstpointer data, gint len)
+static int
+deflate_add(txdrv_t *tx, gconstpointer data, int len)
 {
 	struct attr *attr = tx->opaque;
 	z_streamp outz = attr->outz;
-	gint added = 0;
+	int added = 0;
 
-	if (dbg > 9) {
-		printf("deflate_add: (%s) given %lu bytes (buffer #%d, nagle %s, "
-			"unflushed %lu) [%c%c]\n",
-			host_to_string(&tx->host), (gulong) len, attr->fill_idx,
+	if (GNET_PROPERTY(tx_debug) > 9) {
+		g_debug("deflate_add: (%s) given %lu bytes (buffer #%d, nagle %s, "
+			"unflushed %lu) [%c%c]",
+			gnet_host_to_string(&tx->host), (gulong) len, attr->fill_idx,
 			(attr->flags & DF_NAGLE) ? "on" : "off", (gulong) attr->unflushed,
 			(attr->flags & DF_FLOWC) ? 'C' : '-',
 			(attr->flags & DF_FLUSH) ? 'f' : '-');
-		fflush(stdout);
 	}
 
 	while (added < len) {
 		struct buffer *b = &attr->buf[attr->fill_idx];	/* Buffer we fill */
-		gint ret;
-		gint old_added = added;
+		int ret;
+		int old_added = added;
 		gboolean flush_started = (attr->flags & DF_FLUSH) ? TRUE : FALSE;
-		gint old_avail;
-		const gchar *in, *old_in;
+		int old_avail;
+		const char *in, *old_in;
 
 		/*
 		 * Prepare call to deflate().
@@ -469,7 +479,7 @@ deflate_add(txdrv_t *tx, gconstpointer data, gint len)
 
 		if (Z_OK != ret) {
 			attr->flags |= DF_SHUTDOWN;
-			attr->cb->shutdown(tx->owner, "Compression failed: %s",
+			(*attr->cb->shutdown)(tx->owner, "Compression failed: %s",
 				zlib_strerror(ret));
 			return -1;
 		}
@@ -479,7 +489,7 @@ deflate_add(txdrv_t *tx, gconstpointer data, gint len)
 		 */
 
 		b->wptr = cast_to_gpointer(outz->next_out);
-		added = cast_to_gchar_ptr(outz->next_in) - in;
+		added = ptr_diff(outz->next_in, in);
 
 		g_assert(added >= old_added);
 
@@ -491,7 +501,7 @@ deflate_add(txdrv_t *tx, gconstpointer data, gint len)
 		if (attr->gzip.enabled) {
 			size_t r;
 
-			r = cast_to_gchar_ptr(outz->next_in) - old_in;
+			r = ptr_diff(outz->next_in, old_in);
 			attr->gzip.size += r;
 			attr->gzip.crc = crc32(attr->gzip.crc,
 								cast_to_gconstpointer(old_in), r);
@@ -507,10 +517,10 @@ deflate_add(txdrv_t *tx, gconstpointer data, gint len)
 			if (attr->send_idx >= 0) {
 				attr->flags |= DF_FLOWC;	/* Enter flow control */
 
-				if (dbg > 4)
-					printf("Compressing TX stack for peer %s enters FLOWC\n",
-						host_to_string(&tx->host));
-
+				if (GNET_PROPERTY(tx_debug) > 4) {
+					g_debug("compressing TX stack for peer %s enters FLOWC",
+						gnet_host_to_string(&tx->host));
+				}
 				return added;
 			}
 
@@ -571,16 +581,17 @@ deflate_service(gpointer data)
 
 	g_assert(attr->send_idx < BUFFER_COUNT);
 
-	if (dbg > 9)
-		printf("deflate_service: (%s) %s(buffer #%d, %d bytes held) [%c%c]\n",
-			host_to_string(&tx->host), (tx->flags & TX_ERROR) ? "ERROR " : "",
+	if (GNET_PROPERTY(tx_debug) > 9) {
+		g_debug("deflate_service: (%s) %s(buffer #%d, %lu bytes held) [%c%c]",
+			gnet_host_to_string(&tx->host),
+			(tx->flags & TX_ERROR) ? "ERROR " : "",
 			attr->send_idx,
 			attr->send_idx >= 0 ?
-				(gint) (attr->buf[attr->send_idx].wptr -
+				(unsigned long) (attr->buf[attr->send_idx].wptr -
 						attr->buf[attr->send_idx].rptr) : 0,
 			(attr->flags & DF_FLOWC) ? 'C' : '-',
 			(attr->flags & DF_FLUSH) ? 'f' : '-');
-
+	}
 	/*
 	 * First, attempt to transmit the whole send buffer, if any pending.
 	 */
@@ -603,11 +614,11 @@ deflate_service(gpointer data)
 	b = &attr->buf[attr->fill_idx];	/* Buffer we fill */
 
 	if (b->wptr >= b->end) {
-		if (dbg > 9)
-			printf("deflate_service: (%s) sending fill buffer #%d, %d bytes\n",
-				host_to_string(&tx->host), attr->fill_idx,
-				(gint) (b->wptr - b->rptr));
-
+		if (GNET_PROPERTY(tx_debug) > 9) {
+			g_debug("deflate_service: (%s) sending fill buffer #%d, %lu bytes",
+				gnet_host_to_string(&tx->host), attr->fill_idx,
+				(unsigned long) (b->wptr - b->rptr));
+		}
 		deflate_rotate_and_send(tx);	/* Can set TX_ERROR */
 
 		if (tx->flags & TX_ERROR)
@@ -629,9 +640,10 @@ deflate_service(gpointer data)
 	if (attr->flags & DF_FLOWC) {
 		attr->flags &= ~DF_FLOWC;	/* Leave flow control state */
 
-		if (dbg > 4)
-			printf("Compressing TX stack for peer %s leaves FLOWC\n",
-				host_to_string(&tx->host));
+		if (GNET_PROPERTY(tx_debug) > 4) {
+			g_debug("compressing TX stack for peer %s leaves FLOWC",
+				gnet_host_to_string(&tx->host));
+		}
 	}
 
 	/*
@@ -653,12 +665,13 @@ deflate_service(gpointer data)
 	}
 
 
-	if (dbg > 9)
-		printf("deflate_service: (%s) %sdone locally [%c%c]\n",
-			host_to_string(&tx->host), (tx->flags & TX_ERROR) ? "ERROR " : "",
+	if (GNET_PROPERTY(tx_debug) > 9) {
+		g_debug("deflate_service: (%s) %sdone locally [%c%c]",
+			gnet_host_to_string(&tx->host),
+			(tx->flags & TX_ERROR) ? "ERROR " : "",
 			(attr->flags & DF_FLOWC) ? 'C' : '-',
 			(attr->flags & DF_FLUSH) ? 'f' : '-');
-
+	}
 	/*
 	 * If upper layer wants servicing, do it now.
 	 * Note that this can put us back into flow control.
@@ -685,32 +698,66 @@ tx_deflate_init(txdrv_t *tx, gpointer args)
 	struct attr *attr;
 	struct tx_deflate_args *targs = args;
 	z_streamp outz;
-	gint ret;
-	gint i;
+	int ret;
+	int i;
 
 	g_assert(tx);
 	g_assert(NULL != targs->cb);
 
-	outz = walloc(sizeof *outz);
-
+	WALLOC(outz);
 	outz->zalloc = zlib_alloc_func;
 	outz->zfree = zlib_free_func;
 	outz->opaque = NULL;
 
-	ret = targs->gzip
-		? deflateInit2(outz, Z_DEFAULT_COMPRESSION, Z_DEFLATED, (-MAX_WBITS),
-				MAX_MEM_LEVEL, Z_DEFAULT_STRATEGY)
-		: deflateInit(outz, Z_DEFAULT_COMPRESSION);
+	/*
+	 * Reduce memory requirements for deflation when running as an ultrapeer.
+	 *
+	 * Memory used for deflation is:
+	 *
+	 *	(1 << (window_bits +2)) +  (1 << (mem_level + 9))
+	 *
+	 * For leaves, we use window_bits = 15 and mem_level = 9, which makes
+	 * for 128 KiB + 256 KiB = 384 KiB per connection (TX side).
+	 *
+	 * For ultra peers, we use window_bits = 14 and mem_level = 6, so this
+	 * uses 64 KiB + 32 KiB = 96 KiB only.
+	 *
+	 * Since ultra peers have many more connections than leaves, the memory
+	 * savings are drastic, yet compression levels remain around 50% (varies
+	 * depending on the nature of the traffic, of course).
+	 *
+	 *		--RAM, 2009-04-09
+	 */
+
+	{
+		int window_bits = MAX_WBITS;		/* Must be 8 .. MAX_WBITS */
+		int mem_level = MAX_MEM_LEVEL;		/* Must be 1 .. MAX_MEM_LEVEL */
+		int level = Z_BEST_COMPRESSION;
+
+		if (settings_is_ultra()) {
+			window_bits = 14;
+			mem_level = 6;
+			level = Z_DEFAULT_COMPRESSION;
+		}
+
+		g_assert(window_bits >= 8 && window_bits <= MAX_WBITS);
+		g_assert(mem_level >= 1 && mem_level <= MAX_MEM_LEVEL);
+		g_assert(level == Z_DEFAULT_COMPRESSION ||
+			(level >= Z_BEST_SPEED && level <= Z_BEST_COMPRESSION));
+
+		ret = deflateInit2(outz, level, Z_DEFLATED,
+				targs->gzip ? (-window_bits) : window_bits, mem_level,
+				Z_DEFAULT_STRATEGY);
+	}
 
 	if (Z_OK != ret) {
-		wfree(outz, sizeof *outz);
 		g_warning("unable to initialize compressor for peer %s: %s",
-			host_to_string(&tx->host), zlib_strerror(ret));
+			gnet_host_to_string(&tx->host), zlib_strerror(ret));
+		WFREE(outz);
 		return NULL;
 	}
 
-	attr = walloc(sizeof *attr);
-
+	WALLOC(attr);
 	attr->cq = targs->cq;
 	attr->cb = targs->cb;
 	attr->buffer_size = targs->buffer_size;
@@ -735,7 +782,7 @@ tx_deflate_init(txdrv_t *tx, gpointer args)
 
 	if (attr->gzip.enabled) {
 		/* See RFC 1952 - GZIP file format specification version 4.3 */
-		static const gchar header[] = {
+		static const unsigned char header[] = {
 			0x1f, 0x8b, /* gzip magic */
 			0x08,		/* compression method: deflate */
 			0,			/* flags: none */
@@ -772,8 +819,8 @@ static void
 tx_deflate_destroy(txdrv_t *tx)
 {
 	struct attr *attr = tx->opaque;
-	gint i;
-	gint ret;
+	int i;
+	int ret;
 
 	g_assert(attr->outz);
 
@@ -790,14 +837,11 @@ tx_deflate_destroy(txdrv_t *tx)
 
 	if (Z_OK != ret && Z_DATA_ERROR != ret)
 		g_warning("while freeing compressor for peer %s: %s",
-			host_to_string(&tx->host), zlib_strerror(ret));
+			gnet_host_to_string(&tx->host), zlib_strerror(ret));
 
-	wfree(attr->outz, sizeof *attr->outz);
-
-	if (attr->tm_ev)
-		cq_cancel(attr->cq, attr->tm_ev);
-
-	wfree(attr, sizeof *attr);
+	WFREE(attr->outz);
+	cq_cancel(&attr->tm_ev);
+	WFREE(attr);
 }
 
 /**
@@ -810,14 +854,14 @@ tx_deflate_write(txdrv_t *tx, gconstpointer data, size_t len)
 {
 	struct attr *attr = tx->opaque;
 
-	if (dbg > 9)
-		printf("tx_deflate_write: (%s) (buffer #%d, nagle %s, "
-			"unflushed %lu) [%c%c]\n",
-			host_to_string(&tx->host), attr->fill_idx,
+	if (GNET_PROPERTY(tx_debug) > 9) {
+		g_debug("tx_deflate_write: (%s) (buffer #%d, nagle %s, "
+			"unflushed %lu) [%c%c]",
+			gnet_host_to_string(&tx->host), attr->fill_idx,
 			(attr->flags & DF_NAGLE) ? "on" : "off", (gulong) attr->unflushed,
 			(attr->flags & DF_FLOWC) ? 'C' : '-',
 			(attr->flags & DF_FLUSH) ? 'f' : '-');
-
+	}
 	/*
 	 * If we're flow controlled or shut down, don't accept anything.
 	 */
@@ -834,21 +878,22 @@ tx_deflate_write(txdrv_t *tx, gconstpointer data, size_t len)
  * @return amount of bytes written, or -1 on error.
  */
 static ssize_t
-tx_deflate_writev(txdrv_t *tx, struct iovec *iov, gint iovcnt)
+tx_deflate_writev(txdrv_t *tx, iovec_t *iov, int iovcnt)
 {
 	struct attr *attr = tx->opaque;
-	gint sent = 0;
+	int sent = 0;
 
-	if (dbg > 9)
-		printf("tx_deflate_writev: (%s) (buffer #%d, nagle %s, "
-			"unflushed %lu) [%c%c]\n",
-			host_to_string(&tx->host), attr->fill_idx,
+	if (GNET_PROPERTY(tx_debug) > 9) {
+		g_debug("tx_deflate_writev: (%s) (buffer #%d, nagle %s, "
+			"unflushed %lu) [%c%c]",
+			gnet_host_to_string(&tx->host), attr->fill_idx,
 			(attr->flags & DF_NAGLE) ? "on" : "off", (gulong) attr->unflushed,
 			(attr->flags & DF_FLOWC) ? 'C' : '-',
 			(attr->flags & DF_FLUSH) ? 'f' : '-');
+	}
 
-	while (iovcnt--) {
-		gint ret;
+	while (iovcnt-- > 0) {
+		int ret;
 
 		/*
 		 * If we're flow controlled or shut down, stop sending.
@@ -857,27 +902,27 @@ tx_deflate_writev(txdrv_t *tx, struct iovec *iov, gint iovcnt)
 		if (attr->flags & (DF_FLOWC|DF_SHUTDOWN))
 			return sent;
 
-		ret = deflate_add(tx, iov->iov_base, iov->iov_len);
+		ret = deflate_add(tx, iovec_base(iov), iovec_len(iov));
 
 		if (-1 == ret)
 			return -1;
 
 		sent += ret;
-		if ((guint) ret < iov->iov_len) {
+		if (UNSIGNED(ret) < iovec_len(iov)) {
 			/* Could not write all, flow-controlled */
 			break;
 		}
 		iov++;
 	}
 
-	if (dbg > 9)
-		printf("tx_deflate_writev: (%s) sent %lu bytes (buffer #%d, nagle %s, "
-			"unflushed %lu) [%c%c]\n",
-			host_to_string(&tx->host), (gulong) sent, attr->fill_idx,
+	if (GNET_PROPERTY(tx_debug) > 9) {
+		g_debug("tx_deflate_writev: (%s) sent %lu bytes (buffer #%d, nagle %s, "
+			"unflushed %lu) [%c%c]",
+			gnet_host_to_string(&tx->host), (gulong) sent, attr->fill_idx,
 			(attr->flags & DF_NAGLE) ? "on" : "off", (gulong) attr->unflushed,
 			(attr->flags & DF_FLOWC) ? 'C' : '-',
 			(attr->flags & DF_FLUSH) ? 'f' : '-');
-
+	}
 	return sent;
 }
 
@@ -933,7 +978,7 @@ tx_deflate_flush(txdrv_t *tx)
 
 	if (attr->flags & DF_NAGLE) {
 		g_assert(NULL != attr->tm_ev);
-		cq_expire(attr->cq, attr->tm_ev);
+		cq_expire(attr->tm_ev);
 	} else if (!(attr->flags & DF_FLOWC))
 		deflate_flush_send(tx);
 }
@@ -965,14 +1010,14 @@ tx_deflate_close(txdrv_t *tx, tx_closed_t cb, gpointer arg)
 
 	g_assert(tx->flags & TX_CLOSING);
 
-	if (dbg > 9)
-		printf("tx_deflate_close: (%s) send=%d buffer #%d, nagle %s, "
-			"unflushed %lu) [%c%c]\n",
-			host_to_string(&tx->host), attr->send_idx, attr->fill_idx,
+	if (GNET_PROPERTY(tx_debug) > 9) {
+		g_debug("tx_deflate_close: (%s) send=%d buffer #%d, nagle %s, "
+			"unflushed %lu) [%c%c]",
+			gnet_host_to_string(&tx->host), attr->send_idx, attr->fill_idx,
 			(attr->flags & DF_NAGLE) ? "on" : "off", (gulong) attr->unflushed,
 			(attr->flags & DF_FLOWC) ? 'C' : '-',
 			(attr->flags & DF_FLUSH) ? 'f' : '-');
-
+	}
 	/*
 	 * Flush whatever we can.
 	 */
@@ -1000,9 +1045,9 @@ tx_deflate_close(txdrv_t *tx, tx_closed_t cb, gpointer arg)
 	}
 
 	if (0 == tx_deflate_pending(tx)) {
-		if (dbg > 9)
-			printf("tx_deflate_close: flushed everything immediately\n");
-
+		if (GNET_PROPERTY(tx_debug) > 9) {
+			g_debug("tx_deflate_close: flushed everything immediately");
+		}
 		(*cb)(tx, arg);
 		return;
 	}
@@ -1014,14 +1059,15 @@ tx_deflate_close(txdrv_t *tx, tx_closed_t cb, gpointer arg)
 	attr->closed = cb;
 	attr->closed_arg = arg;
 
-	if (dbg > 9)
-		printf("tx_deflate_close: (%s) delayed! send=%d buffer #%d, nagle %s, "
-			"unflushed %lu) [%c%c]\n",
-			host_to_string(&tx->host),
+	if (GNET_PROPERTY(tx_debug) > 9) {
+		g_debug("tx_deflate_close: (%s) delayed! send=%d buffer #%d, nagle %s, "
+			"unflushed %lu) [%c%c]",
+			gnet_host_to_string(&tx->host),
 			attr->send_idx, attr->fill_idx,
 			(attr->flags & DF_NAGLE) ? "on" : "off", (gulong) attr->unflushed,
 			(attr->flags & DF_FLOWC) ? 'C' : '-',
 			(attr->flags & DF_FLUSH) ? 'f' : '-');
+	}
 }
 
 static const struct txdrv_ops tx_deflate_ops = {
