@@ -1,5 +1,5 @@
 /*
- * $Id: tx_deflate.h,v 1.7 2005/09/07 17:24:05 cbiere Exp $
+ * $Id: tx_deflate.h 16106 2009-03-10 15:16:31Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -36,6 +36,8 @@
 #ifndef _core_tx_deflate_h_
 #define _core_tx_deflate_h_
 
+#include "common.h"
+
 #include "tx.h"
 #include "lib/cq.h"
 
@@ -45,15 +47,15 @@ const struct txdrv_ops *tx_deflate_get_ops(void);
  * Callbacks used by the deflating layer.
  */
 struct tx_deflate_cb {
-	void (*add_tx_deflated)(gpointer owner, gint amount);
-	void (*shutdown)(gpointer owner, const gchar *reason, ...);
+	void (*add_tx_deflated)(gpointer owner, int amount);
+	void (*shutdown)(gpointer owner, const char *reason, ...);
 };
 
 /**
  * Arguments to be passed when the layer is intantiated.
  */
 struct tx_deflate_args {
-	struct tx_deflate_cb *cb;	/**< Callbacks */
+	const struct tx_deflate_cb *cb;	/**< Callbacks */
 	cqueue_t *cq;				/**< Callout queue to use */
 	size_t buffer_size;			/**< Internal buffer size to use */
 	size_t buffer_flush;		/**< Flush after that many bytes */
