@@ -1,5 +1,5 @@
 /*
- * $Id: tx_dgram.c 11800 2006-08-29 19:31:46Z cbiere $
+ * $Id: tx_dgram.c 19142 2011-05-23 20:44:04Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -37,16 +37,16 @@
 
 #include "common.h"
 
-RCSID("$Id: tx_dgram.c 11800 2006-08-29 19:31:46Z cbiere $")
+RCSID("$Id: tx_dgram.c 19142 2011-05-23 20:44:04Z rmanfredi $")
 
+#include "bsched.h"
 #include "sockets.h"
 #include "tx.h"
 #include "tx_dgram.h"
-#include "bsched.h"
+#include "hosts.h"
 #include "inet.h"
 
-#include "if/core/hosts.h"
-
+#include "lib/gnet_host.h"
 #include "lib/misc.h"
 #include "lib/tm.h"
 #include "lib/walloc.h"
@@ -65,7 +65,7 @@ struct attr {
  * Invoked when the output file descriptor can accept more data.
  */
 static void
-is_writable(gpointer data, gint unused_source, inputevt_cond_t cond)
+is_writable(gpointer data, int unused_source, inputevt_cond_t cond)
 {
 	txdrv_t *tx = (txdrv_t *) data;
 
@@ -104,11 +104,11 @@ tx_dgram_init(txdrv_t *tx, gpointer args)
 	g_assert(targs->cb != NULL);
 	g_assert(s_udp_listen != NULL || s_udp_listen6 != NULL);
 
-	attr = walloc(sizeof *attr);
+	WALLOC(attr);
 
 	/*
 	 * Because we handle servicing of the upper layers explicitely within
-	 * the TX stack (i.e. upper layers detect that we were enable to comply
+	 * the TX stack (i.e. upper layers detect that we were unable to comply
 	 * with the whole write and enable us), there is no I/O callback attached
 	 * to the I/O source: we only create it to benefit from bandwidth limiting
 	 * through calls to bio_sendto().
@@ -116,7 +116,7 @@ tx_dgram_init(txdrv_t *tx, gpointer args)
 
 	attr->cb = targs->cb;
 	attr->wio = targs->wio;
-	attr->bio = bsched_source_add(targs->bs, attr->wio, BIO_F_WRITE,
+	attr->bio = bsched_source_add(targs->bws, attr->wio, BIO_F_WRITE,
 		NULL, NULL);
 
 	tx->opaque = attr;
@@ -136,10 +136,10 @@ tx_dgram_destroy(txdrv_t *tx)
 
 	bsched_source_remove(attr->bio);
 
-	wfree(attr, sizeof *attr);
+	WFREE(attr);
 }
 
-static inline gint
+static inline int
 tx_dgram_write_error(txdrv_t *tx, const gnet_host_t *to, const char *func)
 {
 	if (is_temporary_error(errno) || ENOBUFS == errno)
@@ -186,16 +186,14 @@ tx_dgram_write_error(txdrv_t *tx, const gnet_host_t *to, const char *func)
 		 * Don't set TX_ERROR here, we don't care about lost packets.
 		 */
 		g_warning("UDP write to %s failed: %s",
-			host_addr_port_to_string(to->addr, to->port), g_strerror(errno));
+			gnet_host_to_string(to), g_strerror(errno));
 		return -1;
 	default:
 		{
 			int terr = errno;
-			time_t t = tm_time();
 			tx->flags |= TX_ERROR;				/* This should be fatal! */
-			g_error("%s  gtk-gnutella: %s: "
-				"UDP write to %s failed with unexpected errno: %d (%s)\n",
-				ctime(&t), func, host_addr_port_to_string(to->addr, to->port),
+			g_error("%s: UDP write to %s failed with unexpected errno: %d (%s)",
+				func, gnet_host_to_string(to),
 				terr, g_strerror(terr));
 		}
 	}
@@ -215,7 +213,7 @@ tx_dgram_sendto(txdrv_t *tx, const gnet_host_t *to,
 	ssize_t r;
 	struct attr *attr = tx->opaque;
 
-	if (to->port > 0) {
+	if (gnet_host_get_port(to) > 0) {
 		r = bio_sendto(attr->bio, to, data, len);
 	} else {
 		errno = EINVAL;
@@ -227,7 +225,7 @@ tx_dgram_sendto(txdrv_t *tx, const gnet_host_t *to,
 	if (attr->cb->add_tx_written != NULL)
 		attr->cb->add_tx_written(tx->owner, r);
 
-	inet_udp_record_sent(to->addr);
+	inet_udp_record_sent(gnet_host_get_addr(to));
 
 	return r;
 }
