@@ -1,5 +1,5 @@
 /*
- * $Id: tx_dgram.h,v 1.2 2005/06/25 01:37:41 daichik Exp $
+ * $Id: tx_dgram.h 16106 2009-03-10 15:16:31Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -36,10 +36,29 @@
 #ifndef _core_tx_dgram_h_
 #define _core_tx_dgram_h_
 
+#include "common.h"
 #include "tx.h"
 
+#include "if/core/bsched.h"
+
 const struct txdrv_ops *tx_dgram_get_ops(void);
 
+/**
+ * Callbacks used by the datagram layer.
+ */
+struct tx_dgram_cb {
+	void (*add_tx_written)(gpointer owner, int amount);
+};
+
+/**
+ * Arguments to be passed when the layer is intantiated.
+ */
+struct tx_dgram_args {
+	struct tx_dgram_cb *cb;			/**< Callbacks */
+	struct wrap_io *wio;			/**< I/O wrapping routines */
+	bsched_bws_t bws;				/**< Bandwidth scheduler to use */
+};
+
 #endif	/* _core_tx_dgram_h_ */
 
 /* vi: set ts=4 sw=4 cindent: */
