@@ -1,5 +1,5 @@
 /*
- * $Id: uhc.c 14018 2007-07-02 18:30:15Z cbiere $
+ * $Id: uhc.c 19334 2011-07-17 18:36:40Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: uhc.c 14018 2007-07-02 18:30:15Z cbiere $")
+RCSID("$Id: uhc.c 19334 2011-07-17 18:36:40Z rmanfredi $")
 
 #include "gnutella.h"
 #include "guid.h"
@@ -46,13 +46,17 @@ RCSID("$Id: uhc.c 14018 2007-07-02 18:30:15Z cbiere $")
 #include "sockets.h"
 #include "udp.h"
 #include "uhc.h"
+#include "ghc.h"
 
 #include "lib/adns.h"
 #include "lib/atoms.h"
 #include "lib/cq.h"
 #include "lib/endian.h"
 #include "lib/glib-missing.h"
-#include "lib/misc.h"
+#include "lib/hashlist.h"
+#include "lib/halloc.h"
+#include "lib/parse.h"
+#include "lib/random.h"
 
 #include "if/gnet_property_priv.h"
 #include "if/bridge/c2ui.h"
@@ -71,35 +75,33 @@ RCSID("$Id: uhc.c 14018 2007-07-02 18:30:15Z cbiere $")
  * in the pings.
  */
 static struct uhc_context {
-	const gchar *host;			/**< Last selected host (string atom) */
+	const char *host;			/**< Last selected host (static buffer) */
 	cevent_t *timeout_ev;		/**< Ping timeout */
-	gint attempts;				/**< Connection / resolution attempts */
+	int attempts;				/**< Connection / resolution attempts */
 	host_addr_t addr;			/**< Resolved IP address for host */
 	guint16 port;				/**< Port of selected host cache */
-	gchar muid[GUID_RAW_SIZE];	/**< MUID of the ping */
+	struct guid muid;			/**< MUID of the ping */
 } uhc_ctx;
 
-static GList *uhc_avail;	/**< List of UHCs as string */
-static GList *uhc_used;		/**< List of used UHCs as ``struct used_uhc'' */
+static hash_list_t *uhc_list;	/**< List of ``struct uhc'' */
 
-struct used_uhc {
-	const gchar	*host;	/**< An UHC host as "<host>:<port>" (string atom) */
+struct uhc {
+	const char	*host;	/**< An UHC host as "<host>:<port>" (string atom) */
 	time_t		stamp;	/**< Timestamp of the last request */
+	guint		used;	/**< How often have we tried to contact it */
 };
 
 /**
  * The following hosts are there for bootstrapping purposes only.
  */
 static const struct {
-	const gchar *uhc;
+	const char *uhc;
 } boot_hosts[] = {
-	{ "g6.6dns.org:1337" },
-	{ "guruz.udp-host-cache.com:6666" },
-	{ "secondary.udp-host-cache.com:9999" },
-	{ "uhc.udp-host-cache.com:9999" },
-	{ "void.ghostwhitecrab.de:443" },
-	{ "yang.cloud.bishopston.net:33558" },
+#if defined(USE_LOCAL_UHC)
+	{ "localhost:6346" },
+#else	/* !USE_LOCAL_UHC */
 	{ "yin.cloud.bishopston.net:33558" },
+#endif	/* USE_LOCAL_UHC */
 };
 
 static gboolean uhc_connecting = FALSE;
@@ -117,12 +119,12 @@ static void uhc_host_resolved(const host_addr_t *addr, size_t n,
  * @return TRUE if we successfully parsed the string.
  */
 static gboolean
-uhc_get_host_port(const gchar *hp, const gchar **host, guint16 *port)
+uhc_get_host_port(const char *hp, const char **host, guint16 *port)
 {
-	static gchar hostname[MAX_HOSTLEN + 1];
-	const gchar *ep;
+	static char hostname[MAX_HOSTLEN + 1];
+	const char *ep;
 	guint32 u;
-	gint error;
+	int error;
 	size_t len;
 
 
@@ -132,6 +134,7 @@ uhc_get_host_port(const gchar *hp, const gchar **host, guint16 *port)
 
 	*host = NULL;
 	*port = 0;
+	hostname[0] = '\0';
 
 	if (!string_to_host_or_addr(hp, &ep, NULL) || ':' != *ep)
 		return FALSE;
@@ -155,42 +158,111 @@ uhc_get_host_port(const gchar *hp, const gchar **host, guint16 *port)
 	return TRUE;
 }
 
-static void
-add_available_uhc(const gchar *hc)
+static struct uhc *
+uhc_new(const char *host)
 {
-	const gchar *host;
-
-	g_assert(hc);
-
-	host = atom_str_get(hc);
-	uhc_avail = random_value(100) < 50
-		? g_list_append(uhc_avail, deconstify_gchar(host))
-		: g_list_prepend(uhc_avail, deconstify_gchar(host));
-}
-
-static struct used_uhc *
-used_uhc_new(const gchar *host)
-{
-	struct used_uhc *uu;
+	static const struct uhc zero_uhc;
+	struct uhc *uhc;
 
 	g_assert(host);
-	uu = g_malloc(sizeof *uu);
-	uu->host = atom_str_get(host);
-	uu->stamp = tm_time();
-	return uu;
+	uhc = g_malloc(sizeof *uhc);
+	*uhc = zero_uhc;
+	uhc->host = atom_str_get(host);
+	uhc->stamp = 0;
+	uhc->used = 0;
+	return uhc;
 }
 
 static void
-used_uhc_free(struct used_uhc **ptr)
+uhc_free(struct uhc **ptr)
 {
 	if (*ptr) {	
-		struct used_uhc *uu = *ptr;
+		struct uhc *uu = *ptr;
 		atom_str_free_null(&uu->host);
 		G_FREE_NULL(uu);
 		*ptr = NULL;
 	}
 }
 
+static guint
+uhc_hash(gconstpointer key)
+{
+	const struct uhc *uhc = key;
+
+	return g_str_hash(uhc->host);
+}
+
+static int
+uhc_equal(gconstpointer p, gconstpointer q)
+{
+	const struct uhc *a = p, *b = q;
+
+	return 0 == strcmp(a->host, b->host);
+}
+
+
+static void
+uhc_list_add(const char *host)
+{
+	struct uhc *uhc;
+
+	g_return_if_fail(host);
+
+	uhc = uhc_new(host);
+	if (hash_list_contains(uhc_list, uhc)) {
+		g_warning("duplicate bootstrap UHC: \"%s\"", uhc->host);
+		uhc_free(&uhc);
+		return;
+	}
+
+	if (random_value(100) < 50) {
+		hash_list_append(uhc_list, uhc);
+	} else {
+		hash_list_prepend(uhc_list, uhc);
+	}
+}
+
+/**
+ * @return NULL on error, a newly allocated string via halloc() otherwise.
+ */
+static char *
+uhc_get_next(void)
+{
+	struct uhc *uhc;
+	char *host;
+	time_t now;
+
+	g_return_val_if_fail(uhc_list, NULL);
+	
+	now = tm_time();
+	uhc = hash_list_head(uhc_list);
+	if (NULL == uhc)
+		return NULL;
+
+	/*
+	 * Wait UHC_RETRY_AFTER secs before contacting the UHC again.
+	 * Can't be too long because the UDP reply may get lost if the
+	 * requesting host already has a saturated b/w.
+	 * If we come here, it's because we're lacking hosts for establishing
+	 * a Gnutella connection, after we exhausted our caches.
+	 */
+	if (uhc->stamp && delta_time(now, uhc->stamp) < UHC_RETRY_AFTER)
+		return NULL;
+
+	uhc->stamp = now;
+	host = h_strdup(uhc->host);
+
+	if (uhc->used < UHC_MAX_ATTEMPTS) {
+		uhc->used++;
+		hash_list_moveto_tail(uhc_list, uhc);
+	} else {
+		hash_list_remove(uhc_list, uhc);
+		uhc_free(&uhc);
+	}
+
+	return host;
+}
+
 /**
  * Pick host at random among the host array.
  *
@@ -199,64 +271,36 @@ used_uhc_free(struct used_uhc **ptr)
 static gboolean
 uhc_pick(void)
 {
-	gchar *hc;
-	size_t len;
-	guint idx;
-	time_t now = tm_time();
+	gboolean success = FALSE;
+	char *uhc;
 
-	/* First check whether used UHCs can added back */
-	while (uhc_used) {
-		struct used_uhc *uu;
-
-		uu = uhc_used->data;
-		g_assert(uu);
-
-		/*
-		 * Wait UHC_RETRY_AFTER secs before contacting the UHC again.
-		 * Can't be too long because the UDP reply may get lost if the
-		 * requesting host already has a saturated b/w.
-		 * If we come here, it's because we're lacking hosts for establishing
-		 * a Gnutella connection, after we exhausted our caches.
-		 */
-
-		if (delta_time(now, uu->stamp) < UHC_RETRY_AFTER)
-			break;
-
-		add_available_uhc(uu->host);
-		uhc_used = g_list_remove(uhc_used, uu);
-		used_uhc_free(&uu);
-	}
-
-	len = g_list_length(uhc_avail);
-	if (len < 1) {
+	uhc = uhc_get_next();
+	if (NULL == uhc) {
 		if (GNET_PROPERTY(bootstrap_debug))
-			g_warning("BOOT ran out of UHCs");
-		return FALSE;
+			g_warning("BOOT ran out of UHCs, switching to GHCs");
+		ghc_get_hosts();
+		goto finish;
 	}
 
-	idx = random_value(len - 1);
-	hc = g_list_nth_data(uhc_avail, idx);
-	g_assert(hc);
-
-	uhc_avail = g_list_remove(uhc_avail, hc);
-	uhc_used = g_list_append(uhc_used, used_uhc_new(hc));
-
-	if (!uhc_get_host_port(hc, &uhc_ctx.host, &uhc_ctx.port)) {
-		g_warning("cannot parse UDP host cache \"%s\"", hc);
-		return FALSE;
+	if (!uhc_get_host_port(uhc, &uhc_ctx.host, &uhc_ctx.port)) {
+		g_warning("cannot parse UDP host cache \"%s\"", uhc);
+		goto finish;
 	}
 
 	/*
 	 * Give GUI feedback.
 	 */
 	{
-		gchar msg[256];
+		char msg[256];
 
-		gm_snprintf(msg, sizeof msg, _("Looking for UDP host cache %s"), hc);
+		gm_snprintf(msg, sizeof msg, _("Looking for UDP host cache %s"), uhc);
 		gcu_statusbar_message(msg);
 	}
+	success = TRUE;
 
-	return TRUE;
+finish:
+	HFREE_NULL(uhc);
+	return success;
 }
 
 /**
@@ -268,11 +312,10 @@ uhc_try_random(void)
 	g_assert(uhc_connecting);
 	g_assert(uhc_ctx.timeout_ev == NULL);
 
-	if (uhc_ctx.attempts >= UHC_MAX_ATTEMPTS || !uhc_pick()) {
+	if (!uhc_pick()) {
 		uhc_connecting = FALSE;
 		return;
 	}
-	uhc_ctx.attempts++;
 
 	/*
 	 * The following may recurse if resolution is synchronous, but
@@ -309,18 +352,18 @@ uhc_send_ping(void)
 {
 	g_assert(uhc_connecting);
 
-	guid_random_muid(uhc_ctx.muid);	
+	guid_random_muid(&uhc_ctx.muid);	
 
-	if (udp_send_ping(uhc_ctx.muid, uhc_ctx.addr, uhc_ctx.port, TRUE)) {
+	if (udp_send_ping(&uhc_ctx.muid, uhc_ctx.addr, uhc_ctx.port, TRUE)) {
 
 		if (GNET_PROPERTY(bootstrap_debug))
-			g_message("BOOT sent UDP SCP ping %s to %s:%u",
-				guid_hex_str(uhc_ctx.muid), uhc_ctx.host, uhc_ctx.port);
+			g_debug("BOOT sent UDP SCP ping %s to %s:%u",
+				guid_hex_str(&uhc_ctx.muid), uhc_ctx.host, uhc_ctx.port);
 		/*
 		 * Give GUI feedback.
 		 */
 		{
-			gchar msg[256];
+			char msg[256];
 
 			gm_snprintf(msg, sizeof msg,
 				_("Sent ping to UDP host cache %s:%u"),
@@ -335,10 +378,10 @@ uhc_send_ping(void)
 
 		g_assert(uhc_ctx.timeout_ev == NULL);
 
-		uhc_ctx.timeout_ev = cq_insert(callout_queue,
-				UHC_TIMEOUT, uhc_ping_timeout, NULL);
+		uhc_ctx.timeout_ev = cq_main_insert(UHC_TIMEOUT,
+			uhc_ping_timeout, NULL);
 	} else {
-		g_message("BOOT failed to send UDP SCP to %s",
+		g_warning("BOOT failed to send UDP SCP to %s",
 			host_addr_port_to_string(uhc_ctx.addr, uhc_ctx.port));
 	}
 }
@@ -365,10 +408,10 @@ uhc_host_resolved(const host_addr_t *addrs, size_t n, gpointer uu_udata)
 		return;
 	}
 
-	uhc_ctx.addr = addrs[random_raw() % n];
+	uhc_ctx.addr = addrs[random_u32() % n];
 	
 	if (GNET_PROPERTY(bootstrap_debug))
-		g_message("BOOT UDP host cache \"%s\" resolved to %s",
+		g_debug("BOOT UDP host cache \"%s\" resolved to %s",
 			uhc_ctx.host, host_addr_to_string(uhc_ctx.addr));
 
 
@@ -403,15 +446,19 @@ uhc_get_hosts(void)
 	if (uhc_connecting || GNET_PROPERTY(ancient_version))
 		return;
 
-	if (!udp_active())
+	if (!udp_active()) {
+		g_message("BOOT cannot contact UHCs (UDP inactive), using GHCs");
+		ghc_get_hosts();
 		return;
+	}
+
+	g_message("BOOT will be contacting an UHC");
 
 	/*
 	 * Reset context.
 	 */
 
 	uhc_connecting = TRUE;
-	uhc_ctx.attempts = 0;
 
 	g_assert(uhc_ctx.timeout_ev == NULL);
 
@@ -426,19 +473,18 @@ uhc_get_hosts(void)
  * Called when a pong with an "IPP" extension was received.
  */
 void
-uhc_ipp_extract(gnutella_node_t *n, const gchar *payload, gint paylen)
+uhc_ipp_extract(gnutella_node_t *n, const char *payload, int paylen)
 {
-	gint i, cnt;
+	int i, cnt;
 
 	g_assert(0 == paylen % 6);
 
 	cnt = paylen / 6;
 
 	if (GNET_PROPERTY(bootstrap_debug))
-		g_message("extracting %d host%s in UDP IPP pong %s from %s (%s)",
+		g_debug("extracting %d host%s in UDP IPP pong %s from %s",
 			cnt, cnt == 1 ? "" : "s",
-			guid_hex_str(gnutella_header_get_muid(&n->header)), node_addr(n),
-			uhc_connecting ? "expected" : "unsollicited");
+			guid_hex_str(gnutella_header_get_muid(&n->header)), node_addr(n));
 
 	for (i = 0; i < cnt; i++) {
 		host_addr_t ha;
@@ -449,9 +495,9 @@ uhc_ipp_extract(gnutella_node_t *n, const gchar *payload, gint paylen)
 
 		hcache_add_caught(HOST_ULTRA, ha, port, "UDP-HC");
 
-		if (GNET_PROPERTY(bootstrap_debug) > 1)
-			g_message("BOOT collected %s from UDP IPP pong from %s",
-				host_addr_to_string(ha), node_addr(n));
+		if (GNET_PROPERTY(bootstrap_debug) > 2)
+			g_debug("BOOT collected %s from UDP IPP pong from %s",
+				host_addr_port_to_string(ha, port), node_addr(n));
 	}
 
 	if (!uhc_connecting)
@@ -465,11 +511,11 @@ uhc_ipp_extract(gnutella_node_t *n, const gchar *payload, gint paylen)
 	 * check whether we're still in a probing cycle.
 	 */
 
-	if (!guid_eq(uhc_ctx.muid, gnutella_header_get_muid(&n->header)))
+	if (!guid_eq(&uhc_ctx.muid, gnutella_header_get_muid(&n->header)))
 		return;
 
 	if (GNET_PROPERTY(bootstrap_debug)) {
-		g_message("BOOT UDP cache \"%s\" replied: got %d host%s from %s",
+		g_debug("BOOT UDP cache \"%s\" replied: got %d host%s from %s",
 			uhc_ctx.host, cnt, cnt == 1 ? "" : "s", node_addr(n));
 	}
 
@@ -478,9 +524,9 @@ uhc_ipp_extract(gnutella_node_t *n, const gchar *payload, gint paylen)
 	 */
 
 	if (cnt > 0) {
-		gchar msg[256];
+		char msg[256];
 
-		cq_cancel(callout_queue, &uhc_ctx.timeout_ev);
+		cq_cancel(&uhc_ctx.timeout_ev);
 		uhc_connecting = FALSE;
 
 		gm_snprintf(msg, sizeof(msg),
@@ -503,8 +549,11 @@ uhc_init(void)
 {
 	guint i;
 
+	g_return_if_fail(NULL == uhc_list);
+	uhc_list = hash_list_new(uhc_hash, uhc_equal);
+
 	for (i = 0; i < G_N_ELEMENTS(boot_hosts); i++) {
-		const gchar *host, *ep, *uhc;
+		const char *host, *ep, *uhc;
 		guint16 port;
 
 		uhc = boot_hosts[i].uhc;
@@ -518,7 +567,7 @@ uhc_init(void)
 		g_assert(NULL != ep);
 		g_assert(':' == ep[0]);
 
-		add_available_uhc(uhc);
+		uhc_list_add(uhc);
 	}
 }
 
@@ -528,17 +577,16 @@ uhc_init(void)
 void
 uhc_close(void)
 {
-	cq_cancel(callout_queue, &uhc_ctx.timeout_ev);
+	cq_cancel(&uhc_ctx.timeout_ev);
 	uhc_connecting = FALSE;
-	while (uhc_avail) {
-		const gchar *host = uhc_avail->data;
-		uhc_avail = g_list_remove(uhc_avail, uhc_avail->data);
-		atom_str_free_null(&host);
-	}
-	while (uhc_used) {
-		struct used_uhc *uu = uhc_used->data;
-		uhc_used = g_list_remove(uhc_used, uu);
-		used_uhc_free(&uu);
+
+	if (uhc_list) {
+		struct uhc *uhc;
+
+		while (NULL != (uhc = hash_list_shift(uhc_list))) {
+			uhc_free(&uhc);
+		}
+		hash_list_free(&uhc_list);
 	}
 }
 
