@@ -1,5 +1,5 @@
 /*
- * $Id: version.c 15126 2008-03-10 14:35:34Z cbiere $
+ * $Id: version.c 17618 2010-10-31 19:24:38Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,7 +35,9 @@
 
 #include "common.h"
 
-RCSID("$Id: version.c 15126 2008-03-10 14:35:34Z cbiere $")
+RCSID("$Id: version.c 17618 2010-10-31 19:24:38Z rmanfredi $")
+
+#include "gtk-gnutella.h"
 
 #include "version.h"
 #include "token.h"
@@ -47,17 +49,19 @@ RCSID("$Id: version.c 15126 2008-03-10 14:35:34Z cbiere $")
 #include "if/core/main.h"
 
 #include "lib/atoms.h"
+#include "lib/ascii.h"
 #include "lib/getdate.h"
-#include "lib/misc.h"
 #include "lib/glib-missing.h"
+#include "lib/parse.h"
 #include "lib/tm.h"
+#include "lib/timestamp.h"
 #include "lib/utf8.h"
 #include "lib/override.h"		/* Must be the last header included */
 
 #define SECS_PER_DAY	86400
 
-const gchar *version_string = NULL;
-const gchar *version_short_string = NULL;
+const char *version_string = NULL;
+const char *version_short_string = NULL;
 
 static version_t our_version;
 static version_t last_rel_version;
@@ -67,7 +71,7 @@ static guint8 version_code;
 /**
  * Get version string.
  */
-const gchar *
+const char *
 version_get_string(void)
 {
 	return version_string;
@@ -86,10 +90,10 @@ version_get_code(void)
  * Dump original version string and decompiled form to stdout.
  */
 static void
-version_dump(const gchar *str, const version_t *ver, const gchar *cmptag)
+version_dump(const char *str, const version_t *ver, const char *cmptag)
 {
-	printf("VERSION%s \"%s\":\n"
-		"\tmajor=%u minor=%u patch=%u tag=%c taglevel=%u build=%u\n",
+	g_debug("VERSION%s \"%s\": "
+		"major=%u minor=%u patch=%u tag=%c taglevel=%u build=%u",
 		cmptag, str, ver->major, ver->minor, ver->patchlevel,
 		ver->tag ? ver->tag : ' ', ver->taglevel, ver->build);
 }
@@ -98,11 +102,11 @@ version_dump(const gchar *str, const version_t *ver, const gchar *cmptag)
  * @return a user-friendly description of the version.
  * NB: returns pointer to static data.
  */
-const gchar *
+const char *
 version_str(const version_t *ver)
 {
-	static gchar str[80];
-	gint rw;
+	static char str[80];
+	int rw;
 
 	rw = gm_snprintf(str, sizeof(str), "%u.%u", ver->major, ver->minor);
 
@@ -132,10 +136,10 @@ version_str(const version_t *ver)
  * and extract timestamp into `ver'.
  */
 static void
-version_stamp(const gchar *str, version_t *ver)
+version_stamp(const char *str, version_t *ver)
 {
-	static gchar stamp[256];
-	const gchar *p;
+	static char stamp[256];
+	const char *p;
 
 	ver->timestamp = 0;
 
@@ -151,7 +155,7 @@ version_stamp(const gchar *str, version_t *ver)
 
 	p = strchr(str, '(');
 	if (p) {
-		const gchar *end;
+		const char *end;
 
 		p++;
 		end = strchr(p, ';');
@@ -187,10 +191,10 @@ version_stamp(const gchar *str, version_t *ver)
  * were not facing a gtk-gnutella version, or if we did not recognize it.
  */
 static gboolean
-version_parse(const gchar *str, version_t *ver)
+version_parse(const char *str, version_t *ver)
 {
-	const gchar *v;
-	gint error;
+	const char *v;
+	int error;
 
 	/*
 	 * Modern version numbers are formatted like this:
@@ -280,7 +284,7 @@ version_parse(const gchar *str, version_t *ver)
 	} else
 		ver->build = 0;
 
-	if (GNET_PROPERTY(dbg) > 6)
+	if (GNET_PROPERTY(version_debug) > 1)
 		version_dump(str, ver, "#");
 
 	return TRUE;
@@ -290,7 +294,7 @@ version_parse(const gchar *str, version_t *ver)
  * Compare two tag chars, assuming version numbers are equal.
  * @returns -1, 0 or +1 depending on the sign of "a - b".
  */
-static gint
+static int
 version_tagcmp(guchar a, guchar b)
 {
 	if (a == b)
@@ -315,7 +319,7 @@ version_tagcmp(guchar a, guchar b)
  * Compare two gtk-gnutella versions, timestamp and build not withstanding.
  * @returns -1, 0 or +1 depending on the sign of "a - b".
  */
-gint
+int
 version_cmp(const version_t *a, const version_t *b)
 {
 	if (a->major == b->major) {
@@ -342,10 +346,10 @@ version_cmp(const version_t *a, const version_t *b)
  *
  * @returns -1, 0 or +1 depending on the sign of "a - b".
  */
-gint
+int
 version_build_cmp(const version_t *a, const version_t *b)
 {
-	gint cmp = version_cmp(a, b);
+	int cmp = version_cmp(a, b);
 	return 0 == cmp ? CMP(a->build, b->build) : cmp;
 }
 
@@ -355,7 +359,7 @@ version_build_cmp(const version_t *a, const version_t *b)
  * @returns OK if we were able to parse correctly.
  */
 gboolean
-version_fill(const gchar *version, version_t *vs)
+version_fill(const char *version, version_t *vs)
 {
 	if (!version_parse(version, vs))
 		return FALSE;
@@ -369,11 +373,11 @@ version_fill(const gchar *version, version_t *vs)
  * Invoked when a newer version is found.
  */
 static void
-version_new_found(const gchar *text, gboolean stable)
+version_new_found(const char *text, gboolean stable)
 {
-    static gchar last_stable[256] = "";
-    static gchar last_dev[256] = "";
-	gchar s[1024];
+    static char last_stable[256] = "";
+    static char last_dev[256] = "";
+	char s[1024];
 
     if (stable)
         utf8_strlcpy(last_stable, text, sizeof last_stable);
@@ -407,12 +411,12 @@ version_new_found(const gchar *text, gboolean stable)
  * looking as gtk-gnutella but which failed the token-based sanity checks.
  */
 gboolean
-version_check(const gchar *str, const gchar *token, const host_addr_t addr)
+version_check(const char *str, const char *token, const host_addr_t addr)
 {
 	version_t their_version;
 	version_t *target_version;
-	gint cmp;
-	const gchar *version;
+	int cmp;
+	const char *version;
 
 	if (!version_parse(str, &their_version))
 		return TRUE;			/* Not gtk-gnutella, or unparseable */
@@ -428,7 +432,7 @@ version_check(const gchar *str, const gchar *token, const host_addr_t addr)
 
 	cmp = version_cmp(target_version, &their_version);
 
-	if (GNET_PROPERTY(dbg) > 6)
+	if (GNET_PROPERTY(version_debug) > 1)
 		version_dump(str, &their_version,
 			cmp == 0 ? "=" :
 			cmp > 0 ? "-" : "+");
@@ -439,8 +443,8 @@ version_check(const gchar *str, const gchar *token, const host_addr_t addr)
 
 	version_stamp(str, &their_version);
 
-	if (GNET_PROPERTY(dbg) > 6)
-		printf("VERSION time=%d\n", (gint) their_version.timestamp);
+	if (GNET_PROPERTY(version_debug) > 3)
+		g_debug("VERSION time=%d", (int) their_version.timestamp);
 
 	/*
 	 * If version claims something older than TOKEN_START_DATE, then
@@ -451,8 +455,8 @@ version_check(const gchar *str, const gchar *token, const host_addr_t addr)
 		tok_error_t error;
 
 		if (token == NULL) {
-            if (GNET_PROPERTY(dbg)) {
-                g_message("got GTKG vendor string \"%s\" without token!", str);
+            if (GNET_PROPERTY(version_debug)) {
+                g_debug("got GTKG vendor string \"%s\" without token!", str);
             }
 			return FALSE;	/* Can't be correct */
 		}
@@ -474,8 +478,8 @@ version_check(const gchar *str, const gchar *token, const host_addr_t addr)
 			error = TOK_OK;		/* Our keys have expired, cannot validate */
 
 		if (error != TOK_OK) {
-            if (GNET_PROPERTY(dbg)) {
-                g_message("vendor string \"%s\" [%s] has wrong token "
+            if (GNET_PROPERTY(version_debug)) {
+                g_debug("vendor string \"%s\" [%s] has wrong token "
                     "\"%s\": %s ", str, host_addr_to_string(addr), token,
                     tok_strerror(error));
             }
@@ -504,12 +508,12 @@ version_check(const gchar *str, const gchar *token, const host_addr_t addr)
 			|| their_version.build > target_version->build) &&
 		target_version == &last_rel_version
 	) {
-		if (GNET_PROPERTY(dbg) > 6)
-			printf("VERSION is a SVN update of a release\n");
+		if (GNET_PROPERTY(version_debug) > 3)
+			g_debug("VERSION is a SVN update of a release");
 
 		if (version_build_cmp(&last_dev_version, &their_version) > 0) {
-			if (GNET_PROPERTY(dbg) > 6)
-				printf("VERSION is less recent than latest dev we know\n");
+			if (GNET_PROPERTY(version_debug) > 3)
+				g_debug("VERSION is less recent than latest dev we know");
 			return TRUE;
 		}
 		target_version = &last_dev_version;
@@ -539,8 +543,8 @@ version_check(const gchar *str, const gchar *token, const host_addr_t addr)
 	 * We found a more recent version than the last version seen.
 	 */
 
-	if (GNET_PROPERTY(dbg) > 4)
-		printf("more recent %s VERSION \"%s\"\n",
+	if (GNET_PROPERTY(version_debug) > 1)
+		g_debug("more recent %s VERSION \"%s\"",
 			target_version == &last_dev_version ? "dev" : "rel", str);
 
 	*target_version = their_version;		/* struct copy */
@@ -574,15 +578,15 @@ version_check(const gchar *str, const gchar *token, const host_addr_t addr)
  *
  * @return A pointer to a static buffer holding the version string.
  */
-const gchar *
+const char *
 version_build_string(void)
 {
 	static gboolean initialized;
-	static gchar buf[128];
+	static char buf[128];
 
 	if (!initialized) {
-		const gchar *sysname = "Unknown";
-		const gchar *machine = NULL;
+		const char *sysname = "Unknown";
+		const char *machine = NULL;
 
 		initialized = TRUE;
 
@@ -627,13 +631,13 @@ version_init(void)
 		g_assert(ok);
 	}
 
-	g_message("%s", version_string);
+	g_info("%s", version_string);
 
 	version_stamp(version_string, &our_version);
 	g_assert(our_version.timestamp != 0);
 
 	{
-		gchar buf[128];
+		char buf[128];
 
 		gm_snprintf(buf, sizeof(buf),
 			GTA_PRODUCT_NAME "/%s-%u (%s)",
@@ -677,8 +681,7 @@ version_init(void)
  *
  * If the version being ran is not a stable one, warn after 60 days, otherwise
  * warn after a year.  If we're not "expired" yet but are approaching the
- * deadline, start to remind them.  After the deadline plus some grace period,
- * refuse to run unless a special property is set.
+ * deadline, start to remind them. 
  */
 void
 version_ancient_warn(void)
@@ -701,7 +704,16 @@ version_ancient_warn(void)
 	elapsed = delta_time(now, our_version.timestamp);
 
 	if (elapsed > VERSION_ANCIENT_WARN || tok_is_ancient(now)) {
-		g_warning("version of gtk-gnutella is too old, you should upgrade!");
+		static gboolean warned = FALSE;
+		if (GNET_PROPERTY(version_debug)) {
+			g_debug("VERSION our_version = %s (elapsed = %ld, token %s)",
+				timestamp_to_string(our_version.timestamp),
+				(long) elapsed, tok_is_ancient(now) ? "ancient" : "ok");
+		}
+		if (!warned) {
+			g_warning("version of gtk-gnutella is too old, please upgrade!");
+			warned = TRUE;
+		}
         gnet_prop_set_boolean_val(PROP_ANCIENT_VERSION, TRUE);
 		return;
 	}
