@@ -1,5 +1,5 @@
 /*
- * $Id: ui2c.c,v 1.32 2005/12/18 06:09:10 cbiere Exp $
+ * $Id: ui2c.c 19253 2011-06-13 23:25:08Z rmanfredi $
  *
  * Copyright (c) 2004, Emile Roberts
  *
@@ -57,12 +57,12 @@
 #include "if/core/search.h"
 #include "if/core/share.h"
 #include "if/core/settings.h"
-#include "if/core/upload_stats.h"
 #include "if/core/uploads.h"
 #include "if/core/upload_stats.h"
 #include "if/core/version.h"
 #include "if/core/main.h"
 #include "if/bridge/ui2c.h"
+#include "if/dht/dht.h"
 
 /*
  *	Functions the UI uses to access the CORE
@@ -70,34 +70,65 @@
 
 /*	adns interface functions (UI -> Core)*/
 gboolean
-guc_adns_resolve(const gchar *hostname,
+guc_adns_resolve(const char *hostname,
 	adns_callback_t user_callback, gpointer user_data)
 {
-	return adns_resolve(hostname, user_callback, user_data);
+	return adns_resolve(hostname, settings_dns_net(), user_callback, user_data);
 }
 
 /*	bitzi interface functions (UI -> Core)*/
 
 void
-guc_query_bitzi_by_urn(const gchar *urnsha1)
+guc_query_bitzi_by_sha1(const struct sha1 *sha1, filesize_t filesize, gboolean refresh)
 {
-    bitzi_query_byurnsha1(urnsha1);
+    bitzi_query_by_sha1(sha1, filesize, refresh);
 }
 
-bitzi_data_t *
-guc_querycache_bitzi_by_urn(const gchar *urnsha1)
+const char *
+guc_bitzi_ticket_by_sha1(const struct sha1 *sha1, filesize_t filesize)
 {
-    return bitzi_querycache_byurnsha1(urnsha1);
+    return bitzi_ticket_by_sha1(sha1, filesize);
+}
+
+gboolean
+guc_bitzi_data_by_sha1(bitzi_data_t *data,
+	const struct sha1 *sha1, filesize_t filesize)
+{
+    return bitzi_data_by_sha1(data, sha1, filesize);
+}
+
+gboolean
+guc_bitzi_has_cached_ticket(const struct sha1 *sha1)
+{
+	return bitzi_has_cached_ticket(sha1);
 }
 
 /*	download and src interface functions (UI -> Core)*/
-const gchar *
-guc_build_url_from_download(const struct download *d)
+char *
+guc_download_build_url(const struct download *d)
 {
-	return build_url_from_download(d);
+	return download_build_url(d);
 }
 
-gint
+char *
+guc_file_info_build_magnet(gnet_fi_t handle)
+{
+	return file_info_build_magnet(handle);
+}
+
+char *
+guc_file_info_get_file_url(gnet_fi_t handle)
+{
+	return file_info_get_file_url(handle);
+}
+
+const char *
+guc_file_info_status_to_string(const gnet_fi_status_t *status)
+{
+	return file_info_status_to_string(status);
+}
+
+int
 guc_download_get_http_req_percent(const struct download *d)
 {
 	return download_get_http_req_percent(d);
@@ -110,29 +141,13 @@ guc_download_fallback_to_push(struct download *d, gboolean on_timeout,
 	download_fallback_to_push(d, on_timeout, user_request);
 }
 
-gint
-guc_download_remove_all_from_peer(gchar *guid, const host_addr_t addr,
-	guint16 port, gboolean unavailable)
+int
+guc_download_remove_all_from_peer(const struct guid *guid,
+	const host_addr_t addr, guint16 port, gboolean unavailable)
 {
 	return download_remove_all_from_peer(guid, addr, port, unavailable);
 }
 
-gint guc_download_remove_all_named(const gchar *name)
-{
-	return download_remove_all_named(name);
-}
-
-gint guc_download_remove_all_with_sha1(const gchar *sha1)
-{
-	return download_remove_all_with_sha1(sha1);
-}
-
-void
-guc_download_remove_file(struct download *d, gboolean reset)
-{
-	download_remove_file(d, reset);
-}
-
 gboolean
 guc_download_file_exists(const struct download *d)
 {
@@ -142,31 +157,31 @@ guc_download_file_exists(const struct download *d)
 void
 guc_download_requeue(struct download *d)
 {
-	download_requeue(d);
+	download_request_requeue(d);
 }
 
 void
-guc_download_start(struct download *d, gboolean check_allowed)
+guc_download_start(struct download *d)
 {
-	download_start(d, check_allowed);
+	download_request_start(d);
 }
 
-gboolean
-guc_download_remove(struct download *d)
+void
+guc_download_pause(struct download *d)
 {
-	return download_remove(d);
+	download_request_pause(d);
 }
 
 void
 guc_download_abort(struct download *d)
 {
-	download_abort(d);
+	download_request_abort(d);
 }
 
 void
 guc_download_resume(struct download *d)
 {
-	download_resume(d);
+	download_request_resume(d);
 }
 
 void
@@ -181,7 +196,7 @@ guc_download_thaw_queue(void)
 	download_thaw_queue();
 }
 
-gint
+gboolean
 guc_download_queue_is_frozen(void)
 {
 	return download_queue_is_frozen();
@@ -189,52 +204,102 @@ guc_download_queue_is_frozen(void)
 
 void
 guc_download_clear_stopped(gboolean complete,
-	gboolean failed, gboolean unavailable, gboolean now)
+	gboolean failed, gboolean unavailable, gboolean finished, gboolean now)
 {
-	download_clear_stopped(complete, failed, unavailable, now);
+	download_clear_stopped(complete, failed, unavailable, finished, now);
 }
 
-void
-guc_download_auto_new(gchar *file, filesize_t size,
-	guint32 record_index, const host_addr_t addr, guint16 port,
-	const gchar *guid, gchar *hostname, gchar *sha1, time_t stamp,
-	gboolean push, gboolean file_size_known, fileinfo_t *fi,
-	gnet_host_vec_t *proxies, guint32 flags)
+guint
+guc_download_handle_magnet(const char *url)
 {
-	download_auto_new(file, size, record_index, addr, port, guid, hostname,
-		sha1, stamp, push, file_size_known, fi, proxies, flags);
+	return download_handle_magnet(url);
 }
 
 gboolean
-guc_download_new_unknown_size(gchar *file,
-	guint32 record_index, const host_addr_t addr, guint16 port,
-	const gchar *guid, gchar *hostname, gchar *sha1, time_t stamp,
-			gboolean push, fileinfo_t *fi,
-			gnet_host_vec_t *proxies, guint32 flags)
+guc_download_new(const char *filename,
+	const char *uri,
+	filesize_t size,
+	const host_addr_t addr,
+	guint16 port,
+	const struct guid *guid,
+	const char *hostname,
+	const struct sha1 *sha1,
+	const struct tth *tth,
+	time_t stamp,
+	fileinfo_t *fi,
+	gnet_host_vec_t *proxies,
+	guint32 flags,
+	const char *parq_id)
 {
-	return download_new_unknown_size(file, record_index, addr, port, guid,
-		hostname, sha1, stamp, push, fi, proxies, flags);
+	return download_new(filename,
+			uri,
+			size,
+			addr,
+			port,
+		   	guid,
+			hostname,
+			sha1,
+			tth,
+			stamp,
+			fi,
+			proxies,
+			flags,
+			parq_id);
 }
 
-const gchar *
+void
+guc_download_auto_new(const char *filename,
+	filesize_t size,
+	const host_addr_t addr,
+	guint16 port,
+	const struct guid *guid,
+	const char *hostname,
+	const struct sha1 *sha1,
+	const struct tth *tth,
+	time_t stamp,
+	fileinfo_t *fi,
+	gnet_host_vec_t *proxies,
+	guint32 flags)
+{
+	download_auto_new(filename,
+		size,
+		addr,
+		port,
+		guid,
+		hostname,
+		sha1,
+		tth,
+		stamp,
+		fi,
+		proxies,
+		flags);
+}
+
+const char *
 guc_download_get_hostname(const struct download *d)
 {
 	return download_get_hostname(d);
 }
 
-const gchar *
+const char *
 guc_download_get_country(const struct download *d)
 {
 	return iso3166_country_cc(download_country(d));
 }
 
-gdouble
+const char *
+guc_download_get_vendor(const struct download *d)
+{
+	return download_vendor_str(d);
+}
+
+double
 guc_download_source_progress(const struct download *d)
 {
 	return download_source_progress(d);
 }
 
-gdouble
+double
 guc_download_total_progress(const struct download *d)
 {
 	return download_total_progress(d);
@@ -246,33 +311,9 @@ guc_download_something_to_clear(void)
 	return download_something_to_clear();
 }
 
-gboolean
-guc_download_new(gchar *file, filesize_t size,
-			guint32 record_index, const host_addr_t addr, guint16 port,
-			const gchar *guid, gchar *hostname, gchar *sha1, time_t stamp,
-			gboolean push, fileinfo_t *fi,
-			gnet_host_vec_t *proxies, guint32 flags)
-{
-	return download_new(file, size, record_index, addr, port, guid, hostname,
-		sha1, stamp, push, fi, proxies, flags);
-}
-
-gboolean
-guc_download_new_uri(gchar *file, const gchar *uri, filesize_t size,
-	const host_addr_t addr, guint16 port,
-	const gchar *guid, gchar *hostname, gchar *sha1, time_t stamp,
-	gboolean push, fileinfo_t *fi,
-	gnet_host_vec_t *proxies, guint32 flags)
-{
-	return download_new_uri(file, uri, size,
-			addr, port, guid, hostname,
-			sha1, stamp, push, fi, proxies, flags);
-}
-
-
 void
 guc_download_index_changed(const host_addr_t addr, guint16 port,
-	gchar *guid, filesize_t from, filesize_t to)
+	const struct guid *guid, filesize_t from, filesize_t to)
 {
 	download_index_changed(addr, port, guid, from, to);
 }
@@ -298,7 +339,7 @@ guc_src_remove_listener(src_listener_t cb, gnet_src_ev_t ev)
 
 
 /*	fileinfo interface functions (UI -> Core)*/
-const gchar *
+const char *
 guc_file_info_readable_filename(fileinfo_t *fi)
 {
 	return file_info_readable_filename(fi);
@@ -322,16 +363,34 @@ guc_fi_get_status(gnet_fi_t fih, gnet_fi_status_t *s)
 	fi_get_status(fih, s);
 }
 
-gchar **
+char **
 guc_fi_get_aliases(gnet_fi_t fih)
 {
 	return fi_get_aliases(fih);
 }
 
 void
-guc_fi_purge_by_handle_list(const GSList *list)
+guc_fi_purge(gnet_fi_t fih)
 {
-	fi_purge_by_handle_list(list);
+	fi_purge(fih);
+}
+
+void
+guc_fi_pause(gnet_fi_t fih)
+{
+	fi_pause(fih);
+}
+
+void
+guc_fi_resume(gnet_fi_t fih)
+{
+	fi_resume(fih);
+}
+
+gboolean
+guc_fi_rename(gnet_fi_t fih, const char *filename)
+{
+	return fi_rename(fih, filename);
 }
 
 void
@@ -396,6 +455,11 @@ guc_gnet_get_bw_stats(gnet_bw_source type, gnet_bw_stats_t *stats)
 	gnet_get_bw_stats(type, stats);
 }
 
+const char *
+guc_gnet_stats_drop_reason_to_string(msg_drop_reason_t reason)
+{
+	return gnet_stats_drop_reason_to_string(reason);
+}
 
 /*	hcache interface functions (UI -> Core)*/
 void
@@ -417,13 +481,13 @@ guc_hcache_get_stats(hcache_stats_t *stats)
 }
 
 /*	HSEP interface functions (UI -> Core)*/
-const gchar *
-guc_hsep_get_static_str(gint row, gint column)
+const char *
+guc_hsep_get_static_str(int row, int column)
 {
 	return hsep_get_static_str(row, column);
 }
 
-gint
+int
 guc_hsep_get_table_size(void)
 {
 	return hsep_get_table_size();
@@ -451,7 +515,7 @@ guc_hsep_remove_global_table_listener(GCallback cb)
 
 
 /*	HTTP interface functions (UI -> Core)*/
-const gchar *
+const char *
 guc_http_range_to_string(const GSList *list)
 {
 	return http_range_to_string(list);
@@ -520,27 +584,27 @@ guc_node_add(const host_addr_t addr, guint16 port, guint32 flags)
 }
 
 void
-guc_node_remove_by_handle(gnet_node_t n)
+guc_node_remove_by_id(const struct nid *node_id)
 {
-	node_remove_by_handle(n);
+	node_remove_by_id(node_id);
 }
 
 void
-guc_node_remove_nodes_by_handle(GSList *node_list)
+guc_node_remove_nodes_by_id(const GSList *node_list)
 {
-	node_remove_nodes_by_handle(node_list);
+	node_remove_nodes_by_id(node_list);
 }
 
-void
-guc_node_get_status(const gnet_node_t n, gnet_node_status_t *s)
+gboolean
+guc_node_get_status(const struct nid *node_id, gnet_node_status_t *s)
 {
-	node_get_status(n, s);
+	return node_get_status(node_id, s);
 }
 
 gnet_node_info_t *
-guc_node_get_info(const gnet_node_t n)
+guc_node_get_info(const struct nid *node_id)
 {
-	return node_get_info(n);
+	return node_get_info(node_id);
 }
 
 void
@@ -555,56 +619,62 @@ guc_node_free_info(gnet_node_info_t *info)
 	node_free_info(info);
 }
 
-void
-guc_node_fill_flags(gnet_node_t n, gnet_node_flags_t *flags)
+gboolean
+guc_node_fill_flags(const struct nid *node_id, gnet_node_flags_t *flags)
 {
-	node_fill_flags(n, flags);
+	return node_fill_flags(node_id, flags);
 }
 
-void
-guc_node_fill_info(const gnet_node_t n, gnet_node_info_t *info)
+gboolean
+guc_node_fill_info(const struct nid *node_id, gnet_node_info_t *info)
 {
-	node_fill_info(n, info);
+	return node_fill_info(node_id, info);
 }
 
-void
-guc_node_udp_gui_show(void)
+const char *
+guc_node_flags_to_string(const gnet_node_flags_t *flags)
 {
-	node_udp_gui_show();
+	return node_flags_to_string(flags);
 }
 
-void
-guc_node_udp_gui_remove(void)
+const char *
+guc_node_peermode_to_string(node_peer_t m)
 {
-	node_udp_gui_remove();
+	return node_peermode_to_string(m);
 }
 
 /*	parq interface functions (UI -> Core)*/
-gint
+int
 guc_get_parq_dl_position(const struct download *d)
 {
 	return get_parq_dl_position(d);
 }
 
-gint
+int
 guc_get_parq_dl_queue_length(const struct download *d)
 {
 	return get_parq_dl_queue_length(d);
 }
 
-gint
+int
 guc_get_parq_dl_eta(const struct download *d)
 {
 	return get_parq_dl_eta(d);
 }
 
-gint
+int
 guc_get_parq_dl_retry_delay(const struct download *d)
 {
 	return get_parq_dl_retry_delay(d);
 }
 
 /*	search interface functions (UI -> Core)*/
+guint
+guc_search_handle_magnet(const char *url)
+{
+	return search_handle_magnet(url);
+}
+
 void
 guc_search_update_items(gnet_search_t sh, guint32 items)
 {
@@ -617,6 +687,12 @@ guc_search_get_reissue_timeout(gnet_search_t sh)
 	return search_get_reissue_timeout(sh);
 }
 
+unsigned
+guc_search_get_media_type(gnet_search_t sh)
+{
+	return search_get_media_type(sh);
+}
+
 void
 guc_search_set_reissue_timeout(gnet_search_t sh, guint32 timeout)
 {
@@ -638,13 +714,13 @@ guc_search_get_create_time(gnet_search_t sh)
 void
 guc_search_set_create_time(gnet_search_t sh, time_t t)
 {
-	return search_set_create_time(sh, t);
+	search_set_create_time(sh, t);
 }
 
-gboolean
-guc_search_is_passive(gnet_search_t sh)
+const char *
+guc_search_query(gnet_search_t sh)
 {
-	return search_is_passive(sh);
+	return search_query(sh);
 }
 
 gboolean
@@ -653,6 +729,24 @@ guc_search_is_active(gnet_search_t sh)
 	return search_is_active(sh);
 }
 
+gboolean
+guc_search_is_browse(gnet_search_t sh)
+{
+	return search_is_browse(sh);
+}
+
+gboolean
+guc_search_is_local(gnet_search_t sh)
+{
+	return search_is_local(sh);
+}
+
+gboolean
+guc_search_is_passive(gnet_search_t sh)
+{
+	return search_is_passive(sh);
+}
+
 gboolean
 guc_search_is_expired(gnet_search_t sh)
 {
@@ -665,27 +759,59 @@ guc_search_is_frozen(gnet_search_t sh)
 	return search_is_frozen(sh);
 }
 
-gnet_search_t
-guc_search_new(const gchar *query,
-	time_t create_time, guint lifetime, guint32 reissue_timeout, flag_t flags)
+gboolean
+guc_search_is_whats_new(gnet_search_t sh)
 {
-	return search_new(query, create_time, lifetime, reissue_timeout, flags);
+	return search_is_whats_new(sh);
+}
+
+const char *
+guc_search_media_mask_to_string(unsigned mask)
+{
+	return search_media_mask_to_string(mask);
+}
+
+void
+guc_search_associate_sha1(gnet_search_t sh, const struct sha1 *sha1)
+{
+	search_associate_sha1(sh, sha1);
+}
+
+GSList *
+guc_search_associated_sha1(gnet_search_t sh)
+{
+	return search_associated_sha1(sh);
+}
+
+unsigned
+guc_search_associated_sha1_count(gnet_search_t sh)
+{
+	return search_associated_sha1_count(sh);
+}
+
+enum search_new_result
+guc_search_new(gnet_search_t *ptr, const char *query, unsigned mtype,
+	time_t create_time, guint lifetime, guint32 reissue_timeout, guint32 flags)
+{
+	return search_new(ptr, query, mtype,
+				create_time, lifetime, reissue_timeout, flags);
 }
 
 gboolean
 guc_search_browse(gnet_search_t sh,
-	const gchar *hostname, host_addr_t addr, guint16 port,
-	const gchar *guid, gboolean push, const gnet_host_vec_t *proxies)
+	const char *hostname, host_addr_t addr, guint16 port,
+	const struct guid *guid, const gnet_host_vec_t *proxies, guint32 flags)
 {
-	return search_browse(sh, hostname, addr, port, guid, push, proxies);
+	return search_browse(sh, hostname, addr, port, guid, proxies, flags);
 }
 
-void
-guc_search_reissue(gnet_search_t sh)
+gboolean
+guc_search_locally(gnet_search_t sh, const char *query)
 {
-	search_reissue(sh);
+	return search_locally(sh, query);
 }
 
+
 void
 guc_search_close(gnet_search_t sh)
 {
@@ -706,13 +832,27 @@ guc_search_stop(gnet_search_t sh)
 
 
 /*	settings interface functions (UI -> Core)*/
-const gchar *
+guint16
+guc_listen_port(void)
+{
+	guint32 port;
+	gnet_prop_get_guint32_val(PROP_LISTEN_PORT, &port);
+	return port;
+}
+
+host_addr_t
+guc_listen_addr(enum net_type net)
+{
+	return listen_addr_by_net(net);
+}
+
+const char *
 guc_settings_home_dir(void)
 {
 	return settings_home_dir();
 }
 
-const gchar *
+const char *
 guc_settings_config_dir(void)
 {
 	return settings_config_dir();
@@ -721,7 +861,7 @@ guc_settings_config_dir(void)
 
 /*	share interface functions (UI -> Core)*/
 void
-guc_shared_dir_add(const gchar * path)
+guc_shared_dir_add(const char * path)
 {
 	shared_dir_add(path);
 }
@@ -745,22 +885,45 @@ guc_shared_kbytes_scanned(void)
 }
 
 void
-guc_share_add_search_request_listener(search_request_listener_t l)
+guc_search_got_results_listener_add(search_got_results_listener_t l)
 {
-	share_add_search_request_listener(l);
-}
-
-
-void
-guc_share_remove_search_request_listener(search_request_listener_t l)
-{
-	share_remove_search_request_listener(l);
+	search_got_results_listener_add(l);
 }
 
 void
-guc_search_add_kept(gnet_search_t sh, guint32 kept)
+guc_search_got_results_listener_remove(search_got_results_listener_t l)
 {
-	search_add_kept(sh, kept);
+	search_got_results_listener_remove(l);
+}
+
+void
+guc_search_status_change_listener_add(search_status_change_listener_t l)
+{
+	search_status_change_listener_add(l);
+}
+
+void
+guc_search_status_change_listener_remove(search_status_change_listener_t l)
+{
+	search_status_change_listener_remove(l);
+}
+
+void
+guc_search_request_listener_add(search_request_listener_t l)
+{
+	search_request_listener_add(l);
+}
+
+void
+guc_search_request_listener_remove(search_request_listener_t l)
+{
+	search_request_listener_remove(l);
+}
+
+void
+guc_search_add_kept(gnet_search_t sh, const struct guid *muid, guint32 kept)
+{
+	search_add_kept(sh, muid, kept);
 }
 
 /*	upload interface functions (UI -> Core)*/
@@ -838,8 +1001,14 @@ guc_upload_stats_clear_all(void)
 	upload_stats_clear_all();
 }
 
+gboolean
+guc_dht_enabled(void)
+{
+	return dht_enabled();
+}
+
 /**	version interface functions (UI -> Core)*/
-const gchar *
+const char *
 guc_version_get_version_string(void)
 {
 	return version_get_string();
@@ -847,7 +1016,7 @@ guc_version_get_version_string(void)
 
 /**	main interface functions (UI -> Core)*/
 void
-guc_gtk_gnutella_exit(gint code)
+guc_gtk_gnutella_exit(int code)
 {
 	gtk_gnutella_exit(code);
 }
