@@ -1,5 +1,5 @@
 /*
- * $Id: hcache.h 8376 2005-06-25 01:37:45Z daichik $
+ * $Id: hcache.h 18925 2011-04-04 17:41:25Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -26,7 +26,7 @@
 #ifndef _if_core_hcache_h_
 #define _if_core_hcache_h_
 
-#include <glib.h>
+#include "common.h"
 
 /***
  *** Gnet host cache
@@ -45,7 +45,10 @@ typedef enum {
                                gave us a timeout during connection. */
     HCACHE_BUSY,          /**< We put in this list all the Gnet nodes which
                                gave us a 503 (busy) during connection. */
-    HCACHE_UNSTABLE,
+    HCACHE_UNSTABLE,      /**< Unstable IPs */
+    HCACHE_ALIEN,         /**< Alien networks (protected by auth challenges) */
+    HCACHE_GUESS,         /**< Running GUESS cache (MRU-managed) */
+    HCACHE_GUESS_INTRO,   /**< GUESS introduction cache (LRU-managed) */
 	HCACHE_NONE,
     HCACHE_MAX
 } hcache_type_t;
@@ -53,9 +56,15 @@ typedef enum {
 typedef enum {
     HOST_ANY,
     HOST_ULTRA,
+    HOST_GUESS,
     HOST_MAX
 } host_type_t;
 
+typedef enum {
+	HCACHE_CLASS_HOST,		/**< Classic Gnutella host cache */
+	HCACHE_CLASS_GUESS		/**< GUESS-specific host cache */
+} hcache_class_t;
+
 typedef struct hcache_stats {
     gint32      host_count; /**< Number of hosts in cache */
     guint32     hits;       /**< Hits to known hosts */
