@@ -1,5 +1,5 @@
 /*
- * $Id: hsep.h 8376 2005-06-25 01:37:45Z daichik $
+ * $Id: hsep.h 16289 2009-03-29 08:41:43Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -26,7 +26,7 @@
 #ifndef _if_core_hsep_h_
 #define _if_core_hsep_h_
 
-#include <glib.h>
+#include "common.h"
 
 #include "lib/event.h"		/* For frequency_t */
 
@@ -55,11 +55,13 @@ typedef struct {
 	guint32 triples_received;               /**< # of triples received */
 	guint32 msgs_sent;                      /**< # of msgs sent */
 	guint32 triples_sent;                   /**< # of triples sent */
-	gint random_skew;		/**< additonal random delay for next exchange */
+	int random_skew;		/**< additonal random delay for next exchange */
+	guint8 major;			/**< their major version */
+	guint8 minor;			/**< their minor version */
 } hsep_ctx_t;
 
-const gchar *hsep_get_static_str(gint row, gint column);
-gint hsep_get_table_size(void);
+const char *hsep_get_static_str(int row, int column);
+int hsep_get_table_size(void);
 void hsep_get_non_hsep_triple(hsep_triple *tripledest);
 void hsep_add_global_table_listener(GCallback cb,
 	frequency_t type, guint32 interval);
