@@ -1,5 +1,5 @@
 /*
- * $Id: search.h,v 1.1 2004/08/25 05:40:39 rmanfredi Exp $
+ * $Id: search.h 19147 2011-05-24 09:19:13Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -26,7 +26,7 @@
 #ifndef _if_ui_gtk_search_h_
 #define _if_ui_gtk_search_h_
 
-#include <glib.h>
+#include "common.h"
 
 /*
  * Public interface, visible from the bridge.
@@ -34,10 +34,11 @@
 
 #ifdef GUI_SOURCES
 
-typedef struct search search_t;
-
-gboolean search_gui_new_search(const gchar *, flag_t flags, search_t **search);
+struct search;
 
+gboolean search_gui_new_search(const gchar *, guint32 flags, 
+			struct search **search);
+void search_gui_store_searches(void);
 #endif /* GUI_SOURCES */
 #endif /* _if_ui_gtk_search_h_ */
 
