@@ -1,5 +1,5 @@
 /*
- * $Id: uploads.h 11727 2006-08-26 04:20:24Z cbiere $
+ * $Id: uploads.h 12761 2007-01-16 16:07:30Z cbiere $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -26,7 +26,7 @@
 #ifndef _if_ui_gtk_uploads_h_
 #define _if_ui_gtk_uploads_h_
 
-#include <glib.h>
+#include "common.h"
 
 #include "lib/host_addr.h"
 
