@@ -1,5 +1,5 @@
 /*
- * $Id: adns.c 12172 2006-10-31 22:18:12Z cbiere $
+ * $Id: adns.c 19142 2011-05-23 20:44:04Z rmanfredi $
  *
  * Copyright (c) 2004, Christian Biere
  *
@@ -35,26 +35,26 @@
 
 #include "common.h"
 
-RCSID("$Id: adns.c 12172 2006-10-31 22:18:12Z cbiere $")
+RCSID("$Id: adns.c 19142 2011-05-23 20:44:04Z rmanfredi $")
 
 #include "adns.h"
+#include "ascii.h"
 #include "atoms.h"
-#include "inputevt.h"
-#include "misc.h"
+#include "debug.h"
+#include "fd.h"
 #include "glib-missing.h"
+#include "inputevt.h"
+#include "signal.h"
 #include "tm.h"
 #include "walloc.h"
-#include "socket.h"
 
 #include "override.h"		/* Must be the last header included */
 
-static guint32 common_dbg = 0;	/**< @bug XXX need to init lib's props --RAM */
-
 /* private data types */
 
 struct adns_common {
 	void (*user_callback)(void);
-	gpointer user_data;
+	void *user_data;
 	gboolean reverse;
 };
 
@@ -64,16 +64,16 @@ struct adns_reverse_query {
 
 struct adns_query {
 	enum net_type net;
-	gchar hostname[MAX_HOSTLEN + 1];
+	char hostname[MAX_HOSTLEN + 1];
 };
 
 struct adns_reply {
-	gchar hostname[MAX_HOSTLEN + 1];
+	char hostname[MAX_HOSTLEN + 1];
 	host_addr_t addrs[10];
 };
 
 struct adns_reverse_reply {
-	gchar hostname[MAX_HOSTLEN + 1];
+	char hostname[MAX_HOSTLEN + 1];
 	host_addr_t addr;
 };
 
@@ -95,16 +95,16 @@ struct adns_response {
 
 typedef struct adns_async_write {
 	struct adns_request req;	/**< The original ADNS request */
-	gchar *buf;					/**< Remaining data to write; walloc()ed */
+	char *buf;					/**< Remaining data to write; walloc()ed */
 	size_t pos;					/**< Read position */
 	size_t size;				/**< Size of the buffer */
 } adns_async_write_t;
 
 typedef struct adns_cache_entry {
-	gchar *hostname;		/**< atom */
+	const char *hostname;		/**< atom */
 	time_t timestamp;
 	size_t n;				/**< Number of addr items */
-	guint id;
+	unsigned id;
 	host_addr_t addrs[1 /* pseudo-size */];
 } adns_cache_entry_t;
 
@@ -139,12 +139,12 @@ count_addrs(const host_addr_t *addrs, size_t m)
  */
 #define ADNS_CACHE_MAX_SIZE (1024)
 
-static const gchar adns_process_title[] = "DNS helper for gtk-gnutella";
+static const char adns_process_title[] = "DNS helper for gtk-gnutella";
 
 typedef struct adns_cache_struct {
 	GHashTable *ht;
-	guint pos;
-	gint timeout;
+	unsigned pos;
+	int timeout;
 	adns_cache_entry_t *entries[ADNS_CACHE_MAX_SIZE];
 } adns_cache_t;
 
@@ -152,22 +152,10 @@ static adns_cache_t *adns_cache = NULL;
 
 /* private variables */
 
-static gint adns_query_fd = -1;
-static guint adns_query_event_id = 0;
-static guint adns_reply_event_id = 0;
-static gboolean is_helper = FALSE;		/**< Are we the DNS helper process? */
-
-/**
- * Private macros.
- */
-
-#define CLOSE_IF_VALID(fd)	\
-do {						\
-	if (-1 != (fd))	{		\
-		close(fd);			\
-		fd = -1;			\
-	} 						\
-} while(0)
+static int adns_query_fd = -1;
+static unsigned adns_query_event_id;
+static unsigned adns_reply_event_id;
+static gboolean is_helper;		/**< Are we the DNS helper process? */
 
 /**
  * Private functions.
@@ -194,7 +182,7 @@ adns_cache_init(void)
 #undef ADNS_CACHE_TIMEOUT
 
 static inline adns_cache_entry_t *
-adns_cache_get_entry(adns_cache_t *cache, guint i)
+adns_cache_get_entry(adns_cache_t *cache, unsigned i)
 {
 	adns_cache_entry_t *entry;
 	
@@ -211,7 +199,7 @@ adns_cache_get_entry(adns_cache_t *cache, guint i)
 }
 
 static void
-adns_cache_free_entry(adns_cache_t *cache, guint i)
+adns_cache_free_entry(adns_cache_t *cache, unsigned i)
 {
 	adns_cache_entry_t *entry;
 
@@ -233,10 +221,15 @@ adns_cache_free_entry(adns_cache_t *cache, guint i)
 /**
  * Frees all memory allocated by the cache and returns NULL.
  */
-void
-adns_cache_free(adns_cache_t *cache)
+static void
+adns_cache_free(adns_cache_t **cache_ptr)
 {
-	guint i;
+	adns_cache_t *cache = *cache_ptr;
+	unsigned i;
+
+	/* If adns is not used it will not be initialized */
+	if (NULL == cache)
+		return;
 
 	g_assert(cache);
 	g_assert(cache->ht);
@@ -244,8 +237,7 @@ adns_cache_free(adns_cache_t *cache)
 	for (i = 0; i < G_N_ELEMENTS(cache->entries); i++) {
 		adns_cache_free_entry(cache, i);
 	}
-	g_hash_table_destroy(cache->ht);
-	cache->ht = NULL;
+	gm_hash_table_destroy_null(&cache->ht);
 	G_FREE_NULL(cache);
 }
 
@@ -256,7 +248,7 @@ adns_cache_free(adns_cache_t *cache)
  */
 static void
 adns_cache_add(adns_cache_t *cache, time_t now,
-	const gchar *hostname, const host_addr_t *addrs, size_t n)
+	const char *hostname, const host_addr_t *addrs, size_t n)
 {
 	adns_cache_entry_t *entry;
 	size_t i;
@@ -288,7 +280,7 @@ adns_cache_add(adns_cache_t *cache, time_t now,
 	for (i = 0; i < entry->n; i++) {
 		entry->addrs[i] = addrs[i];
 	}
-	g_hash_table_insert(cache->ht, entry->hostname, entry);
+	g_hash_table_insert(cache->ht, deconstify_gchar(entry->hostname), entry);
 	cache->entries[cache->pos++] = entry;
 	cache->pos %= G_N_ELEMENTS(cache->entries);
 }
@@ -307,7 +299,7 @@ adns_cache_add(adns_cache_t *cache, time_t now,
  */
 static size_t
 adns_cache_lookup(adns_cache_t *cache, time_t now,
-	const gchar *hostname, host_addr_t *addrs, size_t n)
+	const char *hostname, host_addr_t *addrs, size_t n)
 {
 	adns_cache_entry_t *entry;
 
@@ -324,7 +316,7 @@ adns_cache_lookup(adns_cache_t *cache, time_t now,
 				if (i < entry->n) {
 					addrs[i] = entry->addrs[i];
 					if (common_dbg > 0)
-						g_message("adns_cache_lookup: \"%s\" cached (addr=%s)",
+						g_debug("adns_cache_lookup: \"%s\" cached (addr=%s)",
 							entry->hostname, host_addr_to_string(addrs[i]));
 				} else {
 					addrs[i] = zero_host_addr;
@@ -332,7 +324,7 @@ adns_cache_lookup(adns_cache_t *cache, time_t now,
 			}
 		} else {
 			if (common_dbg > 0)
-				g_message("adns_cache_lookup: removing \"%s\" from cache",
+				g_debug("adns_cache_lookup: removing \"%s\" from cache",
 						entry->hostname);
 
 			g_hash_table_remove(cache->ht, hostname);
@@ -352,15 +344,15 @@ adns_cache_lookup(adns_cache_t *cache, time_t now,
  * should only be used with a blocking `fd'.
  */
 static gboolean
-adns_do_transfer(gint fd, gpointer buf, size_t len, gboolean do_write)
+adns_do_transfer(int fd, void *buf, size_t len, gboolean do_write)
 {
 	ssize_t ret;
 	size_t n = len;
 
 	while (n > 0) {
 		if (common_dbg > 2)
-			g_message("adns_do_transfer (%s): n=%lu",
-			    do_write ? "write" : "read", (gulong) n);
+			g_debug("adns_do_transfer (%s): n=%lu",
+			    do_write ? "write" : "read", (unsigned long) n);
 
 		if (do_write)
 			ret = write(fd, buf, n);
@@ -385,7 +377,7 @@ adns_do_transfer(gint fd, gpointer buf, size_t len, gboolean do_write)
 			return FALSE;
 		} else if (ret > 0) {
 			n -= ret;
-			buf = (gchar *) buf + ret;
+			buf = (char *) buf + ret;
 		}
 	}
 
@@ -398,7 +390,7 @@ adns_do_transfer(gint fd, gpointer buf, size_t len, gboolean do_write)
  * @return TRUE on success, FALSE if the operation failed
  */
 static gboolean
-adns_do_read(gint fd, gpointer buf, size_t len)
+adns_do_read(int fd, void *buf, size_t len)
 {
 	return adns_do_transfer(fd, buf, len, FALSE);
 }
@@ -409,7 +401,7 @@ adns_do_read(gint fd, gpointer buf, size_t len)
  * @return TRUE on success, FALSE if the operation failed
  */
 static gboolean
-adns_do_write(gint fd, gpointer buf, size_t len)
+adns_do_write(int fd, void *buf, size_t len)
 {
 	return adns_do_transfer(fd, buf, len, TRUE);
 }
@@ -430,17 +422,16 @@ adns_gethostbyname(const struct adns_request *req, struct adns_response *ans)
 	if (req->common.reverse) {
 		const struct adns_reverse_query *query = &req->query.reverse;
 		struct adns_reverse_reply *reply = &ans->reply.reverse;
-		const gchar *host;
-
+		const char *host;
 
 		if (common_dbg > 1) {
-			g_message("adns_gethostbyname: Resolving \"%s\" ...",
+			g_debug("adns_gethostbyname: Resolving \"%s\" ...",
 					host_addr_to_string(query->addr));
 		}
 
 		reply->addr = query->addr;
 		host = host_addr_to_name(query->addr);
-		g_strlcpy(reply->hostname, host ? host : "", sizeof reply->hostname);
+		clamp_strcpy(reply->hostname, sizeof reply->hostname, host ? host : "");
 	} else {
 		const struct adns_query *query = &req->query.by_addr;
 		struct adns_reply *reply = &ans->reply.by_addr;
@@ -448,10 +439,10 @@ adns_gethostbyname(const struct adns_request *req, struct adns_response *ans)
 		size_t i = 0;
 
 		if (common_dbg > 1) {
-			g_message("adns_gethostbyname: Resolving \"%s\" ...",
+			g_debug("adns_gethostbyname: Resolving \"%s\" ...",
 				query->hostname);
 		}
-		g_strlcpy(reply->hostname, query->hostname, sizeof reply->hostname);
+		clamp_strcpy(reply->hostname, sizeof reply->hostname, query->hostname);
 
 		sl_addr = name_to_host_addr(query->hostname, query->net);
 		for (sl = sl_addr; NULL != sl; sl = g_slist_next(sl)) {
@@ -478,13 +469,13 @@ adns_gethostbyname(const struct adns_request *req, struct adns_response *ans)
  * in case of non-recoverable error during read or write.
  */
 static void
-adns_helper(gint fd_in, gint fd_out)
+adns_helper(int fd_in, int fd_out)
 {
 	g_set_prgname(adns_process_title);
 	gm_setproctitle(g_get_prgname());
 
 #ifdef SIGQUIT 
-	set_signal(SIGQUIT, SIG_IGN);	/* Avoid core dumps on SIGQUIT */
+	signal_set(SIGQUIT, SIG_IGN);	/* Avoid core dumps on SIGQUIT */
 #endif
 
 	is_helper = TRUE;
@@ -493,7 +484,7 @@ adns_helper(gint fd_in, gint fd_out)
 		struct adns_request req;
 		struct adns_response ans;
 		size_t size;
-		gpointer buf;
+		void *buf;
 
 		if (!adns_do_read(fd_in, &req.common, sizeof req.common))
 			break;
@@ -526,8 +517,8 @@ adns_helper(gint fd_in, gint fd_out)
 			break;
 	}
 
-	close(fd_in);
-	close(fd_out);
+	fd_close(&fd_in);
+	fd_close(&fd_out);
 	_exit(EXIT_SUCCESS);
 }
 
@@ -579,7 +570,7 @@ adns_reply_ready(const struct adns_response *ans)
 		if (common_dbg > 1) {
 			const struct adns_reverse_reply *reply = &ans->reply.reverse;
 			
-			g_message("adns_reply_ready: Resolved \"%s\" to \"%s\".",
+			g_debug("adns_reply_ready: Resolved \"%s\" to \"%s\".",
 				host_addr_to_string(reply->addr), reply->hostname);
 		}
 	} else {
@@ -593,7 +584,7 @@ adns_reply_ready(const struct adns_response *ans)
 			size_t i;
 			
 			for (i = 0; i < num; i++) {
-				g_message("adns_reply_ready: Resolved \"%s\" to \"%s\".",
+				g_debug("adns_reply_ready: Resolved \"%s\" to \"%s\".",
 					reply->hostname, host_addr_to_string(reply->addrs[i]));
 			}
 		}
@@ -604,7 +595,6 @@ adns_reply_ready(const struct adns_response *ans)
 		}
 	}
 
-
 	g_assert(ans->common.user_callback);
 	adns_invoke_user_callback(ans);
 }
@@ -616,10 +606,10 @@ adns_reply_ready(const struct adns_response *ans)
  * the reply pipe will be closed and the callback will be lost.
  */
 static void
-adns_reply_callback(gpointer data, gint source, inputevt_cond_t condition)
+adns_reply_callback(void *data, int source, inputevt_cond_t condition)
 {
 	static struct adns_response ans;
-	static gpointer buf;
+	static void *buf;
 	static size_t size, pos;
 
 	g_assert(NULL == data);
@@ -655,27 +645,28 @@ adns_reply_callback(gpointer data, gint source, inputevt_cond_t condition)
 
 		n = size - pos;
 		ret = read(source, cast_to_gchar_ptr(buf) + pos, n);
-		if (0 == ret) {
-			errno = ECONNRESET;
-			ret = (ssize_t) -1;
-		}
 		if ((ssize_t) -1 == ret) {
-			if (!is_temporary_error(errno)) {
+		   	if (!is_temporary_error(errno)) {
 				g_warning("adns_reply_callback: read() failed: %s",
-						g_strerror(errno));
-				inputevt_remove(adns_reply_event_id);
-				adns_reply_event_id = 0;
-				g_warning("adns_reply_callback: removed myself");
-				close(source);
+					g_strerror(errno));
+				goto error;
 			}
 			break;
+		} else if (0 == ret) {
+			g_warning("adns_reply_callback: read() failed: EOF");
+			goto error;
 		} else {
 			g_assert(ret > 0);
-			g_assert((size_t) ret <= n);
+			g_assert(UNSIGNED(ret) <= n);
 			pos += (size_t) ret;
 		}
 	}
 	return;
+	
+error:
+	inputevt_remove(&adns_reply_event_id);
+	g_warning("adns_reply_callback: removed myself");
+	fd_close(&source);
 }
 
 /**
@@ -683,7 +674,7 @@ adns_reply_callback(gpointer data, gint source, inputevt_cond_t condition)
  */
 static adns_async_write_t *
 adns_async_write_alloc(const struct adns_request *req,
-	gconstpointer buf, size_t size)
+	const void *buf, size_t size)
 {
 	adns_async_write_t *remain;
 
@@ -691,7 +682,7 @@ adns_async_write_alloc(const struct adns_request *req,
 	g_assert(buf);
 	g_assert(size > 0);
 	
-	remain = walloc(sizeof *remain);
+	WALLOC(remain);
 	remain->req = *req;
 	remain->size = size;
 	remain->buf = wcopy(buf, remain->size);
@@ -711,7 +702,7 @@ adns_async_write_free(adns_async_write_t *remain)
 	g_assert(remain->size > 0);
 	
 	wfree(remain->buf, remain->size);
-	wfree(remain, sizeof *remain);
+	WFREE(remain);
 }
 
 /**
@@ -721,7 +712,7 @@ adns_async_write_free(adns_async_write_t *remain)
  * closed and the blocking adns_fallback() will be invoked.
  */
 static void
-adns_query_callback(gpointer data, gint dest, inputevt_cond_t condition)
+adns_query_callback(void *data, int dest, inputevt_cond_t condition)
 {
 	adns_async_write_t *remain = data;
 
@@ -755,13 +746,12 @@ adns_query_callback(gpointer data, gint dest, inputevt_cond_t condition)
 		}
 
 		g_assert(ret > 0);
-		g_assert((size_t) ret <= n);
+		g_assert(UNSIGNED(ret) <= n);
 		remain->pos += (size_t) ret;
 	}
 	g_assert(remain->pos == remain->size);
 
-	inputevt_remove(adns_query_event_id);
-	adns_query_event_id = 0;
+	inputevt_remove(&adns_query_event_id);
 
 	goto done;	
 
@@ -770,9 +760,8 @@ error:
 	g_warning("adns_query_callback: write() failed: %s", g_strerror(errno));
 abort:
 	g_warning("adns_query_callback: removed myself");
-	inputevt_remove(adns_query_event_id);
-	adns_query_event_id = 0;
-	CLOSE_IF_VALID(adns_query_fd);
+	inputevt_remove(&adns_query_event_id);
+	fd_close(&adns_query_fd);
 	g_warning("adns_query_callback: using fallback");
 	adns_fallback(&remain->req);
 done:
@@ -780,17 +769,16 @@ done:
 	return;
 }
 
-/* public functions */
-
-/**
- * Initializes the adns helper i.e., fork()s a child process which will
- * be used to resolve hostnames asynchronously.
- */
-void
-adns_init(void)
+static void
+adns_helper_init(void)
+#ifdef MINGW32
 {
-	gint fd_query[2] = {-1, -1};
-	gint fd_reply[2] = {-1, -1};
+	mingw_adns_init();
+}
+#else
+{
+	int fd_query[2] = {-1, -1};
+	int fd_reply[2] = {-1, -1};
 	pid_t pid;
 
 	if (-1 == pipe(fd_query) || -1 == pipe(fd_reply)) {
@@ -798,10 +786,6 @@ adns_init(void)
 		goto prefork_failure;
 	}
 	
-#ifdef SIGCHLD 
-	set_signal(SIGCHLD, SIG_IGN); /* prevent a zombie */
-#endif
-
 	pid = fork();
 	if ((pid_t) -1 == pid) {
 		g_warning("adns_init: fork() failed: %s", g_strerror(errno));
@@ -829,8 +813,11 @@ adns_init(void)
 			g_error("adns_init: freopen(\"/dev/null\", \"a\", stderr) failed: "
 					"%s", g_strerror(errno));
 
-		close(fd_query[1]);
-		close(fd_reply[0]);
+		fd_close(&fd_query[1]);
+		fd_close(&fd_reply[0]);
+
+		set_close_on_exec(fd_query[0]);
+		set_close_on_exec(fd_reply[1]);
 
 		adns_helper(fd_query[0], fd_reply[1]);
 		g_assert_not_reached();
@@ -838,12 +825,18 @@ adns_init(void)
 	}
 
 	/* parent process */
-	close(fd_query[0]);
-	close(fd_reply[1]);
+	fd_close(&fd_query[0]);
+	fd_close(&fd_reply[1]);
+	
+	fd_query[1] = get_non_stdio_fd(fd_query[1]);
+	fd_reply[0] = get_non_stdio_fd(fd_reply[0]);
+	
 	adns_query_fd = fd_query[1];
 
-	socket_set_nonblocking(adns_query_fd);
-	socket_set_nonblocking(fd_reply[0]);
+	set_close_on_exec(adns_query_fd);
+	set_close_on_exec(fd_reply[0]);
+	fd_set_nonblocking(adns_query_fd);
+	fd_set_nonblocking(fd_reply[0]);
 	
 	adns_reply_event_id = inputevt_add(fd_reply[0], INPUT_EVENT_RX,
 							adns_reply_callback, NULL);
@@ -852,13 +845,25 @@ prefork_failure:
 
 	if (!adns_reply_event_id) {
 		g_warning("Cannot use ADNS; DNS lookups may cause stalling");
-		CLOSE_IF_VALID(fd_query[0]);
-		CLOSE_IF_VALID(fd_query[1]);
-		CLOSE_IF_VALID(fd_reply[0]);
-		CLOSE_IF_VALID(fd_reply[1]);
+		fd_close(&fd_query[0]);
+		fd_close(&fd_query[1]);
+		fd_close(&fd_reply[0]);
+		fd_close(&fd_reply[1]);
 	}
+}
+#endif	/* MINGW32 */
 
+/* public functions */
+
+/**
+ * Initializes the adns helper i.e., fork()s a child process which will
+ * be used to resolve hostnames asynchronously.
+ */
+void
+adns_init(void)
+{
 	adns_cache = adns_cache_init();
+	adns_helper_init();
 }
 
 /**
@@ -866,8 +871,13 @@ prefork_failure:
  */
 static gboolean
 adns_send_request(const struct adns_request *req)
+#ifdef MINGW32
 {
-	gchar buf[sizeof *req];
+	return mingw_adns_send_request(req);
+}
+#else
+{
+	char buf[sizeof *req];
 	size_t size;
 	ssize_t written;
 
@@ -881,7 +891,7 @@ adns_send_request(const struct adns_request *req)
 	memcpy(buf, &req->common, sizeof req->common);
 	size = sizeof req->common;
 	{
-		gconstpointer p;
+		const void *p;
 		size_t n;
 		
 		if (req->common.reverse) {
@@ -904,9 +914,8 @@ adns_send_request(const struct adns_request *req)
 		if (!is_temporary_error(errno)) {
 			g_warning("adns_resolve: write() failed: %s",
 				g_strerror(errno));
-			inputevt_remove(adns_reply_event_id);
-			adns_reply_event_id = 0;
-			CLOSE_IF_VALID(adns_query_fd);
+			inputevt_remove(&adns_reply_event_id);
+			fd_close(&adns_query_fd);
 			return FALSE;
 		}
 		written = 0;
@@ -920,7 +929,7 @@ adns_send_request(const struct adns_request *req)
 	 * can absorb new data.
 	 */
 
-	if ((size_t) written < size) {
+	if (UNSIGNED(written) < size) {
 		adns_async_write_t *aq;
 	   
 		aq = adns_async_write_alloc(req, &buf[written], size - written);
@@ -930,6 +939,7 @@ adns_send_request(const struct adns_request *req)
 
 	return TRUE;
 }
+#endif	/* MINGW32 */
 
 /**
  * Creates a DNS resolve query for ``hostname''.
@@ -948,8 +958,8 @@ adns_send_request(const struct adns_request *req)
  * returned, adns_resolve() returns FALSE.
  */
 gboolean
-adns_resolve(const gchar *hostname, enum net_type net,
-	adns_callback_t user_callback, gpointer user_data)
+adns_resolve(const char *hostname, enum net_type net,
+	adns_callback_t user_callback, void *user_data)
 {
 	struct adns_request req;
 	struct adns_response ans;
@@ -970,8 +980,8 @@ adns_resolve(const gchar *hostname, enum net_type net,
 	reply->hostname[0] = '\0';
 	reply->addrs[0] = zero_host_addr;
 
-	hostname_len = g_strlcpy(query->hostname, hostname, sizeof query->hostname);
-	if (hostname_len >= sizeof query->hostname) {
+	hostname_len = clamp_strcpy(query->hostname, sizeof query->hostname, hostname);
+	if ('\0' != hostname[hostname_len]) {
 		/* truncation detected */
 		adns_invoke_user_callback(&ans);
 		return FALSE; /* synchronous */
@@ -985,7 +995,7 @@ adns_resolve(const gchar *hostname, enum net_type net,
 	}
 
 	ascii_strlower(query->hostname, hostname);
-	g_strlcpy(reply->hostname, query->hostname, sizeof reply->hostname);
+	clamp_strcpy(reply->hostname, sizeof reply->hostname, query->hostname);
 	
 	if (
 		adns_cache_lookup(adns_cache, tm_time(), query->hostname,
@@ -998,10 +1008,10 @@ adns_resolve(const gchar *hostname, enum net_type net,
 	if (adns_send_request(&req))
 		return TRUE; /* asynchronous */
 
-	if (adns_reply_event_id)
+	if (adns_reply_event_id) {
 		g_warning("adns_resolve: using synchronous resolution for \"%s\"",
 			query->hostname);
-
+	}
 	adns_fallback(&req);
 
 	return FALSE; /* synchronous */
@@ -1022,7 +1032,7 @@ adns_resolve(const gchar *hostname, enum net_type net,
  */
 gboolean
 adns_reverse_lookup(const host_addr_t addr,
-	adns_reverse_callback_t user_callback, gpointer user_data)
+	adns_reverse_callback_t user_callback, void *user_data)
 {
 	struct adns_request req;
 	struct adns_reverse_query *query = &req.query.reverse;
@@ -1051,17 +1061,14 @@ adns_reverse_lookup(const host_addr_t addr,
 void
 adns_close(void)
 {
-	if (adns_reply_event_id) {
-		inputevt_remove(adns_reply_event_id);
-		adns_reply_event_id = 0;
-	}
-	if (adns_query_event_id) {
-		inputevt_remove(adns_query_event_id);
-		adns_query_event_id = 0;
-	}
-	
-	adns_cache_free(adns_cache);
-	adns_cache = NULL;
+#ifdef MINGW32
+	mingw_adns_close();
+#else
+	inputevt_remove(&adns_reply_event_id);
+	inputevt_remove(&adns_query_event_id);
+#endif	/* MINGW32 */
+
+	adns_cache_free(&adns_cache);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
