@@ -1,5 +1,5 @@
 /*
- * $Id: atoms.c 11958 2006-09-13 23:30:37Z cbiere $
+ * $Id: atoms.c 19191 2011-05-30 22:18:28Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -41,8 +41,9 @@
 #include "common.h"
 
 #include "atoms.h"
+#include "endian.h"
 
-RCSID("$Id: atoms.c 11958 2006-09-13 23:30:37Z cbiere $")
+RCSID("$Id: atoms.c 19191 2011-05-30 22:18:28Z rmanfredi $")
 
 #if 0
 #define PROTECT_ATOMS
@@ -78,6 +79,7 @@ typedef enum {
 #endif	/* ATOMS_HAVE_MAGIC */
 
 
+#include "stringify.h"
 #include "misc.h"
 #include "glib-missing.h"
 #include "walloc.h"
@@ -85,13 +87,8 @@ typedef enum {
 #include "override.h"		/* Must be the last header included */
 
 union mem_chunk {
-  void      *next;
-  guint8   u8;
-  guint16  u16;
-  guint32  u32;
-  guint64  u64;
-  gfloat     f;
-  gdouble    d;
+  void *next;
+  char align[MEM_ALIGNBYTES];
 };
 
 #define ATOM_TYPE_MASK	((size_t) 0x07)
@@ -100,28 +97,34 @@ union mem_chunk {
  * Atoms are ref-counted.
  *
  * The reference count is held at the beginning of the data arena.
- * What we return to the outside is a pointer to arena[], not to
- * the atom structure.
+ * What we return to the outside is the value of atom_arena(), not a
+ * pointer to the atom structure.
  */
 typedef struct atom {
+#ifdef ATOMS_HAVE_MAGIC
+	atom_prot_magic_t magic;	/**< Magic should be at the beginning */
+#endif /* ATOM_HAVE_MAGIC */
+	int refcnt;				/**< Amount of references */
 #ifdef TRACK_ATOMS
 	GHashTable *get;		/**< Allocation spots */
 	GHashTable *free;		/**< Free spots */
 #endif
-#ifdef ATOMS_HAVE_MAGIC
-	atom_prot_magic_t magic;
-#endif /* PROTECT_ATOMS */
-	gint refcnt;			/**< Amount of references */
-	/* Ensure `arena' is properly aligned */
-	union mem_chunk arena[1];			/* Start of user arena */
 } atom_t;
 
-#define ARENA_OFFSET	G_STRUCT_OFFSET(struct atom, arena)
+#define ARENA_OFFSET \
+	(MEM_ALIGNBYTES * ((sizeof(atom_t) / MEM_ALIGNBYTES) + \
+		((sizeof(atom_t) % MEM_ALIGNBYTES) ? 1 : 0)))
 
 static inline atom_t *
 atom_from_arena(gconstpointer key)
 {
-	return (gpointer) ((gchar *) key - ARENA_OFFSET);
+	return (atom_t *) (((char *) key) - ARENA_OFFSET);
+}
+
+static inline void *
+atom_arena(const atom_t *a)
+{
+	return ((char *) a) + ARENA_OFFSET;
 }
 
 static inline void
@@ -144,7 +147,7 @@ struct mem_cache {
 	size_t hold;
 };
 
-static struct mem_cache *mc[9];
+static struct mem_cache *mem_cache[9];
 
 static struct mem_pool *
 mem_pool_new(size_t size, size_t hold)
@@ -159,7 +162,7 @@ mem_pool_new(size_t size, size_t hold)
   ps = compat_pagesize();
   len = hold * size;
   
-  mp = alloc_pages(len);
+  mp = vmm_alloc(len);
   if (mp) {
     size_t i, step = size / sizeof mp->chunks[0];
     
@@ -259,9 +262,9 @@ atom_get_mc(size_t size)
 {
 	guint i;
 
-	for (i = 0; i < G_N_ELEMENTS(mc); i++) {
-		if (size <= mc[i]->size)
-			return mc[i];
+	for (i = 0; i < G_N_ELEMENTS(mem_cache); i++) {
+		if (size <= mem_cache[i]->size)
+			return mem_cache[i];
 	}
 	return NULL;
 }
@@ -277,7 +280,7 @@ mem_protect(gpointer ptr, size_t size)
 	size_t ps;
 
 	ps = compat_pagesize();
-	p = (gchar *) ptr - (uintptr_t) ptr % ps;
+	p = (char *) ptr - (uintptr_t) ptr % ps;
 	size = round_size(ps, size);
 
 	if (-1 == mprotect(p, size, PROT_READ))
@@ -296,7 +299,7 @@ mem_unprotect(gpointer ptr, size_t size)
 	size_t ps;
 
 	ps = compat_pagesize();
-	p = (gchar *) ptr - (uintptr_t) ptr % ps;
+	p = (char *) ptr - (uintptr_t) ptr % ps;
 	size = round_size(ps, size);
 
 	if (-1 == mprotect(p, size, PROT_READ | PROT_WRITE))
@@ -343,7 +346,7 @@ atom_alloc(size_t size)
 		a = mem_alloc(mc_ptr);
 		mem_unprotect(a, size);
 	} else {
-		a = alloc_pages(size);
+		a = vmm_alloc(size);
 	}
 	ATOM_SET_MAGIC(a);
 
@@ -377,7 +380,7 @@ atom_dealloc(atom_t *a, size_t size)
 		 * relies on executable heap-memory because the page is now
 		 * mapped (PROT_READ | PROT_WRITE).
 		 */
-		FREE_PAGES_NULL(a, size);
+		VMM_FREE_NULL(a, size);
 	}
 }
 #else
@@ -401,13 +404,13 @@ atom_dealloc(atom_t *a, size_t size)
 #endif /* PROTECT_ATOMS */
 
 typedef size_t (*len_func_t)(gconstpointer v);
-typedef const gchar *(*str_func_t)(gconstpointer v);
+typedef const char *(*str_func_t)(gconstpointer v);
 
 /**
  * Description of atom types.
  */
 typedef struct table_desc {
-	const gchar *type;			/**< Type of atoms */
+	const char *type;			/**< Type of atoms */
 	GHashTable *table;			/**< Table of atoms: "atom value" => 1 */
 	GHashFunc hash_func;		/**< Hashing function for atoms */
 	GCompareFunc eq_func;		/**< Atom equality function */
@@ -416,15 +419,20 @@ typedef struct table_desc {
 } table_desc_t;
 
 static size_t str_len(gconstpointer v);
-static const gchar *str_str(gconstpointer v);
+static const char *str_str(gconstpointer v);
 static size_t guid_len(gconstpointer v);
-static const gchar *guid_str(gconstpointer v);
+static const char *guid_str(gconstpointer v);
 static size_t sha1_len(gconstpointer v);
-static const gchar *sha1_str(gconstpointer v);
+static const char *sha1_str(gconstpointer v);
+static size_t tth_len(gconstpointer v);
+static const char *tth_str(gconstpointer v);
 static size_t uint64_len(gconstpointer v);
-static const gchar *uint64_str(gconstpointer v);
+static const char *uint64_str(gconstpointer v);
 static size_t filesize_len(gconstpointer v);
-static const gchar *filesize_str(gconstpointer v);
+static const char *filesize_str(gconstpointer v);
+static size_t uint32_len(gconstpointer v);
+static const char *uint32_str(gconstpointer v);
+static const char *gnet_host_str(gconstpointer v);
 
 /**
  * The set of all atom types we know about.
@@ -433,9 +441,13 @@ static table_desc_t atoms[] = {
 	{ "String",	NULL, g_str_hash,  g_str_equal, str_len,    str_str  },	/* 0 */
 	{ "GUID",	NULL, guid_hash,   guid_eq,	    guid_len,   guid_str },	/* 1 */
 	{ "SHA1",	NULL, sha1_hash,   sha1_eq,	    sha1_len,   sha1_str },	/* 2 */
-	{ "uint64",	NULL, uint64_hash, uint64_eq,   uint64_len, uint64_str},/* 3 */
-	{ "filesize",
-		NULL, filesize_hash, filesize_eq, filesize_len, filesize_str},  /* 4 */
+	{ "TTH",	NULL, tth_hash,    tth_eq,	    tth_len,    tth_str },	/* 3 */
+	{ "uint64",	NULL, uint64_hash, uint64_eq,   uint64_len, uint64_str},/* 4 */
+	{ "filesize", NULL,
+		filesize_hash, filesize_eq, filesize_len, filesize_str},		/* 5 */
+	{ "uint32",	NULL, uint32_hash, uint32_eq,   uint32_len, uint32_str},/* 6 */
+	{ "host", NULL,
+		gnet_host_hash, gnet_host_eq, gnet_host_length, gnet_host_str},	/* 7 */
 };
 
 static GHashTable *ht_all_atoms;
@@ -446,43 +458,52 @@ static GHashTable *ht_all_atoms;
 static size_t 
 str_len(gconstpointer v)
 {
-	return strlen((const gchar *) v) + 1;
+	return strlen((const char *) v) + 1;
 }
 
 /**
  * @return printable form of a string, i.e. self.
  */
-static const gchar *
+static const char *
 str_str(gconstpointer v)
 {
-	return (const gchar *) v;
+	return (const char *) v;
 }
 
 /**
- * Hash `len' bytes (multiple of 4) starting from `key'.
+ * Hash `len' bytes starting from `data'.
  */
 guint
-binary_hash(const guchar *key, guint len)
+binary_hash(gconstpointer data, size_t len)
 {
-	guint hash = len;
-	guint i;
+	const unsigned char *key = data;
+	size_t i, remain, t4;
+	guint32 hash;
 
-	g_assert(0 == (len & 0x3));		/* Multiple of 4 */
+	remain = len & 0x3;
+	t4 = len & ~0x3U;
 
-	for (i = 0; i < len; i += 4) {
+	g_assert(remain + t4 == len);
+	g_assert(remain <= 3);
+
+	hash = len;
+	for (i = 0; i < t4; i += 4) {
 		static const guint32 x[] = {
 			0xb0994420, 0x01fa96e3, 0x05066d0e, 0x50c3c22a,
 			0xec99f01f, 0xc0eaa79d, 0x157d4257, 0xde2b8419
 		};
-		hash ^= key[i] |
-			((guint) key[i + 1] << 8) |
-			((guint) key[i + 2] << 16) |
-			((guint) key[i + 3] << 24);
+		hash ^= peek_le32(&key[i]);
 		hash += x[(i >> 2) & 0x7];
 		hash = (hash << 24) ^ (hash >> 8);
 	}
 
-	return hash;
+	for (i = 0; i < remain; i++) {
+		hash += key[t4 + i];
+		hash ^= key[t4 + i] << (i * 8);
+		hash = (hash << 24) ^ (hash >> 8);
+	}
+
+	return pointer_hash_func((void *) (size_t) hash);
 }
 
 /**
@@ -497,7 +518,7 @@ guid_hash(gconstpointer key)
 /**
  * Test two GUIDs for equality.
  */
-gint
+int
 guid_eq(gconstpointer a, gconstpointer b)
 {
 /* FIXME:	Disabled because of alignment problems
@@ -531,49 +552,29 @@ guid_len(gconstpointer unused_guid)
 /**
  * @return printable form of a GUID, as pointer to static data.
  */
-static const gchar *
+static const char *
 guid_str(gconstpointer v)
 {
-	return guid_hex_str((const gchar *) v);
+	const struct guid *guid = v;
+	return guid_hex_str(guid);
 }
 
 /**
  * Hash a SHA1 (20 bytes).
- *
- * @attention
- * NB: This routine is visible for the download mesh.
  */
 guint
 sha1_hash(gconstpointer key)
 {
-	return binary_hash(key, 20);
+	return binary_hash(key, SHA1_RAW_SIZE);
 }
 
 /**
  * Test two SHA1s for equality.
- *
- * @attention
- * NB: This routine is visible for the download mesh.
  */
-gint
+int
 sha1_eq(gconstpointer a, gconstpointer b)
 {
-/* FIXME:	Disabled because of alignment problems
- *			This should work if sha1_t (?) was declared as guint32[5].
- */
-#if 0
-	const guint32 *ax = (const guint32 *) a;
-	const guint32 *bx = (const guint32 *) b;
-
-	return
-		ax[0] == bx[0] &&
-		ax[1] == bx[1] &&
-		ax[2] == bx[2] &&
-		ax[3] == bx[3] &&
-		ax[4] == bx[4];
-#else
-	return a == b || 0 == memcmp(a, b, 20);
-#endif
+	return a == b || 0 == memcmp(a, b, SHA1_RAW_SIZE);
 }
 
 /**
@@ -583,16 +584,56 @@ static size_t
 sha1_len(gconstpointer unused_sha1)
 {
 	(void) unused_sha1;
-	return 20;
+	return SHA1_RAW_SIZE;
 }
 
 /**
  * @return printable form of a SHA1, as pointer to static data.
  */
-static const gchar *
+static const char *
 sha1_str(gconstpointer sha1)
 {
-	return sha1_base32((const gchar *) sha1);
+	return sha1_base32(sha1);
+}
+
+/**
+ * Hash a TTH (24 bytes).
+ */
+guint
+tth_hash(gconstpointer key)
+{
+	return binary_hash(key, TTH_RAW_SIZE);
+}
+
+/**
+ * Test two TTHs for equality.
+ *
+ * @attention
+ * NB: This routine is visible for the download mesh.
+ */
+int
+tth_eq(gconstpointer a, gconstpointer b)
+{
+	return a == b || 0 == memcmp(a, b, TTH_RAW_SIZE);
+}
+
+/**
+ * @return length of TTH.
+ */
+static size_t 
+tth_len(gconstpointer unused_tth)
+{
+	(void) unused_tth;
+	return TTH_RAW_SIZE;
+}
+
+/**
+ * @return printable form of a TTH, as pointer to static data.
+ */
+static const char *
+tth_str(gconstpointer tth)
+{
+	return tth_base32(tth);
 }
 
 /**
@@ -615,13 +656,23 @@ filesize_len(gconstpointer unused_v)
 	return sizeof(filesize_t);
 }
 
+/**
+ * @return length of a 32-bit integer.
+ */
+static size_t
+uint32_len(gconstpointer unused_v)
+{
+	(void) unused_v;
+	return sizeof(guint32);
+}
+
 /**
  * @return printable form of a 64-bit integer, as pointer to static data.
  */
-static const gchar *
+static const char *
 uint64_str(gconstpointer v)
 {
-	static gchar buf[UINT64_DEC_BUFLEN];
+	static char buf[UINT64_DEC_BUFLEN];
 
 	uint64_to_string_buf(*(const guint64 *) v, buf, sizeof buf);
 	return buf;
@@ -632,7 +683,7 @@ uint64_str(gconstpointer v)
  *
  * @return whether both referenced 64-bit integers are equal.
  */
-gint
+int
 uint64_eq(gconstpointer a, gconstpointer b)
 {
 	return *(const guint64 *) a == *(const guint64 *) b;
@@ -650,24 +701,60 @@ uint64_hash(gconstpointer p)
 	return v ^ (v >> 32);
 }
 
+/**
+ * Test two 64-bit integers for equality, with pointers not necessarily
+ * aligned on 64-bit quantities.
+ *
+ * @return whether both referenced 64-bit integers are equal.
+ */
+int
+uint64_mem_eq(gconstpointer a, gconstpointer b)
+{
+	return a == b || 0 == memcmp(a, b, sizeof(guint64));
+}
+
+/**
+ * Calculate the 32-bit hash of a 64-bit integer whose address is not
+ * necessarily aligned on 64-bit quantities.
+ *
+ * @return the 32-bit hash value for the referenced 64-bit integer.
+ */
+guint
+uint64_mem_hash(gconstpointer p)
+{
+	return binary_hash(p, sizeof(guint64));
+}
+
 /**
  * @return printable form of a filesize_t, as pointer to static data.
  */
-static const gchar *
+static const char *
 filesize_str(gconstpointer v)
 {
-	static gchar buf[UINT64_DEC_BUFLEN];
+	static char buf[UINT64_DEC_BUFLEN];
 
 	uint64_to_string_buf(*(const filesize_t *) v, buf, sizeof buf);
 	return buf;
 }
 
+/**
+ * @return printable form of a gnet_host_t *, as pointer to static data.
+ */
+static const char *
+gnet_host_str(gconstpointer v)
+{
+	static char buf[HOST_ADDR_PORT_BUFLEN];
+
+	gnet_host_to_string_buf(v, buf, sizeof buf);
+	return buf;
+}
+
 /**
  * Test two filesize_t for equality.
  *
  * @return whether both referenced 64-bit integers are equal.
  */
-gint
+int
 filesize_eq(gconstpointer a, gconstpointer b)
 {
 	return *(const filesize_t *) a == *(const filesize_t *) b;
@@ -685,6 +772,40 @@ filesize_hash(gconstpointer p)
 	return v ^ (v >> 32);
 }
 
+/**
+ * Test two 32-bit integers for equality.
+ *
+ * @return whether both referenced 32-bit integers are equal.
+ */
+int
+uint32_eq(gconstpointer a, gconstpointer b)
+{
+	return *(const guint32 *) a == *(const guint32 *) b;
+}
+
+/**
+ * Calculate the 32-bit hash of a 32-bit integer
+ *
+ * @return the 32-bit hash value for the referenced 32-bit integer.
+ */
+guint
+uint32_hash(gconstpointer p)
+{
+	guint32 v = *(const guint32 *) p;
+	return v;
+}
+
+/**
+ * @return printable form of a 32-bit integer, as pointer to static data.
+ */
+static const char *
+uint32_str(gconstpointer v)
+{
+	static char buf[UINT32_DEC_BUFLEN];
+
+	uint32_to_string_buf(*(const guint32 *) v, buf, sizeof buf);
+	return buf;
+}
 
 /**
  * Initialize atom structures.
@@ -692,14 +813,22 @@ filesize_hash(gconstpointer p)
 void
 atoms_init(void)
 {
+	gboolean has_setting = FALSE;
+	struct atom_settings {
+		guint8 track_atoms;
+		guint8 protect_atoms;
+		guint8 atoms_have_magic;
+	} settings;
 	guint i;
 
+	ZERO(&settings);
+
 	STATIC_ASSERT(NUM_ATOM_TYPES <= (ATOM_TYPE_MASK + 1));
 	STATIC_ASSERT(NUM_ATOM_TYPES == G_N_ELEMENTS(atoms));
 
 #ifdef PROTECT_ATOMS
-	for (i = 0; i < G_N_ELEMENTS(mc); i++) {
-		mc[i] = mem_new((2 * sizeof (union mem_chunk)) << i);
+	for (i = 0; i < G_N_ELEMENTS(mem_cache); i++) {
+		mem_cache[i] = mem_new((2 * sizeof (union mem_chunk)) << i);
 	}
 #endif /* PROTECT_ATOMS */
 
@@ -708,20 +837,49 @@ atoms_init(void)
 
 		td->table = g_hash_table_new(td->hash_func, td->eq_func);
 	}
-	ht_all_atoms = g_hash_table_new(NULL, NULL);
+	ht_all_atoms = g_hash_table_new(pointer_hash_func, NULL);
+
+	/*
+	 * Log atoms configuration.
+	 */
+
+#ifdef TRACK_ATOMS
+	settings.track_atoms = TRUE;
+	has_setting = TRUE;
+#endif
+#ifdef PROTECT_ATOMS
+	settings.protect_atoms = TRUE;
+	has_setting = TRUE;
+#endif
+#ifdef ATOMS_HAVE_MAGIC
+	settings.atoms_have_magic = TRUE;
+	has_setting = TRUE;
+#endif
+
+	if (has_setting) {
+		g_message("atom settings: %s%s%s",
+			settings.track_atoms ? "TRACK_ATOMS " : "",
+			settings.protect_atoms ? "PROTECT_ATOMS " : "",
+			settings.atoms_have_magic ? "ATOMS_HAVE_MAGIC " : "");
+	}
 }
 
+/**
+ * Check whether ``key'' is an atom of type ``type''.
+ *
+ * @return the size of the atom if found, 0 otherwise.
+ */
 static inline size_t
 atom_is_registered(enum atom_type type, gconstpointer key)
 {
 	gpointer value;
 
 	if (g_hash_table_lookup_extended(ht_all_atoms, key, NULL, &value)) {
-		/* If the address is already registered in the global atom table,
+		/*
+		 * If the address is already registered in the global atom table,
 		 * this is definitely an atom. However, the same memory object
 		 * could be shared by atoms of different types (in theory at least),
-		 * thus we must check whether the types are identical. Otherwise,
-		 * a new atom must be created.
+		 * thus we must check whether the types are identical.
 		 */
 
 		if (((size_t) value & ATOM_TYPE_MASK) == (guint) type) {
@@ -733,43 +891,57 @@ atom_is_registered(enum atom_type type, gconstpointer key)
 	return 0;
 }
 
+/**
+ * Check whether atom exists.
+ *
+ * @return TRUE if ``key'' points to a ``type'' atom.
+ */
+gboolean
+atom_exists(enum atom_type type, gconstpointer key)
+{
+	g_assert(key != NULL);
+
+	return atom_is_registered(type, key) > 0 ||
+		gm_hash_table_contains(atoms[type].table, key);
+}
+
 /**
  * Get atom of given `type', whose value is `key'.
  * If the atom does not exist yet, `key' is cloned and makes up the new atom.
  *
  * @return the atom's value.
  */
-gpointer
+gconstpointer
 atom_get(enum atom_type type, gconstpointer key)
 {
 	table_desc_t *td;
 	gpointer orig_key;
 	size_t size;
 
-#ifdef __GNUC__
 	STATIC_ASSERT(0 == ARENA_OFFSET % MEM_ALIGNBYTES);
-#else
-	/* ARENA_OFFSET is no constant expression with some Sun compiler */
-	g_assert(0 == ARENA_OFFSET % MEM_ALIGNBYTES);
-#endif /* __GNUC__ */
+	STATIC_ASSERT(ARENA_OFFSET >= sizeof(atom_t));
 	
     g_assert(key != NULL);
-	g_assert((gint) type >= 0 && (guint) type < G_N_ELEMENTS(atoms));
+	g_assert(UNSIGNED(type) < G_N_ELEMENTS(atoms));
 
 	td = &atoms[type];		/* Where atoms of this type are held */
 
-	/* If the address is already registered in the global atom table,
-	 * this is definitely an atom. However, the same memory object
-	 * could be shared by atoms of different types (in theory at least),
-	 * thus we must check whether the types are identical. Otherwise,
-	 * a new atom must be created.
-	 */
-
 	size = atom_is_registered(type, key);
 	if (size > 0) {
+		/* Atom already exists for key and type */
 		orig_key = deconstify_gpointer(key);
 	} else {
 		gpointer value;
+
+		/*
+		 * Normally, if atom exists, it will be found by atom_is_registered().
+		 * If not, look in the type-specific table.
+		 *
+		 * This happens when the first few bytes of the atom arena are shared
+		 * by atoms of different types (it creates conflicts in the global
+		 * ht_all_atoms table).
+		 */
+
 		if (g_hash_table_lookup_extended(td->table, key, &orig_key, &value)) {
 			size = (size_t) value;
 			g_assert(size >= ARENA_OFFSET);
@@ -803,11 +975,11 @@ atom_get(enum atom_type type, gconstpointer key)
 
 		len = (*td->len_func)(key);
 		g_assert(len < ((size_t) -1) - ARENA_OFFSET);
-		size = round_size((ATOM_TYPE_MASK + 1), ARENA_OFFSET + len);
+		size = round_size_fast((ATOM_TYPE_MASK + 1), ARENA_OFFSET + len);
 
 		a = atom_alloc(size);
 		a->refcnt = 1;
-		memcpy(a->arena, key, len);
+		memcpy(atom_arena(a), key, len);
 		atom_protect(a, size);
 
 		/*
@@ -815,10 +987,10 @@ atom_get(enum atom_type type, gconstpointer key)
 		 */
 
 		value = (gpointer) (size | (guint) type);
-		g_hash_table_insert(ht_all_atoms, a->arena, value);
-		g_hash_table_insert(td->table, a->arena, (gpointer) size);
+		g_hash_table_insert(ht_all_atoms, atom_arena(a), value);
+		g_hash_table_insert(td->table, atom_arena(a), (gpointer) size);
 
-		return a->arena;
+		return atom_arena(a);
 	}
 }
 
@@ -833,7 +1005,7 @@ atom_free(enum atom_type type, gconstpointer key)
 	atom_t *a;
 
     g_assert(key != NULL);
-	g_assert((gint) type >= 0 && (guint) type < G_N_ELEMENTS(atoms));
+	g_assert(UNSIGNED(type) < G_N_ELEMENTS(atoms));
 
 	size = atom_is_registered(type, key);
 	g_assert(size > 0);
@@ -862,7 +1034,7 @@ atom_free(enum atom_type type, gconstpointer key)
 
 /** Information about given spot. */
 struct spot {
-	gint count;			/**< Amount of allocation/free performed at spot */
+	int count;			/**< Amount of allocation/free performed at spot */
 };
 
 /**
@@ -870,12 +1042,12 @@ struct spot {
  *
  * @returns the atom's value.
  */
-gpointer
-atom_get_track(enum atom_type type, gconstpointer key, gchar *file, gint line)
+gconstpointer
+atom_get_track(enum atom_type type, gconstpointer key, char *file, int line)
 {
-	gpointer atom;
+	gconstpointer atom;
 	atom_t *a;
-	gchar buf[512];
+	char buf[512];
 	gpointer k;
 	gpointer v;
 	struct spot *sp;
@@ -898,7 +1070,7 @@ atom_get_track(enum atom_type type, gconstpointer key, gchar *file, gint line)
 		sp = (struct spot *) v;
 		sp->count++;
 	} else {
-		sp = walloc(sizeof(*sp));
+		WALLOC(sp);
 		sp->count = 1;
 		g_hash_table_insert(a->get, g_strdup(buf), sp);
 	}
@@ -914,7 +1086,7 @@ tracking_free_kv(gpointer key, gpointer value, gpointer uu_user)
 {
 	(void) uu_user;
 
-	g_free(key);
+	G_FREE_NULL(key);
 	wfree(value, sizeof(struct spot));
 	return TRUE;
 }
@@ -935,10 +1107,10 @@ destroy_tracking_table(GHashTable *h)
  * Dispose of atom if nobody references it anymore.
  */
 void
-atom_free_track(enum atom_type type, gconstpointer key, gchar *file, gint line)
+atom_free_track(enum atom_type type, gconstpointer key, char *file, int line)
 {
 	atom_t *a;
-	gchar buf[512];
+	char buf[512];
 	gpointer k;
 	gpointer v;
 	struct spot *sp;
@@ -959,7 +1131,7 @@ atom_free_track(enum atom_type type, gconstpointer key, gchar *file, gint line)
 			sp = (struct spot *) v;
 			sp->count++;
 		} else {
-			sp = walloc(sizeof(*sp));
+			WALLOC(sp);
 			sp->count = 1;
 			g_hash_table_insert(a->free, g_strdup(buf), sp);
 		}
@@ -976,16 +1148,16 @@ static void
 dump_tracking_entry(gpointer key, gpointer value, gpointer user)
 {
 	struct spot *sp = (struct spot *) value;
-	const gchar *what = (const gchar *) user;
+	const char *what = (const char *) user;
 
-	g_warning("%10d %s at \"%s\"", sp->count, what, (gchar *) key);
+	g_warning("%10d %s at \"%s\"", sp->count, what, (char *) key);
 }
 
 /**
  * Dump the values held in the tracking table `h'.
  */
 static void
-dump_tracking_table(gpointer atom, GHashTable *h, gchar *what)
+dump_tracking_table(gpointer atom, GHashTable *h, char *what)
 {
 	guint count = g_hash_table_size(h);
 
@@ -1039,9 +1211,9 @@ atoms_close(void)
 		table_desc_t *td = &atoms[i];
 
 		g_hash_table_foreach_remove(td->table, atom_warn_free, td);
-		g_hash_table_destroy(td->table);
+		gm_hash_table_destroy_null(&td->table);
 	}
-	g_hash_table_destroy(ht_all_atoms);
+	gm_hash_table_destroy_null(&ht_all_atoms);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
