@@ -1,5 +1,5 @@
 /*
- * $Id: base32.c 13653 2007-05-15 00:24:25Z cbiere $
+ * $Id: base32.c 19191 2011-05-30 22:18:28Z rmanfredi $
  *
  * Copyright (c) 2006, Christian Biere
  *
@@ -65,6 +65,7 @@
 
 #include "common.h"
 #include "base32.h"
+#include "ascii.h"		/* For ascii_toupper() */
 
 /*
  * See RFC 3548 for details about Base 32 encoding:
@@ -78,6 +79,16 @@ static const char base32_alphabet[32] = {
 	'Y', 'Z', '2', '3', '4', '5', '6', '7'
 };
 
+/**
+ * Encode in base32 `len' bytes of `data' into the buffer `dst'.
+ *
+ * @param dst		destination buffer
+ * @param size		length of destination
+ * @param data		start of data to encode
+ * @param len		amount of bytes to encode
+ *
+ * @return the amount of bytes generated into the destination.
+ */
 size_t
 base32_encode(char *dst, size_t size, const void *data, size_t len)
 {
@@ -149,21 +160,18 @@ base32_encode(char *dst, size_t size, const void *data, size_t len)
 	return q - dst;
 }
 
-static inline int
-ascii_toupper(int c)
-{
-	return c >= 97 && c <= 122 ? c - 32 : c;
-}
-
-static inline int
-ascii_tolower(int c)
-{
-	return c >= 65 && c <= 90 ? c + 32 : c;
-}
-
-
 static char base32_map[(unsigned char) -1];
 
+/**
+ * Decode a base32 encoding of `len' bytes of `data' into the buffer `dst'.
+ *
+ * @param dst		destination buffer
+ * @param size		length of destination
+ * @param data		start of data to decode
+ * @param len		amount of encoded data to decode
+ *
+ * @return the amount of bytes decoded into the destination.
+ */
 size_t
 base32_decode(char *dst, size_t size, const void *data, size_t len)
 {
@@ -184,7 +192,7 @@ base32_decode(char *dst, size_t size, const void *data, size_t len)
 		}
 	}
 
-	memset(&s[0], 0, sizeof s);
+	ZERO(&s);
 	si = 0;
 	i = 0;
 
