@@ -1,14 +1,8 @@
 /*
- * $Id: bg.c,v 1.4 2005/04/28 19:41:33 cbiere Exp $
+ * $Id: bg.c 19321 2011-07-06 16:50:51Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
- * Background task management.
- *
- * A background task is some CPU or I/O intensive operation that needs to
- * be split up in small chunks of processing because it would block the
- * process for too long if executed atomically.
- *
  *----------------------------------------------------------------------
  * This file is part of gtk-gnutella.
  *
@@ -29,23 +23,51 @@
  *----------------------------------------------------------------------
  */
 
+/**
+ * @ingroup lib
+ * @file
+ *
+ * Background task management.
+ *
+ * A background task is some CPU or I/O intensive operation that needs to
+ * be split up in small chunks of processing because it would block the
+ * process for too long if executed atomically.
+ *
+ * @author Raphael Manfredi
+ * @date 2002-2003
+ */
+
 #include "common.h"
 
-RCSID("$Id: bg.c,v 1.4 2005/04/28 19:41:33 cbiere Exp $");
+RCSID("$Id: bg.c 19321 2011-07-06 16:50:51Z rmanfredi $")
 
 #include "bg.h"
+#include "cq.h"
+#include "misc.h"
+#include "tm.h"
 #include "walloc.h"
+#include "glib-missing.h"
+#include "stacktrace.h"
+
 #include "override.h"		/* Must be the last header included */
 
-#define BT_MAGIC		0xbacc931dU		/* Internal bgtask magic number */
+enum bgtask_magic {
+	BGTASK_MAGIC	  = 0x3acc931dU,
+	BGTASK_DEAD_MAGIC = 0x6f5c8a03U
+};
 
-#define MAX_LIFE		150000			/* In useconds, MUST be << 1 sec */
-#define MIN_LIFE		40000			/* Min lifetime per task, in usecs */
-#define DELTA_FACTOR	4				/* Max variations are 400% */
+#define MAX_LIFE		100000UL		/**< In usecs, MUST be << 400 ms */
+#define DELTA_FACTOR	2				/**< Max variations are 200% */
 
-static guint32 common_dbg = 0;	/* XXX -- need to init lib's props --RAM */
+#define BG_TICK_IDLE	1000			/**< Tick every second when idle */
+#define BG_TICK_BUSY	400				/**< Tick every 400 ms when busy */
 
-/*
+static struct {
+	cperiodic_t *pev;		/**< Ticker periodic event */
+	int period;				/**< Current period */
+} bg_ticker;
+
+/**
  * Internal representation of a user-defined task.
  *
  * `step' is the current processing step.  Several processing steps can be
@@ -59,76 +81,105 @@ static guint32 common_dbg = 0;	/* XXX -- need to init lib's props --RAM */
  * `stepvec' is the set of steps we have to run (normally in sequence).
  */
 struct bgtask {
-	guint magic;			/* Magic number */
-	guint32 flags;			/* Operating flags */
-	gchar *name;			/* Task name */
-	gint step;				/* Current processing step */
-	gint seqno;				/* Number of calls at same step */
-	bgstep_cb_t *stepvec;	/* Set of steps to run in sequence */
-	gint stepcnt;			/* Amount of steps in the `stepvec' array */
-	gpointer ucontext;		/* User context */
-	time_t ctime;			/* Creation time */
-	gint wtime;				/* Wall-clock run time sofar, in ms */
-	bgclean_cb_t uctx_free;	/* Free routine for context */
-	bgdone_cb_t done_cb;	/* Called when done */
-	gpointer done_arg;		/* "done" callback argument */
-	gint exitcode;			/* Final "exit" code */
-	bgsig_t signal;			/* Last signal delivered */
-	GSList *signals;		/* List of signals pending delivery */
-	jmp_buf env;			/* Only valid when TASK_F_RUNNING */
-	GTimeVal start;			/* Start time of scheduling "tick" */
-	gint ticks;				/* Scheduling ticks for time slice */
-	gint ticks_used;		/* Amount of ticks used by processing step */
-	gint prev_ticks;		/* Ticks used when measuring `elapsed' below */
-	gint elapsed;			/* Elapsed during last run, in usec */
-	gdouble tick_cost;		/* Time in ms. spent by each tick */
-	bgsig_cb_t sigh[BG_SIG_COUNT];	/* Signal handlers */
+	enum bgtask_magic magic;/**< Magic number */
+	guint32 flags;			/**< Operating flags */
+	const char *name;		/**< Task name */
+	int step;				/**< Current processing step */
+	int seqno;				/**< Number of calls at same step */
+	bgstep_cb_t *stepvec;	/**< Set of steps to run in sequence */
+	int stepcnt;			/**< Amount of steps in the `stepvec' array */
+	gpointer ucontext;		/**< User context */
+	time_t created;			/**< Creation time */
+	int wtime;				/**< Wall-clock run time sofar, in ms */
+	bgclean_cb_t uctx_free;	/**< Free routine for context */
+	bgdone_cb_t done_cb;	/**< Called when done */
+	gpointer done_arg;		/**< "done" callback argument */
+	int exitcode;			/**< Final "exit" code */
+	bgsig_t signal;			/**< Last signal delivered */
+	GSList *signals;		/**< List of signals pending delivery */
+	jmp_buf env;			/**< Only valid when TASK_F_RUNNING */
+	tm_t start;				/**< Start time of scheduling "tick" */
+	int ticks;				/**< Scheduling ticks for time slice */
+	int ticks_used;			/**< Amount of ticks used by processing step */
+	int prev_ticks;			/**< Ticks used when measuring `elapsed' below */
+	int elapsed;			/**< Elapsed during last run, in usec */
+	double tick_cost;		/**< Time in ms. spent by each tick */
+	bgsig_cb_t sigh[BG_SIG_COUNT];	/**< Signal handlers */
 
 	/*
 	 * Daemon tasks.
 	 */
 
-	GSList *wq;				/* Work queue (daemon task only) */
-	bgstart_cb_t start_cb;	/* Called when starting working on an item */
-	bgend_cb_t end_cb;		/* Called when finished working on an item */
-	bgclean_cb_t item_free;	/* Free routine for work queue items */
-	bgnotify_cb_t notify;	/* Start/Stop notification (optional) */
+	GSList *wq;				/**< Work queue (daemon task only) */
+	bgstart_cb_t start_cb;	/**< Called when starting working on an item */
+	bgend_cb_t end_cb;		/**< Called when finished working on an item */
+	bgclean_cb_t item_free;	/**< Free routine for work queue items */
+	bgnotify_cb_t notify;	/**< Start/Stop notification (optional) */
 };
 
 /*
  * Operating flags.
  */
 
-#define TASK_F_EXITED		0x00000001	/* Exited */
-#define TASK_F_SIGNAL		0x00000002	/* Signal received */
-#define TASK_F_RUNNING		0x00000004	/* Task is running */
-#define TASK_F_ZOMBIE		0x00000008	/* Task waiting status collect */
-#define TASK_F_NOTICK		0x00000010	/* Do no recompute tick info */
-#define TASK_F_SLEEPING		0x00000020	/* Task is sleeping */
-#define TASK_F_RUNNABLE		0x00000040	/* Task is runnable */
-#define TASK_F_DAEMON		0x80000000	/* Task is a daemon */
+enum {
+	TASK_F_EXITED	=	1 << 0,	/**< Exited */
+	TASK_F_SIGNAL	=	1 << 1,	/**< Signal received */
+	TASK_F_RUNNING	=	1 << 2,	/**< Task is running */
+	TASK_F_ZOMBIE	=	1 << 3,	/**< Task waiting status collect */
+	TASK_F_NOTICK	=	1 << 4,	/**< Do no recompute tick info */
+	TASK_F_SLEEPING	=	1 << 5,	/**< Task is sleeping */
+	TASK_F_RUNNABLE	=	1 << 6,	/**< Task is runnable */
+	TASK_F_DAEMON	=	1 << 7	/**< Task is a daemon */
+};
+
+static inline void
+bg_task_check(const struct bgtask * const bt)
+{
+	g_assert(bt);
+	g_assert(BGTASK_MAGIC == bt->magic);
+}
+
+static unsigned bg_debug;
+
+/**
+ * Set debugging level.
+ */
+void
+bg_set_debug(unsigned level)
+{
+	bg_debug = level;
+}
 
 /*
  * Access routines to internal fields.
  */
 
-#define TASK(x)		((struct bgtask *) (x))
+int
+bg_task_seqno(const struct bgtask *bt)
+{
+	bg_task_check(bt);
+	return bt->seqno;
+}
 
-gint bg_task_seqno(gpointer h)			{ return TASK(h)->seqno; }
-gpointer bg_task_context(gpointer h)	{ return TASK(h)->ucontext; }
+gpointer
+bg_task_context(const struct bgtask *bt)
+{
+	bg_task_check(bt);
+	return bt->ucontext;
+}
 
-static GSList *runq = NULL;
-static GSList *sleepq = NULL;
-static gint runcount = 0;
-static GSList *dead_tasks = NULL;
+static int bg_runcount;
+static GSList *runq;
+static GSList *sleepq;
+static GSList *dead_tasks;
 
-/*
- * bg_sched_add
- *
+/**
  * Add new task to the scheduler (run queue).
  */
-static void bg_sched_add(struct bgtask *bt)
+static void
+bg_sched_add(struct bgtask *bt)
 {
+	bg_task_check(bt);
 	g_assert(!(bt->flags & TASK_F_RUNNABLE));	/* Not already in list */
 
 	/*
@@ -140,13 +191,14 @@ static void bg_sched_add(struct bgtask *bt)
 	runq = g_slist_append(runq, bt);
 }
 
-/*
- * bg_sched_remove
- *
+/**
  * Remove task from the scheduler (run queue).
  */
-static void bg_sched_remove(struct bgtask *bt)
+static void
+bg_sched_remove(struct bgtask *bt)
 {
+	bg_task_check(bt);
+
 	/*
 	 * We currently have only one run queue: we don't handle priorities.
 	 */
@@ -155,30 +207,40 @@ static void bg_sched_remove(struct bgtask *bt)
 	bt->flags &= ~TASK_F_RUNNABLE;
 }
 
-/*
- * bg_sched_pick
- *
+/**
  * Pick next task to schedule.
  */
-static struct bgtask *bg_sched_pick(void)
+static struct bgtask *
+bg_sched_pick(void)
 {
 	/*
 	 * All task in run queue have equal priority, pick the first.
 	 */
-
-	return (runq != NULL) ? (struct bgtask *) runq->data : NULL;
+	if (runq) {
+		struct bgtask *bt = runq->data;
+		bg_task_check(bt);
+		return bt;
+	} else {
+		return NULL;
+	}
 }
 
-/*
- * bg_task_suspend
- *
+/**
  * Suspend task.
+ *
+ * As a side effect, update the tick cost statistics and elapsed time
+ * information for the last scheduling period.
+ *
+ * @param bt		the task to suspend
+ * @param target	the runtime target of the task (0 if unknown)
  */
-static void bg_task_suspend(struct bgtask *bt)
+static void
+bg_task_suspend(struct bgtask *bt, int target)
 {
-	GTimeVal end;
-	gint elapsed;
+	tm_t end;
+	time_delta_t elapsed;
 
+	bg_task_check(bt);
 	g_assert(bt->flags & TASK_F_RUNNING);
 
 	bg_sched_add(bt);
@@ -188,9 +250,8 @@ static void bg_task_suspend(struct bgtask *bt)
 	 * Update task running time.
 	 */
 
-	g_get_current_time(&end);
-	elapsed = (glong) ((end.tv_sec - bt->start.tv_sec) * 1000 * 1000 +
-		(end.tv_usec - bt->start.tv_usec));
+	tm_now_exact(&end);
+	elapsed = tm_elapsed_us(&end, &bt->start); 
 
 	/*
 	 * Compensate any clock adjustment by reusing the previous value we
@@ -210,108 +271,134 @@ static void bg_task_suspend(struct bgtask *bt)
 
 	/*
 	 * Now update the tick cost, if elapsed is not null.
-	 * We use a slow EMA to keep track of it, to smooth variations.
 	 *
 	 * If task is flagged TASK_F_NOTICK, it was scheduled only to deliver
 	 * a signal and we cannot really update the tick cost.
 	 */
 
 	if (!(bt->flags & TASK_F_NOTICK)) {
-		gdouble new_cost =
-			(4 * bt->tick_cost + (elapsed / bt->ticks_used)) / 5.0;
+		double new_cost;
 
-		if (common_dbg > 4)
-			printf("BGTASK \"%s\" total=%d msecs, elapsed=%d, ticks=%d, "
-				"used=%d, tick_cost=%f usecs (was %f)\n",
-				bt->name, bt->wtime, elapsed, bt->ticks, bt->ticks_used,
+		/*
+		 * If the task spent more than its target, then the tick cost
+		 * was severely under-estimated and we compute a new one.
+		 * Otherwise, we use a slow EMA to update the tick cost, in order
+		 * to smooth variations.
+		 */
+
+		if (target != 0 && elapsed > target) {
+			if (bg_debug > 4)
+				g_message("BGTASK \"%s\" resetting tick_cost", bt->name);
+			new_cost = elapsed / bt->ticks_used;
+		} else {
+			new_cost = (4 * bt->tick_cost + (elapsed / bt->ticks_used)) / 5.0;
+		}
+
+		if (bg_debug > 4) {
+			g_debug("BGTASK \"%s\" total=%d msecs, "
+				"elapsed=%lu usecs (targeted %d), "
+				"ticks=%d, used=%d, tick_cost=%f usecs (was %f)",
+				bt->name, bt->wtime, (gulong) elapsed, target,
+				bt->ticks, bt->ticks_used,
 				new_cost, bt->tick_cost);
+		}
 
 		bt->tick_cost = new_cost;
 	}
 }
 
-/*
- * bg_task_resume
- *
- * Suspend task.
+/**
+ * Resume task execution.
  */
-static void bg_task_resume(struct bgtask *bt)
+static void
+bg_task_resume(struct bgtask *bt)
 {
+	bg_task_check(bt);
 	g_assert(!(bt->flags & TASK_F_RUNNING));
 
 	bg_sched_remove(bt);
 	bt->flags |= TASK_F_RUNNING;
 
-	g_get_current_time(&bt->start);
+	tm_now_exact(&bt->start);
 }
 
-/*
- * bg_sched_sleep
- *
+/**
  * Add task to the sleep queue.
  */
-static void bg_sched_sleep(struct bgtask *bt)
+static void
+bg_sched_sleep(struct bgtask *bt)
 {
+	bg_task_check(bt);
 	g_assert(!(bt->flags & TASK_F_SLEEPING));
 	g_assert(!(bt->flags & TASK_F_RUNNING));
-	g_assert(runcount > 0);
+	g_assert(bg_runcount > 0);
 
 	bg_sched_remove(bt);			/* Can no longer be scheduled */
-	runcount--;
+	bg_runcount--;
 	bt->flags |= TASK_F_SLEEPING;
 	sleepq = g_slist_prepend(sleepq, bt);
 }
 
-/*
- * bg_sched_wakeup
- *
+/**
  * Remove task from the sleep queue and insert it to the runqueue.
  */
-static void bg_sched_wakeup(struct bgtask *bt)
+static void
+bg_sched_wakeup(struct bgtask *bt)
 {
+	bg_task_check(bt);
 	g_assert(bt->flags & TASK_F_SLEEPING);
 	g_assert(!(bt->flags & TASK_F_RUNNING));
 
 	sleepq = g_slist_remove(sleepq, bt);
 	bt->flags &= ~TASK_F_SLEEPING;
-	runcount++;
+	bg_runcount++;
 	bg_sched_add(bt);
 }
 
 
-static struct bgtask *current_task = NULL;
+static struct bgtask *current_task;
 
-/*
- * bg_task_switch
- *
+/**
  * Switch to new task `bt'.
  * If argument is NULL, suspends current task.
  *
- * Returns previously scheduled task, if any.
+ * @param bt		the new task being scheduled
+ * @param target	the running time target of current task (0 if unknown)
+ *
+ * @returns previously scheduled task, if any.
  */
-static struct bgtask *bg_task_switch(struct bgtask *bt)
+static struct bgtask *
+bg_task_switch(struct bgtask *bt, int target)
 {
 	struct bgtask *old = current_task;
 
 	g_assert(bt == NULL || !(bt->flags & TASK_F_RUNNING));
 
 	if (old) {
-		bg_task_suspend(old);
+		bg_task_suspend(old, target);
 		current_task = NULL;
 	}
-
-	if (bt == NULL)
-		return old;
-
-	bg_task_resume(bt);
-	current_task = bt;
-
+	if (bt) {
+		bg_task_check(bt);
+		bg_task_resume(bt);
+		current_task = bt;
+	}
 	return old;
 }
 
-/*
- * bg_task_create
- *
+static struct bgtask *
+bg_task_alloc(void)
+{
+	static const struct bgtask zero_bt;
+	struct bgtask *bt;
+
+	WALLOC(bt);
+	*bt = zero_bt;
+	bt->magic = BGTASK_MAGIC;
+	return bt;
+}
+
+/**
  * Create a new background task.
  * The `steps' array is cloned, so it can be built on the caller's stack.
  *
@@ -324,25 +411,24 @@ static struct bgtask *bg_task_switch(struct bgtask *bt)
  * The user-supplied argument `done_arg' will also be given to that callback.
  * Note that "done" does not necessarily mean success.
  *
- * Returns an opaque handle.
+ * @returns an opaque handle.
  */
-gpointer bg_task_create(
-	gchar *name,						/* Task name (for tracing) */
-	bgstep_cb_t *steps, gint stepcnt,	/* Work to perform (copied) */
-	gpointer ucontext,					/* User context */
-	bgclean_cb_t ucontext_free,			/* Free routine for context */
-	bgdone_cb_t done_cb,				/* Notification callback when done */
-	gpointer done_arg)					/* Callback argument */
+struct bgtask *
+bg_task_create(const char *name,	/**< Task name (for tracing) */
+	const bgstep_cb_t *steps,		/**< Work to perform (copied) */
+	int stepcnt,					/**< Number of steps */
+	gpointer ucontext,				/**< User context */
+	bgclean_cb_t ucontext_free,		/**< Free routine for context */
+	bgdone_cb_t done_cb,			/**< Notification callback when done */
+	gpointer done_arg)				/**< Callback argument */
 {
 	struct bgtask *bt;
-	gint stepsize;
+	int stepsize;
 
 	g_assert(stepcnt > 0);
 	g_assert(steps);
 
-	bt = walloc0(sizeof(*bt));
-
-	bt->magic = BT_MAGIC;
+	bt = bg_task_alloc();
 	bt->name = name;
 	bt->ucontext = ucontext;
 	bt->uctx_free = ucontext_free;
@@ -355,14 +441,17 @@ gpointer bg_task_create(
 	memcpy(bt->stepvec, steps, stepsize);
 
 	bg_sched_add(bt);					/* Let scheduler know about it */
-	runcount++;							/* One more task to schedule */
+	bg_runcount++;						/* One more task to schedule */
+
+	if (bg_debug > 1) {
+		g_debug("BGTASK created task \"%s\" (%d step%s)",
+			name, stepcnt, 1 == stepcnt ? "" : "s");
+	}
 
 	return bt;
 }
 
-/*
- * bg_daemon_create
- *
+/**
  * A "daemon" is a task equipped with a work queue.
  *
  * When the daemon is initially created, it has an empty work queue and it is
@@ -373,7 +462,7 @@ gpointer bg_task_create(
  *
  * The `steps' given represent the processing to be done on each item of
  * the work queue.  The `start_cb' callback is invoked before working on a
- * new item, so that the context can be initialized.  The `end+cb' callback
+ * new item, so that the context can be initialized.  The `end_cb' callback
  * is invoked when the item has been processed (successfully or not).
  *
  * Since a daemon is not supposed to exit (although it can), there is no
@@ -381,25 +470,25 @@ gpointer bg_task_create(
  *
  * Use bg_daemon_enqueue() to enqueue more work to the daemon.
  */
-gpointer bg_daemon_create(
-	gchar *name,						/* Task name (for tracing) */
-	bgstep_cb_t *steps, gint stepcnt,	/* Work to perform (copied) */
-	gpointer ucontext,					/* User context */
-	bgclean_cb_t ucontext_free,			/* Free routine for context */
-	bgstart_cb_t start_cb,				/* Starting working on an item */
-	bgend_cb_t end_cb,					/* Done working on an item */
-	bgclean_cb_t item_free,				/* Free routine for work queue items */
-	bgnotify_cb_t notify)				/* Start/Stop notify (optional) */
+struct bgtask *
+bg_daemon_create(
+	const char *name,			/**< Task name (for tracing) */
+	const bgstep_cb_t *steps,	/**< Work to perform (copied) */
+	int stepcnt,				/**< Number of steps */
+	gpointer ucontext,			/**< User context */
+	bgclean_cb_t ucontext_free,	/**< Free routine for context */
+	bgstart_cb_t start_cb,		/**< Starting working on an item */
+	bgend_cb_t end_cb,			/**< Done working on an item */
+	bgclean_cb_t item_free,		/**< Free routine for work queue items */
+	bgnotify_cb_t notify)		/**< Start/Stop notify (optional) */
 {
 	struct bgtask *bt;
-	gint stepsize;
+	int stepsize;
 
 	g_assert(stepcnt > 0);
 	g_assert(steps);
 
-	bt = walloc0(sizeof(*bt));
-
-	bt->magic = BT_MAGIC;
+	bt = bg_task_alloc();
 	bt->flags |= TASK_F_DAEMON;
 	bt->name = name;
 	bt->ucontext = ucontext;
@@ -414,31 +503,32 @@ gpointer bg_daemon_create(
 	bt->stepvec = walloc(stepsize);
 	memcpy(bt->stepvec, steps, stepsize);
 
-	runcount++;							/* One more task to schedule */
+	bg_runcount++;						/* One more task to schedule */
 	bg_sched_sleep(bt);					/* Record sleeping task */
 
+	if (bg_debug > 1) {
+		g_debug("BGTASK created daemon task \"%s\" (%d step%s)",
+			name, stepcnt, 1 == stepcnt ? "" : "s");
+	}
+
 	return bt;
 }
 
-/*
- * bg_daemon_enqueue
- *
+/**
  * Enqueue work item to the daemon task.
  * If task was sleeping, wake it up.
  */
-void bg_daemon_enqueue(gpointer h, gpointer item)
+void
+bg_daemon_enqueue(struct bgtask *bt, gpointer item)
 {
-	struct bgtask *bt = TASK(h);
-
-	g_assert(h);
-	g_assert(bt->magic == BT_MAGIC);
+	bg_task_check(bt);
 	g_assert(bt->flags & TASK_F_DAEMON);
 
 	bt->wq = g_slist_append(bt->wq, item);
 
 	if (bt->flags & TASK_F_SLEEPING) {
-		if (common_dbg > 1)
-			printf("BGTASK waking up daemon \"%s\" task\n", bt->name);
+		if (bg_debug > 1)
+			g_debug("BGTASK waking up daemon \"%s\" task", bt->name);
 
 		bg_sched_wakeup(bt);
 		if (bt->notify)
@@ -446,17 +536,19 @@ void bg_daemon_enqueue(gpointer h, gpointer item)
 	}
 }
 
-/*
- * bg_task_free
- *
+/**
  * Free task structure.
  */
-static void bg_task_free(struct bgtask *bt)
+static void
+bg_task_free(struct bgtask *bt)
 {
 	GSList *l;
-	gint stepsize;
-	gint count;
+	int stepsize;
+	int count;
 
+	g_assert(bt);
+	g_assert(BGTASK_DEAD_MAGIC == bt->magic);
+	
 	g_assert(!(bt->flags & TASK_F_RUNNING));
 	g_assert(bt->flags & TASK_F_EXITED);
 
@@ -468,24 +560,25 @@ static void bg_task_free(struct bgtask *bt)
 		if (bt->item_free)
 			(*bt->item_free)(l->data);
 	}
-	g_slist_free(bt->wq);
+	gm_slist_free_null(&bt->wq);
 
 	if (count)
-		g_warning("freed %d pending item%s for daemon \"%s\" task",
+		g_carp("freed %d pending item%s for daemon \"%s\" task",
 			count, count == 1 ? "" : "s", bt->name);
 
-	wfree(bt, sizeof(*bt));
+	bt->magic = 0;
+	WFREE(bt);
 }
 
-/*
- * bg_task_terminate
- *
+/**
  * Terminate the task, invoking the completion callback if defined.
  */
-static void bg_task_terminate(struct bgtask *bt)
+static void
+bg_task_terminate(struct bgtask *bt)
 {
 	bgstatus_t status;
 
+	bg_task_check(bt);
 	g_assert(!(bt->flags & TASK_F_EXITED));
 
 	/*
@@ -500,9 +593,10 @@ static void bg_task_terminate(struct bgtask *bt)
 	 * When we come here, the task is no longer running.
 	 */
 
-	if (common_dbg > 1)
-		printf("BGTASK terminating \"%s\"%s, ran %d msecs\n",
+	if (bg_debug > 1) {
+		g_debug("BGTASK terminating \"%s\"%s, ran %d msecs",
 			bt->name, (bt->flags & TASK_F_DAEMON) ? " daemon" : "", bt->wtime);
+	}
 
 	g_assert(!(bt->flags & TASK_F_RUNNING));
 
@@ -511,9 +605,9 @@ static void bg_task_terminate(struct bgtask *bt)
 
 	bt->flags |= TASK_F_EXITED;		/* Task has now exited */
 	bg_sched_remove(bt);			/* Ensure it's no longer scheduled */
-	runcount--;						/* One task less to run */
+	bg_runcount--;					/* One task less to run */
 
-	g_assert(runcount >= 0);
+	g_assert(bg_runcount >= 0);
 
 	/*
 	 * Compute proper status.
@@ -545,8 +639,7 @@ static void bg_task_terminate(struct bgtask *bt)
 		(*bt->done_cb)(bt, bt->ucontext, status, bt->done_arg);
 
 		if (bt->flags & TASK_F_ZOMBIE)
-			g_warning("user code lost exit status of task \"%s\"",
-				bt->name);
+			g_carp("user code lost exit status of task \"%s\"", bt->name);
 
 		bt->flags &= ~TASK_F_ZOMBIE;		/* Is now totally DEAD */
 	}
@@ -556,7 +649,7 @@ static void bg_task_terminate(struct bgtask *bt)
 	 */
 
 	(*bt->uctx_free)(bt->ucontext);
-	bt->magic = 0;							/* Prevent further uses! */
+	bt->magic = BGTASK_DEAD_MAGIC;	/* Prevent further uses! */
 
 	/*
 	 * Do not free the task structure immediately, in case the calling
@@ -569,18 +662,14 @@ static void bg_task_terminate(struct bgtask *bt)
 	dead_tasks = g_slist_prepend(dead_tasks, bt);
 }
 
-/*
- * bg_task_exit
- *
+/**
  * Called by user code to "exit" the task.
  * We exit immediately, not returning to the user code.
  */
-void bg_task_exit(gpointer h, gint code)
+void
+bg_task_exit(struct bgtask *bt, int code)
 {
-	struct bgtask *bt = TASK(h);
-
-	g_assert(bt);
-	g_assert(bt->magic == BT_MAGIC);
+	bg_task_check(bt);
 	g_assert(bt->flags & TASK_F_RUNNING);
 
 	bt->exitcode = code;
@@ -593,13 +682,13 @@ void bg_task_exit(gpointer h, gint code)
 	longjmp(bt->env, 1);		/* Will call bg_task_terminate() */
 }
 
-/*
- * bg_task_sendsig
- *
+/**
  * Deliver signal via the user's signal handler.
  */
-static void bg_task_sendsig(struct bgtask *bt, bgsig_t sig, bgsig_cb_t handler)
+static void
+bg_task_sendsig(struct bgtask *bt, bgsig_t sig, bgsig_cb_t handler)
 {
+	bg_task_check(bt);
 	g_assert(bt->flags & TASK_F_RUNNING);
 
 	bt->flags |= TASK_F_SIGNAL;
@@ -611,20 +700,17 @@ static void bg_task_sendsig(struct bgtask *bt, bgsig_t sig, bgsig_cb_t handler)
 	bt->signal = BG_SIG_ZERO;
 }
 
-/*
- * bg_task_kill
- *
+/**
  * Send a signal to the given task.
- * Returns -1 if the task could not be signalled.
+ *
+ * @returns -1 if the task could not be signalled.
  */
-static gint bg_task_kill(gpointer h, bgsig_t sig)
+static int
+bg_task_kill(struct bgtask *bt, bgsig_t sig)
 {
-	struct bgtask *bt = TASK(h);
 	bgsig_cb_t sighandler;
 
-	g_assert(bt);
-	g_assert(bt->magic == BT_MAGIC);
-
+	bg_task_check(bt);
 	if (bt->flags & TASK_F_EXITED)		/* Already exited */
 		return -1;
 
@@ -659,7 +745,7 @@ static gint bg_task_kill(gpointer h, bgsig_t sig)
 	 */
 
 	if (!(bt->flags & TASK_F_RUNNING) || (bt->flags & TASK_F_SIGNAL)) {
-		bt->signals = g_slist_append(bt->signals, (gpointer) sig);
+		bt->signals = g_slist_append(bt->signals, GUINT_TO_POINTER(sig));
 		return 1;
 	}
 
@@ -673,33 +759,30 @@ static gint bg_task_kill(gpointer h, bgsig_t sig)
 	return 1;
 }
 
-/*
- * bg_task_signal
- *
+/**
  * Install user-level signal handler for a task signal.
- * Returns previously installed signal handler.
+ *
+ * @returns previously installed signal handler.
  */
-bgsig_cb_t bg_task_signal(gpointer h, bgsig_t sig, bgsig_cb_t handler)
+bgsig_cb_t
+bg_task_signal(struct bgtask *bt, bgsig_t sig, bgsig_cb_t handler)
 {
-	struct bgtask *bt = TASK(h);
 	bgsig_cb_t oldhandler;
 
-	g_assert(bt);
-	g_assert(bt->magic == BT_MAGIC);
-
+	bg_task_check(bt);
 	oldhandler = bt->sigh[sig];
 	bt->sigh[sig] = handler;
 
 	return oldhandler;
 }
 
-/*
- * bg_task_deliver_signals
- *
+/**
  * Deliver all the signals queued so far for the task.
  */
-static void bg_task_deliver_signals(struct bgtask *bt)
+static void
+bg_task_deliver_signals(struct bgtask *bt)
 {
+	bg_task_check(bt);
 	g_assert(bt->flags & TASK_F_RUNNING);
 
 	/*
@@ -711,7 +794,7 @@ static void bg_task_deliver_signals(struct bgtask *bt)
 
 	while (bt->signals != NULL) {
 		GSList *lnk = bt->signals;
-		bgsig_t sig = (bgsig_t) lnk->data;
+		bgsig_t sig = (bgsig_t) GPOINTER_TO_UINT(lnk->data);
 
 		/*
 		 * If signal kills the thread (it calls bg_task_exit() from the
@@ -725,19 +808,15 @@ static void bg_task_deliver_signals(struct bgtask *bt)
 	}
 }
 
-/*
- * bg_task_cancel
- *
+/**
  * Cancel a given task.
  */
-void bg_task_cancel(gpointer h)
+void
+bg_task_cancel(struct bgtask *bt)
 {
-	struct bgtask *bt = TASK(h);
 	struct bgtask *old = NULL;
 
-	g_assert(bt);
-	g_assert(bt->magic == BT_MAGIC);
-
+	bg_task_check(bt);
 	if (bt->flags & TASK_F_EXITED)		/* Already exited */
 		return;
 
@@ -754,12 +833,12 @@ void bg_task_cancel(gpointer h)
 		 */
 
 		if (!(bt->flags & TASK_F_RUNNING)) {
-			old = bg_task_switch(bt);		/* Switch to `bt' */
+			old = bg_task_switch(bt, 0);	/* Switch to `bt' */
 			switched = TRUE;
 		}
 
 		g_assert(bt->flags & TASK_F_RUNNING);
-		bg_task_kill(h, BG_SIG_TERM);		/* Let task cleanup nicely */
+		bg_task_kill(bt, BG_SIG_TERM);		/* Let task cleanup nicely */
 
 		/*
 		 * We only come back if the signal did not kill the task, i.e.
@@ -768,62 +847,56 @@ void bg_task_cancel(gpointer h)
 
 		if (switched) {
 			bt->flags |= TASK_F_NOTICK;		/* Disable tick recomputation */
-			(void) bg_task_switch(old);		/* Restore old thread */
+			(void) bg_task_switch(old, 0);	/* Restore old thread */
 		}
 	}
 
-	bg_task_kill(h, BG_SIG_KILL);			/* Kill task immediately */
+	bg_task_kill(bt, BG_SIG_KILL);			/* Kill task immediately */
 
 	g_assert(bt->flags & TASK_F_EXITED);	/* Task is now terminated */
 }
 
-/*
- * bg_task_ticks_used
- *
+/**
  * This routine can be called by the task when a single step is not using
  * all its ticks and it matters for the computation of the cost per tick.
  */
-void bg_task_ticks_used(gpointer h, gint used)
+void
+bg_task_ticks_used(struct bgtask *bt, int used)
 {
-	struct bgtask *bt = TASK(h);
-
-	g_assert(bt);
-	g_assert(bt->magic == BT_MAGIC);
+	bg_task_check(bt);
 	g_assert(bt->flags & TASK_F_RUNNING);
 	g_assert(used >= 0);
-	g_assert(used <= bt->ticks);
 
-	bt->ticks_used = used;
+	bt->ticks_used = MIN(used, bt->ticks);
 
 	if (used == 0)
 		bt->flags |= TASK_F_NOTICK;			/* Won't update tick info */
 }
 
-/*
- * bg_reclaim_dead
- *
+/**
  * Reclaim all dead tasks
  */
-static void bg_reclaim_dead(void)
+static void
+bg_reclaim_dead(void)
 {
-	GSList *l;
+	GSList *sl;
 
-	for (l = dead_tasks; l; l = l->next)
-		bg_task_free((struct bgtask *) l->data);
-
-	g_slist_free(dead_tasks);
-	dead_tasks = NULL;
+	for (sl = dead_tasks; sl; sl = g_slist_next(sl)) {
+		bg_task_free(sl->data);
+	}
+	gm_slist_free_null(&dead_tasks);
 }
 
-/*
- * bg_task_ended
- *
+/**
  * Called when a task has ended its processing.
  */
-static void bg_task_ended(struct bgtask *bt)
+static void
+bg_task_ended(struct bgtask *bt)
 {
 	gpointer item;
 
+	bg_task_check(bt);
+
 	/*
 	 * Non-daemon task: reroute to bg_task_terminate().
 	 */
@@ -841,9 +914,10 @@ static void bg_task_ended(struct bgtask *bt)
 
 	item = bt->wq->data;
 
-	if (common_dbg > 2)
-		printf("BGTASK daemon \"%s\" done with item 0x%lx\n",
+	if (bg_debug > 2) {
+		g_debug("BGTASK daemon \"%s\" done with item 0x%lx",
 			bt->name, (gulong) item);
+	}
 
 	(*bt->end_cb)(bt, bt->ucontext, item);
 	bt->wq = g_slist_remove(bt->wq, item);
@@ -854,7 +928,7 @@ static void bg_task_ended(struct bgtask *bt)
 	 * The following makes sure we pickup a new item at the next iteration.
 	 */
 
-	bt->tick_cost = 0;					/* Will restart at 1 tick next time */
+	bt->tick_cost = 0.0;			/* Will restart at 1 tick next time */
 	bt->seqno = 0;
 	bt->step = 0;
 
@@ -863,8 +937,8 @@ static void bg_task_ended(struct bgtask *bt)
 	 */
 
 	if (bt->wq == NULL) {
-		if (common_dbg > 1)
-			printf("BGTASK daemon \"%s\" going back to sleep\n", bt->name);
+		if (bg_debug > 1)
+			g_debug("BGTASK daemon \"%s\" going back to sleep", bt->name);
 
 		bg_sched_sleep(bt);
 		if (bt->notify)
@@ -872,36 +946,67 @@ static void bg_task_ended(struct bgtask *bt)
 	}
 }
 
-/*
- * bg_sched_timer
- *
- * Main task scheduling timer, called once per second.
+/**
+ * Adjust the period of the tick delivery event.
  */
-void bg_sched_timer(void)
+static void
+bg_ticker_adjust_period(void)
+{
+	int target;
+
+	/*
+	 * Schedule once every BG_TICK_IDLE ms if we have nothing runable.
+	 * Otherwise, increase the frequency to once every BG_TICK_BUSY ms.
+	 */
+
+	target = 0 == bg_runcount ? BG_TICK_IDLE : BG_TICK_BUSY;
+
+	if (bg_ticker.period != target) {
+		bg_ticker.period = target;
+		cq_periodic_resched(bg_ticker.pev, target);
+
+		if (bg_debug > 5) {
+			g_debug("BGTASK will be ticking every %d msecs (runable = %d)",
+				bg_ticker.period, bg_runcount);
+		}
+	}
+}
+
+/**
+ * Main task scheduling timer.
+ */
+static gboolean
+bg_sched_timer(void *unused_arg)
 {
 	struct bgtask * volatile bt;
-	volatile gint remain = MAX_LIFE;
-	gint target;
-	volatile gint ticks;
+	volatile int remain = MAX_LIFE;
+	volatile int target;
+	volatile unsigned schedules = 0;
+	volatile int ticks;
 	bgret_t ret;
 
 	g_assert(current_task == NULL);
-	g_assert(runcount >= 0);
+	g_assert(bg_runcount >= 0);
+
+	(void) unused_arg;
+
+	bg_ticker_adjust_period();
 
 	/*
 	 * Loop as long as there are tasks to be scheduled and we have some
 	 * time left to spend.
 	 */
 
-	while (runcount > 0 && remain > 0) {
+	while (bg_runcount > 0 && remain > 0) {
 		/*
 		 * Compute how much time we can spend for this task.
 		 */
 
-		target = MAX(MIN_LIFE, MAX_LIFE / runcount);
+		target = MAX_LIFE / bg_runcount;
+		target = MIN(target, remain);
 
 		bt = bg_sched_pick();
-		g_assert(bt);					/* runcount > 0 => there is a task */
+		g_assert(bt != NULL);		/* bg_runcount > 0 => there is a task */
 		g_assert(bt->flags & TASK_F_RUNNABLE);
 
 		bt->flags &= ~TASK_F_NOTICK;	/* We'll want tick cost update */
@@ -913,25 +1018,38 @@ void bg_sched_timer(void)
 		 * than DELTA_FACTOR.
 		 */
 
-		if (bt->tick_cost) {
-			ticks = 1 + target / bt->tick_cost;
+		if (bt->tick_cost > 0.0) {
+			g_assert(bt->prev_ticks >= 0);
+			g_assert(bt->prev_ticks <= INT_MAX / DELTA_FACTOR);
+
+			if (target < bt->tick_cost * (INT_MAX / DELTA_FACTOR - 1))
+				ticks = 1 + target / bt->tick_cost;
+			else
+				ticks = INT_MAX / DELTA_FACTOR;
+
 			if (bt->prev_ticks) {
-				if (ticks > bt->prev_ticks * DELTA_FACTOR)
+				if (ticks > bt->prev_ticks * DELTA_FACTOR) {
 					ticks = bt->prev_ticks * DELTA_FACTOR;
-				else if (DELTA_FACTOR * ticks < bt->prev_ticks)
-					ticks = bt->prev_ticks / DELTA_FACTOR;
+				} else if (ticks < bt->prev_ticks / DELTA_FACTOR) {
+					if (bt->prev_ticks > DELTA_FACTOR)
+						ticks = bt->prev_ticks / DELTA_FACTOR;
+					else
+						ticks = 1;
+				}
 			}
 			g_assert(ticks > 0);
-		} else
+		} else {
 			ticks = 1;
-
-		bt->ticks = bt->ticks_used = ticks;
+		}
+		bt->ticks = ticks;
+		bt->ticks_used = ticks;
 
 		/*
 		 * Switch to the selected task.
 		 */
 
-		bg_task_switch(bt);
+		bg_task_switch(bt, 0);
+		schedules++;
 
 		g_assert(current_task == bt);
 		g_assert(bt->flags & TASK_F_RUNNING);
@@ -946,11 +1064,16 @@ void bg_sched_timer(void)
 			 * So they exited, or someone is killing the task.
 			 */
 
-			if (common_dbg > 1)
-				printf("BGTASK back from setjmp() for \"%s\"\n", bt->name);
+			if (bg_debug > 1)
+				g_debug("BGTASK back from setjmp() for \"%s\"", bt->name);
 
 			bt->flags |= TASK_F_NOTICK;
-			bg_task_switch(NULL);
+			bg_task_switch(NULL, target);
+			if (bg_debug > 0 && remain < bt->elapsed) {
+				g_debug("%s: remain=%d, bt->elapsed=%d",
+					G_STRFUNC, remain, bt->elapsed);
+			}
+			remain -= MIN(remain, bt->elapsed);
 			bg_task_terminate(bt);
 			continue;
 		}
@@ -959,9 +1082,17 @@ void bg_sched_timer(void)
 		 * Run the next step.
 		 */
 
-		if (common_dbg > 4)
-			printf("BGTASK \"%s\" running step #%d.%d with %d tick%s\n",
+		if (bg_debug > 2 && 0 == bt->seqno) {
+			g_debug("BGTASK \"%s\" starting step #%d (%s)",
+				bt->name, bt->step,
+				stacktrace_routine_name(
+					func_to_pointer(bt->stepvec[bt->step]), FALSE));
+		}
+
+		if (bg_debug > 4) {
+			g_debug("BGTASK \"%s\" running step #%d.%d with %d tick%s",
 				bt->name, bt->step, bt->seqno, ticks, ticks == 1 ? "" : "s");
+		}
 
 		bg_task_deliver_signals(bt);	/* Send any queued signal */
 
@@ -977,8 +1108,8 @@ void bg_sched_timer(void)
 
 			item = bt->wq->data;
 
-			if (common_dbg > 2)
-				printf("BGTASK daemon \"%s\" starting with item 0x%lx\n",
+			if (bg_debug > 2)
+				g_debug("BGTASK daemon \"%s\" starting with item 0x%lx",
 					bt->name, (gulong) item);
 
 			(*bt->start_cb)(bt, bt->ucontext, item);
@@ -988,12 +1119,16 @@ void bg_sched_timer(void)
 
 		ret = (*bt->stepvec[bt->step])(bt, bt->ucontext, ticks);
 
-		bg_task_switch(NULL);		/* Stop current task, update stats */
-		remain -= bt->elapsed;
+		bg_task_switch(NULL, target);	/* Stop current task, update stats */
+		if (bg_debug > 0 && remain < bt->elapsed) {
+			g_debug("%s: remain=%d, bt->elapsed=%d",
+				G_STRFUNC, remain, bt->elapsed);
+		}
+		remain -= MIN(remain, bt->elapsed);
 
-		if (common_dbg > 4)
-			printf("BGTASK \"%s\" step #%d.%d ran %d tick%s "
-				"in %d usecs [ret=%d]\n",
+		if (bg_debug > 4)
+			g_debug("BGTASK \"%s\" step #%d.%d ran %d tick%s "
+				"in %d usecs [ret=%d]",
 				bt->name, bt->step, bt->seqno,
 				bt->ticks_used, bt->ticks_used == 1 ? "" : "s",
 				bt->elapsed, ret);
@@ -1012,7 +1147,7 @@ void bg_sched_timer(void)
 			else {
 				bt->seqno = 0;
 				bt->step++;
-				bt->tick_cost = 0;	/* Don't know cost of this new step */
+				bt->tick_cost = 0.0;	/* Don't know cost of this new step */
 			}
 			break;
 		case BGR_MORE:
@@ -1027,40 +1162,75 @@ void bg_sched_timer(void)
 
 	if (dead_tasks != NULL)
 		bg_reclaim_dead();			/* Free dead tasks */
+
+	if (bg_debug > 3 && MAX_LIFE != remain) {
+		g_debug("BGTASK runable=%d, ran for %lu usecs, scheduling %u task%s",
+			bg_runcount, MAX_LIFE - remain,
+			schedules, 1 == schedules ? "" : "s");
+	}
+
+	return TRUE;		/* Keep calling */
 }
 
-/*
- * bg_close
- *
+static guint
+bg_task_terminate_all(GSList **ptr)
+{
+	guint count;
+
+	count = 0;
+	if (*ptr) {
+		GSList *iter, *copy;
+
+		copy = g_slist_copy(*ptr);
+		for (iter = copy; NULL != iter; iter = g_slist_next(iter)) {
+			count++;
+			bg_task_terminate(iter->data);
+		}
+		gm_slist_free_null(&copy);
+		gm_slist_free_null(ptr);
+	}
+	return count;
+}
+
+/**
+ * Initialize background task scheduling.
+ */
+void
+bg_init(void)
+{
+	/*
+	 * Initially, the periodic event providing "scheduling ticks" triggers
+	 * every second.  This time is adjusted when there is work to do so
+	 * that background tasks can nicely blend in the middle of other activities.
+	 */
+
+	bg_ticker.period = BG_TICK_IDLE;
+	bg_ticker.pev = cq_periodic_main_add(BG_TICK_IDLE, bg_sched_timer, NULL);
+}
+
+/**
  * Called at shutdown time.
  */
-void bg_close(void)
+void
+bg_close(void)
 {
-	GSList *l;
-	GSList *c;
-	gint count;
+	guint count;
 
-	for (count = 0, c = g_slist_copy(runq), l = c; l; l = l->next) {
-		count++;
-		bg_task_terminate(l->data);
+	count = bg_task_terminate_all(&runq);
+	if (count > 0) {
+		g_warning("terminated %u running task%s",
+			count, count == 1 ? "" : "s");
 	}
-	g_slist_free(runq);
-	g_slist_free(c);
 
-	if (count)
-		g_warning("terminated %d running task%s", count, count == 1 ? "" : "s");
-
-	for (count = 0, c = g_slist_copy(sleepq), l = c; l; l = l->next) {
-		count++;
-		bg_task_terminate(l->data);
+	count = bg_task_terminate_all(&sleepq);
+	if (count > 0) {
+		g_warning("terminated %d daemon task%s",
+			count, count == 1 ? "" : "s");
 	}
-	g_slist_free(sleepq);
-	g_slist_free(c);
-
-	if (count)
-		g_warning("terminated %d daemon task%s", count, count == 1 ? "" : "s");
 
 	bg_reclaim_dead();				/* Free dead tasks */
+	bg_runcount = 0;
+	cq_periodic_remove(&bg_ticker.pev);
 }
 
 /* bg_task_goto */
@@ -1068,4 +1238,4 @@ void bg_close(void)
 /* bg_task_get_exitcode */
 /* bg_task_get_signal */
 
-/* vi: set ts=4: */
+/* vi: set ts=4 sw=4 cindent: */
