@@ -1,5 +1,5 @@
 /*
- * $Id: crc.c 11437 2006-08-05 00:24:20Z cbiere $
+ * $Id: crc.c 19315 2011-06-28 22:03:42Z rmanfredi $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -35,7 +35,7 @@
 
 #include "common.h"
 
-RCSID("$Id: crc.c 11437 2006-08-05 00:24:20Z cbiere $")
+RCSID("$Id: crc.c 19315 2011-06-28 22:03:42Z rmanfredi $")
 
 #include "crc.h"
 #include "override.h"		/* Must be the last header included */
@@ -74,7 +74,7 @@ crc32_gen_crc_table(void)
 	guint32 i, crc_accum;
 
 	for (i = 0; i < 256; i++) {
-		gint j;
+		int j;
 
 		crc_accum = i << 24;
 		for (j = 0; j < 8; j++) {
@@ -96,7 +96,7 @@ crc32_gen_crc_table(void)
  *
  */
 guint32
-crc32_update_crc(guint32 crc_accum, gconstpointer data, size_t len)
+crc32_update(guint32 crc_accum, gconstpointer data, size_t len)
 {
 	const guchar *p = data;
 	size_t j;
@@ -104,7 +104,7 @@ crc32_update_crc(guint32 crc_accum, gconstpointer data, size_t len)
 	for (j = 0; j < len; j++) {
 		guint8 i;
 
-		i = (crc_accum >> 24) ^ p[j];
+		i = (crc_accum >> 24) ^ *p++;
 		crc_accum = (crc_accum << 8) ^ crc_table[i];
 	}
 
