@@ -1,5 +1,5 @@
 /*
- * $Id: crc.h,v 1.2 2005/06/25 01:37:42 daichik Exp $
+ * $Id: crc.h 19315 2011-06-28 22:03:42Z rmanfredi $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -36,12 +36,12 @@
 #ifndef _crc_h_
 #define _crc_h_
 
-#include <glib.h>
+#include "common.h"
 
 void crc_init(void);
-guint32 crc32_update_crc(guint32 crc_accum, const gchar *data, gint len);
+guint32 crc32_update(guint32 crc_accum, gconstpointer data, size_t len);
 
 #endif	/* _crc_h_ */
 
-/* vi: set ts=4: */
+/* vi: set ts=4 sw=4 cindent: */
 
