@@ -1,5 +1,5 @@
 /*
- * $Id: fifo.h 8376 2005-06-25 01:37:45Z daichik $
+ * $Id: fifo.h 17892 2010-12-09 21:03:24Z rmanfredi $
  *
  * Copyright (c) 2004, Raphael Manfredi
  *
@@ -36,18 +36,17 @@
 #ifndef _fifo_h_
 #define _fifo_h_
 
-#include <glib.h>
+#include "common.h" 
 
-typedef struct fifo {
-	gpointer data;
-} fifo_t;
+struct fifo;
+typedef struct fifo fifo_t;
 
 typedef void (*fifo_free_t)(gpointer item, gpointer udata);
 
 fifo_t *fifo_make(void);
 void fifo_free(fifo_t *f);
 void fifo_free_all(fifo_t *f, fifo_free_t cb, gpointer udata);
-gint fifo_count(fifo_t *f);
+int fifo_count(fifo_t *f);
 void fifo_put(fifo_t *f, gconstpointer data);
 gpointer fifo_remove(fifo_t *f);
 
