@@ -1,5 +1,5 @@
 /*
- * $Id: file.c 12272 2006-11-09 06:14:06Z cbiere $
+ * $Id: file.c 18871 2011-03-15 11:33:05Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,19 +35,24 @@
 
 #include "common.h"
 
-RCSID("$Id: file.c 12272 2006-11-09 06:14:06Z cbiere $")
+RCSID("$Id: file.c 18871 2011-03-15 11:33:05Z rmanfredi $")
 
+#include "concat.h"			/* For concat_strings() */
 #include "file.h"
-#include "misc.h"
+#include "debug.h"
+#include "fd.h"
+#include "halloc.h"
+#include "log.h"			/* For s_warning() */
+#include "path.h"
+#include "timestamp.h"
 #include "tm.h"
+#include "glib-missing.h"
 #include "override.h"		/* Must be the last header included */
 
-static guint32 common_dbg = 0;	/**< XXX -- need to init lib's props --RAM */
-
-static const gchar orig_ext[] = "orig";
-static const gchar new_ext[] = "new";
-static const gchar instead_str[] = " instead";
-static const gchar empty_str[] = "";
+static const char orig_ext[] = "orig";
+static const char new_ext[] = "new";
+static const char instead_str[] = " instead";
+static const char empty_str[] = "";
 
 /**
  * In order to avoid having a dependency between file.c and ban.c,
@@ -57,6 +62,19 @@ static const gchar empty_str[] = "";
  */
 static reclaim_fd_t reclaim_fd = NULL;
 
+/**
+ * Check for file existence.
+ */
+gboolean
+file_exists(const char *pathname)
+{
+  	filestat_t st;
+
+    g_assert(pathname != NULL);
+
+    return 0 == stat(pathname, &st) && S_ISREG(st.st_mode);
+}
+
 /**
  * Register fd reclaiming callback.
  * Use NULL to unregister it.
@@ -67,6 +85,67 @@ file_register_fd_reclaimer(reclaim_fd_t callback)
 	reclaim_fd = callback;
 }
 
+/**
+ * Search executable within the user's PATH.
+ *
+ * @return full path if found, NULL otherwise.
+ * The returned string is allocated with halloc().
+ */
+char *
+file_locate_from_path(const char *argv0)
+{
+	static gboolean already_done;
+	char *path;
+	char *tok;
+	char filepath[MAX_PATH_LEN + 1];
+	char *result = NULL;
+
+	if (filepath_basename(argv0) != argv0) {
+		if (!already_done) {
+			s_warning("can't locate \"%s\" in PATH: name contains '%c' already",
+				argv0,
+				strchr(argv0, G_DIR_SEPARATOR) != NULL ? G_DIR_SEPARATOR : '/');
+		}
+		goto done;
+	}
+
+	path = getenv("PATH");
+	if (NULL == path) {
+		if (!already_done) {
+			s_warning("can't locate \"%s\" in PATH: "
+				"no such environment variable", argv0);
+		}
+		goto done;
+	}
+
+	path = h_strdup(path);
+
+	tok = strtok(path, G_SEARCHPATH_SEPARATOR_S);
+	while (NULL != tok) {
+		const char *dir = tok;
+		filestat_t buf;
+
+		if ('\0' == *dir)
+			dir = ".";
+		concat_strings(filepath, sizeof filepath,
+			dir, G_DIR_SEPARATOR_S, argv0, NULL);
+
+		if (-1 != stat(filepath, &buf)) {
+			if (S_ISREG(buf.st_mode) && -1 != access(filepath, X_OK)) {
+				result = h_strdup(filepath);
+				break;
+			}
+		}
+		tok = strtok(NULL, G_SEARCHPATH_SEPARATOR_S);
+	}
+
+	hfree(path);
+
+done:
+	already_done = TRUE;	/* No warning on subsequent invocation */
+	return result;
+}
+
 /**
  * Open configuration file, renaming it as ".orig" when `renaming' is TRUE.
  * If configuration file cannot be found, try opening the ".orig" variant
@@ -88,39 +167,52 @@ file_register_fd_reclaimer(reclaim_fd_t callback)
  */
 static FILE *
 open_read(
-	const gchar *what, const file_path_t *fv, gint fvcnt, gboolean renaming,
-	gint *chosen)
+	const char *what, const file_path_t *fv, int fvcnt, gboolean renaming,
+	int *chosen)
 {
 	FILE *in;
-	gchar *path;
-	gchar *path_orig;
-	const gchar *instead = empty_str;
-	gint idx = 0;
+	char *path;
+	char *path_orig;
+	const char *instead = empty_str;
+	int idx = 0;
 
 	g_assert(fv != NULL);
 	g_assert(fvcnt >= 1);
 
 	path = make_pathname(fv->dir, fv->name);
 	if (!is_absolute_path(path)) {
-		G_FREE_NULL(path);
+		HFREE_NULL(path);
 		return NULL;
 	}
 
-	path_orig = g_strdup_printf("%s.%s", path, orig_ext);
+	path_orig = h_strdup_printf("%s.%s", path, orig_ext);
 	in = fopen(path, "r");
 	if (in) {
-		if (renaming && -1 == rename(path, path_orig))
+		if (is_running_on_mingw()) {
+			/* Windows can't rename an open file */
+			fclose(in);
+			in = NULL;
+		}
+		if (renaming && -1 == rename(path, path_orig)) {
 			g_warning("[%s] could not rename \"%s\" as \"%s\": %s",
 				what, path, path_orig, g_strerror(errno));
+		}
+		if (NULL == in) {
+			in = fopen(path_orig, "r");
+		}
 		goto out;
     } else {
-		if (errno != ENOENT) {
+		if (ENOENT == errno) {
+			if (common_dbg > 0) {
+				g_debug("[%s] cannot load non-existent \"%s\"", what, path);
+			}
+		} else {
 			instead = instead_str;			/* Regular file was present */
 			g_warning("[%s] failed to retrieve from \"%s\": %s", what, path,
 				g_strerror(errno));
 		}
         if (fvcnt > 1 && common_dbg > 0)
-            g_message("[%s] trying to load from alternate locations...", what);
+            g_debug("[%s] trying to load from alternate locations...", what);
     }
 
 	/*
@@ -137,7 +229,7 @@ open_read(
 	if (in != NULL) {
 		instead = instead_str;
 
-		G_FREE_NULL(path);
+		HFREE_NULL(path);
 		path = path_orig;
 		path_orig = NULL;
 	}
@@ -148,34 +240,39 @@ open_read(
 
 	if (in == NULL && fvcnt > 1) {
 		const file_path_t *xfv;
-		gint xfvcnt;
+		int xfvcnt;
 
 		instead = instead_str;
 
 		for (xfv = fv + 1, xfvcnt = fvcnt - 1; xfvcnt; xfv++, xfvcnt--) {
-			G_FREE_NULL(path);
+			HFREE_NULL(path);
 			path = make_pathname(xfv->dir, xfv->name);
 			idx++;
 			if (NULL != path && NULL != (in = fopen(path, "r")))
 				break;
+			if (path != NULL && common_dbg > 0) {
+				g_debug("[%s] cannot load non-existent \"%s\" either",
+					what, path);
+			}
 		}
 	}
 
-	if (in) {
-		if (common_dbg > 0)
-			g_message("[%s] retrieving from \"%s\"%s", what, path, instead);
-	} else if (instead == instead_str) {
-		g_warning("[%s] unable to retrieve: tried %d alternate location%s",
-			what, fvcnt, fvcnt == 1 ? "" : "s");
-    } else {
-		g_warning("[%s] unable to retrieve: no alternate locations known",
-			what);
+	if (common_dbg > 0) {
+		if (in) {
+			g_debug("[%s] retrieving from \"%s\"%s", what, path, instead);
+		} else if (instead == instead_str) {
+			g_debug("[%s] unable to retrieve: tried %d alternate location%s",
+				what, fvcnt, fvcnt == 1 ? "" : "s");
+		} else {
+			g_debug("[%s] unable to retrieve: no alternate locations known",
+				what);
+		}
 	}
 
 out:
 
-	G_FREE_NULL(path);
-	G_FREE_NULL(path_orig);
+	HFREE_NULL(path);
+	HFREE_NULL(path_orig);
 	if (in != NULL && chosen != NULL)
 		*chosen = idx;
 
@@ -193,7 +290,7 @@ out:
  * @returns opened FILE, or NULL if we were unable to open any.
  */
 FILE *
-file_config_open_read(const gchar *what, const file_path_t *fv, gint fvcnt)
+file_config_open_read(const char *what, const file_path_t *fv, int fvcnt)
 {
 	return open_read(what, fv, fvcnt, TRUE, NULL);
 }
@@ -210,7 +307,7 @@ file_config_open_read(const gchar *what, const file_path_t *fv, gint fvcnt)
  */
 FILE *
 file_config_open_read_norename(
-	const gchar *what, const file_path_t *fv, gint fvcnt)
+	const char *what, const file_path_t *fv, int fvcnt)
 {
 	return open_read(what, fv, fvcnt, FALSE, NULL);
 }
@@ -221,7 +318,7 @@ file_config_open_read_norename(
  */
 FILE *
 file_config_open_read_norename_chosen(
-	const gchar *what, const file_path_t *fv, gint fvcnt, gint *chosen)
+	const char *what, const file_path_t *fv, int fvcnt, int *chosen)
 {
 	return open_read(what, fv, fvcnt, FALSE, chosen);
 }
@@ -234,12 +331,12 @@ file_config_open_read_norename_chosen(
  * @returns opened FILE if success, NULL on error.
  */
 static FILE *
-file_config_open(const gchar *what, const file_path_t *fv)
+file_config_open(const char *what, const file_path_t *fv)
 {
 	FILE *out = NULL;
 	char *path;
 
-	path = g_strconcat(fv->dir, G_DIR_SEPARATOR_S, fv->name, ".",
+	path = h_strconcat(fv->dir, G_DIR_SEPARATOR_S, fv->name, ".",
 				new_ext, (void *) 0);
 	g_return_val_if_fail(NULL != path, NULL);
 
@@ -247,8 +344,8 @@ file_config_open(const gchar *what, const file_path_t *fv)
 		out = file_fopen(path, "w");
 		if (out == NULL)
 			g_warning("unable to persist %s", what);
-		G_FREE_NULL(path);
 	}
+	HFREE_NULL(path);
 	return out;
 }
 
@@ -256,7 +353,7 @@ file_config_open(const gchar *what, const file_path_t *fv)
  * Open configuration file for writing.
  */
 FILE *
-file_config_open_write(const gchar *what, const file_path_t *fv)
+file_config_open_write(const char *what, const file_path_t *fv)
 {
     return file_config_open(what, fv);
 }
@@ -280,7 +377,7 @@ file_config_close(FILE *out, const file_path_t *fv)
 
 	path = make_pathname(fv->dir, fv->name);
 	g_return_val_if_fail(NULL != path, FALSE);
-	path_new = g_strdup_printf("%s.%s", path, new_ext);
+	path_new = h_strdup_printf("%s.%s", path, new_ext);
 	if (NULL == path_new)
 		goto failed;
 
@@ -294,8 +391,8 @@ file_config_close(FILE *out, const file_path_t *fv)
 
 failed:
 
-	G_FREE_NULL(path_new);
-	G_FREE_NULL(path);
+	HFREE_NULL(path_new);
+	HFREE_NULL(path);
 	return success;
 }
 
@@ -303,14 +400,14 @@ failed:
  * Emit the configuration preamble.
  */
 void
-file_config_preamble(FILE *out, const gchar *what)
+file_config_preamble(FILE *out, const char *what)
 {
 	time_t now = tm_time();
 
 	g_assert(out);
 
 	fputs("# THIS FILE IS AUTOMATICALLY GENERATED -- DO NOT EDIT\n", out);
-	fprintf(out, "#\n# %s saved on %s#\n\n", what, ctime(&now));
+	fprintf(out, "#\n# %s saved on %s\n#\n\n", what, timestamp_to_string(now));
 }
 
 /**
@@ -333,13 +430,15 @@ file_path_set(file_path_t *fp, const char *dir, const char *name)
  * Errors are logged as a warning, unless `missing' is true, in which
  * case no error is logged for ENOENT.
  */
-static gint
-do_open(const gchar *path, gint flags, gint mode, gboolean missing)
+static int
+do_open(const char *path, int flags, int mode,
+	gboolean missing, gboolean absolute)
 {
-	gint fd;
-	const gchar *what;
+	const char *what;
+	int fd;
 
-	if (!is_absolute_path(path)) {
+	if (absolute && !is_absolute_path(path)) {
+		g_warning("do_open(): can't open absolute \"%s\": relative path", path);
 		errno = EPERM;
 		return -1;
 	}
@@ -349,34 +448,39 @@ do_open(const gchar *path, gint flags, gint mode, gboolean missing)
 #endif /* O_NOCTTY */
 
 	fd = open(path, flags, mode);
-	if (fd >= 0)
-		return fd;
+	if (fd < 0) {
+		if (flags & O_CREAT)
+			what = "create";
+		else if (flags & O_RDONLY)
+			what = "read";
+		else if (flags & O_WRONLY)
+			what = "write into";
+		else
+			what = "open";
 
-	if (flags & O_CREAT)
-		what = "create";
-	else if (flags & O_RDONLY)
-		what = "read";
-	else if (flags & O_WRONLY)
-		what = "write into";
-	else
-		what = "open";
+		/*
+		 * If we ran out of file descriptors, try to reclaim one from the
+		 * banning pool and retry.
+		 */
 
-	/*
-	 * If we ran out of file descriptors, try to reclaim one from the
-	 * banning pool and retry.
-	 */
-
-	if (
-		(errno == EMFILE || errno == ENFILE) &&
-		reclaim_fd != NULL && (*reclaim_fd)()
-	) {
-		fd = open(path, flags, mode);
-		if (fd >= 0) {
-			g_warning("do_open(): had to close a banned fd to %s file", what);
-			return fd;
+		if (
+			(errno == EMFILE || errno == ENFILE) &&
+			reclaim_fd != NULL && (*reclaim_fd)()
+		) {
+			fd = open(path, flags, mode);
+			if (fd >= 0) {
+				g_warning("do_open(): had to close a banned fd to %s file",
+					what);
+			}
 		}
 	}
 
+	if (fd >= 0) {
+		fd = get_non_stdio_fd(fd);
+		set_close_on_exec(fd);	/* Just in case */
+		return fd;
+	}
+
 	/*
 	 * Hack for broken libc, which can return -1 with errno = 0!
 	 * This happens when compiling with gcc-3.x and linking with -lpthread
@@ -401,32 +505,57 @@ do_open(const gchar *path, gint flags, gint mode, gboolean missing)
 /**
  * Open file, returning file descriptor or -1 on error with errno set.
  * Errors are logged as a warning.
+ *
+ * This is a perfect replacement for the open() system call, with logging
+ * on errors.
  */
-gint
-file_open(const gchar *path, gint flags)
+int
+file_open(const char *path, int flags, int mode)
 {
-	return do_open(path, flags, 0, FALSE);
+	return do_open(path, flags, mode, FALSE, FALSE);
 }
 
 /**
- * Open file, returning file descriptor or -1 on error with errno set.
+ * Open file given with absolute pathname.
+ * Returns file descriptor or -1 on error with errno set.
+ * Errors are logged as a warning.
+ */
+int
+file_absolute_open(const char *path, int flags, int mode)
+{
+	return do_open(path, flags, mode, FALSE, TRUE);
+}
+
+/**
+ * Open absolute file, returning file descriptor or -1 on error with errno set.
  * Errors are logged as a warning, unless the file is missing, in which
  * case nothing is logged.
  */
-gint
-file_open_missing(const gchar *path, gint flags)
+int
+file_open_missing(const char *path, int flags)
 {
-	return do_open(path, flags, 0, TRUE);
+	return do_open(path, flags, 0, TRUE, TRUE);
 }
 
 /**
- * Create file, returning file descriptor or -1 on error with errno set.
+ * Create absolute file, returning file descriptor or -1 on error with errno set.
  * Errors are logged as a warning.
  */
-gint
-file_create(const gchar *path, gint flags, gint mode)
+int
+file_create(const char *path, int flags, int mode)
 {
-	return do_open(path, flags | O_CREAT, mode, FALSE);
+	return do_open(path, flags | O_CREAT, mode, FALSE, TRUE);
+}
+
+/**
+ * Create absolute file, returning file descriptor or -1 on error with errno set.
+ * Errors are logged as a warning, unless the error is ENOENT which means
+ * the directory does not exist.
+ */
+int
+file_create_missing(const char *path, int flags, int mode)
+{
+	return do_open(path, flags | O_CREAT, mode, TRUE, TRUE);
 }
 
 /**
@@ -435,11 +564,11 @@ file_create(const gchar *path, gint flags, gint mode)
  * is TRUE.
  */
 static FILE *
-do_fopen(const gchar *path, const gchar *mode, gboolean missing)
+do_fopen(const char *path, const char *mode, gboolean missing)
 {
-	gchar m;
+	char m;
 	FILE *f;
-	const gchar *what;
+	const char *what;
 
 	if (!is_absolute_path(path)) {
 		errno = EPERM;
@@ -487,7 +616,7 @@ do_fopen(const gchar *path, const gchar *mode, gboolean missing)
  * Errors are logged as a warning.
  */
 FILE *
-file_fopen(const gchar *path, const gchar *mode)
+file_fopen(const char *path, const char *mode)
 {
 	return do_fopen(path, mode, FALSE);
 }
@@ -498,7 +627,7 @@ file_fopen(const gchar *path, const gchar *mode)
  * case nothing is logged.
  */
 FILE *
-file_fopen_missing(const gchar *path, const gchar *mode)
+file_fopen_missing(const char *path, const char *mode)
 {
 	return do_fopen(path, mode, TRUE);
 }
