@@ -1,5 +1,5 @@
 /*
- * $Id: getline.h 10743 2006-04-20 21:04:20Z cbiere $
+ * $Id: getline.h 18011 2010-12-22 18:34:00Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -36,7 +36,7 @@
 #ifndef _getline_h_
 #define _getline_h_
 
-#include <glib.h>
+#include "common.h"
 
 #define MAX_LINE_SIZE	1024	/**< Maximum length for regular line */
 
@@ -47,34 +47,25 @@
 typedef enum getline_result {
 	READ_MORE,		/**< OK, expecting more */
 	READ_DONE,		/**< OK, got whole line */
-	READ_OVERFLOW,	/**< Reached max line size */
+	READ_OVERFLOW	/**< Reached max line size */
 } getline_result_t;
 
-/**
- * A getline "object".
- */
-
-typedef struct getline {
-	size_t maxlen;				/**< Maximum authorized length */
-	size_t size;				/**< Current allocated size for `line' */
-	gchar *line;				/**< Accumulator, NUL terminated when done */
-	size_t pos;					/**< Next writing position in line[] */
-} getline_t;
-
-#define getline_maxlen(o)	((o)->maxlen)
+struct getline;
+typedef struct getline getline_t;
 
 /*
  * Public interface.
  */
 
 getline_t *getline_make(size_t maxsize);
-void getline_free(getline_t *o);
-void getline_reset(getline_t *o);
-getline_result_t getline_read(getline_t *o,
-					const gchar *data, size_t len, size_t *used);
-const gchar *getline_str(getline_t *o);
-size_t getline_length(getline_t *o);
-void getline_copy(getline_t *source, getline_t *dest);
+void getline_free(getline_t *);
+void getline_reset(getline_t *);
+getline_result_t getline_read(getline_t *,
+					const char *data, size_t len, size_t *used);
+const char *getline_str(const getline_t *);
+size_t getline_length(const getline_t *);
+void getline_copy(const getline_t *source, getline_t *dest);
+void getline_set_maxlen(getline_t *, size_t);
 
 #endif	/* _getline_h_ */
 
