@@ -1,5 +1,5 @@
 /*
- * $Id: header.c 13369 2007-04-19 23:23:34Z cbiere $
+ * $Id: header.c 19142 2011-05-23 20:44:04Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  *
@@ -35,20 +35,28 @@
 
 #include "common.h"
 
-RCSID("$Id: header.c 13369 2007-04-19 23:23:34Z cbiere $")
+RCSID("$Id: header.c 19142 2011-05-23 20:44:04Z rmanfredi $")
 
 #include "header.h"
-#include "glib-missing.h"
-#include "misc.h"
-#include "walloc.h"
+#include "ascii.h"
+#include "atoms.h"
 #include "getline.h"		/* For MAX_LINE_SIZE */
+#include "glib-missing.h"
+#include "halloc.h"
+#include "log.h"			/* For log_file_printable() */
+#include "misc.h"
 #include "slist.h"
+#include "str.h"
+#include "unsigned.h"
+#include "walloc.h"
 
 #include "override.h"		/* Must be the last header included */
 
+enum header_magic { HEADER_MAGIC = 0x71b8484fU };
+
 /*
  * The `headers' field is a hash table indexed by field name (case-insensitive).
- * Each value (GString) holds a private copy of the string making that header,
+ * Each value (str_t *) holds a private copy of the string making that header,
  * with all continuations removed (leading spaces collapsed into one), and
  * indentical fields concatenated using ", " separators, per RFC2616.
  *
@@ -58,13 +66,25 @@ RCSID("$Id: header.c 13369 2007-04-19 23:23:34Z cbiere $")
  */
 
 struct header {
+	enum header_magic magic;
 	GHashTable *headers;		/**< Indexed by name */
 	slist_t *fields;			/**< Ordered list of header_field_t */
-	gint flags;					/**< Various operating flags */
-	gint size;					/**< Total header size, in bytes */
-	gint num_lines;				/**< Total header lines seen */
+	int flags;					/**< Various operating flags */
+	int size;					/**< Total header size, in bytes */
+	int num_lines;				/**< Total header lines seen */
+	int refcnt;					/**< Reference count on the structure */
 };
 
+static inline void
+header_check(const header_t * const h)
+{
+	g_assert(h != NULL);
+	g_assert(HEADER_MAGIC == h->magic);
+	g_assert(h->refcnt > 0);
+}
+
+enum header_field { HEADER_FIELD_MAGIC = 0x6e29aad7U };
+
 /**
  * A header field.
  *
@@ -85,10 +105,18 @@ struct header {
  */
 
 typedef struct {
-	gchar *name;				/**< Field name */
+	enum header_field magic;
+	char *name;					/**< Field name */
 	slist_t *lines;				/**< List of lines making this header */
 } header_field_t;
 
+static inline void
+header_field_check(const header_field_t * const hf)
+{
+	g_assert(hf != NULL);
+	g_assert(HEADER_FIELD_MAGIC == hf->magic);
+}
+
 /***
  *** Operating flags
  ***/
@@ -117,7 +145,7 @@ static const char *error_str[] = {
 /**
  * @return human-readable error string corresponding to error code `errnum'.
  */
-const gchar *
+const char *
 header_strerror(guint errnum)
 {
 	if (errnum >= G_N_ELEMENTS(error_str))
@@ -126,7 +154,7 @@ header_strerror(guint errnum)
 	return error_str[errnum];
 }
 
-gint
+int
 header_num_lines(const header_t *h)
 {
 	return h->num_lines;
@@ -141,21 +169,21 @@ header_num_lines(const header_t *h)
  * A private copy of `name' is done.
  */
 static header_field_t *
-hfield_make(const gchar *name)
+hfield_make(const char *name)
 {
 	header_field_t *h;
 
-	h = walloc0(sizeof *h);
-	h->name = g_strdup(name);
+	WALLOC0(h);
+	h->magic = HEADER_FIELD_MAGIC;
+	h->name = h_strdup(name);
 
 	return h;
 }
 
 static void
-hfield_free_item(gpointer p, gpointer unused_data)
+hfield_free_item(void *p)
 {
-	(void) unused_data;
-	G_FREE_NULL(p);
+	HFREE_NULL(p);
 }
 
 /**
@@ -164,12 +192,12 @@ hfield_free_item(gpointer p, gpointer unused_data)
 static void
 hfield_free(header_field_t *h)
 {
-	if (h->lines) {
-		slist_foreach(h->lines, hfield_free_item, NULL);
-		slist_free(&h->lines);
-	}
-	G_FREE_NULL(h->name);
-	wfree(h, sizeof *h);
+	header_field_check(h);
+
+	slist_free_all(&h->lines, hfield_free_item);
+	HFREE_NULL(h->name);
+	h->magic = 0;
+	WFREE(h);
 }
 
 /**
@@ -177,12 +205,14 @@ hfield_free(header_field_t *h)
  * A private copy of the data is made.
  */
 static void
-hfield_append(header_field_t *h, const gchar *text)
+hfield_append(header_field_t *h, const char *text)
 {
+	header_field_check(h);
+
 	if (!h->lines) {
 		h->lines = slist_new();
 	}
-	slist_append(h->lines, g_strdup(text));
+	slist_append(h->lines, h_strdup(text));
 }
 
 /**
@@ -194,6 +224,7 @@ hfield_dump(const header_field_t *h, FILE *out)
 	slist_iter_t *iter;
 	gboolean first;
 
+	header_field_check(h);
 	g_assert(h->lines);
 
 	fprintf(out, "%s: ", h->name);
@@ -201,11 +232,30 @@ hfield_dump(const header_field_t *h, FILE *out)
 	first = TRUE;
 	iter = slist_iter_on_head(h->lines);
 	for (/* NOTHING */; slist_iter_has_item(iter); slist_iter_next(iter)) {
+		const char *s;
+
 		if (!first) {
 			first = FALSE;
 			fputs("    ", out);			/* Continuation line */
 		}
-		fputs(slist_iter_current(iter), out);
+		s = slist_iter_current(iter);
+		if (is_printable_iso8859_string(s)) {
+			fputs(s, out);
+		} else {
+			char buf[80];
+			const char *p = s;
+			int c;
+			size_t len = strlen(s);
+			gm_snprintf(buf, sizeof buf, "<%u non-printable byte%s>",
+				(unsigned) len, 1 == len ? "" : "s");
+			fputs(buf, out);
+			while ((c = *p++)) {
+				if (is_ascii_print(c) || is_ascii_space(c))
+					fputc(c, out);
+				else
+					fputc('.', out);	/* Less visual clutter than '?' */
+			}
+		}
 		fputc('\n', out);
 	}
 	slist_iter_free(&iter);	
@@ -218,9 +268,11 @@ hfield_dump(const header_field_t *h, FILE *out)
 static GHashTable *
 header_get_table(header_t *o)
 {
-	if (!o->headers) {
-		o->headers = g_hash_table_new(str_case_hash_func, str_case_eq_func);
-	}
+	header_check(o);
+
+	if (!o->headers)
+		o->headers = g_hash_table_new(ascii_strcase_hash, ascii_strcase_eq);
+
 	return o->headers;
 }
 
@@ -232,7 +284,9 @@ header_make(void)
 {
 	header_t *o;
 
-	o = walloc0(sizeof *o);
+	WALLOC0(o);
+	o->magic = HEADER_MAGIC;
+	o->refcnt = 1;
 	return o;
 }
 
@@ -244,28 +298,54 @@ free_header_data(gpointer key, gpointer value, gpointer unused_udata)
 {
 	(void) unused_udata;
 
-	G_FREE_NULL(key);		/* XXX if shared, don't do that */
-	g_string_free(value, TRUE);
+	HFREE_NULL(key);			/* XXX if shared, don't do that */
+	str_destroy(value);
 	return TRUE;
 }
 
+/**
+ * Take an extra reference on the header object.
+ * @return the header object.
+ */
+header_t *
+header_refcnt_inc(header_t *o)
+{
+	header_check(o);
+
+	o->refcnt++;
+	return o;
+}
+
 /**
  * Destroy header object.
  */
 void
 header_free(header_t *o)
 {
-	g_assert(o);
+	header_check(o);
+
+	if (o->refcnt > 1) {
+		o->refcnt--;
+		return;
+	}
 
 	header_reset(o);
-	wfree(o, sizeof *o);
+	o->magic = 0;
+	WFREE(o);
 }
 
-static void
-header_reset_item(gpointer p, gpointer unused_data)
+/**
+ * Destroy header object and nullify pointer holding it.
+ */
+void
+header_free_null(header_t **o_ptr)
 {
-	(void) unused_data;
-	hfield_free(p);
+	header_t *o = *o_ptr;
+
+	if (o != NULL) {
+		header_free(o);
+		*o_ptr = NULL;
+	}
 }
 
 /**
@@ -274,20 +354,14 @@ header_reset_item(gpointer p, gpointer unused_data)
 void
 header_reset(header_t *o)
 {
-	static const header_t zero_header;
-
-	g_assert(o);
+	header_check(o);
 
 	if (o->headers) {
 		g_hash_table_foreach_remove(o->headers, free_header_data, NULL);
-		g_hash_table_destroy(o->headers);
-		o->headers = NULL;
+		gm_hash_table_destroy_null(&o->headers);
 	}
-	if (o->fields) {
-		slist_foreach(o->fields, header_reset_item, NULL);
-		slist_free(&o->fields);
-	}
-	*o = zero_header;
+	slist_free_all(&o->fields, cast_to_slist_destroy(hfield_free));
+	o->flags = o->size = o->num_lines = 0;
 }
 
 /**
@@ -295,28 +369,45 @@ header_reset(header_t *o)
  * pointer to the internals of the header structure, so it must not be
  * kept around.
  */
-gchar *
-header_get(const header_t *o, const gchar *field)
+char *
+header_get(const header_t *o, const char *field)
 {
-	GString *v;
+	str_t *v;
+
+	header_check(o);
 
 	if (o->headers) {
 		v = g_hash_table_lookup(o->headers, deconstify_gchar(field));
 	} else {
 		v = NULL;
 	}
-	return v ? v->str : NULL;
+	return str_2c(v);
 }
 
 /**
  * Get field value, or NULL if not present.  The value returned is a
- * copy of the internal value, so it may be kept around, but must be
- * freed by the caller.
+ * pointer to the internals of the header structure, so it must not be
+ * kept around.
+ *
+ * If the len_ptr pointer is not NULL, it is filled with the length
+ * of the header string.
  */
-gchar *
-header_getdup(const header_t *o, const gchar *field)
+char *
+header_get_extended(const header_t *o, const char *field, size_t *len_ptr)
 {
-	return g_strdup(header_get(o, field));
+	str_t *v;
+
+	header_check(o);
+
+	if (o->headers) {
+		v = g_hash_table_lookup(o->headers, deconstify_gchar(field));
+	} else {
+		v = NULL;
+	}
+	if (v && len_ptr != NULL) {
+		*len_ptr = str_len(v);
+	}
+	return str_2c(v);
 }
 
 /**
@@ -324,10 +415,12 @@ header_getdup(const header_t *o, const gchar *field)
  * A private copy of the `field' name and of the `text' data is made.
  */
 static void
-add_header(header_t *o, const gchar *field, const gchar *text)
+add_header(header_t *o, const char *field, const char *text)
 {
 	GHashTable *ht;
-	GString *v;
+	str_t *v;
+
+	header_check(o);
 
 	ht = header_get_table(o);
 	v = g_hash_table_lookup(ht, field);
@@ -337,18 +430,18 @@ add_header(header_t *o, const gchar *field, const gchar *text)
 		 * the value, comma-separated.
 		 */
 
-		g_string_append(v, ", ");
-		g_string_append(v, text);
+		str_cat(v, ", ");
+		str_cat(v, text);
 
 	} else {
-		gchar *key;
+		char *key;
 
 		/*
 		 * Create a new header entry in the hash table.
 		 */
 
-		key = g_strdup(field);
-		v = g_string_new(text);
+		key = h_strdup(field);
+		v = str_new_from(text);
 		g_hash_table_insert(ht, key, v);
 	}
 }
@@ -358,15 +451,17 @@ add_header(header_t *o, const gchar *field, const gchar *text)
  * A private copy of the data is made.
  */
 static void
-add_continuation(header_t *o, const gchar *field, const gchar *text)
+add_continuation(header_t *o, const char *field, const char *text)
 {
-	GString *v;
+	str_t *v;
 
+	header_check(o);
 	g_assert(o->headers);
+
 	v = g_hash_table_lookup(o->headers, field);
-	g_assert(v);
-	g_string_append_c(v, ' ');
-	g_string_append(v, text);
+	g_assert(v != NULL);
+	str_putc(v, ' ');
+	str_cat(v, text);
 }
 
 /**
@@ -375,14 +470,17 @@ add_continuation(header_t *o, const gchar *field, const gchar *text)
  *
  * @return an error code, or HEAD_OK if appending was successful.
  */
-gint
-header_append(header_t *o, const gchar *text, gint len)
+int
+header_append(header_t *o, const char *text, int len)
 {
-	gchar buf[MAX_LINE_SIZE];
-	const gchar *p = text;
+	char buf[MAX_LINE_SIZE];
+	const char *p = text;
 	guchar c;
 	header_field_t *hf;
 
+	header_check(o);
+	g_assert(len >= 0);
+
 	if (o->flags & HEAD_F_EOH)
 		return HEAD_EOH_REACHED;
 
@@ -467,7 +565,7 @@ header_append(header_t *o, const gchar *text, gint len)
 		 */
 
 	} else {
-		gchar *b;
+		char *b;
 		gboolean seen_space = FALSE;
 
 		/*
@@ -554,21 +652,30 @@ header_dump_item(gpointer p, gpointer user_data)
 }
 
 /**
- * Dump whole header on specified file.
+ * Dump whole header on specified file, followed by trailer string
+ * (if not NULL) and a final "\n".
  */
 void
-header_dump(const header_t *o, FILE *out)
+header_dump(FILE *out, const header_t *o, const char *trailer)
 {
+	header_check(o);
+
+	if (!log_file_printable(out))
+		return;
+
 	if (o->fields) {
 		slist_foreach(o->fields, header_dump_item, out);
 	}
+	if (trailer)
+		fprintf(out, "%s\n", trailer);
 }
 
 /***
  *** Header formatting with continuations.
  ***/
 
-#define HEADER_FMT_MAGIC		0xf7a91c
+enum header_fmt_magic { HEADER_FMT_MAGIC = 0xf7a91cU };
+
 #define HEADER_FMT_DFLT_LEN		256		/**< Default field length if no hint */
 #define HEADER_FMT_LINE_LEN		78		/**< Try to never emit longer lines */
 #define HEADER_FMT_MAX_SIZE		1024	/**< Max line size for header */
@@ -577,38 +684,45 @@ header_dump(const header_t *o, FILE *out)
  * Header formatting context.
  */
 struct header_fmt {
-	guint32 magic;
-	gint maxlen;			/**< Maximum line length before continuation */
-	GString *header;		/**< Header being built */
-	gchar sep[257];			/**< Optional separator */
-	gint seplen;			/**< Length of separator string */
-	gint stripped_seplen;	/**< Length of separator without trailing space */
-	gint current_len;		/**< Length of currently built line */
-	gboolean data_emitted;	/**< Whether data was ever emitted */
-	gboolean frozen;		/**< Header terminated */
+	enum header_fmt_magic magic;
+	size_t max_size;		/**< Maximum line length, including "\r\n" + NUL */
+	size_t maxlen;			/**< Maximum line length before continuation */
+	str_t *header;			/**< Header being built */
+	const char *sep;		/**< Optional separator */
+	size_t seplen;			/**< Length of separator string */
+	size_t stripped_seplen;	/**< Length of separator without trailing space */
+	size_t current_len;		/**< Length of currently built line */
+	unsigned data_emitted:1;/**< Whether data was ever emitted */
+	unsigned frozen:1;		/**< Header terminated */
+	unsigned empty:1;		/**< Header max-size too small, must stay empty */
 };
 
+static inline void
+header_fmt_check(const header_fmt_t *fmt)
+{
+	g_assert(fmt != NULL);
+	g_assert(HEADER_FMT_MAGIC == fmt->magic);
+}
+
 /**
  * Compute the length of the string `s' whose length is `len' with trailing
- * whitespace ignored.
+ * blanks ignored.
  */
-static gint
-stripped_strlen(const gchar *s, gint len)
+static size_t
+stripped_strlen(const char *s, size_t len)
 {
-	const gchar *end = s + len;
-	gint i;
+	const char *end;
 
 	/*
-	 * Locate last non-space char in separator.
+	 * Locate last non-blank char in separator.
 	 */
 
-	/* XXX: Shouldn't this allow at least \t or maybe use isspace()? */
-	for (i = len - 1; i >= 0; i--, end--) {
-		if (s[i] != ' ')
-			return end - s;
+	for (end = &s[len]; end != s; end--) {
+		if (!is_ascii_blank(end[-1]))
+			break;
 	}
 
-	return 0;
+	return end - s;
 }
 
 /**
@@ -628,30 +742,49 @@ stripped_strlen(const gchar *s, gint len)
  * @param `len_hint' is the expected line size, for pre-sizing purposes.
  * (0 to guess).
  *
- * @return opaque pointer.
+ * @param `max_size' is the maximum header size, including the final "\r\n"
+ * and the trailing NUL.  If the initial field name is larger than the
+ * configured maximum size, the header field will remain completely empty.
+ *
+ * @return pointer to the formatting object.
  */
-gpointer
-header_fmt_make(const gchar *field, const gchar *separator, gint len_hint)
+header_fmt_t *
+header_fmt_make(const char *field, const char *separator,
+	size_t len_hint, size_t max_size)
 {
 	struct header_fmt *hf;
-	size_t len;
 
-	g_assert(!separator || strlen(separator) < sizeof(hf->sep));
-	g_assert(len_hint >= 0);
+	g_assert(size_is_non_negative(len_hint));
 
-	hf = walloc(sizeof(*hf));
+	WALLOC(hf);
 	hf->magic = HEADER_FMT_MAGIC;
-	hf->header = g_string_sized_new(len_hint ? len_hint : HEADER_FMT_DFLT_LEN);
+	hf->header = str_new(len_hint ? len_hint : HEADER_FMT_DFLT_LEN);
 	hf->maxlen = HEADER_FMT_LINE_LEN;
 	hf->data_emitted = FALSE;
 	hf->frozen = FALSE;
-	len = g_strlcpy(hf->sep, separator ? separator : "", sizeof(hf->sep));
-	hf->seplen = MIN(len, (sizeof hf->sep - 1));
+	hf->max_size = max_size;
+	hf->sep = atom_str_get(separator ? separator : "");
+	hf->seplen = strlen(hf->sep);
 	hf->stripped_seplen = stripped_strlen(hf->sep, hf->seplen);
-	g_string_append(hf->header, field);
-	g_string_append(hf->header, ": ");
+	str_cat(hf->header, field);
+	str_cat(hf->header, ": ");
 
-	hf->current_len = hf->header->len;
+	hf->current_len = str_len(hf->header);
+
+	/*
+	 * If right from the start the header would be larger than the configured
+	 * size, force it to stay empty.  That means, the final string returned
+	 * will be "", the empty string.
+	 */
+
+	if (str_len(hf->header) + sizeof("\r\n") > hf->max_size) {
+		hf->empty = TRUE;
+		str_setlen(hf->header, 0);
+	} else {
+		hf->empty = FALSE;
+	}
+
+	header_fmt_check(hf);
 
 	return hf;
 }
@@ -660,12 +793,10 @@ header_fmt_make(const gchar *field, const gchar *separator, gint len_hint)
  * Set max line length.
  */
 void
-header_fmt_set_line_length(gpointer o, gint maxlen)
+header_fmt_set_line_length(header_fmt_t *hf, size_t maxlen)
 {
-	struct header_fmt *hf = o;
-
-	g_assert(hf->magic == HEADER_FMT_MAGIC);
-	g_assert(maxlen > 0);
+	header_fmt_check(hf);
+	g_assert(size_is_positive(maxlen));
 
 	hf->maxlen = maxlen;
 }
@@ -674,33 +805,38 @@ header_fmt_set_line_length(gpointer o, gint maxlen)
  * Dispose of header formatting context.
  */
 void
-header_fmt_free(gpointer o)
+header_fmt_free(header_fmt_t **hf_ptr)
 {
-	struct header_fmt *hf = o;
+	header_fmt_t *hf = *hf_ptr;
 
-	g_assert(hf->magic == HEADER_FMT_MAGIC);
+	if (hf) {
+		header_fmt_check(hf);
 
-	g_string_free(hf->header, TRUE);
-	hf->magic = 0;
-	wfree(hf, sizeof *hf);
+		str_destroy_null(&hf->header);
+		atom_str_free_null(&hf->sep);
+		hf->magic = 0;
+		WFREE(hf);
+		*hf_ptr = NULL;
+	}
 }
 
 /**
  * Checks whether appending `len' bytes of data to the header would fit
- * within the `maxlen' total header size requirement in case a continuation
+ * within the maximum header size requirement in case a continuation
  * is emitted, and using the configured separator.
- *
- * @attention
- * NB: The `maxlen' parameter is the amount of data that can be generated for
- * the header string, not counting the final "\r\n" + the trailing NUL byte.
  */
 gboolean
-header_fmt_value_fits(gpointer o, gint len, gint maxlen)
+header_fmt_value_fits(const header_fmt_t *hf, size_t len)
 {
-	struct header_fmt *hf = o;
-	gint final_len;
+	size_t final_len;
+	size_t maxlen, n;
 
-	g_assert(hf->magic == HEADER_FMT_MAGIC);
+	header_fmt_check(hf);
+
+	if (hf->empty)
+		return FALSE;
+
+	maxlen = size_saturate_sub(hf->max_size, sizeof("\r\n"));
 
 	/*
 	 * If it fits on the line, no continuation will have to be emitted.
@@ -708,10 +844,15 @@ header_fmt_value_fits(gpointer o, gint len, gint maxlen)
 	 * followed by "\r\n\t" (3 chars).
 	 */
 
-	if (hf->current_len + len + hf->seplen <= hf->maxlen)
-		final_len = hf->header->len + len + hf->seplen;
-	else
-		final_len = hf->header->len + len + hf->stripped_seplen + 3;
+	final_len = size_saturate_add(str_len(hf->header), len);
+
+	n = size_saturate_add(hf->current_len, size_saturate_add(len, hf->seplen));
+	if (n <= hf->maxlen) {
+		final_len = size_saturate_add(final_len, hf->seplen);
+	} else {
+		final_len = size_saturate_add(final_len, hf->stripped_seplen);
+		final_len = size_saturate_add(final_len, 3);
+	}
 
 	return final_len < maxlen;	/* Could say "<=" perhaps, but let's be safe */
 }
@@ -725,42 +866,73 @@ header_fmt_value_fits(gpointer o, gint len, gint maxlen)
  *         BEFORE outputting the data, and only when nothing has been emitted
  *         already.
  * @param `slen' is the separator length, 0 if empty.
- * @param `sslen' is the stripped separator length, -1 if unknown yet.
+ * @param `sslen' is the stripped separator length, (size_t)-1 if unknown yet.
+ *
+ * @return TRUE if we were able to fit the string, FALSE if it would have
+ * resulted in the header being larger than the configured max size (the
+ * header line is left in the state it was in upon entry, in that case).
  */
-static void
-header_fmt_append_full(struct header_fmt *hf, const gchar *str,
-	const gchar *separator, gint slen, gint sslen)
+static gboolean
+header_fmt_append_full(header_fmt_t *hf, const char *str,
+	const char *separator, size_t slen, size_t sslen)
 {
-	size_t len;
-	gint curlen;
+	size_t len, curlen;
+	gsize gslen;
+	gboolean success;
 
+	header_fmt_check(hf);
+	g_assert(size_is_non_negative(slen));
+	g_assert((size_t)-1 == sslen || size_is_non_negative(sslen));
+
+	if (hf->empty)
+		return FALSE;
+
+	gslen = str_len(hf->header);
 	len = strlen(str);
-	g_assert(len <= INT_MAX);	/* Legacy bug */
 	curlen = hf->current_len;
+	g_assert(size_is_non_negative(curlen));
 
-	if (curlen + len + slen > (size_t) hf->maxlen) {
+	g_assert(len <= INT_MAX);	/* Legacy bug */
+
+	if (
+		size_saturate_add(curlen, size_saturate_add(len, slen)) >
+			UNSIGNED(hf->maxlen)
+	) {
 		/*
 		 * Emit sperator, if any and data was already emitted.
 		 */
 
 		if (separator != NULL && hf->data_emitted) {
-			gint s = sslen >= 0 ? sslen : stripped_strlen(separator, slen);
-			const gchar *p;
-
-			for (p = separator; s > 0; p++, s--)
-				g_string_append_c(hf->header, *p);
+			sslen = (size_t)-1 != sslen ? sslen :
+				stripped_strlen(separator, slen);
+			str_cat_len(hf->header, separator, sslen);
 		}
 
-		g_string_append(hf->header, "\r\n\t");	/* Includes continuation */
+		str_cat(hf->header, "\r\n\t");			/* Includes continuation */
 		curlen = 1;								/* One tab */
 	} else if (hf->data_emitted) {
-		g_string_append(hf->header, separator);
+		str_cat(hf->header, separator);
 		curlen += slen;
 	}
 
-	hf->data_emitted = TRUE;
-	g_string_append(hf->header, str);
-	hf->current_len = curlen + len;
+	str_cat(hf->header, str);
+
+	/*
+	 * Check for overflows, undoing string changes if needed.
+	 */
+
+	if (str_len(hf->header) + sizeof("\r\n") > hf->max_size) {
+		success = FALSE;
+		str_setlen(hf->header, gslen);			/* Undo! */
+	} else {
+		success = TRUE;
+		hf->data_emitted = TRUE;
+		hf->current_len = curlen + len;
+	}
+
+	g_assert(str_len(hf->header) + sizeof("\r\n") <= hf->max_size);
+
+	return success;
 }
 
 /**
@@ -772,19 +944,21 @@ header_fmt_append_full(struct header_fmt *hf, const gchar *str,
  * end of a line.  It supersedes any separator configured at make time.
  *
  * To use the standard separator, use header_fmt_append_value().
+ *
+ * @return TRUE if we were able to append the data whilst remaining under
+ * the configured maximum length.
  */
-void
-header_fmt_append(gpointer o, const gchar *str, const gchar *separator)
+gboolean
+header_fmt_append(header_fmt_t *hf, const char *str, const char *separator)
 {
-	struct header_fmt *hf = o;
-	gint seplen;
+	size_t seplen;
 
-	g_assert(hf->magic == HEADER_FMT_MAGIC);
+	header_fmt_check(hf);
 	g_assert(!hf->frozen);
 
 	seplen = (separator == NULL) ? 0 : strlen(separator);
 
-	header_fmt_append_full(hf, str, separator, seplen, -1);
+	return header_fmt_append_full(hf, str, separator, seplen, (size_t)-1);
 }
 
 /**
@@ -795,29 +969,29 @@ header_fmt_append(gpointer o, const gchar *str, const gchar *separator)
  * whitespaces is used.
  *
  * To supersede the default separator, use header_fmt_append().
+ *
+ * @return TRUE if we were able to append the data whilst remaining under
+ * the configured maximum length.
  */
-void
-header_fmt_append_value(gpointer o, const gchar *str)
+gboolean
+header_fmt_append_value(header_fmt_t *hf, const char *str)
 {
-	struct header_fmt *hf = o;
-
-	g_assert(hf->magic == HEADER_FMT_MAGIC);
+	header_fmt_check(hf);
 	g_assert(!hf->frozen);
 
-	header_fmt_append_full(hf, str, hf->sep, hf->seplen, hf->stripped_seplen);
+	return header_fmt_append_full(hf,
+		str, hf->sep, hf->seplen, hf->stripped_seplen);
 }
 
 /**
  * @return length of currently formatted header.
  */
-gint
-header_fmt_length(gpointer o)
+size_t
+header_fmt_length(const header_fmt_t *hf)
 {
-	struct header_fmt *hf = o;
+	header_fmt_check(hf);
 
-	g_assert(hf->magic == HEADER_FMT_MAGIC);
-
-	return hf->header->len;
+	return str_len(hf->header);
 }
 
 /**
@@ -825,28 +999,27 @@ header_fmt_length(gpointer o)
  * Further appending is forbidden.
  */
 void
-header_fmt_end(gpointer o)
+header_fmt_end(header_fmt_t *hf)
 {
-	struct header_fmt *hf = o;
-
-	g_assert(hf->magic == HEADER_FMT_MAGIC);
+	header_fmt_check(hf);
 	g_assert(!hf->frozen);
 
-	g_string_append(hf->header, "\r\n");
+	if (!hf->empty)
+		str_cat(hf->header, "\r\n");
 	hf->frozen = TRUE;
+
+	g_assert(str_len(hf->header) < hf->max_size);
 }
 
 /**
  * @return current header string.
  */
-gchar *
-header_fmt_string(gpointer o)
+const char *
+header_fmt_string(const header_fmt_t *hf)
 {
-	struct header_fmt *hf = o;
+	header_fmt_check(hf);
 
-	g_assert(hf->magic == HEADER_FMT_MAGIC);
-
-	return hf->header->str;		/* Guaranteed to be always NUL-terminated */
+	return str_2c(hf->header);	/* Guaranteed to be always NUL-terminated */
 }
 
 /**
@@ -855,21 +1028,18 @@ header_fmt_string(gpointer o)
  * @attention
  * NB: returns pointer to static data!
  */
-gchar *
-header_fmt_to_string(gpointer o)
+const char *
+header_fmt_to_string(const header_fmt_t *hf)
 {
-	static gchar line[HEADER_FMT_MAX_SIZE + 1];
-	struct header_fmt *hf = o;
+	static char line[HEADER_FMT_MAX_SIZE + 1];
 
-	g_assert(hf->magic == HEADER_FMT_MAGIC);
-
-	if (hf->header->len > HEADER_FMT_MAX_SIZE)
-		g_warning("trying to format too long an HTTP line (%d bytes)",
-			(int) hf->header->len);
-
-	strncpy(line, hf->header->str, HEADER_FMT_MAX_SIZE);
-	line[HEADER_FMT_MAX_SIZE] = '\0';
+	header_fmt_check(hf);
 
+	if (str_len(hf->header) >= sizeof line) {
+		g_warning("trying to format too long an HTTP line (%lu bytes)",
+			(unsigned long) str_len(hf->header));
+	}
+	clamp_strncpy(line, sizeof line, str_2c(hf->header), str_len(hf->header));
 	return line;
 }
 
