@@ -1,5 +1,5 @@
 /*
- * $Id: host_addr.c 13876 2007-06-18 19:02:15Z cbiere $
+ * $Id: host_addr.c 19266 2011-06-15 16:02:31Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  * Copyright (c) 2005, Christian Biere
@@ -36,7 +36,7 @@
 
 #include "common.h"
 
-RCSID("$Id: host_addr.c 13876 2007-06-18 19:02:15Z cbiere $")
+RCSID("$Id: host_addr.c 19266 2011-06-15 16:02:31Z rmanfredi $")
 
 #ifdef I_NET_IF
 #include <net/if.h>		/* For IFF_* flags */
@@ -50,10 +50,15 @@ RCSID("$Id: host_addr.c 13876 2007-06-18 19:02:15Z cbiere $")
 #include <netdb.h>				/* For gethostbyname() */
 #endif /* I_NETDB */
 
+#include "ascii.h"
 #include "atoms.h"				/* For binary_hash */
 #include "host_addr.h"
-#include "misc.h"
+#include "concat.h"
 #include "endian.h"
+#include "glib-missing.h"		/* For g_strlcpy() */
+#include "parse.h"
+#include "random.h"
+#include "stringify.h"
 #include "walloc.h"
 
 #include "override.h"		/* Must be the last header included */
@@ -158,22 +163,33 @@ ipv4_addr_is_routable(guint32 ip)
 static inline gboolean
 host_addr_is_6to4(const host_addr_t ha)
 {
-	return NET_TYPE_IPV6 == host_addr_net(ha) &&
-		0x2002 == peek_be16(&ha.addr.ipv6[0]);
+	return host_addr_is_ipv6(ha) && 0x2002 == peek_be16(&ha.addr.ipv6[0]);
+}
+
+static inline gboolean
+host_addr_is_teredo(const host_addr_t ha)
+{
+	return host_addr_is_ipv6(ha) && 0x20010000UL == peek_be32(&ha.addr.ipv6[0]);
 }
 
 static inline guint32
-host_addr_6to4_ipv4(const host_addr_t ha)
+host_addr_6to4_client_ipv4(const host_addr_t ha)
 {
 	return peek_be32(&ha.addr.ipv6[2]);	/* 2002:AABBCCDD::/48 */
 }
 
+static inline guint32
+host_addr_teredo_client_ipv4(const host_addr_t ha)
+{
+	return ~peek_be32(&ha.addr.ipv6[12]);	/* 2001::~A~B~C~D */
+}
+
 gboolean
-host_addr_6to4_to_ipv4(const host_addr_t from, host_addr_t *to)
+host_addr_6to4_client(const host_addr_t from, host_addr_t *to)
 {
 	if (host_addr_is_6to4(from)) {
 		if (to) {
-			*to = host_addr_peek_ipv4(&from.addr.ipv6[2]);
+			*to = host_addr_get_ipv4(host_addr_6to4_client_ipv4(from));
 		}
 		return TRUE;
 	} else {
@@ -184,6 +200,42 @@ host_addr_6to4_to_ipv4(const host_addr_t from, host_addr_t *to)
 	}
 }
 
+gboolean
+host_addr_teredo_client(const host_addr_t from, host_addr_t *to)
+{
+	if (host_addr_is_teredo(from)) {
+		if (to) {
+			*to = host_addr_get_ipv4(host_addr_teredo_client_ipv4(from));
+		}
+		return TRUE;
+	} else {
+		if (to) {
+			*to = zero_host_addr;
+		}
+		return FALSE;
+	}
+}
+
+static gboolean
+host_addr_is_tunneled(const host_addr_t ha)
+{
+	return host_addr_is_teredo(ha) || host_addr_is_6to4(ha);
+}
+
+gboolean
+host_addr_tunnel_client(const host_addr_t from, host_addr_t *to)
+{
+	return host_addr_teredo_client(from, to) || host_addr_6to4_client(from, to);
+}
+
+guint32
+host_addr_tunnel_client_ipv4(const host_addr_t from)
+{
+	host_addr_t to;
+	(void) host_addr_tunnel_client(from, &to);
+	return host_addr_ipv4(to);
+}
+
 /**
  * Checks whether the given address is 127.0.0.1 or ::1.
  */
@@ -211,6 +263,28 @@ host_addr_is_loopback(const host_addr_t addr)
 	return FALSE;
 }
 
+/**
+ * Checks whether the given address is unspecified (all zeroes).
+ */
+gboolean
+host_addr_is_unspecified(const host_addr_t addr)
+{
+	switch (host_addr_net(addr)) {
+	case NET_TYPE_IPV4:
+		return host_addr_ipv4(addr) == 0;
+
+	case NET_TYPE_IPV6:
+		return host_addr_equal(addr, ipv6_unspecified);
+
+	case NET_TYPE_LOCAL:
+	case NET_TYPE_NONE:
+		break;
+	}
+
+	g_assert_not_reached();
+	return FALSE;
+}
+
 /**
  * Check whether host can be reached from the Internet.
  * We rule out IPs of private networks, plus some other invalid combinations.
@@ -236,8 +310,8 @@ host_addr_is_routable(const host_addr_t addr)
 				!host_addr_matches(ha, ipv6_site_local, 10) &&
 				!host_addr_matches(ha, ipv6_link_local, 10) &&
 				!(
-						host_addr_is_6to4(ha) &&
-						!ipv4_addr_is_routable(host_addr_6to4_ipv4(ha))
+						host_addr_is_tunneled(ha) &&
+						!ipv4_addr_is_routable(host_addr_tunnel_client_ipv4(ha))
 				);
 
 	case NET_TYPE_LOCAL:
@@ -356,7 +430,7 @@ host_addr_convert(const host_addr_t from, host_addr_t *to,
  * @return The length of the resulting string assuming ``size'' is sufficient.
  */
 size_t
-host_addr_to_string_buf(const host_addr_t ha, gchar *dst, size_t size)
+host_addr_to_string_buf(const host_addr_t ha, char *dst, size_t size)
 {
 	switch (host_addr_net(ha)) {
 	case NET_TYPE_IPV4:
@@ -380,10 +454,24 @@ host_addr_to_string_buf(const host_addr_t ha, gchar *dst, size_t size)
  * @return a pointer to a static buffer holding a NUL-terminated string
  *         representing the given host address.
  */
-const gchar *
+const char *
 host_addr_to_string(const host_addr_t ha)
 {
-	static gchar buf[HOST_ADDR_BUFLEN];
+	static char buf[HOST_ADDR_BUFLEN];
+	size_t n;
+
+	n = host_addr_to_string_buf(ha, buf, sizeof buf);
+	g_assert(n < sizeof buf);
+	return buf;
+}
+
+/**
+ * Same as host_addr_to_string(), but in another static buffer.
+ */
+const char *
+host_addr_to_string2(const host_addr_t ha)
+{
+	static char buf[HOST_ADDR_BUFLEN];
 	size_t n;
 
 	n = host_addr_to_string_buf(ha, buf, sizeof buf);
@@ -405,10 +493,10 @@ host_addr_to_string(const host_addr_t ha)
  */
 size_t
 host_addr_port_to_string_buf(const host_addr_t ha, guint16 port,
-		gchar *dst, size_t size)
+		char *dst, size_t size)
 {
-	gchar host_buf[HOST_ADDR_BUFLEN];
-	gchar port_buf[UINT32_DEC_BUFLEN];
+	char host_buf[HOST_ADDR_BUFLEN];
+	char port_buf[UINT32_DEC_BUFLEN];
 	size_t n;
 
 	host_addr_to_string_buf(ha, host_buf, sizeof host_buf);
@@ -438,10 +526,10 @@ host_addr_port_to_string_buf(const host_addr_t ha, guint16 port,
  * @return a pointer to a static buffer holding a NUL-terminated string
  *         representing the given host address and port.
  */
-const gchar *
+const char *
 host_addr_port_to_string(const host_addr_t ha, guint16 port)
 {
-	static gchar buf[HOST_ADDR_PORT_BUFLEN];
+	static char buf[HOST_ADDR_PORT_BUFLEN];
 	size_t n;
 
 	n = host_addr_port_to_string_buf(ha, port, buf, sizeof buf);
@@ -449,10 +537,10 @@ host_addr_port_to_string(const host_addr_t ha, guint16 port)
 	return buf;
 }
 
-const gchar *
+const char *
 host_addr_port_to_string2(const host_addr_t ha, guint16 port)
 {
-	static gchar buf[HOST_ADDR_PORT_BUFLEN];
+	static char buf[HOST_ADDR_PORT_BUFLEN];
 	size_t n;
 
 	n = host_addr_port_to_string_buf(ha, port, buf, sizeof buf);
@@ -460,19 +548,74 @@ host_addr_port_to_string2(const host_addr_t ha, guint16 port)
 	return buf;
 }
 
-const gchar *
-host_port_to_string(const gchar *hostname, host_addr_t addr, guint16 port)
+/**
+ * Prints the ``port'' followed by host address ``ha'' to ``dst''. The string
+ * written to ``dst'' is always NUL-terminated unless ``size'' is zero. If
+ * ``size'' is too small, the string will be truncated.
+ *
+ * @param dst the destination buffer; may be NULL iff ``size'' is zero.
+ * @param port the port number.
+ * @param ha the host address.
+ * @param size the size of ``dst'' in bytes.
+ *
+ * @return The length of the resulting string assuming ``size'' is sufficient.
+ */
+size_t
+host_port_addr_to_string_buf(guint16 port, const host_addr_t ha,
+	char *dst, size_t size)
 {
-	static gchar buf[MAX_HOSTLEN + 32];
+	char port_buf[UINT32_DEC_BUFLEN];
+	char host_buf[HOST_ADDR_BUFLEN];
+	size_t n;
+
+	uint32_to_string_buf(port, port_buf, sizeof port_buf);
+	host_addr_to_string_buf(ha, host_buf, sizeof host_buf);
+
+	switch (host_addr_net(ha)) {
+	case NET_TYPE_IPV6:
+		n = concat_strings(dst, size, port_buf, ":[", host_buf, "]",
+			(void *) 0);
+		break;
+	case NET_TYPE_IPV4:
+		n = concat_strings(dst, size, port_buf, ":", host_buf, (void *) 0);
+		break;
+	default:
+		n = g_strlcpy(dst, host_buf, size);
+	}
+
+	return n;
+}
+
+/**
+ * Prints the host address ``ha'' followed by ``port'' to a static buffer. 
+ *
+ * @param ha the host address.
+ * @param port the port number.
+ *
+ * @return a pointer to a static buffer holding a NUL-terminated string
+ *         representing the given host address and port.
+ */
+const char *
+port_host_addr_to_string(guint16 port, const host_addr_t ha)
+{
+	static char buf[HOST_ADDR_PORT_BUFLEN];
+	size_t n;
+
+	n = host_port_addr_to_string_buf(port, ha, buf, sizeof buf);
+	g_assert(n < sizeof buf);
+	return buf;
+}
+
+const char *
+host_port_to_string(const char *hostname, host_addr_t addr, guint16 port)
+{
+	static char buf[MAX_HOSTLEN + 32];
 
 	if (hostname) {
-		gchar port_buf[UINT32_DEC_BUFLEN];
+		char port_buf[UINT32_DEC_BUFLEN];
 
 		uint32_to_string_buf(port, port_buf, sizeof port_buf);
-		strncpy(buf, hostname, MAX_HOSTLEN);
-		buf[MAX_HOSTLEN] = '\0';
-		strncat(buf, ":", sizeof buf);
-		strncat(buf, port_buf, sizeof buf);
+		concat_strings(buf, sizeof buf, hostname, ":", port_buf, (void *) 0);
 	} else {
 		host_addr_port_to_string_buf(addr, port, buf, sizeof buf);
 	}
@@ -491,7 +634,7 @@ host_port_to_string(const gchar *hostname, host_addr_t addr, guint16 port)
  * @return Returns TRUE on success; otherwise FALSE.
  */
 gboolean
-string_to_host_addr(const char *s, const gchar **endptr, host_addr_t *addr_ptr)
+string_to_host_addr(const char *s, const char **endptr, host_addr_t *addr_ptr)
 {
 	guint32 ip;
 
@@ -534,9 +677,9 @@ string_to_host_addr(const char *s, const gchar **endptr, host_addr_t *addr_ptr)
  *         hostname.
  */
 gboolean
-string_to_host_or_addr(const char *s, const gchar **endptr, host_addr_t *ha)
+string_to_host_or_addr(const char *s, const char **endptr, host_addr_t *ha)
 {
-	const gchar *ep;
+	const char *ep;
 	size_t len;
 	host_addr_t addr;
 
@@ -576,24 +719,34 @@ string_to_host_or_addr(const char *s, const gchar **endptr, host_addr_t *ha)
 	return len > 0 && len <= MAX_HOSTLEN;
 }
 
+/**
+ * Parse "IP:port" string to retrieve the host address and port.
+ *
+ * @param str		the string to parse
+ * @param endptr	if not NULL, it will point the first character after
+ * 					the parsed elements.
+ * @param addr_ptr	where the parsed address is written
+ * @param port_ptr	where the parsed port is written
+ *
+ * @return TRUE if we parsed the string correctly, FALSE if it did not
+ * look like a valid "IP:port" string.
+ */
 gboolean
-string_to_host_addr_port(const gchar *str, const gchar **endptr,
+string_to_host_addr_port(const char *str, const char **endptr,
 	host_addr_t *addr_ptr, guint16 *port_ptr)
 {
-	const gchar *ep;
+	const char *ep;
 	host_addr_t addr;
 	gboolean ret;
 	guint16 port;
 
 	ret = string_to_host_or_addr(str, &ep, &addr);
 	if (ret && ':' == *ep && is_host_addr(addr)) {
-		guint32 u;
-		gint error;
+		int error;
 
 		ep++;
-		u = parse_uint32(ep, &ep, 10, &error);
-		port = error || u > 65535 ? 0 : u;
-		ret = 0 != port;
+		port = parse_uint16(ep, &ep, 10, &error);
+		ret = 0 == error && 0 != port;
 	} else {
 		ret = FALSE;
 		port = 0;
@@ -605,11 +758,53 @@ string_to_host_addr_port(const gchar *str, const gchar **endptr,
 		*port_ptr = port;
 	if (endptr)
 		*endptr = ep;
+
 	return ret;
 }
 
-void
-gethostbyname_error(const gchar *host)
+/**
+ * Parse "port:IP" string to retrieve the host address and port.
+ *
+ * @param str		the string to parse
+ * @param endptr	if not NULL, it will point the first character after
+ * 					the parsed elements.
+ * @param port_ptr	where the parsed port is written
+ * @param addr_ptr	where the parsed address is written
+ *
+ * @return TRUE if we parsed the string correctly, FALSE if it did not
+ * look like a valid "port:IP" string.
+ */
+gboolean
+string_to_port_host_addr(const char *str, const char **endptr,
+	guint16 *port_ptr, host_addr_t *addr_ptr)
+{
+	const char *ep;
+	host_addr_t addr;
+	gboolean ret;
+	guint16 port;
+	int error;
+
+	port = parse_uint16(str, &ep, 10, &error);
+	if (!error && ':' == *ep) {
+		ret = string_to_host_or_addr(ep, &ep, &addr);
+		ret = ret && is_host_addr(addr);
+	} else {
+		addr = zero_host_addr;
+		ret = FALSE;
+	}
+
+	if (addr_ptr)
+		*addr_ptr = addr;
+	if (port_ptr)
+		*port_ptr = port;
+	if (endptr)
+		*endptr = ep;
+
+	return ret;
+}
+
+static void
+gethostbyname_error(const char *host)
 {
 #if defined(HAS_HSTRERROR)
 		g_warning("cannot resolve \"%s\": %s", host, hstrerror(h_errno));
@@ -639,14 +834,9 @@ socket_addr_set(socket_addr_t *sa_ptr, const host_addr_t addr, guint16 port)
 			static const struct sockaddr_in zero_sin;
 
 			sa_ptr->inet4 = zero_sin;
-			/* Note: The next line is a cheap(?) hack to be used until
-			 *       HAS_SIN_LEN is initialized by Configure. sin_len if
-			 *       available.
-			 */
-			sa_ptr->len = sizeof sa_ptr->inet4;
-#ifdef HAS_SIN_LEN
+#ifdef HAS_SOCKADDR_IN_SIN_LEN
 			sa_ptr->inet4.sin_len = sizeof sa_ptr->inet4;
-#endif /* HAS_SIN_LEN */
+#endif /* HAS_SOCKADDR_IN_SIN_LEN */
 			sa_ptr->inet4.sin_family = AF_INET;
 			sa_ptr->inet4.sin_port = htons(port);
 			sa_ptr->inet4.sin_addr.s_addr = htonl(host_addr_ipv4(addr));
@@ -696,6 +886,78 @@ socket_addr_init(socket_addr_t *sa_ptr, enum net_type net)
 	return 0;
 }
 
+/**
+ * Fill `p_addr' with the socket's local address/port information.
+ */
+int
+socket_addr_getsockname(socket_addr_t *p_addr, int fd)
+{
+	struct sockaddr_in sin4;
+	socklen_t len;
+	host_addr_t addr = zero_host_addr;
+	guint16 port = 0;
+	gboolean success = FALSE;
+
+	len = sizeof sin4;
+	if (-1 != getsockname(fd, cast_to_gpointer(&sin4), &len)) {
+		addr = host_addr_peek_ipv4(&sin4.sin_addr.s_addr);
+		port = ntohs(sin4.sin_port);
+		success = TRUE;
+	}
+
+#ifdef HAS_IPV6
+	if (!success) {
+		struct sockaddr_in6 sin6;
+
+		len = sizeof sin6;
+		if (-1 != getsockname(fd, cast_to_gpointer(&sin6), &len)) {
+			addr = host_addr_peek_ipv6(sin6.sin6_addr.s6_addr);
+			port = ntohs(sin6.sin6_port);
+			success = TRUE;
+		}
+	}
+#endif	/* HAS_IPV6 */
+
+	if (success) {
+		socket_addr_set(p_addr, addr, port);
+	}
+	return success ? 0 : -1;
+}
+
+/**
+ * Fill `p_addr' with socket's remote address/port information.
+ */
+int
+socket_addr_getpeername(socket_addr_t *p_addr, int fd)
+{
+	socklen_t addr_len;
+
+	g_return_val_if_fail(p_addr, -1);
+	g_return_val_if_fail(fd >= 0, -1);
+
+	errno = 0;
+	addr_len = socket_addr_init(p_addr, NET_TYPE_IPV4);
+	if (
+		-1 != getpeername(fd, socket_addr_get_sockaddr(p_addr), &addr_len) &&
+		AF_INET == socket_addr_get_family(p_addr)
+	) {
+		return 0;
+	}
+
+#ifdef HAS_IPV6
+	errno = 0;
+	addr_len = socket_addr_init(p_addr, NET_TYPE_IPV6);
+	if (
+		-1 != getpeername(fd, socket_addr_get_sockaddr(p_addr), &addr_len) &&
+		AF_INET6 == socket_addr_get_family(p_addr)
+	) {
+		return 0;
+	}
+#endif	/* HAS_IPV6 */
+
+	return -1;
+}
+
 /**
  * Resolves an IP address to a hostname per DNS.
  *
@@ -703,7 +965,7 @@ socket_addr_init(socket_addr_t *sa_ptr, enum net_type net)
  * @return		On success, the hostname is returned. Otherwise, NULL is
  *				returned. The resulting string points to a static buffer.
  */
-const gchar *
+const char *
 host_addr_to_name(host_addr_t addr)
 {
 	socket_addr_t sa;
@@ -717,13 +979,13 @@ host_addr_to_name(host_addr_t addr)
 
 #ifdef HAS_GETNAMEINFO
 	{
-		static gchar host[1025];
-		gint error;
+		static char host[1025];
+		int error;
 
 		error = getnameinfo(socket_addr_get_const_sockaddr(&sa),
 					socket_addr_get_len(&sa), host, sizeof host, NULL, 0, 0);
 		if (error) {
-			gchar buf[HOST_ADDR_BUFLEN];
+			char buf[HOST_ADDR_BUFLEN];
 
 			host_addr_to_string_buf(addr, buf, sizeof buf);
 			g_message("getnameinfo() failed for \"%s\": %s",
@@ -736,7 +998,7 @@ host_addr_to_name(host_addr_t addr)
 	{
 		const struct hostent *he;
 		socklen_t len = 0;
-		const gchar *ptr = NULL;
+		const char *ptr = NULL;
 
 		switch (host_addr_net(addr)) {
 		case NET_TYPE_IPV4:
@@ -758,7 +1020,7 @@ host_addr_to_name(host_addr_t addr)
 
 		he = gethostbyaddr(ptr, len, socket_addr_get_family(&sa));
 		if (!he) {
-			gchar buf[HOST_ADDR_BUFLEN];
+			char buf[HOST_ADDR_BUFLEN];
 
 			host_addr_to_string_buf(addr, buf, sizeof buf);
 			gethostbyname_error(buf);
@@ -769,15 +1031,48 @@ host_addr_to_name(host_addr_t addr)
 #endif	/* HAS_GETNAMEINFO */
 }
 
+#ifdef HAS_GETADDRINFO
+host_addr_t
+addrinfo_to_addr(const struct addrinfo *ai)
+{
+	host_addr_t addr = zero_host_addr;
+
+	switch (ai->ai_family) {
+	case PF_INET:
+		if (ai->ai_addrlen >= 4) {
+			const struct sockaddr_in *sin4;
+
+			sin4 = cast_to_gconstpointer(ai->ai_addr);
+			addr = host_addr_peek_ipv4(&sin4->sin_addr.s_addr);
+		}
+		break;
+
+#ifdef HAS_IPV6
+	case PF_INET6:
+		if (ai->ai_addrlen >= 16) {
+			const struct sockaddr_in6 *sin6;
+
+			sin6 = cast_to_gconstpointer(ai->ai_addr);
+			addr = host_addr_peek_ipv6(cast_to_gconstpointer(
+						sin6->sin6_addr.s6_addr));
+		}
+		break;
+#endif /* HAS_IPV6 */
+	}
+
+	return addr;
+}
+#endif	/* HAS_GETADDRINFO */
+
 static GSList *
-resolve_hostname(const gchar *host, enum net_type net)
-#if defined(HAS_GETADDRINFO)
+resolve_hostname(const char *host, enum net_type net)
+#ifdef HAS_GETADDRINFO
 {
 	static const struct addrinfo zero_hints;
 	struct addrinfo hints, *ai, *ai0 = NULL;
 	GHashTable *ht;
 	GSList *sl_addr;
-	gint error;
+	int error;
 
 	g_assert(host);
 	
@@ -799,29 +1094,7 @@ resolve_hostname(const gchar *host, enum net_type net)
 		if (!ai->ai_addr)
 			continue;
 
-		addr = zero_host_addr;
-		switch (ai->ai_family) {
-		case PF_INET:
-			if (ai->ai_addrlen >= 4) {
-				const struct sockaddr_in *sin4;
-
-				sin4 = cast_to_gconstpointer(ai->ai_addr);
-				addr = host_addr_peek_ipv4(&sin4->sin_addr.s_addr);
-			}
-			break;
-
-#ifdef HAS_IPV6
-		case PF_INET6:
-			if (ai->ai_addrlen >= 16) {
-				const struct sockaddr_in6 *sin6;
-
-				sin6 = cast_to_gconstpointer(ai->ai_addr);
-				addr = host_addr_peek_ipv6(cast_to_gconstpointer(
-							sin6->sin6_addr.s6_addr));
-			}
-			break;
-#endif /* HAS_IPV6 */
-		}
+		addr = addrinfo_to_addr(ai);
 
 		if (is_host_addr(addr) && !g_hash_table_lookup(ht, &addr)) {
 			host_addr_t *addr_copy;
@@ -843,7 +1116,7 @@ resolve_hostname(const gchar *host, enum net_type net)
 	const struct hostent *he;
 	GHashTable *ht;
 	GSList *sl_addr;
-	gint af;
+	int af;
 	size_t i;
 
 	g_assert(host);
@@ -931,9 +1204,9 @@ resolve_hostname(const gchar *host, enum net_type net)
  *         On failure, NULL is returned.
  */
 GSList *
-name_to_host_addr(const gchar *host, enum net_type net)
+name_to_host_addr(const char *host, enum net_type net)
 {
-	const gchar *endptr;
+	const char *endptr;
 	host_addr_t addr;
 	
 	g_assert(host);
@@ -964,10 +1237,9 @@ host_addr_free_list(GSList **sl_ptr)
 
 		for (sl = *sl_ptr; NULL != sl; sl = g_slist_next(sl)) {
 			host_addr_t *addr_ptr = sl->data;
-			wfree(addr_ptr, sizeof *addr_ptr);
+			WFREE(addr_ptr);
 		}
-		g_slist_free(*sl_ptr);
-		*sl_ptr = NULL;
+		gm_slist_free_null(sl_ptr);
 	}
 }
 
@@ -977,7 +1249,7 @@ host_addr_free_list(GSList **sl_ptr)
  * and return it.
  */
 host_addr_t
-name_to_single_host_addr(const gchar *host, enum net_type net)
+name_to_single_host_addr(const char *host, enum net_type net)
 {
 	GSList *sl_addr;
 	host_addr_t addr;
@@ -989,7 +1261,7 @@ name_to_single_host_addr(const gchar *host, enum net_type net)
 		size_t i, len;
 
 		len = g_slist_length(sl_addr);
-		i = len > 1 ? (random_raw() % len) : 0;
+		i = len > 1 ? (random_u32() % len) : 0;
 
 		for (sl = sl_addr; NULL != sl; sl = g_slist_next(sl)) {
 			const host_addr_t *addr_ptr = sl->data;
@@ -1021,11 +1293,23 @@ host_addr_eq_func(gconstpointer p, gconstpointer q)
 	return host_addr_equal(*a, *b);
 }
 
+/**
+ * List free callback.
+ */
+void
+wfree_host_addr1(void *key)
+{
+	wfree(key, sizeof(host_addr_t));
+}
+
+/**
+ * Aging table callback.
+ */
 void
 wfree_host_addr(gpointer key, gpointer unused_data)
 {
 	(void) unused_data;
-	wfree(key, sizeof (host_addr_t));
+	wfree(key, sizeof(host_addr_t));
 }
 
 /**
@@ -1105,19 +1389,20 @@ host_addr_free_interface_addrs(GSList **sl_ptr)
 		for (sl = *sl_ptr; NULL != sl; sl = g_slist_next(sl)) {
             host_addr_t *addr = sl->data;
 			g_assert(host_addr_initialized(*addr));
-			wfree(addr, sizeof *addr);
+			WFREE(addr);
 		}
-		g_slist_free(*sl_ptr);
-		*sl_ptr = NULL;
+		gm_slist_free_null(sl_ptr);
 	}
 }
 
 struct packed_host_addr
 host_addr_pack(const host_addr_t addr)
 {
+	static struct packed_host_addr zero_paddr;
 	struct packed_host_addr paddr;
 	enum net_type net;
 
+	paddr = zero_paddr;			/* Shut compiler warning up */
 	net = host_addr_net(addr);
 	switch (net) {
 	case NET_TYPE_IPV4:
@@ -1177,37 +1462,57 @@ host_pack(const host_addr_t addr, guint16 port)
 	return phost;
 }
 
-gboolean
-packed_host_unpack(const struct packed_host phost,
+void
+packed_host_unpack(const struct packed_host *phost,	/* MUST be a pointer */
 	host_addr_t *addr_ptr, guint16 *port_ptr)
 {
 	if (port_ptr) {
-		*port_ptr = peek_be16(&phost.port);
+		*port_ptr = peek_be16(&phost->port);
 	}
-	switch (phost.ha.net) {
+	switch (phost->ha.net) {
 	case NET_TYPE_IPV4:
 		if (addr_ptr) {
-			*addr_ptr = host_addr_peek_ipv4(phost.ha.addr);
+			/*
+			 * Compiler hack alert!
+			 * 
+			 * Ensure generated code will NEVER try to access the whole array
+			 * since only the necessary bytes may have been allocated to hold
+			 * the packed representation! When "phost->ha.addr" causes a
+			 * memory fault if the structure is tighly allocated, taking the
+			 * address of the first byte alleviates all problems since we
+			 * know host_addr_peek_ipv4() will behave sanely with the pointer
+			 * it is passed and will not try to access beyond the size of
+			 * the expected IPv4 address.
+			 *
+			 *		--RAM, 2011-05-03
+			 */
+			*addr_ptr = host_addr_peek_ipv4(&phost->ha.addr[0]);
 		}
-		return TRUE;
+		return;
 	case NET_TYPE_IPV6:
 		if (addr_ptr) {
-			*addr_ptr = host_addr_peek_ipv6(phost.ha.addr);
+			*addr_ptr = host_addr_peek_ipv6(phost->ha.addr);
 		}
-		return TRUE;
+		return;
 	case NET_TYPE_LOCAL:
 		if (addr_ptr) {
 			*addr_ptr = local_host_addr;
 		}
-		return TRUE;
+		return;
 	case NET_TYPE_NONE:
 		if (addr_ptr) {
 			*addr_ptr = zero_host_addr;
 		}
-		return TRUE;
+		return;
 	}
-	g_assert_not_reached();
-	return FALSE;
+	/*
+	 * Because this routine can be used through gnet_host_get_addr() to grab
+	 * the address from a packed structure coming from a DBMW file, we cannot
+	 * abort the execution when faced with an invalid structure (coming from
+	 * the disk).
+	 */
+	g_carp("corrupted packed host: unknown net address type %d", phost->ha.net);
+	*addr_ptr = zero_host_addr;
 }
 
 /**
