@@ -1,5 +1,5 @@
 /*
- * $Id: host_addr.h 12229 2006-11-05 15:23:10Z cbiere $
+ * $Id: host_addr.h 19266 2011-06-15 16:02:31Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  * Copyright (c) 2005, Christian Biere
@@ -38,6 +38,9 @@
 #define _host_addr_h_
 
 #include "common.h"
+
+#include "endian.h"
+#include "glib-missing.h"		/* For g_carp() */
 #include "misc.h"
 
 /**
@@ -50,10 +53,10 @@ enum net_type {
 	NET_TYPE_NONE	= 0,
 	NET_TYPE_LOCAL	= 1,
 	NET_TYPE_IPV4	= 4,
-	NET_TYPE_IPV6	= 6,
+	NET_TYPE_IPV6	= 6
 };
 
-static inline gint
+static inline int
 net_type_to_pf(enum net_type net)
 {
 	switch (net) {
@@ -61,17 +64,17 @@ net_type_to_pf(enum net_type net)
 	case NET_TYPE_LOCAL: return PF_LOCAL;
 	case NET_TYPE_IPV4:  return PF_INET;
 	case NET_TYPE_IPV6:
-#ifdef USE_IPV6
+#ifdef HAS_IPV6
 		return PF_INET6;
 #else
 		return PF_UNSPEC;
-#endif /* USE_IPV6 */
+#endif /* HAS_IPV6 */
 	}
 	g_assert_not_reached();
 	return PF_UNSPEC;
 }
 
-static inline gint
+static inline int
 net_type_to_af(enum net_type net)
 {
 	switch (net) {
@@ -79,11 +82,11 @@ net_type_to_af(enum net_type net)
 	case NET_TYPE_LOCAL: return AF_LOCAL;
 	case NET_TYPE_IPV4:  return AF_INET;
 	case NET_TYPE_IPV6:
-#ifdef USE_IPV6
+#ifdef HAS_IPV6
 		return AF_INET6;
 #else
 		return AF_UNSPEC;
-#endif /* USE_IPV6 */
+#endif /* HAS_IPV6 */
 	}
 	g_assert_not_reached();
 	return AF_UNSPEC;
@@ -94,11 +97,7 @@ typedef struct host_addr {
 	guint32 net;	/**< The address network type */
 	union {
 		guint8	ipv6[16];	/**< This is valid if "net == NET_TYPE_IPV6" */
-		guint32 ipv4;		/**< @attention: Always in host byte order! */
-
-		guint8	u8[16];
-		guint16 u16[8];
-		guint32 u32[4];
+		guint32 ipv4;	/**< @attention: Always in host byte order! */
 	} addr;
 } host_addr_t;
 
@@ -113,11 +112,15 @@ struct packed_host {
 };
 
 typedef union socket_addr {
-	guint8 len;
+	/*
+	 * Both structures in the union start with an sa_family_t field
+	 * (either sin_family or sin6_family) so we can discriminate the
+	 * union correctly by probing inet4.sin_family and inet6.sin6_family.
+	 */
 	struct sockaddr_in inet4;
-#ifdef USE_IPV6
+#ifdef HAS_IPV6
 	struct sockaddr_in6 inet6;
-#endif /* USE_IPV6 */
+#endif /* HAS_IPV6 */
 } socket_addr_t;
 
 static const host_addr_t ipv4_unspecified = {	/* 0.0.0.0/32 */
@@ -125,9 +128,13 @@ static const host_addr_t ipv4_unspecified = {	/* 0.0.0.0/32 */
 	{ { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 } },
 };
 
-static const host_addr_t ipv4_loopback = {	/* 0.0.0.0/32 */
+static const host_addr_t ipv4_loopback = {	/* 127.0.0.1/32 */
 	NET_TYPE_IPV4,
+#if G_BYTE_ORDER == G_BIG_ENDIAN
 	{ { 0x7f, 0x00, 0x00, 0x01, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 } },
+#else
+	{ { 0x01, 0x00, 0x00, 0x7f, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 } },
+#endif
 };
 
 static const host_addr_t ipv6_unspecified = {	/* ::/128 */
@@ -166,12 +173,38 @@ static const host_addr_t ipv6_6to4 = {			/* 2002::/16 */
 };
 
 
+static const host_addr_t local_host_addr = {
+	NET_TYPE_LOCAL,
+	{ { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 } },
+};
+
 static const host_addr_t zero_host_addr;
 
+static inline size_t
+packed_host_length(const struct packed_host *ph)
+{
+	switch (ph->ha.net) {
+	case NET_TYPE_IPV4:
+		return ptr_diff(&ph->ha.addr[4], ph);
+	case NET_TYPE_IPV6:
+	case NET_TYPE_LOCAL:
+		return ptr_diff(&ph->ha.addr[16], ph);
+	case NET_TYPE_NONE:
+		return ptr_diff(&ph->ha.addr[0], ph);
+	}
+	/*
+	 * Because this routine can be used through gnet_host_length() to compute
+	 * the length of keys from a DBMW file, we cannot abort the execution when
+	 * faced with an invalid structure (coming from the disk).
+	 */
+	g_carp("corrupted packed host: unknown net address type %d", ph->ha.net);
+	return ptr_diff(&ph->ha.addr[0], ph);
+}
+
 gboolean host_addr_convert(const host_addr_t from, host_addr_t *to,
 	enum net_type to_net);
 gboolean host_addr_can_convert(const host_addr_t from, enum net_type to_net);
-gboolean host_addr_6to4_to_ipv4(const host_addr_t from, host_addr_t *to);
+gboolean host_addr_tunnel_client(const host_addr_t from, host_addr_t *to);
 
 
 static inline gboolean
@@ -189,7 +222,7 @@ host_addr_initialized(const host_addr_t ha)
 	return FALSE;
 }
 
-static inline const gchar *
+static inline const char *
 net_type_to_string(enum net_type net)
 {
 	switch (net) {
@@ -208,6 +241,18 @@ host_addr_net(const host_addr_t ha)
 	return ha.net;
 }
 
+static inline G_GNUC_CONST gboolean
+host_addr_is_ipv4(const host_addr_t ha)
+{
+	return NET_TYPE_IPV4 == ha.net;
+}
+
+static inline G_GNUC_CONST gboolean
+host_addr_is_ipv6(const host_addr_t ha)
+{
+	return NET_TYPE_IPV6 == ha.net;
+}
+
 static inline guint32
 host_addr_ipv4(const host_addr_t ha)
 {
@@ -231,7 +276,13 @@ host_addr_get_ipv4(guint32 ip)
 }
 
 static inline host_addr_t
-host_addr_get_ipv6(const guint8 *ipv6)
+host_addr_peek_ipv4(const void *ipv4)
+{
+	return host_addr_get_ipv4(peek_be32(ipv4));
+}
+
+static inline host_addr_t
+host_addr_peek_ipv6(const guint8 *ipv6)
 {
 	host_addr_t ha;
 	
@@ -246,14 +297,14 @@ host_addr_equal(const host_addr_t a, const host_addr_t b)
 	if (a.net == b.net) {
 		switch (a.net) {
 		case NET_TYPE_IPV4:
-			return a.addr.ipv4 == b.addr.ipv4;
+			return host_addr_ipv4(a) == host_addr_ipv4(b);
 		case NET_TYPE_IPV6:
 			if (0 != memcmp(a.addr.ipv6, b.addr.ipv6, sizeof a.addr.ipv6)) {
 				host_addr_t a_ipv4, b_ipv4;
 
 				return host_addr_convert(a, &a_ipv4, NET_TYPE_IPV4) &&
 					host_addr_convert(b, &b_ipv4, NET_TYPE_IPV4) &&
-					a_ipv4.addr.ipv4 == b_ipv4.addr.ipv4;
+					host_addr_ipv4(a_ipv4) == host_addr_ipv4(b_ipv4);
 			}
 			return TRUE;
 
@@ -270,10 +321,10 @@ host_addr_equal(const host_addr_t a, const host_addr_t b)
 	return FALSE;
 }
 
-static inline gint
+static inline int
 host_addr_cmp(host_addr_t a, host_addr_t b)
 {
-	gint r;
+	int r;
 
 	r = CMP(a.net, b.net);
 	if (0 != r) {
@@ -286,7 +337,7 @@ host_addr_cmp(host_addr_t a, host_addr_t b)
 
 	switch (a.net) {
 	case NET_TYPE_IPV4:
-		return CMP(a.addr.ipv4, b.addr.ipv4);
+		return CMP(host_addr_ipv4(a), host_addr_ipv4(b));
 	case NET_TYPE_IPV6:
 		{
 			guint i;
@@ -318,11 +369,11 @@ host_addr_matches(const host_addr_t a, const host_addr_t b, guint8 bits)
 	switch (a.net) {
 	case NET_TYPE_IPV4:
 		shift = bits < 32 ? 32 - bits : 0;
-		return (a.addr.ipv4 >> shift) == (to.addr.ipv4 >> shift);
+		return host_addr_ipv4(a) >> shift == host_addr_ipv4(to) >> shift;
 
 	case NET_TYPE_IPV6:
 		{
-			gint i;
+			int i;
 
 			bits = MIN(128, bits);
 			for (i = 0; bits >= 8; i++, bits -= 8) {
@@ -353,7 +404,7 @@ is_host_addr(const host_addr_t ha)
 {
 	switch (host_addr_net(ha)) {
 	case NET_TYPE_IPV4:
-		return 0 != ha.addr.ipv4;
+		return 0 != host_addr_ipv4(ha);
 	case NET_TYPE_IPV6:
 		return 0 != memcmp(ha.addr.ipv6, zero_host_addr.addr.ipv6,
 						sizeof ha.addr.ipv6);
@@ -386,13 +437,13 @@ host_addr_hash(host_addr_t ha)
 		}
 		/* FALL THROUGH */
 	case NET_TYPE_IPV4:
-		return ha.net ^ ha.addr.ipv4;
+		return ha.net ^ host_addr_ipv4(ha);
 	case NET_TYPE_LOCAL:
 	case NET_TYPE_NONE:
 		return ha.net;
 	}
 	g_assert_not_reached();
-	return -1;
+	return (guint32) -1;
 }
 
 /**
@@ -409,11 +460,11 @@ socket_addr_get_addr(const socket_addr_t *addr)
 	g_assert(addr);
 
 	if (AF_INET == addr->inet4.sin_family) {
-		ha = host_addr_get_ipv4(ntohl(addr->inet4.sin_addr.s_addr));
-#if defined(USE_IPV6)
+		ha = host_addr_peek_ipv4(&addr->inet4.sin_addr.s_addr);
+#if defined(HAS_IPV6)
 	} else if (AF_INET6 == addr->inet6.sin6_family) {
-		ha = host_addr_get_ipv6(addr->inet6.sin6_addr.s6_addr);
-#endif /* USE_IPV6 */
+		ha = host_addr_peek_ipv6(addr->inet6.sin6_addr.s6_addr);
+#endif /* HAS_IPV6 */
 	} else {
 		ha = zero_host_addr;
 	}
@@ -434,10 +485,10 @@ socket_addr_get_port(const socket_addr_t *addr)
 
 	if (AF_INET == addr->inet4.sin_family) {
 		return ntohs(addr->inet4.sin_port);
-#if defined(USE_IPV6)
+#if defined(HAS_IPV6)
 	} else if (AF_INET6 == addr->inet6.sin6_family) {
 		return ntohs(addr->inet6.sin6_port);
-#endif /* USE_IPV6 */
+#endif /* HAS_IPV6 */
 	}
 
 	return 0;
@@ -450,48 +501,48 @@ socket_addr_get_len(const socket_addr_t *addr)
 
 	if (AF_INET == addr->inet4.sin_family) {
 		return sizeof addr->inet4;
-#if defined(USE_IPV6)
+#if defined(HAS_IPV6)
 	} else if (AF_INET6 == addr->inet6.sin6_family) {
 		return sizeof addr->inet6;
-#endif /* USE_IPV6 */
+#endif /* HAS_IPV6 */
 	}
 
 	return 0;
 }
 
-static inline struct sockaddr *
-socket_addr_get_sockaddr(socket_addr_t *addr)
+static inline const struct sockaddr *
+socket_addr_get_const_sockaddr(const socket_addr_t *addr)
 {
 	g_assert(addr != NULL);
 
 	if (AF_INET == addr->inet4.sin_family) {
-		return cast_to_gpointer(&addr->inet4);
-#if defined(USE_IPV6)
+		return cast_to_gconstpointer(&addr->inet4);
+#if defined(HAS_IPV6)
 	} else if (AF_INET6 == addr->inet6.sin6_family) {
-		return cast_to_gpointer(&addr->inet6);
-#endif /* USE_IPV6 */
+		return cast_to_gconstpointer(&addr->inet6);
+#endif /* HAS_IPV6 */
 	}
 
 	return NULL;
 }
 
-static inline const struct sockaddr *
-socket_addr_get_const_sockaddr(const socket_addr_t *addr)
+static inline struct sockaddr *
+socket_addr_get_sockaddr(socket_addr_t *addr)
 {
-	return socket_addr_get_sockaddr((socket_addr_t *) addr);
+	return (struct sockaddr *) socket_addr_get_const_sockaddr(addr);
 }
 
-static inline gint
+static inline int
 socket_addr_get_family(const socket_addr_t *addr)
 {
 	g_assert(addr != NULL);
 
 	if (AF_INET == addr->inet4.sin_family) {
 		return AF_INET;
-#if defined(USE_IPV6)
+#if defined(HAS_IPV6)
 	} else if (AF_INET6 == addr->inet6.sin6_family) {
 		return AF_INET6;
-#endif /* USE_IPV6 */
+#endif /* HAS_IPV6 */
 	}
 
 	return 0;
@@ -500,43 +551,61 @@ socket_addr_get_family(const socket_addr_t *addr)
 socklen_t socket_addr_set(socket_addr_t *sa_ptr,
 			const host_addr_t addr, guint16 port);
 socklen_t socket_addr_init(socket_addr_t *sa_ptr, enum net_type net);
+int socket_addr_getpeername(socket_addr_t *p_addr, int fd);
+int socket_addr_getsockname(socket_addr_t *p_addr, int fd);
 
 guint host_addr_hash_func(gconstpointer key);
 gboolean host_addr_eq_func(gconstpointer p, gconstpointer q);
+void wfree_host_addr1(void *key);
 void wfree_host_addr(gpointer key, gpointer unused_data);
 
 int host_addr_family(const host_addr_t ha);
 gboolean is_private_addr(const host_addr_t addr);
 gboolean host_addr_is_routable(const host_addr_t addr);
 gboolean host_addr_is_loopback(const host_addr_t addr);
+gboolean host_addr_is_unspecified(const host_addr_t addr);
 
 static inline gboolean
 host_addr_is_ipv4_mapped(const host_addr_t addr)
 {
-	return NET_TYPE_IPV6 == host_addr_net(addr) && 
+	return host_addr_is_ipv6(addr) && 
 		host_addr_matches(addr, ipv6_ipv4_mapped, 96);
 }
 
-const gchar *host_addr_to_string(const host_addr_t addr);
-size_t host_addr_to_string_buf(const host_addr_t addr, gchar *, size_t);
-gboolean string_to_host_addr(const gchar *s, const gchar **endptr, host_addr_t *addr_ptr);
-const gchar *host_addr_port_to_string(const host_addr_t addr, guint16 port);
-const gchar *host_addr_port_to_string2(const host_addr_t addr, guint16 port);
+const char *host_addr_to_string(const host_addr_t ha);
+const char *host_addr_to_string2(const host_addr_t ha);
+size_t host_addr_to_string_buf(const host_addr_t addr, char *, size_t);
+gboolean string_to_host_addr(const char *s, const char **endptr,
+	host_addr_t *addr_ptr);
+const char *host_addr_port_to_string(const host_addr_t addr, guint16 port);
+const char *host_addr_port_to_string2(const host_addr_t addr, guint16 port);
 size_t host_addr_port_to_string_buf(const host_addr_t addr,
-				guint16 port, gchar *, size_t);
-gboolean string_to_host_addr_port(const gchar *str, const gchar **endptr,
+				guint16 port, char *, size_t);
+gboolean string_to_host_addr_port(const char *str, const char **endptr,
 	host_addr_t *addr_ptr, guint16 *port_ptr);
+gboolean
+string_to_port_host_addr(const char *str, const char **endptr,
+	guint16 *port_ptr, host_addr_t *addr_ptr);
+const char *host_port_to_string(const char *hostname,
+				host_addr_t addr, guint16 port);
+const char *port_host_addr_to_string(guint16 port, const host_addr_t ha);
+size_t host_port_addr_to_string_buf(guint16 port, const host_addr_t ha,
+	char *dst, size_t size);
 
-GSList *name_to_host_addr(const gchar *host, enum net_type net);
+GSList *name_to_host_addr(const char *host, enum net_type net);
 void host_addr_free_list(GSList **sl_ptr);
 
-host_addr_t name_to_single_host_addr(const gchar *host, enum net_type net);
+host_addr_t name_to_single_host_addr(const char *host, enum net_type net);
+#ifdef HAS_GETADDRINFO
+struct addrinfo;
+host_addr_t addrinfo_to_addr(const struct addrinfo *ai);
+#endif
 
-const gchar *host_addr_to_name(const host_addr_t addr);
-gboolean string_to_host_or_addr(const char *s, const gchar **endptr,
+const char *host_addr_to_name(const host_addr_t addr);
+gboolean string_to_host_or_addr(const char *s, const char **endptr,
 		host_addr_t *ha);
 
-GSList *host_addr_get_interface_addrs(void);
+GSList *host_addr_get_interface_addrs(enum net_type net);
 void host_addr_free_interface_addrs(GSList **sl_ptr);
 
 guint packed_host_addr_size(const struct packed_host_addr paddr);
@@ -545,9 +614,13 @@ host_addr_t packed_host_addr_unpack(const struct packed_host_addr paddr);
 
 guint packed_host_size(const struct packed_host paddr);
 struct packed_host host_pack(const host_addr_t addr, guint16 port);
-gboolean packed_host_unpack(const struct packed_host phost,
+void packed_host_unpack(const struct packed_host *phost,
 		host_addr_t *addr_ptr, guint16 *port_ptr);
 
+guint packed_host_hash_func(gconstpointer key);
+gboolean packed_host_eq_func(gconstpointer p, gconstpointer q);
+gpointer walloc_packed_host(const host_addr_t addr, guint16 port);
+void wfree_packed_host(gpointer key, gpointer unused_data);
 
 #endif /* _host_addr_h_ */
 /* vi: set ts=4 sw=4 cindent: */
