@@ -1,5 +1,5 @@
 /*
- * $Id: listener.h,v 1.8 2005/09/10 08:58:40 daichik Exp $
+ * $Id: listener.h 19113 2011-05-19 15:00:41Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -36,7 +36,7 @@
 #ifndef _listener_h_
 #define _listener_h_
 
-#include <glib.h>
+#include "common.h"
 
 /**
  * To use the macros below with a "node_added" signal for example,
@@ -50,7 +50,7 @@
  * You also need a special type defined which holds the signature of the
  * callback function. For example:
  *
- * typedef void (*node_added_listener_t) (gnutella_node_t *, const gchar *);
+ * typedef void (*node_added_listener_t) (gnutella_node_t *, const char *);
  *
  * Again the name is important (like above).
  */
@@ -59,14 +59,14 @@ typedef GSList *listeners_t;
 
 #define LISTENER_ADD(signal, callback) 										\
 G_STMT_START {																\
-	gpointer p = cast_func_to_gpointer((func_ptr_t) (callback));			\
+	void *p = func_to_pointer(callback);									\
 	g_assert(NULL != p);				 									\
 	CAT2(signal,_listeners) = g_slist_append(CAT2(signal,_listeners), p);	\
 } G_STMT_END
 
 #define LISTENER_REMOVE(signal, callback)									\
 G_STMT_START {																\
-	gpointer p = cast_func_to_gpointer((func_ptr_t) (callback));			\
+	void *p = func_to_pointer(callback);									\
 	g_assert(NULL != p);													\
 	CAT2(signal,_listeners) = g_slist_remove(CAT2(signal,_listeners), p);	\
 } G_STMT_END
@@ -77,7 +77,7 @@ G_STMT_START {																\
 	for (sl = CAT2(signal,_listeners); sl != NULL; sl = g_slist_next(sl)) { \
 		CAT2(signal,_listener_t) fn;										\
 		g_assert(NULL != sl->data);	  										\
-		fn = (CAT2(signal,_listener_t)) cast_gpointer_to_func(sl->data);	\
+		fn = (CAT2(signal,_listener_t)) cast_pointer_to_func(sl->data);	\
 		fn params;															\
 	}																		\
 } G_STMT_END
