@@ -1,5 +1,5 @@
 /*
- * $Id: pattern.c 11811 2006-08-29 21:24:49Z rmanfredi $
+ * $Id: pattern.c 19142 2011-05-23 20:44:04Z rmanfredi $
  *
  * Copyright (c) 2001-2004, Raphael Manfredi
  *
@@ -35,27 +35,21 @@
 
 #include "common.h"
 
-RCSID("$Id: pattern.c 11811 2006-08-29 21:24:49Z rmanfredi $")
+RCSID("$Id: pattern.c 19142 2011-05-23 20:44:04Z rmanfredi $")
 
 #include "misc.h"
 #include "pattern.h"
-#include "zalloc.h"
+#include "halloc.h"
+#include "walloc.h"
 #include "override.h"		/* Must be the last header included */
 
-static zone_t *pat_zone = NULL;		/**< Compiled patterns */
-
 /**
  * Initialize pattern data structures.
  */
 void
 pattern_init(void)
 {
-	/*
-	 * Patterns are not only used for query matching but also for filters,
-	 * therefore we can expect quite a few to be created at the same time.
-	 */
-
-	pat_zone = zget(sizeof(cpattern_t), 64);
+	/* Nothing to do */
 }
 
 /**
@@ -64,7 +58,7 @@ pattern_init(void)
 void
 pattern_close(void)
 {
-	zdestroy(pat_zone);
+	/* Nothing to do */
 }
 
 /*
@@ -83,17 +77,19 @@ pattern_close(void)
  * @return a compiled pattern structure.
  */
 cpattern_t *
-pattern_compile(const gchar *pattern)
+pattern_compile(const char *pattern)
 {
-	cpattern_t *p = zalloc(pat_zone);
-	size_t plen, i, *pd = p->delta;
+	cpattern_t *p;
+	size_t plen, i, *pd;
 	const guchar *c;
 
-	p->pattern = g_strdup(pattern);
+	WALLOC(p);
+	p->pattern = h_strdup(pattern);
 	p->len = plen = strlen(p->pattern);
 	p->duped = TRUE;
 
 	plen++;			/* Avoid increasing within the loop */
+	pd = p->delta;
 
 	for (i = 0; i < ALPHA_SIZE; i++)
 		*pd++ = plen;
@@ -115,17 +111,19 @@ pattern_compile(const gchar *pattern)
  * NB: There is no pattern_free_fast(), just call zfree() on the result.
  */
 cpattern_t *
-pattern_compile_fast(const gchar *pattern, size_t plen)
+pattern_compile_fast(const char *pattern, size_t plen)
 {
-	cpattern_t *p = zalloc(pat_zone);
-	size_t i, *pd = p->delta;
+	cpattern_t *p;
+	size_t i, *pd;
 	const guchar *c;
 
+	WALLOC(p);
 	p->pattern = pattern;
 	p->len = plen;
 	p->duped = FALSE;
 
 	plen++;			/* Avoid increasing within the memset() inlined macro */
+	pd = p->delta;
 
 	for (i = 0; i < ALPHA_SIZE; i++)
 		*pd++ = plen;
@@ -146,10 +144,10 @@ void
 pattern_free(cpattern_t *cpat)
 {
 	if (cpat->duped) {
-		g_free(deconstify_gchar(cpat->pattern));
-		cpat->pattern = NULL;
+		hfree(deconstify_gchar(cpat->pattern));
+		cpat->pattern = NULL; /* Don't use HFREE_NULL b/c of lvalue cast */
 	}
-	zfree(pat_zone, cpat);
+	WFREE(cpat);
 }
 
 /**
@@ -159,19 +157,19 @@ pattern_free(cpattern_t *cpat)
  *
  * @return pointer to beginning of matching substring, NULL if not found.
  */
-const gchar *
+const char *
 pattern_qsearch(
 	cpattern_t *cpat,		/**< Compiled pattern */
-	const gchar *text,		/**< Text we're scanning */
+	const char *text,		/**< Text we're scanning */
 	size_t tlen,			/**< Text length, 0 = compute strlen(text) */
 	size_t toffset,			/**< Offset within text for search start */
 	qsearch_mode_t word)	/**< Beginning/whole word matching? */
 {
-	const gchar *p;			/* Pointer within string pattern */
-	const gchar *t;			/* Pointer within text */
-	const gchar *tp;		/* Initial local search text pointer */
-	const gchar *start;		/* Start of matching */
-	const gchar *end;		/* End of text (first byte after physical end) */
+	const char *p;			/* Pointer within string pattern */
+	const char *t;			/* Pointer within text */
+	const char *tp;			/* Initial local search text pointer */
+	const char *start;		/* Start of matching */
+	const char *end;		/* End of text (first byte after physical end) */
 	size_t i;				/* Position within pattern string */
 	size_t plen;
 
