@@ -1,5 +1,5 @@
 /*
- * $Id: slist.h 12689 2006-12-29 13:26:50Z rmanfredi $
+ * $Id: slist.h 19185 2011-05-29 21:25:28Z rmanfredi $
  *
  * Copyright (c) 2003, Christian Biere
  *
@@ -32,6 +32,8 @@
 typedef struct slist_iter slist_iter_t;
 typedef struct slist slist_t;
 
+typedef void (*slist_destroy_cb)(void *data);
+
 slist_t *slist_new(void);
 void slist_free(slist_t **slist_ptr);
 gboolean slist_remove(slist_t *slist, gpointer key);
@@ -49,13 +51,22 @@ gboolean slist_contains(const slist_t *slist, gconstpointer key,
 gboolean slist_contains_identical(const slist_t *slist, gconstpointer key);
 void slist_foreach(const slist_t *slist, GFunc func, gpointer user_data);
 
-slist_iter_t *slist_iter_on_head(slist_t *slist);
-slist_iter_t *slist_iter_before_head(slist_t *slist);
+slist_iter_t *slist_iter_on_head(const slist_t *slist);
+slist_iter_t *slist_iter_before_head(const slist_t *slist);
+slist_iter_t * slist_iter_removable_on_head(slist_t *slist);
+slist_iter_t * slist_iter_removable_before_head(slist_t *slist);
 void slist_iter_free(slist_iter_t **iter_ptr);
 gboolean slist_iter_has_item(const slist_iter_t *iter);
 gboolean slist_iter_has_next(const slist_iter_t *iter);
 gpointer slist_iter_next(slist_iter_t *iter);
 gpointer slist_iter_current(const slist_iter_t *iter);
 void slist_iter_remove(slist_iter_t *iter);
+void slist_free_all(slist_t **slist_ptr, slist_destroy_cb freecb);
+
+static inline slist_destroy_cb
+cast_to_slist_destroy(const func_ptr_t fn)
+{
+	return (slist_destroy_cb) fn;
+}
 
 #endif	/* _slist_h_ */
