@@ -1,5 +1,5 @@
 /*
- * $Id: url.c 12290 2006-11-09 22:07:21Z cbiere $
+ * $Id: url.c 19325 2011-07-07 12:48:24Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,16 +35,21 @@
 
 #include "common.h"
 
-RCSID("$Id: url.c 12290 2006-11-09 22:07:21Z cbiere $")
+RCSID("$Id: url.c 19325 2011-07-07 12:48:24Z rmanfredi $")
 
+#include "ascii.h"
+#include "debug.h"
 #include "glib-missing.h"
+#include "halloc.h"
 #include "host_addr.h"
+#include "misc.h"			/* For is_strprefix() */
+#include "parse.h"
+#include "path.h"
+#include "str.h"
 #include "url.h"
-#include "misc.h"
 #include "walloc.h"
-#include "override.h"		/* Must be the last header included */
 
-extern guint32 lib_debug;
+#include "override.h"		/* Must be the last header included */
 
 #define ESCAPE_CHAR		'%'
 
@@ -56,30 +61,47 @@ extern guint32 lib_debug;
  * - Bit 0 encodes regular transparent set (pathnames, '/' is transparent).
  * - Bit 1 encodes regular transparent set minus '+' (query string).
  * - Bit 2 encodes the set for fixing an incomplete escaping.
+ * - Bit 3 encodes the set for use in a shell.
  */
 static const guint8 is_transparent[96] = {
-/*  0 1 2 3 4 5 6 7 8 9 a b c d e f */	/* 0123456789abcdef -            */
-    0,7,0,0,7,0,4,7,7,7,7,0,7,7,7,7,	/*  !"#$%&'()*+,-./ -  32 -> 47  */
-    7,7,7,7,7,7,7,7,7,7,4,0,0,4,0,4,	/* 0123456789:;<=>? -  48 -> 63  */
-    0,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,	/* @ABCDEFGHIJKLMNO -  64 -> 79  */
-    7,7,7,7,7,7,7,7,7,7,7,0,0,0,0,7,	/* PQRSTUVWXYZ[\]^_ -  80 -> 95  */
-    0,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,	/* `abcdefghijklmno -  96 -> 111 */
-    7,7,7,7,7,7,7,7,7,7,7,0,0,0,0,0,	/* pqrstuvwxyz{|}~  - 112 -> 127 */
+/*  0   1   2   3   4   5   6   7 */	/* 01234567 -            */
+    0x0,0x0,0x0,0x0,0x7,0x0,0x4,0x0,	/*  !"#$%&' -	 32..39  */
+	0x7,0x7,0x7,0x0,0xf,0xf,0xf,0xf,	/* ()*+,-./ -	 40..47	 */
+    0xf,0xf,0xf,0xf,0xf,0xf,0xf,0xf,	/* 01234567 -	 48..55  */
+	0xf,0xf,0x6,0x0,0x0,0x4,0x0,0x4,	/* 89:;<=>? -    56..63  */
+    0x0,0xf,0xf,0xf,0xf,0xf,0xf,0xf,	/* @ABCDEFG -	 64..71	 */
+	0xf,0xf,0xf,0xf,0xf,0xf,0xf,0xf,	/* HIJKLMNO - 	 72..79  */
+    0xf,0xf,0xf,0xf,0xf,0xf,0xf,0xf,	/* PQRSTUVW -    80..87	 */
+	0xf,0xf,0xf,0x4,0x0,0x4,0x0,0xf,	/* XYZ[\]^_ -	 88..95  */
+    0x0,0xf,0xf,0xf,0xf,0xf,0xf,0xf,	/* `abcdefg	-	 96..103 */
+	0xf,0xf,0xf,0xf,0xf,0xf,0xf,0xf,	/* hijklmno -	104..111 */
+    0xf,0xf,0xf,0xf,0xf,0xf,0xf,0xf,	/* pqrstuvw -   112..119 */
+	0xf,0xf,0xf,0x0,0x0,0x0,0x0,0x0,	/* xyz{|}~  -	120..127 */
 };
 
 enum escape_mask {
 	PATH_MASK	= (1 << 0),
 	QUERY_MASK	= (1 << 1),
-	FIX_MASK	= (1 << 2)
+	FIX_MASK	= (1 << 2),
+	SHELL_MASK	= (1 << 3)
 };
 
 static inline gboolean
-is_transparent_char(const gint c, const enum escape_mask m)
+is_transparent_char(const int c, const enum escape_mask m)
 {
 	return c >= 32 && c < 128 && (is_transparent[c - 32] & (guint8) m);
 }
 
 static const char hex_alphabet[] = "0123456789ABCDEF";
+static const char http_prefix[] = "http://";
+
+/**
+ * Parsed URL parameters (from query string).
+ */
+struct url_params {
+	GHashTable *params;		/**< parameter => value (halloc'ed) */
+	size_t count;			/**< Amount of parameters */
+};
 
 /**
  * Escape undesirable characters using %xx, where xx is an hex code.
@@ -87,16 +109,17 @@ static const char hex_alphabet[] = "0123456789ABCDEF";
  * @param `url' no brief description.
  * @param `mask' tells us whether we're escaping an URL path or a query string.
  *
- * @return argument if no escaping is necessary, or a new string otherwise.
+ * @return argument if no escaping is necessary, or a new string otherwise,
+ * which must be freed via hfree().
  */
-static gchar *
-url_escape_mask(const gchar *url, guint8 mask)
+static char *
+url_escape_mask(const char *url, guint8 mask)
 {
-	const gchar *p;
-	gchar *q;
+	const char *p;
+	char *q;
 	int need_escape = 0;
 	guchar c;
-	gchar *new;
+	char *new;
 
 	for (p = url, c = *p++; c; c = *p++)
 		if (!is_transparent_char(c, mask))
@@ -105,7 +128,7 @@ url_escape_mask(const gchar *url, guint8 mask)
 	if (need_escape == 0)
 		return deconstify_gchar(url);
 
-	new = g_malloc(p - url + (need_escape << 1));
+	new = halloc(p - url + (need_escape << 1));
 
 	for (p = url, q = new, c = *p++; c; c = *p++) {
 		if (is_transparent_char(c, mask))
@@ -129,13 +152,13 @@ url_escape_mask(const gchar *url, guint8 mask)
  * @return amount of characters written into buffer (not counting trailing
  * NUL), or -1 if the buffer was too small.
  */
-static gint
-url_escape_mask_into(const gchar *url, gchar *target, gint len, guint8 mask)
+static int
+url_escape_mask_into(const char *url, char *target, int len, guint8 mask)
 {
-	const gchar *p = url;
-	gchar *q;
+	const char *p = url;
+	char *q;
 	guchar c;
-	gchar *end = target + len;
+	char *end = target + len;
 
 	for (q = target, c = *p++; c && q < end; c = *p++) {
 		if (is_transparent_char(c, mask))
@@ -163,23 +186,55 @@ url_escape_mask_into(const gchar *url, gchar *target, gint len, guint8 mask)
  *
  * @return argument if no escaping is necessary, or a new string otherwise.
  */
-gchar *
-url_escape(const gchar *url)
+char *
+url_escape(const char *url)
 {
 	return url_escape_mask(url, PATH_MASK);
 }
 
 /**
- * Same as url_escape(), but '+' are also escaped for the query string.
+ * Same as url_escape() except for:
+ *
+ * '+' is also escaped for the query string.
+ * ':' is not escaped.
  *
  * @return argument if no escaping is necessary, or a new string otherwise.
  */
-gchar *
-url_escape_query(const gchar *url)
+char *
+url_escape_query(const char *url)
 {
 	return url_escape_mask(url, QUERY_MASK);
 }
 
+/**
+ * Escapes the given string for safe use in a shell or file: URL.
+ * @return argument if no escaping is necessary, or a new string otherwise.
+ */
+char *
+url_escape_shell(const char *url)
+{
+	return url_escape_mask(url, SHELL_MASK);
+}
+
+/**
+ * Creates a file: URL from the given absolute path and escapes it properly
+ * for safe use in a shell too.
+ * @return A new string or NULL on failure.
+ */
+char *
+url_from_absolute_path(const char *path)
+{
+	char *escaped, *url;
+	
+	g_return_val_if_fail(is_absolute_path(path), NULL);
+	escaped = url_escape_mask(path, SHELL_MASK);
+	url = g_strconcat("file://", escaped, (void *) 0);
+	if (escaped != path) {
+		HFREE_NULL(escaped);
+	}
+	return url;
+}
+
 /**
  * Escape undesirable characters using %xx, where xx is an hex code.
  * This is done in the `target' buffer, whose size is `len'.
@@ -187,41 +242,43 @@ url_escape_query(const gchar *url)
  * @return amount of characters written into buffer (not counting trailing
  * NUL), or -1 if the buffer was too small.
  */
-gint
-url_escape_into(const gchar *url, gchar *target, gint len)
+int
+url_escape_into(const char *url, char *target, int len)
 {
 	return url_escape_mask_into(url, target, len, PATH_MASK);
 }
 
 /**
- * Don't touch '?', '&', '=', ':', %HH.
+ * Don't touch '?', '&', '=', ':', '[', ']', %HH.
+ *
+ * @return newly allocated string through halloc().
  */
-gchar *
-url_fix_escape(const gchar *url)
+char *
+url_fix_escape(const char *url)
 {
-	const gchar *p;
-	GString *gs;
+	const char *p;
+	str_t *s;
 	guchar c;
 
-	gs = g_string_new(NULL);
+	s = str_new(0);
 
 	for (p = url; '\0' != (c = *p); p++) {
 		if (
 			is_transparent_char(c, FIX_MASK) ||
 			('%' == c && is_ascii_xdigit(p[1]) && is_ascii_xdigit(p[2]))
 		) {
-			gs = g_string_append_c(gs, c);
+			str_putc(s, c);
 		} else {
-			gchar buf[3];
+			char buf[3];
 
 			buf[0] = ESCAPE_CHAR;
 			buf[1] = hex_alphabet[c >> 4];
 			buf[2] = hex_alphabet[c & 0xf];
-			gs = g_string_append_len(gs, buf, sizeof buf);
+			str_cat_len(s, buf, sizeof buf);
 		}
 	}
 
-	return gm_string_finalize(gs);
+	return str_s2c_null(&s);
 }
 
 /**
@@ -229,36 +286,40 @@ url_fix_escape(const gchar *url)
  *
  * @return argument if no escaping is necessary, or a new string otherwise.
  */
-gchar *
-url_escape_cntrl(gchar *url)
+char *
+url_escape_cntrl(const char *url)
 {
-	gchar *p;
-	gchar *q;
-	int need_escape = 0;
-	guchar c;
-	gchar *new;
+	size_t need_escape = 0;
+	const char *p;
 
-	for (p = url, c = *p++; c; c = *p++)
-		if (iscntrl(c) || c == ESCAPE_CHAR)
+	for (p = url; '\0' != *p; p++) {
+		if (is_ascii_cntrl(*p) || ESCAPE_CHAR == *p)
 			need_escape++;
-
-	if (need_escape == 0)
-		return url;
-
-	new = g_malloc(p - url + (need_escape << 1));
-
-	for (p = url, q = new, c = *p++; c; c = *p++) {
-		if (!iscntrl(c) && c != ESCAPE_CHAR)
-			*q++ = c;
-		else {
-			*q++ = ESCAPE_CHAR;
-			*q++ = hex_alphabet[c >> 4];
-			*q++ = hex_alphabet[c & 0xf];
-		}
 	}
-	*q = '\0';
 
-	return new;
+	if (need_escape > 0) {
+		char *escaped, *q;
+		size_t size;
+		guchar c;
+
+		size = p - url + 1 + need_escape * 2;
+		escaped = halloc(size);
+		q = escaped;
+
+		for (p = url; '\0' != (c = *p); p++) {
+			if (!is_ascii_cntrl(c) && ESCAPE_CHAR != c)
+				*q++ = c;
+			else {
+				*q++ = ESCAPE_CHAR;
+				*q++ = hex_alphabet[c >> 4];
+				*q++ = hex_alphabet[c & 0xf];
+			}
+		}
+		*q = '\0';
+		return escaped;
+	} else {
+		return deconstify_gchar(url);
+	}
 }
 
 /**
@@ -269,15 +330,15 @@ url_escape_cntrl(gchar *url)
  *
  * @return NULL if the argument isn't valid encoded.
  */
-gchar *
-url_unescape(gchar *url, gboolean inplace)
+char *
+url_unescape(char *url, gboolean inplace)
 {
-	gchar *p;
-	gchar *q;
-	gint need_unescape = 0;
+	char *p;
+	char *q;
+	int need_unescape = 0;
 	guint unescaped_memory = 0;
 	guchar c;
-	gchar *new;
+	char *new;
 
 	for (p = url; (c = *p) != '\0'; c = *p++)
 		if (c == ESCAPE_CHAR) {
@@ -297,7 +358,7 @@ url_unescape(gchar *url, gboolean inplace)
 		return url;
 
 	/*
-	 * The "+ 1" in the g_malloc() call below is for the impossible case where
+	 * The "+ 1" in the halloc() call below is for the impossible case where
 	 * the string would finish on a truncated escape sequence.  In that
 	 * case, we would not have enough room for the final trailing NUL.
 	 */
@@ -306,7 +367,7 @@ url_unescape(gchar *url, gboolean inplace)
 		new = url;
 	else {
 		unescaped_memory = p - url - (need_unescape << 1) + 1;
-		new = g_malloc(unescaped_memory);
+		new = halloc(unescaped_memory);
 	}
 
 	for (p = url, q = new, c = *p++; c; c = *p++) {
@@ -314,7 +375,7 @@ url_unescape(gchar *url, gboolean inplace)
 			*q++ = c;
 		else {
 			if ((c = *p++)) {
-				gint v = hex2int_inline(c) << 4;
+				int v = hex2int_inline(c) << 4;
 				if ((c = *p++))
 					v += hex2int_inline(c);
 				else
@@ -344,33 +405,33 @@ url_unescape(gchar *url, gboolean inplace)
  *         or NULL if the argument isn't valid encoded.
  */
 url_params_t *
-url_params_parse(gchar *query)
+url_params_parse(char *query)
 {
 	url_params_t *up;
-	gchar *q;
-	gchar *start;
-	gchar *name = NULL;
-	gchar *value = NULL;
+	char *q;
+	char *start;
+	char *name = NULL;
+	char *value = NULL;
 	gboolean in_value = FALSE;
 
-	up = walloc(sizeof *up);
+	WALLOC(up);
 	up->params = g_hash_table_new(g_str_hash, g_str_equal);
 	up->count = 0;
 
 	for (q = start = query; /* empty */; q++) {
-		gchar c = *q;
+		char c = *q;
 
 		if (in_value) {
 			if (c == '&' || c == '\0') {		/* End of value */
 				*q = '\0';
 				value = url_unescape(start, FALSE);
 				if (!value) {
-					G_FREE_NULL(name);
+					HFREE_NULL(name);
 					url_params_free(up);
 					return NULL;
 				}
 				if (value == start)				/* No unescaping took place */
-					value = g_strdup(start);
+					value = h_strdup(start);
 				*q = c;
 				g_hash_table_insert(up->params, name, value);
 				up->count++;
@@ -382,7 +443,7 @@ url_params_parse(gchar *query)
 		} else {
 			if (c == '=') {						/* End of parameter name */
 				*q = '\0';
-				name = g_strdup(start);
+				name = h_strdup(start);
 				*q = c;
 				in_value = TRUE;
 				start = q + 1;					/* Value will start there */
@@ -403,8 +464,8 @@ url_params_parse(gchar *query)
  * Get the value of a parameter, or NULL if the parameter is not present.
  * The value returned has already been URL-unescaped.
  */
-const gchar *
-url_params_get(url_params_t *up, const gchar *name)
+const char *
+url_params_get(const url_params_t *up, const char *name)
 {
 	g_assert(up != NULL);
 	g_assert(up->params != NULL);
@@ -416,8 +477,8 @@ static void
 free_params_kv(gpointer key, gpointer value, gpointer unused_udata)
 {
 	(void) unused_udata;
-	g_free(key);
-	g_free(value);
+	HFREE_NULL(key);
+	HFREE_NULL(value);
 }
 
 /**
@@ -429,13 +490,22 @@ url_params_free(url_params_t *up)
 	g_assert(up != NULL);
 
 	g_hash_table_foreach(up->params, free_params_kv, NULL);
-	g_hash_table_destroy(up->params);
+	gm_hash_table_destroy_null(&up->params);
 
-	wfree(up, sizeof *up);
+	WFREE(up);
+}
+
+/**
+ * How many URL parameters are present?
+ */
+size_t
+url_params_count(const url_params_t *up)
+{
+	return up->count;
 }
 
 static gboolean
-url_safe_char(gchar c, url_policy_t p)
+url_safe_char(char c, url_policy_t p)
 {
 	if (!isascii(c) || is_ascii_cntrl(c))
 		return FALSE;
@@ -446,8 +516,7 @@ url_safe_char(gchar c, url_policy_t p)
 			!is_ascii_digit(c) &&
 			NULL == strchr("/._-~", c)
 		) {
-			if (lib_debug)
-      			g_warning("Unsafe character in GWC URL; rejected");
+      		/* Unsafe character in GWC URL; rejected */
       		return FALSE;
     	}
 	}
@@ -455,13 +524,72 @@ url_safe_char(gchar c, url_policy_t p)
 	return TRUE;
 }
 
+/**
+ * Creates the canonical representation of a path in place, delimited by '/'.
+ *
+ * @param path a NUL-terminated string representing the input path.
+ * @return zero on sucess, non-zero on failure.
+ */
+int
+url_canonize_path(char *path)
+{
+	const char *p;
+	char c, *q, *ep, *dst = path;
+
+	g_assert(path);
+
+	/* Scan path */
+	for (p = path, q = dst; '\0' != (c = *p); q++, p++) {
+
+		/* Handle relative paths i.e., /. and /.. */
+		if ('/' != c) {
+			*q = c;
+			continue;
+		}
+
+		/* Special handling for '/' follows */
+
+		do {
+			*q = '/';
+
+			while ('/' == p[1]) {
+				p++;
+			}
+
+			if (0 == strcmp(p, "/.")) {
+				/* Ignoring trailing "/." in URI */
+				p++;
+			} else if (0 == strcmp(p, "/..")) {
+				return -1;
+			} else if (NULL != (ep = is_strprefix(p, "/./"))) {
+				/* Ignoring unnecessary "/./" in URI */
+				p = ep - 1;
+			} else if (NULL != (ep = is_strprefix(p, "/../"))) {
+				p = ep - 1;
+
+				/* Ascending one component in URI */
+				do {
+					if (q == dst)
+						return -1; /* beyond root */
+				} while ('/' != *--q);
+			} else {
+				break;
+			}
+		} while ('/' == p[0] && ('/' == p[1] || '.' == p[1]));
+	}
+
+	*q = '\0';
+
+	return 0;
+}
+
 /**
  * @attention
- * NB: May modify ``url'' in all cased; pass a copy if necessary!
+ * NB: May modify ``url'' in all cases; pass a copy if necessary!
  *
  * @returns NULL if ``url'' isn't a valid resp. allowed URL. Otherwise,
- * it returns either a pointer to the original URL or a g_malloc()ed
- * modified copy which has to be g_free()ed.
+ * it returns either a pointer to the original URL or a newly allocated
+ * string holding a modified copy.  Use hfree() to free the allocated string.
  *
  * The URL is validated according to the specified policy. Unnecessary
  * parts such as ":80" (for the port) and "/./" are removed: the hostname
@@ -469,21 +597,18 @@ url_safe_char(gchar c, url_policy_t p)
  * This allows comparing different variants of the same URL to detect
  * duplicates.
  */
-gchar *
-url_normalize(gchar *url, url_policy_t pol)
+char *
+url_normalize(char *url, url_policy_t pol)
 {
-	static const char http_prefix[] = "http://";
-	const gchar *p, *uri, *endptr, *tld = NULL, *warn = NULL;
-	gchar c, *q;
+	const char *p, *uri, *endptr, *tld = NULL, *warn = NULL;
+	char c, *q;
 	host_addr_t addr;
 
 	g_assert(url);
 
-	if (NULL == (q = is_strcaseprefix(url, http_prefix))) {
-		if (lib_debug)
-			g_warning("URL \"%s\" isn't preceded by \"%s\"", url, http_prefix);
+	if (NULL == (q = is_strcaseprefix(url, http_prefix)))
 		return NULL;
-	}
+
 	/* Make sure the prefix is all-lowercase */
 	memcpy(url, http_prefix, CONST_STRLEN(http_prefix));
 
@@ -538,10 +663,6 @@ url_normalize(gchar *url, url_policy_t pol)
 			} else if ('\0' == c || '/' == c || ':' == c) {
 				break;
 			} else {
-				if (lib_debug)
-					g_warning("Bad URL \"%s\": "
-						"invalid character '%c' in ``host:port'' part.",
-						url, is_ascii_print(c) ? c : '?');
 				return NULL;
 			}
 		}
@@ -556,7 +677,7 @@ url_normalize(gchar *url, url_policy_t pol)
 	p = q;
 	if (':' == *q) {
 		guint32 port;
-		gint error;
+		int error;
 
 		q++; /* Skip ':' */
 
@@ -579,7 +700,7 @@ url_normalize(gchar *url, url_policy_t pol)
 			80 != port &&
 			443 != port
 		) {
-			warn = "Ports below 1024 other than 80 and 443 are disallowed";
+			warn = "ports below 1024 other than 80 and 443 are disallowed";
 			goto bad;
 		}
 
@@ -617,14 +738,14 @@ url_normalize(gchar *url, url_policy_t pol)
 	}
 	uri = q;
 
-	if (0 != canonize_path(q, q)) {
+	if (0 != url_canonize_path(q)) {
 		warn = "Could not canonize URI";
 		goto bad;
 	}
 
 	if (!(URL_POLICY_ALLOW_STATIC_FILES & pol)) {
 		static const struct {
-			const gchar *ext;
+			const char *ext;
 			size_t len;
 		} static_types[] = {
 #define D(x) { (x), CONST_STRLEN(x) }
@@ -648,25 +769,116 @@ url_normalize(gchar *url, url_policy_t pol)
 	/* Add a trailing slash; if the URI is empty (to prevent dupes) */
 	if ('\0' == uri[0]) {
 		ssize_t len = q - url;
-		gchar *s;
+		char *s;
 
 		g_assert(len > 0);
-		s = g_malloc(len + sizeof "/");
+		s = halloc(len + sizeof "/");
 		memcpy(s, url, len);
 		s[len] = '/';
 		s[len + 1] = '\0';
 		url = s;
 	}
 
-	if (lib_debug > 1)
-		g_message("url=\"%s\"", url);
-
 	return url;
 
 bad:
-	if (lib_debug)
-		g_warning("invalid URL \"%s\": %s", url, warn);
+	if (common_dbg)
+		g_warning("%s() rejecting URL \"%s\": %s", G_STRFUNC, url, warn);
+
 	return NULL;
 }
 
+/**
+ * Is URL absolute?
+ */
+gboolean
+url_is_absolute(const char *url)
+{
+	return is_strprefix(url, http_prefix) != NULL;
+}
+
+/**
+ * Interpret possible relative URL in the context of a base (absolute) one.
+ *
+ * If the relative URL is actually absolute, return it.  Otherwise returns
+ * a newly allocated string that must tbe freed with hfree().
+ *
+ * @param base		the base URL
+ * @param relative	the supposedly relative URL
+ *
+ * @return the relative URL if absolute, otherwise a newly allocated string.
+ */
+char *
+url_absolute_within(const char *base, const char *relative)
+{
+	char *dbase = NULL;
+
+	g_assert(url_is_absolute(base));
+
+	if (url_is_absolute(relative))
+		return deconstify_char(relative);		/* Was already absolute */
+
+	if (is_strprefix(relative, "//"))
+		/* Was missing the scheme */
+		return h_strconcat("http:", relative, NULL);
+
+	if (is_strprefix(relative, "#")) {
+		char *p;
+
+		if (NULL == (p = strchr(base, '#'))) {
+			return h_strconcat(base, relative, NULL);
+		} else {
+			/* Replace the fragment */
+			dbase = h_strdup(base);
+			p = dbase + (p - base);
+			g_assert(*p == '#');
+			*p = '\0';
+			goto concatenate;
+		}
+	}
+
+	if (is_strprefix(relative, "/")) {
+		char *p;
+		
+		dbase = h_strdup(base);
+		p = is_strprefix(dbase, http_prefix);
+		g_assert(p != NULL);		/* base was absolute */
+		p = strchr(p, '/');
+
+		if (NULL == p) {
+			/* base was "http://host" with no trailing path */
+			hfree(dbase);
+			return h_strconcat(base, relative, NULL);
+		} else {
+			/* replace relative path in the base */
+			*p = '\0';
+			goto concatenate;
+		}
+	} else {
+		char *p;
+
+		/*
+		 * Find the last '/' in the base and append the relative URL after it.
+		 */
+
+		dbase = h_strdup(base);
+		p = strrchr(dbase, '/');
+		g_assert(p != NULL);		/* base was absolute */
+		*(++p) = '\0';				/* truncate string after last '/' */
+
+		/* FALL THROUGH */
+	}
+
+concatenate:
+	{
+		char *result;
+
+		g_assert(dbase != NULL);
+
+		result = h_strconcat(dbase, relative, NULL);
+		hfree(dbase);
+		return result;
+	}
+}
+
 /* vi: set ts=4 sw=4 cindent: */
