@@ -1,5 +1,5 @@
 /*
- * $Id: url.h 10893 2006-05-17 10:39:11Z cbiere $
+ * $Id: url.h 18239 2011-01-09 21:24:44Z cbiere $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -36,17 +36,10 @@
 #ifndef _url_h_
 #define _url_h_
 
-#include <glib.h>
+#include "common.h"
 
-/**
- * Parsed URL parameters (from query string).
- */
-typedef struct {
-	GHashTable *params;		/**< parameter => value (malloc'ed) */
-	gint count;				/**< Amount of parameters */
-} url_params_t;
-
-#define url_params_count(x)	((x)->count)
+struct url_params;
+typedef struct url_params url_params_t;
 
 typedef enum {
 	URL_POLICY_ALLOW_IP_AS_HOST		= (1 << 0),
@@ -62,17 +55,24 @@ typedef enum {
  * Public interface.
  */
 
-gchar *url_escape(const gchar *url);
-gchar *url_escape_query(const gchar *url);
-gchar *url_fix_escape(const gchar *url);
-gint url_escape_into(const gchar *url, gchar *target, gint len);
-gchar *url_escape_cntrl(gchar *url);
-gchar *url_unescape(gchar *url, gboolean inplace);
+char *url_escape(const char *url);
+char *url_escape_query(const char *url);
+char *url_escape_shell(const char *url);
+char *url_fix_escape(const char *url);
+int url_escape_into(const char *url, char *target, int len);
+char *url_escape_cntrl(const char *url);
+char *url_unescape(char *url, gboolean inplace);
+char *url_from_absolute_path(const char *path);
 
-url_params_t *url_params_parse(gchar *query);
-const gchar *url_params_get(url_params_t *up, const gchar *name);
+url_params_t *url_params_parse(char *query);
+const char *url_params_get(const url_params_t *up, const char *name);
+size_t url_params_count(const url_params_t *up);
 void url_params_free(url_params_t *up);
-gchar *url_normalize(gchar *url, url_policy_t pol);
+char *url_normalize(char *url, url_policy_t pol);
+
+gboolean url_is_absolute(const char *url);
+char *url_absolute_within(const char *base, const char *relative);
+int url_canonize_path(char *path);
 
 #endif	/* _url_h_ */
 
