@@ -1,5 +1,5 @@
 /*
- * $Id: vendors.c,v 1.9 2005/06/29 14:24:27 daichik Exp $
+ * $Id: vendors.c 17427 2010-05-07 20:53:46Z cbiere $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -35,16 +35,18 @@
 
 #include "common.h"
 
-RCSID("$Id: vendors.c,v 1.9 2005/06/29 14:24:27 daichik Exp $");
+RCSID("$Id: vendors.c 17427 2010-05-07 20:53:46Z cbiere $")
 
 #include "vendors.h"
+#include "ascii.h"
 #include "endian.h"
+#include "glib-missing.h"	/* For g_strlcpy() */
 #include "misc.h"
 #include "override.h"	/* Must be the last header included */
 
-struct vendor {
+static const struct vendor {
     guint32 code;
-    gchar *name;
+    const char *name;
 } vendor_map[] = {
 	/* This array MUST be sorted, because it is searched dichotomically */
 
@@ -64,6 +66,7 @@ struct vendor {
     { T_FEVR, "FileFever" },
     { T_FIRE, "FireFly" },
     { T_FISH, "PEERanha" },
+    { T_FOXY, "Foxy" },
     { T_FZZN, "Fuzzon" },
     { T_GDNA, "Gnucleus DNA" },
     { T_GIFT, "giFT" },
@@ -74,6 +77,8 @@ struct vendor {
     { T_GNUC, "Gnucleus" },
     { T_GNUM, "Gnuminous" },
     { T_GNUT, "Gnut" },
+    { T_GNZL, "Gnoozle" },
+    { T_GSHR, "GnuShare" },
     { T_GTKG, "gtk-gnutella" },
     { T_HSLG, "Hagelslag" },
     { T_HUIT, "Huitella" },
@@ -104,7 +109,9 @@ struct vendor {
     { T_QTEL, "Qtella" },
     { T_RASP, "Rasputin" },
     { T_RAZA, "Shareaza" },
+    { T_RAZB, "Shareaza Beta" },
     { T_SHNB, "Shinobu" },
+    { T_SNOW, "FrostWire" },
     { T_SNUT, "SwapNut" },
     { T_STRM, "Storm" },
     { T_SWAP, "Swapper" },
@@ -127,13 +134,9 @@ struct vendor {
  *
  * @returns vendor string if found, NULL otherwise.
  */
-static gchar *
-find_vendor(guchar raw[4])
+static const char *
+find_vendor(guint32 code)
 {
-	guint32 code;
-
-    READ_GUINT32_BE(raw, code);
-
 #define GET_KEY(i) (vendor_map[(i)].code)
 #define FOUND(i) G_STMT_START { \
 	return vendor_map[(i)].name; \
@@ -142,8 +145,8 @@ find_vendor(guchar raw[4])
 
 	BINARY_SEARCH(guint32, code, G_N_ELEMENTS(vendor_map), VENDOR_CODE_CMP,
 		GET_KEY, FOUND);
-	
-#undef FOUND 
+
+#undef FOUND
 #undef GET_KEY
 	return NULL; /* not found */
 }
@@ -152,38 +155,74 @@ find_vendor(guchar raw[4])
  * @return true is gtk-gnutella knows the given 4-byte vendor code.
  */
 gboolean
-is_vendor_known(guchar raw[4])
+is_vendor_known(vendor_code_t code)
 {
-    if (raw[0] == '\0')
+    if (code.u32 == T_0000)
         return FALSE;
 
-	return find_vendor(raw) != NULL;
+	return find_vendor(code.u32) != NULL;
+}
+
+/**
+ * @return TRUE If the 4-byte vendor code is acceptable.
+ */
+gboolean
+is_vendor_acceptable(vendor_code_t code)
+{
+	char temp[4];
+
+	memcpy(temp, &code.u32, 4);
+	return is_ascii_alnum(temp[0]) &&
+		is_ascii_alnum(temp[1]) &&
+		is_ascii_alnum(temp[2]) &&
+		is_ascii_alnum(temp[3]);
 }
 
 /**
  * Make up a printable version of the vendor code.
  *
+ * @param code A 4-letter Gnutella vendor ID in host-endian order thus
+ *        after peek_be32() or ntohl().
+ * @param buf  The destination buffer to hold the string.
+ * @param size The size of buf in bytes.
+ *
+ * @return Length of the resulting string before potential truncation.
+ */
+size_t
+vendor_code_to_string_buf(guint32 code, char *buf, size_t size)
+{
+    if (code == 0) {
+		return g_strlcpy(buf, "null", size);
+	} else {
+		char temp[5];
+		size_t i;
+
+		poke_be32(&temp[0], code);
+
+		for (i = 0; i < G_N_ELEMENTS(temp) - 1; i++) {
+			if (!is_ascii_print(temp[i]))
+				temp[i] = '.';
+		}
+		temp[4] = '\0';
+		return g_strlcpy(buf, temp, size);
+	}
+}
+
+/**
+ * Make up a printable version of the vendor code.
+ *
+ * @param code A 4-letter Gnutella vendor ID in host-endian order thus
+ *        after peek_be32() or ntohl().
+ *
  * @return pointer to static data.
  */
-gchar *
-vendor_code_str(guint32 code)
+const char *
+vendor_code_to_string(guint32 code)
 {
-	static gchar temp[5];
-    gint i;
+	static char buf[5];
 
-	if (code == 0)
-		return "null";
-
-	WRITE_GUINT32_BE(code, temp);
-	temp[4] = '\0';
-
-	for (i = 0; i < 4; i++) {
-        guchar c = temp[i];
-		if (!is_ascii_print(c))
-			temp[i] = '.';
-	}
-
-	return temp;
+	vendor_code_to_string_buf(code, buf, sizeof buf);
+	return buf;
 }
 
 /**
@@ -191,33 +230,40 @@ vendor_code_str(guint32 code)
  * If we can't understand the code return NULL or if the 4-byte code
  * consists only of printable characters, return the code as a string.
  */
-gchar *
-lookup_vendor_name(guchar raw[4])
+const char *
+vendor_get_name(guint32 code)
 {
-	static gchar temp[5];
-	gchar *name;
-    guint i;
+	const char *name;
 
-    if (raw[0] == '\0')
-        return NULL;
-
-	name = find_vendor(raw);
-	if (name != NULL)
+    if (0 == code) {
+		return NULL;
+	} else if (NULL != (name = find_vendor(code))) {
 		return name;
+	} else {
+		static char bytes[5];
+		unsigned i;
 
-	/* Unknown type, look whether we have all printable ASCII */
-	for (i = 0; i < sizeof(raw); i++) {
-        guchar c = raw[i];
-		if (is_ascii_print(c))
-            temp[i] = c;
-		else {
-            temp[0] = '\0';
-			break;
+		STATIC_ASSERT(sizeof code == G_N_ELEMENTS(bytes) - 1);
+		poke_be32(&bytes, code);
+
+		/* Unknown type, look whether we have all printable ASCII */
+		for (i = 0; i < G_N_ELEMENTS(bytes) - 1; i++) {
+			if (!is_ascii_alnum(bytes[i]))
+				return NULL;
 		}
+		bytes[i] = '\0';
+		return bytes;
 	}
-	temp[4] = '\0';
+}
 
-	return temp[0] ? temp : NULL;
+/**
+ * Initialize the vendor lookup.
+ */
+void
+vendor_init(void)
+{
+	BINARY_ARRAY_SORTED(vendor_map, struct vendor, code,
+		VENDOR_CODE_CMP, vendor_code_to_string);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
