@@ -1,5 +1,5 @@
 /*
- * $Id: walloc.c,v 1.8 2005/09/26 16:51:29 cbiere Exp $
+ * $Id: walloc.c 19316 2011-07-05 15:25:34Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,21 +35,18 @@
 
 #include "common.h"
 
-RCSID("$Id: walloc.c,v 1.8 2005/09/26 16:51:29 cbiere Exp $");
+RCSID("$Id: walloc.c 19316 2011-07-05 15:25:34Z rmanfredi $")
 
+#include "glib-missing.h"
+#include "pow2.h"
+#include "unsigned.h"
+#include "halloc.h"
+#include "unsigned.h"
 #include "walloc.h"
 #include "zalloc.h"
-#include "override.h"		/* Must be the last header included */
+#include "vmm.h"
 
-#if ZALLOC_ALIGNBYTES == 2
-#define ZALLOC_ALIGNBITS 1
-#elif ZALLOC_ALIGNBYTES == 4
-#define ZALLOC_ALIGNBITS 2
-#elif ZALLOC_ALIGNBYTES == 8
-#define ZALLOC_ALIGNBITS 3
-#else
-#error "Unexpected ZALLOC_ALIGNBYTES value"
-#endif
+#include "override.h"		/* Must be the last header included */
 
 #ifdef TRACK_ZALLOC
 #undef walloc				/* We want to define the real routines */
@@ -57,25 +54,71 @@ RCSID("$Id: walloc.c,v 1.8 2005/09/26 16:51:29 cbiere Exp $");
 #undef wrealloc
 #endif
 
-#define WALLOC_MAX		4096	/**< Passed this size, use malloc() */
-#define WALLOC_CHUNK	4096	/**< Target chunk size for small structs */
-#define WALLOC_MINCOUNT	8		/**< Minimum amount of structs in a chunk */
+/**
+ * Maximum size for a walloc().  Anything larger is allocated by using
+ * either halloc() or malloc().
+ */
+#define WALLOC_MAX		4096
 
-#define WZONE_SIZE	(WALLOC_MAX / ZALLOC_ALIGNBYTES)
+#define WALLOC_MINCOUNT	8			/**< Minimum amount of structs in a chunk */
+
+#define WZONE_SIZE	(WALLOC_MAX / ZALLOC_ALIGNBYTES + 1)
 
 static struct zone *wzone[WZONE_SIZE];
+static size_t halloc_threshold = -1;
 
 /*
  * Under REMAP_ZALLOC, do not define walloc(), wfree() and wrealloc().
  */
 
 #ifndef REMAP_ZALLOC
+/**
+ * Compute index in wzone[] for a (rounded) size.
+ */
+static inline size_t
+wzone_index(size_t rounded)
+{
+	size_t idx;
+
+	g_assert(rounded == zalloc_round(rounded));
+	g_assert(size_is_positive(rounded) && rounded <= WALLOC_MAX);
+
+	STATIC_ASSERT(IS_POWER_OF_2(ZALLOC_ALIGNBYTES));
+	idx = rounded / ZALLOC_ALIGNBYTES;
+
+	STATIC_ASSERT(WALLOC_MAX / ZALLOC_ALIGNBYTES + 1 == WZONE_SIZE);
+	g_assert(idx < WZONE_SIZE);
+
+	return idx;
+}
+
+/**
+ * Allocate a new zone of given rounded size.
+ */
+static zone_t *
+wzone_get(size_t rounded)
+{
+	zone_t *zone;
+
+	g_assert(rounded == zalloc_round(rounded));
+
+	/*
+	 * We're paying this computation/allocation cost once per size!
+	 * Create chunks capable of holding at least WALLOC_MINCOUNT structures.
+	 */
+
+	if (!(zone = zget(rounded, WALLOC_MINCOUNT)))
+		g_error("zget() failed?");
+
+	return zone;
+}
+
 /**
  * Allocate memory from a zone suitable for the given size.
  *
  * The basics for this algorithm is to allocate from fixed-sized zones, which
  * are multiples of ZALLOC_ALIGNBYTES until WALLOC_MAX (e.g. 8, 16, 24, 40, ...)
- * and to malloc() if size is greater or equal to WALLOC_MAX.
+ * and to halloc()/malloc() if size is greater than WALLOC_MAX.
  * Naturally, zones are allocated on demand only.
  *
  * @return a pointer to the start of the allocated block.
@@ -87,33 +130,22 @@ walloc(size_t size)
 	size_t rounded = zalloc_round(size);
 	size_t idx;
 
-	g_assert(size > 0);
+	g_assert(size_is_positive(size));
 
-	if (rounded >= WALLOC_MAX)
-		return g_malloc(size);		/* Too big for efficient zalloc() */
+	if (rounded > WALLOC_MAX) {
+		/* Too big for efficient zalloc() */
+		void *p = size >= halloc_threshold ? halloc(size) : malloc(size);
+		if (NULL == p)
+			g_error("out of memory");
 
-	idx = rounded >> ZALLOC_ALIGNBITS;
-
-	STATIC_ASSERT(WALLOC_MAX >> ZALLOC_ALIGNBITS == WZONE_SIZE);
-	g_assert(idx < WZONE_SIZE);
-
-	if (!(zone = wzone[idx])) {
-		size_t count;
-
-		/*
-		 * We're paying this computation/allocation cost once per size!
-		 *
-		 * Try to create approximately WALLOC_CHUNK byte chunks, but
-		 * capable of holding at least WALLOC_MINCOUNT structures.
-		 */
-
-		count = WALLOC_CHUNK / rounded;
-		count = MAX(count, WALLOC_MINCOUNT);
-
-		if (!(zone = wzone[idx] = zget(rounded, count)))
-			g_error("zget() failed?");
+		return p;
 	}
 
+	idx = wzone_index(rounded);
+
+	if (!(zone = wzone[idx]))
+		zone = wzone[idx] = wzone_get(rounded);
+
 	return zalloc(zone);
 }
 
@@ -144,39 +176,106 @@ wfree(gpointer ptr, size_t size)
 	size_t rounded = zalloc_round(size);
 	size_t idx;
 
-	g_assert(ptr);
-	g_assert(size > 0);
+	g_assert(ptr != NULL);
+	g_assert(size_is_positive(size));
 
-	memset(ptr, 1, size);
-	if (rounded >= WALLOC_MAX) {
-		g_free(ptr);
+	if (rounded > WALLOC_MAX) {
+#ifdef TRACK_ZALLOC
+		/* halloc_track() is going to walloc_track() which uses malloc() */ 
+		free(ptr);
+#else
+		if (rounded >= halloc_threshold) {
+			hfree(ptr);
+		} else {
+			free(ptr);
+		}
+#endif
 		return;
 	}
 
-	idx = rounded >> ZALLOC_ALIGNBITS;
+	idx = rounded / ZALLOC_ALIGNBYTES;
 
 	g_assert(idx < WZONE_SIZE);
 
 	zone = wzone[idx];
-	g_assert(zone);
+	g_assert(zone != NULL);
 
 	zfree(zone, ptr);
 }
 
+/**
+ * Zero content and free a block allocated via walloc().
+ */
+void
+wfree0(gpointer ptr, size_t size)
+{
+	g_assert(ptr != NULL);
+	g_assert(size_is_positive(size));
+
+	memset(ptr, 0, size);
+	wfree(ptr, size);
+}
+
+/**
+ * Move block around if that can serve memory compaction.
+ * @return new location for block.
+ */
+void *
+wmove(void *ptr, size_t size)
+{
+	size_t idx = wzone_index(zalloc_round(size));
+	zone_t *zone = wzone[idx];
+
+	g_assert(zone != NULL);
+
+	return zmove(zone, ptr);
+}
+
 /**
  * Reallocate a block allocated via walloc().
  *
+ * @param old		old block address (may be NULL)
+ * @param old_size	size of the old block (ignored if old is NULL)
+ * @param new_size	the new size of the block
+ *
  * @return new block address.
  */
 gpointer
 wrealloc(gpointer old, size_t old_size, size_t new_size)
 {
 	gpointer new;
-	size_t rounded = zalloc_round(new_size);
+	size_t new_rounded = zalloc_round(new_size);
+	size_t old_rounded = zalloc_round(old_size);
+	size_t idx_old, idx_new;
 
-	if (zalloc_round(old_size) == rounded)
+	if (NULL == old)
+		return walloc(new_size);
+
+	if (old_rounded == new_rounded)
 		return old;
 
+	if (new_rounded > WALLOC_MAX || old_rounded > WALLOC_MAX)
+		goto resize_block;
+
+	/*
+	 * Due to upward rounding in zalloc() to avoid wasting bytes, it is
+	 * possible that the two sizes be actually served by the same underlying
+	 * zone, in which case there is nothing to do.
+	 */
+
+	idx_old = wzone_index(old_rounded);
+	idx_new = wzone_index(new_rounded);
+
+	g_assert(wzone[idx_old] != NULL);
+
+	if (NULL == wzone[idx_new])
+		wzone[idx_new] = wzone_get(new_rounded);
+
+	if (wzone[idx_old] == wzone[idx_new])
+		return zmove(wzone[idx_old], old);	/* Move around if interesting */
+
+resize_block:
+
 	new = walloc(new_size);
 	memcpy(new, old, MIN(old_size, new_size));
 	wfree(old, old_size);
@@ -196,40 +295,41 @@ wrealloc(gpointer old, size_t old_size, size_t new_size)
  * @returns a pointer to the start of the allocated block.
  */
 gpointer
-walloc_track(size_t size, gchar *file, gint line)
+walloc_track(size_t size, const char *file, int line)
 {
 	zone_t *zone;
 	size_t rounded = zalloc_round(size);
 	size_t idx;
 
-	g_assert(size > 0);
+	g_assert(size_is_positive(size));
 
-	if (rounded >= WALLOC_MAX)
+	if (rounded > WALLOC_MAX) {
+		/* Too big for efficient zalloc() */
+		void *p = 
 #ifdef TRACK_MALLOC
-		return malloc_track(size, file, line);
+			malloc_track(size, file, line);
 #else
-		return g_malloc(size);		/* Too big for efficient zalloc() */
+			/* Can't reroute to halloc() since it may come back here */
+			malloc(size);
 #endif
+			if (NULL == p)
+				g_error("out of memory");
 
-	idx = rounded >> ZALLOC_ALIGNBITS;
+			return p;
+	}
 
-	g_assert(WALLOC_MAX >> ZALLOC_ALIGNBITS == WZONE_SIZE);
+	idx = rounded / ZALLOC_ALIGNBYTES;
+
+	g_assert(WALLOC_MAX / ZALLOC_ALIGNBYTES + 1 == WZONE_SIZE);
 	g_assert(idx < WZONE_SIZE);
 
 	if (!(zone = wzone[idx])) {
-		size_t count;
-
 		/*
 		 * We're paying this computation/allocation cost once per size!
-		 *
-		 * Try to create approximately WALLOC_CHUNK byte chunks, but
-		 * capable of holding at least WALLOC_MINCOUNT structures.
+		 * Create chunks capable of holding at least WALLOC_MINCOUNT structures.
 		 */
 
-		count = WALLOC_CHUNK / rounded;
-		count = MAX(count, WALLOC_MINCOUNT);
-
-		if (!(zone = wzone[idx] = zget(rounded, count)))
+		if (!(zone = wzone[idx] = zget(rounded, WALLOC_MINCOUNT)))
 			g_error("zget() failed?");
 	}
 
@@ -240,7 +340,7 @@ walloc_track(size_t size, gchar *file, gint line)
  * Same as walloc_track(), but zeroes the allocated memory before returning.
  */
 gpointer
-walloc0_track(size_t size, gchar *file, gint line)
+walloc0_track(size_t size, const char *file, int line)
 {
 	gpointer p = walloc_track(size, file, line);
 
@@ -255,7 +355,7 @@ walloc0_track(size_t size, gchar *file, gint line)
  * to the allocated memory before returning.
  */
 gpointer
-wcopy_track(gconstpointer ptr, size_t size, gchar *file, gint line)
+wcopy_track(gconstpointer ptr, size_t size, const char *file, int line)
 {
 	gpointer p = walloc_track(size, file, line);
 
@@ -272,11 +372,14 @@ wcopy_track(gconstpointer ptr, size_t size, gchar *file, gint line)
  */
 gpointer
 wrealloc_track(gpointer old, size_t old_size, size_t new_size,
-	gchar *file, gint line)
+	const char *file, int line)
 {
 	gpointer new;
 	size_t rounded = zalloc_round(new_size);
 
+	if (NULL == old)
+		return walloc_track(new_size, file, line);
+
 	if (zalloc_round(old_size) == (size_t) rounded)
 		return old;
 
@@ -304,4 +407,20 @@ wdestroy(void)
 	}
 }
 
+/**
+ * Initialize the width-based allocator.
+ */
+void
+walloc_init(void)
+{
+	/*
+	 * We know that halloc() will redirect to walloc() if the size of the
+	 * block is slightly smaller than the system's page size.  If the maximum
+	 * size of blocks we can handle via walloc() is equal to the page size
+	 * or larger, then it is safe to halloc() the larger blocks.
+	 */
+
+	halloc_threshold = MAX(WALLOC_MAX, compat_pagesize());
+}
+
 /* vi: set ts=4 sw=4 cindent: */
