@@ -1,5 +1,5 @@
 /*
- * $Id: zalloc.h 12033 2006-09-23 01:04:57Z cbiere $
+ * $Id: zalloc.h 17535 2010-10-20 17:47:46Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -36,7 +36,7 @@
 #ifndef _zalloc_h_
 #define _zalloc_h_
 
-#include <glib.h>
+#include "common.h" 
 
 #define ZALLOC_ALIGNBYTES	MEM_ALIGNBYTES
 
@@ -54,8 +54,8 @@ typedef struct zone zone_t;
  * Memory allocation routines.
  */
 
-zone_t *zcreate(gint, gint);
-zone_t *zget(gint, gint);
+zone_t *zcreate(size_t, unsigned);
+zone_t *zget(size_t, unsigned);
 void zdestroy(zone_t *zone);
 
 /*
@@ -71,16 +71,27 @@ void zdestroy(zone_t *zone);
 #error "TRACK_ZALLOC and REMAP_ZALLOC are mutually exclusive"
 #endif	/* REMAP_ZALLOC && TRACK_ZALLOC */
 
-gpointer zalloc(zone_t *);
-void zfree(zone_t *, gpointer);
+void *zalloc(zone_t *) WARN_UNUSED_RESULT G_GNUC_MALLOC;
+void zfree(zone_t *, void *);
+void *zmove(zone_t *zone, void *p) WARN_UNUSED_RESULT;
+void zgc(gboolean overloaded);
+
+void zinit(void);
+void zclose(void);
+void set_zalloc_debug(guint32 level);
+void set_zalloc_always_gc(gboolean val);
 
 #ifdef TRACK_ZALLOC
 
-#define zalloc(z)	zalloc_track(z, __FILE__, __LINE__)
+#define zalloc(z)	zalloc_track(z, _WHERE_, __LINE__)
 
-gpointer zalloc_track(zone_t *z, gchar *file, gint line);
+void *zalloc_track(zone_t *z, const char *file, int line);
 
 #endif	/* TRACK_ZALLOC */
 
+#if defined(TRACK_ZALLOC) || defined(MALLOC_STATS)
+void zalloc_shift_pointer(const void *allocated, const void *used);
+#endif
+
 #endif /* _zalloc_h_ */
 
