@@ -1,5 +1,5 @@
 /*
- * $Id: zlib_util.c,v 1.13 2005/09/10 10:10:36 daichik Exp $
+ * $Id: zlib_util.c 19272 2011-06-18 11:44:53Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Raphael Manfredi
  *
@@ -35,10 +35,14 @@
 
 #include "common.h"
 
-RCSID("$Id: zlib_util.c,v 1.13 2005/09/10 10:10:36 daichik Exp $");
+RCSID("$Id: zlib_util.c 19272 2011-06-18 11:44:53Z rmanfredi $")
+
+#include <zlib.h>
 
 #include "zlib_util.h"
+#include "glib-missing.h"
 #include "misc.h"
+#include "halloc.h"
 #include "walloc.h"
 #include "override.h"		/* Must be the last header included */
 
@@ -49,8 +53,8 @@ RCSID("$Id: zlib_util.c,v 1.13 2005/09/10 10:10:36 daichik Exp $");
  *
  * @return human-readable error string corresponding to error code `errnum'.
  */
-gchar *
-zlib_strerror(gint errnum)
+const char *
+zlib_strerror(int errnum)
 {
 	switch (errnum) {
 	case Z_OK:				return "OK";
@@ -68,6 +72,25 @@ zlib_strerror(gint errnum)
 	return "Invalid error code";
 }
 
+gpointer
+zlib_alloc_func(gpointer unused_opaque, guint n, guint m)
+{
+	(void) unused_opaque;
+
+	g_return_val_if_fail(n > 0, NULL);
+	g_return_val_if_fail(m > 0, NULL);
+	g_return_val_if_fail(m < ((size_t) -1) / n, NULL);
+
+	return halloc((size_t) n * m);
+}
+
+void
+zlib_free_func(gpointer unused_opaque, gpointer p)
+{
+	(void) unused_opaque;
+	hfree(p);
+}
+
 /**
  * Creates an incremental zlib deflater for `len' bytes starting at `data',
  * with specified compression `level'.
@@ -82,30 +105,32 @@ zlib_strerror(gint errnum)
  */
 static zlib_deflater_t *
 zlib_deflater_alloc(
-	gconstpointer data, gint len, gpointer dest, gint destlen, gint level)
+	gconstpointer data, int len, gpointer dest, int destlen, int level)
 {
 	zlib_deflater_t *zd;
 	z_streamp outz;
-	gint ret;
+	int ret;
 
+	g_assert(len >= 0);
+	g_assert(destlen >= 0);
 	g_assert(level == Z_DEFAULT_COMPRESSION || (level >= 0 && level <= 9));
 
-	outz = walloc(sizeof(*outz));
-	outz->zalloc = (alloc_func) NULL;
-	outz->zfree = (free_func) NULL;
+	WALLOC(outz);
+	outz->zalloc = zlib_alloc_func;
+	outz->zfree = zlib_free_func;
 	outz->opaque = NULL;
 
 	ret = deflateInit(outz, level);
 
 	if (ret != Z_OK) {
-		wfree(outz, sizeof(*outz));
-		g_warning("unable to initialize compressor: %s", zlib_strerror(ret));
+		WFREE(outz);
+		g_carp("unable to initialize compressor: %s", zlib_strerror(ret));
 		return NULL;
 	}
 
-	zd = walloc(sizeof(*zd));
-
+	WALLOC(zd);
 	zd->opaque = outz;
+	zd->closed = FALSE;
 
 	zd->in = data;
 	zd->inlen = data ? len : 0;
@@ -116,7 +141,7 @@ zlib_deflater_alloc(
 	 *
 	 * NB: strictly speaking, we shouldn't need to store this information
 	 * here, we could rely on the information in the Z stream.  However, to
-	 * be able to inspect what's going on and add assertion,s, let's be
+	 * be able to inspect what's going on and add assertions, let's be
 	 * redundant.
 	 */
 
@@ -125,8 +150,11 @@ zlib_deflater_alloc(
 		if (data == NULL && len == 0)
 			len = 512;
 
-		zd->outlen = (gint) (len * 1.005 + 12.0);
-		zd->out = g_malloc(zd->outlen);
+		zd->outlen = (unsigned) (UNSIGNED(len) * 1.005 + 12.0);
+		g_assert(zd->outlen > len);
+		g_assert(zd->outlen - len >= 12);
+
+		zd->out = halloc(zd->outlen);
 		zd->allocated = TRUE;
 	} else {
 		/* Compressed data go to a supplied buffer, not-resizable */
@@ -159,7 +187,7 @@ zlib_deflater_alloc(
  * @return new deflater, or NULL if error.
  */
 zlib_deflater_t *
-zlib_deflater_make(gconstpointer data, gint len, gint level)
+zlib_deflater_make(gconstpointer data, int len, int level)
 {
 	return zlib_deflater_alloc(data, len, NULL, 0, level);
 }
@@ -177,7 +205,7 @@ zlib_deflater_make(gconstpointer data, gint len, gint level)
  */
 zlib_deflater_t *
 zlib_deflater_make_into(
-	gconstpointer data, gint len, gpointer dest, gint destlen, gint level)
+	gconstpointer data, int len, gpointer dest, int destlen, int level)
 {
 	return zlib_deflater_alloc(data, len, dest, destlen, level);
 }
@@ -191,23 +219,24 @@ zlib_deflater_make_into(
  *
  * @return -1 on error, 1 if work remains, 0 when done.
  */
-static gint
-zlib_deflate_step(zlib_deflater_t *zd, gint amount, gboolean may_close)
+static int
+zlib_deflate_step(zlib_deflater_t *zd, int amount, gboolean may_close)
 {
 	z_streamp outz = zd->opaque;
-	gint remaining;
-	gint process;
+	int remaining;
+	int process;
 	gboolean finishing;
-	gint ret;
+	int ret;
 
 	g_assert(amount > 0);
+	g_assert(!zd->closed);
 	g_assert(outz != NULL);			/* Stream not closed yet */
 
 	/*
 	 * Compute amount of input data to process.
 	 */
 
-	remaining = zd->inlen - ((gchar *) outz->next_in - (gchar *) zd->in);
+	remaining = zd->inlen - ((char *) outz->next_in - (char *) zd->in);
 	g_assert(remaining >= 0);
 
 	process = MIN(remaining, amount);
@@ -224,12 +253,12 @@ zlib_deflate_step(zlib_deflater_t *zd, gint amount, gboolean may_close)
 	switch (ret) {
 	case Z_OK:
 		if (outz->avail_out == 0) {
-			g_warning("under-estimated output buffer size: input=%d, output=%d",
+			g_carp("under-estimated output buffer size: input=%d, output=%d",
 				zd->inlen, zd->outlen);
 
 			if (zd->allocated) {
 				zd->outlen += OUT_GROW;
-				zd->out = g_realloc(zd->out, zd->outlen);
+				zd->out = hrealloc(zd->out, zd->outlen);
 				outz->next_out = (guchar *) zd->out + (zd->outlen - OUT_GROW);
 				outz->avail_out = OUT_GROW;
 			} else
@@ -242,21 +271,22 @@ zlib_deflate_step(zlib_deflater_t *zd, gint amount, gboolean may_close)
 	case Z_STREAM_END:
 		g_assert(finishing);
 
-		zd->outlen = (gchar *) outz->next_out - (gchar *) zd->out;
+		zd->outlen = (char *) outz->next_out - (char *) zd->out;
 		g_assert(zd->outlen > 0);
 
 		ret = deflateEnd(outz);
 		if (ret != Z_OK)
-			g_warning("while freeing compressor: %s", zlib_strerror(ret));
+			g_carp("while freeing compressor: %s", zlib_strerror(ret));
 
-		wfree(outz, sizeof(*outz));
+		WFREE(outz);
 		zd->opaque = NULL;
+		zd->closed = TRUE;
 
 		return 0;				/* Done */
 		/* NOTREACHED */
 
 	default:
-		g_warning("error during compression: %s", zlib_strerror(ret));
+		g_carp("error during compression: %s", zlib_strerror(ret));
 	}
 
 	/* FALL THROUGH */
@@ -264,8 +294,8 @@ zlib_deflate_step(zlib_deflater_t *zd, gint amount, gboolean may_close)
 error:
 	ret = deflateEnd(outz);
 	if (ret != Z_OK && ret != Z_DATA_ERROR)
-		g_warning("while freeing compressor: %s", zlib_strerror(ret));
-	wfree(outz, sizeof(*outz));
+		g_carp("while freeing compressor: %s", zlib_strerror(ret));
+	WFREE(outz);
 	zd->opaque = NULL;
 
 	return -1;				/* Error! */
@@ -277,8 +307,8 @@ error:
  *
  * @return -1 on error, 1 if work remains, 0 when done.
  */
-gint
-zlib_deflate(zlib_deflater_t *zd, gint amount)
+int
+zlib_deflate(zlib_deflater_t *zd, int amount)
 {
 	return zlib_deflate_step(zd, amount, TRUE);
 }
@@ -290,7 +320,7 @@ zlib_deflate(zlib_deflater_t *zd, gint amount)
  * @returns TRUE if OK, FALSE on error.
  */
 gboolean
-zlib_deflate_data(zlib_deflater_t *zd, gconstpointer data, gint len)
+zlib_deflate_data(zlib_deflater_t *zd, gconstpointer data, int len)
 {
 	z_streamp outz = zd->opaque;
 
@@ -315,7 +345,9 @@ gboolean
 zlib_deflate_close(zlib_deflater_t *zd)
 {
 	z_streamp outz = zd->opaque;
+	int ret;
 
+	g_assert(!zd->closed);
 	g_assert(outz != NULL);			/* Stream not closed yet */
 
 	zd->in = NULL;
@@ -324,7 +356,11 @@ zlib_deflate_close(zlib_deflater_t *zd)
 	outz->next_in = deconstify_gpointer(zd->in);
 	outz->avail_in = 0;
 
-	return zlib_deflate_step(zd, 1, TRUE) == 0 ? TRUE : FALSE;
+	ret = zlib_deflate_step(zd, 1, TRUE) == 0 ? TRUE : FALSE;
+
+	zd->closed = TRUE;				/* Even if there was an error */
+
+	return ret;
 }
 
 /**
@@ -337,52 +373,130 @@ zlib_deflater_free(zlib_deflater_t *zd, gboolean output)
 	z_streamp outz = zd->opaque;
 
 	if (outz) {
-		gint ret = deflateEnd(outz);
+		int ret = deflateEnd(outz);
 
 		if (ret != Z_OK && ret != Z_DATA_ERROR)
-			g_warning("while freeing compressor: %s", zlib_strerror(ret));
+			g_carp("while freeing compressor: %s", zlib_strerror(ret));
 
-		wfree(outz, sizeof(*outz));
+		WFREE(outz);
 	}
 
-	if (output && zd->allocated)
-		G_FREE_NULL(zd->out);
-
-	wfree(zd, sizeof(*zd));
+	if (output && zd->allocated) {
+		HFREE_NULL(zd->out);
+	}
+	WFREE(zd);
 }
 
 /**
  * Inflate data, whose final uncompressed size is known.
  *
- * @return allocated uncompressed data if OK, NULL on error.
+ * @return allocated uncompressed data via halloc() if OK, NULL on error.
+ * Use hfree() to free the data.
  */
 gpointer
-zlib_uncompress(gconstpointer data, gint len, gulong uncompressed_len)
+zlib_uncompress(gconstpointer data, int len, gulong uncompressed_len)
 {
-	gint ret;
-	guchar *out = g_malloc(uncompressed_len);
+	int ret;
+	guchar *out = halloc(uncompressed_len);
 	gulong retlen = uncompressed_len;
 
+	g_return_val_if_fail(uncompressed_len != 0, NULL);
+
 	ret = uncompress(out, &retlen, data, len);
 
 	if (ret == Z_OK) {
 		if (retlen != uncompressed_len)
-			g_warning("expected %lu bytes of decompressed data, got %ld",
+			g_carp("expected %lu bytes of decompressed data, got %ld",
 				uncompressed_len, retlen);
 		return out;
 	}
 
-	g_warning("while decompressing data: %s", zlib_strerror(ret));
-	G_FREE_NULL(out);
+	g_carp("while decompressing data: %s", zlib_strerror(ret));
+	HFREE_NULL(out);
 
 	return NULL;
 }
 
+/**
+ * Inflate data into supplied buffer.
+ *
+ * @param data		the data to inflate
+ * @param len		length of data
+ * @param out		buffer where inflated data should be put
+ * @param outlen	on input the length of out buffer, on output the length of
+ *					the deflated data.
+ *
+ * @return zlib's status: Z_OK on OK, error code otherwise.
+ */
+int
+zlib_inflate_into(gconstpointer data, int len, gpointer out, int *outlen)
+{
+	z_streamp inz;
+	int ret;
+	int inflated = 0;
+
+	g_assert(data != NULL);
+	g_assert(len > 0);
+	g_assert(out != NULL);
+	g_assert(outlen != NULL);
+	g_assert(*outlen > 0);
+
+	/*
+	 * Allocate decompressor.
+	 */
+
+	WALLOC(inz);
+	inz->zalloc = zlib_alloc_func;
+	inz->zfree = zlib_free_func;
+	inz->opaque = NULL;
+
+	ret = inflateInit(inz);
+
+	if (ret != Z_OK) {
+		g_carp("unable to setup decompressor: %s", zlib_strerror(ret));
+		goto done;
+	}
+
+	/*
+	 * Prepare call to inflate().
+	 */
+
+	inz->next_in = deconstify_gpointer(data);
+	inz->avail_in = len;
+
+	inz->next_out = out;
+	inz->avail_out = *outlen;
+
+	/*
+	 * Decompress data.
+	 */
+
+	ret = inflate(inz, Z_SYNC_FLUSH);
+
+	inflated = *outlen - inz->avail_out;
+
+	if (ret == Z_STREAM_END) {
+		ret = Z_OK;
+		*outlen = inflated;
+		goto done;
+	}
+
+	if (ret == Z_OK)		/* Expected Z_STREAM_END! */
+		ret = Z_DATA_ERROR;
+
+	/* FALL THROUGH */
+
+done:
+	(void) inflateEnd(inz);
+	WFREE(inz);
+	return ret;
+}
+
 /**
  * Check whether first bytes of data make up a valid zlib marker.
  */
 gboolean
-zlib_is_valid_header(gconstpointer data, gint len)
+zlib_is_valid_header(gconstpointer data, int len)
 {
 	const guchar *p = data;
 	guint16 check;
