@@ -1,5 +1,5 @@
 /*
- * $Id: bitzi.c 11437 2006-08-05 00:24:20Z cbiere $
+ * $Id: bitzi.c 19229 2011-06-03 10:26:31Z rmanfredi $
  *
  * Copyright (c) 2004, Alex Bennee <alex@bennee.com>
  *
@@ -35,15 +35,17 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: bitzi.c 11437 2006-08-05 00:24:20Z cbiere $")
+RCSID("$Id: bitzi.c 19229 2011-06-03 10:26:31Z rmanfredi $")
 
-#include "gtk/search.h"			/* search_t */
+#include "gtk/search.h"
 #include "gtk/misc.h"			/* gui_record_sha1_eq() */
-#include "gtk/gtk-missing.h"
 
 #include "if/gnet_property.h"
 #include "if/bridge/ui2c.h"
 #include "if/core/bitzi.h"    	/* bitzi_data_t */
+
+#include "lib/halloc.h"
+#include "lib/stringify.h"
 #include "lib/override.h"		/* Must be the last header included */
 
 /**
@@ -52,8 +54,11 @@ RCSID("$Id: bitzi.c 11437 2006-08-05 00:24:20Z cbiere $")
  * It assumes the enum's are in order.
  */
 
-static const gchar * const bitzi_fj_table[] = {
+static const char * const bitzi_fj_table[] = {
 	N_("Bitzi|Unknown"),				/**< UNKNOWN */
+	N_("Bitzi|Unrated"),				/**< UNRATED */
+	N_("Bitzi|Bitzi lookup failure"),	/**< FAILURE */
+	N_("Bitzi|Filesize mismatch"),		/**< WRONG_FILESIZE */
 	N_("Bitzi|Dangerous/Misleading"),	/**< DANGEROUS_MISLEADING */
 	N_("Bitzi|Incomplete/Damaged"),		/**< INCOMPLETE_DAMAGED */
 	N_("Bitzi|Substandard"),			/**< SUBSTANDARD */
@@ -65,12 +70,12 @@ static const gchar * const bitzi_fj_table[] = {
 	N_("Bitzi|Best Version"),			/**< BEST_VERSION*/
 };
 
-const gchar *
+const char *
 bitzi_fj_to_string(bitzi_fj_t fj)
 {
 	STATIC_ASSERT(NUM_BITZI_FJ == G_N_ELEMENTS(bitzi_fj_table));
-	g_assert((gint) fj >= 0 && fj < G_N_ELEMENTS(bitzi_fj_table));
-	return Q_(bitzi_fj_table[fj]);
+	g_assert(UNSIGNED(fj) < G_N_ELEMENTS(bitzi_fj_table));
+	return Q_(bitzi_fj_table[UNSIGNED(fj)]);
 }
 
 void
@@ -81,13 +86,13 @@ bitzi_gui_update(const bitzi_data_t *bitzi_data)
 	g_assert(bitzi_data != NULL);
 
     gnet_prop_get_guint32_val(PROP_BITZI_DEBUG, &bitzi_debug);
-	if (bitzi_debug)
-    	g_message("bitzi_gui_update: data %p, size %s\n"
-			  "goodness %f, judgement %d, type %s, details %s",
+	if (bitzi_debug > 10)
+    	g_debug("bitzi_gui_update: data %p, size %s, "
+			  "goodness %f, judgment %d, type %s, details %s",
 			cast_to_gconstpointer(bitzi_data),
 			uint64_to_string(bitzi_data->size),
 			bitzi_data->goodness,
-			bitzi_data->judgement,
+			bitzi_data->judgment,
 			NULL_STRING(bitzi_data->mime_type),
 			NULL_STRING(bitzi_data->mime_desc));
 
@@ -96,7 +101,7 @@ bitzi_gui_update(const bitzi_data_t *bitzi_data)
 	search_gui_metadata_update(bitzi_data);
 }
 
-gchar *
+char *
 bitzi_gui_get_metadata(const bitzi_data_t *data)
 {
 	g_assert(data != NULL);
@@ -105,26 +110,33 @@ bitzi_gui_get_metadata(const bitzi_data_t *data)
 	 * Build string
 	 */
 
-	if (data->mime_type) {
+	if (
+		data->judgment == BITZI_FJ_FAILURE ||
+		data->judgment == BITZI_FJ_WRONG_FILESIZE
+	) {
+		return h_strdup(bitzi_fj_to_string(data->judgment));
+	} else if (data->mime_type) {
 		if (data->mime_desc) {
-			return g_strdup_printf("%s (%1.1f): %s (%s)",
-					bitzi_fj_to_string(data->judgement),
+			return h_strdup_printf("%s (%1.1f): %s (%s)%s%s",
+					bitzi_fj_to_string(data->judgment),
 					data->goodness,
 					data->mime_type,
-					data->mime_desc);
+					data->mime_desc,
+					data->duration != 0 ? "; " : "",
+					data->duration != 0 ? short_time(data->duration) : "");
 		} else {
-			return g_strdup_printf("%s (%1.1f): %s",
-					bitzi_fj_to_string(data->judgement),
+			return h_strdup_printf("%s (%1.1f): %s%s%s",
+					bitzi_fj_to_string(data->judgment),
 					data->goodness,
-					data->mime_type);
-		}
-	} else {
-		if (data->judgement != BITZI_FJ_UNKNOWN) {
-			return g_strdup_printf("%s (%1.1f): %s",
-					bitzi_fj_to_string(data->judgement),
-					data->goodness,
-					_("No other data"));
+					data->mime_type,
+					data->duration != 0 ? "; " : "",
+					data->duration != 0 ? short_time(data->duration) : "");
 		}
+	} else if (data->judgment != BITZI_FJ_UNKNOWN) {
+		return h_strdup_printf("%s (%1.1f): %s",
+				bitzi_fj_to_string(data->judgment),
+				data->goodness,
+				_("No other data"));
 	}
 
 	return NULL;
