@@ -1,5 +1,5 @@
 /*
- * $Id: columns.h,v 1.13 2005/11/09 19:55:20 cbiere Exp $
+ * $Id: columns.h 15329 2008-05-03 04:26:42Z cbiere $
  *
  * Copyright (c) 2001-2004, Raphael Manfredi & Richard Eckart
  *
@@ -41,8 +41,8 @@
  * Gnet table columns.
  */
 
-enum {
-	c_gnet_host = 0,
+enum c_gnet {
+	c_gnet_host,
 	c_gnet_loc,
 	c_gnet_flags,
 	c_gnet_user_agent,
@@ -63,7 +63,7 @@ enum {
  */
 
 enum {
-    c_ul_filename = 0,
+    c_ul_filename,
     c_ul_host,
     c_ul_loc,
     c_ul_size,
@@ -84,92 +84,48 @@ enum {
  * Upload stats columns.
  */
 
-enum {
-    c_us_filename = 0,
+enum c_us {
+    c_us_filename,
     c_us_size,
     c_us_attempts,
     c_us_complete,
     c_us_norm,
-#define UPLOAD_STATS_GUI_VISIBLE_COLUMNS ((guint) c_us_norm + 1)
-	c_us_stat,
+	c_us_rtime,
+	c_us_dtime,
 
+#define UPLOAD_STATS_GUI_VISIBLE_COLUMNS ((guint) c_us_num)
 	c_us_num
 };
 
-/*
- * Downloads table columns.
- */
+enum c_src {
+	c_src_host,
+	c_src_country,
+	c_src_server,
+	c_src_range,
+	c_src_progress,
+	c_src_status,
 
-enum {
-    c_dl_filename = 0,
-    c_dl_host,
-    c_dl_loc,
-    c_dl_size,
-    c_dl_range,
-    c_dl_server,
-    c_dl_progress,
-    c_dl_status,
-#define DOWNLOADS_VISIBLE_COLUMNS ((guint) c_dl_status + 1)
-#ifdef USE_GTK2
-    c_dl_fg,	/**< invisible, holds the foreground color for the row */
-    c_dl_bg,	/**< invisible, holds the background color for the row */
-    c_dl_record, /**< invisible, pointer to the record_t of this entry */
-#endif /* USE_GTK2 */
-    c_dl_num
+#define SOURCES_VISIBLE_COLUMNS ((guint) c_src_num)
+	c_src_num
 };
 
-/*
- * Queue table columns.
- */
-
-#ifdef USE_GTK1
-enum {
-    c_queue_filename = 0,
-    c_queue_host,
-    c_queue_loc,
-    c_queue_size,
-    c_queue_server,
-    c_queue_status,
-	c_queue_num
-#define DOWNLOAD_QUEUE_VISIBLE_COLUMNS ((guint) c_queue_num)
-};
-#endif
-
-#ifdef USE_GTK2
-enum {
-    c_queue_filename = 0,
-    c_queue_size,
-    c_queue_host,
-    c_queue_loc,
-    c_queue_server,
-    c_queue_status,
-#define DOWNLOAD_QUEUE_VISIBLE_COLUMNS ((guint) c_queue_status + 1)
-	c_queue_fg,	/**< invisible, holds the foreground color for the row */
-	c_queue_bg,	/**< invisible, holds the background color for the row */
-	c_queue_record, /**< invisible, pointer to the record_t of this entry */
-	c_queue_num
-};
-#endif
-
-
 /**
  * Fileinfo table columns.
  */
 
-enum {
-	c_fi_filename = 0,
+enum c_fi {
+	c_fi_filename,
 	c_fi_size,
+	c_fi_progress,
+	c_fi_rx,
 	c_fi_done,
+	c_fi_uploaded,
 	c_fi_sources,
+	c_fi_created,
+	c_fi_modified,
 	c_fi_status,
-#define FILEINFO_VISIBLE_COLUMNS ((guint) c_fi_status + 1)
-#ifdef USE_GTK2
-	c_fi_handle,
-	c_fi_isize,
-	c_fi_idone,
-	c_fi_isources,
-	c_fi_istatus,
-#endif
+
+#define FILEINFO_VISIBLE_COLUMNS ((guint) c_fi_num)
 	c_fi_num
 };
 
@@ -177,20 +133,28 @@ enum {
  * Searches table columns.
  */
 
-enum {
-    c_sr_filename = 0,
+enum c_sr_columns {
+    c_sr_filename,
 	c_sr_ext,
     c_sr_charset,
     c_sr_size,
+	c_sr_mime,
 	c_sr_count,
     c_sr_loc,
     c_sr_meta,	/**< meta-data info column */
+    c_sr_vendor,
     c_sr_info,
-#define SEARCH_RESULTS_VISIBLE_COLUMNS ((guint) c_sr_info + 1)
-#ifdef USE_GTK2
-	c_sr_fg, /**< invisible, holds the foreground color for the row */
-	c_sr_bg, /**< invisible, holds the background color for the row */
-#endif
+    c_sr_route,
+    c_sr_protocol,
+    c_sr_hops,
+    c_sr_ttl,
+    c_sr_spam,
+    c_sr_hostile,
+    c_sr_owned,
+    c_sr_sha1,
+	c_sr_ctime,
+
+#define SEARCH_RESULTS_VISIBLE_COLUMNS ((guint) c_sr_num)
 	c_sr_num
 };
 
