@@ -1,5 +1,5 @@
 /*
- * $Id: filter_cb.c 12195 2006-11-03 02:20:22Z cbiere $
+ * $Id: filter_cb.c 17891 2010-12-08 22:23:22Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -35,17 +35,15 @@
 
 #include "gui.h"
 
-RCSID("$Id: filter_cb.c 12195 2006-11-03 02:20:22Z cbiere $")
-
-#include <gdk/gdkkeysyms.h>
+RCSID("$Id: filter_cb.c 17891 2010-12-08 22:23:22Z rmanfredi $")
 
 #include "filter_core.h"
 #include "filter_cb.h"
 #include "filter.h"
-#include "gtk-missing.h"
 
 #include "if/gui_property_priv.h"
 
+#include "lib/glib-missing.h"
 #include "lib/utf8.h"
 #include "lib/override.h"		/* Must be the last header included */
 
@@ -56,7 +54,7 @@ RCSID("$Id: filter_cb.c 12195 2006-11-03 02:20:22Z cbiere $")
 /**
  * Private variables.
  */
-static rule_t *rule_clipboard = NULL;
+static rule_t *rule_clipboard;
 
 /**
  * Private functions.
@@ -225,7 +223,7 @@ on_button_filter_abort_rule_clicked(GtkButton *unused_button,
 	(void) unused_button;
 	(void) unused_udata;
 
-    gtk_notebook_set_page(
+    gtk_notebook_set_current_page(
         GTK_NOTEBOOK(gui_filter_dialog_lookup("notebook_filter_detail")),
         nb_filt_page_buttons);
 }
@@ -332,26 +330,6 @@ on_button_filter_reset_all_clicked(GtkButton *unused_button,
     }
 }
 
-void
-on_ctree_filter_filters_resize_column(GtkCList *unused_clist, gint column,
-	gint width, gpointer unused_udata)
-{
-	(void) unused_clist;
-	(void) unused_udata;
-
-    *(gint *) &filter_filters_col_widths[column] = width;
-}
-
-void
-on_clist_filter_rules_resize_column(GtkCList *unused_clist, gint column,
-	gint width, gpointer unused_udata)
-{
-	(void) unused_clist;
-	(void) unused_udata;
-
-    *(gint *) &filter_rules_col_widths[column] = width;
-}
-
 void
 on_clist_filter_rules_select_row(GtkCList *clist, gint row,
 	gint unused_column, GdkEvent *unused_event, gpointer unused_udata)
@@ -536,9 +514,8 @@ on_button_filter_clear_clicked(GtkButton *unused_button, gpointer unused_udata)
     }
 
     gtk_clist_thaw(clist);
-    gtk_notebook_set_page(
-        GTK_NOTEBOOK
-            (gui_filter_dialog_lookup("notebook_filter_detail")),
+    gtk_notebook_set_current_page(
+        GTK_NOTEBOOK(gui_filter_dialog_lookup("notebook_filter_detail")),
         nb_filt_page_buttons);
 }
 
@@ -560,7 +537,7 @@ on_button_filter_remove_rule_clicked(GtkButton *unused_button,
 
     filter_remove_rule_from_session(work_filter, r);
 
-    gtk_notebook_set_page(
+    gtk_notebook_set_current_page(
         GTK_NOTEBOOK(gui_filter_dialog_lookup("notebook_filter_detail")),
         nb_filt_page_buttons);
 }
@@ -988,10 +965,9 @@ on_button_filter_clear_clicked(GtkButton *unused_button, gpointer unused_udata)
 
 	for (sl = sl_rules; sl != NULL; sl = g_slist_next(sl))	
 		filter_remove_rule_from_session(work_filter, sl->data);
-	g_slist_free(sl_rules);
-	sl_rules = NULL;
-	
-    gtk_notebook_set_page(
+	gm_slist_free_null(&sl_rules);
+
+    gtk_notebook_set_current_page(
         GTK_NOTEBOOK(gui_filter_dialog_lookup("notebook_filter_detail")),
         nb_filt_page_buttons);
 }
@@ -1024,7 +1000,7 @@ on_button_filter_remove_rule_clicked(GtkButton *unused_button,
 		gtk_tree_model_get(model, &iter, 0, &p, (-1));
     	filter_remove_rule_from_session(work_filter, p);
 
-    	gtk_notebook_set_page(
+    	gtk_notebook_set_current_page(
           GTK_NOTEBOOK(gui_filter_dialog_lookup("notebook_filter_detail")),
           nb_filt_page_buttons);
 	}
