@@ -1,5 +1,5 @@
 /*
- * $Id: gtk-missing.c 14025 2007-07-03 02:41:59Z cbiere $
+ * $Id: gtk-missing.c 19169 2011-05-25 15:32:46Z rmanfredi $
  *
  * Copyright (c) 2002-2003, Richard Eckart
  *
@@ -37,11 +37,16 @@
 
 #include "gui.h"
 
-RCSID("$Id: gtk-missing.c 14025 2007-07-03 02:41:59Z cbiere $")
+RCSID("$Id: gtk-missing.c 19169 2011-05-25 15:32:46Z rmanfredi $")
 
-#include "gtk-missing.h"
 #include "columns.h"
 
+#ifdef USE_GTK1
+/* For gtk_window_iconify() */
+#include <gdk/gdkx.h>
+#include <X11/Xlib.h>
+#endif
+
 #include "if/gui_property.h"
 #include "if/gui_property_priv.h"
 
@@ -75,13 +80,95 @@ gtk_paned_get_position(GtkPaned *paned)
  * @warning EVIL HACK
  */
 void
-gtk_clist_set_column_name(GtkCList * clist, gint col, gchar *title)
+gtk_clist_set_column_name(GtkCList *clist, int col, const char *title)
 {
     if (col >= 0 && col < clist->columns) {
-		G_FREE_NULL(clist->column[col].title);
-		clist->column[col].title = g_strdup(title);
+		if (
+			NULL == title ||
+			NULL == clist->column[col].title ||
+			0 != strcmp(clist->column[col].title, title)
+		) {
+			G_FREE_NULL(clist->column[col].title);
+			clist->column[col].title = NOT_LEAKING(g_strdup(title));
+		}
 	}
 }
+
+void
+clist_sync_rows(GtkCList *clist, void (*func)(int, void *))
+{
+	GList *iter;
+	int i;
+
+	g_return_if_fail(clist);
+	g_return_if_fail(func);
+	
+	i = 0;
+	for (iter = clist->row_list; NULL != iter; iter = g_list_next(iter), i++) {
+		GtkCListRow *row;
+
+		row = iter->data;
+		(*func)(i, row->data);
+	}
+}
+
+char *
+clist_copy_text(GtkCList *clist, int row, int column)
+{
+	char *text;
+
+	g_return_val_if_fail(clist, NULL);
+	
+	if (
+		row < 0 ||
+		column < 0 ||
+		!gtk_clist_get_text(GTK_CLIST(clist), row, column, &text)
+	) {
+		text = NULL;
+	}
+	return g_strdup(text);
+}
+
+static int
+clist_get_focus_row(GtkCList *clist)
+{
+	g_return_val_if_fail(clist, -1);
+
+	if (
+		GTK_WIDGET_HAS_FOCUS(GTK_WIDGET(clist))
+		&& clist->focus_row >= 0
+		&& clist->focus_row < clist->rows
+	) {
+		return clist->focus_row;
+	}
+	return -1;
+}
+
+static gboolean
+clist_row_is_selected(GtkCList *clist, int row)
+{
+	const GList *node;
+
+	g_return_val_if_fail(clist, FALSE);
+	g_return_val_if_fail(row >= 0, FALSE);
+	g_return_val_if_fail(row < clist->rows, FALSE);
+
+	node = g_list_nth(clist->row_list, row);
+	g_return_val_if_fail(node, FALSE);
+
+	return GTK_STATE_SELECTED == GTK_CLIST_ROW(node)->state;
+}
+
+int
+clist_get_cursor_row(GtkCList *clist)
+{
+	int row;
+
+	g_return_val_if_fail(clist, -1);
+	
+	row = clist_get_focus_row(clist);
+	return row >= 0 && clist_row_is_selected(clist, row) ? row : -1;
+}
 #endif /* USE_GTK1 */
 
 #define GTK_ITERATION_MAX	100		/* Don't spend too much time in GUI */
@@ -240,6 +327,19 @@ void gtk_label_printf(GtkLabel *label, const gchar *format, ...)
     va_end(args);
 }
 
+
+#ifdef USE_GTK1
+const char *
+gtk_label_get_text(GtkLabel *label)
+{
+	const char *text = NULL;
+
+	g_return_val_if_fail(label, NULL);
+	gtk_label_get(GTK_LABEL(label), (char **) &text);
+	return text;
+}
+#endif	/* USE_GTK1 */
+
 /**
  * Takes an array of strings which are supposed to be widgets names found with
  * the given top-level widget. Sets all to the given sentitivity state.
@@ -331,7 +431,7 @@ w_tree_iter_new(void)
 {
 	GtkTreeIter *iter;
 
-	iter = walloc(sizeof *iter);
+	WALLOC(iter);
 	return iter;
 }
 
@@ -344,8 +444,8 @@ w_tree_iter_copy(GtkTreeIter *iter)
 {
 	GtkTreeIter *copy;
 
-	copy = walloc(sizeof *copy);
-	*copy = *iter;
+	WALLOC(copy);
+	*copy = *iter;		/* Struct copy */
 	return copy;
 }
 
@@ -355,7 +455,7 @@ w_tree_iter_copy(GtkTreeIter *iter)
 void
 w_tree_iter_free(GtkTreeIter *iter)
 {
-	wfree(iter, sizeof *iter);
+	WFREE(iter);
 }
 
 /**
@@ -365,7 +465,7 @@ void
 ht_w_tree_iter_free(gpointer p)
 {
 	GtkTreeIter *iter = p;
-	wfree(iter, sizeof *iter);
+	WFREE(iter);
 }
 
 
@@ -482,7 +582,7 @@ tree_selection_collect_data(GtkTreeSelection *selection,
     /*
      * Cleanup before exit.
      */
-    g_slist_free(cdata.to_unselect);
+    gm_slist_free_null(&cdata.to_unselect);
 
     return cdata.results;
 }
@@ -553,10 +653,13 @@ on_tree_view_motion_notify(GtkWidget *widget,
 		}
 #undef EVENT_TYPE
 
-		g_message("on_tree_view_motion_notify(): type=%s, x=%d, y=%d, axes=%p, x_root=%d, y_root=%d",
-				type,
-				(gint) event->x, (gint) event->y, event->axes,
-				(gint) event->x_root, (gint) event->y_root);
+		if (GUI_PROPERTY(gui_debug) {
+			g_debug("on_tree_view_motion_notify(): "
+				"type=%s, x=%d, y=%d, axes=%p, x_root=%d, y_root=%d",
+					type,
+					(gint) event->x, (gint) event->y, event->axes,
+					(gint) event->x_root, (gint) event->y_root);
+		}
 	}
 #endif /* 0 */
 
@@ -603,29 +706,86 @@ tree_view_motion_set_callback(GtkTreeView *tv,
 {
 	tree_view_motion_t *tvm;
 
-	g_assert(tv != NULL);
-	g_assert(cb != NULL);
+	g_return_val_if_fail(tv, NULL);
+	g_return_val_if_fail(cb, NULL);
 
-	tvm = g_malloc(sizeof *tvm);
-	tvm->tv = tv;
+	WALLOC(tvm);
+	tvm->ready = FALSE;
+	tvm->tv = GTK_TREE_VIEW(g_object_ref(tv));
 	tvm->cb = cb;
 	tvm->timeout_id = g_timeout_add(interval, tree_view_motion_timeout, tvm);
-	tvm->signal_id = g_signal_connect(GTK_OBJECT(tv),
-		"motion-notify-event", G_CALLBACK(on_tree_view_motion_notify), tvm);
+	tvm->signal_id = gui_signal_connect(tv,
+						"motion-notify-event", on_tree_view_motion_notify, tvm);
 	return tvm;
 }
 
 void
-tree_view_motion_clear_callback(GtkTreeView *tv, tree_view_motion_t *tvm)
+tree_view_motion_clear_callback(tree_view_motion_t **ptr)
 {
-	g_assert(tv != NULL);
-	g_assert(tvm != NULL);
+	if (*ptr) {
+		tree_view_motion_t *tvm = *ptr;
 
-	g_signal_handler_disconnect(GTK_OBJECT(tv), tvm->signal_id);
-	g_source_remove(tvm->timeout_id);
-	G_FREE_NULL(tvm);
+		g_signal_handler_disconnect(tvm->tv, tvm->signal_id);
+		g_source_remove(tvm->timeout_id);
+		g_object_unref(tvm->tv);
+		tvm->tv = NULL;
+		WFREE_NULL(tvm, sizeof *tvm);
+		*ptr = NULL;
+	}
 }
 
+void
+tree_view_set_fixed_height_mode(GtkTreeView *tv, gboolean fixed)
+{
+	g_return_if_fail(tv);
+	
+#if GTK_CHECK_VERSION(2, 4, 0)
+    g_object_set(GTK_TREE_VIEW(tv), "fixed_height_mode", fixed, (void *) 0);
+#endif /* GTK+ >= 2.4.0 */
+}
+
+void
+tree_model_iter_changed(GtkTreeModel *model, GtkTreeIter *iter)
+{
+	GtkTreePath *path;
+
+	path = gtk_tree_model_get_path(model, iter);
+	gtk_tree_model_row_changed(model, path, iter);
+	gtk_tree_path_free(path);
+}
+
+void
+list_store_set_pointer(GtkListStore *store, GtkTreeIter *iter,
+	int column, void *data)
+{
+	static const GValue zero_value;
+	GValue value = zero_value;
+
+	g_value_init(&value, G_TYPE_POINTER);
+	g_value_set_pointer(&value, data);
+	gtk_list_store_set_value(store, iter, column, &value);
+}
+
+void
+list_store_append_pointer(GtkListStore *store, GtkTreeIter *iter,
+	int column, void *data)
+#if GTK_CHECK_VERSION(2,6,0)
+{
+	static const GValue zero_value;
+	GValue value = zero_value;
+	const int row = INT_MAX;
+
+	g_value_init(&value, G_TYPE_POINTER);
+	g_value_set_pointer(&value, data);
+	gtk_list_store_insert_with_valuesv(store, iter, row, &column, &value, 1);
+}
+#else	/* Gtk+ < 2.6 */
+{
+	gtk_list_store_append(store, iter);
+	list_store_set_pointer(store, iter, column, data);
+}
+#endif	/* Gtk+ >= 2.6 */
+
 #endif /* USE_GTK2 */
 
 gdouble
@@ -1008,6 +1168,7 @@ gtk_widget_fix_width(GtkWidget *w, GtkWidget *l, guint chars, guint extra)
     max_width = PANGO_PIXELS((gint)
 		(pango_font_metrics_get_approximate_digit_width(pfm) * chars + extra));
 
+	max_width *= 1.2;	/* The above still allocates less than it should */
     gtk_widget_set_size_request(w, max_width, -1);
 
     pango_font_metrics_unref(pfm);
@@ -1036,6 +1197,20 @@ gtk_progress_bar_set_text(GtkProgressBar *pb, const gchar *text)
 
 	gtk_progress_set_format_string(GTK_PROGRESS(pb), buf);
 }
+
+void
+gtk_window_iconify(GtkWindow *window)
+{
+	GtkWidget *widget;
+
+	g_return_if_fail(GTK_IS_WINDOW(window));
+
+	widget = GTK_WIDGET(window);
+	if (widget != NULL && widget->window != NULL) {
+		XIconifyWindow(GDK_DISPLAY(), GDK_WINDOW_XWINDOW(widget->window),
+			((_XPrivDisplay) GDK_DISPLAY())->default_screen);
+	}
+}
 #endif
 
 
@@ -1055,4 +1230,30 @@ gtk_paned_get_child2(GtkPaned *paned)
 }
 #endif
 
+void
+statusbar_set_shadow_type(GtkStatusbar *sb, GtkShadowType shadow)
+{
+	g_return_if_fail(sb);
+	(void) shadow;
+
+#if !GTK_CHECK_VERSION(2,0,0)
+	gtk_frame_set_shadow_type(GTK_FRAME(sb->frame), shadow);
+#endif	/* Gtk+ 1.2 */
+}
+
+/**
+ * @return TRUE if the runtime Gtk+ version is the given version or newer.
+ */
+gboolean
+check_gtk_version(unsigned major, unsigned minor, unsigned micro)
+{
+	if (gtk_major_version != major)
+		return gtk_major_version > major;
+	if (gtk_minor_version != minor)
+		return gtk_minor_version > minor;
+	if (gtk_micro_version != micro)
+		return gtk_micro_version > micro;
+	return TRUE;
+}
+
 /* vi: set ts=4 sw=4 cindent: */
