@@ -1,5 +1,5 @@
 /*
- * $Id: nodes.c,v 1.10 2005/04/24 19:30:03 cbiere Exp $
+ * $Id: nodes.c 18899 2011-03-30 15:17:39Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -25,12 +25,12 @@
  *----------------------------------------------------------------------
  */
 
+#include "common.h"
+
+RCSID("$Id: nodes.c 18899 2011-03-30 15:17:39Z rmanfredi $")
+
 #include "gtk/gui.h"
-
-RCSID("$Id: nodes.c,v 1.10 2005/04/24 19:30:03 cbiere Exp $");
-
-#include "interface-glade.h"
-#include "gtk/gtk-missing.h"
+#include "gtk/misc.h"
 #include "gtk/nodes_common.h"
 #include "gtk/nodes.h"
 #include "gtk/notebooks.h"
@@ -42,35 +42,41 @@ RCSID("$Id: nodes.c,v 1.10 2005/04/24 19:30:03 cbiere Exp $");
 
 #include "lib/glib-missing.h"
 #include "lib/iso3166.h"
-#include "lib/override.h"		/* Must be the last header included */
+#include "lib/nid.h"
+#include "lib/stringify.h"
+#include "lib/tm.h"
+#include "lib/utf8.h"
 
-#define UPDATE_MIN	300		/* Update screen every 5 minutes at least */
+#include "lib/override.h"	/* Must be the last header included */
 
 /*
  * These hash tables record which information about which nodes has
  * changed. By using this the number of updates to the gui can be
  * significantly reduced.
  */
-static GHashTable *ht_node_info_changed = NULL;
-static GHashTable *ht_node_flags_changed = NULL;
+static GHashTable *ht_node_info_changed;
+static GHashTable *ht_node_flags_changed;
 
 static void nodes_gui_update_node_info(gnet_node_info_t *n, gint row);
-static void nodes_gui_update_node_flags(
-	gnet_node_t n, gnet_node_flags_t *flags, gint row);
+static void nodes_gui_update_node_flags(const struct nid *node_id,
+				gnet_node_flags_t *flags, gint row);
 
-
-static gboolean
-nodes_gui_is_visible(void)
+static gboolean 
+remove_item(GHashTable *ht, const struct nid *node_id)
 {
-	static GtkNotebook *notebook = NULL;
-	gint current_page;
+	gpointer orig_key;
 
-	if (notebook == NULL)
-		notebook = GTK_NOTEBOOK(lookup_widget(main_window, "notebook_main"));
-
-	current_page = gtk_notebook_get_current_page(notebook);
-
-	return current_page == nb_main_page_gnet;
+	g_return_val_if_fail(ht, FALSE);
+	g_return_val_if_fail(node_id, FALSE);
+	
+	orig_key = g_hash_table_lookup(ht, node_id);
+	if (orig_key) {
+    	g_hash_table_remove(ht, orig_key);
+		nid_unref(orig_key);
+		return TRUE;
+	} else {
+		return FALSE;
+	}
 }
 
 /***
@@ -83,12 +89,12 @@ nodes_gui_is_visible(void)
  * Removes all references to the node from the frontend.
  */
 static void
-nodes_gui_node_removed(gnet_node_t n)
+nodes_gui_node_removed(const struct nid *node_id)
 {
-    if (gui_debug >= 5)
-        printf("nodes_gui_node_removed(%u)\n", n);
+    if (GUI_PROPERTY(gui_debug) >= 5)
+        g_debug("nodes_gui_node_removed(%s)", nid_to_string(node_id));
 
-    nodes_gui_remove_node(n);
+    nodes_gui_remove_node(node_id);
 }
 
 /**
@@ -97,14 +103,14 @@ nodes_gui_node_removed(gnet_node_t n)
  * Adds the node to the gui.
  */
 static void
-nodes_gui_node_added(gnet_node_t n)
+nodes_gui_node_added(const struct nid *node_id)
 {
     gnet_node_info_t info;
 
-    if (gui_debug >= 5)
-        printf("nodes_gui_node_added(%u)\n", n);
+    if (GUI_PROPERTY(gui_debug) >= 5)
+        g_debug("nodes_gui_node_added(%s)", nid_to_string(node_id));
 
-    guc_node_fill_info(n, &info);
+    guc_node_fill_info(node_id, &info);
     nodes_gui_add_node(&info);
     guc_node_clear_info(&info);
 }
@@ -116,10 +122,12 @@ nodes_gui_node_added(gnet_node_t n)
  * next tick.
  */
 static void
-nodes_gui_node_info_changed(gnet_node_t n)
+nodes_gui_node_info_changed(const struct nid *node_id)
 {
-    g_hash_table_insert(ht_node_info_changed,
-        GUINT_TO_POINTER(n), GUINT_TO_POINTER(1));
+    if (!g_hash_table_lookup(ht_node_info_changed, node_id)) {
+		const struct nid *key = nid_ref(node_id);
+    	gm_hash_table_insert_const(ht_node_info_changed, key, key);
+	}
 }
 
 /**
@@ -129,10 +137,12 @@ nodes_gui_node_info_changed(gnet_node_t n)
  * next tick.
  */
 static void
-nodes_gui_node_flags_changed(gnet_node_t n)
+nodes_gui_node_flags_changed(const struct nid *node_id)
 {
-    g_hash_table_insert(ht_node_flags_changed,
-        GUINT_TO_POINTER(n), GUINT_TO_POINTER(1));
+    if (!g_hash_table_lookup(ht_node_flags_changed, node_id)) {
+		const struct nid *key = nid_ref(node_id);
+    	gm_hash_table_insert_const(ht_node_flags_changed, key, key);
+	}
 }
 
 
@@ -142,49 +152,48 @@ nodes_gui_node_flags_changed(gnet_node_t n)
 
 /**
  * Update the row with the given nodeinfo. If row is -1 the row number
- * is determined by the node_handle contained in the gnet_node_info_t.
+ * is determined by the node_id contained in the gnet_node_info_t.
  */
 static void
 nodes_gui_update_node_info(gnet_node_info_t *n, gint row)
 {
-    GtkCList *clist = GTK_CLIST
-        (lookup_widget(main_window, "clist_nodes"));
+    GtkCList *clist = GTK_CLIST(gui_main_window_lookup("clist_nodes"));
 
     g_assert(n != NULL);
 
     if (row == -1) {
-        row = gtk_clist_find_row_from_data(
-            clist, GUINT_TO_POINTER(n->node_handle));
+        row = gtk_clist_find_row_from_data(clist,
+					deconstify_gpointer(n->node_id));
     }
 
     if (row != -1) {
 		gchar ver_buf[64];
         gnet_node_status_t status;
-        time_t now = time((time_t *) NULL);
+        time_t now = tm_time();
 
-        guc_node_get_status(n->node_handle, &status);
+        if (guc_node_get_status(n->node_id, &status)) {
+			gtk_clist_set_text(clist, row, c_gnet_user_agent,
+					n->vendor ? lazy_utf8_to_locale(n->vendor) : "...");
 
-        gtk_clist_set_text(clist, row, c_gnet_user_agent,
-			n->vendor ? n->vendor : "...");
+			gtk_clist_set_text(clist, row, c_gnet_loc,
+					deconstify_gchar(iso3166_country_cc(n->country)));
 
-        gtk_clist_set_text(clist, row, c_gnet_loc,
-			deconstify_gchar(iso3166_country_cc(n->country)));
+			gm_snprintf(ver_buf, sizeof ver_buf, "%d.%d",
+					n->proto_major, n->proto_minor);
+			gtk_clist_set_text(clist, row, c_gnet_version, ver_buf);
 
-        gm_snprintf(ver_buf, sizeof ver_buf, "%d.%d",
-            n->proto_major, n->proto_minor);
-        gtk_clist_set_text(clist, row, c_gnet_version, ver_buf);
+			if (status.status == GTA_NODE_CONNECTED)
+				gtk_clist_set_text(clist, row, c_gnet_connected,
+						short_uptime(delta_time(now, status.connect_date)));
 
-		if (status.status == GTA_NODE_CONNECTED)
-	        gtk_clist_set_text(clist, row, c_gnet_connected,
-       			short_uptime(delta_time(now, status.connect_date)));
+			if (status.up_date)
+				gtk_clist_set_text(clist, row, c_gnet_uptime,
+						status.up_date
+						? short_uptime(delta_time(now, status.up_date)) : "...");
 
-		if (status.up_date)
-    	    gtk_clist_set_text(clist, row, c_gnet_uptime,
-	        	status.up_date
-					? short_uptime(delta_time(now, status.up_date)) : "...");
-
-        gtk_clist_set_text(clist, row, c_gnet_info,
-			nodes_gui_common_status_str(&status));
+			gtk_clist_set_text(clist, row, c_gnet_info,
+					nodes_gui_common_status_str(&status));
+		}
     } else {
         g_warning("%s: no matching row found", G_GNUC_PRETTY_FUNCTION);
     }
@@ -194,18 +203,17 @@ nodes_gui_update_node_info(gnet_node_info_t *n, gint row)
  * Updates the flags for given node and row.
  */
 static void
-nodes_gui_update_node_flags(
-    gnet_node_t n, gnet_node_flags_t *flags, gint row)
+nodes_gui_update_node_flags(const struct nid *node_id, gnet_node_flags_t *flags,
+	gint row)
 {
-    GtkCList *clist = GTK_CLIST
-        (lookup_widget(main_window, "clist_nodes"));
+    GtkCList *clist = GTK_CLIST(gui_main_window_lookup("clist_nodes"));
 
     if (row == -1)
-        row = gtk_clist_find_row_from_data(clist, GUINT_TO_POINTER(n));
+        row = gtk_clist_find_row_from_data(clist, deconstify_gpointer(node_id));
 
     if (row != -1) {
         gtk_clist_set_text(clist, row, c_gnet_flags,
-			nodes_gui_common_flags_str(flags));
+			guc_node_flags_to_string(flags));
 	if (NODE_P_LEAF == flags->peermode || NODE_P_NORMAL == flags->peermode) {
 		GdkColor *color = &(gtk_widget_get_style(GTK_WIDGET(clist))
 			->fg[GTK_STATE_INSENSITIVE]);
@@ -227,9 +235,37 @@ nodes_gui_update_node_flags(
 void
 nodes_gui_early_init(void)
 {
-	popup_nodes = create_popup_nodes();
-    gtk_widget_set_sensitive(lookup_widget(popup_nodes, "popup_nodes_remove"),
-		FALSE);
+	static const struct {
+		const gchar *name;
+	} items[] = {
+		{ "popup_nodes_disconnect" },
+		{ "popup_nodes_browse_host" },
+	};
+	guint i;
+	
+	for (i = 0; i < G_N_ELEMENTS(items); i++) {
+    	gtk_widget_set_sensitive(gui_popup_nodes_lookup(items[i].name),
+			FALSE);
+	}
+}
+
+static const char *
+nodes_gui_column_title(int column)
+{
+	switch ((enum c_gnet) column) {
+    case c_gnet_host:		return _("Host");
+    case c_gnet_loc:		return _("Country");
+	case c_gnet_flags:		return _("Flags");
+	case c_gnet_user_agent:	return _("User-Agent");
+	case c_gnet_version:	return _("Version");
+	case c_gnet_connected:	return _("Connected time");
+	case c_gnet_uptime:		return _("Uptime");
+	case c_gnet_info:		return _("Status");
+	case c_gnet_num:
+		break;
+	}
+	g_assert_not_reached();
+	return NULL;
 }
 
 /**
@@ -238,28 +274,56 @@ nodes_gui_early_init(void)
 void
 nodes_gui_init(void)
 {
+	unsigned i;
 	GtkCList *clist;
 
-	clist = GTK_CLIST(lookup_widget(main_window, "clist_nodes"));
+	clist = GTK_CLIST(gui_main_window_lookup("clist_nodes"));
 
     gtk_clist_column_titles_passive(clist);
-    gtk_clist_set_column_name(clist, c_gnet_host, _("Host"));
-    gtk_clist_set_column_name(clist, c_gnet_loc, _("Loc"));
-    gtk_clist_set_column_name(clist, c_gnet_flags, _("Flags"));
-    gtk_clist_set_column_name(clist, c_gnet_user_agent, _("User-Agent"));
-    gtk_clist_set_column_name(clist, c_gnet_version, _("Ver"));
-    gtk_clist_set_column_name(clist, c_gnet_connected, _("Connected"));
-    gtk_clist_set_column_name(clist, c_gnet_uptime, _("Uptime"));
-    gtk_clist_set_column_name(clist, c_gnet_info, _("Info"));
-	gtk_clist_restore_visibility(clist, PROP_NODES_COL_VISIBLE);
+	for (i = 0; i < c_gnet_num; i++) {
+    	gtk_clist_set_column_name(clist, i, nodes_gui_column_title(i));
+	}
+	clist_restore_visibility(clist, PROP_NODES_COL_VISIBLE);
+	clist_restore_widths(clist, PROP_NODES_COL_WIDTHS);
 
-    ht_node_info_changed = g_hash_table_new(g_direct_hash, g_direct_equal);
-    ht_node_flags_changed = g_hash_table_new(g_direct_hash, g_direct_equal);
+	widget_add_popup_menu(GTK_WIDGET(clist), nodes_gui_get_popup_menu);
+
+    ht_node_info_changed = g_hash_table_new(nid_hash, nid_equal);
+    ht_node_flags_changed = g_hash_table_new(nid_hash, nid_equal);
 
     guc_node_add_node_added_listener(nodes_gui_node_added);
     guc_node_add_node_removed_listener(nodes_gui_node_removed);
     guc_node_add_node_info_changed_listener(nodes_gui_node_info_changed);
     guc_node_add_node_flags_changed_listener(nodes_gui_node_flags_changed);
+
+	main_gui_add_timer(nodes_gui_timer);
+}
+
+static gboolean
+free_node_id(gpointer key, gpointer value, gpointer unused_udata)
+{
+	g_assert(key == value);
+	(void) unused_udata;
+	nid_unref(key);
+	return TRUE;
+}
+
+static void
+nodes_gui_remove_all_nodes(void)
+{
+	GtkCList *clist;
+	GList *iter;
+
+    clist = GTK_CLIST(gui_main_window_lookup("clist_nodes"));
+	g_return_if_fail(clist);
+
+    gtk_clist_freeze(clist);
+	for (iter = clist->row_list; NULL != iter; iter = g_list_next(iter)) {
+		const struct nid *node_id = ((GtkCListRow *) iter->data)->data;
+		nid_unref(node_id);
+	}
+    gtk_clist_thaw(clist);
+
 }
 
 /**
@@ -270,31 +334,34 @@ nodes_gui_shutdown(void)
 {
 	GtkCList *clist;
 
-	clist = GTK_CLIST(lookup_widget(main_window, "clist_nodes"));
-	gtk_clist_save_visibility(clist, PROP_NODES_COL_VISIBLE);
+	clist = GTK_CLIST(gui_main_window_lookup("clist_nodes"));
+	clist_save_visibility(clist, PROP_NODES_COL_VISIBLE);
+	clist_save_widths(clist, PROP_NODES_COL_WIDTHS);
 
     guc_node_remove_node_added_listener(nodes_gui_node_added);
     guc_node_remove_node_removed_listener(nodes_gui_node_removed);
     guc_node_remove_node_info_changed_listener(nodes_gui_node_info_changed);
     guc_node_remove_node_flags_changed_listener(nodes_gui_node_flags_changed);
 
-    g_hash_table_destroy(ht_node_info_changed);
-    g_hash_table_destroy(ht_node_flags_changed);
+	g_hash_table_foreach_remove(ht_node_info_changed, free_node_id, NULL);
+    gm_hash_table_destroy_null(&ht_node_info_changed);
 
-    ht_node_info_changed = NULL;
-    ht_node_flags_changed = NULL;
+	g_hash_table_foreach_remove(ht_node_flags_changed, free_node_id, NULL);
+    gm_hash_table_destroy_null(&ht_node_flags_changed);
+
+	nodes_gui_remove_all_nodes();
 }
 
 /**
  * Removes all references to the given node handle in the gui.
  */
 void
-nodes_gui_remove_node(gnet_node_t n)
+nodes_gui_remove_node(const struct nid *node_id)
 {
     GtkWidget *clist_nodes;
     gint row;
 
-    clist_nodes = lookup_widget(main_window, "clist_nodes");
+    clist_nodes = gui_main_window_lookup("clist_nodes");
 
     /*
      * Make sure node is remove from the "changed" hash table so
@@ -303,15 +370,17 @@ nodes_gui_remove_node(gnet_node_t n)
     g_assert(NULL != ht_node_info_changed);
     g_assert(NULL != ht_node_flags_changed);
 
-    g_hash_table_remove(ht_node_info_changed, GUINT_TO_POINTER(n));
-    g_hash_table_remove(ht_node_flags_changed, GUINT_TO_POINTER(n));
+    remove_item(ht_node_info_changed, node_id);
+    remove_item(ht_node_flags_changed, node_id);
 
 	row = gtk_clist_find_row_from_data(GTK_CLIST(clist_nodes),
-		GUINT_TO_POINTER(n));
-    if (row != -1)
+				deconstify_gpointer(node_id));
+    if (row != -1) {
         gtk_clist_remove(GTK_CLIST(clist_nodes), row);
-    else
+		nid_unref(node_id);
+	} else {
         g_warning("nodes_gui_remove_node: no matching row found");
+	}
 }
 
 /**
@@ -321,90 +390,75 @@ void
 nodes_gui_add_node(gnet_node_info_t *n)
 {
     GtkCList *clist_nodes;
-    gint row;
 	const gchar *titles[c_gnet_num];
 	gchar proto_tmp[32];
+    gint row;
 
     g_assert(n != NULL);
 
-   	gm_snprintf(proto_tmp, sizeof(proto_tmp), "%d.%d",
+   	gm_snprintf(proto_tmp, sizeof proto_tmp, "%d.%d",
 		n->proto_major, n->proto_minor);
 
-    titles[c_gnet_host]       = ip_port_to_gchar(n->ip, n->port);
+    titles[c_gnet_host]       = host_addr_port_to_string(n->addr, n->port);
     titles[c_gnet_flags]      = "...";
-    titles[c_gnet_user_agent] = n->vendor ? n->vendor : "...";
+    titles[c_gnet_user_agent] = n->vendor
+									? lazy_utf8_to_locale(n->vendor)
+									: "...";
     titles[c_gnet_loc]        = iso3166_country_cc(n->country);
     titles[c_gnet_version]    = proto_tmp;
     titles[c_gnet_connected]  = "...";
     titles[c_gnet_uptime]     = "...";
     titles[c_gnet_info]       = "...";
 
-    clist_nodes = GTK_CLIST(lookup_widget(main_window, "clist_nodes"));
+    clist_nodes = GTK_CLIST(gui_main_window_lookup("clist_nodes"));
 
     row = gtk_clist_append(clist_nodes, (gchar **) titles); /* override const */
-    gtk_clist_set_row_data(clist_nodes, row, GUINT_TO_POINTER(n->node_handle));
+    gtk_clist_set_row_data(clist_nodes, row,
+		deconstify_gpointer(nid_ref(n->node_id)));
 }
 
 /**
  * Update all the nodes at the same time.
  *
- * FIXME: we should remember for every node when it was last
+ * @bug
+ * FIXME: We should remember for every node when it was last
  *        updated and only refresh every node at most once every
  *        second. This information should be kept in a struct pointed
  *        to by the row user_data and should be automatically freed
  *        when removing the row (see upload stats code).
  */
 void
-nodes_gui_update_nodes_display(time_t now)
+nodes_gui_update_display(time_t now)
 {
 	GtkCList *clist;
 	GList *l;
 	gint row = 0;
     gnet_node_status_t status;
-    static time_t last_update = 0;
 
-    if (last_update == now)
-        return;
-
-	/*
-	 * Usually don't perform updates if nobody is watching.  However,
-	 * we do need to perform periodic cleanup of dead entries or the
-	 * memory usage will grow.  Perform an update every UPDATE_MIN minutes
-	 * at least.
-	 *		--RAM, 28/12/2003
-	 */
-	if (!nodes_gui_is_visible() && delta_time(now, last_update) < UPDATE_MIN)
-		return;
-
-    last_update = now;
-
-    clist = GTK_CLIST(lookup_widget(main_window, "clist_nodes"));
+    clist = GTK_CLIST(gui_main_window_lookup("clist_nodes"));
     gtk_clist_freeze(clist);
 
 	for (l = clist->row_list, row = 0; l; l = l->next, row++) {
-		gnet_node_t n =
-			(gnet_node_t) GPOINTER_TO_UINT(((GtkCListRow *) l->data)->data);
+		const struct nid *node_id = ((GtkCListRow *) l->data)->data;
 
-        guc_node_get_status(n, &status);
+        guc_node_get_status(node_id, &status);
 
         /*
          * Update additional info too if it has recorded changes.
          */
-        if (g_hash_table_lookup(ht_node_info_changed, GUINT_TO_POINTER(n))) {
+        if (remove_item(ht_node_info_changed, node_id)) {
             gnet_node_info_t info;
 
-            g_hash_table_remove(ht_node_info_changed, GUINT_TO_POINTER(n));
-            guc_node_fill_info(n, &info);
+            guc_node_fill_info(node_id, &info);
             nodes_gui_update_node_info(&info, row);
             guc_node_clear_info(&info);
         }
 
-        if (g_hash_table_lookup(ht_node_flags_changed, GUINT_TO_POINTER(n))) {
+        if (remove_item(ht_node_flags_changed, node_id)) {
             gnet_node_flags_t flags;
 
-            g_hash_table_remove(ht_node_flags_changed, GUINT_TO_POINTER(n));
-            guc_node_fill_flags(n, &flags);
-            nodes_gui_update_node_flags(n, &flags, -1);
+            guc_node_fill_flags(node_id, &flags);
+            nodes_gui_update_node_flags(node_id, &flags, -1);
         }
 
 		/*
