@@ -1,5 +1,5 @@
 /*
- * $Id: nodes_cb.c 12195 2006-11-03 02:20:22Z cbiere $
+ * $Id: nodes_cb.c 18899 2011-03-30 15:17:39Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -25,23 +25,25 @@
  *----------------------------------------------------------------------
  */
 
+#include "common.h"
+
+RCSID("$Id: nodes_cb.c 18899 2011-03-30 15:17:39Z rmanfredi $")
+
 #include "gtk/gui.h"
-
-RCSID("$Id: nodes_cb.c 12195 2006-11-03 02:20:22Z cbiere $")
-
 #include "nodes_cb.h"
 
 #include "gtk/gtkcolumnchooser.h"
 #include "gtk/nodes_common.h"
 #include "gtk/settings.h"
-#include "gtk/gtk-missing.h"
-#include "gtk/search_common.h"
+#include "gtk/search.h"
 
 #include "if/gui_property_priv.h"
 #include "if/bridge/ui2c.h"
 
 #include "lib/override.h"		/* Must be the last header included */
 
+struct nid;
+
 static void
 update_sensitivity(gboolean sensitive)
 {
@@ -80,16 +82,6 @@ on_clist_nodes_unselect_row(GtkCList *clist, gint unused_row,
 	update_sensitivity(clist->selection != NULL);
 }
 
-void
-on_clist_nodes_resize_column(GtkCList *unused_clist, gint column,
-		gint width, gpointer unused_udata)
-{
-	(void) unused_clist;
-	(void) unused_udata;
-	g_assert(column >= 0 && column <= (gint) G_N_ELEMENTS(nodes_col_widths));
-    *(guint32 *) &nodes_col_widths[column] = width; /* override const */
-}
-
 gboolean
 on_clist_nodes_button_press_event(GtkWidget *unused_widget,
 		GdkEventButton *event, gpointer unused_udata)
@@ -122,13 +114,13 @@ on_clist_nodes_button_press_event(GtkWidget *unused_widget,
 static void
 remove_selected_nodes(void)
 {
-    GSList *node_list = NULL;
     GtkCList *clist = GTK_CLIST(gui_main_window_lookup("clist_nodes"));
+    GSList *node_list;
 
     g_assert(clist != NULL);
 
     node_list = clist_collect_data(clist, TRUE, NULL);
-    guc_node_remove_nodes_by_handle(node_list);
+    guc_node_remove_nodes_by_id(node_list);
     g_slist_free(node_list);
 }
 
@@ -204,7 +196,8 @@ on_popup_nodes_config_cols_activate(GtkMenuItem *unused_menuitem,
 	(void) unused_udata;
 
     cc = gtk_column_chooser_new(gui_main_window_lookup("clist_nodes"));
-    gtk_menu_popup(GTK_MENU(cc), NULL, NULL, NULL, NULL, 1, 0);
+    gtk_menu_popup(GTK_MENU(cc), NULL, NULL, NULL, NULL, 1,
+		gtk_get_current_event_time());
 
     /* GtkColumnChooser takes care of cleaning up itself */
 }
@@ -227,12 +220,15 @@ on_popup_nodes_browse_host_activate(GtkMenuItem *unused_menuitem,
     node_list = clist_collect_data(clist, TRUE, NULL);
 
 	for (sl = node_list; sl != NULL; sl = g_slist_next(sl)) {
-		gnet_node_t handle = GPOINTER_TO_UINT(sl->data);
+		const struct nid *handle = sl->data;
 		gnet_node_info_t *info = guc_node_get_info(handle);
 
+		if (!info)
+			continue;
+
 		if (!info->is_pseudo) {
 			search_gui_new_browse_host(NULL, info->gnet_addr, info->gnet_port,
-				info->gnet_guid, NULL, 0);
+				&info->gnet_guid, NULL, 0);
 		}
 		guc_node_free_info(info);
 	}
