@@ -1,5 +1,5 @@
 /*
- * $Id: search_stats.c,v 1.10 2005/11/14 12:43:56 cbiere Exp $
+ * $Id: search_stats.c 17891 2010-12-08 22:23:22Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi
  * Copyright (c) 2002, Michael Tesch
@@ -46,15 +46,18 @@
  * Released with gtk-gnutella & its license.
  */
 
+#include "common.h"
+
+RCSID("$Id: search_stats.c 17891 2010-12-08 22:23:22Z rmanfredi $")
+
 #include "gtk/gui.h"
 
-RCSID("$Id: search_stats.c,v 1.10 2005/11/14 12:43:56 cbiere Exp $");
-
-#include "gtk/gtk-missing.h"
-#include "gtk/search_stats.h"
 #include "gtk/columns.h"
+#include "gtk/misc.h"
+#include "gtk/search_stats.h"
 
 #include "if/core/share.h"
+#include "if/gui_property.h"
 #include "if/gui_property_priv.h"
 #include "if/bridge/ui2c.h"
 
@@ -211,8 +214,8 @@ stats_hash_to_clist(gpointer key, gpointer value, gpointer unused_udata)
 
 	/* try to keep the number of infrequent terms down */
 	if (
-		((gfloat) val->total_cnt / (val->periods + 2.0)) * 100 <
-			search_stats_delcoef
+		(1.0 * val->total_cnt / (val->periods + 2.0)) * 100 <
+			GUI_PROPERTY(search_stats_delcoef)
 	) {
 		G_FREE_NULL(key);
 		G_FREE_NULL(val);
@@ -233,7 +236,7 @@ stats_hash_to_clist(gpointer key, gpointer value, gpointer unused_udata)
 
     {
         GtkWidget *clist_search_stats =
-            lookup_widget(main_window, "clist_search_stats");
+            gui_main_window_lookup("clist_search_stats");
 
         gtk_clist_insert(GTK_CLIST(clist_search_stats), 0, text);
     }
@@ -251,7 +254,7 @@ static void
 search_stats_gui_enable(search_request_listener_t lst)
 {
     if (!callback_registered) {
-        guc_share_add_search_request_listener(lst);
+        guc_search_request_listener_add(lst);
         callback_registered = TRUE;
     }
 }
@@ -260,9 +263,9 @@ static void
 search_stats_gui_disable(void)
 {
     if (callback_registered) {
-        guc_share_remove_search_request_listener(search_stats_notify_word);
-        guc_share_remove_search_request_listener(search_stats_notify_whole);
-        guc_share_remove_search_request_listener(search_stats_notify_routed);
+        guc_search_request_listener_remove(search_stats_notify_word);
+        guc_search_request_listener_remove(search_stats_notify_whole);
+        guc_search_request_listener_remove(search_stats_notify_routed);
         callback_registered = FALSE;
     }
 
@@ -309,7 +312,7 @@ search_stats_gui_reset(void)
 {
 	empty_hash_table();
 	gtk_clist_clear(GTK_CLIST(
-        lookup_widget(main_window, "clist_search_stats")));
+        gui_main_window_lookup("clist_search_stats")));
 }
 
 void
@@ -340,47 +343,19 @@ search_stats_gui_set_type(gint type)
     }
 }
 
-void
-search_stats_gui_init(void)
-{
-    GtkWidget *clist_search_stats =
-        lookup_widget(main_window, "clist_search_stats");
-
-    /* set up the clist to be sorted properly */
-	gtk_clist_set_sort_column(GTK_CLIST(clist_search_stats), c_st_total);
-	gtk_clist_set_sort_type(GTK_CLIST(clist_search_stats),
-		GTK_SORT_DESCENDING);
-
-	stat_hash = g_hash_table_new(g_str_hash, g_str_equal);
-}
-
-void
-search_stats_gui_shutdown(void)
-{
-    search_stats_gui_set_type(NO_SEARCH_STATS);
-    g_hash_table_destroy(stat_hash);
-	stat_hash = NULL;
-}
-
 /**
  * Display the data gathered during the last time period.
  * Perhaps it would be better to have this done on a button click(?)
  */
-void
-search_stats_gui_update(time_t now)
+static void
+search_stats_gui_update_display(void)
 {
-	static time_t last_update = 0;
     GtkWidget *clist_search_stats;
     GtkWidget *label_search_stats_count;
 
-    if (delta_time(now, last_update) < search_stats_update_interval)
-        return;
-
-    last_update = now;
-
-    clist_search_stats = lookup_widget(main_window, "clist_search_stats");
+    clist_search_stats = gui_main_window_lookup("clist_search_stats");
     label_search_stats_count =
-        lookup_widget(main_window, "label_search_stats_count");
+        gui_main_window_lookup("label_search_stats_count");
 
 	stat_count = 0;
 	gtk_clist_freeze(GTK_CLIST(clist_search_stats));
@@ -398,4 +373,50 @@ search_stats_gui_update(time_t now)
 		stat_count);
 }
 
+static gboolean
+search_stats_gui_is_visible(void)
+{
+	return main_gui_window_visible() &&
+		nb_main_page_search_stats == main_gui_notebook_get_page();
+}
+
+static void
+search_stats_gui_timer(time_t now)
+{
+	static time_t last_update;
+	time_delta_t interval = GUI_PROPERTY(search_stats_update_interval);
+
+    if (
+		search_stats_gui_is_visible() &&
+		delta_time(now, last_update) > interval
+	) {
+    	last_update = now;
+		search_stats_gui_update_display();
+	}
+}
+
+void
+search_stats_gui_init(void)
+{
+    GtkCList *clist = GTK_CLIST(gui_main_window_lookup("clist_search_stats"));
+
+    /* set up the clist to be sorted properly */
+	gtk_clist_set_sort_column(clist, c_st_total);
+	gtk_clist_set_sort_type(clist, GTK_SORT_DESCENDING);
+	clist_restore_widths(clist, PROP_SEARCH_STATS_COL_WIDTHS);
+
+	stat_hash = g_hash_table_new(g_str_hash, g_str_equal);
+	main_gui_add_timer(search_stats_gui_timer);
+}
+
+void
+search_stats_gui_shutdown(void)
+{
+	clist_save_widths(
+		GTK_CLIST(gui_main_window_lookup("clist_search_stats")),
+		PROP_SEARCH_STATS_COL_WIDTHS);
+    search_stats_gui_set_type(NO_SEARCH_STATS);
+    gm_hash_table_destroy_null(&stat_hash);
+}
+
 /* vi: set ts=4 sw=4 cindent: */
