@@ -1,5 +1,5 @@
 /*
- * $Id: uploads.c,v 1.18 2005/12/11 13:52:51 rmanfredi Exp $
+ * $Id: uploads.c 19325 2011-07-07 12:48:24Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -23,15 +23,17 @@
  *----------------------------------------------------------------------
  */
 
+#include "common.h"
+
+RCSID("$Id: uploads.c 19325 2011-07-07 12:48:24Z rmanfredi $")
+
 #include "gtk/gui.h"
 
-RCSID("$Id: uploads.c,v 1.18 2005/12/11 13:52:51 rmanfredi Exp $");
-
-#include "interface-glade.h"
+#include "gtk/columns.h"
+#include "gtk/misc.h"
+#include "gtk/settings.h"
 #include "gtk/uploads.h"
 #include "gtk/uploads_common.h"
-#include "gtk/columns.h"
-#include "gtk/notebooks.h"
 
 #include "if/bridge/ui2c.h"
 
@@ -40,12 +42,12 @@ RCSID("$Id: uploads.c,v 1.18 2005/12/11 13:52:51 rmanfredi Exp $");
 #include "lib/iso3166.h"
 #include "lib/tm.h"
 #include "lib/utf8.h"
+#include "lib/walloc.h"
+
 #include "lib/override.h"	/* Must be the last header included */
 
-#define UPDATE_MIN	300		/**< Update screen every 5 minutes at least */
-
-static gboolean uploads_remove_lock = FALSE;
-static guint uploads_rows_done = 0;
+static gboolean uploads_remove_lock;
+static guint uploads_rows_done;
 
 static gint find_row(gnet_upload_t u, upload_row_data_t **data);
 
@@ -64,24 +66,20 @@ static void uploads_gui_add_upload(gnet_upload_info_t *u);
  * try to use the handle to communicate with the backend.
  */
 static void
-upload_removed(gnet_upload_t uh, const gchar *reason,
-    guint32 unused_running, guint32 unused_registered)
+upload_removed(gnet_upload_t uh, const gchar *reason)
 {
     gint row;
     upload_row_data_t *data;
 
-	(void) unused_running;
-	(void) unused_registered;
-
     /* Invalidate row and remove it from the gui if autoclear is on */
     row = find_row(uh, &data);
     if (row != -1) {
         GtkCList *clist =
-            GTK_CLIST(lookup_widget(main_window, "clist_uploads"));
+            GTK_CLIST(gui_main_window_lookup("clist_uploads"));
         data->valid = FALSE;
 
         gtk_widget_set_sensitive(
-            lookup_widget(main_window, "button_uploads_clear_completed"),
+            gui_main_window_lookup("button_uploads_clear_completed"),
             TRUE);
 
         if (reason != NULL)
@@ -95,13 +93,10 @@ upload_removed(gnet_upload_t uh, const gchar *reason,
  * Adds the upload to the gui.
  */
 static void
-upload_added(gnet_upload_t n, guint32 unused_running, guint32 unused_registered)
+upload_added(gnet_upload_t n)
 {
     gnet_upload_info_t *info;
 
-	(void) unused_running;
-	(void) unused_registered;
-
     info = guc_upload_get_info(n);
     uploads_gui_add_upload(info);
     guc_upload_free_info(info);
@@ -112,14 +107,10 @@ upload_added(gnet_upload_t n, guint32 unused_running, guint32 unused_registered)
  * This updates the upload information in the gui.
  */
 static void
-upload_info_changed(gnet_upload_t u,
-	guint32 unused_running, guint32 unused_registered)
+upload_info_changed(gnet_upload_t u)
 {
     gnet_upload_info_t *info;
 
-	(void) unused_running;
-	(void) unused_registered;
-
     info = guc_upload_get_info(u);
     uploads_gui_update_upload_info(info);
     guc_upload_free_info(info);
@@ -140,13 +131,9 @@ find_row(gnet_upload_t u, upload_row_data_t **data)
 {
     GtkCList *clist;
     GList *iter;
-    upload_row_data_t fake;
     gint row = 0;
 
-    fake.handle = u;
-    fake.valid  = TRUE;
-
-    clist = GTK_CLIST(lookup_widget(main_window, "clist_uploads"));
+    clist = GTK_CLIST(gui_main_window_lookup("clist_uploads"));
 
     for (iter = clist->row_list; iter != NULL; iter = g_list_next(iter)) {
         GtkCListRow *r = iter->data;
@@ -180,11 +167,8 @@ upload_row_data_t *
 uploads_gui_get_row_data(gnet_upload_t uhandle)
 {
     upload_row_data_t *rd;
-	gint row;
 
-    row =  find_row(uhandle, &rd);
-
-	return row == -1 ? NULL : rd;
+	return find_row(uhandle, &rd) < 0 ? NULL : rd;
 }
 
 static void
@@ -198,7 +182,7 @@ uploads_gui_update_upload_info(gnet_upload_info_t *u)
 	gchar range_tmp[256];
 	guint range_len;
 
-    clist_uploads = GTK_CLIST(lookup_widget(main_window, "clist_uploads"));
+    clist_uploads = GTK_CLIST(gui_main_window_lookup("clist_uploads"));
     row =  find_row(u->upload_handle, &rd);
 	if (row == -1) {
         g_warning("%s: no matching row found [handle=%u]",
@@ -215,16 +199,18 @@ uploads_gui_update_upload_info(gnet_upload_info_t *u)
 		gtk_clist_set_text(clist_uploads, row, c_ul_size, "...");
 		gtk_clist_set_text(clist_uploads, row, c_ul_range, "...");
 	} else {
-		g_strlcpy(size_tmp, short_size(u->file_size), sizeof(size_tmp));
+		g_strlcpy(size_tmp, short_size(u->file_size, show_metric_units()),
+			sizeof size_tmp);
 
-		range_len = gm_snprintf(range_tmp, sizeof(range_tmp), "%s%s",
-			u->partial ? "*" : "",
-			short_size(u->range_end - u->range_start + 1));
+        range_len = gm_snprintf(range_tmp, sizeof range_tmp, "%s%s",
+            short_size(u->range_end - u->range_start + 1,
+				show_metric_units()),
+			u->partial ? _(" (partial)") : "");
 
 		if (u->range_start)
 			range_len += gm_snprintf(
 				&range_tmp[range_len], sizeof(range_tmp)-range_len,
-					" @ %s", short_size(u->range_start));
+					" @ %s", short_size(u->range_start, show_metric_units()));
 
 		g_assert(range_len < sizeof(range_tmp));
 
@@ -264,7 +250,7 @@ uploads_gui_update_upload_info(gnet_upload_info_t *u)
 static void
 free_data(gpointer o)
 {
-	g_free(o);
+	wfree(o, sizeof(upload_row_data_t));
 }
 
 /**
@@ -281,23 +267,25 @@ uploads_gui_add_upload(gnet_upload_info_t *u)
     GtkWidget *clist_uploads;
     upload_row_data_t *data;
 
-    clist_uploads = lookup_widget(main_window, "clist_uploads");
+    clist_uploads = gui_main_window_lookup("clist_uploads");
 
-	memset(titles, 0, sizeof(titles));
+	ZERO(&titles);
 
     if ((u->range_start == 0) && (u->range_end == 0)) {
         titles[c_ul_size] = titles[c_ul_range] =  "...";
     } else {
-        g_strlcpy(size_tmp, short_size(u->file_size), sizeof(size_tmp));
+        g_strlcpy(size_tmp, short_size(u->file_size, show_metric_units()),
+			sizeof size_tmp);
 
-        range_len = gm_snprintf(range_tmp, sizeof(range_tmp), "%s%s",
-			u->partial ? "*" : "",
-            short_size(u->range_end - u->range_start + 1));
+        range_len = gm_snprintf(range_tmp, sizeof range_tmp, "%s%s",
+            short_size(u->range_end - u->range_start + 1,
+				show_metric_units()),
+			u->partial ? _(" (partial)") : "");
 
         if (u->range_start)
             range_len += gm_snprintf(
                 &range_tmp[range_len], sizeof(range_tmp)-range_len,
-                " @ %s", short_size(u->range_start));
+                " @ %s", short_size(u->range_start, show_metric_units()));
 
         g_assert(range_len < sizeof(range_tmp));
 
@@ -315,7 +303,7 @@ uploads_gui_add_upload(gnet_upload_info_t *u)
 	titles[c_ul_progress]   = "...";
 	titles[c_ul_status]   = "...";
 
-    data = g_new(upload_row_data_t, 1);
+    WALLOC0(data);
     data->handle      = u->upload_handle;
     data->range_start = u->range_start;
     data->range_end   = u->range_end;
@@ -330,56 +318,12 @@ uploads_gui_add_upload(gnet_upload_info_t *u)
         data, free_data);
 }
 
-/***
- *** Public functions
- ***/
-
-void
-uploads_gui_early_init(void)
-{
-    popup_uploads = create_popup_uploads();
-}
-
-void
-uploads_gui_init(void)
-{
-    gtk_clist_set_column_justification(
-        GTK_CLIST(lookup_widget(main_window, "clist_uploads")),
-        c_ul_size, GTK_JUSTIFY_RIGHT);
-
-    gtk_clist_set_column_justification(
-        GTK_CLIST(lookup_widget(main_window, "clist_uploads")),
-        c_ul_progress, GTK_JUSTIFY_RIGHT);
-
-	gtk_clist_column_titles_passive(
-        GTK_CLIST(lookup_widget(main_window, "clist_uploads")));
-
-    guc_upload_add_upload_added_listener(upload_added);
-    guc_upload_add_upload_removed_listener(upload_removed);
-    guc_upload_add_upload_info_changed_listener
-		(upload_info_changed);
-}
-
-/**
- * Unregister callbacks in the backend and clean up.
- */
-void
-uploads_gui_shutdown(void)
-{
-    guc_upload_remove_upload_added_listener(upload_added);
-    guc_upload_remove_upload_removed_listener(upload_removed);
-    guc_upload_remove_upload_info_changed_listener
-		(upload_info_changed);
-}
-
 /**
  * Update all the uploads at the same time.
  */
-void
+static void
 uploads_gui_update_display(time_t now)
 {
-	static GtkNotebook *notebook = NULL;
-    static time_t last_update = 0;
 	GtkCList *clist;
 	GList *iter;
 	gint row = 0;
@@ -387,29 +331,8 @@ uploads_gui_update_display(time_t now)
     GSList *to_remove = NULL;
     GSList *sl;
 	gboolean all_removed = TRUE;
-	gint current_page;
 
-	/*
-	 * Usually don't perform updates if nobody is watching.  However,
-	 * we do need to perform periodic cleanup of dead entries or the
-	 * memory usage will grow.  Perform an update every UPDATE_MIN minutes
-	 * at least.
-	 *		--RAM, 28/12/2003
-	 */
-
-	if (notebook == NULL)
-		notebook = GTK_NOTEBOOK(lookup_widget(main_window, "notebook_main"));
-
-	current_page = gtk_notebook_get_current_page(notebook);
-	if (current_page != nb_main_page_uploads && now - last_update < UPDATE_MIN)
-		return;
-
-    if (last_update == now)
-        return;
-
-    last_update = now;
-
-    clist = GTK_CLIST(lookup_widget(main_window, "clist_uploads"));
+    clist = GTK_CLIST(gui_main_window_lookup("clist_uploads"));
     gtk_clist_freeze(clist);
 
 	row = 0;
@@ -450,7 +373,7 @@ uploads_gui_update_display(time_t now)
 
 	if (all_removed)
 		gtk_widget_set_sensitive(
-			lookup_widget(main_window, "button_uploads_clear_completed"),
+			gui_main_window_lookup("button_uploads_clear_completed"),
 			FALSE);
 }
 
@@ -461,7 +384,7 @@ uploads_clear_helper(gpointer unused_udata)
     GSList *to_remove= NULL;
     GSList *sl;
     guint row = 0;
-    GtkCList *clist = GTK_CLIST(lookup_widget(main_window, "clist_uploads"));
+    GtkCList *clist = GTK_CLIST(gui_main_window_lookup("clist_uploads"));
 
 	(void) unused_udata;
     gtk_clist_freeze(clist);
@@ -492,8 +415,8 @@ uploads_clear_helper(gpointer unused_udata)
     gtk_clist_thaw(clist);
 
     if (iter == NULL) {
-		gtk_widget_set_sensitive(lookup_widget(
-			main_window, "button_uploads_clear_completed"), FALSE);
+		gtk_widget_set_sensitive(
+			gui_main_window_lookup("button_uploads_clear_completed"), FALSE);
     	uploads_remove_lock = FALSE;
     	return FALSE;
     }
@@ -511,4 +434,74 @@ uploads_gui_clear_completed(void)
 	}
 }
 
+static void
+uploads_gui_timer(time_t now)
+{
+	if (uploads_gui_update_required(now)) {
+		uploads_gui_update_display(now);
+	}
+}
+
+static const char *
+uploads_gui_column_title(int column)
+{
+	switch ((enum c_gnet) column) {
+	case c_ul_filename:	return _("Filename");
+	case c_ul_host:		return _("Host");
+	case c_ul_loc:		return _("Country");
+	case c_ul_size:		return _("Size");
+	case c_ul_range:	return _("Range");
+	case c_ul_agent:	return _("User-Agent");
+	case c_ul_progress:	return _("Progress");
+	case c_ul_status:	return _("Status");
+	case c_ul_num:
+		break;
+	}
+	g_assert_not_reached();
+	return NULL;
+}
+
+/***
+ *** Public functions
+ ***/
+
+void
+uploads_gui_init(void)
+{
+	GtkCList *clist;
+	unsigned i;
+
+	clist = GTK_CLIST(gui_main_window_lookup("clist_uploads"));
+    gtk_clist_set_column_justification(clist, c_ul_size, GTK_JUSTIFY_RIGHT);
+    gtk_clist_set_column_justification(clist, c_ul_progress, GTK_JUSTIFY_RIGHT);
+	gtk_clist_column_titles_passive(clist);
+	for (i = 0; i < c_ul_num; i++) {
+    	gtk_clist_set_column_name(clist, i, uploads_gui_column_title(i));
+	}
+	clist_restore_visibility(clist, PROP_UPLOADS_COL_VISIBLE);
+	clist_restore_widths(clist, PROP_UPLOADS_COL_WIDTHS);
+
+    guc_upload_add_upload_added_listener(upload_added);
+    guc_upload_add_upload_removed_listener(upload_removed);
+    guc_upload_add_upload_info_changed_listener(upload_info_changed);
+
+	main_gui_add_timer(uploads_gui_timer);
+}
+
+/**
+ * Unregister callbacks in the backend and clean up.
+ */
+void
+uploads_gui_shutdown(void)
+{
+	GtkCList *clist;
+
+	clist = GTK_CLIST(gui_main_window_lookup("clist_uploads"));
+	clist_save_visibility(clist, PROP_UPLOADS_COL_VISIBLE);
+	clist_save_widths(clist, PROP_UPLOADS_COL_WIDTHS);
+    guc_upload_remove_upload_added_listener(upload_added);
+    guc_upload_remove_upload_removed_listener(upload_removed);
+    guc_upload_remove_upload_info_changed_listener(upload_info_changed);
+}
+
 /* vi: set ts=4 sw=4 cindent: */
