@@ -1,5 +1,5 @@
 /*
- * $Id: pbarcellrenderer.c,v 1.11 2005/09/11 13:27:26 daichik Exp $
+ * $Id: pbarcellrenderer.c 19142 2011-05-23 20:44:04Z rmanfredi $
  *
  * Copyright (c) 2004, Russell Francis
  *
@@ -61,7 +61,7 @@
 
 #include "gtk/gui.h"
 
-RCSID("$Id: pbarcellrenderer.c,v 1.11 2005/09/11 13:27:26 daichik Exp $");
+RCSID("$Id: pbarcellrenderer.c 19142 2011-05-23 20:44:04Z rmanfredi $")
 
 #include "pbarcellrenderer.h"
 
@@ -173,10 +173,7 @@ gtk_cell_renderer_progress_get_type(void)
 static void
 gtk_cell_renderer_progress_init(GtkCellRendererProgress *cellprogress)
 {
-    cellprogress->priv =
-		(GtkCellRendererProgressPriv*) g_new0(GtkCellRendererProgressPriv, 1);
-
-	cellprogress->priv->value = 0;
+    WALLOC0(cellprogress->priv);
 }
 
 /**
@@ -433,7 +430,8 @@ static void
 gtk_cell_renderer_progress_finalize(GObject *object)
 {
 	GtkCellRendererProgress *cellprogress = GTK_CELL_RENDERER_PROGRESS(object);
-	G_FREE_NULL(cellprogress->priv);
+	WFREE(cellprogress->priv);
+	cellprogress->priv = NULL;
 	(*G_OBJECT_CLASS(parent_class)->finalize)(object);
 }
 
