@@ -1,5 +1,5 @@
 /*
- * $Id: uploads.c 12195 2006-11-03 02:20:22Z cbiere $
+ * $Id: uploads.c 19191 2011-05-30 22:18:28Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -35,17 +35,10 @@
 
 #include "gtk/gui.h"
 
-#include "interface-glade.h"
-#if !GTK_CHECK_VERSION(2,5,0)
-#include "pbarcellrenderer.h"
-#endif
-
 #include "gtk/uploads.h"
 #include "gtk/uploads_common.h"
 #include "gtk/columns.h"
-#include "gtk/gtk-missing.h"
 #include "gtk/misc.h"
-#include "gtk/notebooks.h"
 #include "gtk/settings.h"
 
 #include "if/gui_property.h"
@@ -60,21 +53,19 @@
 #include "lib/walloc.h"
 #include "lib/override.h"		/* Must be the last header included */
 
-RCSID("$Id: uploads.c 12195 2006-11-03 02:20:22Z cbiere $")
+RCSID("$Id: uploads.c 19191 2011-05-30 22:18:28Z rmanfredi $")
 
-#define UPDATE_MIN	300		/**< Update screen every 5 minutes at least */
+static gboolean uploads_remove_lock;
+static gboolean uploads_shutting_down;
 
-static gboolean uploads_remove_lock = FALSE;
-static gboolean uploads_shutting_down = FALSE;
-
-static GtkTreeView *treeview_uploads = NULL;
-static GtkListStore *store_uploads = NULL;
-static GtkWidget *button_uploads_clear_completed = NULL;
+static GtkTreeView *treeview_uploads;
+static GtkListStore *store_uploads;
+static GtkWidget *button_uploads_clear_completed;
 
 /** hash table for fast handle -> GtkTreeIter mapping */
-static GHashTable *upload_handles = NULL;
+static GHashTable *upload_handles;
 /** list of all *removed* uploads; contains the handles */
-static GSList *sl_removed_uploads = NULL;
+static GSList *sl_removed_uploads;
 
 static void uploads_gui_update_upload_info(const gnet_upload_info_t *u);
 static void uploads_gui_add_upload(gnet_upload_info_t *u);
@@ -82,7 +73,7 @@ static void uploads_gui_add_upload(gnet_upload_info_t *u);
 static const char * const column_titles[UPLOADS_GUI_VISIBLE_COLUMNS] = {
 	N_("Filename"),
 	N_("Host"),
-	N_("Loc"),
+	N_("Country"),
 	N_("Size"),
 	N_("Range"),
 	N_("User-Agent"),
@@ -152,13 +143,9 @@ on_button_press_event(GtkWidget *unused_widget, GdkEventButton *event,
  * try to use the handle to communicate with the backend.
  */
 static void
-upload_removed(gnet_upload_t uh, const gchar *reason,
-		guint32 running, guint32 registered)
+upload_removed(gnet_upload_t uh, const gchar *reason)
 {
-    upload_row_data_t *rd = NULL;
-
-	(void) running;
-	(void) registered;
+    upload_row_data_t *rd;
 
     /* Invalidate row and remove it from the GUI if autoclear is on */
 	rd = find_upload(uh);
@@ -180,13 +167,10 @@ upload_removed(gnet_upload_t uh, const gchar *reason,
  * Adds the upload to the gui.
  */
 static void
-upload_added(gnet_upload_t n, guint32 running, guint32 registered)
+upload_added(gnet_upload_t n)
 {
     gnet_upload_info_t *info;
 
-	(void) running;
-	(void) registered;
-
     info = guc_upload_get_info(n);
     uploads_gui_add_upload(info);
     guc_upload_free_info(info);
@@ -206,14 +190,10 @@ uploads_gui_get_row_data(gnet_upload_t uhandle)
  * This updates the upload information in the gui.
  */
 static void
-upload_info_changed(gnet_upload_t u,
-    guint32 running, guint32 registered)
+upload_info_changed(gnet_upload_t u)
 {
     gnet_upload_info_t *info;
 
-	(void) running;
-	(void) registered;
-
     info = guc_upload_get_info(u);
     uploads_gui_update_upload_info(info);
     guc_upload_free_info(info);
@@ -314,11 +294,8 @@ uploads_gui_update_upload_info(const gnet_upload_info_t *u)
 
 	/* Exploit that u->name is an atom! */
 	if (u->name != rd->name) {
-		g_assert(NULL != u->name);
-		if (NULL != rd->name)
-			atom_str_free(rd->name);
-		rd->name = atom_str_get(u->name);
-
+		atom_str_free_null(&rd->name);
+		rd->name = u->name ? atom_str_get(u->name) : NULL;
 		gtk_list_store_set(store_uploads, &rd->iter,
 			c_ul_filename, rd->name,
 			(-1));
@@ -326,10 +303,8 @@ uploads_gui_update_upload_info(const gnet_upload_info_t *u)
 
 	/* Exploit that u->user_agent is an atom! */
 	if (u->user_agent != rd->user_agent) {
-		g_assert(NULL != u->user_agent);
-		if (NULL != rd->user_agent)
-			atom_str_free(rd->user_agent);
-		rd->user_agent = atom_str_get(u->user_agent);
+		atom_str_free_null(&rd->user_agent);
+		rd->user_agent = u->user_agent ? atom_str_get(u->user_agent) : NULL;
 		gtk_list_store_set(store_uploads, &rd->iter,
 			c_ul_agent, rd->user_agent,
 			(-1));
@@ -368,12 +343,13 @@ uploads_gui_add_upload(gnet_upload_info_t *u)
 {
 	gint range_len, progress;
 	const gchar *titles[UPLOADS_GUI_VISIBLE_COLUMNS];
-    upload_row_data_t *rd = walloc(sizeof *rd);
+    upload_row_data_t *rd;
 	gnet_upload_status_t status;
 	static gchar size_tmp[256];
 
-	memset(titles, 0, sizeof titles);
+	ZERO(&titles);
 
+	WALLOC(rd);
     rd->handle      = u->upload_handle;
     rd->range_start = u->range_start;
     rd->range_end   = u->range_end;
@@ -525,14 +501,152 @@ create_uploads_model(void)
 	return GTK_LIST_STORE(store);
 }
 
+static inline void
+free_row_data(upload_row_data_t *rd)
+{
+	atom_str_free_null(&rd->user_agent);
+	atom_str_free_null(&rd->name);
+	WFREE(rd);
+}
 
-/***
- *** Public functions
- ***/
+static inline void
+free_handle(gpointer key, gpointer value,
+	gpointer user_data)
+{
+	(void) key;
+	(void) user_data;
+
+	free_row_data(value);
+}
+
+static inline void
+remove_row(upload_row_data_t *rd, remove_row_ctx_t *ctx)
+{
+	g_assert(NULL != rd);
+	if (ctx->force || upload_should_remove(ctx->now, rd)) {
+    	gtk_list_store_remove(store_uploads, &rd->iter);
+		free_row_data(rd);
+	} else
+		ctx->sl_remaining = g_slist_prepend(ctx->sl_remaining, rd);
+}
+
+static inline void
+update_row(gpointer key, gpointer data, gpointer unused_udata)
+{
+	time_t now;
+	upload_row_data_t *rd = data;
+	gnet_upload_status_t status;
+	guint progress;
+
+	(void) unused_udata;
+	g_assert(NULL != rd);
+	g_assert(GPOINTER_TO_UINT(key) == rd->handle);
+
+	now = tm_time();
+	if (delta_time(now, rd->last_update) < 2)
+		return;
+
+	rd->last_update = now;
+	guc_upload_get_status(rd->handle, &status);
+	progress = 100.0 * uploads_gui_progress(&status, rd);
+	progress = MIN(progress, 100);
+	gtk_list_store_set(store_uploads, &rd->iter,
+		c_ul_progress, progress,
+		c_ul_status, uploads_gui_status_str(&status, rd),
+		(-1));
+}
+
+/**
+ * Update all the uploads at the same time.
+ */
+static void
+uploads_gui_update_display(time_t now)
+{
+   	static gboolean locked = FALSE;
+	remove_row_ctx_t ctx;
+
+	ctx.force = FALSE;
+	ctx.now = now;
+	ctx.sl_remaining = NULL;
+
+	g_return_if_fail(!locked);
+	locked = TRUE;
+
+	g_object_freeze_notify(G_OBJECT(treeview_uploads));
+	/* Remove all rows with `removed' uploads. */
+	G_SLIST_FOREACH_WITH_DATA(sl_removed_uploads, remove_row, &ctx);
+	g_slist_free(sl_removed_uploads);
+	sl_removed_uploads = ctx.sl_remaining;
+
+	/* Update the status column for all active uploads. */
+	g_hash_table_foreach(upload_handles, update_row, NULL);
+	g_object_thaw_notify(G_OBJECT(treeview_uploads));
+
+	gtk_widget_set_sensitive(button_uploads_clear_completed,
+		NULL != sl_removed_uploads);
+
+	locked = FALSE;
+}
+
+static void
+uploads_gui_timer(time_t now)
+{
+	if (uploads_gui_update_required(now)) {
+		uploads_gui_update_display(now);
+	}
+}
+
+static gboolean
+uploads_clear_helper(gpointer user_data)
+{
+	guint counter = 0;
+    GSList *sl, *sl_remaining = NULL;
+	remove_row_ctx_t ctx = { TRUE, 0, NULL };
+
+	(void) user_data;
+
+	if (uploads_shutting_down)
+		return FALSE; /* Finished. */
+
+	g_object_freeze_notify(G_OBJECT(treeview_uploads));
+	/* Remove all rows with `removed' uploads. */
+
+	G_SLIST_FOREACH_WITH_DATA(sl_removed_uploads, remove_row, &ctx);
+	g_slist_free(sl_removed_uploads);
+	sl_removed_uploads = ctx.sl_remaining;
+
+    for (sl = sl_removed_uploads; sl; sl = g_slist_next(sl_removed_uploads)) {
+		remove_row((upload_row_data_t *) sl->data, &ctx);
+		/* Interrupt and come back later to prevent GUI stalling. */
+		if (0 == (++counter & 0x7f)) {
+			/* Remember which elements haven't been traversed yet. */
+			sl_remaining = g_slist_remove_link(sl, sl);
+			break;
+		}
+    }
+	g_object_thaw_notify(G_OBJECT(treeview_uploads));
+
+	/* The elements' data has been freed or is now referenced
+	 * by ctx->remaining. */
+	g_slist_free(sl_removed_uploads);
+	sl_removed_uploads = g_slist_concat(ctx.sl_remaining, sl_remaining);
+
+    if (NULL == sl_removed_uploads) {
+		gtk_widget_set_sensitive(button_uploads_clear_completed, FALSE);
+    	uploads_remove_lock = FALSE;
+    	return FALSE; /* Finished. */
+    }
+
+    return TRUE; /* More rows to remove. */
+}
 
 void
-uploads_gui_early_init(void)
+uploads_gui_clear_completed(void)
 {
+	if (!uploads_remove_lock) {
+		uploads_remove_lock = TRUE;
+		gtk_timeout_add(100, uploads_clear_helper, store_uploads);
+	}
 }
 
 void
@@ -561,13 +675,14 @@ uploads_gui_init(void)
 	treeview_uploads =
 		GTK_TREE_VIEW(gui_main_window_lookup("treeview_uploads"));
 	gtk_tree_view_set_model(treeview_uploads, GTK_TREE_MODEL(store_uploads));
+	tree_view_set_fixed_height_mode(treeview_uploads, TRUE);
 
-	for (i = 0; i < G_N_ELEMENTS(cols); i++)
+	for (i = 0; i < G_N_ELEMENTS(cols); i++) {
 		add_column(cols[i].id, cols[i].sortfunc,
 			c_ul_progress == cols[i].id
 				? GTK_TYPE_CELL_RENDERER_PROGRESS
 				: GTK_TYPE_CELL_RENDERER_TEXT);
-
+	}
 	tree_view_restore_widths(treeview_uploads, PROP_UPLOADS_COL_WIDTHS);
 	tree_view_restore_visibility(treeview_uploads, PROP_UPLOADS_COL_VISIBLE);
 
@@ -575,174 +690,12 @@ uploads_gui_init(void)
 
     guc_upload_add_upload_added_listener(upload_added);
     guc_upload_add_upload_removed_listener(upload_removed);
-    guc_upload_add_upload_info_changed_listener
-		(upload_info_changed);
+    guc_upload_add_upload_info_changed_listener(upload_info_changed);
 
-	g_signal_connect(GTK_OBJECT(treeview_uploads), "button_press_event",
-		G_CALLBACK(on_button_press_event), NULL);
-}
+	gui_signal_connect(treeview_uploads,
+		"button_press_event", on_button_press_event, NULL);
 
-static inline void
-free_row_data(upload_row_data_t *rd)
-{
-	if (NULL != rd->user_agent) {
-		atom_str_free(rd->user_agent);
-		rd->user_agent = NULL;
-	}
-	if (NULL != rd->name) {
-		atom_str_free(rd->name);
-		rd->user_agent = NULL;
-	}
-	wfree(rd, sizeof *rd);
-}
-
-static inline void
-free_handle(gpointer key, gpointer value,
-	gpointer user_data)
-{
-	(void) key;
-	(void) user_data;
-
-	free_row_data(value);
-}
-
-static inline void
-remove_row(upload_row_data_t *rd, remove_row_ctx_t *ctx)
-{
-	g_assert(NULL != rd);
-	if (ctx->force || upload_should_remove(ctx->now, rd)) {
-    	gtk_list_store_remove(store_uploads, &rd->iter);
-		free_row_data(rd);
-	} else
-		ctx->sl_remaining = g_slist_prepend(ctx->sl_remaining, rd);
-}
-
-static inline void
-update_row(gpointer key, gpointer data, gpointer user_data)
-{
-	time_t now = *(const time_t *) user_data;
-	upload_row_data_t *rd = data;
-	gnet_upload_status_t status;
-	gint progress;
-
-	g_assert(NULL != rd);
-	g_assert(GPOINTER_TO_UINT(key) == rd->handle);
-	if (delta_time(now, rd->last_update) < 2)
-		return;
-
-	rd->last_update = now;
-	guc_upload_get_status(rd->handle, &status);
-	progress = 100.0 * uploads_gui_progress(&status, rd);
-	gtk_list_store_set(store_uploads, &rd->iter,
-		c_ul_progress, CLAMP(progress, 0, 100),
-		c_ul_status, uploads_gui_status_str(&status, rd),
-		(-1));
-}
-
-/**
- * Update all the uploads at the same time.
- */
-void
-uploads_gui_update_display(time_t now)
-{
-    static time_t last_update;
-	gint current_page;
-	static GtkNotebook *notebook = NULL;
-
-	/*
-	 * Usually don't perform updates if nobody is watching.  However,
-	 * we do need to perform periodic cleanup of dead entries or the
-	 * memory usage will grow.  Perform an update every UPDATE_MIN minutes
-	 * at least.
-	 *		--RAM, 28/12/2003
-	 */
-
-	if (notebook == NULL)
-		notebook = GTK_NOTEBOOK(gui_main_window_lookup("notebook_main"));
-
-	current_page = gtk_notebook_get_current_page(notebook);
-	if (
-		current_page != nb_main_page_uploads &&
-		delta_time(now, last_update) < UPDATE_MIN
-	) {
-		return;
-	}
-
-    if (last_update != now) {
-    	static gboolean locked = FALSE;
-		remove_row_ctx_t ctx;
-
-    	last_update = now;
-		ctx.force = FALSE;
-		ctx.now = now;
-		ctx.sl_remaining = NULL;
-
-		g_return_if_fail(!locked);
-		locked = TRUE;
-
-		/* Remove all rows with `removed' uploads. */
-		G_SLIST_FOREACH_WITH_DATA(sl_removed_uploads, remove_row, &ctx);
-		g_slist_free(sl_removed_uploads);
-		sl_removed_uploads = ctx.sl_remaining;
-
-		/* Update the status column for all active uploads. */
-		g_hash_table_foreach(upload_handles, update_row, &now);
-
-		if (NULL == sl_removed_uploads)
-			gtk_widget_set_sensitive(button_uploads_clear_completed, FALSE);
-
-		locked = FALSE;
-	}
-}
-
-static gboolean
-uploads_clear_helper(gpointer user_data)
-{
-	guint counter = 0;
-    GSList *sl, *sl_remaining = NULL;
-	remove_row_ctx_t ctx = { TRUE, 0, NULL };
-
-	(void) user_data;
-
-	if (uploads_shutting_down)
-		return FALSE; /* Finished. */
-
-	/* Remove all rows with `removed' uploads. */
-
-	G_SLIST_FOREACH_WITH_DATA(sl_removed_uploads, remove_row, &ctx);
-	g_slist_free(sl_removed_uploads);
-	sl_removed_uploads = ctx.sl_remaining;
-
-    for (sl = sl_removed_uploads; sl; sl = g_slist_next(sl_removed_uploads)) {
-		remove_row((upload_row_data_t *) sl->data, &ctx);
-		/* Interrupt and come back later to prevent GUI stalling. */
-		if (0 == (++counter & 0x7f)) {
-			/* Remember which elements haven't been traversed yet. */
-			sl_remaining = g_slist_remove_link(sl, sl);
-			break;
-		}
-    }
-	/* The elements' data has been freed or is now referenced
-	 * by ctx->remaining. */
-	g_slist_free(sl_removed_uploads);
-	sl_removed_uploads = g_slist_concat(ctx.sl_remaining, sl_remaining);
-
-    if (NULL == sl_removed_uploads) {
-		gtk_widget_set_sensitive(button_uploads_clear_completed, FALSE);
-    	uploads_remove_lock = FALSE;
-    	return FALSE; /* Finished. */
-    }
-
-    return TRUE; /* More rows to remove. */
-}
-
-void
-uploads_gui_clear_completed(void)
-{
-	if (!uploads_remove_lock) {
-		uploads_remove_lock = TRUE;
-		gtk_timeout_add(100, uploads_clear_helper, store_uploads);
-	}
+	main_gui_add_timer(uploads_gui_timer);
 }
 
 /**
@@ -763,11 +716,9 @@ uploads_gui_shutdown(void)
 	gtk_list_store_clear(store_uploads);
 
 	g_hash_table_foreach(upload_handles, free_handle, NULL);
-	g_hash_table_destroy(upload_handles);
-	upload_handles = NULL;
+	gm_hash_table_destroy_null(&upload_handles);
 	G_SLIST_FOREACH(sl_removed_uploads, free_row_data);
-	g_slist_free(sl_removed_uploads);
-	sl_removed_uploads = NULL;
+	gm_slist_free_null(&sl_removed_uploads);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
