@@ -1,5 +1,5 @@
 /*
- * $Id: nodes.h 7120 2004-10-21 16:44:17Z cbiere $
+ * $Id: nodes.h 18899 2011-03-30 15:17:39Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -34,7 +34,7 @@ void nodes_gui_init(void);
 void nodes_gui_shutdown(void);
 
 void nodes_gui_add_node(gnet_node_info_t *n);
-void nodes_gui_remove_node(gnet_node_t n);
+void nodes_gui_remove_node(const struct nid *node_id);
 void nodes_gui_update_node(gnet_node_info_t *, gboolean);
 void nodes_gui_update_nodes_display(time_t now);
 
