@@ -1,5 +1,5 @@
 /*
- * $Id: nodes_common.c 11755 2006-08-26 21:08:47Z cbiere $
+ * $Id: nodes_common.c 19142 2011-05-23 20:44:04Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -36,7 +36,9 @@
 
 #include "gui.h"
 
-RCSID("$Id: nodes_common.c 11755 2006-08-26 21:08:47Z cbiere $")
+RCSID("$Id: nodes_common.c 19142 2011-05-23 20:44:04Z rmanfredi $")
+
+#include "gtk-gnutella.h"
 
 #include "nodes_common.h"
 #include "settings.h"
@@ -48,12 +50,22 @@ RCSID("$Id: nodes_common.c 11755 2006-08-26 21:08:47Z cbiere $")
 #include "if/core/sockets.h"
 #include "if/gui_property_priv.h"
 
+#include "lib/ascii.h"
 #include "lib/glib-missing.h"
-#include "lib/misc.h"
+#include "lib/parse.h"
+#include "lib/random.h"
+
+#include "lib/halloc.h"
 #include "lib/walloc.h"
 #include "lib/override.h"	/* Must be the last header included */
 
-static gchar gui_tmp[4096];
+#define UPDATE_MIN	60		/**< Update screen every minute at least */
+
+GtkMenu *
+nodes_gui_get_popup_menu(void)
+{
+	return GTK_MENU(gui_popup_nodes());
+}
 
 /**
  * Compute info string for node.
@@ -63,6 +75,7 @@ static gchar gui_tmp[4096];
 const gchar *
 nodes_gui_common_status_str(const gnet_node_status_t *n)
 {
+	static gchar gui_tmp[4096];
 	const gchar *a;
 
 	switch (n->status) {
@@ -82,9 +95,9 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 		if (n->sent || n->received) {
 			size_t slen = 0;
 
-			if (!node_show_detailed_info) {
+			if (!GUI_PROPERTY(node_show_detailed_info)) {
 				gm_snprintf(gui_tmp, sizeof(gui_tmp),
-					"TX=%d RX=%d Q=%d,%d%% %s",
+					"TX=%u RX=%u Q=%u,%u%% %s",
 					n->sent, n->received,
 					n->mqueue_count, n->mqueue_percent_used,
 					n->in_tx_swift_control ? " [SW]" :
@@ -93,26 +106,29 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 				break;
 			}
 
-			if (n->tx_compressed && show_gnet_info_txc)
-				slen += gm_snprintf(gui_tmp, sizeof(gui_tmp), "TXc=%d,%d%%",
-					(gint) n->sent, (gint) (n->tx_compression_ratio * 100));
+			if (n->tx_compressed && GUI_PROPERTY(show_gnet_info_txc))
+				slen += gm_snprintf(gui_tmp, sizeof(gui_tmp), "TXc=%u,%d%%",
+						n->sent, (int) (n->tx_compression_ratio * 100.0));
 			else
-				slen += gm_snprintf(gui_tmp, sizeof(gui_tmp), "TX=%d",
-							(gint) n->sent);
+				slen += gm_snprintf(gui_tmp, sizeof(gui_tmp), "TX=%u",
+						n->sent);
 
-			if (show_gnet_info_tx_speed || show_gnet_info_tx_wire) {
+			if (
+				GUI_PROPERTY(show_gnet_info_tx_speed) ||
+				GUI_PROPERTY(show_gnet_info_tx_wire)
+			) {
 				gboolean is_first = TRUE;
 
 				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 					" (" /* ')' */);
 
-				if (show_gnet_info_tx_wire) {
+				if (GUI_PROPERTY(show_gnet_info_tx_wire)) {
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 					"%s", compact_size(n->tx_written, show_metric_units()));
 					is_first = FALSE;
 				}
 
-				if (show_gnet_info_tx_speed)
+				if (GUI_PROPERTY(show_gnet_info_tx_speed))
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 						"%s%s", is_first ? "" : ", ",
 						compact_rate(n->tx_bps, show_metric_units()));
@@ -121,27 +137,30 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 					/* '(' */ ")");
 			}
 
-			if (n->rx_compressed && show_gnet_info_rxc)
+			if (n->rx_compressed && GUI_PROPERTY(show_gnet_info_rxc))
 				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-					" RXc=%d,%d%%",
-					(gint) n->received, (gint) (n->rx_compression_ratio * 100));
+					" RXc=%u,%d%%",
+					n->received, (int) (n->rx_compression_ratio * 100.0));
 			else
 				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-					" RX=%d", (gint) n->received);
+					" RX=%u", n->received);
 
-			if (show_gnet_info_rx_speed || show_gnet_info_rx_wire) {
+			if (
+				GUI_PROPERTY(show_gnet_info_rx_speed) ||
+				GUI_PROPERTY(show_gnet_info_rx_wire)
+			) {
 				gboolean is_first = TRUE;
 
 				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 					" (" /* ')' */);
 
-				if (show_gnet_info_rx_wire) {
+				if (GUI_PROPERTY(show_gnet_info_rx_wire)) {
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 						"%s", compact_size(n->rx_given, show_metric_units()));
 					is_first = FALSE;
 				}
 
-				if (show_gnet_info_rx_speed)
+				if (GUI_PROPERTY(show_gnet_info_rx_speed))
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 						"%s%s", is_first ? "" : ", ",
 						compact_rate(n->rx_bps, show_metric_units()));
@@ -151,30 +170,32 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 			}
 
 			if (
-				show_gnet_info_tx_queries || show_gnet_info_rx_queries ||
-				show_gnet_info_gen_queries || show_gnet_info_sq_queries
+				GUI_PROPERTY(show_gnet_info_tx_queries) ||
+				GUI_PROPERTY(show_gnet_info_rx_queries) ||
+				GUI_PROPERTY(show_gnet_info_gen_queries) ||
+				GUI_PROPERTY(show_gnet_info_sq_queries)
 			) {
 				gboolean is_first = TRUE;
 
 				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 					" Query(" /* ')' */);
 
-				if (show_gnet_info_gen_queries) {
+				if (GUI_PROPERTY(show_gnet_info_gen_queries)) {
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-						"Gen=%d", n->squeue_sent);
+						"Gen=%u", n->squeue_sent);
 					is_first = FALSE;
 				}
-				if (show_gnet_info_sq_queries) {
+				if (GUI_PROPERTY(show_gnet_info_sq_queries)) {
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-						"%sQ=%d", is_first ? "" : ", ", n->squeue_count);
+						"%sQ=%u", is_first ? "" : ", ", n->squeue_count);
 					is_first = FALSE;
 				}
-				if (show_gnet_info_tx_queries) {
+				if (GUI_PROPERTY(show_gnet_info_tx_queries)) {
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 						"%sTX=%u", is_first ? "" : ", ", n->tx_queries);
 					is_first = FALSE;
 				}
-				if (show_gnet_info_rx_queries)
+				if (GUI_PROPERTY(show_gnet_info_rx_queries))
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 						"%sRX=%u", is_first ? "" : ", ", n->rx_queries);
 
@@ -182,18 +203,21 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 					/* '(' */ ")");
 			}
 
-			if (show_gnet_info_tx_hits || show_gnet_info_rx_hits) {
+			if (
+				GUI_PROPERTY(show_gnet_info_tx_hits) ||
+				GUI_PROPERTY(show_gnet_info_rx_hits)
+			) {
 				gboolean is_first = TRUE;
 
 				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 					" QHit(" /* ')' */);
 
-				if (show_gnet_info_tx_hits) {
+				if (GUI_PROPERTY(show_gnet_info_tx_hits)) {
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 						"TX=%u", n->tx_qhits);
 					is_first = FALSE;
 				}
-				if (show_gnet_info_rx_hits)
+				if (GUI_PROPERTY(show_gnet_info_rx_hits))
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 						"%sRX=%u", is_first ? "" : ", ", n->rx_qhits);
 
@@ -201,18 +225,21 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 					/* '(' */ ")");
 			}
 
-			if (show_gnet_info_tx_dropped || show_gnet_info_rx_dropped) {
+			if (
+				GUI_PROPERTY(show_gnet_info_tx_dropped) ||
+				GUI_PROPERTY(show_gnet_info_rx_dropped)
+			) {
 				gboolean is_first = TRUE;
 
 				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 					" Drop(" /* ')' */);
 
-				if (show_gnet_info_tx_dropped) {
+				if (GUI_PROPERTY(show_gnet_info_tx_dropped)) {
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 						"TX=%u", n->tx_dropped);
 					is_first = FALSE;
 				}
-				if (show_gnet_info_rx_dropped)
+				if (GUI_PROPERTY(show_gnet_info_rx_dropped))
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 						"%sRX=%u", is_first ? "" : ", ", n->rx_dropped);
 
@@ -220,28 +247,37 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 					/* '(' */ ")");
 			}
 
-			if (show_gnet_info_shared_size || show_gnet_info_shared_files) {
+			if (
+				GUI_PROPERTY(show_gnet_info_shared_size) ||
+				GUI_PROPERTY(show_gnet_info_shared_files)
+			) {
 				gboolean is_first = TRUE;
 
 				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 					" Lib(" /* ')' */);
 
-				if (show_gnet_info_shared_size && n->gnet_info_known) {
+				if (
+					GUI_PROPERTY(show_gnet_info_shared_size) &&
+					n->gnet_info_known
+				) {
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 						"%s",
 						compact_kb_size(n->gnet_files_count
 							? n->gnet_kbytes_count : 0, show_metric_units()));
 					is_first = FALSE;
 				}
-				if (show_gnet_info_shared_files && n->gnet_info_known)
+				if (
+					GUI_PROPERTY(show_gnet_info_shared_files) &&
+					n->gnet_info_known
+				) {
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 						"%s#=%u", is_first ? "" : ", ", n->gnet_files_count);
-
+				}
 				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 					/* '(' */ "%s)", n->gnet_info_known ? "" : "?");
 			}
 
-			if (show_gnet_info_qrp_stats) {
+			if (GUI_PROPERTY(show_gnet_info_qrp_stats)) {
 				if (n->has_qrp)
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 						" QRP=%u%%",
@@ -249,46 +285,49 @@ nodes_gui_common_status_str(const gnet_node_status_t *n)
 
 				if (n->qrt_slots != 0)
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-						" QRT(%s, g=%d, f=%d%%, t=%d%%, e=%d%%)",
+						" QRT(%s, g=%u, f=%u%%, t=%u%%, e=%u%%)",
 						compact_size(n->qrt_slots, show_metric_units()),
 						n->qrt_generation,
 						n->qrt_fill_ratio, n->qrt_pass_throw,
 						(guint) (n->qrp_efficiency * 100.0));
 			}
 
-			if (show_gnet_info_dbw)
+			if (GUI_PROPERTY(show_gnet_info_dbw))
 				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 				" Dup=%u Bad=%u W=%u H=%u S=%u E=%u",
 				n->n_dups, n->n_bad, n->n_weird,
 				n->n_hostile, n->n_spam, n->n_evil);
 
-			if (show_gnet_info_rt) {
+			if (GUI_PROPERTY(show_gnet_info_rt)) {
 				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-				" RT(avg=%d, last=%d", n->rt_avg, n->rt_last);	/* ) */
+				" RT(avg=%u, last=%u", n->rt_avg, n->rt_last);	/* ) */
 				if (n->tcp_rtt)
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-						", tcp=%d", n->tcp_rtt);
+						", tcp=%u", n->tcp_rtt);
 				if (n->udp_rtt)
 					slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-						", udp=%d", n->udp_rtt);
+						", udp=%u", n->udp_rtt);
 				slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
 					/* ( */ ")");
 			}
 
 			slen += gm_snprintf(&gui_tmp[slen], sizeof(gui_tmp)-slen,
-				" Q=%d,%d%% %s",
+				" Q=%u,%u%% %s",
 				n->mqueue_count, n->mqueue_percent_used,
 				n->in_tx_swift_control ? " [SW]" :
 				n->in_tx_flow_control ? " [FC]" : "");
 			a = gui_tmp;
-		} else
+		} else if (n->is_pseudo) {
+			a = _("No UDP traffic yet");
+		} else {
 			a = _("Connected");
+		}
 		break;
 
 	case GTA_NODE_SHUTDOWN:
 		{
 			gm_snprintf(gui_tmp, sizeof(gui_tmp),
-				_("Closing: %s [Stop in %ds] RX=%d Q=%d,%d%%"),
+				_("Closing: %s [Stop in %us] RX=%u Q=%u,%u%%"),
 				n->message, n->shutdown_remain, n->received,
 				n->mqueue_count, n->mqueue_percent_used);
 			a = gui_tmp;
@@ -325,10 +364,10 @@ add_node_helper(const host_addr_t *addrs, size_t n, gpointer data)
 	g_assert(0 != ctx->port);
 
 	if (n > 0) {
-		guc_node_add(addrs[random_raw() % n], ctx->port, ctx->flags);
+		guc_node_add(addrs[random_u32() % n], ctx->port, ctx->flags);
 	}
 
-	wfree(ctx, sizeof *ctx);
+	WFREE(ctx);
 }
 
 /**
@@ -381,7 +420,7 @@ nodes_gui_common_connect_by_name(const gchar *line)
 		}
 
 		if (!string_to_host_or_addr(q, &endptr, &addr)) {
-			g_message("Expected hostname or IP address");
+			g_message("expected hostname or IP address");
 			break;
 		}
 
@@ -397,7 +436,7 @@ nodes_gui_common_connect_by_name(const gchar *line)
 
 			port = parse_uint16(&q[1], &endptr, 10, &error);
 			if (error || 0 == port) {
-				g_message("Cannot parse port");
+				g_message("cannot parse port");
 				break;
 			}
 
@@ -405,7 +444,7 @@ nodes_gui_common_connect_by_name(const gchar *line)
 		} else {
 			q = skip_ascii_spaces(endptr);
 			if ('\0' != *q && ',' != *q) {
-				g_message("Expected \",\" or \":\"");
+				g_message("expected \",\" or \":\"");
 				break;
 			}
 		}
@@ -422,19 +461,51 @@ nodes_gui_common_connect_by_name(const gchar *line)
 				size_t n = 1 + hostname_len;
 
 				g_assert(n > hostname_len);
-				p = g_malloc(n);
+				p = halloc(n);
 				g_strlcpy(p, hostname, n);
 				hostname = p;
 			}
 
-			ctx = walloc(sizeof *ctx);
+			WALLOC(ctx);
 			ctx->port = port;
 			ctx->flags = flags;
 			guc_adns_resolve(hostname, add_node_helper, ctx);
 
-			G_FREE_NULL(p);
+			HFREE_NULL(p);
 		}
 	}
 }
 
+static gboolean
+nodes_gui_is_visible(void)
+{
+	return main_gui_window_visible() &&
+		nb_main_page_network == main_gui_notebook_get_page();
+}
+
+void
+nodes_gui_timer(time_t now)
+{
+    static time_t last_update;
+
+    if (last_update == now)
+        return;
+
+	/*
+	 * Usually don't perform updates if nobody is watching.  However,
+	 * we do need to perform periodic cleanup of dead entries or the
+	 * memory usage will grow.  Perform an update every UPDATE_MIN minutes
+	 * at least.
+	 *		--RAM, 28/12/2003
+	 */
+
+	if (
+		nodes_gui_is_visible() ||
+		delta_time(now, last_update) >= UPDATE_MIN
+	) {
+    	last_update = now;
+		nodes_gui_update_display(now);
+	}
+}
+
 /* vi: set ts=4 sw=4 cindent: */
