@@ -1,5 +1,5 @@
 /*
- * $Id: notebooks.h,v 1.7 2005/09/11 03:40:17 daichik Exp $
+ * $Id: notebooks.h 14628 2007-08-22 16:09:06Z cbiere $
  *
  * Copyright (c) 2001-2004, Raphael Manfredi & Richard Eckart
  *
@@ -30,19 +30,16 @@
  * For GTK1 these must be in the same order as the notebook tabs.
  */
 enum {
-    nb_main_page_gnet = 0,
-    nb_main_page_gnet_stats,
-    nb_main_page_hostcache,
-
-    nb_main_page_uploads,
-    nb_main_page_uploads_stats,
-
-    nb_main_page_dl_files,
-    nb_main_page_dl_active,
-    nb_main_page_dl_queue,
-
+    nb_main_page_network,
     nb_main_page_search,
+    nb_main_page_downloads,
+    nb_main_page_uploads,
+
+    nb_main_page_stats,
     nb_main_page_monitor,
+
+    nb_main_page_uploads_stats,
+    nb_main_page_hostcache,
     nb_main_page_search_stats,
 
     nb_main_page_num
@@ -64,15 +61,6 @@ enum {
     nb_prefs_num
 };
 
-/**
- * Notebook tabs in the downloads page.
- */
-enum {
-	nb_downloads_page_downloads = 0,
-	nb_downloads_page_fileinfo,
-	nb_downloads_page_num
-};
-
 #endif /* _gtk_notebooks_h_ */
 
 /* vi: set sw=4 ts=4 cindent: */
