@@ -1,5 +1,5 @@
 /*
- * $Id: search_common.h 15037 2007-12-01 14:51:07Z cbiere $
+ * $Id: search_common.h 19239 2011-06-04 14:10:20Z rmanfredi $
  *
  * Copyright (c) 2003, Raphael Manfredi
  *
@@ -53,11 +53,19 @@ typedef struct search {
 	struct slist *queue;		/**< records to be inserted */
 
 	gboolean	list_refreshed;
+	gboolean	clicked;
 	gboolean	sort;
 
     int        sort_col;
     int        sort_order;
 
+	/*
+	 * Cached attributes.
+	 */
+
+	unsigned	cached_attributes;	/**< Mask for things we cached */
+	unsigned	attributes;			/**< Actual attributes we know */
+
 	/*
 	 * Search stats.
 	 */
@@ -73,6 +81,13 @@ typedef struct search {
 	guint32		duplicates;			/**< Duplicate hits ignored */
 } search_t;
 
+enum {
+	SEARCH_GUI_F_WHATS_NEW	= 1 << 3,
+	SEARCH_GUI_F_BROWSE		= 1 << 2,
+	SEARCH_GUI_F_LOCAL		= 1 << 1,
+	SEARCH_GUI_F_PASSIVE	= 1 << 0
+};
+
 #ifdef USE_GTK1
 
 /**
@@ -101,7 +116,13 @@ enum gui_color {
 	GUI_COLOR_MARKED,
 	GUI_COLOR_MAYBE_SPAM,
 	GUI_COLOR_SPAM,
+	GUI_COLOR_ALIEN,
 	GUI_COLOR_UNREQUESTED,
+	GUI_COLOR_PUSH,
+	GUI_COLOR_PUSH_PROXY,
+	GUI_COLOR_PARTIAL_PUSH,
+	GUI_COLOR_PARTIAL,
+	GUI_COLOR_MEDIA,
 	GUI_COLOR_BACKGROUND,
 
 	NUM_GUI_COLORS
@@ -115,6 +136,7 @@ GdkColor *gui_color_get(enum gui_color);
  */
 
 void search_gui_common_init(void);
+void search_gui_common_shutdown(void);
 
 void search_gui_init_tree(struct search *);
 void search_gui_set_current_search(struct search *);
@@ -127,7 +149,7 @@ const char *search_gui_get_route(const struct results_set *);
 const char *search_gui_get_filename_extension(const char *filename_utf8);
 void search_gui_set_sort_defaults(void);
 
-struct query *search_gui_handle_query(const char *, flag_t flags,
+struct query *search_gui_handle_query(const char *, guint32 flags,
 						const char **error_str);
 void search_gui_query_free(struct query **query_ptr);
 void search_gui_filter_new(search_t *, GList *rules);
@@ -173,6 +195,7 @@ void search_gui_sort_column(struct search *, int column);
 void search_gui_expand_all(struct search *);
 void search_gui_collapse_all(struct search *);
 void search_gui_flush_queues(void);
+unsigned search_gui_queue_length(const struct search *);
 void search_gui_remove_search(search_t *);
 void search_gui_update_list_label(const struct search *);
 void search_gui_clear_search(struct search *);
@@ -185,11 +208,14 @@ void search_gui_option_menu_searches_freeze(void);
 
 gboolean search_gui_is_enabled(const struct search *);
 
-void search_gui_download(record_t *);
+void search_gui_download(record_t *, gnet_search_t sh);
 const char *search_gui_nice_size(const record_t *);
 const char *search_gui_get_vendor(const struct results_set *);
 
+gboolean search_gui_item_is_inspected(const record_t *);
 void search_gui_set_details(const record_t *);
+void search_gui_set_bitzi_metadata(const record_t *);
+void search_gui_set_bitzi_metadata_text(const char *);
 void search_gui_clear_details(void);
 void search_gui_append_detail(const char *title, const char *value);
 const char *search_new_error_to_string(enum search_new_result);
@@ -211,6 +237,8 @@ void search_gui_add_record(struct search *, record_t *, enum gui_color);
 void search_gui_hide_search(struct search *);
 void search_gui_show_search(struct search *);
 
+void search_gui_media_type_clear(void);
+
 /*
  * Search result record comparison functions.
  */
@@ -220,9 +248,6 @@ int gui_record_sha1_eq(const void *, const void *);
 int gui_record_host_eq(const void *, const void *);
 int gui_record_sha1_or_name_eq(const void *, const void *);
 
-/* FIXME: This does not belong here. */
-char *gnet_host_vec_to_string(const gnet_host_vec_t *);
-
 #endif /* _gtk_search_common_h_ */
 
 /* vi: set ts=4 sw=4 cindent: */
