@@ -1,5 +1,5 @@
 /*
- * $Id: settings_cb.c,v 1.24 2005/12/19 16:06:10 cbiere Exp $
+ * $Id: settings_cb.c 19324 2011-07-07 12:44:29Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -36,9 +36,8 @@
 
 #include "gui.h"
 
-RCSID("$Id: settings_cb.c,v 1.24 2005/12/19 16:06:10 cbiere Exp $");
+RCSID("$Id: settings_cb.c 19324 2011-07-07 12:44:29Z rmanfredi $")
 
-#include "gtk-missing.h"
 #include "misc.h"
 #include "settings_cb.h"
 #include "settings.h"
@@ -48,6 +47,8 @@ RCSID("$Id: settings_cb.c,v 1.24 2005/12/19 16:06:10 cbiere Exp $");
 #include "if/gnet_property.h"
 #include "if/bridge/ui2c.h"
 
+#include "lib/glib-missing.h"	/* For g_strlcpy() */
+#include "lib/str.h"
 #include "lib/override.h"		/* Must be the last header included */
 
 /*
@@ -69,56 +70,16 @@ RCSID("$Id: settings_cb.c,v 1.24 2005/12/19 16:06:10 cbiere Exp $");
         CAT2(pref,_prop_set_boolean)(p, &val, 0, 1);                    \
     } while (0)
 
-void
-on_spinbutton_search_reissue_timeout_changed(GtkEditable *editable,
-		gpointer unused_udata)
-{
-    static gboolean lock = FALSE;
-    search_t *current_search;
-    guint32 timeout_real;
-    guint32 timeout;
-
-	(void) unused_udata;
-
-    if (lock)		/* Prevents recursion */
-        return;
-
-    lock = TRUE;
-
-    current_search = search_gui_get_current_search();
-
-    if (
-		!current_search ||
-		!guc_search_is_active(current_search->search_handle)
-	) {
-        lock = FALSE;
-        return;
-    }
-
-    timeout = gtk_spin_button_get_value(GTK_SPIN_BUTTON(editable));
-
-    guc_search_set_reissue_timeout
-		(current_search->search_handle, timeout);
-    timeout_real = guc_search_get_reissue_timeout
-		(current_search->search_handle);
-
-    if (timeout != timeout_real)
-        gtk_spin_button_set_value(GTK_SPIN_BUTTON(editable), timeout_real);
-
-    lock = FALSE;
-}
-
 static void
-on_entry_config_proxy_hostname_activate_helper(const host_addr_t *addr,
-		gpointer unused_udata)
+on_entry_config_proxy_hostname_activate_helper(const host_addr_t *addrs,
+		size_t n, gpointer unused_udata)
 {
-	const gchar *s;
-
 	(void) unused_udata;
 
-	if (addr) {
-		s = host_addr_to_string(*addr);
-    	gnet_prop_set_string(PROP_PROXY_ADDR, s);
+	g_assert(addrs);
+	if (n > 0) {
+		/* Just pick the first address */
+    	gnet_prop_set_ip_val(PROP_PROXY_ADDR, addrs[0]);
 	}
 }
 
@@ -170,7 +131,7 @@ on_entry_config_extensions_activate(GtkEditable *editable, gpointer unused_data)
 	(void) unused_data;
     ext = STRTRACK(gtk_editable_get_chars(editable, 0, -1));
     gnet_prop_set_string(PROP_SCAN_EXTENSIONS, ext);
-    g_free(ext);
+    G_FREE_NULL(ext);
 }
 FOCUS_TO_ACTIVATE(entry_config_extensions)
 
@@ -182,10 +143,10 @@ on_entry_config_path_activate(GtkEditable *editable, gpointer unused_udata)
 
 	(void) unused_udata;
     gnet_prop_set_string(PROP_SHARED_DIRS_PATHS, path);
-    g_free(path);
+    G_FREE_NULL(path);
 }
 FOCUS_TO_ACTIVATE(entry_config_path)
-#endif /* GTK1 */
+#endif /* USE_GTK1 */
 
 #ifdef USE_GTK2
 void
@@ -196,12 +157,12 @@ on_button_config_remove_dir_clicked(GtkButton *unused_button,
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 	GtkTreeSelection *selection;
-	GString *gs;
+	str_t *s;
 
 	(void) unused_button;
 	(void) unused_udata;
 
-	tv = GTK_TREE_VIEW(lookup_widget(dlg_prefs, "treeview_shared_dirs"));
+	tv = GTK_TREE_VIEW(gui_dlg_prefs_lookup("treeview_shared_dirs"));
 	model = gtk_tree_view_get_model(tv);
 
 	if (!gtk_tree_model_get_iter_first(model, &iter))
@@ -209,7 +170,7 @@ on_button_config_remove_dir_clicked(GtkButton *unused_button,
 
 	/* Regenerate the string property holding a list of paths */
 	selection = gtk_tree_view_get_selection(tv);
-	gs = g_string_new("");
+	s = str_new(0);
 
 	do {
 		gchar *dir = NULL;
@@ -219,30 +180,36 @@ on_button_config_remove_dir_clicked(GtkButton *unused_button,
 			continue;
 
 		gtk_tree_model_get(model, &iter, 0, &dir, (-1));
-		if ('\0' != gs->str[0])
-			gs = g_string_append_c(gs, ':');
-		gs = g_string_append(gs, dir);
+		if (str_len(s) > 0)
+			str_putc(s, ':');
+		str_cat(s, dir);
 		G_FREE_NULL(dir);
 	} while (gtk_tree_model_iter_next(model, &iter));
 
-	gnet_prop_set_string(PROP_SHARED_DIRS_PATHS, gs->str);
-	g_string_free(gs, TRUE);
+	gnet_prop_set_string(PROP_SHARED_DIRS_PATHS, str_2c(s));
+	str_destroy(s);
 }
-#endif /* GTK2 */
+#endif /* USE_GTK2 */
 
 void
 on_entry_config_force_ip_activate(GtkEditable *unused_editable,
 		gpointer unused_udata)
 {
    	gchar *text;
+	host_addr_t addr;
+	const gchar *endptr;
 
 	(void) unused_editable;
 	(void) unused_udata;
 	text = STRTRACK(gtk_editable_get_chars(
-        GTK_EDITABLE(lookup_widget(dlg_prefs, "entry_config_force_ip")),
+        GTK_EDITABLE(gui_dlg_prefs_lookup("entry_config_force_ip")),
         0, -1));
 	g_strstrip(text);
-	gnet_prop_set_string(PROP_FORCED_LOCAL_IP, text);
+	if (string_to_host_addr(text, &endptr, &addr) && '\0' == endptr[0]) {
+		gnet_prop_set_ip_val(PROP_FORCED_LOCAL_IP, addr);
+	} else if (0 == strcmp(text, "") || 0 == strcmp(text, "<none>")) {
+		gnet_prop_set_ip_val(PROP_FORCED_LOCAL_IP, zero_host_addr);
+	}
 	G_FREE_NULL(text);
 }
 FOCUS_TO_ACTIVATE(entry_config_force_ip)
@@ -251,12 +218,114 @@ void
 on_entry_config_force_ip_changed(GtkEditable *editable, gpointer unused_udata)
 {
     gchar *text = STRTRACK(gtk_editable_get_chars(editable, 0, -1));
+	const gchar *endptr;
+	gboolean sensitive;
 
 	(void) unused_udata;
 	g_strstrip(text);
+	sensitive = string_to_host_addr(text, &endptr, NULL) && '\0' == endptr[0];
 	gtk_widget_set_sensitive(
-        lookup_widget(dlg_prefs, "checkbutton_config_force_ip"),
-        is_host_addr(string_to_host_addr(text, NULL)));
+        gui_dlg_prefs_lookup("checkbutton_config_force_ip"),
+		sensitive);
+	gtk_widget_set_sensitive(
+        gui_dlg_prefs_lookup("checkbutton_config_bind_ipv4"),
+		sensitive);
+	G_FREE_NULL(text);
+}
+
+void
+on_entry_config_force_ipv6_activate(GtkEditable *unused_editable,
+		gpointer unused_udata)
+{
+   	gchar *text;
+	host_addr_t addr;
+	const gchar *endptr;
+
+	(void) unused_editable;
+	(void) unused_udata;
+	text = STRTRACK(gtk_editable_get_chars(
+        GTK_EDITABLE(gui_dlg_prefs_lookup("entry_config_force_ipv6")),
+        0, -1));
+	g_strstrip(text);
+	if (
+		string_to_host_addr(text, &endptr, &addr)
+			&& '\0' == endptr[0]
+			&& host_addr_is_ipv6(addr)
+	) {
+		gnet_prop_set_ip_val(PROP_FORCED_LOCAL_IP6, addr);
+	} else if (0 == strcmp(text, "") || 0 == strcmp(text, "<none>")) {
+		gnet_prop_set_ip_val(PROP_FORCED_LOCAL_IP6, zero_host_addr);
+	}
+	G_FREE_NULL(text);
+}
+FOCUS_TO_ACTIVATE(entry_config_force_ipv6)
+
+void
+on_entry_config_force_ipv6_changed(GtkEditable *editable, gpointer unused_udata)
+{
+    gchar *text = STRTRACK(gtk_editable_get_chars(editable, 0, -1));
+	const gchar *endptr;
+	host_addr_t addr;
+	gboolean sensitive;
+
+	(void) unused_udata;
+	g_strstrip(text);
+	sensitive = string_to_host_addr(text, &endptr, &addr)
+			&& '\0' == endptr[0]
+			&& host_addr_is_ipv6(addr);
+	gtk_widget_set_sensitive(
+        gui_dlg_prefs_lookup("checkbutton_config_force_ipv6"),
+		sensitive);
+	gtk_widget_set_sensitive(
+        gui_dlg_prefs_lookup("checkbutton_config_bind_ipv6"),
+		sensitive);
+	G_FREE_NULL(text);
+}
+
+void
+on_entry_config_ipv6_trt_prefix_activate(GtkEditable *unused_editable,
+		gpointer unused_udata)
+{
+	const gchar *endptr;
+	host_addr_t addr;
+   	gchar *text;
+
+	(void) unused_editable;
+	(void) unused_udata;
+	text = STRTRACK(gtk_editable_get_chars(
+        GTK_EDITABLE(gui_dlg_prefs_lookup("entry_config_ipv6_trt_prefix")),
+        0, -1));
+	g_strstrip(text);
+	if (
+		string_to_host_addr(text, &endptr, &addr)
+			&& '\0' == endptr[0]
+			&& host_addr_is_ipv6(addr)
+	) {
+		gnet_prop_set_ip_val(PROP_IPV6_TRT_PREFIX, addr);
+	} else if (0 == strcmp(text, "") || 0 == strcmp(text, "<none>")) {
+		gnet_prop_set_ip_val(PROP_IPV6_TRT_PREFIX, zero_host_addr);
+	}
+	G_FREE_NULL(text);
+}
+FOCUS_TO_ACTIVATE(entry_config_ipv6_trt_prefix)
+
+void
+on_entry_config_ipv6_trt_prefix_changed(GtkEditable *editable,
+	gpointer unused_udata)
+{
+    gchar *text = STRTRACK(gtk_editable_get_chars(editable, 0, -1));
+	const gchar *endptr;
+	host_addr_t addr;
+	gboolean sensitive;
+
+	(void) unused_udata;
+	g_strstrip(text);
+	sensitive = string_to_host_addr(text, &endptr, &addr)
+			&& '\0' == endptr[0]
+			&& host_addr_is_ipv6(addr);
+	gtk_widget_set_sensitive(
+        gui_dlg_prefs_lookup("checkbutton_config_ipv6_trt_enable"),
+		sensitive);
 	G_FREE_NULL(text);
 }
 
@@ -269,7 +338,7 @@ on_entry_server_hostname_activate(GtkEditable *unused_editable,
 	(void) unused_editable;
 	(void) unused_udata;
 	text = STRTRACK(gtk_editable_get_chars(
-        GTK_EDITABLE(lookup_widget(dlg_prefs, "entry_server_hostname")),
+        GTK_EDITABLE(gui_dlg_prefs_lookup("entry_server_hostname")),
         0, -1));
 	g_strstrip(text);
 	gnet_prop_set_string(PROP_SERVER_HOSTNAME, text);
@@ -286,7 +355,7 @@ on_entry_server_hostname_changed(GtkEditable *editable, gpointer unused_udata)
 	
 	g_strstrip(text);
 	gtk_widget_set_sensitive(
-        lookup_widget(dlg_prefs, "checkbutton_give_server_hostname"),
+        gui_dlg_prefs_lookup("checkbutton_give_server_hostname"),
         strlen(text) > 3);		/* Minimum: "x.cx" */
 	G_FREE_NULL(text);
 }
@@ -368,10 +437,7 @@ on_leave_notify(GtkWidget *widget, GdkEventCrossing *unused_event,
 
 	tv = GTK_TREE_VIEW(data);
 	update_tooltip(GTK_TREE_VIEW(widget), NULL);
-	if (tvm_dbg_property) {
-		tree_view_motion_clear_callback(tv, tvm_dbg_property);
-		tvm_dbg_property = NULL;
-	}
+	tree_view_motion_clear_callback(&tvm_dbg_property);
 	return FALSE;
 }
 
@@ -389,7 +455,7 @@ on_cell_edited(GtkCellRendererText *unused_renderer, const gchar *path_str,
 	(void) unused_renderer;
 	(void) unused_data;
 
-	tv = GTK_TREE_VIEW(lookup_widget(dlg_prefs, "treeview_dbg_property"));
+	tv = GTK_TREE_VIEW(gui_dlg_prefs_lookup("treeview_dbg_property"));
 	g_return_if_fail(NULL != tv);
 	model = gtk_tree_view_get_model(tv);
 	g_return_if_fail(NULL != model);
@@ -404,6 +470,42 @@ on_cell_edited(GtkCellRendererText *unused_renderer, const gchar *path_str,
 	gtk_list_store_set(GTK_LIST_STORE(model), &iter,
 		dbg_col_value, gnet_prop_to_string(prop),
 		(-1));
+
+	gtk_tree_path_free(path);
+}
+
+static gboolean
+refresh_property(GtkTreeModel *model, GtkTreePath *unused_path,
+	GtkTreeIter *iter, gpointer unused_data)
+{
+	property_t prop;
+	guint u;
+
+	(void) unused_data;
+	(void) unused_path;
+
+	u = 0;
+	gtk_tree_model_get(model, iter, dbg_col_property, &u, (-1));
+	prop = (property_t) u;
+	gtk_list_store_set(GTK_LIST_STORE(model), iter,
+		dbg_col_value, gnet_prop_to_string(prop),
+		(-1));
+	return FALSE;
+}
+
+void
+on_button_dbg_property_refresh_clicked(GtkButton *unused_button,
+	gpointer unused_udata)
+{
+	GtkTreeModel *model;
+	GtkTreeView *tv;
+
+	(void) unused_button;
+	(void) unused_udata;
+
+	tv = GTK_TREE_VIEW(gui_dlg_prefs_lookup("treeview_dbg_property"));
+	model = gtk_tree_view_get_model(tv);
+	gtk_tree_model_foreach(GTK_TREE_MODEL(model), refresh_property, NULL);
 }
 
 static void
@@ -416,7 +518,7 @@ dbg_property_update_selection(void)
 	GtkWidget *widget;
 	const gchar *text;
 
-	tv = GTK_TREE_VIEW(lookup_widget(dlg_prefs, "treeview_dbg_property"));
+	tv = GTK_TREE_VIEW(gui_dlg_prefs_lookup("treeview_dbg_property"));
 	s = gtk_tree_view_get_selection(tv);
 	if (gtk_tree_selection_get_selected(s, &model, &iter)) {
 		guint u = 0;
@@ -427,7 +529,7 @@ dbg_property_update_selection(void)
 		text = _("<no property selected>");
 	}
 	
-    widget = lookup_widget(dlg_prefs, "label_dbg_property_default");
+    widget = gui_dlg_prefs_lookup("label_dbg_property_default");
 	gtk_label_set_text(GTK_LABEL(widget), text);
 }
 
@@ -460,7 +562,7 @@ dbg_tree_init(void)
 
 	STATIC_ASSERT(G_N_ELEMENTS(columns) == (guint) num_dbg_cols);
 	
-	tv = GTK_TREE_VIEW(lookup_widget(dlg_prefs, "treeview_dbg_property"));
+	tv = GTK_TREE_VIEW(gui_dlg_prefs_lookup("treeview_dbg_property"));
 	store = GTK_LIST_STORE(gtk_list_store_new(G_N_ELEMENTS(columns),
 				G_TYPE_STRING,		/* Type */
 				G_TYPE_STRING,		/* Name */
@@ -482,8 +584,7 @@ dbg_tree_init(void)
 		renderer = gtk_cell_renderer_text_new();
 
 		if (columns[i].editable) {
-			g_signal_connect(renderer, "edited",
-					G_CALLBACK(on_cell_edited), NULL);
+			gui_signal_connect(renderer, "edited", on_cell_edited, NULL);
 			g_object_set(renderer,
 					"editable", TRUE,
 					(void *) 0);
@@ -521,12 +622,9 @@ dbg_tree_init(void)
 		gtk_tree_view_append_column(tv, column);
 	}
 
-	g_signal_connect(GTK_OBJECT(tv),
-			"enter-notify-event", G_CALLBACK(on_enter_notify), tv);
-	g_signal_connect(GTK_OBJECT(tv),
-			"leave-notify-event", G_CALLBACK(on_leave_notify), tv);
-	g_signal_connect(GTK_OBJECT(tv),
-			"cursor-changed", G_CALLBACK(on_cursor_changed), tv);
+	gui_signal_connect(tv, "enter-notify-event", on_enter_notify, tv);
+	gui_signal_connect(tv, "leave-notify-event", on_leave_notify, tv);
+	gui_signal_connect(tv, "cursor-changed", on_cursor_changed, tv);
 }
 
 void
@@ -536,7 +634,7 @@ dbg_property_show_list(const GSList *props)
 	GtkTreeView *tv;
 	GtkListStore *store;
 	
-	tv = GTK_TREE_VIEW(lookup_widget(dlg_prefs, "treeview_dbg_property"));
+	tv = GTK_TREE_VIEW(gui_dlg_prefs_lookup("treeview_dbg_property"));
 	if (!gtk_tree_view_get_model(tv))
 		dbg_tree_init();
 		
@@ -547,19 +645,19 @@ dbg_property_show_list(const GSList *props)
 		const gchar *text = _("<no property selected>");
 		GtkWidget *widget;
 		
-		widget = lookup_widget(dlg_prefs, "label_dbg_property_limits");
+		widget = gui_dlg_prefs_lookup("label_dbg_property_limits");
 		gtk_label_set_text(GTK_LABEL(widget), text);
 		/* Gtk+ 2.x has editable column cells */
 #ifdef USE_GTK1
-		widget = lookup_widget(dlg_prefs, "label_dbg_property_name");
+		widget = gui_dlg_prefs_lookup("label_dbg_property_name");
 		gtk_label_set_text(GTK_LABEL(widget), text);
-		widget = lookup_widget(dlg_prefs, "entry_dbg_property_value");
+		widget = gui_dlg_prefs_lookup("entry_dbg_property_value");
 		gtk_entry_set_text(GTK_ENTRY(widget), text);
-		widget = lookup_widget(dlg_prefs, "entry_dbg_property_default");
+		widget = gui_dlg_prefs_lookup("entry_dbg_property_default");
 		gtk_entry_set_text(GTK_ENTRY(widget), text);
 #endif /* USE_GTK1 */
 #ifdef USE_GTK2
-		widget = lookup_widget(dlg_prefs, "label_dbg_property_default");
+		widget = gui_dlg_prefs_lookup("label_dbg_property_default");
 		gtk_label_set_text(GTK_LABEL(widget), text);
 #endif /* USE_GTK2 */
 	}
@@ -593,10 +691,10 @@ dbg_property_update_selection(void)
 	gpointer data;
 	gint row;
 
-   	clist = GTK_CLIST(lookup_widget(dlg_prefs, "clist_dbg_property"));
-	label = GTK_LABEL(lookup_widget(dlg_prefs, "label_dbg_property_name"));
-    value = GTK_ENTRY(lookup_widget(dlg_prefs, "entry_dbg_property_value"));
-    def = GTK_ENTRY(lookup_widget(dlg_prefs, "entry_dbg_property_default"));
+   	clist = GTK_CLIST(gui_dlg_prefs_lookup("clist_dbg_property"));
+	label = GTK_LABEL(gui_dlg_prefs_lookup("label_dbg_property_name"));
+    value = GTK_ENTRY(gui_dlg_prefs_lookup("entry_dbg_property_value"));
+    def = GTK_ENTRY(gui_dlg_prefs_lookup("entry_dbg_property_default"));
 
     if (
 		clist->selection &&
@@ -661,7 +759,7 @@ dbg_property_show_list(const GSList *props)
 	GtkCList *clist;
 	const GSList *sl;
 
-   	clist = GTK_CLIST(lookup_widget(dlg_prefs, "clist_dbg_property"));
+   	clist = GTK_CLIST(gui_dlg_prefs_lookup("clist_dbg_property"));
 	gtk_clist_freeze(clist);
 	gtk_clist_clear(clist);
 
@@ -686,7 +784,7 @@ on_entry_dbg_property_value_activate(GtkEditable *editable,
 
 	(void) unused_udata;
 
-   	clist = GTK_CLIST(lookup_widget(dlg_prefs, "clist_dbg_property"));
+   	clist = GTK_CLIST(gui_dlg_prefs_lookup("clist_dbg_property"));
 
     if (clist->selection) {
    		gchar *text;
@@ -792,7 +890,8 @@ on_clist_dbg_property_click_column(GtkCList *clist, gint column,
 		g_assert_not_reached();
 	}
 			
-	gtk_clist_sort(clist);
+	if (do_sort)
+		gtk_clist_sort(clist);
 }
 #endif /* USE_GTK1 */
 
@@ -807,7 +906,7 @@ on_entry_dbg_property_pattern_activate(GtkEditable *unused_editable,
 	(void) unused_udata;
 
 	text = STRTRACK(gtk_editable_get_chars(
-        GTK_EDITABLE(lookup_widget(dlg_prefs, "entry_dbg_property_pattern")),
+        GTK_EDITABLE(gui_dlg_prefs_lookup("entry_dbg_property_pattern")),
         0, -1));
 	g_strstrip(text);
 
@@ -822,8 +921,7 @@ on_entry_dbg_property_pattern_activate(GtkEditable *unused_editable,
 		
 		dbg_property_show_list(props);
 		dbg_property_update_selection();
-		g_slist_free(props);
-		props = NULL;
+		gm_slist_free_null(&props);
 	}
 	G_FREE_NULL(text);
 }
@@ -832,24 +930,9 @@ void
 on_menu_searchbar_visible_activate(GtkMenuItem *menuitem,
 	gpointer unused_udata)
 {
-	GtkWidget *viewport, *entry;
-	
 	(void) unused_udata;
-	
-	viewport = lookup_widget(main_window, "viewport_searchbar");
-	entry = lookup_widget(main_window, "entry_search");
 
-	if (GTK_WIDGET_VISIBLE(viewport)) {
-		if (GTK_WIDGET_HAS_FOCUS(entry))
-			gtk_widget_hide(viewport);
-	} else {
-		gtk_widget_show(viewport);
-	}
-
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem),
-		GTK_WIDGET_VISIBLE(viewport));
-	if (GTK_WIDGET_VISIBLE(viewport) && !GTK_WIDGET_HAS_FOCUS(entry))
-		gtk_widget_grab_focus(entry);
+	checkmenu_changed(gui, PROP_SEARCHBAR_VISIBLE, menuitem);
 }
 
 void
@@ -869,7 +952,7 @@ on_menu_sidebar_visible_activate(GtkMenuItem *menuitem, gpointer unused_udata)
 		gboolean sidebar;
 
 		gui_prop_get_boolean_val(PROP_SIDEBAR_VISIBLE, &sidebar);
-		paned = GTK_PANED(lookup_widget(main_window, "hpaned_main"));
+		paned = GTK_PANED(gui_main_window_lookup("hpaned_main"));
 		if (sidebar) {
 			paned_restore_position(paned, PROP_MAIN_DIVIDER_POS);
 		} else {
@@ -881,43 +964,12 @@ on_menu_sidebar_visible_activate(GtkMenuItem *menuitem, gpointer unused_udata)
 }
 
 void
-on_menu_navtree_visible_activate(GtkMenuItem *menuitem, gpointer unused_udata)
-{
-	gboolean sidebar, navtree;
-	
-	(void) unused_udata;
-
-	checkmenu_changed(gui, PROP_NAVTREE_VISIBLE, menuitem);
-
-	gui_prop_get_boolean_val(PROP_NAVTREE_VISIBLE, &navtree);
-	gui_prop_get_boolean_val(PROP_SIDEBAR_VISIBLE, &sidebar);
-	if (navtree && !sidebar)
-		gui_prop_set_boolean_val(PROP_SIDEBAR_VISIBLE, TRUE);
-
-	/*
-	 * Gtk+ 2.x automagically moves the gutter when a child's
-	 * visibility status changes.
-	 */
-#ifdef USE_GTK1
-	{
-		GtkPaned *paned;
-		
-		paned = GTK_PANED(lookup_widget(main_window, "vpaned_sidebar"));
-		if (navtree) {
-			paned_restore_position(paned, PROP_SIDE_DIVIDER_POS);
-		} else {
-			paned_save_position(paned, PROP_SIDE_DIVIDER_POS);
-			gtk_paned_set_position(paned, 0);
-		}
-	}
-#endif /* USE_GTK1 */
-}
-
-void
-on_menu_toolbar_visible_activate(GtkMenuItem *menuitem, gpointer unused_udata)
+on_menu_menubar_visible_activate(GtkMenuItem *menuitem,
+	gpointer unused_udata)
 {
 	(void) unused_udata;
-	checkmenu_changed(gui, PROP_TOOLBAR_VISIBLE, menuitem);
+
+	checkmenu_changed(gui, PROP_MENUBAR_VISIBLE, menuitem);
 }
 
 void
@@ -1000,17 +1052,27 @@ on_menu_autohide_bws_gleaf_activate(GtkMenuItem *menuitem,
 }
 
 void
-on_popup_search_toggle_tabs_activate(GtkMenuItem *unused_menuitem,
-		gpointer unused_udata)
+on_menu_autohide_bws_dht_activate(GtkMenuItem *menuitem,
+	gpointer unused_udata)
 {
-    gboolean val;
-
-	(void) unused_menuitem;
 	(void) unused_udata;
+	checkmenu_changed(gui, PROP_AUTOHIDE_BWS_DHT, menuitem);
+}
 
-    gui_prop_get_boolean_val(PROP_SEARCH_RESULTS_SHOW_TABS, &val);
-    val = !val;
-    gui_prop_set_boolean_val(PROP_SEARCH_RESULTS_SHOW_TABS, val);
+void
+on_menu_bws_dht_in_visible_activate(GtkMenuItem *menuitem,
+	gpointer unused_udata)
+{
+	(void) unused_udata;
+	checkmenu_changed(gui, PROP_PROGRESSBAR_BWS_DHT_IN_VISIBLE, menuitem);
+}
+
+void
+on_menu_bws_dht_out_visible_activate(GtkMenuItem *menuitem,
+	gpointer unused_udata)
+{
+	(void) unused_udata;
+	checkmenu_changed(gui, PROP_PROGRESSBAR_BWS_DHT_OUT_VISIBLE, menuitem);
 }
 
 /* vi: set ts=4 sw=4 cindent: */
