@@ -1,5 +1,5 @@
 /*
- * $Id: upload_stats.h,v 1.1 2004/08/24 23:18:29 rmanfredi Exp $
+ * $Id: upload_stats.h 17204 2009-11-07 19:38:16Z rmanfredi $
  *
  * Copyright (c) 2001-2003, Richard Eckart
  *
@@ -29,7 +29,10 @@
 #include "if/core/uploads.h"
 #include "if/ui/gtk/upload_stats.h"
 
-void upload_stats_gui_init(void);
 void upload_stats_gui_shutdown(void);
+void upload_stats_common_gui_init(void);
+void upload_stats_common_gui_shutdown(void);
+
+void upload_stats_gui_update_model(struct ul_stats *us);
 
 #endif /* _gtk_uploads_stats_h_ */
