@@ -1,5 +1,5 @@
 /*
- * $Id: uploads_common.h 10360 2005-12-11 13:52:52Z rmanfredi $
+ * $Id: uploads_common.h 14356 2007-08-07 21:05:41Z cbiere $
  *
  * Copyright (c) 2001-2003, Raphael Manfredi, Richard Eckart
  *
@@ -33,13 +33,15 @@
 
 #include "if/ui/gtk/uploads.h"
 
-gfloat uploads_gui_progress(
-	const gnet_upload_status_t *u, const upload_row_data_t *data);
-const gchar *uploads_gui_status_str(
-    const gnet_upload_status_t *u, const upload_row_data_t *data);
+gdouble uploads_gui_progress(const gnet_upload_status_t *u,
+		const upload_row_data_t *data);
+const gchar *uploads_gui_status_str(const gnet_upload_status_t *u,
+		const upload_row_data_t *data);
 gboolean upload_should_remove(time_t now, const upload_row_data_t *ul);
 const gchar *uploads_gui_host_string(const gnet_upload_info_t *u);
 void uploads_gui_browse_host(host_addr_t addr, guint16 port);
 upload_row_data_t *uploads_gui_get_row_data(gnet_upload_t uhandle);
 
+gboolean uploads_gui_update_required(time_t now);
+
 #endif /* _gtk_uploads_common_h_ */
