@@ -1,5 +1,5 @@
 /*
- * $Id: visual_progress.c 14173 2007-07-24 03:34:25Z cbiere $
+ * $Id: visual_progress.c 19142 2011-05-23 20:44:04Z rmanfredi $
  *
  * Copyright (c) 2003-2005, Hans de Graaff
  *
@@ -44,7 +44,7 @@
 
 #include "gui.h"
 
-RCSID("$Id: visual_progress.c 14173 2007-07-24 03:34:25Z cbiere $")
+RCSID("$Id: visual_progress.c 19142 2011-05-23 20:44:04Z rmanfredi $")
 
 #include "visual_progress.h"
 
@@ -53,6 +53,8 @@ RCSID("$Id: visual_progress.c 14173 2007-07-24 03:34:25Z cbiere $")
 #include "if/bridge/ui2c.h"
 
 #include "lib/atoms.h"
+#include "lib/glib-missing.h"
+#include "lib/stringify.h"
 #include "lib/walloc.h"
 #include "lib/override.h"	/* Must be the last header included */
 
@@ -346,7 +348,7 @@ vp_get_chunks_initial(gnet_fi_t fih) {
 			} else {
 				result = g_slist_delete_link(result, sl);
 			}
-			wfree(chunk, sizeof *chunk);
+			WFREE(chunk);
 		} else {
 			prev = sl;
 		}
@@ -372,7 +374,7 @@ vp_gui_fi_added(gnet_fi_t fih)
     fi = guc_fi_get_info(fih);
     guc_fi_get_status(fih, &s);
 
-    new_vp_info = walloc0(sizeof *new_vp_info);
+    WALLOC0(new_vp_info);
     new_vp_info->fi_handle = fih;
     new_vp_info->filename = atom_str_get(fi->filename);
     new_vp_info->filesize = s.size;
@@ -405,7 +407,7 @@ vp_info_free(vp_info_t **v_ptr)
 		guc_fi_free_chunks(v->chunks_initial);
 		guc_fi_free_ranges(v->ranges_list);
 		atom_str_free_null(&v->filename);
-		wfree(v, sizeof *v);
+		WFREE(v);
 		*v_ptr = NULL;
 	}
 }
@@ -483,7 +485,7 @@ vp_create_chunk(filesize_t from, filesize_t to,
 {
 	gnet_fi_chunks_t *chunk;
 
-	chunk = walloc(sizeof *chunk);
+	WALLOC(chunk);
 	chunk->from = from;
 	chunk->to = to;
 	chunk->status = status;
@@ -831,8 +833,7 @@ vp_gui_shutdown(void)
     guc_fi_remove_listener(vp_gui_fi_ranges_changed, EV_FI_RANGES_CHANGED);
 
     g_hash_table_foreach(vp_info_hash, vp_free_key_value, NULL);
-    g_hash_table_destroy(vp_info_hash);
-	vp_info_hash = NULL;
+    gm_hash_table_destroy_null(&vp_info_hash);
 }
 
 /*
